#ifndef BFRAME_H
#define BFRAME_H

#include "myglheader.h"

#define CURRENT_FRAME 65536
#define CURRENT_MOTION 65536
#define MAX_MOTIONS 255

class Boxer;
class BCooords;
class BFrame;

enum {
	BASIC_PAUSE,
	MOVE_RHYTHM,
	MOVE_FOWARD,
	MOVE_BACK,
	MOVE_RIGHT,
	MOVE_LEFT,
	GET_HIT1,
	GET_HIT2,
	GET_DOWN1,
	GET_DOWN2,	
	RIGHT_PUNCH0,
	RIGHT_PUNCH1,
	RIGHT_PUNCH2,
	RIGHT_PUNCH3,
	RIGHT_PUNCH4,
	RIGHT_PUNCH5,
	RIGHT_PUNCH6,
	RIGHT_PUNCH7,
	RIGHT_PUNCH8,
	LEFT_PUNCH0,
	LEFT_PUNCH1,
	LEFT_PUNCH2,
	LEFT_PUNCH3,
	LEFT_PUNCH4,
	LEFT_PUNCH5,
	LEFT_PUNCH6,
	LEFT_PUNCH7,
	LEFT_PUNCH8,
	DEFENSE0,	
	DEFENSE1,	
	DEFENSE2,	
	DEFENSE3,	
	DEFENSE4,	
	DEFENSE5,	
	DEFENSE6,	
	DEFENSE7,	
	DEFENSE8,
	DEADLY1,
	DEADLY2,
	DEADLY3,
	DEADLY4,
	MAX_ASSIGN,		
};

class BFrame {
public:
	BFrame();
	~BFrame();
	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Interpolate(BFrame *base, BFrame *next, int current_cut, int cut_number);
	void LagrangeInterpolate(BFrame *pre, BFrame *base, BFrame *next, BFrame *nearnext, int current_cut, int pre_number, int base_number, int next_number);
//	BFrame& operator=(const BFrame &cbframe); 
 	void Copy(const BFrame &cbframe); 
 	float& operator[](int number) {return joints[number];}
private:
	float joints[44];
	friend class BCoords;
};

class BFrameList : public BFrame {
public:
	BFrameList();
	~BFrameList();	
	void SetPre(BFrameList *cpre) {pre = cpre;}
	void SetNext(BFrameList *cnext) {next = cnext;}
	BFrameList& GetPre() const {return *pre;}
	BFrameList& GetNext() const {return *next;}
//	BFrameList& operator=(const BFrameList &cbframe); 
	void From(const BFrame &cbframe, int number); 
	void SetNumber(int cnumber) {cut_number = cnumber;}
	int GetNumber() const {return cut_number;}
private:
	int cut_number;
	BFrameList *pre;
	BFrameList *next;	
};

class BCoords {
public:
	BCoords();
	~BCoords();
	bool CalcAngles(Boxer *cboxer, int crfist_fix, int clfist_fix, int crtoe_fix, int cltoe_fix, int cwaist_fix, int chead_fix);
private:
	Boxer *boxer;
	void SetCoords();
	float CalcFunc();
	float dargs[44];
	BFrame change, changep, changem;
	int rfist_fix, lfist_fix, rtoe_fix, ltoe_fix, head_fix, waist_fix;
	float rf_x, rf_y, rf_z;
	float rf_a, rf_b, rf_g;
	float lf_x, lf_y, lf_z;
	float lf_a, lf_b, lf_g;
	
	float rt_x, rt_y, rt_z;
	float rt_a, rt_b, rt_g;
	float lt_x, lt_y, lt_z;
	float lt_a, lt_b, lt_g;

	float h_x, h_y, h_z;
	float h_a, h_b, h_g;
	float w_x, w_y, w_z;
	float w_a, w_b, w_g;

	float drf_x, drf_y, drf_z, rfc;
	float drf_a, drf_b, drf_g, rfa;
	float dlf_x, dlf_y, dlf_z, lfc;
	float dlf_a, dlf_b, dlf_g, lfa;
	
	float drt_x, drt_y, drt_z, rtc;
	float drt_a, drt_b, drt_g, rta;
	float dlt_x, dlt_y, dlt_z, ltc;
	float dlt_a, dlt_b, dlt_g, lta;

	float dh_x, dh_y, dh_z, hc;
	float dh_a, dh_b, dh_g, ha;
	float dw_x, dw_y, dw_z, wc;
	float dw_a, dw_b, dw_g, wa;
	float f, fb, sfb, fm, fp, oldfb;
};

class BMotion {
public:
	BMotion();
	BMotion(int number);
	~BMotion();

	int Init(int number);
	void Free();

	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Set(int number, Boxer *boxer);	
	void Get(int number, Boxer *boxer);	
	void Link();	
	
	void Add();
	void Del(int number = CURRENT_FRAME);
	void Insert(int number = CURRENT_FRAME);

	int TotalCut() const {return total_cut;}
	int CutNumber() const {return cut_number;}
	int Count() const {return max_number;}
	void Choice(int number);
	bool WasInit();

	BFrameList& operator[](int number);
	void SetFrame(int number);
	void PreFrame();
	void NextFrame();
	BFrame& CurrentFrame() {return frame;}
	BMotion& operator=(const BMotion &cbmotion); 
private:
	BFrameList *bframes;
	BFrame frame;
	int total_cut;	
	int cut_number;	
	int current_cut;	
	int current_frame;	
	int max_number, current_number;
	friend class BComboMotion;
};

class BCombo {
public:
	BCombo();
	BCombo(int number);
	~BCombo();

	int Init(int number);
	void Free();

	bool IsCombo(int number, int number_next);
	bool WasInit();
	void SetCombo(int number, int first_number, int second_number);
private:
	int *first, *second;
	int max_number, current_number;
};

class BComboMotion {
public:
	BComboMotion();
	~BComboMotion();

	void Link(const BFrame &base, BMotion &bmotion);

	int TotalCut() const {return total_cut;}
	int CutNumber() const {return cut_number;}
	int Count() const {return max_number;}
	void SetFrame(int number);
	void PreFrame();
	void NextFrame();
	BFrame& CurrentFrame() {return frame;}
private:
	BFrame frame;
	BFrameList bframes[80];
	int total_cut;	
	int cut_number;	
	int current_cut;	
	int current_frame;	
	int max_number, current_number;
};

class BMotions {
public:
	BMotions();
	BMotions(int number);
	~BMotions();

	int Init(int number);
	void Free();

	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Set(int number, Boxer *boxer);	
	void Get(int number, Boxer *boxer);	
	
	void Add();
	void Del(int number = CURRENT_FRAME);
	void Insert(int number = CURRENT_FRAME);

	int Count() const {return max_number;}
	void Choice(int number);
	bool WasInit();
	BMotion& operator[](int number);
	BMotions& operator=(const BMotions &cbmotions);

	void SetRhythm() {bmotions[MOVE_RHYTHM].SetFrame(0);}
	void NextRhythm() {bmotions[MOVE_RHYTHM].NextFrame();}
	BFrame& MotionRhythm() {return bmotions[MOVE_RHYTHM].CurrentFrame();}

	void SetAction(int number) {
		cmotion.Link(MotionRhythm(), bmotions[number]);
		cmotion.SetFrame(0);
	}
	void NextAction() {cmotion.NextFrame();}
	BFrame& MotionAction() {return cmotion.CurrentFrame();}
	bool IsActionEnd() {
		if (cmotion.TotalCut() == cmotion.CutNumber()){ 
			return true;
		} else {
			return false;		
		}
	}
	
//	Load();
	BCombo combo;
	int assign[MAX_ASSIGN];
private:
	BMotion *bmotions;
	BComboMotion cmotion;
	int max_number, current_number;
};

#endif // BFRAME_H
