#include "bodypage.h"
#include "myglcanvas.h"
#include "myslider.h"
#include "boxer.h"

myPart::myPart(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name) : wxPanel(parent, -1) {
	canvas = ccanvas;
	tbutton = new wxToggleButton(parent, -1, name);			
	Hide();
}

myArm::myArm(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright)
	: myPart(parent, ccanvas, name) {
	right = cright;
	SetProperties();
	DoLayout();
}

void myArm::SetProperties() {
	fist_r = new mySlider(this, canvas, wxT("R : "), 10, 100);
	forearm_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 100);
	forearm_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 100);
	forearm_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 100);
	forearm_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 100);
	forearm_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	forearm_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	upperarm_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 100);
	upperarm_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 100);
	upperarm_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 100);
	upperarm_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 100);
	upperarm_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	upperarm_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	shoulder_r = new mySlider(this, canvas, wxT("R : "), 10, 100);
	fist_r->SetValue(75);
	forearm_a1->SetValue( 50);
	forearm_a2->SetValue( 50);
	forearm_b1->SetValue( 40);
	forearm_b2->SetValue( 40);
	forearm_c1->SetValue(100);
	forearm_c2->SetValue(200);
	upperarm_a1->SetValue( 55);
	upperarm_a2->SetValue( 55);
	upperarm_b1->SetValue( 65);
	upperarm_b2->SetValue( 55);
	upperarm_c1->SetValue( 50);
	upperarm_c2->SetValue(150);
	shoulder_r->SetValue(70);
}

void myArm::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer4 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxHORIZONTAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer4, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(fist_r, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(forearm_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(forearm_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(forearm_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(forearm_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(forearm_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(forearm_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(upperarm_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(upperarm_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(upperarm_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(upperarm_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(upperarm_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(upperarm_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer4->Add(shoulder_r, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

myLeg::myLeg(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright)
	: myPart(parent, ccanvas, name) {
	right = cright;
	SetProperties();
	DoLayout();
}

void myLeg::SetProperties() {
	foot_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 100);
	foot_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 100);
	foot_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	foot_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	foot_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 100);
	shank_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 100);
	shank_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 100);
	shank_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 100);
	shank_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 100);
	shank_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	shank_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	thigh_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 200);
	thigh_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 200);
	thigh_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	thigh_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	thigh_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	thigh_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 800); 
	if (right) {
		foot_a1->SetValue( 40);
		foot_a2->SetValue( 60);
	} else {
		foot_a1->SetValue( 60);
		foot_a2->SetValue( 40);	
	}
	foot_b1->SetValue( 60);
	foot_b2->SetValue(150);
	foot_c1->SetValue( 40);
	shank_a1->SetValue( 60);
	shank_a2->SetValue( 60);
	shank_b1->SetValue( 70);
	shank_b2->SetValue( 60);
	shank_c1->SetValue(210);
	shank_c2->SetValue(310);
	thigh_a1->SetValue( 80);
	thigh_a2->SetValue( 80);
	thigh_b1->SetValue( 80);
	thigh_b2->SetValue( 80);
	thigh_c1->SetValue(120);
	thigh_c2->SetValue(500);
}

void myLeg::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("­")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(foot_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(foot_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(foot_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(foot_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(foot_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(shank_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(shank_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(shank_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(shank_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(shank_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(shank_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(thigh_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(thigh_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(thigh_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(thigh_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(thigh_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(thigh_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
} 

myBody::myBody(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name)
	: myPart(parent, ccanvas, name) {
	SetProperties();
	DoLayout();
}

void myBody::SetProperties() {
	waist_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 200);
	waist_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 200);
	waist_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	waist_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	waist_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	waist_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	abdomen_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 200);
	abdomen_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 200);
	abdomen_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	abdomen_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	abdomen_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	abdomen_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	breast_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 400);
	breast_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 400);
	breast_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 400);
	breast_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 400);
	breast_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	breast_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	waist_a1->SetValue(125);
	waist_a2->SetValue(125);
	waist_b1->SetValue(100);
	waist_b2->SetValue(125);
	waist_c1->SetValue(175);
	waist_c2->SetValue( 50);
	abdomen_a1->SetValue(100);
	abdomen_a2->SetValue(100);
	abdomen_b1->SetValue( 60);
	abdomen_b2->SetValue( 80);
	abdomen_c1->SetValue(150);
	abdomen_c2->SetValue(150);
	breast_a1->SetValue(170);
	breast_a2->SetValue(170);
	breast_b1->SetValue(110);
	breast_b2->SetValue(130);
	breast_c1->SetValue(200);
	breast_c2->SetValue( 75);
}

void myBody::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ʢ")), wxVERTICAL);
  	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
 
	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(waist_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(waist_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(waist_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(waist_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(waist_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(waist_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(abdomen_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(abdomen_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(abdomen_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(abdomen_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(abdomen_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(abdomen_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(breast_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(breast_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(breast_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(breast_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(breast_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer3->Add(breast_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
} 

myHead::myHead(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name)
	: myPart(parent, ccanvas, name) {
	SetProperties();
	DoLayout();
}

void myHead::SetProperties() {
	neck_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 200);
	neck_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 200);
	neck_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	neck_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	neck_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 200);
	neck_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 200);
	face_a1 = new mySlider(this, canvas, wxT("A1 : "), 10, 200);
	face_a2 = new mySlider(this, canvas, wxT("A2 : "), 10, 200);
	face_b1 = new mySlider(this, canvas, wxT("B1 : "), 10, 200);
	face_b2 = new mySlider(this, canvas, wxT("B2 : "), 10, 200);
	face_c1 = new mySlider(this, canvas, wxT("C1 : "), 10, 400);
	face_c2 = new mySlider(this, canvas, wxT("C2 : "), 10, 400);
	neck_a1->SetValue(60);
	neck_a2->SetValue(60);
	neck_b1->SetValue(60);
	neck_b2->SetValue(60);
	neck_c1->SetValue(70);
	neck_c2->SetValue(70);
	face_a1->SetValue(100);
	face_a2->SetValue(100);
	face_b1->SetValue( 80);
	face_b2->SetValue(100);
	face_c1->SetValue(180);
	face_c2->SetValue(120);
}

void myHead::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
 
	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(neck_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(face_a1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_a2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_b1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_b2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_c1, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_c2, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
} 

BEGIN_EVENT_TABLE(BodyPage, wxScrolledWindow)
	EVT_TOGGLEBUTTON(-1, BodyPage::OnToggle) 
	EVT_BUTTON(ON_MORE_WEIGHT_ID, BodyPage::OnMoreWeight) 
	EVT_BUTTON(ON_LESS_WEIGHT_ID, BodyPage::OnLessWeight) 
	EVT_BUTTON(ON_MORE_HEIGHT_ID, BodyPage::OnMoreHeight) 
	EVT_BUTTON(ON_LESS_HEIGHT_ID, BodyPage::OnLessHeight) 
END_EVENT_TABLE()

BodyPage::BodyPage(wxWindow* parent, myGLCanvas *ccanvas) : wxScrolledWindow(parent, -1) {
	canvas = ccanvas;
	SetProperties();
	DoLayout();
}

void BodyPage::SetProperties() {
	int osv, mav, miv;
	osv = wxGetOsVersion(&mav, &miv);

	SetScrollRate(10,10);
	myPart *right_arm = new myArm(this, canvas, wxT(""));
	myPart *left_arm = new myArm(this, canvas, wxT(""), false);
	myPart *right_leg = new myLeg(this, canvas, wxT(""));
	myPart *left_leg = new myLeg(this, canvas, wxT(""), false);
	myPart *body = new myBody(this, canvas, wxT("ƹ")); 
	myPart *head = new myHead(this, canvas, wxT("Ƭ"));

	parts.Add(right_arm);
	parts.Add(right_leg);
	parts.Add(body);
	parts.Add(left_arm);
	parts.Add(left_leg);
	parts.Add(head);
	max_parts = parts.Count();
	select_part = 0;

	height = new wxStaticText(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize);
	weight = new wxStaticText(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize);
	reach = new wxStaticText(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize);
//	height->SetWindowStyle(wxSIMPLE_BORDER);
	if (osv != wxWIN386 && osv != wxWIN95 && osv != wxWIN32S) {
		height->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxNORMAL));
		weight->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxNORMAL));
 		reach->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxNORMAL));
	}
	moreweight = new wxButton(this, ON_MORE_WEIGHT_ID, wxT(""));
	lessweight = new wxButton(this, ON_LESS_WEIGHT_ID, wxT(""));
	moreheight = new wxButton(this, ON_MORE_HEIGHT_ID, wxT(""));
	lessheight = new wxButton(this, ON_LESS_HEIGHT_ID, wxT(""));

	s_width = new mySlider(this, canvas, wxT(" : "), 10, 400);
	w_width = new mySlider(this, canvas, wxT(" : "), 10, 400);
	s_width->SetValue(200);
	w_width->SetValue(80);

	GetPart()->tbutton->SetValue(true);
	gsizer = new wxGridSizer(max_parts/3, 3, 0, 0);
    tsizer = new wxBoxSizer(wxVERTICAL);
}

void BodyPage::DoLayout() {
	int i;
	wxStaticBoxSizer *bsizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("Ĺν")), wxHORIZONTAL);
	wxStaticBoxSizer *bsizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ȹ")), wxVERTICAL);
	wxStaticBoxSizer *bsizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ĹνŤ")), wxVERTICAL);
	wxStaticBoxSizer *bsizer4 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);

	wxGridSizer *gsizer2 = new wxGridSizer(2, 2, 0, 0);

	bsizer1->Add(height, 1, wxGROW | wxALL, 5);
	bsizer1->Add(weight, 1, wxGROW | wxALL, 5);
	bsizer1->Add(reach, 1, wxGROW | wxALL, 5);

 	bsizer2->Add(s_width, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
 	bsizer2->Add(w_width, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	bsizer3->Add(gsizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	bsizer4->Add(gsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	gsizer2->Add(moreweight,  1, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	gsizer2->Add(lessweight,  1, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	gsizer2->Add(moreheight,  1, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	gsizer2->Add(lessheight,  1, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);

	tsizer->Add(bsizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(bsizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(bsizer1, 0, wxGROW | wxALL, 5);
	tsizer->Add(bsizer4, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	for (i = 0; i < max_parts; i++) {
		gsizer->Add(parts[i]->tbutton,  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
	GetPart()->Show();	
	tsizer->Add(GetPart(), 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	SetSizer(tsizer);
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
    Layout();
}

void BodyPage::SetSelectPart(int new_select) {
	parts[new_select]->tbutton->SetValue(true);

	if (new_select != SelectPart()) {
		GetPart()->tbutton->SetValue(false);
		tsizer->Remove(GetPart());
		parts[SelectPart()]->Hide();

		select_part = new_select;
		GetPart()->Show();	
		tsizer->Add(GetPart(), 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	}

 	tsizer->SetVirtualSizeHints(this);
	FitInside();
	Layout();
}


void BodyPage::OnToggle(wxCommandEvent &event){
	int i, new_select = 0;
	wxObject *obj = event.GetEventObject();

	for (i = 0; i < MaxPart(); i++) {
		if(obj == (wxObject*) parts[i]->tbutton) {
			new_select = i;
			break;
		}
	}
	SetSelectPart(new_select);
}



void BodyPage::SetLabel(Boxer &boxer) {
	wxString str;
	str.Printf("Ĺ:%5.1fcm", boxer.Height()*100.0);
	height->SetLabel(str);
	str.Printf("ν:%5.1fkg", boxer.Weight()*1000.0);
	weight->SetLabel(str);
 	tsizer->SetVirtualSizeHints(this);
	str.Printf("꡼:%5.1fcm", boxer.Reach()*100.0);
	reach->SetLabel(str);
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
	Layout();
}

void BodyPage::OnMoreWeight(wxCommandEvent &event) {
	myArm *arm = (myArm*)parts[0];
	arm->forearm_a1->ValueUp(1);
	arm->forearm_a2->ValueUp(1);
	arm->forearm_b1->ValueUp(1);
	arm->forearm_b2->ValueUp(1);
	arm->upperarm_a1->ValueUp(1);
	arm->upperarm_a2->ValueUp(1);
	arm->upperarm_b1->ValueUp(1);
	arm->upperarm_b2->ValueUp(1);
	arm->shoulder_r->ValueUp(1);

	arm = (myArm*)parts[3];
	arm->forearm_a1->ValueUp(1);
	arm->forearm_a2->ValueUp(1);
	arm->forearm_b1->ValueUp(1);
	arm->forearm_b2->ValueUp(1);
	arm->upperarm_a1->ValueUp(1);
	arm->upperarm_a2->ValueUp(1);
	arm->upperarm_b1->ValueUp(1);
	arm->upperarm_b2->ValueUp(1);
	arm->shoulder_r->ValueUp(1);

	myLeg *leg = (myLeg*)parts[1];
	leg->foot_a1->ValueUp(1);
	leg->foot_a2->ValueUp(1);
	leg->foot_b1->ValueUp(1);
	leg->foot_b2->ValueUp(1);
	leg->shank_a1->ValueUp(1);
	leg->shank_a2->ValueUp(1);
	leg->shank_b1->ValueUp(1);
	leg->shank_b2->ValueUp(1);
	leg->thigh_a1->ValueUp(1);
	leg->thigh_a2->ValueUp(1);
	leg->thigh_b1->ValueUp(1);
	leg->thigh_b2->ValueUp(1);

	leg = (myLeg*)parts[4];
	leg->foot_a1->ValueUp(1);
	leg->foot_a2->ValueUp(1);
	leg->foot_b1->ValueUp(1);
	leg->foot_b2->ValueUp(1);
	leg->shank_a1->ValueUp(1);
	leg->shank_a2->ValueUp(1);
	leg->shank_b1->ValueUp(1);
	leg->shank_b2->ValueUp(1);
	leg->thigh_a1->ValueUp(1);
	leg->thigh_a2->ValueUp(1);
	leg->thigh_b1->ValueUp(1);
	leg->thigh_b2->ValueUp(1);

	myBody *body = (myBody*)parts[2];
	body->waist_a1->ValueUp(1);
	body->waist_a2->ValueUp(1);
	body->waist_b1->ValueUp(1);
	body->waist_b2->ValueUp(1);
	body->abdomen_a1->ValueUp(1);
	body->abdomen_a2->ValueUp(1);
	body->abdomen_b1->ValueUp(1);
	body->abdomen_b2->ValueUp(1);
	body->breast_a1->ValueUp(1);
	body->breast_a2->ValueUp(1);
	body->breast_b1->ValueUp(1);
	body->breast_b2->ValueUp(1);
	
	myHead *head = (myHead*)parts[5];
    head->neck_a1->ValueUp(1);
	head->neck_a2->ValueUp(1);
	head->neck_b1->ValueUp(1);
	head->neck_b2->ValueUp(1);
	head->face_a1->ValueUp(1);
	head->face_a2->ValueUp(1);
	head->face_b1->ValueUp(1);
	head->face_b2->ValueUp(1);
	canvas->Refresh();
}

void BodyPage::OnLessWeight(wxCommandEvent &event) {
	myArm *arm = (myArm*)parts[0];
	arm->forearm_a1->ValueDown(1);
	arm->forearm_a2->ValueDown(1);
	arm->forearm_b1->ValueDown(1);
	arm->forearm_b2->ValueDown(1);
	arm->upperarm_a1->ValueDown(1);
	arm->upperarm_a2->ValueDown(1);
	arm->upperarm_b1->ValueDown(1);
	arm->upperarm_b2->ValueDown(1);
	arm->shoulder_r->ValueDown(1);

	arm = (myArm*)parts[3];
	arm->forearm_a1->ValueDown(1);
	arm->forearm_a2->ValueDown(1);
	arm->forearm_b1->ValueDown(1);
	arm->forearm_b2->ValueDown(1);
	arm->upperarm_a1->ValueDown(1);
	arm->upperarm_a2->ValueDown(1);
	arm->upperarm_b1->ValueDown(1);
	arm->upperarm_b2->ValueDown(1);
	arm->shoulder_r->ValueDown(1);

	myLeg *leg = (myLeg*)parts[1];
	leg->foot_a1->ValueDown(1);
	leg->foot_a2->ValueDown(1);
	leg->foot_b1->ValueDown(1);
	leg->foot_b2->ValueDown(1);
	leg->shank_a1->ValueDown(1);
	leg->shank_a2->ValueDown(1);
	leg->shank_b1->ValueDown(1);
	leg->shank_b2->ValueDown(1);
	leg->thigh_a1->ValueDown(1);
	leg->thigh_a2->ValueDown(1);
	leg->thigh_b1->ValueDown(1);
	leg->thigh_b2->ValueDown(1);

	leg = (myLeg*)parts[4];
	leg->foot_a1->ValueDown(1);
	leg->foot_a2->ValueDown(1);
	leg->foot_b1->ValueDown(1);
	leg->foot_b2->ValueDown(1);
	leg->shank_a1->ValueDown(1);
	leg->shank_a2->ValueDown(1);
	leg->shank_b1->ValueDown(1);
	leg->shank_b2->ValueDown(1);
	leg->thigh_a1->ValueDown(1);
	leg->thigh_a2->ValueDown(1);
	leg->thigh_b1->ValueDown(1);
	leg->thigh_b2->ValueDown(1);

	myBody *body = (myBody*)parts[2];
	body->waist_a1->ValueDown(1);
	body->waist_a2->ValueDown(1);
	body->waist_b1->ValueDown(1);
	body->waist_b2->ValueDown(1);
	body->abdomen_a1->ValueDown(1);
	body->abdomen_a2->ValueDown(1);
	body->abdomen_b1->ValueDown(1);
	body->abdomen_b2->ValueDown(1);
	body->breast_a1->ValueDown(1);
	body->breast_a2->ValueDown(1);
	body->breast_b1->ValueDown(1);
	body->breast_b2->ValueDown(1);
	
	myHead *head = (myHead*)parts[5];
    head->neck_a1->ValueDown(1);
	head->neck_a2->ValueDown(1);
	head->neck_b1->ValueDown(1);
	head->neck_b2->ValueDown(1);
	head->face_a1->ValueDown(1);
	head->face_a2->ValueDown(1);
	head->face_b1->ValueDown(1);
	head->face_b2->ValueDown(1);
	canvas->Refresh();
}

void BodyPage::OnMoreHeight(wxCommandEvent &event) {
	myArm *arm = (myArm*)parts[0];
	arm->forearm_c1->ValueUp(1);
	arm->forearm_c2->ValueUp(1);
	arm->upperarm_c1->ValueUp(1);
	arm->upperarm_c2->ValueUp(1);

	arm = (myArm*)parts[3];
	arm->forearm_c1->ValueUp(1);
	arm->forearm_c2->ValueUp(1);
	arm->upperarm_c1->ValueUp(1);
	arm->upperarm_c2->ValueUp(1);

	myLeg *leg = (myLeg*)parts[1];
	leg->foot_c1->ValueUp(1);
	leg->shank_c1->ValueUp(1);
	leg->shank_c2->ValueUp(1);
	leg->thigh_c1->ValueUp(1);
	leg->thigh_c2->ValueUp(1);

	leg = (myLeg*)parts[4];
	leg->foot_c1->ValueUp(1);
	leg->shank_c1->ValueUp(1);
	leg->shank_c2->ValueUp(1);
	leg->thigh_c1->ValueUp(1);
	leg->thigh_c2->ValueUp(1);

	myBody *body = (myBody*)parts[2];
	body->waist_c1->ValueUp(1);
	body->waist_c2->ValueUp(1);
	body->abdomen_c1->ValueUp(1);
	body->abdomen_c2->ValueUp(1);
	body->breast_c1->ValueUp(1);
	body->breast_c2->ValueUp(1);
	
	myHead *head = (myHead*)parts[5];
	head->neck_c1->ValueUp(1);
	head->neck_c2->ValueUp(1);
	head->face_c1->ValueUp(1);
	head->face_c2->ValueUp(1);
	canvas->Refresh();
}

void BodyPage::OnLessHeight(wxCommandEvent &event) {
	myArm *arm = (myArm*)parts[0];
	arm->forearm_c1->ValueDown(1);
	arm->forearm_c2->ValueDown(1);
	arm->upperarm_c1->ValueDown(1);
	arm->upperarm_c2->ValueDown(1);

	arm = (myArm*)parts[3];
	arm->forearm_c1->ValueDown(1);
	arm->forearm_c2->ValueDown(1);
	arm->upperarm_c1->ValueDown(1);
	arm->upperarm_c2->ValueDown(1);

	myLeg *leg = (myLeg*)parts[1];
	leg->foot_c1->ValueDown(1);
	leg->shank_c1->ValueDown(1);
	leg->shank_c2->ValueDown(1);
	leg->thigh_c1->ValueDown(1);
	leg->thigh_c2->ValueDown(1);

	leg = (myLeg*)parts[4];
	leg->foot_c1->ValueDown(1);
	leg->shank_c1->ValueDown(1);
	leg->shank_c2->ValueDown(1);
	leg->thigh_c1->ValueDown(1);
	leg->thigh_c2->ValueDown(1);

	myBody *body = (myBody*)parts[2];
	body->waist_c1->ValueDown(1);
	body->waist_c2->ValueDown(1);
	body->abdomen_c1->ValueDown(1);
	body->abdomen_c2->ValueDown(1);
	body->breast_c1->ValueDown(1);
	body->breast_c2->ValueDown(1);
	
	myHead *head = (myHead*)parts[5];
	head->neck_c1->ValueDown(1);
	head->neck_c2->ValueDown(1);
	head->face_c1->ValueDown(1);
	head->face_c2->ValueDown(1);
	canvas->Refresh();
}

void BodyPage::GetBody(Boxer &boxer) {
	s_width->SetValueF(boxer.ShoulderWidth(), 1000);
	w_width->SetValueF(boxer.WaistWidth(), 1000);

	myArm *arm = (myArm*)parts[0];
	arm->fist_r->SetValueF(boxer.right_arm.fist.A1(), 1000);
	arm->forearm_a1->SetValueF(boxer.right_arm.forearm.A1(), 1000);
	arm->forearm_a2->SetValueF(boxer.right_arm.forearm.A2(), 1000);
	arm->forearm_b1->SetValueF(boxer.right_arm.forearm.B1(), 1000);
	arm->forearm_b2->SetValueF(boxer.right_arm.forearm.B2(), 1000);
	arm->forearm_c1->SetValueF(boxer.right_arm.forearm.C1(), 1000);
	arm->forearm_c2->SetValueF(boxer.right_arm.forearm.C2(), 1000);
	arm->upperarm_a1->SetValueF(boxer.right_arm.upperarm.A1(), 1000);
	arm->upperarm_a2->SetValueF(boxer.right_arm.upperarm.A2(), 1000);
	arm->upperarm_b1->SetValueF(boxer.right_arm.upperarm.B1(), 1000);
	arm->upperarm_b2->SetValueF(boxer.right_arm.upperarm.B2(), 1000);
	arm->upperarm_c1->SetValueF(boxer.right_arm.upperarm.C1(), 1000);
	arm->upperarm_c2->SetValueF(boxer.right_arm.upperarm.C2(), 1000);
	arm->shoulder_r->SetValueF(boxer.right_arm.shoulder.A1(), 1000);

	arm = (myArm*)parts[3];
	arm->fist_r->SetValueF(boxer.left_arm.fist.A1(), 1000);
	arm->forearm_a1->SetValueF(boxer.left_arm.forearm.A1(), 1000);
	arm->forearm_a2->SetValueF(boxer.left_arm.forearm.A2(), 1000);
	arm->forearm_b1->SetValueF(boxer.left_arm.forearm.B1(), 1000);
	arm->forearm_b2->SetValueF(boxer.left_arm.forearm.B2(), 1000);
	arm->forearm_c1->SetValueF(boxer.left_arm.forearm.C1(), 1000);
	arm->forearm_c2->SetValueF(boxer.left_arm.forearm.C2(), 1000);
	arm->upperarm_a1->SetValueF(boxer.left_arm.upperarm.A1(), 1000);
	arm->upperarm_a2->SetValueF(boxer.left_arm.upperarm.A2(), 1000);
	arm->upperarm_b1->SetValueF(boxer.left_arm.upperarm.B1(), 1000);
	arm->upperarm_b2->SetValueF(boxer.left_arm.upperarm.B2(), 1000);
	arm->upperarm_c1->SetValueF(boxer.left_arm.upperarm.C1(), 1000);
	arm->upperarm_c2->SetValueF(boxer.left_arm.upperarm.C2(), 1000);
	arm->shoulder_r->SetValueF(boxer.left_arm.shoulder.A1(), 1000);

	myLeg *leg = (myLeg*)parts[1];
	leg->foot_a1->SetValueF(boxer.right_leg.toe.A1(), 1000);
	leg->foot_a2->SetValueF(boxer.right_leg.toe.A2(), 1000);
	leg->foot_b1->SetValueF(boxer.right_leg.toe.B1(), 1000);
	leg->foot_b2->SetValueF(boxer.right_leg.foot.B2(), 1000);
	leg->foot_c1->SetValueF(boxer.right_leg.foot.C1(), 1000);
	leg->shank_a1->SetValueF(boxer.right_leg.shank.A1(), 1000);
	leg->shank_a2->SetValueF(boxer.right_leg.shank.A2(), 1000);
	leg->shank_b1->SetValueF(boxer.right_leg.shank.B1(), 1000);
	leg->shank_b2->SetValueF(boxer.right_leg.shank.B2(), 1000);
	leg->shank_c1->SetValueF(boxer.right_leg.shank.C1(), 1000);
	leg->shank_c2->SetValueF(boxer.right_leg.shank.C2(), 1000);
	leg->thigh_a1->SetValueF(boxer.right_leg.thigh.A1(), 1000);
	leg->thigh_a2->SetValueF(boxer.right_leg.thigh.A1(), 1000);
	leg->thigh_b1->SetValueF(boxer.right_leg.thigh.B1(), 1000);
	leg->thigh_b2->SetValueF(boxer.right_leg.thigh.B2(), 1000);
	leg->thigh_c1->SetValueF(boxer.right_leg.thigh.C1(), 1000);
	leg->thigh_c2->SetValueF(boxer.right_leg.thigh.C2(), 1000);

	leg = (myLeg*)parts[4];
	leg->foot_a1->SetValueF(boxer.left_leg.toe.A1(), 1000);
	leg->foot_a2->SetValueF(boxer.left_leg.toe.A2(), 1000);
	leg->foot_b1->SetValueF(boxer.left_leg.toe.B1(), 1000);
	leg->foot_b2->SetValueF(boxer.left_leg.foot.B2(), 1000);
	leg->foot_c1->SetValueF(boxer.left_leg.foot.C1(), 1000);
	leg->shank_a1->SetValueF(boxer.left_leg.shank.A1(), 1000);
	leg->shank_a2->SetValueF(boxer.left_leg.shank.A2(), 1000);
	leg->shank_b1->SetValueF(boxer.left_leg.shank.B1(), 1000);
	leg->shank_b2->SetValueF(boxer.left_leg.shank.B2(), 1000);
	leg->shank_c1->SetValueF(boxer.left_leg.shank.C1(), 1000);
	leg->shank_c2->SetValueF(boxer.left_leg.shank.C2(), 1000);
	leg->thigh_a1->SetValueF(boxer.left_leg.thigh.A1(), 1000);
	leg->thigh_a2->SetValueF(boxer.left_leg.thigh.A1(), 1000);
	leg->thigh_b1->SetValueF(boxer.left_leg.thigh.B1(), 1000);
	leg->thigh_b2->SetValueF(boxer.left_leg.thigh.B2(), 1000);
	leg->thigh_c1->SetValueF(boxer.left_leg.thigh.C1(), 1000);
	leg->thigh_c2->SetValueF(boxer.left_leg.thigh.C2(), 1000);

	myBody *body = (myBody*)parts[2];
	body->waist_a1->SetValueF(boxer.body.waist.A1(), 1000);
	body->waist_a2->SetValueF(boxer.body.waist.A1(), 1000);
	body->waist_b1->SetValueF(boxer.body.waist.B1(), 1000);
	body->waist_b2->SetValueF(boxer.body.waist.B2(), 1000);
	body->waist_c1->SetValueF(boxer.body.waist.C1(), 1000);
	body->waist_c2->SetValueF(boxer.body.waist.C2(), 1000);
	body->abdomen_a1->SetValueF(boxer.body.abdomen.A1(), 1000);
	body->abdomen_a2->SetValueF(boxer.body.abdomen.A1(), 1000);
	body->abdomen_b1->SetValueF(boxer.body.abdomen.B1(), 1000);
	body->abdomen_b2->SetValueF(boxer.body.abdomen.B2(), 1000);
	body->abdomen_c1->SetValueF(boxer.body.abdomen.C1(), 1000);
	body->abdomen_c2->SetValueF(boxer.body.abdomen.C2(), 1000);
	body->breast_a1->SetValueF(boxer.body.breast.A1(), 1000);
	body->breast_a2->SetValueF(boxer.body.breast.A1(), 1000);
	body->breast_b1->SetValueF(boxer.body.breast.B1(), 1000);
	body->breast_b2->SetValueF(boxer.body.breast.B2(), 1000);
	body->breast_c1->SetValueF(boxer.body.breast.C1(), 1000);
	body->breast_c2->SetValueF(boxer.body.breast.C2(), 1000);

	myHead *head = (myHead*)parts[5];
	head->neck_a1->SetValueF(boxer.head.neck.A1(), 1000);
	head->neck_a2->SetValueF(boxer.head.neck.A1(), 1000);
	head->neck_b1->SetValueF(boxer.head.neck.B1(), 1000);
	head->neck_b2->SetValueF(boxer.head.neck.B2(), 1000);
	head->neck_c1->SetValueF(boxer.head.neck.C1(), 1000);
	head->neck_c2->SetValueF(boxer.head.neck.C2(), 1000);
	head->face_a1->SetValueF(boxer.head.face.A1(), 1000);
	head->face_a2->SetValueF(boxer.head.face.A1(), 1000);
	head->face_b1->SetValueF(boxer.head.face.B1(), 1000);
	head->face_b2->SetValueF(boxer.head.face.B2(), 1000);
	head->face_c1->SetValueF(boxer.head.face.C1(), 1000);
	head->face_c2->SetValueF(boxer.head.face.C2(), 1000);
}

void BodyPage::SetBody(Boxer &boxer) {
	boxer.SetShoulderWidth(s_width->GetValueF(1000));
	boxer.SetWaistWidth(w_width->GetValueF(1000));

	myArm *arm = (myArm*)parts[0];
	boxer.right_arm.fist.SetSize(
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000));
	boxer.right_arm.forearm.SetSize(
		arm->forearm_a1->GetValueF(1000),
		arm->forearm_a2->GetValueF(1000),
		arm->forearm_b1->GetValueF(1000),
		arm->forearm_b2->GetValueF(1000),
		arm->forearm_c1->GetValueF(1000),
		arm->forearm_c2->GetValueF(1000));
	boxer.right_arm.upperarm.SetSize(
		arm->upperarm_a1->GetValueF(1000),
		arm->upperarm_a2->GetValueF(1000),
		arm->upperarm_b1->GetValueF(1000),
		arm->upperarm_b2->GetValueF(1000),
		arm->upperarm_c1->GetValueF(1000),
		arm->upperarm_c2->GetValueF(1000));
	boxer.right_arm.shoulder.SetSize(
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000));

	arm = (myArm*)parts[3];
	boxer.left_arm.fist.SetSize(
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000),
		arm->fist_r->GetValueF(1000));
	boxer.left_arm.forearm.SetSize(
		arm->forearm_a1->GetValueF(1000),
		arm->forearm_a2->GetValueF(1000),
		arm->forearm_b1->GetValueF(1000),
		arm->forearm_b2->GetValueF(1000),
		arm->forearm_c1->GetValueF(1000),
		arm->forearm_c2->GetValueF(1000));
	boxer.left_arm.upperarm.SetSize(
		arm->upperarm_a1->GetValueF(1000),
		arm->upperarm_a2->GetValueF(1000),
		arm->upperarm_b1->GetValueF(1000),
		arm->upperarm_b2->GetValueF(1000),
		arm->upperarm_c1->GetValueF(1000),
		arm->upperarm_c2->GetValueF(1000));
	boxer.left_arm.shoulder.SetSize(
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000),
		arm->shoulder_r->GetValueF(1000));

	myLeg *leg = (myLeg*)parts[1];
	boxer.right_leg.toe.SetSize(
		leg->foot_a1->GetValueF(1000),
		leg->foot_a2->GetValueF(1000),
		leg->foot_b1->GetValueF(1000),
		leg->foot_b2->GetValueF(1000),
		leg->foot_c1->GetValueF(1000), 0.0);
	boxer.right_leg.foot.SetSize(
		leg->foot_a1->GetValueF(1000),
		leg->foot_a2->GetValueF(1000),
		leg->foot_b1->GetValueF(1000),
		leg->foot_b2->GetValueF(1000),
		leg->foot_c1->GetValueF(1000), 0.0);
	boxer.right_leg.shank.SetSize(
		leg->shank_a1->GetValueF(1000),
		leg->shank_a2->GetValueF(1000),
		leg->shank_b1->GetValueF(1000),
		leg->shank_b2->GetValueF(1000),
		leg->shank_c1->GetValueF(1000),
		leg->shank_c2->GetValueF(1000));
	boxer.right_leg.thigh.SetSize(
		leg->thigh_a1->GetValueF(1000),
		leg->thigh_a2->GetValueF(1000),
		leg->thigh_b1->GetValueF(1000),
		leg->thigh_b2->GetValueF(1000),
		leg->thigh_c1->GetValueF(1000),
		leg->thigh_c2->GetValueF(1000));

	leg = (myLeg*)parts[4];
	boxer.left_leg.toe.SetSize(
		leg->foot_a1->GetValueF(1000),
		leg->foot_a2->GetValueF(1000),
		leg->foot_b1->GetValueF(1000),
		leg->foot_b2->GetValueF(1000),
		leg->foot_c1->GetValueF(1000), 0.0);
	boxer.left_leg.foot.SetSize(
		leg->foot_a1->GetValueF(1000),
		leg->foot_a2->GetValueF(1000),
		leg->foot_b1->GetValueF(1000),
		leg->foot_b2->GetValueF(1000),
		leg->foot_c1->GetValueF(1000), 0.0);
	boxer.left_leg.shank.SetSize(
		leg->shank_a1->GetValueF(1000),
		leg->shank_a2->GetValueF(1000),
		leg->shank_b1->GetValueF(1000),
		leg->shank_b2->GetValueF(1000),
		leg->shank_c1->GetValueF(1000),
		leg->shank_c2->GetValueF(1000));
	boxer.left_leg.thigh.SetSize(
		leg->thigh_a1->GetValueF(1000),
		leg->thigh_a2->GetValueF(1000),
		leg->thigh_b1->GetValueF(1000),
		leg->thigh_b2->GetValueF(1000),
		leg->thigh_c1->GetValueF(1000),
		leg->thigh_c2->GetValueF(1000));

	myBody *body = (myBody*)parts[2];
	boxer.body.waist.SetSize(
		body->waist_a1->GetValueF(1000),
		body->waist_a2->GetValueF(1000),
		body->waist_b1->GetValueF(1000),
		body->waist_b2->GetValueF(1000),
		body->waist_c1->GetValueF(1000),
		body->waist_c2->GetValueF(1000));
	boxer.body.abdomen.SetSize(
		body->abdomen_a1->GetValueF(1000),
		body->abdomen_a2->GetValueF(1000),
		body->abdomen_b1->GetValueF(1000),
		body->abdomen_b2->GetValueF(1000),
		body->abdomen_c1->GetValueF(1000),
		body->abdomen_c2->GetValueF(1000));
	boxer.body.breast.SetSize(
		body->breast_a1->GetValueF(1000),
		body->breast_a2->GetValueF(1000),
		body->breast_b1->GetValueF(1000),
		body->breast_b2->GetValueF(1000),
		body->breast_c1->GetValueF(1000),
		body->breast_c2->GetValueF(1000));

	myHead *head = (myHead*)parts[5];
	boxer.head.neck.SetSize(
		head->neck_a1->GetValueF(1000),
		head->neck_a2->GetValueF(1000),
		head->neck_b1->GetValueF(1000),
		head->neck_b2->GetValueF(1000),
		head->neck_c1->GetValueF(1000),
		head->neck_c2->GetValueF(1000));
	boxer.head.face.SetSize(
		head->face_a1->GetValueF(1000),
		head->face_a2->GetValueF(1000),
		head->face_b1->GetValueF(1000),
		head->face_b2->GetValueF(1000),
		head->face_c1->GetValueF(1000),
		head->face_c2->GetValueF(1000));
}

bool BodyPage::Save(wxTextOutputStream &out) {
	out << wxT("#Body Data") << endl;
	out << wxT("#Select Part") << endl;
	out << SelectPart() << endl;
	out << wxT("#Shoulder Width") << endl;
	out << s_width->GetValue() << endl;
	out << wxT("#Waist Width") << endl;
	out << w_width->GetValue() << endl;

	myArm *arm = (myArm*)parts[0];
	out << wxT("#Right Arm") << endl;
	out << arm->fist_r->GetValue() << endl;
	out << arm->forearm_a1->GetValue() << endl;
	out << arm->forearm_a2->GetValue() << endl;
	out << arm->forearm_b1->GetValue() << endl;
	out << arm->forearm_b2->GetValue() << endl;
	out << arm->forearm_c1->GetValue() << endl;
	out << arm->forearm_c2->GetValue() << endl;
	out << arm->upperarm_a1->GetValue() << endl;
	out << arm->upperarm_a2->GetValue() << endl;
	out << arm->upperarm_b1->GetValue() << endl;
	out << arm->upperarm_b2->GetValue() << endl;
	out << arm->upperarm_c1->GetValue() << endl;
	out << arm->upperarm_c2->GetValue() << endl;
	out << arm->shoulder_r->GetValue() << endl;

	arm = (myArm*)parts[3];
	out << wxT("#Left Arm") << endl;
	out << arm->fist_r->GetValue() << endl;
	out << arm->forearm_a1->GetValue() << endl;
	out << arm->forearm_a2->GetValue() << endl;
	out << arm->forearm_b1->GetValue() << endl;
	out << arm->forearm_b2->GetValue() << endl;
	out << arm->forearm_c1->GetValue() << endl;
	out << arm->forearm_c2->GetValue() << endl;
	out << arm->upperarm_a1->GetValue() << endl;
	out << arm->upperarm_a2->GetValue() << endl;
	out << arm->upperarm_b1->GetValue() << endl;
	out << arm->upperarm_b2->GetValue() << endl;
	out << arm->upperarm_c1->GetValue() << endl;
	out << arm->upperarm_c2->GetValue() << endl;
	out << arm->shoulder_r->GetValue() << endl;

	myLeg *leg = (myLeg*)parts[1];
	out << wxT("#Right Leg") << endl;
	out << leg->foot_a1->GetValue() << endl;
	out << leg->foot_a2->GetValue() << endl;
	out << leg->foot_b1->GetValue() << endl;
	out << leg->foot_b2->GetValue() << endl;
	out << leg->foot_c1->GetValue() << endl;
	out << leg->shank_a1->GetValue() << endl;
	out << leg->shank_a2->GetValue() << endl;
	out << leg->shank_b1->GetValue() << endl;
	out << leg->shank_b2->GetValue() << endl;
	out << leg->shank_c1->GetValue() << endl;
	out << leg->shank_c2->GetValue() << endl;
	out << leg->thigh_a1->GetValue() << endl;
	out << leg->thigh_a2->GetValue() << endl;
	out << leg->thigh_b1->GetValue() << endl;
	out << leg->thigh_b2->GetValue() << endl;
	out << leg->thigh_c1->GetValue() << endl;
	out << leg->thigh_c2->GetValue() << endl;

	leg = (myLeg*)parts[4];
	out << wxT("#Left Leg") << endl;
	out << leg->foot_a1->GetValue() << endl;
	out << leg->foot_a2->GetValue() << endl;
	out << leg->foot_b1->GetValue() << endl;
	out << leg->foot_b2->GetValue() << endl;
	out << leg->foot_c1->GetValue() << endl;
	out << leg->shank_a1->GetValue() << endl;
	out << leg->shank_a2->GetValue() << endl;
	out << leg->shank_b1->GetValue() << endl;
	out << leg->shank_b2->GetValue() << endl;
	out << leg->shank_c1->GetValue() << endl;
	out << leg->shank_c2->GetValue() << endl;
	out << leg->thigh_a1->GetValue() << endl;
	out << leg->thigh_a2->GetValue() << endl;
	out << leg->thigh_b1->GetValue() << endl;
	out << leg->thigh_b2->GetValue() << endl;
	out << leg->thigh_c1->GetValue() << endl;
	out << leg->thigh_c2->GetValue() << endl;

	myBody *body = (myBody*)parts[2];
	out << wxT("#Body") << endl;
	out << body->waist_a1->GetValue() << endl;
	out << body->waist_a2->GetValue() << endl;
	out << body->waist_b1->GetValue() << endl;
	out << body->waist_b2->GetValue() << endl;
	out << body->waist_c1->GetValue() << endl;
	out << body->waist_c2->GetValue() << endl;
	out << body->abdomen_a1->GetValue() << endl;
	out << body->abdomen_a2->GetValue() << endl;
	out << body->abdomen_b1->GetValue() << endl;
	out << body->abdomen_b2->GetValue() << endl;
	out << body->abdomen_c1->GetValue() << endl;
	out << body->abdomen_c2->GetValue() << endl;
	out << body->breast_a1->GetValue() << endl;
	out << body->breast_a2->GetValue() << endl;
	out << body->breast_b1->GetValue() << endl;
	out << body->breast_b2->GetValue() << endl;
	out << body->breast_c1->GetValue() << endl;
	out << body->breast_c2->GetValue() << endl;

	myHead *head = (myHead*)parts[5];
	out << wxT("#Head") << endl;
	out << head->neck_a1->GetValue() << endl;
	out << head->neck_a2->GetValue() << endl;
	out << head->neck_b1->GetValue() << endl;
	out << head->neck_b2->GetValue() << endl;
	out << head->neck_c1->GetValue() << endl;
	out << head->neck_c2->GetValue() << endl;
	out << head->face_a1->GetValue() << endl;
	out << head->face_a2->GetValue() << endl;
	out << head->face_b1->GetValue() << endl;
	out << head->face_b2->GetValue() << endl;
	out << head->face_c1->GetValue() << endl;
	out << head->face_c2->GetValue() << endl;
	return true;
}

bool BodyPage::Load(wxTextInputStream &in) {
	int num;
	wxString str;

	str = in.ReadLine();
//	out << wxT("#Body Data") << endl;
	str = in.ReadLine();
//	out << wxT("#Select Part") << endl;
	in >> num;
	if (num >= MaxPart()) {
		wxLogError("γʥǡ۾Ǥ");
		return false;
	}
	SetSelectPart(num);
	str = in.ReadLine();
	in >> num;
	s_width->SetValue(num);
	str = in.ReadLine();
	in >> num;
	w_width->SetValue(num);

	myArm *arm = (myArm*)parts[0];
	str = in.ReadLine();
	in >> num;
	arm->fist_r->GetValue();
	in >> num;
	arm->forearm_a1->SetValue(num);
	in >> num;
	arm->forearm_a2->SetValue(num);
	in >> num;
	arm->forearm_b1->SetValue(num);
	in >> num;
	arm->forearm_b2->SetValue(num);
	in >> num;
	arm->forearm_c1->SetValue(num);
	in >> num;
	arm->forearm_c2->SetValue(num);
	in >> num;
	arm->upperarm_a1->SetValue(num);
	in >> num;
	arm->upperarm_a2->SetValue(num);
	in >> num;
	arm->upperarm_b1->SetValue(num);
	in >> num;
	arm->upperarm_b2->SetValue(num);
	in >> num;
	arm->upperarm_c1->SetValue(num);
	in >> num;
	arm->upperarm_c2->SetValue(num);
	in >> num;
	arm->shoulder_r->SetValue(num);

	arm = (myArm*)parts[3];
	str = in.ReadLine();
	in >> num;
	arm->fist_r->SetValue(num);
	in >> num;
	arm->forearm_a1->SetValue(num);
	in >> num;
	arm->forearm_a2->SetValue(num);
	in >> num;
	arm->forearm_b1->SetValue(num);
	in >> num;
	arm->forearm_b2->SetValue(num);
	in >> num;
	arm->forearm_c1->SetValue(num);
	in >> num;
	arm->forearm_c2->SetValue(num);
	in >> num;
	arm->upperarm_a1->SetValue(num);
	in >> num;
	arm->upperarm_a2->SetValue(num);
	in >> num;
	arm->upperarm_b1->SetValue(num);
	in >> num;
	arm->upperarm_b2->SetValue(num);
	in >> num;
	arm->upperarm_c1->SetValue(num);
	in >> num;
	arm->upperarm_c2->SetValue(num);
	in >> num;
	arm->shoulder_r->SetValue(num);

	myLeg *leg = (myLeg*)parts[1];
	str = in.ReadLine();
	in >> num;
	leg->foot_a1->SetValue(num);
	in >> num;
	leg->foot_a2->SetValue(num);
	in >> num;
	leg->foot_b1->SetValue(num);
	in >> num;
	leg->foot_b2->SetValue(num);
	in >> num;
	leg->foot_c1->SetValue(num);
	in >> num;
	leg->shank_a1->SetValue(num);
	in >> num;
	leg->shank_a2->SetValue(num);
	in >> num;
	leg->shank_b1->SetValue(num);
	in >> num;
	leg->shank_b2->SetValue(num);
	in >> num;
	leg->shank_c1->SetValue(num);
	in >> num;
	leg->shank_c2->SetValue(num);
	in >> num;
	leg->thigh_a1->SetValue(num);
	in >> num;
	leg->thigh_a2->SetValue(num);
	in >> num;
	leg->thigh_b1->SetValue(num);
	in >> num;
	leg->thigh_b2->SetValue(num);
	in >> num;
	leg->thigh_c1->SetValue(num);
	in >> num;
	leg->thigh_c2->SetValue(num);

	leg = (myLeg*)parts[4];
	str = in.ReadLine();
	in >> num;
	leg->foot_a1->SetValue(num);
	in >> num;
	leg->foot_a2->SetValue(num);
	in >> num;
	leg->foot_b1->SetValue(num);
	in >> num;
	leg->foot_b2->SetValue(num);
	in >> num;
	leg->foot_c1->SetValue(num);
	in >> num;
	leg->shank_a1->SetValue(num);
	in >> num;
	leg->shank_a2->SetValue(num);
	in >> num;
	leg->shank_b1->SetValue(num);
	in >> num;
	leg->shank_b2->SetValue(num);
	in >> num;
	leg->shank_c1->SetValue(num);
	in >> num;
	leg->shank_c2->SetValue(num);
	in >> num;
	leg->thigh_a1->SetValue(num);
	in >> num;
	leg->thigh_a2->SetValue(num);
	in >> num;
	leg->thigh_b1->SetValue(num);
	in >> num;
	leg->thigh_b2->SetValue(num);
	in >> num;
	leg->thigh_c1->SetValue(num);
	in >> num;
	leg->thigh_c2->SetValue(num);

	myBody *body = (myBody*)parts[2];
	str = in.ReadLine();
	in >> num;
	body->waist_a1->SetValue(num);
	in >> num;
	body->waist_a2->SetValue(num);
	in >> num;
	body->waist_b1->SetValue(num);
	in >> num;
	body->waist_b2->SetValue(num);
	in >> num;
	body->waist_c1->SetValue(num);
	in >> num;
	body->waist_c2->SetValue(num);
	in >> num;
	body->abdomen_a1->SetValue(num);
	in >> num;
	body->abdomen_a2->SetValue(num);
	in >> num;
	body->abdomen_b1->SetValue(num);
	in >> num;
	body->abdomen_b2->SetValue(num);
	in >> num;
	body->abdomen_c1->SetValue(num);
	in >> num;
	body->abdomen_c2->SetValue(num);
	in >> num;
	body->breast_a1->SetValue(num);
	in >> num;
	body->breast_a2->SetValue(num);
	in >> num;
	body->breast_b1->SetValue(num);
	in >> num;
	body->breast_b2->SetValue(num);
	in >> num;
	body->breast_c1->SetValue(num);
	in >> num;
	body->breast_c2->SetValue(num);

	myHead *head = (myHead*)parts[5];
	str = in.ReadLine();
	in >> num;
	head->neck_a1->SetValue(num);
	in >> num;
	head->neck_a2->SetValue(num);
	in >> num;
	head->neck_b1->SetValue(num);
	in >> num;
	head->neck_b2->SetValue(num);
	in >> num;
	head->neck_c1->SetValue(num);
	in >> num;
	head->neck_c2->SetValue(num);
	in >> num;
	head->face_a1->SetValue(num);
	in >> num;
	head->face_a2->SetValue(num);
	in >> num;
	head->face_b1->SetValue(num);
	in >> num;
	head->face_b2->SetValue(num);
	in >> num;
	head->face_c1->SetValue(num);
	in >> num;
	head->face_c2->SetValue(num);
	
	canvas->SetBody();	
	canvas->GetBoxer()->MakeDLists();
	SetLabel(*canvas->GetBoxer());
	return true;
}
