#ifndef	BODYPAGE_H
#define BODYPAGE_H

#include <wx/wx.h>
#include <wx/tglbtn.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>

class myGLCanvas;
class mySlider;
class BodyPage;
class Boxer;

class myPart : public wxPanel {
public:
	myPart(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
protected:
    virtual void SetProperties() = 0;
    virtual void DoLayout() = 0;
	wxToggleButton *tbutton;	
	myGLCanvas *canvas;
	friend class BodyPage;
};

WX_DEFINE_ARRAY(myPart*, myParts);

class myArm : public myPart {
public:
	myArm(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright = true);
private:
    void SetProperties();
    void DoLayout();

	mySlider *fist_r;
	mySlider *forearm_a1;
	mySlider *forearm_a2;
	mySlider *forearm_b1;
	mySlider *forearm_b2;
	mySlider *forearm_c1;
	mySlider *forearm_c2;
	mySlider *upperarm_a1;
	mySlider *upperarm_a2;
	mySlider *upperarm_b1;
	mySlider *upperarm_b2;
	mySlider *upperarm_c1;
	mySlider *upperarm_c2;
	mySlider *shoulder_r;
	bool right;
	friend class BodyPage;
};

class myLeg : public myPart {
public:
	myLeg(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright = true);
private:
    void SetProperties();
    void DoLayout();

	mySlider *foot_a1;
	mySlider *foot_a2;
	mySlider *foot_b1;
	mySlider *foot_b2;
	mySlider *foot_c1;
	mySlider *shank_a1;
	mySlider *shank_a2;
	mySlider *shank_b1;
	mySlider *shank_b2;
	mySlider *shank_c1;
	mySlider *shank_c2;
	mySlider *thigh_a1;
	mySlider *thigh_a2;
	mySlider *thigh_b1;
	mySlider *thigh_b2;
	mySlider *thigh_c1;
	mySlider *thigh_c2;
	bool right;
	friend class BodyPage;
};

class myBody : public myPart {
public:
	myBody(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
private:
    void SetProperties();
    void DoLayout();

	mySlider *waist_a1;
	mySlider *waist_a2;
	mySlider *waist_b1;
	mySlider *waist_b2;
	mySlider *waist_c1;
	mySlider *waist_c2;
	mySlider *abdomen_a1;
	mySlider *abdomen_a2;
	mySlider *abdomen_b1;
	mySlider *abdomen_b2;
	mySlider *abdomen_c1;
	mySlider *abdomen_c2;
	mySlider *breast_a1;
	mySlider *breast_a2;
	mySlider *breast_b1;
	mySlider *breast_b2;
	mySlider *breast_c1;
	mySlider *breast_c2;
	friend class BodyPage;
};

class myHead : public myPart {
public:
	myHead(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
private:
    void SetProperties();
    void DoLayout();

	mySlider *face_a1;
	mySlider *face_a2;
	mySlider *face_b1;
	mySlider *face_b2;
	mySlider *face_c1;
	mySlider *face_c2;
	mySlider *neck_a1;
	mySlider *neck_a2;
	mySlider *neck_b1;
	mySlider *neck_b2;
	mySlider *neck_c1;
	mySlider *neck_c2;
	friend class BodyPage;
};

class BodyPage : public wxScrolledWindow {
public:
	BodyPage(wxWindow* parent, myGLCanvas *ccanvas);
	myPart* GetPart() const {return parts[select_part];}
	int MaxPart() const {return max_parts;}
	int SelectPart() const {return select_part;}
	void SetSelectPart(int new_select);
	void GetBody(Boxer &boxer);
	void SetBody(Boxer &boxer);
	void SetLabel(Boxer &boxer);
	bool Save(wxTextOutputStream &out);
	bool Load(wxTextInputStream &in);
private:
    void SetProperties();
    void DoLayout();
	void OnToggle(wxCommandEvent &event);		
	void OnMoreWeight(wxCommandEvent &even);
	void OnLessWeight(wxCommandEvent &even);
	void OnMoreHeight(wxCommandEvent &even);
	void OnLessHeight(wxCommandEvent &even);	

	myParts parts;
	mySlider *s_width, *w_width;
	int max_parts;
	int select_part;
	wxButton *moreweight, *lessweight;
	wxButton *moreheight, *lessheight;	
	wxStaticText *height, *weight, *reach;
	wxBoxSizer *tsizer;
	wxGridSizer *gsizer;
	myGLCanvas *canvas;
	DECLARE_EVENT_TABLE()
};

enum {
	ON_MORE_WEIGHT_ID,
	ON_LESS_WEIGHT_ID,
	ON_MORE_HEIGHT_ID,
	ON_LESS_HEIGHT_ID
};

#endif // BODYPAGE_H
