#include "boxer.h"
#include "glmatrix.h"

static GLMatrix matrix;

BodyPart::BodyPart() : dlist(false) {
	w = 0.0;	l = 0.0;	t = 0.0;
	a = 0.0;	b = 0.0;	g = 0.0;	
	x = 0.0;	y = 0.0;	z = 0.0;	
	gx = 0.0;	gy = 0.0;	gz = 0.0;	
	ox = 0.0;	oy = 0.0;	oz = 0.0;	
	aa = 0.0;	ab = 0.0;	ag = 0.0;	
	a1 = 0.0;	b1 = 0.0;	c1 = 0.0;	
	a2 = 0.0;	b2 = 0.0;	c2 = 0.0;	
}

void BodyPart::SetSize(float ca1, float ca2, float cb1, float cb2, float cc1, float cc2) {
	a1 = ca1;
	a2 = ca2;
	b1 = cb1;
	b2 = cb2;
	c1 = cc1;
	c2 = cc2;
}

void BodyPart::CalcWeight() {
//	w = (a1 + a2) *	(b1 + b2) * (c1 + c2) / 8.0 * 4.0 / 3.0 * PI;
	w = (a1 + a2) *	(b1 + b2) * (c1 + c2) / 6.0 * PI;
	ox = (a2 - a1) * 3.0 / 8.0;
	oy = (b2 - b1) * 3.0 / 8.0;
	oz = (c2 - c1) * 3.0 / 8.0;
}

void BodyPart::CalcAbsAngle() {
	matrix.CalcUnit(1.0, 0.0, 0.0);	
	if (fabs(matrix.X()) > VERY_SMALL) {
		ag = DEG(atan(matrix.Y()/matrix.X()));	
	} else if (fabs(matrix.Y()) < VERY_SMALL) {
		ag = 0.0;	
	} else if (matrix.Y() * matrix.X() > 0.0) {
		ag =  90.0;	
	} else {
		ag = -90.0;		
	}

	matrix.CalcUnit(0.0, 1.0, 0.0);
	if (fabs(matrix.Y()) > VERY_SMALL) {
		aa = DEG(atan(matrix.Z()/matrix.Y()));	
	} else if (fabs(matrix.Z()) < VERY_SMALL) {
		aa = 0.0;	
	} else if (matrix.Z() * matrix.Y() > 0.0) {
	    aa =  90.0;
	} else {
	    aa = -90.0;	
	}

	matrix.CalcUnit(0.0, 0.0, 1.0);	
	if (fabs(matrix.Z()) > VERY_SMALL) {
		ab = DEG(atan(matrix.X()/matrix.Z()));	
	} else if (fabs(matrix.X()) < VERY_SMALL) {
		ab = 0.0;	
	} else if (matrix.X() * matrix.Z() > 0.0) {
		ab =  90.0;	
	} else {
		ab = -90.0;		
	}
}

void BodyPart::InitVelocity() {
	x_pre = x;
	y_pre = y;
	z_pre = z;
	vx = 0.0;
	vy = 0.0;
	vz = 0.0;
/*	ax = 0.0;
	ay = 0.0;
	az = 0.0;  */
}

void BodyPart::CalcVelocity(float gx, float gy, float gz) {
	vx = x - x_pre;
	vy = y - y_pre;
	vz = z - z_pre;
	x_pre = x;
	y_pre = y;
	z_pre = z;

/*	ax = vx - vx_pre;
	ay = vy - vy_pre;
	az = vz - vz_pre;
	vx_pre = vx;
	vy_pre = vy;
	vz_pre = vz;  */
	
	rx = x - gx;
	ry = y - gy;
	rz = z - gz;
	wa = w * (ry * vz - rz * vy);
	wb = w * (rz * vx - rx * vz);
	wg = w * (rx * vy - ry * vx);

	rx2 = rx * rx;
	ry2 = ry * ry;
	rz2 = rz * rz;
	ai = w * (ry2 + rz2);
	bi = w * (rz2 + rx2);
	gi = w * (rx2 + ry2);
}

void BodyPart::Copy(BodyPart *part) {
	this->l = part->l;
	this->w = part->w;
	this->t = part->t;
	this->x = part->x;
	this->y = part->y;
	this->z = part->z;
	this->a = part->a;
	this->b = part->b;
	this->g = part->g;
	this->a1 = part->a1;
	this->a2 = part->a2;
	this->b1 = part->b1;
	this->b2 = part->b2;
	this->c1 = part->c1;
	this->c2 = part->c2;
}	

Fist::Fist(bool right) : range(false) {
	a1 = 0.075;
	a2 = 0.075;
	b1 = 0.075;
	b2 = 0.075;
	c1 = 0.075;
	c2 = 0.075;
}

Fist::~Fist() {
}

void Fist::MakeDList(int partition) {
	CalcWeight();
	l = c2;
	t = c1;
	dlist.Generate();
	dlist.NewList();
	glove->Set();
	glTranslatef(0.0, 0.0, t);
	elipse2(a1, a2, b1, b2*0.8, c1, c2, partition);
	dlist.EndList();
}

void Fist::MakePeople(int partition) {
	CalcWeight();
	l = c2;
	t = c1;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
	elipse2(a1, a2, b1, b2*0.8, c1, c2, partition);
	dlist.EndList();
}

void Fist::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	glTranslatef(0.0, 0.0, t);
	sphere(r, partition);
	range.EndList();
}

void Fist::CalcCoord() {
	glTranslatef(0.0, 0.0, t);
//	glTranslatef(0.0, 0.0, l);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

	matrix.CalcUnit(0.0, 0.0, 1.0);
	nx = matrix.X();
	ny = matrix.Y();
	nz = matrix.Z();
}

void Fist::SetMaterial(GLMaterial *cmat) {
	glove = cmat;
}


Forearm::Forearm(bool right) : range(false) {
	a1 = 0.05;
	a2 = 0.05;
	b1 = 0.04;
	b2 = 0.04;
	c1 = 0.10;
	c2 = 0.20;	
}

Forearm::~Forearm() {
}

void Forearm::MakeDList(int partition){
	CalcWeight();
	l = c2*0.8;
	t = c1*0.8;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
 	elipse2c(a1, a2, b1, b2, c1, c2, 0, partition/3+1, partition);
	bandage->Set();
	elipse2c(a1*1.2, a2*1.2, b1*1.2, b2*1.2, c1, c2, partition/3 , partition/2, partition);
	dlist.EndList();
}

void Forearm::MakePeople(int partition){
	CalcWeight();
	l = c2*0.8;
	t = c1*0.8;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
//	skin->Set();
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}


void Forearm::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	glTranslatef(0.0, 0.0, t);
	pillar(-t, l, r, partition);
	range.EndList();
}

void Forearm::CalcCoord() {
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

	x1 = matrix.M13() * l + matrix.M14();
	y1 = matrix.M23() * l + matrix.M24();
	z1 = matrix.M33() * l + matrix.M34();
	x2 = -matrix.M13() * t + matrix.M14();
	y2 = -matrix.M23() * t + matrix.M24();
	z2 = -matrix.M33() * t + matrix.M34();
} 

void Forearm::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2) {
	bandage = cmat1;
	skin = cmat2;
}

Upperarm::Upperarm(bool right) : range(false) {
	a1 = 0.055;
	a2 = 0.055;
	b1 = 0.065;
	b2 = 0.055;
	c1 = 0.05;
	c2 = 0.15;
}

Upperarm::~Upperarm() {
}

void Upperarm::MakeDList(int partition){
	CalcWeight();
	l = c2*0.8;
	t = c1*0.5;	
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
	glTranslatef(0.0, 0.0, t);
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Upperarm::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	glTranslatef(0.0, 0.0, t);
	pillar(-t, l, r, partition);
	range.EndList();
}

void Upperarm::CalcCoord() {
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

	x1 = matrix.M13() * l + matrix.M14();
	y1 = matrix.M23() * l + matrix.M24();
	z1 = matrix.M33() * l + matrix.M34();
	x2 = -matrix.M13() * t + matrix.M14();
	y2 = -matrix.M23() * t + matrix.M24();
	z2 = -matrix.M33() * t + matrix.M34();
} 

void Upperarm::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Shoulder::Shoulder(bool right) : range(false) {
	a1 = 0.07;
	a2 = 0.07;
	b1 = 0.07;
	b2 = 0.07;
	c1 = 0.07;
	c2 = 0.07;	
	a = 180.0;
	if (right) {
		b = 90.0;
	} else {
		b = -90.0;	
	}
}

Shoulder::~Shoulder() {
}

void Shoulder::MakeDList(int partition){
	CalcWeight();
	l = c2 * 0.5;
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
 	dlist.EndList();
}

void Shoulder::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	sphere(r, partition);
	range.EndList();
}

void Shoulder::CalcCoord() {
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
} 

void Shoulder::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Arm::Arm(bool right) : fist(right), forearm(right), upperarm(right), shoulder(right){
}

Arm::~Arm() {
}

void Arm::CallLists() {
	skin->Set();
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
//	glRotatef(shoulder.Gamma(), 0.0, 0.0, 1.0);
	shoulder.CallList();
	glTranslatef(0.0, 0.0, shoulder.Length());
//	glRotatef(upperarm.Alpha(), 1.0, 0.0, 0.0);
//	glRotatef(upperarm.Beta(),  0.0, 1.0, 0.0);
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	upperarm.CallList();     
	glTranslatef(0.0, 0.0, upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
//	glRotatef(forearm.Beta(),  0.0, 1.0, 0.0);
//	glRotatef(forearm.Gamma(), 0.0, 0.0, 1.0);
	forearm.CallList();
	glTranslatef(0.0, 0.0, forearm.Length());
//	glRotatef(fist.Alpha(), 1.0, 0.0, 0.0);
//	glRotatef(fist.Beta(),  0.0, 1.0, 0.0);
	glRotatef(fist.Gamma(), 0.0, 0.0, 1.0);
	fist.CallList();
}

void Arm::CallPeople() {
	bandage->Set();
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
	shoulder.CallList();
	skin->Set();
	glTranslatef(0.0, 0.0, shoulder.Length());
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	upperarm.CallList();     
	glTranslatef(0.0, 0.0, upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
	forearm.CallList();
	glTranslatef(0.0, 0.0, forearm.Length());
	glRotatef(fist.Gamma(), 0.0, 0.0, 1.0);
	fist.CallList();
}

void Arm::CallListsNoFist() {
	skin->Set();
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
	shoulder.CallList();
	glTranslatef(0.0, 0.0, shoulder.Length());
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	upperarm.CallList();     
	glTranslatef(0.0, 0.0, upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
	forearm.CallList();
}

void Arm::CallListsFistOnly() {
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
	glTranslatef(0.0, 0.0, shoulder.Translate() + shoulder.Length());
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, 0.0, upperarm.Translate() + upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
	glTranslatef(0.0, 0.0, forearm.Translate() + forearm.Length());
	glRotatef(fist.Gamma(), 0.0, 0.0, 1.0);
	fist.CallList();
}

void Arm::CallListsSelect(bool right) {
	if (right)  {
        glLoadName(SELECT_RIGHT_FIST);	
	} else {
        glLoadName(SELECT_LEFT_FIST);		
	}
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
	glTranslatef(0.0, 0.0, shoulder.Translate());
	glTranslatef(0.0, 0.0, shoulder.Length());
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, 0.0, upperarm.Translate());
	glTranslatef(0.0, 0.0, upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
	glTranslatef(0.0, 0.0, forearm.Translate());
	glTranslatef(0.0, 0.0, forearm.Length());
	glRotatef(fist.Gamma(), 0.0, 0.0, 1.0);
	fist.CallList();
}

void Arm::CalcCoords() {
	glRotatef(shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(shoulder.Beta(),  0.0, 1.0, 0.0);
	shoulder.CalcCoord();
	glTranslatef(0.0, 0.0, shoulder.Length());
	glRotatef(upperarm.Gamma(), 0.0, 0.0, 1.0);
	upperarm.CalcCoord();     
	glTranslatef(0.0, 0.0, upperarm.Length());  
	glRotatef(forearm.Alpha(), 1.0, 0.0, 0.0);
	forearm.CalcCoord();
	glTranslatef(0.0, 0.0, forearm.Length());
	glRotatef(fist.Gamma(), 0.0, 0.0, 1.0);
	fist.CalcCoord();
} 

void Arm::MakeDLists(int partition) {
	fist.MakeDList(partition);
	forearm.MakeDList(partition);
	upperarm.MakeDList(partition);
	shoulder.MakeDList(partition);
}

void Arm::MakePeople(int partition) {
	fist.MakePeople(partition);
	forearm.MakePeople(partition);
	upperarm.MakeDList(partition);
	shoulder.MakeDList(partition);
}

void Arm::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2, GLMaterial *cmat3) {
	glove = cmat1;
	bandage = cmat2;
	skin = cmat3;
	fist.SetMaterial(glove);
	forearm.SetMaterial(bandage, skin);
	upperarm.SetMaterial(skin);
	shoulder.SetMaterial(skin);
}

Toe::Toe(bool right) {
	if (right) {
		a1 = 0.04;
		a2 = 0.06;
	} else {
		a1 = 0.06;
		a2 = 0.04;	
	}
	b1 = 0.06;
	b2 = b1 * 0.1;
	c1 = 0.04;
	c2 = 0.0;
}

Toe::~Toe() {
}

void Toe::MakeDList(int partition){
	CalcWeight();
	l = b1;	
	t = c1*0.5;
	dlist.Generate();
	dlist.NewList();
	shoes->Set();
	glTranslatef(0.0, 0.0, t);
	half_elipse2(a1, a2, b1, b1 * 0.1, c1, partition);
	dlist.EndList();
}

void Toe::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

//	glTranslatef(0.0, l, 0.0);
//	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
	CalcAbsAngle();	
}

void Toe::SetMaterial(GLMaterial *cmat) {
	shoes = cmat;
}

Foot::Foot(bool right) {
	if (right) {
		a1 = 0.04;
		a2 = 0.06;
	} else {
		a1 = 0.06;
		a2 = 0.04;	
	}
	b2 = 0.15;
	b1 = b2 * 0.1;
	c1 = 0.04;
	c2 = 0.0;
}

Foot::~Foot() {
}

void Foot::MakeDList(int partition){
	CalcWeight();
	l = -c1 * 0.5;	
	t = c1;
	dlist.Generate();
	dlist.NewList();
	shoes->Set();
	glTranslatef(0.0, b2*0.5, t);
	half_elipse2(a1, a2, b2 * 0.1, b2, c1, partition);
	dlist.EndList();
}

void Foot::CalcCoord(){
	glTranslatef(0.0, b2*0.5, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
}

void Foot::SetMaterial(GLMaterial *cmat) {
	shoes = cmat;
}

Shank::Shank(bool right) {
	a1 = 0.06;
	a2 = 0.06;
	b1 = 0.07;
	b2 = 0.06;
	c1 = 0.21;
	c2 = 0.31;
}

Shank::~Shank() {
}

void Shank::MakeDList(int partition){
	CalcWeight();
	l = c2*0.9;	
	t = c1*0.9;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
	skin->Set();
 	elipse2c(a1, a2, b1, b2, c1, c2, 0, partition/4, partition);
	shoes->Set();
	elipse2c(a1, a2, b1, b2, c1, c2, partition/4-1, partition/2, partition);
	dlist.EndList();
}

void Shank::MakePeople(int partition){
	CalcWeight();
	l = c2*0.9;	
	t = c1*0.9;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Shank::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();

 	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
}

void Shank::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2) {
	shoes = cmat1;
	skin = cmat2;
}

Thigh::Thigh(bool right) {
	a1 = 0.08;
	a2 = 0.08;
	b1 = 0.08;
	b2 = 0.08;
	c1 = 0.12;
	c2 = 0.50;
	a = 180.0;
	g = 180.0;
}

Thigh::~Thigh() {
}

void Thigh::MakeDList(int partition){
	CalcWeight();
	l = c2*0.9;	
	t = c1;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
	trunks->Set();
 	elipse2c(a1*1.3, a2*1.3, b1*1.3, b2*1.3, c1, c2, 0, partition/3+1, partition);
	skin->Set();
	elipse2c(a1, a2, b1, b2, c1, c2, partition/3, partition/2, partition);
	dlist.EndList();
}

void Thigh::MakePeople(int partition){
	CalcWeight();
	l = c2*0.9;	
	t = c1;
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
	trunks->Set();
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Thigh::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
}

void Thigh::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2) {
	skin = cmat1;
	trunks = cmat2;
}

Leg::Leg(bool right) : toe(right), foot(right), shank(right), thigh(right) {
}

Leg::~Leg() {
}

void Leg::CallLists() {
	glRotatef(thigh.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(thigh.Beta(),  0.0, 1.0, 0.0);
	glRotatef(thigh.Gamma(), 0.0, 0.0, 1.0);
	thigh.CallList();
	glTranslatef(0.0, 0.0, thigh.Length());
	glRotatef(shank.Alpha(), 1.0, 0.0, 0.0);
//	glRotatef(shank.Beta(),  0.0, 1.0, 0.0);
//	glRotatef(shank.Gamma(), 0.0, 0.0, 1.0);
	shank.CallList();     
	glTranslatef(0.0, 0.0, shank.Length());  
	glRotatef(foot.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(foot.Beta(),  0.0, 1.0, 0.0);
	glRotatef(foot.Gamma(), 0.0, 0.0, 1.0);
	foot.CallList();
	glTranslatef(0.0, 0.0, foot.Length());
	glRotatef(toe.Alpha(), 1.0, 0.0, 0.0);
//	glRotatef(toe.Beta(),  0.0, 1.0, 0.0);
//	glRotatef(toe.Gamma(), 0.0, 0.0, 1.0);
	toe.CallList();
}

void Leg::CallListsSelect(bool right) {
	if (right)  {
        glLoadName(SELECT_RIGHT_TOE);	
	} else {
        glLoadName(SELECT_LEFT_TOE);		
	}
	glRotatef(thigh.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(thigh.Beta(),  0.0, 1.0, 0.0);
	glRotatef(thigh.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, 0.0, thigh.Translate());
	glTranslatef(0.0, 0.0, thigh.Length());
	glRotatef(shank.Alpha(), 1.0, 0.0, 0.0);
	glTranslatef(0.0, 0.0, shank.Translate());  
	glTranslatef(0.0, 0.0, shank.Length());  
	glRotatef(foot.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(foot.Beta(),  0.0, 1.0, 0.0);
	glRotatef(foot.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, foot.B2()*0.5, foot.Translate());
	glTranslatef(0.0, 0.0, foot.Length());
	glRotatef(toe.Alpha(), 1.0, 0.0, 0.0);
	toe.CallList();
}

void Leg::CalcCoords(){
	glRotatef(thigh.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(thigh.Beta(),  0.0, 1.0, 0.0);
	glRotatef(thigh.Gamma(), 0.0, 0.0, 1.0);
	thigh.CalcCoord();
	glTranslatef(0.0, 0.0, thigh.Length());
	glRotatef(shank.Alpha(), 1.0, 0.0, 0.0);
	shank.CalcCoord();     
	glTranslatef(0.0, 0.0, shank.Length());  
	glRotatef(foot.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(foot.Beta(),  0.0, 1.0, 0.0);
	glRotatef(foot.Gamma(), 0.0, 0.0, 1.0);
	foot.CalcCoord();
	glTranslatef(0.0, 0.0, foot.Length());
	glRotatef(toe.Alpha(), 1.0, 0.0, 0.0);
	toe.CalcCoord();
}

void Leg::MakeDLists(int partition) {
	toe.MakeDList(partition);
	foot.MakeDList(partition);
	shank.MakeDList(partition);
	thigh.MakeDList(partition);
}

void Leg::MakePeople(int partition) {
	toe.MakeDList(partition);
	foot.MakeDList(partition);
	trunks->Set();
	shank.MakePeople(partition);
	thigh.MakePeople(partition);
}

void Leg::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2, GLMaterial *cmat3) {
	shoes = cmat1;
	skin = cmat2;
	trunks = cmat3;
	toe.SetMaterial(shoes);
	foot.SetMaterial(shoes);
	shank.SetMaterial(shoes, skin);
	thigh.SetMaterial(skin, trunks);
}

Waist::Waist() {
	a1 = 0.125;
	a2 = 0.125;
	b1 = 0.100;
	b2 = 0.125;
	c1 = 0.175;
	c2 = 0.05;  
}

Waist::~Waist() {
}

void Waist::MakeDList(int partition){
	CalcWeight();
	l = c2;	
	dlist.Generate();
	dlist.NewList();
	trunks->Set();
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Waist::CalcCoord(){
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
	CalcAbsAngle();

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
}

void Waist::SetMaterial(GLMaterial *cmat) {
	trunks = cmat;
}

Abdomen::Abdomen() : range(false) , liver(false) {
	a1 = 0.100;
	a2 = 0.100;
	b1 = 0.060;
	b2 = 0.080;
	c1 = 0.15;
	c2 = 0.15;  
}

Abdomen::~Abdomen() {
}

void Abdomen::MakeDList(int partition) {
	CalcWeight();
	l = c2*0.15;	
	t = c1*0.25;
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
	glTranslatef(0.0, 0.0, t);
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Abdomen::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	glTranslatef(0.0, 0.0, t);
	pillar(-bottom, top, r, partition);
	range.EndList();
}

void Abdomen::MakeLiver(int partition) {
	liver.Generate();
	liver.NewList();
	glTranslatef(0.0, 0.0, t);
	glTranslatef(lrx, lry, lrz);
	sphere(lr, partition);
	liver.EndList();
}

void Abdomen::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

	x1 = -matrix.M13() * bottom + matrix.M14();
	y1 = -matrix.M23() * bottom + matrix.M24();
	z1 = -matrix.M33() * bottom + matrix.M34();
	x2 = matrix.M13() * top + matrix.M14();
	y2 = matrix.M23() * top + matrix.M24();
	z2 = matrix.M33() * top + matrix.M34();
	matrix.Calc(lrx, lry, lrz);
	lx = matrix.X();
	ly = matrix.Y();
	lz = matrix.Z();
}

void Abdomen::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Breast::Breast() {
	a1 = 0.17;
	a2 = 0.17;
	b1 = 0.11;
	b2 = 0.13;
	c1 = 0.20;
	c2 = 0.075;  
}

Breast::~Breast() {
}

void Breast::MakeDList(int partition){
	CalcWeight();
	l = c2;
	t = c1;	
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
	glTranslatef(0.0, 0.0, t);
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
}

void Breast::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();
 	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

//	matrix.Calc(0.0, -1.0, 0.0);
//	SetDir(matrix.X(), matrix.Y(), matrix.Z());
}

void Breast::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Body::Body() {
}

Body::~Body() {
}

void Body::CallLists() {
	waist.CallList(); 
	glTranslatef(0.0, 0.0, waist.Length());
	glRotatef(abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(abdomen.Gamma(), 0.0, 0.0, 1.0);
	skin->Set();
	abdomen.CallList(); 
	glTranslatef(0.0, 0.0, abdomen.Length());  
	glRotatef(breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(breast.Gamma(), 0.0, 0.0, 1.0);  
	breast.CallList();    
}


void Body::CallListsSelect() {
	glLoadName(SELECT_WAIST);
	waist.CallList(); 
	glTranslatef(0.0, 0.0, waist.Length());
	glRotatef(abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(abdomen.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, 0.0, abdomen.Translate());  
	glTranslatef(0.0, 0.0, abdomen.Length());  
	glRotatef(breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(breast.Gamma(), 0.0, 0.0, 1.0);  
	glTranslatef(0.0, 0.0, breast.Translate());  
}	

void Body::CalcCoords(){
	waist.CalcCoord(); 
	glTranslatef(0.0, 0.0, waist.Length());
	glRotatef(abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(abdomen.Gamma(), 0.0, 0.0, 1.0);
	abdomen.CalcCoord(); 
	glTranslatef(0.0, 0.0, abdomen.Length());  
	glRotatef(breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(breast.Gamma(), 0.0, 0.0, 1.0);  
	breast.CalcCoord();    
}

void Body::MakeDLists(int partition) {
	waist.MakeDList(partition);
	abdomen.MakeDList(partition);
	breast.MakeDList(partition);
}

void Body::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2) {
	trunks = cmat1;
	skin = cmat2;
	waist.SetMaterial(trunks);
	abdomen.SetMaterial(skin);
	breast.SetMaterial(skin);
}

Neck::Neck() {
	a1 = 0.06;
	a2 = 0.06;
	b1 = 0.06;
	b2 = 0.06;
	c1 = 0.07;
	c2 = 0.07;  
}

Neck::~Neck() {
}

void Neck::MakeDList(int partition){
	CalcWeight();
	l = c2*0.5;
	t = c1*0.5;	
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
	glTranslatef(0.0, 0.0, c1);
 	elipse2(a1, a2, b1, b2, c1, t, partition);
	dlist.EndList();
}

void Neck::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
//	CalcAbsAngle();	

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());
}

void Neck::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Face::Face() : wig(false), range(false), jaw(false)  {
	a1 = 0.10;
	a2 = 0.10;
	b1 = 0.08;
	b2 = 0.10;
	c1 = 0.18;
	c2 = 0.12;
}

Face::~Face() {
}

void Face::MakeDList(int partition) {
	CalcWeight();
	w *= 1.5;
	l = c2;
	t = c1*0.8;	
	dlist.Generate();
	dlist.NewList();
//	skin->Set();
	glTranslatef(0.0, 0.0, t);
 	elipse2wt(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
} 

void Face::MakeNoFace(int partition) {
	CalcWeight();
	w *= 1.5;
	l = c2;
	t = c1*0.8;	
	dlist.Generate();
	dlist.NewList();
	glTranslatef(0.0, 0.0, t);
 	elipse2(a1, a2, b1, b2, c1, c2, partition);
	dlist.EndList();
} 

void Face::MakePolygonFace(Uint8 eye_form , Uint8 nose_form, Uint8 mouth_form, int partition) {
	int form;
	float eye_width;
	GLMaterial face_color;

	CalcWeight();
	w *= 1.5;
	l = c2;
	t = c1*0.8;	
	dlist.Generate();
	dlist.NewList();
	face_color.DefSpecular(0.0, 0.0, 0.0);
	face_color.DefShininess(0.0);

	glTranslatef(0.0, 0.0, t);
 	elipse2(a1, a2, b1, b2, c1, c2, partition);

	glPushMatrix();
	glTranslatef(a1*0.9, 0.0, -c1*0.125);
	glRotatef(-90, 0.0, 1.0, 0.0);
 	ear(c2*0.20, c1*0.30, b1*0.25, b2*0.25, a1*0.4, 12);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(-a2*0.9, 0.0, -c1*0.125);
	glRotatef(90, 0.0, 1.0, 0.0);
 	ear(c1*0.30, c2*0.20, b1*0.25, b2*0.25, a2*0.4, 12);
	glPopMatrix();

	switch (nose_form) {
		case 1:
			nose(a1+a2, b1, c1);
		break;	
		case 2:
			nose((a1+a2)*1.20, b1, c1);
		break;	
		case 3:
			nose((a1+a2)*0.75, b1, c1);
		break;	
		case 4:
			nose2(a1+a2, b1, c1);
		break;	
		case 5:
			nose2((a1+a2)*1.20, b1, c1);
		break;	
		case 6:
			nose2((a1+a2)*0.75, b1, c1);
		break;	
		case 7:
			nose3(a1+a2, b1, c1);
		break;	
		case 8:
			nose3((a1+a2)*1.20, b1, c1);
		break;	
		case 9:
			nose3((a1+a2)*0.75, b1, c1);
		break;	
		case 10:
			nose4(a1+a2, b1, c1);
		break;	
		case 11:
			nose4((a1+a2)*1.20, b1, c1);
		break;	
		case 12:
			nose4((a1+a2)*0.75, b1, c1);
		break;	
		case 13:
			nose5(a1+a2, b1, c1);
		break;	
		case 14:
			nose5((a1+a2)*1.20, b1, c1);
		break;	
		case 15:
			nose5((a1+a2)*0.75, b1, c1);
		break;	
		default:
			nose(a1+a2, b1, c1);
		break;
	}

	switch (eye_form) {
		case 2:	
			form = 2;
			eye_width = 0.40;
		break;
		case 3:	
			form = 3;
			eye_width = 0.40;
		break;
		case 4:	
			form = 4;
			eye_width = 0.40;
		break;
		case 5:	
			form = 5;
			eye_width = 0.40;
		break;
		case 6:	
			form = 6;
			eye_width = 0.40;
		break;
		case 7:	
			form = 7;
			eye_width = 0.40;
		break;
		case 8:	
			form = 1;
			eye_width = 0.35;
		break;
		case 9:	
			form = 2;
			eye_width = 0.35;
		break;
		case 10:	
			form = 3;
			eye_width = 0.35;
		break;
		case 11:	
			form = 4;
			eye_width = 0.35;
		break;
		case 12:	
			form = 5;
			eye_width = 0.35;
		break;
		case 13:	
			form = 6;
			eye_width = 0.35;
		break;
		case 14:	
			form = 7;
			eye_width = 0.35;
		break;
		case 15:	
			form = 1;
			eye_width = 0.45;
		break;
		case 16:	
			form = 2;
			eye_width = 0.45;
		break;
		case 17:	
			form = 3;
			eye_width = 0.45;
		break;
		case 18:	
			form = 4;
			eye_width = 0.45;
		break;
		case 19:	
			form = 5;
			eye_width = 0.45;
		break;
		case 20:	
			form = 6;
			eye_width = 0.45;
		break;
		case 21:	
			form = 7;
			eye_width = 0.45;
		break;
		default:
			form = 1;
			eye_width = 0.40;
		break;	
	}
	face_color.DefDiffuse(0.0, 0.0, 0.0);
	face_color.Set();
	eye_ball(a1, a2, b1, c1, eye_width);
	face_color.DefDiffuse(1.0, 1.0, 1.0);
    face_color.Set();
	eye(a1, a2, b1, c1, eye_width, form);

	switch (mouth_form) {
		case 4:
			face_color.DefDiffuse(1.0, 1.0, 1.0);
		break;
		default:
			face_color.DefDiffuse(1.0, 0.25, 0.25);
		break;
	}
 	face_color.Set();
	switch (mouth_form) {
	 	case 2:
			mouth(a1 + a2, b1, c1, 2);
		break;
	 	case 3:
			mouth(a1 + a2, b1, c1, 3);
		break;		
	 	case 4:
			mouth(a1 + a2, b1, c1, 4);
		break;		
	 	case 5:
			mouth(a1 + a2, b1, c1, 5);
		break;		
	 	case 6:
			mouth(a1 + a2, b1, c1, 6);
		break;		
		default:
			mouth(a1 + a2, b1, c1, 1);
		break;	
	}
	dlist.EndList();
}


void Face::MakeNoWig() {
	wig.Generate();
	wig.NewList();
	wig.EndList();
}

void Face::MakeWig(Uint8 hair_style, Uint8 eyebrow_form) {
	wig.Generate();
	wig.NewList();

	switch (eyebrow_form) {
		case 1 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.0, 0);
		break;
		case 2 :
 			eyebrow(a1, a2, b1, c1, 0.0, 1.0, 0);
		break;
		case 3 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 1.0, 0);
		break;
		case 4 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.5, 0);
		break;
		case 5 :
 			eyebrow(a1, a2, b1, c1, 0.0, 1.5, 0);
		break;
		case 6 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 1.5, 0);
		break;
		case 7 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 0.5, 0);
		break;
		case 8 :
 			eyebrow(a1, a2, b1, c1, 0.0, 0.5, 0);
		break;
		case 9 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 0.5, 0);
		break;
		case 10 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.0, 1);
		break;
		case 11 :
 			eyebrow(a1, a2, b1, c1, 0.0, 1.0, 1);
		break;
		case 12 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 1.0, 1);
		break;
		case 13 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.5, 1);
		break;
		case 14 :
 			eyebrow(a1, a2, b1, c1, 0.0, 1.5, 1);
		break;
		case 15 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 1.5, 1);
		break;
		case 16 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 0.5, 1);
		break;
		case 17 :
 			eyebrow(a1, a2, b1, c1, 0.0, 0.5, 1);
		break;
		case 18 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 0.5, 1);
		break;
  		case 19 : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.0, 2);
		break;
 		case 20 : 
 			eyebrow(a1, a2, b1, c1,  0.0, 1.0, 2);
		break;
 		case 21 : 
 			eyebrow(a1, a2, b1, c1, -15.0, 1.0, 2);
		break;
 		case 22: 
 			eyebrow(a1, a2, b1, c1, 0.0, 1.0, 3);
		break;
 		case 23: 
 			eyebrow(a1, a2, b1, c1, 0.0, 1.5, 3);
		break;
 		case 24: 
 			eyebrow(a1, a2, b1, c1, 0.0, 0.5, 3);
		break;
		case 25 : 
		break;
		default : 
 			eyebrow(a1, a2, b1, c1, 15.0, 1.0, 0);
		break;		
	}

	glPushMatrix();
	glTranslatef(0.0, 0.0, c2);
	switch (hair_style) {
		case 1 : 
			hair(a1, a2, b1, b2, c1, c2);
        break;
		case 2 : 
        break;
		case 3 : 
			hair_sport(a1, a2, b1, b2, c1, c2);
        break;
		case 4 : 
			hair_mushroom(a1, a2, b1, b2, c1, c2);
        break;
		case 5 : 
			hair_perma(a1, a2, b1, b2, c1, c2);
        break;
		case 6 : 
			hair_regent(a1, a2, b1, b2, c1, c2);
        break;
		case 7 : 
			hair_mohican(a1, a2, b1, b2, c1, c2);
        break;
		case 8 : 
			hair_hedgehog(a1, a2, b1, b2, c1, c2);
        break;
		case 9 : 
			hair_oneline(a1, a2, b1, b2, c1, c2);
        break;
		case 10 : 
			hair_tree(a1, a2, b1, b2, c1, c2);
        break;
		default:
			hair(a1, a2, b1, b2, c1, c2);
   		break;
	}
	glPopMatrix();
	wig.EndList();
}

void Face::MakeRange(int partition) {
	range.Generate();
	range.NewList();
	glTranslatef(0.0, 0.0, t);
	pillar(-bottom, top, r, partition);
	range.EndList();
}

void Face::MakeJaw(int partition) {
	jaw.Generate();
	jaw.NewList();
	glTranslatef(0.0, 0.0, t);
	glTranslatef(jrx, jry, jrz);
	sphere(jr, partition);
	jaw.EndList();
}

void Face::CalcCoord(){
	glTranslatef(0.0, 0.0, t);
	matrix.Get();
	SetPos(matrix.M14(), matrix.M24(), matrix.M34());
	CalcAbsAngle();

	matrix.Calc(0.0, b1, 0.0);
	SetDir(matrix.X(), matrix.Y(), matrix.Z());
	matrix.CalcUnit(0.0, 0.0, c2);
	SetHead(matrix.X(), matrix.Y(), matrix.Z());

	matrix.Calc(OX(), OY(), OZ());
	SetGPos(matrix.X(), matrix.Y(), matrix.Z());

	x1 = -matrix.M13() * bottom + matrix.M14();
	y1 = -matrix.M23() * bottom + matrix.M24();
	z1 = -matrix.M33() * bottom + matrix.M34();
	x2 = matrix.M13() * top + matrix.M14();
	y2 = matrix.M23() * top + matrix.M24();
	z2 = matrix.M33() * top + matrix.M34();
	matrix.Calc(jrx, jry, jrz);
	jx = matrix.X();
	jy = matrix.Y();
	jz = matrix.Z();
}

void Face::SetMaterial(GLMaterial *cmat) {
	skin = cmat;
}

Head::Head() {
}

Head::~Head() {
}

void Head::CallLists() {
	skin->Set();
	glRotatef(neck.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(neck.Beta(),  0.0, 1.0, 0.0);
	glRotatef(neck.Gamma(), 0.0, 0.0, 1.0); 
	neck.CallList(); 
	glTranslatef(0.0, 0.0, neck.Length());
	glRotatef(face.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(face.Beta(),  0.0, 1.0, 0.0);
	glRotatef(face.Gamma(), 0.0, 0.0, 1.0);
	face.CallList(); 
	hair->Set();
	face.CallWig(); 
}

void Head::CallListsSelect() {
	glLoadName(SELECT_HEAD);	
	glRotatef(neck.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(neck.Beta(),  0.0, 1.0, 0.0);
	glRotatef(neck.Gamma(), 0.0, 0.0, 1.0); 
	glTranslatef(0.0, 0.0, neck.Translate());
	glTranslatef(0.0, 0.0, neck.Length());
	glRotatef(face.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(face.Beta(),  0.0, 1.0, 0.0);
	glRotatef(face.Gamma(), 0.0, 0.0, 1.0);
	face.CallList(); 
}

void Head::CalcCoords() {
	glRotatef(neck.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(neck.Beta(),  0.0, 1.0, 0.0);
	glRotatef(neck.Gamma(), 0.0, 0.0, 1.0); 
	neck.CalcCoord(); 
	glTranslatef(0.0, 0.0, neck.Length());
	glRotatef(face.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(face.Beta(),  0.0, 1.0, 0.0);
	glRotatef(face.Gamma(), 0.0, 0.0, 1.0);
	face.CalcCoord(); 
}

void Head::MakeDLists(int partition) {
	neck.MakeDList(partition);
	face.MakeDList(partition);
	face.MakeNoWig();
}

void Head::MakeNoFace(int partition) {
	neck.MakeDList(partition);
	face.MakeNoFace(partition);
	face.MakeNoWig();
}

void Head::MakePolygonFace(Uint8 hair_style, Uint8 eyebrow_form, Uint8 eye_form, Uint8 nose_form, Uint8 mouth_form, int partition) {
	neck.MakeDList(partition);
	face.MakePolygonFace(eye_form, nose_form, mouth_form, partition);
	face.MakeWig(hair_style, eyebrow_form);
}

void Head::SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2) {
	skin = cmat1;
	hair = cmat2;
	neck.SetMaterial(skin);
	face.SetMaterial(skin);
}

Boxer::Boxer(bool makedl) : left_arm(false), left_leg(false) {
	sw = 0.20;
	ww = 0.08;	
	shoes.DefDiffuse(1.0, 0.0, 0.0);
	shoes.DefSpecular(1.0, 1.0, 1.0);
	shoes.DefShininess(100.0);
	skin.DefDiffuse(1.0, 0.6, 0.6);
	bandage.DefDiffuse(1.0, 1.0, 1.0);
	glove.DefDiffuse(1.0, 0.0, 0.0);
	glove.DefSpecular(1.0, 1.0, 1.0);
	glove.DefShininess(100.0);
	trunks.DefDiffuse(0.1, 0.1, 0.1);
	trunks.DefSpecular(1.0, 1.0, 1.0);
	trunks.DefShininess(100.0);
	SetMaterial();
	if (makedl)  {
		MakeDLists();
	} 
	ox = 0.0;
	oy = 0.0;
	ot = 0.0;
}

Boxer::~Boxer() {
}

void Boxer::SetMaterial() {
	head.SetMaterial(&skin, &hair);
	body.SetMaterial(&trunks, &skin);
	left_arm.SetMaterial(&glove, &bandage, &skin);
	right_arm.SetMaterial(&glove, &bandage, &skin);
	left_leg.SetMaterial(&shoes, &skin, &trunks);
	right_leg.SetMaterial(&shoes, &skin, &trunks);
} 

#define SHOW_FISTS

void Boxer::CallLists(bool is_boxer_view) {
	glPushMatrix();

	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);

	glTranslatef(X(), Y(), Z());
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glPushMatrix();
#ifdef SHOW_FISTS
	glPushMatrix();
	glTranslatef(0.0, 0.0, body.waist.Translate() + body.waist.Length());  
	glRotatef(body.abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.abdomen.Gamma(), 0.0, 0.0, 1.0);
	glTranslatef(0.0, 0.0, body.abdomen.Translate() + body.abdomen.Length());  
	glRotatef(body.breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.breast.Gamma(), 0.0, 0.0, 1.0);  
	glTranslatef(0.0, 0.0, body.breast.Translate()); 

	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	left_arm.CallListsFistOnly(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	right_arm.CallListsFistOnly(); 
	glPopMatrix();
	glPopMatrix();
#endif 
	body.CallLists(); 

	if (!is_boxer_view) {
		glPushMatrix();
		head.CallLists(); 
		glPopMatrix();  
	}

	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
#ifdef SHOW_FISTS
	left_arm.CallListsNoFist(); 
#else	
	left_arm.CallLists(); 
#endif 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
#ifdef SHOW_FISTS
	right_arm.CallListsNoFist(); 
#else	
	right_arm.CallLists(); 
#endif 
	glPopMatrix();

	glPopMatrix();

	glPushMatrix();
	glTranslatef(-ww, 0.0, 0.0);
	left_leg.CallLists(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( ww, 0.0, 0.0);
	right_leg.CallLists(); 
	glPopMatrix(); 

	glPopMatrix(); 
} 


void Boxer::CallPeople() {
	glPushMatrix();

	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);

	glTranslatef(X(), Y(), Z());
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glPushMatrix();

	body.waist.CallList();
	glTranslatef(0.0, 0.0, body.waist.Translate());  
	glRotatef(body.abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.abdomen.Gamma(), 0.0, 0.0, 1.0);
	bandage.Set();
	body.abdomen.CallList();
	glTranslatef(0.0, 0.0, body.abdomen.Translate());  
	glRotatef(body.breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.breast.Gamma(), 0.0, 0.0, 1.0);  
	body.breast.CallList();

	glPushMatrix();
	head.CallLists(); 
	glPopMatrix();  

	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	left_arm.CallPeople(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	right_arm.CallPeople(); 
	glPopMatrix();

	glPopMatrix();

	glPushMatrix();
	glTranslatef(-ww, 0.0, 0.0);
	left_leg.CallLists(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( ww, 0.0, 0.0);
	right_leg.CallLists(); 
	glPopMatrix(); 

	glPopMatrix(); 
} 


void Boxer::CallRanges() {
	static GLMaterial color;
	color.Set();
	glPushMatrix();

	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);

	glTranslatef(X(), Y(), Z());
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glTranslatef(0.0, 0.0, body.waist.Translate() + body.waist.Length());  
	glRotatef(body.abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.abdomen.Gamma(), 0.0, 0.0, 1.0);

	glPushMatrix();
	color.Diffuse(1.0, 0.7, 0.7);
	body.abdomen.CallRange();
	glPopMatrix();
	glPushMatrix();
	color.Diffuse(0.0, 0.0, 1.0);
	body.abdomen.CallLiver();
	glPopMatrix();

	glTranslatef(0.0, 0.0, body.abdomen.Translate() + body.abdomen.Length());  
	glRotatef(body.breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.breast.Gamma(), 0.0, 0.0, 1.0);  
	glTranslatef(0.0, 0.0, body.breast.Translate()); 

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	glRotatef(right_arm.shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(right_arm.shoulder.Beta(),  0.0, 1.0, 0.0);
	color.Diffuse(1.0, 0.7, 0.7);
	right_arm.shoulder.CallRange();
	glTranslatef(0.0, 0.0, right_arm.shoulder.Length());
	glRotatef(right_arm.upperarm.Gamma(), 0.0, 0.0, 1.0);
	right_arm.upperarm.CallRange();     
	glTranslatef(0.0, 0.0, right_arm.upperarm.Length());  
	glRotatef(right_arm.forearm.Alpha(), 1.0, 0.0, 0.0);
	right_arm.forearm.CallRange();
	glTranslatef(0.0, 0.0, right_arm.forearm.Length());
	glRotatef(right_arm.fist.Gamma(), 0.0, 0.0, 1.0);
	color.Diffuse(1.0, 0.0, 0.0);
	right_arm.fist.CallRange();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	glRotatef(left_arm.shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(left_arm.shoulder.Beta(),  0.0, 1.0, 0.0);
	color.Diffuse(1.0, 0.7, 0.7);
	left_arm.shoulder.CallRange();
	glTranslatef(0.0, 0.0, left_arm.shoulder.Length());
	glRotatef(left_arm.upperarm.Gamma(), 0.0, 0.0, 1.0);
	left_arm.upperarm.CallRange();     
	glTranslatef(0.0, 0.0, left_arm.upperarm.Length());  
	glRotatef(left_arm.forearm.Alpha(), 1.0, 0.0, 0.0);
	left_arm.forearm.CallRange();
	glTranslatef(0.0, 0.0, left_arm.forearm.Length());
	glRotatef(left_arm.fist.Gamma(), 0.0, 0.0, 1.0);
	color.Diffuse(1.0, 0.0, 0.0);
	left_arm.fist.CallRange();
	glPopMatrix();  

	glRotatef(head.neck.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(head.neck.Beta(),  0.0, 1.0, 0.0);
	glRotatef(head.neck.Gamma(), 0.0, 0.0, 1.0); 
	glTranslatef(0.0, 0.0, head.neck.Translate() + head.neck.Length());
	glRotatef(head.face.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(head.face.Beta(),  0.0, 1.0, 0.0);
	glRotatef(head.face.Gamma(), 0.0, 0.0, 1.0);
	glPushMatrix();
	color.Diffuse(1.0, 0.7, 0.7);
	head.face.CallRange(); 
	glPopMatrix();  
	glPushMatrix();
	color.Diffuse(0.0, 0.0, 1.0);
	head.face.CallJaw(); 
	glPopMatrix();  

	glPopMatrix();
}

void Boxer::CallHitRanges(Boxer &another_boxer) {
	static GLMaterial color;
	static float xp1, yp1, zp1, rp1;
	static float xp2, yp2, zp2, rp2;
	another_boxer.CalcCoords();
	xp1 = another_boxer.right_arm.fist.X();
	yp1 = another_boxer.right_arm.fist.Y();
	zp1 = another_boxer.right_arm.fist.Z();
	rp1 = another_boxer.right_arm.fist.R();
	xp2 = another_boxer.left_arm.fist.X();
	yp2 = another_boxer.left_arm.fist.Y();
	zp2 = another_boxer.left_arm.fist.Z();
	rp2 = another_boxer.left_arm.fist.R();

	CalcCoords();		
	
	color.Set();
	glPushMatrix();

	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);

	glTranslatef(X(), Y(), Z());
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glTranslatef(0.0, 0.0, body.waist.Translate() + body.waist.Length());  
	glRotatef(body.abdomen.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.abdomen.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.abdomen.Gamma(), 0.0, 0.0, 1.0);

	glPushMatrix();
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			body.abdomen.X1(),
			body.abdomen.Y1(),
			body.abdomen.Z1(),
			body.abdomen.X2(),
			body.abdomen.Y2(),
			body.abdomen.Z2(),
			body.abdomen.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			body.abdomen.X1(),
			body.abdomen.Y1(),
			body.abdomen.Z1(),
			body.abdomen.X2(),
			body.abdomen.Y2(),
			body.abdomen.Z2(),
			body.abdomen.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	body.abdomen.CallRange();
	glPopMatrix();

	glPushMatrix();
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			body.abdomen.LX(),
			body.abdomen.LY(),
			body.abdomen.LZ(),
			body.abdomen.LR()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			body.abdomen.LX(),
			body.abdomen.LY(),
			body.abdomen.LZ(),
			body.abdomen.LR())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(0.0, 0.0, 1.0);
    }
	body.abdomen.CallLiver();
	glPopMatrix();

	glTranslatef(0.0, 0.0, body.abdomen.Translate() + body.abdomen.Length());  
	glRotatef(body.breast.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(body.breast.Beta(),  0.0, 1.0, 0.0);
	glRotatef(body.breast.Gamma(), 0.0, 0.0, 1.0);  
	glTranslatef(0.0, 0.0, body.breast.Translate()); 

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	glRotatef(right_arm.shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(right_arm.shoulder.Beta(),  0.0, 1.0, 0.0);
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			right_arm.shoulder.X(),
			right_arm.shoulder.Y(),
			right_arm.shoulder.Z(),
			right_arm.shoulder.R()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			right_arm.shoulder.X(),
			right_arm.shoulder.Y(),
			right_arm.shoulder.Z(),
			right_arm.shoulder.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	right_arm.shoulder.CallRange();
	glTranslatef(0.0, 0.0, right_arm.shoulder.Length());
	glRotatef(right_arm.upperarm.Gamma(), 0.0, 0.0, 1.0);
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			right_arm.upperarm.X1(),
			right_arm.upperarm.Y1(),
			right_arm.upperarm.Z1(),
			right_arm.upperarm.X2(),
			right_arm.upperarm.Y2(),
			right_arm.upperarm.Z2(),
			right_arm.upperarm.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			right_arm.upperarm.X1(),
			right_arm.upperarm.Y1(),
			right_arm.upperarm.Z1(),
			right_arm.upperarm.X2(),
			right_arm.upperarm.Y2(),
			right_arm.upperarm.Z2(),
			right_arm.upperarm.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	right_arm.upperarm.CallRange();     
	glTranslatef(0.0, 0.0, right_arm.upperarm.Length());  
	glRotatef(right_arm.forearm.Alpha(), 1.0, 0.0, 0.0);
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			right_arm.forearm.X1(),
			right_arm.forearm.Y1(),
			right_arm.forearm.Z1(),
			right_arm.forearm.X2(),
			right_arm.forearm.Y2(),
			right_arm.forearm.Z2(),
			right_arm.forearm.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			right_arm.forearm.X1(),
			right_arm.forearm.Y1(),
			right_arm.forearm.Z1(),
			right_arm.forearm.X2(),
			right_arm.forearm.Y2(),
			right_arm.forearm.Z2(),
			right_arm.forearm.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	right_arm.forearm.CallRange();
	glTranslatef(0.0, 0.0, right_arm.forearm.Length());
	glRotatef(right_arm.fist.Gamma(), 0.0, 0.0, 1.0);
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			right_arm.fist.X(),
			right_arm.fist.Y(),
			right_arm.fist.Z(),
			right_arm.fist.R()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			right_arm.fist.X(),
			right_arm.fist.Y(),
			right_arm.fist.Z(),
			right_arm.fist.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.0, 0.0);
    }
	right_arm.fist.CallRange();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	glRotatef(left_arm.shoulder.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(left_arm.shoulder.Beta(),  0.0, 1.0, 0.0);
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			left_arm.shoulder.X(),
			left_arm.shoulder.Y(),
			left_arm.shoulder.Z(),
			left_arm.shoulder.R()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			left_arm.shoulder.X(),
			left_arm.shoulder.Y(),
			left_arm.shoulder.Z(),
			left_arm.shoulder.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	left_arm.shoulder.CallRange();
	glTranslatef(0.0, 0.0, left_arm.shoulder.Length());
	glRotatef(left_arm.upperarm.Gamma(), 0.0, 0.0, 1.0);
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			left_arm.upperarm.X1(),
			left_arm.upperarm.Y1(),
			left_arm.upperarm.Z1(),
			left_arm.upperarm.X2(),
			left_arm.upperarm.Y2(),
			left_arm.upperarm.Z2(),
			left_arm.upperarm.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			left_arm.upperarm.X1(),
			left_arm.upperarm.Y1(),
			left_arm.upperarm.Z1(),
			left_arm.upperarm.X2(),
			left_arm.upperarm.Y2(),
			left_arm.upperarm.Z2(),
			left_arm.upperarm.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	left_arm.upperarm.CallRange();     
	glTranslatef(0.0, 0.0, left_arm.upperarm.Length());  
	glRotatef(left_arm.forearm.Alpha(), 1.0, 0.0, 0.0);
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			left_arm.forearm.X1(),
			left_arm.forearm.Y1(),
			left_arm.forearm.Z1(),
			left_arm.forearm.X2(),
			left_arm.forearm.Y2(),
			left_arm.forearm.Z2(),
			left_arm.forearm.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			left_arm.forearm.X1(),
			left_arm.forearm.Y1(),
			left_arm.forearm.Z1(),
			left_arm.forearm.X2(),
			left_arm.forearm.Y2(),
			left_arm.forearm.Z2(),
			left_arm.forearm.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	left_arm.forearm.CallRange();
	glTranslatef(0.0, 0.0, left_arm.forearm.Length());
	glRotatef(left_arm.fist.Gamma(), 0.0, 0.0, 1.0);
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			left_arm.fist.X(),
			left_arm.fist.Y(),
			left_arm.fist.Z(),
			left_arm.fist.R()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			left_arm.fist.X(),
			left_arm.fist.Y(),
			left_arm.fist.Z(),
			left_arm.fist.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.0, 0.0);
    }
	left_arm.fist.CallRange();
	glPopMatrix();  

	glRotatef(head.neck.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(head.neck.Beta(),  0.0, 1.0, 0.0);
	glRotatef(head.neck.Gamma(), 0.0, 0.0, 1.0); 
	glTranslatef(0.0, 0.0, head.neck.Translate() + head.neck.Length());
	glRotatef(head.face.Alpha(), 1.0, 0.0, 0.0);
	glRotatef(head.face.Beta(),  0.0, 1.0, 0.0);
	glRotatef(head.face.Gamma(), 0.0, 0.0, 1.0);
	glPushMatrix();
	if (IsHitPillar(xp1, yp1, zp1, rp1,
			head.face.X1(),
			head.face.Y1(),
			head.face.Z1(),
			head.face.X2(),
			head.face.Y2(),
			head.face.Z2(),
			head.face.R()) || 
		IsHitPillar(xp2, yp2, zp2, rp2,
			head.face.X1(),
			head.face.Y1(),
			head.face.Z1(),
			head.face.X2(),
			head.face.Y2(),
			head.face.Z2(),
			head.face.R())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(1.0, 0.7, 0.7);
    }
	head.face.CallRange(); 
	glPopMatrix();  
	glPushMatrix();
	if (IsHitSphere(xp1, yp1, zp1, rp1,
			head.face.JX(),
			head.face.JY(),
			head.face.JZ(),
			head.face.JR()) ||
		IsHitSphere(xp2, yp2, zp2, rp2,
			head.face.JX(),
			head.face.JY(),
			head.face.JZ(),
			head.face.JR())) {
		color.Diffuse(1.0, 1.0, 1.0);			
	} else {
		color.Diffuse(0.0, 0.0, 1.0);
    }
	head.face.CallJaw(); 
	glPopMatrix();  

	glPopMatrix();
}

int Boxer::IsHitRightPunch(Boxer &another_boxer) {
	return IsHitPunch(another_boxer, 1);
}

int Boxer::IsHitLeftPunch(Boxer &another_boxer) {
	return IsHitPunch(another_boxer, 0);
}

int Boxer::IsHitPunch(Boxer &another_boxer, int right) {
	static int hit_parts;
	static float xp, yp, zp, rp;
	
	if (right) {
		xp = another_boxer.right_arm.fist.X();
		yp = another_boxer.right_arm.fist.Y();
		zp = another_boxer.right_arm.fist.Z();
		rp = another_boxer.right_arm.fist.R();
	} else {
		xp = another_boxer.left_arm.fist.X();
		yp = another_boxer.left_arm.fist.Y();
		zp = another_boxer.left_arm.fist.Z();
		rp = another_boxer.left_arm.fist.R();
	}

	hit_parts = HIT_NOTHING;

	if (IsHitPillar(xp, yp, zp, rp,
			body.abdomen.X1(),
			body.abdomen.Y1(),
			body.abdomen.Z1(),
			body.abdomen.X2(),
			body.abdomen.Y2(),
			body.abdomen.Z2(),
			body.abdomen.R())) {			
		hit_parts |= HIT_BODY;
    }

	if (IsHitSphere(xp, yp, zp, rp,
			body.abdomen.LX(),
			body.abdomen.LY(),
			body.abdomen.LZ(),
			body.abdomen.LR())) {
		hit_parts |= HIT_LIVER;
    }

	if (IsHitSphere(xp, yp, zp, rp,
			right_arm.shoulder.X(),
			right_arm.shoulder.Y(),
			right_arm.shoulder.Z(),
			right_arm.shoulder.R())) {
		hit_parts |= HIT_RSHOUL;
	}

	if (IsHitPillar(xp, yp, zp, rp,
			right_arm.upperarm.X1(),
			right_arm.upperarm.Y1(),
			right_arm.upperarm.Z1(),
			right_arm.upperarm.X2(),
			right_arm.upperarm.Y2(),
			right_arm.upperarm.Z2(),
			right_arm.upperarm.R())) {
		hit_parts |= HIT_RUARM;
	}
	
	if (IsHitPillar(xp, yp, zp, rp,
			right_arm.forearm.X1(),
			right_arm.forearm.Y1(),
			right_arm.forearm.Z1(),
			right_arm.forearm.X2(),
			right_arm.forearm.Y2(),
			right_arm.forearm.Z2(),
			right_arm.forearm.R())) {
		hit_parts |= HIT_RUARM;
	}
	
	if (IsHitSphere(xp, yp, zp, rp,
			right_arm.fist.X(),
			right_arm.fist.Y(),
			right_arm.fist.Z(),
			right_arm.fist.R())) {
		hit_parts |= HIT_RFIST;
	}

	if (IsHitSphere(xp, yp, zp, rp,
			left_arm.shoulder.X(),
			left_arm.shoulder.Y(),
			left_arm.shoulder.Z(),
			left_arm.shoulder.R())) {
		hit_parts |= HIT_LSHOUL;
	}
	
	if (IsHitPillar(xp, yp, zp, rp,
			left_arm.upperarm.X1(),
			left_arm.upperarm.Y1(),
			left_arm.upperarm.Z1(),
			left_arm.upperarm.X2(),
			left_arm.upperarm.Y2(),
			left_arm.upperarm.Z2(),
			left_arm.upperarm.R())) {
		hit_parts |= HIT_LUARM;
	}
	
	if (IsHitPillar(xp, yp, zp, rp,
			left_arm.forearm.X1(),
			left_arm.forearm.Y1(),
			left_arm.forearm.Z1(),
			left_arm.forearm.X2(),
			left_arm.forearm.Y2(),
			left_arm.forearm.Z2(),
			left_arm.forearm.R())) {
		hit_parts |= HIT_LFARM;
	}
	
	if (IsHitSphere(xp, yp, zp, rp,
			left_arm.fist.X(),
			left_arm.fist.Y(),
			left_arm.fist.Z(),
			left_arm.fist.R())) {			
		hit_parts |= HIT_LFIST;
	}

	if (IsHitPillar(xp, yp, zp, rp,
			head.face.X1(),
			head.face.Y1(),
			head.face.Z1(),
			head.face.X2(),
			head.face.Y2(),
			head.face.Z2(),
			head.face.R())) {
		hit_parts |= HIT_FACE;
	}
	
	if (IsHitSphere(xp, yp, zp, rp,
			head.face.JX(),
			head.face.JY(),
			head.face.JZ(),
			head.face.JR())) {
		hit_parts |= HIT_JAW;
	}       
	
	return hit_parts;
}

float Boxer::CalcDamageRight() {
	return CalcDamage(1);
}

float Boxer::CalcDamageLeft() {
	return CalcDamage(0);
}

float Boxer::CalcEnergy() {
	vg_energy = Weight()*(VGX()*VGX() + VGY()*VGY() + VGZ()*VGZ()) * 25000.0;
	wg_energy = (GAI()*WGA()*WGA() + GBI()*WGB()*WGB() + GGI()*WGG()*WGG()) * 25000.0;
	return (vg_energy + wg_energy);
}

float Boxer::CalcDamage(int right) {
/*	float vnx, vny, vnz;
	float vfx, vfy, vfz;
	float wna, wnb, wng;
	float wfa, wfb, wfg;
	float rfx, rfy, rfz, rf_abs;

	float vgvn, vfvn;
	float wgwn, wfwn;
	float damage_center, damage_rotate;
*/	
	if (right) {
		vnx = right_arm.fist.NX();
		vny = right_arm.fist.NY();
		vnz = right_arm.fist.NZ();
		vfx = right_arm.fist.VX();
		vfy = right_arm.fist.VY();
		vfz = right_arm.fist.VZ();

		rfx = right_arm.fist.RX();
		rfy = right_arm.fist.RY();
		rfz = right_arm.fist.RZ();
    } else {
		vnx = left_arm.fist.NX();
		vny = left_arm.fist.NY();
		vnz = left_arm.fist.NZ();
		vfx = left_arm.fist.VX();
		vfy = left_arm.fist.VY();
		vfz = left_arm.fist.VZ();

		rfx = left_arm.fist.RX();
		rfy = left_arm.fist.RY();
		rfz = left_arm.fist.RZ();
	}

/*	vgxs = VGX();
	vgys = VGY();
	vgzs = VGZ();
	wgas = WGA();
	wgbs = WGB();
	wggs = WGG();  */

//	rf_abs = sqrt(rfx*rfx + rfy*rfy + rfz*rfz);
	wna = rfy * vnz - rfz * vny;
	wnb = rfz * vnx - rfx * vnz;
	wng = rfx * vny - rfy * vnx;

	wfa = rfy * vfz - rfz * vfy;
	wfb = rfz * vfx - rfx * vfz;
	wfg = rfx * vfy - rfy * vfx;

	vfvn = vfx * vnx + vfy * vny + vfz * vnz;
	vgvn = VGX() * vnx + VGY() * vny + VGZ() * vnz;

	wfwn = wfa * wna + wfb * wnb + wfg * wng;
	wgwn = WGA() * wna + WGB() * wnb + WGG() * wng;

	if ((vfvn > 0.0) && (vgvn > 0.0)) {
		damage_center = vgvn * Weight() * 50000.0;
	} else {
		damage_center = 0.0;		
	}

	if ((wfwn > 0.0) && (wgwn > 0.0)) {
		damage_rotate = wgwn * Weight() * 50000.0;
	} else {
		damage_rotate = 0.0;
	}
	
	return damage_center + damage_rotate; 
}

float Boxer::CalcCounterDamage(Boxer &another_boxer) {
	counter_vgvn = -another_boxer.VGX() * vnx - another_boxer.VGY() * vny - another_boxer.VGZ() * vnz;
	counter_wgwn = -another_boxer.WGA() * wna - another_boxer.WGB() * wnb - another_boxer.WGG() * wng;

	if ((vfvn > 0.0) && (counter_vgvn > 0.0)) {
		counter_damage_center = counter_vgvn * another_boxer.Weight() * 50000.0;
	} else {
		counter_damage_center = 0.0;	
	}		

	if ((wfwn > 0.0) && (counter_wgwn > 0.0)) {
		counter_damage_rotate = counter_wgwn * another_boxer.Weight() * 50000.0;
	} else {
		counter_damage_rotate = 0.0;
	}
	
	return counter_damage_center + counter_damage_rotate; 
}

void Boxer::CallListsSelect() {
	glPushMatrix();

	glTranslatef(X(), Y(), Z());
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glPushMatrix();
	body.CallListsSelect(); 

	glPushMatrix();
	head.CallListsSelect(); 
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	left_arm.CallListsSelect(false); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	right_arm.CallListsSelect(true); 
	glPopMatrix();

	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(-ww, 0.0, 0.0);
	left_leg.CallListsSelect(false); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( ww, 0.0, 0.0);
	right_leg.CallListsSelect(true); 
	glPopMatrix(); 

	glPopMatrix(); 
}

void Boxer::MakeDLists(int partition, int face, int pattern) {
	switch (face) {
		case 0 : 
			head.MakeNoFace(partition);
		break;
		case 1 :
			head.MakeDLists(partition);
		break;
		default :
			head.MakePolygonFace(hair_style, eyebrow, eye, nose, mouth, partition);
		break;
	}
	body.MakeDLists(partition);
	switch (pattern) {
		case 1 :
			left_arm.MakePeople(partition);
			right_arm.MakePeople(partition);
			left_leg.MakePeople(partition);
			right_leg.MakePeople(partition);
		break;	
		default :
			left_arm.MakeDLists(partition);
			right_arm.MakeDLists(partition);
			left_leg.MakeDLists(partition);
			right_leg.MakeDLists(partition);
		break;
	}

	CalcWeight();
	CalcHeight();
	CalcReach();
}

void Boxer::MoveStart() {
	glPushMatrix();

	glLoadIdentity();
	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);
	glTranslatef(X(), Y(), Z());

	matrix.Get();
	oxs = matrix.M14() - ox;
	oys = matrix.M24() - oy;

	glPopMatrix();
}

void Boxer::Move() {
	glPushMatrix();

	glLoadIdentity();
	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);
	glTranslatef(X(), Y(), Z());

	matrix.Get();
	ox = matrix.M14();
	oy = matrix.M24();

	glPopMatrix();
}

void Boxer::MoveToRhythm() {
	glPushMatrix();

	glLoadIdentity();
	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);
	glTranslatef(X(), Y(), Z());

	matrix.Get();
	ox = matrix.M14() - oxs;
	oy = matrix.M24() - oys;

	glPopMatrix();
}

void Boxer::GoNeutralCorner(Boxer &another_boxer) {
	float d1, x1, y1, d2, x2, y2;
	x1 = 2.3 - another_boxer.OX();
	y1 = 2.3 - another_boxer.OY();
	d1 = x1*x1 + y1*y1;
	x2 = -2.3 - another_boxer.OX();
	y2 = -2.3 - another_boxer.OY();
	d2 = x2*x2 + y2*y2;

	if (d1 > d2) {
		ox += (2.3 - ox) * 0.005;
		oy += (2.3 - oy) * 0.005;		
	} else {
		ox += (-2.3 - ox) * 0.005;
		oy += (-2.3 - oy) * 0.005;			
	}
}

void Boxer::GoBlueCorner() {
	ox += (-2.3 - ox) * 0.005;
	oy += ( 2.3 - oy) * 0.005;		
}

void Boxer::GoRedCorner() {
	ox += ( 2.3 - ox) * 0.005;
	oy += (-2.3 - oy) * 0.005;		
}

void Boxer::MakeRanges(int partition) {
	right_arm.fist.MakeRange(partition);
	right_arm.forearm.MakeRange(partition);
	right_arm.upperarm.MakeRange(partition);
	right_arm.shoulder.MakeRange(partition);
	left_arm.fist.MakeRange(partition);
	left_arm.forearm.MakeRange(partition);
	left_arm.upperarm.MakeRange(partition);
	left_arm.shoulder.MakeRange(partition); 
	body.abdomen.MakeRange(partition);
	body.abdomen.MakeLiver(partition);
	head.face.MakeRange(partition);
	head.face.MakeJaw(partition); 
}

void Boxer::SetRanges() {
	right_arm.fist.SetRange(right_arm.fist.C1());
	right_arm.forearm.SetRange((
		right_arm.forearm.A1() + right_arm.forearm.A2() + 
		right_arm.forearm.B1() + right_arm.forearm.B2()) * 0.25);
	right_arm.upperarm.SetRange((
		right_arm.upperarm.A1() + right_arm.upperarm.A2() + 
		right_arm.upperarm.B1() + right_arm.upperarm.B2()) * 0.25);
	right_arm.shoulder.SetRange(right_arm.shoulder.C1());

	left_arm.fist.SetRange(left_arm.fist.C1());
	left_arm.forearm.SetRange((
		left_arm.forearm.A1() + left_arm.forearm.A2() + 
		left_arm.forearm.B1() + left_arm.forearm.B2()) * 0.25);
	left_arm.upperarm.SetRange((
		left_arm.upperarm.A1() + left_arm.upperarm.A2() + 
		left_arm.upperarm.B1() + left_arm.upperarm.B2()) * 0.25);
	left_arm.shoulder.SetRange(left_arm.shoulder.C1());

	body.abdomen.SetRange((
		body.abdomen.A1() + body.abdomen.A2() + 
		body.abdomen.B1() + body.abdomen.B2()) * 0.25,
		body.abdomen.C1() * 0.5, body.abdomen.C2() * 0.5);
	body.abdomen.SetLiver(0.05,  body.abdomen.A2() * 0.5, body.abdomen.B1(), body.abdomen.C2() * 0.5);

	head.face.SetRange((
		head.face.A1() + head.face.A2() + 
		head.face.B1() + head.face.B2()) * 0.25,
		head.face.C1() * 0.8, head.face.C2() * 0.8);
	head.face.SetJaw(0.05, 0.0, head.face.B1(), -head.face.C2()*0.8);
}

void Boxer::SetMaterial(GLMaterial cmat1, GLMaterial cmat2, GLMaterial cmat3, GLMaterial cmat4, GLMaterial cmat5) {
	skin = cmat1;
	glove = cmat2;
	bandage = cmat3;
	trunks = cmat4;
	shoes = cmat5;
	head.SetMaterial(&skin, &hair);
	body.SetMaterial(&trunks, &skin);
	left_arm.SetMaterial(&glove, &bandage, &skin);
	right_arm.SetMaterial(&glove, &bandage, &skin);
	left_leg.SetMaterial(&shoes, &skin, &trunks);
	right_leg.SetMaterial(&shoes, &skin, &trunks);
}

void Boxer::SetDefaultFace() {
	hair.DefDiffuse(0.0, 0.0, 0.0);
	hair.DefSpecular(0.5, 0.5, 0.5);
	hair.DefShininess(128.0);

	hair_style = 1;
	eyebrow = 1;
	eye = 1;
	nose = 1;
	mouth = 1;	
}

void Boxer::CalcCoords() {
	glPushMatrix();

	glLoadIdentity();
	glTranslatef(ox, oy, 0.0);
	glRotatef(ot, 0.0, 0.0, 1.0);
	
	glTranslatef(X(), Y(), Z());
/*	matrix.Get();
	cx = matrix.M14();
	cy = matrix.M24(); */
	glRotatef(Alpha(), 1.0, 0.0, 0.0);
	glRotatef(Beta(),  0.0, 1.0, 0.0);
	glRotatef(Gamma(), 0.0, 0.0, 1.0); 

	glPushMatrix();
	body.CalcCoords(); 

	glPushMatrix();
	head.CalcCoords(); 
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(-sw, 0.0, 0.0);
	left_arm.CalcCoords(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( sw, 0.0, 0.0);
	right_arm.CalcCoords(); 
	glPopMatrix();

	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(-ww, 0.0, 0.0);
	left_leg.CalcCoords(); 
	glPopMatrix();

	glPushMatrix();
	glTranslatef( ww, 0.0, 0.0);
	right_leg.CalcCoords(); 
	glPopMatrix(); 

	glPopMatrix();
}

void Boxer::Copy(Boxer *cboxer) {
	this->sw = cboxer->sw;
	this->ww = cboxer->ww;
	this->ox = cboxer->ox;
	this->oy = cboxer->oy;
	this->ot = cboxer->ot;
	this->SetPos(cboxer->X(), cboxer->Y(), cboxer->Z()); 

	this->right_arm.fist.Copy(&cboxer->right_arm.fist);
	this->right_arm.forearm.Copy(&cboxer->right_arm.forearm);
	this->right_arm.upperarm.Copy(&cboxer->right_arm.upperarm);
	this->right_arm.shoulder.Copy(&cboxer->right_arm.shoulder);

	this->left_arm.fist.Copy(&cboxer->left_arm.fist);
	this->left_arm.forearm.Copy(&cboxer->left_arm.forearm);
	this->left_arm.upperarm.Copy(&cboxer->left_arm.upperarm);
	this->left_arm.shoulder.Copy(&cboxer->left_arm.shoulder);

	this->right_leg.toe.Copy(&cboxer->right_leg.toe);
	this->right_leg.foot.Copy(&cboxer->right_leg.foot);
	this->right_leg.shank.Copy(&cboxer->right_leg.shank);
	this->right_leg.thigh.Copy(&cboxer->right_leg.thigh);

	this->left_leg.toe.Copy(&cboxer->left_leg.toe);
	this->left_leg.foot.Copy(&cboxer->left_leg.foot);
	this->left_leg.shank.Copy(&cboxer->left_leg.shank);
	this->left_leg.thigh.Copy(&cboxer->left_leg.thigh);

	this->body.waist.Copy(&cboxer->body.waist);
	this->body.abdomen.Copy(&cboxer->body.abdomen);
	this->body.breast.Copy(&cboxer->body.breast);

	this->head.neck.Copy(&cboxer->head.neck);
	this->head.face.Copy(&cboxer->head.face);

	this->glove = cboxer->glove;
	this->bandage = cboxer->bandage;	
	this->skin = cboxer->skin;
	this->trunks = cboxer->trunks;
	this->shoes = cboxer->shoes;
} 	

void Boxer::CalcWeight() {
	w = 0.0;
	w += left_arm.fist.Weight();
	w += left_arm.forearm.Weight();
	w += left_arm.upperarm.Weight();
	w += left_arm.shoulder.Weight();
	w += right_arm.fist.Weight();
	w += right_arm.forearm.Weight();
	w += right_arm.upperarm.Weight();
	w += right_arm.shoulder.Weight();
	w += left_leg.toe.Weight();
	w += left_leg.foot.Weight();
	w += left_leg.shank.Weight();
	w += left_leg.thigh.Weight();
	w += right_leg.toe.Weight();
	w += right_leg.foot.Weight();
	w += right_leg.shank.Weight();
	w += right_leg.thigh.Weight();
	w += body.breast.Weight();
	w += body.abdomen.Weight();
	w += body.waist.Weight();
	w += head.face.Weight();
	w += head.neck.Weight();
}

void Boxer::CalcHeight() {
	h = 0.0;
	h += body.waist.Translate();
	h += body.waist.Length();
	h += body.abdomen.Translate();
	h += body.abdomen.Length();
	h += body.breast.Translate();
	h += head.neck.Translate();
	h += head.neck.Length();
	h += head.face.Translate();
	h += head.face.Length();

	CalcLegHeight();
	h += LegHeight();	
}

void Boxer::CalcReach() {
	r = 0.0;
	r += ShoulderWidth() * 2.0;
	r += right_arm.shoulder.Length();
	r += right_arm.upperarm.Translate();
	r += right_arm.upperarm.Length();
	r += right_arm.forearm.Translate();
	r += right_arm.forearm.Length();
	r += right_arm.fist.Translate();
	r += right_arm.fist.Length();
	r += left_arm.shoulder.Length();
	r += left_arm.upperarm.Translate();
	r += left_arm.upperarm.Length();
	r += left_arm.forearm.Translate();
	r += left_arm.forearm.Length();
	r += left_arm.fist.Translate();
	r += left_arm.fist.Length();
}

void Boxer::CalcLegHeight() {
	float lh_right = 0.0, lh_left = 0.0;	

	lh_right += right_leg.thigh.Translate();
	lh_right += right_leg.thigh.Length();
	lh_right += right_leg.shank.Translate();
	lh_right += right_leg.shank.Length();
	lh_right += right_leg.foot.Translate();

	lh_left += left_leg.thigh.Translate();
	lh_left += left_leg.thigh.Length();
	lh_left += left_leg.shank.Translate();
	lh_left += left_leg.shank.Length();
	lh_left += left_leg.foot.Translate();
	
	if (lh_right > lh_left) {
		lh = lh_right;	
	} else {
		lh = lh_left;		
	}
}

void Boxer::InitCenter() {
	CalcCoords();
	CalcCenter();

	gx_pre = gx;
	gy_pre = gy;
	gz_pre = gz;	

	vgx = 0.0;
	vgy = 0.0;
	vgz = 0.0;	

	right_arm.fist.InitVelocity();
	right_arm.forearm.InitVelocity();
	right_arm.upperarm.InitVelocity();
	right_arm.shoulder.InitVelocity();
	left_arm.fist.InitVelocity();
	left_arm.forearm.InitVelocity();
	left_arm.upperarm.InitVelocity();
	left_arm.shoulder.InitVelocity();
	right_leg.toe.InitVelocity();
	right_leg.foot.InitVelocity();
	right_leg.shank.InitVelocity();
	right_leg.thigh.InitVelocity();
	left_leg.toe.InitVelocity();
	left_leg.foot.InitVelocity();
	left_leg.shank.InitVelocity();
	left_leg.thigh.InitVelocity();
	body.waist.InitVelocity();
	body.abdomen.InitVelocity();
	body.breast.InitVelocity();
	head.neck.InitVelocity();
	head.face.InitVelocity();
}

void Boxer::CalcCenter() {
	gx = right_arm.fist.Weight() * right_arm.fist.GX() +
		right_arm.forearm.Weight() * right_arm.forearm.GX() +
		right_arm.upperarm.Weight() * right_arm.upperarm.GX() +
		right_arm.shoulder.Weight() * right_arm.shoulder.GX() +
		left_arm.fist.Weight() * left_arm.fist.GX() +
		left_arm.forearm.Weight() * left_arm.forearm.GX() +
		left_arm.upperarm.Weight() * left_arm.upperarm.GX() +
		left_arm.shoulder.Weight() * left_arm.shoulder.GX() +
		right_leg.toe.Weight() * right_leg.toe.GX() +
		right_leg.foot.Weight() * right_leg.foot.GX() +
		right_leg.shank.Weight() * right_leg.shank.GX() +
		right_leg.thigh.Weight() * right_leg.thigh.GX() +
		left_leg.toe.Weight() * left_leg.toe.GX() +
		left_leg.foot.Weight() * left_leg.foot.GX() +
		left_leg.shank.Weight() * left_leg.shank.GX() +
		left_leg.thigh.Weight() * left_leg.thigh.GX() +
		body.waist.Weight() * body.waist.GX() +
		body.abdomen.Weight() * body.abdomen.GX() +
		body.breast.Weight() * body.breast.GX() +
		head.neck.Weight() * head.neck.GX() +
		head.face.Weight() * head.face.GX();
	gx /= Weight();
	
	gy = right_arm.fist.Weight() * right_arm.fist.GY() +
		right_arm.forearm.Weight() * right_arm.forearm.GY() +
		right_arm.upperarm.Weight() * right_arm.upperarm.GY() +
		right_arm.shoulder.Weight() * right_arm.shoulder.GY() +
		left_arm.fist.Weight() * left_arm.fist.GY() +
		left_arm.forearm.Weight() * left_arm.forearm.GY() +
		left_arm.upperarm.Weight() * left_arm.upperarm.GY() +
		left_arm.shoulder.Weight() * left_arm.shoulder.GY() +
		right_leg.toe.Weight() * right_leg.toe.GY() +
		right_leg.foot.Weight() * right_leg.foot.GY() +
		right_leg.shank.Weight() * right_leg.shank.GY() +
		right_leg.thigh.Weight() * right_leg.thigh.GY() +
		left_leg.toe.Weight() * left_leg.toe.GY() +
		left_leg.foot.Weight() * left_leg.foot.GY() +
		left_leg.shank.Weight() * left_leg.shank.GY() +
		left_leg.thigh.Weight() * left_leg.thigh.GY() +
		body.waist.Weight() * body.waist.GY() +
		body.abdomen.Weight() * body.abdomen.GY() +
		body.breast.Weight() * body.breast.GY() +
		head.neck.Weight() * head.neck.GY() +
		head.face.Weight() * head.face.GY();
	gy /= Weight();
	
	gz = right_arm.fist.Weight() * right_arm.fist.GZ() +
		right_arm.forearm.Weight() * right_arm.forearm.GZ() +
		right_arm.upperarm.Weight() * right_arm.upperarm.GZ() +
		right_arm.shoulder.Weight() * right_arm.shoulder.GZ() +
		left_arm.fist.Weight() * left_arm.fist.GZ() +
		left_arm.forearm.Weight() * left_arm.forearm.GZ() +
		left_arm.upperarm.Weight() * left_arm.upperarm.GZ() +
		left_arm.shoulder.Weight() * left_arm.shoulder.GZ() +
		right_leg.toe.Weight() * right_leg.toe.GZ() +
		right_leg.foot.Weight() * right_leg.foot.GZ() +
		right_leg.shank.Weight() * right_leg.shank.GZ() +
		right_leg.thigh.Weight() * right_leg.thigh.GZ() +
		left_leg.toe.Weight() * left_leg.toe.GZ() +
		left_leg.foot.Weight() * left_leg.foot.GZ() +
		left_leg.shank.Weight() * left_leg.shank.GZ() +
		left_leg.thigh.Weight() * left_leg.thigh.GZ() +
		body.waist.Weight() * body.waist.GZ() +
		body.abdomen.Weight() * body.abdomen.GZ() +
		body.breast.Weight() * body.breast.GZ() +
		head.neck.Weight() * head.neck.GZ() +
		head.face.Weight() * head.face.GZ();
	gz /= Weight();
	
	vgx = gx - gx_pre;
	vgy = gy - gy_pre;
	vgz = gz - gz_pre;
	gx_pre = gx;
	gy_pre = gy;
	gz_pre = gz;
/*	
	agx = vgx - vgx_pre;
	agy = vgy - vgy_pre;
	agz = vgz - vgz_pre;
	vgx_pre = vgx;
	vgy_pre = vgy;
	vgz_pre = vgz;
*/
	right_arm.fist.CalcVelocity(gx, gy, gz);
	right_arm.forearm.CalcVelocity(gx, gy, gz);
	right_arm.upperarm.CalcVelocity(gx, gy, gz);
	right_arm.shoulder.CalcVelocity(gx, gy, gz);
	left_arm.fist.CalcVelocity(gx, gy, gz);
	left_arm.forearm.CalcVelocity(gx, gy, gz);
	left_arm.upperarm.CalcVelocity(gx, gy, gz);
	left_arm.shoulder.CalcVelocity(gx, gy, gz);
	right_leg.toe.CalcVelocity(gx, gy, gz);
	right_leg.foot.CalcVelocity(gx, gy, gz);
	right_leg.shank.CalcVelocity(gx, gy, gz);
	right_leg.thigh.CalcVelocity(gx, gy, gz);
	left_leg.toe.CalcVelocity(gx, gy, gz);
	left_leg.foot.CalcVelocity(gx, gy, gz);
	left_leg.shank.CalcVelocity(gx, gy, gz);
	left_leg.thigh.CalcVelocity(gx, gy, gz);
	body.waist.CalcVelocity(gx, gy, gz);
	body.abdomen.CalcVelocity(gx, gy, gz);
	body.breast.CalcVelocity(gx, gy, gz);
	head.neck.CalcVelocity(gx, gy, gz);
	head.face.CalcVelocity(gx, gy, gz);

	wga = right_arm.fist.WA() + 
		right_arm.forearm.WA() + 
		right_arm.upperarm.WA() + 
		right_arm.shoulder.WA() + 
		left_arm.fist.WA() + 
		left_arm.forearm.WA() + 
		left_arm.upperarm.WA() + 
		left_arm.shoulder.WA() + 
		right_leg.toe.WA() + 
		right_leg.foot.WA() + 
		right_leg.shank.WA() + 
		right_leg.thigh.WA() + 
		left_leg.toe.WA() + 
		left_leg.foot.WA() + 
		left_leg.shank.WA() + 
		left_leg.thigh.WA() + 
		body.waist.WA() + 
		body.abdomen.WA() + 
		body.breast.WA() + 
		head.neck.WA() + 
		head.face.WA();
	gai = right_arm.fist.AI() + 
		right_arm.forearm.AI() + 
		right_arm.upperarm.AI() + 
		right_arm.shoulder.AI() + 
		left_arm.fist.AI() + 
		left_arm.forearm.AI() + 
		left_arm.upperarm.AI() + 
		left_arm.shoulder.AI() + 
		right_leg.toe.AI() + 
		right_leg.foot.AI() + 
		right_leg.shank.AI() + 
		right_leg.thigh.AI() + 
		left_leg.toe.AI() + 
		left_leg.foot.AI() + 
		left_leg.shank.AI() + 
		left_leg.thigh.AI() + 
		body.waist.AI() + 
		body.abdomen.AI() + 
		body.breast.AI() + 
		head.neck.AI() + 
		head.face.AI();
	wga /= gai;

	wgb = right_arm.fist.WB() + 
		right_arm.forearm.WB() + 
		right_arm.upperarm.WB() + 
		right_arm.shoulder.WB() + 
		left_arm.fist.WB() + 
		left_arm.forearm.WB() + 
		left_arm.upperarm.WB() + 
		left_arm.shoulder.WB() + 
		right_leg.toe.WB() + 
		right_leg.foot.WB() + 
		right_leg.shank.WB() + 
		right_leg.thigh.WB() + 
		left_leg.toe.WB() + 
		left_leg.foot.WB() + 
		left_leg.shank.WB() + 
		left_leg.thigh.WB() + 
		body.waist.WB() + 
		body.abdomen.WB() + 
		body.breast.WB() + 
		head.neck.WB() + 
		head.face.WB();
	gbi = right_arm.fist.BI() + 
		right_arm.forearm.BI() + 
		right_arm.upperarm.BI() + 
		right_arm.shoulder.BI() + 
		left_arm.fist.BI() + 
		left_arm.forearm.BI() + 
		left_arm.upperarm.BI() + 
		left_arm.shoulder.BI() + 
		right_leg.toe.BI() + 
		right_leg.foot.BI() + 
		right_leg.shank.BI() + 
		right_leg.thigh.BI() + 
		left_leg.toe.BI() + 
		left_leg.foot.BI() + 
		left_leg.shank.BI() + 
		left_leg.thigh.BI() + 
		body.waist.BI() + 
		body.abdomen.BI() + 
		body.breast.BI() + 
		head.neck.BI() + 
		head.face.BI();
	wgb /= gbi;

	wgg = right_arm.fist.WG() + 
		right_arm.forearm.WG() + 
		right_arm.upperarm.WG() + 
		right_arm.shoulder.WG() + 
		left_arm.fist.WG() + 
		left_arm.forearm.WG() + 
		left_arm.upperarm.WG() + 
		left_arm.shoulder.WG() + 
		right_leg.toe.WG() + 
		right_leg.foot.WG() + 
		right_leg.shank.WG() + 
		right_leg.thigh.WG() + 
		left_leg.toe.WG() + 
		left_leg.foot.WG() + 
		left_leg.shank.WG() + 
		left_leg.thigh.WG() + 
		body.waist.WG() + 
		body.abdomen.WG() + 
		body.breast.WG() + 
		head.neck.WG() + 
		head.face.WG();
	ggi = right_arm.fist.GI() + 
		right_arm.forearm.GI() + 
		right_arm.upperarm.GI() + 
		right_arm.shoulder.GI() + 
		left_arm.fist.GI() + 
		left_arm.forearm.GI() + 
		left_arm.upperarm.GI() + 
		left_arm.shoulder.GI() + 
		right_leg.toe.GI() + 
		right_leg.foot.GI() + 
		right_leg.shank.GI() + 
		right_leg.thigh.GI() + 
		left_leg.toe.GI() + 
		left_leg.foot.GI() + 
		left_leg.shank.GI() + 
		left_leg.thigh.GI() + 
		body.waist.GI() + 
		body.abdomen.GI() + 
		body.breast.GI() + 
		head.neck.GI() + 
		head.face.GI();
	wgg /= ggi;
}

bool Boxer::CanMove() {
	if (ox > 2.3) {
		return false;
	}
	if (ox < -2.3) {
		return false;
	}
	if (oy > 2.3) {
		return false;
	}
	if (oy < -2.3) {
		return false;
	}
	return true;
}

int Boxer::WhichPosition() {
	if (ox > 2.0) {
		if (oy > 2.0) {
			return CORNER_OF_RING;
		} else if (oy >= -2.0) {
			return ROPE_SIDE_OF_RING;			
		} else {
			return CORNER_OF_RING;	
		}
	} else if (ox >= -2.0) {
		if (oy > 2.0) {
			return ROPE_SIDE_OF_RING;		
		} else if (oy >= -2.0) {
			return CENTER_OF_RING;				
		} else {
			return ROPE_SIDE_OF_RING;			
		}	
	} else {
		if (oy > 2.0) {
			return CORNER_OF_RING;
		} else if (oy >= -2.0) {
			return ROPE_SIDE_OF_RING;		
		} else {
			return CORNER_OF_RING;	
		}	
	}
}

float Boxer::CalcDistance(Boxer &another_boxer) {
	float distance, dx, dy;
	dx = body.waist.GX() - another_boxer.body.waist.GX();
	dy = body.waist.GY() - another_boxer.body.waist.GY();
	distance = sqrt(dx * dx + dy * dy);
	return distance;
}

bool Boxer::CanForward(float distance, float adjust_distance) {
	if (distance < adjust_distance) {
		return false;
	}
	return CanMove();
}

void Boxer::PrintCoords() {
	fprintf(stdout, "--- Boxer Coordinates ---\n");
	fprintf(stdout, "Waist       x = %f y = %f z = %f\n", body.waist.X(),   body.waist.Y(),   body.waist.Z());
	fprintf(stdout, "Abdomen     x = %f y = %f z = %f\n", body.abdomen.X(), body.abdomen.Y(), body.abdomen.Z());
	fprintf(stdout, "Breast      x = %f y = %f z = %f\n", body.breast.X(),  body.breast.Y(),  body.breast.Z());
	fprintf(stdout, "Neck        x = %f y = %f z = %f\n", head.neck.X(), head.neck.Y(), head.neck.Z());
	fprintf(stdout, "Face        x = %f y = %f z = %f\n", head.face.X(), head.face.Y(), head.face.Z());
	fprintf(stdout, "Face Dir    x = %f y = %f z = %f\n", head.face.DirX(), head.face.DirY(), head.face.DirZ());
	fprintf(stdout, "Face Head   x = %f y = %f z = %f\n", head.face.HeadX(), head.face.HeadY(), head.face.HeadZ());
	fprintf(stdout, "Fist R      x = %f y = %f z = %f\n", right_arm.fist.X(),     right_arm.fist.Y(),     right_arm.fist.Z());
	fprintf(stdout, "Forearm R   x = %f y = %f z = %f\n", right_arm.forearm.X(),  right_arm.forearm.Y(),  right_arm.forearm.Z());
	fprintf(stdout, "Upperarm R  x = %f y = %f z = %f\n", right_arm.upperarm.X(), right_arm.upperarm.Y(), right_arm.upperarm.Z());
	fprintf(stdout, "Shoulder R  x = %f y = %f z = %f\n", right_arm.shoulder.X(), right_arm.shoulder.Y(), right_arm.shoulder.Z());
	fprintf(stdout, "Fist L      x = %f y = %f z = %f\n", left_arm.fist.X(),     left_arm.fist.Y(),     left_arm.fist.Z());
	fprintf(stdout, "Forearm L   x = %f y = %f z = %f\n", left_arm.forearm.X(),  left_arm.forearm.Y(),  left_arm.forearm.Z());
	fprintf(stdout, "Upperarm L  x = %f y = %f z = %f\n", left_arm.upperarm.X(), left_arm.upperarm.Y(), left_arm.upperarm.Z());
	fprintf(stdout, "Shoulder L  x = %f y = %f z = %f\n", left_arm.shoulder.X(), left_arm.shoulder.Y(), left_arm.shoulder.Z());
	fprintf(stdout, "Toe R       x = %f y = %f z = %f\n", right_leg.toe.X(),   right_leg.toe.Y(),   right_leg.toe.Z());
	fprintf(stdout, "Foot R      x = %f y = %f z = %f\n", right_leg.foot.X(),  right_leg.foot.Y(),  right_leg.foot.Z());
	fprintf(stdout, "Shank R     x = %f y = %f z = %f\n", right_leg.shank.X(), right_leg.shank.Y(), right_leg.shank.Z());
	fprintf(stdout, "Thigh R     x = %f y = %f z = %f\n", right_leg.thigh.X(), right_leg.thigh.Y(), right_leg.thigh.Z());
	fprintf(stdout, "Toe L       x = %f y = %f z = %f\n", left_leg.toe.X(),   left_leg.toe.Y(),   left_leg.toe.Z());
	fprintf(stdout, "Foot L      x = %f y = %f z = %f\n", left_leg.foot.X(),  left_leg.foot.Y(),  left_leg.foot.Z());
	fprintf(stdout, "Shank L     x = %f y = %f z = %f\n", left_leg.shank.X(), left_leg.shank.Y(), left_leg.shank.Z());
	fprintf(stdout, "Thigh L     x = %f y = %f z = %f\n", left_leg.thigh.X(), left_leg.thigh.Y(), left_leg.thigh.Z());
}

void Boxer::PrintAngles() {
	fprintf(stdout, "--- Boxer Angles ---\n");
	fprintf(stdout, "Waist       a = %f b = %f g = %f\n", body.waist.Alpha(),   body.waist.Beta(),   body.waist.Gamma());
	fprintf(stdout, "Abdomen     a = %f b = %f g = %f\n", body.abdomen.Alpha(), body.abdomen.Beta(), body.abdomen.Gamma());
	fprintf(stdout, "Breast      a = %f b = %f g = %f\n", body.breast.Alpha(),  body.breast.Beta(),  body.breast.Gamma());
	fprintf(stdout, "Neck        a = %f b = %f g = %f\n", head.neck.Alpha(), head.neck.Beta(), head.neck.Gamma());
	fprintf(stdout, "Face        a = %f b = %f g = %f\n", head.face.Alpha(), head.face.Beta(), head.face.Gamma());
	fprintf(stdout, "Fist R      a = %f b = %f g = %f\n", right_arm.fist.Alpha(),     right_arm.fist.Beta(),     right_arm.fist.Gamma());
	fprintf(stdout, "Forearm R   a = %f b = %f g = %f\n", right_arm.forearm.Alpha(),  right_arm.forearm.Beta(),  right_arm.forearm.Gamma());
	fprintf(stdout, "Upperarm R  a = %f b = %f g = %f\n", right_arm.upperarm.Alpha(), right_arm.upperarm.Beta(), right_arm.upperarm.Gamma());
	fprintf(stdout, "Shoulder R  a = %f b = %f g = %f\n", right_arm.shoulder.Alpha(), right_arm.shoulder.Beta(), right_arm.shoulder.Gamma());
	fprintf(stdout, "Fist L      a = %f b = %f g = %f\n", left_arm.fist.Alpha(),     left_arm.fist.Beta(),     left_arm.fist.Gamma());
	fprintf(stdout, "Forearm L   a = %f b = %f g = %f\n", left_arm.forearm.Alpha(),  left_arm.forearm.Beta(),  left_arm.forearm.Gamma());
	fprintf(stdout, "Upperarm L  a = %f b = %f g = %f\n", left_arm.upperarm.Alpha(), left_arm.upperarm.Beta(), left_arm.upperarm.Gamma());
	fprintf(stdout, "Shoulder L  a = %f b = %f g = %f\n", left_arm.shoulder.Alpha(), left_arm.shoulder.Beta(), left_arm.shoulder.Gamma());
	fprintf(stdout, "Toe R       a = %f b = %f g = %f\n", right_leg.toe.Alpha(),   right_leg.toe.Beta(),   right_leg.toe.Gamma());
	fprintf(stdout, "Foot R      a = %f b = %f g = %f\n", right_leg.foot.Alpha(),  right_leg.foot.Beta(),  right_leg.foot.Gamma());
	fprintf(stdout, "Shank R     a = %f b = %f g = %f\n", right_leg.shank.Alpha(), right_leg.shank.Beta(), right_leg.shank.Gamma());
	fprintf(stdout, "Thigh R     a = %f b = %f g = %f\n", right_leg.thigh.Alpha(), right_leg.thigh.Beta(), right_leg.thigh.Gamma());
	fprintf(stdout, "Toe L       a = %f b = %f g = %f\n", left_leg.toe.Alpha(),   left_leg.toe.Beta(),   left_leg.toe.Gamma());
	fprintf(stdout, "Foot L      a = %f b = %f g = %f\n", left_leg.foot.Alpha(),  left_leg.foot.Beta(),  left_leg.foot.Gamma());
	fprintf(stdout, "Shank L     a = %f b = %f g = %f\n", left_leg.shank.Alpha(), left_leg.shank.Beta(), left_leg.shank.Gamma());
	fprintf(stdout, "Thigh L     a = %f b = %f g = %f\n", left_leg.thigh.Alpha(), left_leg.thigh.Beta(), left_leg.thigh.Gamma());
}

void Boxer::PrintAbsAngles() {
	fprintf(stdout, "--- Boxer Absolute Angles ---\n");
	fprintf(stdout, "Waist       a = %f b = %f g = %f\n", body.waist.AbsAlpha(),   body.waist.AbsBeta(),   body.waist.AbsGamma());
	fprintf(stdout, "Abdomen     a = %f b = %f g = %f\n", body.abdomen.AbsAlpha(), body.abdomen.AbsBeta(), body.abdomen.AbsGamma());
	fprintf(stdout, "Breast      a = %f b = %f g = %f\n", body.breast.AbsAlpha(),  body.breast.AbsBeta(),  body.breast.AbsGamma());
	fprintf(stdout, "Neck        a = %f b = %f g = %f\n", head.neck.AbsAlpha(), head.neck.AbsBeta(), head.neck.AbsGamma());
	fprintf(stdout, "Face        a = %f b = %f g = %f\n", head.face.AbsAlpha(), head.face.AbsBeta(), head.face.AbsGamma());
	fprintf(stdout, "Fist R      a = %f b = %f g = %f\n", right_arm.fist.AbsAlpha(),     right_arm.fist.AbsBeta(),     right_arm.fist.AbsGamma());
	fprintf(stdout, "Forearm R   a = %f b = %f g = %f\n", right_arm.forearm.AbsAlpha(),  right_arm.forearm.AbsBeta(),  right_arm.forearm.AbsGamma());
	fprintf(stdout, "Upperarm R  a = %f b = %f g = %f\n", right_arm.upperarm.AbsAlpha(), right_arm.upperarm.AbsBeta(), right_arm.upperarm.AbsGamma());
	fprintf(stdout, "Shoulder R  a = %f b = %f g = %f\n", right_arm.shoulder.AbsAlpha(), right_arm.shoulder.AbsBeta(), right_arm.shoulder.AbsGamma());
	fprintf(stdout, "Fist L      a = %f b = %f g = %f\n", left_arm.fist.AbsAlpha(),     left_arm.fist.AbsBeta(),     left_arm.fist.AbsGamma());
	fprintf(stdout, "Forearm L   a = %f b = %f g = %f\n", left_arm.forearm.AbsAlpha(),  left_arm.forearm.AbsBeta(),  left_arm.forearm.AbsGamma());
	fprintf(stdout, "Upperarm L  a = %f b = %f g = %f\n", left_arm.upperarm.AbsAlpha(), left_arm.upperarm.AbsBeta(), left_arm.upperarm.AbsGamma());
	fprintf(stdout, "Shoulder L  a = %f b = %f g = %f\n", left_arm.shoulder.AbsAlpha(), left_arm.shoulder.AbsBeta(), left_arm.shoulder.AbsGamma());
	fprintf(stdout, "Toe R       a = %f b = %f g = %f\n", right_leg.toe.AbsAlpha(),   right_leg.toe.AbsBeta(),   right_leg.toe.AbsGamma());
	fprintf(stdout, "Foot R      a = %f b = %f g = %f\n", right_leg.foot.AbsAlpha(),  right_leg.foot.AbsBeta(),  right_leg.foot.AbsGamma());
	fprintf(stdout, "Shank R     a = %f b = %f g = %f\n", right_leg.shank.AbsAlpha(), right_leg.shank.AbsBeta(), right_leg.shank.AbsGamma());
	fprintf(stdout, "Thigh R     a = %f b = %f g = %f\n", right_leg.thigh.AbsAlpha(), right_leg.thigh.AbsBeta(), right_leg.thigh.AbsGamma());
	fprintf(stdout, "Toe L       a = %f b = %f g = %f\n", left_leg.toe.AbsAlpha(),   left_leg.toe.AbsBeta(),   left_leg.toe.AbsGamma());
	fprintf(stdout, "Foot L      a = %f b = %f g = %f\n", left_leg.foot.AbsAlpha(),  left_leg.foot.AbsBeta(),  left_leg.foot.AbsGamma());
	fprintf(stdout, "Shank L     a = %f b = %f g = %f\n", left_leg.shank.AbsAlpha(), left_leg.shank.AbsBeta(), left_leg.shank.AbsGamma());
	fprintf(stdout, "Thigh L     a = %f b = %f g = %f\n", left_leg.thigh.AbsAlpha(), left_leg.thigh.AbsBeta(), left_leg.thigh.AbsGamma());
}
