#ifndef BOXER_H
#define BOXER_H

#include "myglheader.h"
#include "glshapes.h"
#include "gldlist.h"
#include "glmaterial.h"

#ifndef Uint8
#define Uint8 unsigned char
#endif

class BodyPart;
class Fist;
class Forearm;
class Upperarm;
class Shoulder;
class Arm;

class Toe;
class Foot;
class Shank;
class Thigh;
class Leg;

class Waist;
class Abdomen;
class Breast;
class Body;

class Neck;
class Face;
class Head;

class Boxer;

class BodyPart {
public:
	BodyPart();
	virtual ~BodyPart() {}
	void CallList() {dlist.CallList();}
	float Length() const {return l;}
	float Weight() const {return w;}
	float Translate() const {return t;}
	void CalcWeight();
	void CalcAbsAngle();
	float Alpha() const {return a;}
	float Beta() const {return b;}
	float Gamma() const {return g;}
	float AbsAlpha() const {return aa;}
	float AbsBeta() const {return ab;}
	float AbsGamma() const {return ag;}
	void SetSize(float ca1, float ca2, float cb1, float cb2, float cc1, float cc2);
	void SetPos(float cx, float cy, float cz) {x = cx; y = cy; z = cz;}
	void SetPosPre(float cx, float cy, float cz) {x_pre = cx; y_pre = cy; z_pre = cz;}
	void SetGPos(float cx, float cy, float cz) {gx = cx; gy = cy; gz = cz;}
	void SetAngle(float ca, float cb, float cg) {a = ca; b = cb; g = cg;}	
	void SetAbsAngle(float ca, float cb, float cg) {aa = ca; ab = cb; ag = cg;}	
	void MovePos(float dx = 0.0, float dy = 0.0, float dz = 0.0) {x += dx; y += dy; z += dz;}
	void MoveAngle(float da = 0.0, float db= 0.0, float dg = 0.0) {a += da; b += db; g = dg;}
	void MoveAbsAngle(float daa = 0.0, float dab= 0.0, float dag = 0.0) {aa += daa; ab += dab; ag += dag;}
	void Copy(BodyPart *cbodypart);
	void InitVelocity();
	void CalcVelocity(float gx, float gy, float gz);
	void AlphaP() {a+=1.0;}
	void BetaP()  {b+=1.0;}
	void GammaP() {g+=1.0;}
	void AlphaM() {a-=1.0;}
	void BetaM()  {b-=1.0;}
	void GammaM() {g-=1.0;}
	float A1() const {return a1;} 
	float A2() const {return a2;} 
	float B1() const {return b1;} 
	float B2() const {return b2;} 
	float C1() const {return c1;} 
	float C2() const {return c2;} 
	float X() const {return x;} 
	float Y() const {return y;} 
	float Z() const {return z;} 
	float XPre() const {return x_pre;} 
	float YPre() const {return y_pre;} 
	float ZPre() const {return z_pre;} 
	float GX() const {return gx;} 
	float GY() const {return gy;} 
	float GZ() const {return gz;} 
	float OX() const {return ox;} 
	float OY() const {return oy;} 
	float OZ() const {return oz;} 
	float RX() const {return rx;} 
	float RY() const {return ry;} 
	float RZ() const {return rz;} 
	float VX() const {return vx;}
	float VY() const {return vy;}
	float VZ() const {return vz;}	
/*	float AX() const {return ax;}
	float AY() const {return ay;}
	float AZ() const {return az;}*/	
	float WA() const {return wa;}
	float WB() const {return wb;}
	float WG() const {return wg;}	
	float AI() const {return ai;}
	float BI() const {return bi;}
	float GI() const {return gi;}	
	virtual void MakeDList() {}
protected:
	GLDList dlist;
	float w;
	float l, t;
	float x, y, z;
	float rx, ry, rz;
	float rx2, ry2, rz2;
	float wa, wb, wg;
	float ai, bi, gi;
	float gx, gy, gz;
	float ox, oy, oz;
	float a, b, g;	
	float aa, ab, ag;	
	float a1, b1, c1;
	float a2, b2, c2;	
	float vx, vy, vz;
	float x_pre, y_pre, z_pre;
/*	float ax, ay, az;
	float vx_pre, vy_pre, vz_pre; */
};

class Fist : public BodyPart {
public:
	Fist(bool right = true);
	~Fist();
	void MakeDList(int partition = 36);
	void MakePeople(int partition = 36);
	void MakeRange(int partition = 36);
	void CallRange() {range.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
	void SetRange(float cr) {r = cr;}
	float NX() const {return nx;}
	float NY() const {return ny;}
	float NZ() const {return nz;}	
	float R() const {return r;}
private:
	float nx, ny, nz;
	float r;
	GLDList range;
	GLMaterial *glove;
};

class Forearm : public BodyPart {
public:
	Forearm(bool right = true);
	~Forearm();
	void MakeDList(int partition = 36);
	void MakePeople(int partition = 36);
	void MakeRange(int partition = 36);
	void CallRange() {range.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2);
	float X1() const {return x1;} 
	float Y1() const {return y1;} 
	float Z1() const {return z1;} 
	float X2() const {return x2;} 
	float Y2() const {return y2;} 
	float Z2() const {return z2;} 
	float R() const {return r;}
	void SetRange(float cr) {r = cr;}
private:
	float r;
	float x1, y1, z1;
	float x2, y2, z2;	
	GLDList range;
	GLMaterial *skin, *bandage;
};

class Upperarm : public BodyPart {
public:
	Upperarm(bool right = true);
	~Upperarm();
	void MakeDList(int partition = 36);
	void MakeRange(int partition = 36);
	void CallRange() {range.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat1);
	float X1() const {return x1;} 
	float Y1() const {return y1;} 
	float Z1() const {return z1;} 
	float X2() const {return x2;} 
	float Y2() const {return y2;} 
	float Z2() const {return z2;} 
	float R() const {return r;}
	void SetRange(float cr) {r = cr;}
private:
	float r;
	float x1, y1, z1;
	float x2, y2, z2;	
	GLDList range;
	GLMaterial* skin;
};

class Shoulder : public BodyPart {
public:
	Shoulder(bool right = true);
	~Shoulder();
	void MakeDList(int partition = 36);
	void MakeRange(int partition = 36);
	void CallRange() {range.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
	float R() const {return r;}
	void SetRange(float cr) {r = cr;}
private:
	float r;
	GLDList range;
	GLMaterial* skin;
};

class Arm {
public:
	Arm(bool right = true);
	~Arm();
	void CallLists();
	void CallPeople();
	void CallListsFistOnly();
	void CallListsNoFist();
	void CallListsSelect(bool right);
	void MakeDLists(int partition = 36);
	void MakePeople(int partition = 36);
	void CalcCoords();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2, GLMaterial *cmat3);
	Fist fist;
	Forearm forearm;
	Upperarm upperarm;
	Shoulder shoulder;
private:
	GLMaterial *glove, *bandage, *skin;	 
};

class Toe : public BodyPart {
public:
	Toe(bool right = true);
	~Toe();
	void MakeDList(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
private:
	GLMaterial *shoes;
};

class Foot : public BodyPart {
public:
	Foot(bool right = true);
	~Foot();
	void MakeDList(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
private:
	GLMaterial *shoes;
};

class Shank : public BodyPart {
public:
	Shank(bool right = true);
	~Shank();
	void MakeDList(int partition = 36);
	void MakePeople(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2);
private:
	GLMaterial *shoes, *skin;
};

class Thigh : public BodyPart {
public:
	Thigh(bool right = true);
	~Thigh();
	void MakeDList(int partition = 36);
	void MakePeople(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2);
private:
	GLMaterial *skin, *trunks;
};

class Leg {
public:
	Leg(bool right = true);
	~Leg();
	void CallLists();
	void CallListsSelect(bool right);
	void MakeDLists(int partition = 36);
	void MakePeople(int partition = 36);
	void CalcCoords();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2, GLMaterial *cmat3);
	Toe toe;
	Foot foot;
	Shank shank;
	Thigh thigh;
private:
	GLMaterial *shoes, *skin, *trunks;	 	
};

class Waist : public BodyPart {
public:
	Waist();
	~Waist();
	void MakeDList(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
	void SetRelPos(float crel_x, float crel_y, float crel_z) {rel_x = crel_x; rel_y = crel_y; rel_z = crel_z;}
	float RelX() const {return rel_x;}
	float RelY() const {return rel_y;}		
	float RelZ() const {return rel_z;}
private:
	float rel_x, rel_y, rel_z;
	GLMaterial *trunks;
};

class Abdomen : public BodyPart {
public:
	Abdomen();
	~Abdomen();
	void MakeDList(int partition = 36);
	void MakeRange(int partition = 36);
	void MakeLiver(int partition = 36);
	void CallRange() {range.CallList();}
	void CallLiver() {liver.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
	float X1() const {return x1;} 
	float Y1() const {return y1;} 
	float Z1() const {return z1;} 
	float X2() const {return x2;} 
	float Y2() const {return y2;} 
	float Z2() const {return z2;} 
	float R() const {return r;}
	float Top() const {return top;}
	float Bottom() const {return bottom;}
	void SetRange(float cr, float ctop, float cbottom) {r = cr; top = ctop; bottom = cbottom;}
	float LR() const {return lr;}
	float LX() const {return lx;}
	float LY() const {return ly;}
	float LZ() const {return lz;}
	float LRX() const {return lrx;}
	float LRY() const {return lry;}
	float LRZ() const {return lrz;}
	void SetLiver(float clr, float clrx, float clry, float clrz) {lr = clr; lrx = clrx; lry = clry; lrz = clrz;}
private:
	float x1, y1, z1;
	float x2, y2, z2;	
	float r, top, bottom;
	float lr, lx, ly, lz;
	float lrx, lry, lrz;
	GLDList range;
	GLDList liver;	
	GLMaterial *skin;
};

class Breast : public BodyPart {
public:
	Breast();
	~Breast();
	void MakeDList(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
//	float DirX() const {return dx;}
//	float DirY() const {return dy;}
//	float DirZ() const {return dz;}
//	void SetDir(float cdx, float cdy, float cdz) {dx = cdx; dy =cdy; dz = cdz;}
private:
//	float dx, dy, dz;	
	GLMaterial *skin;
};

class Body {
public:
	Body();
	~Body();
	void CallLists();
	void CallListsSelect();
	void MakeDLists(int partition = 36);
	void CalcCoords();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2);
	Waist waist;
	Abdomen abdomen;
	Breast breast;
private:
	GLMaterial *trunks, *skin;
};

class Face : public BodyPart {
public:
	Face();
	~Face();
	void MakeDList(int partition = 36);
	void MakeNoFace(int partition = 36);
	void MakePolygonFace(Uint8 eye_form = 1, Uint8 nose_form = 1, Uint8 mouth_form = 1, int partition = 36);
	void MakeWig(Uint8 hair_style = 1, Uint8 eyebrow = 1);
	void MakeNoWig();
	void MakeRange(int partition = 36);
	void MakeJaw(int partition = 36);
	void CallWig() {wig.CallList();}
	void CallRange() {range.CallList();}
	void CallJaw() {jaw.CallList();}
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
	void SetDir(float cdx, float cdy, float cdz) {dx = cdx; dy =cdy; dz = cdz;}
	void SetHead(float chx, float chy, float chz) {hx = chx; hy =chy; hz = chz;}
	float DirX() const {return dx;}
	float DirY() const {return dy;}
	float DirZ() const {return dz;}
	float HeadX() const {return hx;}
	float HeadY() const {return hy;}
	float HeadZ() const {return hz;}
	float X1() const {return x1;} 
	float Y1() const {return y1;} 
	float Z1() const {return z1;} 
	float X2() const {return x2;} 
	float Y2() const {return y2;} 
	float Z2() const {return z2;} 
	float R() const {return r;}
	float Top() const {return top;}
	float Bottom() const {return bottom;}
	void SetRange(float cr, float ctop, float cbottom) {r = cr; top = ctop; bottom = cbottom;}
	float JR() const {return jr;}
	float JX() const {return jx;}
	float JY() const {return jy;}
	float JZ() const {return jz;}
	float JRX() const {return jrx;}
	float JRY() const {return jry;}
	float JRZ() const {return jrz;}
	void SetJaw(float cjr, float cjrx, float cjry, float cjrz) {jr = cjr; jrx = cjrx; jry = cjry; jrz = cjrz;}
private:
	float hx, hy, hz;
	float dx, dy, dz;	
	float x1, y1, z1;
	float x2, y2, z2;	
	float r, top, bottom;
	float jr, jx, jy, jz;
	float jrx, jry, jrz;
	GLDList wig;
	GLDList range;
	GLDList jaw;	 
	GLMaterial *skin;
};

class Neck : public BodyPart {
public:
	Neck();
	~Neck();
	void MakeDList(int partition = 36);
	void CalcCoord();
	void SetMaterial(GLMaterial *cmat);
private:
	GLMaterial *skin;
};

class Head {
public:
	Head();
	~Head();
	void CallLists();
	void CallListsSelect();
	void MakeDLists(int partition = 36);
	void MakeNoFace(int partition = 36);
	void MakePolygonFace(Uint8 hair_style = 1, Uint8 eyebrow_form = 1, Uint8 eye_form = 1, Uint8 nose_form = 1, Uint8 mouth_form = 1, int partition = 36);
	void CalcCoords();
	void SetMaterial(GLMaterial *cmat1, GLMaterial *cmat2);
	Face face;
	Neck neck;
private:
	GLMaterial *skin, *hair;
};

class Boxer {
public:
	Boxer(bool makedl = false);
	~Boxer();
	void CallLists(bool is_boxer_view = false);
	void CallPeople();
	void CallListsSelect();
	void CalcCoords();
	void SetRanges();
	void SetPoint(float cox, float coy, float cot) {ox = cox; oy = coy; ot = cot;} 
	void SetSPoint(float cox, float coy) {oxs = cox; oys = coy;} 
	void MoveStart();
	void Move();
	void MoveToRhythm();
	void GoNeutralCorner(Boxer &another_boxer);
	void GoRedCorner();
	void GoBlueCorner();
	void MakeDLists(int partition = 36, int face = 0, int pattern = 0);
	void MakeRanges(int partition = 36);
	void CallRanges();
	void CallHitRanges(Boxer &another_boxer);
	int IsHitRightPunch(Boxer &another_boxer);
	int IsHitLeftPunch(Boxer &another_boxer);
	int IsHitPunch(Boxer &another_boxer, int right);
	float CalcDamageRight();
	float CalcDamageLeft();
	float CalcDamage(int right);
	float CalcCounterDamage(Boxer &another_boxer);
	float CalcEnergy();
	void SetMaterial(GLMaterial cmat1, GLMaterial cmat2, GLMaterial cmat3, GLMaterial cmat4, GLMaterial cmat5);
	void SetMaterial();
	void SetDefaultFace();
	float X() const {return body.waist.RelX();}
	float Y() const {return body.waist.RelY();}
	float Z() const {return body.waist.RelZ();}
/*	float CX() const {return cx;}
	float CY() const {return cy;}  */
	float OffSetZ() const {return body.waist.RelZ() - (lh - h * 0.5);}
	float GX() const {return gx;}
	float GY() const {return gy;}
	float GZ() const {return gz;}
	float VGX() const {return vgx;}
	float VGY() const {return vgy;}
	float VGZ() const {return vgz;}
	float GXPre() const {return gx_pre;}
	float GYPre() const {return gy_pre;}
	float GZPre() const {return gz_pre;}
/*	float AGX() const {return agx;}
	float AGY() const {return agy;}
	float AGZ() const {return agz;}  */
	float WGA() const {return wga;}
	float WGB() const {return wgb;}
	float WGG() const {return wgg;}
	float GAI() const {return gai;}
	float GBI() const {return gbi;}
	float GGI() const {return ggi;}
	float OX() const {return ox;}
	float OY() const {return oy;}
	float OT() const {return ot;}
	float OXS() const {return oxs;}
	float OYS() const {return oys;}
	float Alpha() const {return body.waist.Alpha();}
	float Beta() const {return body.waist.Beta();}
	float Gamma() const {return body.waist.Gamma();}
	float AbsAlpha() const {return body.waist.AbsAlpha();}
	float AbsBeta() const {return body.waist.AbsBeta();}
	float AbsGamma() const {return body.waist.AbsGamma();}
	float Weight() const {return w;}
	float Height() const {return h;}
	float Reach() const {return r;}
	float LegHeight() const {return lh;}
	void CalcWeight();
	void CalcHeight();
	void CalcReach();	
	void CalcLegHeight();	
	void InitCenter();	
	void CalcCenter();	
	void ShowShadow();	
	float ShoulderWidth() const {return sw;}
	float WaistWidth() const {return ww;}
	void SetShoulderWidth(float csw) {sw = csw;}
	void SetWaistWidth(float cww) {ww = cww;}
	void SetPos(float cx, float cy, float cz) {body.waist.SetRelPos(cx, cy, cz);}
	void SetGPosPre(float cx, float cy, float cz) {gx_pre = cx; gy_pre = cy; gz_pre = cz;}
	void SetAngle(float ca, float cb, float cg) {body.waist.SetAngle(ca, cb, cg);}
	void SetAbsAngle(float ca, float cb, float cg) {body.waist.SetAbsAngle(ca, cb, cg);}	
	float CalcDistance(Boxer &another_boxer);
	bool CanMove();
	bool CanForward(float distance, float adjust_distance);
	int WhichPosition();

	void PrintCoords();
	void PrintAngles();
	void PrintAbsAngles();
	void Copy(Boxer *cboxer);

	Head head;
	Body body;
	Arm left_arm, right_arm;
	Leg left_leg, right_leg;

	Uint8 hair_style;
	Uint8 eyebrow;
	Uint8 eye;
	Uint8 nose;
	Uint8 mouth;
	
	GLMaterial skin, glove, bandage, trunks, shoes, hair;
private:
	float h, w, r, lh;
	float sw, ww;	
	float gx, gy, gz;
	float gx_pre, gy_pre, gz_pre;
	float vgx, vgy, vgz;
/*	float vgx_pre, vgy_pre, vgz_pre;
	float agx, agy, agz;   */
	float wga, wgb, wgg;
	float gai, gbi, ggi;
	float ox, oy, ot;
	float oxs, oys;
/*	float cx, cy;  */

	float vnx, vny, vnz;
	float vfx, vfy, vfz;
	float wna, wnb, wng;
	float wfa, wfb, wfg;
/*	float vgxs, vgys, vgzs;
	float wgas, wgbs, wggs;   */
	float rfx, rfy, rfz;

	float vgvn, vfvn;
	float wgwn, wfwn;
	float damage_center, damage_rotate;
 
 	float counter_vgvn;
	float counter_wgwn;
  	float counter_damage_center, counter_damage_rotate;

	float vg_energy, wg_energy;
};

enum SELECT_PART {
	SELECT_RIGHT_FIST = 1,
	SELECT_LEFT_FIST,
	SELECT_RIGHT_TOE,
	SELECT_LEFT_TOE,
	SELECT_WAIST,
	SELECT_HEAD
};

enum HIT_PART {
	HIT_NOTHING = 0x0000,
	HIT_BODY    = 0x0001,
	HIT_LIVER   = 0x0002,
	HIT_FACE    = 0x0004,
	HIT_JAW     = 0x0008,
	HIT_RFIST   = 0x0010,
	HIT_RFARM   = 0x0020,
	HIT_RUARM   = 0x0040,
	HIT_RSHOUL  = 0x0080,
	HIT_LFIST   = 0x0100,
	HIT_LFARM   = 0x0200,
	HIT_LUARM   = 0x0400,
	HIT_LSHOUL  = 0x0800	
};

enum WHICH_POSITION {
	CENTER_OF_RING,
	ROPE_SIDE_OF_RING,
	CORNER_OF_RING	
};

#endif // BOXER_H
