#ifndef	CAMERAPAGE_H
#define CAMERAPAGE_H

#include <wx/wx.h>
#include <wx/tglbtn.h>
#include <wx/textctrl.h>
#include <wx/slider.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>

#include "myslider.h"
#include "glcamera.h"

class myGLCanvas;
class CameraPage;
class Boxer;

class myCamera : public wxToggleButton {
public:
	myCamera(wxWindow *parent, int cnumber);
 private:
    void SetProperties();
    void DoLayout();
	GLCamera cam;
	int back_view, boxer_view;
	
	friend class CameraPage;
};

WX_DEFINE_ARRAY(myCamera*, myCameras);

class CameraPage : public wxScrolledWindow {
public:
	CameraPage(wxWindow* parent, myGLCanvas *ccanvas);
	myCamera* GetCamera()  const {return cameras[select_camera];}
	int MaxCamera() const {return max_cameras;}
	int SelectCamera() const {return select_camera;}
	void Add();		
	void Del();		
	void SetSelectCamera(int new_select);
	void ReLayout();
	float ZNear() {return znear->GetValueF(1000);}
	float ZFar() {return zfar->GetValueF(1000);}
	float Angle() {return angle->GetValue();}
	void SetCamera(GLCamera &glcamera, int i);
	void SetCamera(GLCamera &glcamera) {SetCamera(glcamera, select_camera);}
	void SetOrigin(Boxer &boxer);

	bool IsBoxerView() const {return boxer_view->GetValue();}
	bool IsBackView() const {return back_view->GetValue();}	
	float GetDistance() const {return (float)zoom_bar->GetValue()/5000.0;}

	void SaveCamera();
	void LoadCamera();

	bool Save(wxTextOutputStream &out);
	bool Load(wxTextInputStream &in);
private:
    void SetProperties();
    void DoLayout();

    void OnSetOrigin(wxCommandEvent& event);
	void OnBackView();
	void OnBoxerView();
	void OnZoomIn(wxCommandEvent& event);
    void OnZoomOut(wxCommandEvent& event);
    void OnZoomBar(wxCommandEvent& event);

	void OnAdd(wxCommandEvent &event);		
	void OnDel(wxCommandEvent &event);		
	void OnCameraName(wxCommandEvent &event);		
	void OnToggle(wxCommandEvent &event);		

	int max_cameras;
	int select_camera;
	mySlider *angle;
	mySlider *znear;
	mySlider *zfar;
	myCameras cameras;
	wxBoxSizer *tsizer;
	wxGridSizer *gsizer;
	myGLCanvas *canvas;
	wxButton *add_camera;
	wxButton *del_camera;
	wxTextCtrl *name_text;

	wxToggleButton *boxer_view;
	wxToggleButton *back_view;
	mySlider *pos_x;
	mySlider *pos_y;
	mySlider *pos_z;
	mySlider *dir_x;
	mySlider *dir_y;
	mySlider *dir_z;
	mySlider *rot;
	wxButton *set_origin;
	wxButton *zoom_in;
	wxButton *zoom_out;
	wxSlider *zoom_bar;
	DECLARE_EVENT_TABLE()
};

enum {
	ON_ADD_CAMERA_ID,
	ON_DEL_CAMERA_ID,
	ON_CAMERA_NAME_ID,
	ON_SET_ORIGIN_ID,
	ON_BACK_VIEW_ID,
	ON_BOXER_VIEW_ID,
	ON_ZOOM_IN_ID,
	ON_ZOOM_OUT_ID,
	ON_ZOOM_BAR_ID
};

#endif // CAMERAPAGE_H
