#include "combopage.h"
#include "myglcanvas.h"
#include "mynotebook.h"

BEGIN_EVENT_TABLE(myCombo, wxPanel)
	EVT_COMBOBOX(ON_MY_COMBO1_ID, myCombo::OnSelect1)
	EVT_COMBOBOX(ON_MY_COMBO2_ID, myCombo::OnSelect2)
END_EVENT_TABLE()

myCombo::myCombo(wxWindow* parent, myGLCanvas *ccanvas, int cnumber) : wxPanel(parent, -1) {
	canvas = ccanvas;
	select_first = 0;
	select_second = 0;
	wxString name;
	name.Printf("COMBO %d",cnumber);
	tbutton = new wxToggleButton(this, -1, name);			
	SetProperties();
	DoLayout();
}

void myCombo::SetProperties() {
	first = new wxComboBox(this, ON_MY_COMBO1_ID, wxT(""), wxDefaultPosition, wxSize(80, -1), 0, NULL, wxCB_READONLY);
	second = new wxComboBox(this, ON_MY_COMBO2_ID, wxT(""), wxDefaultPosition, wxSize(80, -1), 0, NULL, wxCB_READONLY);
//	first->SetSize(80, -1);
//	second->SetSize(80, -1);
}

void myCombo::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxHORIZONTAL);

	tsizer->Add(tbutton, 0, wxALIGN_CENTRE_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	tsizer->Add(first, 0, wxALIGN_CENTRE_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	tsizer->Add(second, 0, wxALIGN_CENTRE_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);

 	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

void myCombo::SetSelect1(int select) {
	first->SetSelection(select);	
	select_first = select;
}

void myCombo::SetSelect2(int select){
	second->SetSelection(select);	
	select_second = select;
}

void myCombo::OnSelect1(wxCommandEvent& event) {
	if(first->GetSelection() == -1) {
		first->SetSelection(select_first);	
	} else {
		select_first = first->GetSelection();	
	}
}

void myCombo::OnSelect2(wxCommandEvent& event) {
	if(second->GetSelection() == -1) {
		second->SetSelection(select_second);	
	} else {
		select_second = second->GetSelection();		
	}
}

BEGIN_EVENT_TABLE(ComboPage, wxScrolledWindow)
	EVT_TEXT(ON_COMBO_NAME_ID, ComboPage::OnChangeName)  
	EVT_SLIDER(ON_COMBO_SPEED_ID, ComboPage::OnSpeed)
	EVT_BUTTON(ON_ADD_COMBO_ID, ComboPage::OnAdd) 
	EVT_BUTTON(ON_DEL_COMBO_ID, ComboPage::OnDel) 
	EVT_TOGGLEBUTTON(ON_DO_COMBO_ID, ComboPage::OnDoCombo) 
	EVT_TIMER(ON_COMBO_TIMER_ID, ComboPage::OnComboTimer)
	EVT_TOGGLEBUTTON(-1, ComboPage::OnToggle)
END_EVENT_TABLE()

ComboPage::ComboPage(wxWindow* parent, myGLCanvas *ccanvas, MotionPage *cpage)
	 : wxScrolledWindow(parent, -1), combo_timer(this, ON_COMBO_TIMER_ID) {
	note = (myNotebook *) parent;
	canvas = ccanvas;
	page = cpage;
	max_combos = 1;
	select_combo = 0;

	SetProperties();
	DoLayout();
}

void ComboPage::SetProperties() {
	SetScrollRate(10,10);
	int i;
	for (i = 0; i < max_combos; i++) {
		myCombo *combo = new myCombo(this, canvas, i);
		combos.Add(combo);
		MakeComboList(i);
	}

	combos[0]->tbutton->SetValue(true);
	speed = new wxSlider(this, ON_COMBO_SPEED_ID, 20, 10, 100);
	speed->SetSize(120, 20);
	fps = new wxTextCtrl(this, -1, wxT("0"), wxDefaultPosition, wxDefaultSize, wxTE_READONLY );
	fps->SetSize(40,-1);
	comboname = new wxTextCtrl(this, ON_COMBO_NAME_ID, combos[0]->GetName());//, wxDefaultPosition, wxDefaultSize, wxTE_RICH);
	comboname->SetSize(80,-1);
	add_combo = new wxButton(this, ON_ADD_COMBO_ID, wxT("Ϣ"));
	del_combo = new wxButton(this, ON_DEL_COMBO_ID, wxT("Ϣ"));	
	do_combo = new wxToggleButton(this, ON_DO_COMBO_ID, wxT("Ϣ¹"));	
	do_combo->SetValue(false);
    tsizer = new wxBoxSizer(wxVERTICAL);
	vsizer = new wxBoxSizer(wxVERTICAL);
}

void ComboPage::DoLayout() {
	int i;
    wxBoxSizer *lsizer = new wxBoxSizer(wxHORIZONTAL);
    wxStaticBoxSizer *bsizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("Ϣ")), wxVERTICAL);
    wxStaticBoxSizer *hsizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("Ϣ")), wxHORIZONTAL);

	lsizer->Add(comboname, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	lsizer->Add(speed, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	lsizer->Add(fps, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	hsizer->Add(add_combo, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	hsizer->Add(del_combo, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	hsizer->Add(do_combo, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	for (i = 0; i < max_combos; i++) {
		vsizer->Add(combos[i],  0, wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}

	bsizer->Add(lsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	bsizer->Add(vsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	bsizer->Add(hsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
 	tsizer->Add(bsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

  	SetSizer(tsizer);
	ReLayout();
}

void ComboPage::ReLayout() {
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
	Layout();
}

void ComboPage::MakeComboList(int number) {
	int j;
	myCombo *c = combos[number];
	c->first->Clear();
	c->second->Clear();
	
	for (j = 0; j < (int)page->infos.GetCount(); j++) {
		c->first->Append(page->infos[j].name);
		c->second->Append(page->infos[j].name);
	}
	if (c->first->GetCount() > c->select_first) {
		c->first->SetSelection(c->select_first);		
	} else {
		c->select_first = 0;
		c->first->SetSelection(0);		
	}
	if (c->second->GetCount() > c->select_second) {
		c->second->SetSelection(c->select_second);		
	} else {
		c->select_second = 0;
		c->second->SetSelection(0);		
	}
}

void ComboPage::MakeComboLists() {
	int i;
	for (i = 0; i < MaxCombo(); i++) {
		MakeComboList(i);
	}  
}

void ComboPage::Add() {
	int i;
	for (i = 0; i < MaxCombo(); i++) {
		vsizer->Remove(combos[i]);	
	}
	myCombo *combo = new myCombo(this, canvas, max_combos);

	combos.Insert(combo, SelectCombo());
	GetCombo()->select_first = combos[select_combo+1]->select_first;
	GetCombo()->select_second = combos[select_combo+1]->select_second;
	MakeComboList(SelectCombo());
	select_combo++;
	max_combos++;
	for (i = 0; i < MaxCombo(); i++) {
		vsizer->Add(combos[i], 0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
}

void ComboPage::OnAdd(wxCommandEvent &event) {
	if (IsDoingCombo()) {
		return;	
	}
	Add();
	ReLayout();
}

void ComboPage::Del() {
	if (max_combos < 2) {
		return;	
	}
	max_combos--;
	myCombo *combo = GetCombo();
	vsizer->Remove(combo);
	combos.Remove(combo);
	delete combo;	
	if (select_combo >= MaxCombo() && MaxCombo()) {
		select_combo--;	
	}
	comboname->SetValue(GetCombo()->GetName());
	GetCombo()->tbutton->SetValue(true);
}

void ComboPage::OnDel(wxCommandEvent &event) {
	if (IsDoingCombo()) {
		return ;	
	}
	Del();
	ReLayout();
}

void ComboPage::OnChangeName(wxCommandEvent &event) {
	GetCombo()->SetName(comboname->GetValue());	
}


void ComboPage::SetSelectCombo(int new_select) {
	if (note->IsMoving()) {
		note->MovingOff();
	}

	combos[new_select]->tbutton->SetValue(true);

	if (new_select != SelectCombo()){
		GetCombo()->tbutton->SetValue(false);

		select_combo = new_select;
		comboname->SetValue(GetCombo()->GetName());	
		if (note->GetSelection() == 5) {
			page->infos[GetCombo()->select_first].frames[0].bframe.Set(canvas->GetBoxer());
			canvas->DoMotion();
		}
	}
	ReLayout();
}

void ComboPage::OnToggle(wxCommandEvent &event){
	int i, new_select = 0;
	wxObject *obj = event.GetEventObject();

	for (i = 0; i < max_combos; i++) {
		if(obj == (wxObject*) combos[i]->tbutton) {
			new_select = i;
			break;
		}
	}	
	SetSelectCombo(new_select);
}

void ComboPage::OnDoCombo(wxCommandEvent &event){
	if (IsDoingCombo()){
		if (note->IsMoving()) {
			note->MovingOff();
		}
		ComboOn();	
	} else {	
		TimerOff();	
	} 
}

void ComboPage::OnComboTimer(wxTimerEvent& event) {
	static BFrame bframe, *base, *next;
	static wxDateTime old_time = wxDateTime::UNow();
	static wxDateTime new_time = wxDateTime::UNow();
	static wxTimeSpan dif_time;
	static wxString str;
 
	if (current_cut >= cut_number){
		current_cut = 0;	
		current_frame++;

		new_time = wxDateTime::UNow();
		dif_time = new_time.Subtract(old_time);
		if (!dif_time.IsNull()) {
			str.Printf("%3.0f", cut_number * 1000.0/(float)dif_time.GetMilliseconds().ToLong());
			fps->SetValue(str);
		}
		old_time = new_time;
	} 
                                              
	if (current_frame >= (int)frames.GetCount()) {
		current_frame = 0;
	}

	base = &(frames[current_frame].bframe);
	cut_number = frames[current_frame].number;
	if (current_frame <  (int)frames.GetCount() - 1) {
		next = &frames[current_frame + 1].bframe;	
	} else {
		next = &frames[0].bframe;		
	}

	bframe.Interpolate(base, next, current_cut, cut_number);
	bframe.Set(canvas->GetBoxer());
	canvas->DoMotion();
	current_cut++;   
} 

void ComboPage::TimerOn() {
	int i;
	page->SaveInfo(page->SelectMotion());
	combo_timer.Start(speed->GetValue());
	current_frame = 0;
	current_cut = 0;
	frames.Empty();
	FrameInfo frame;
	MotionInfo motion;

	motion = page->infos[GetCombo()->select_first];
	for (i = 0; i < motion.max_frames; i++) {
		frame = motion.frames[i];
		frames.Add(frame);	
	}

//	wxLogError("%d", combos[0]->select_second);
	motion = page->infos[GetCombo()->select_second];
	for (i = 1; i < motion.max_frames; i++) {
		frame = motion.frames[i];	
		frames.Add(frame);	
	}
}

void ComboPage::TimerOff() {
	if (combo_timer.IsRunning()) {
		combo_timer.Stop();		
	}
}

void ComboPage::OnSpeed(wxCommandEvent& event) {
	if (IsDoingCombo()) {
		if (combo_timer.IsRunning()) {
			combo_timer.Stop();		
		}
		combo_timer.Start(speed->GetValue());
	}
}


void ComboPage::ComboOn(){
	do_combo->SetValue(true);
	TimerOn();
}

void ComboPage::ComboOff(){
	if (IsDoingCombo()) {
		do_combo->SetValue(false);
		TimerOff();	
//		page->GetFrame()->bframe.Set(canvas->GetBoxer());
//		canvas->SetMotionNoCalc();
	}
}  

bool ComboPage::Save(wxTextOutputStream &out) {
	int i;
	wxString str;
	out << wxT("#Combination Data") << endl;
	out << wxT("#Select Combination") << endl;
	out << SelectCombo() << endl;;
	out << wxT("#Max Combination") << endl;
	out << MaxCombo() << endl;;
	out << wxT("#Combination Speed") << endl;
	out << speed->GetValue() << endl;;
	for (i = 0; i < MaxCombo(); i++) {
		myCombo *com = combos[i];
		str.Printf("#Combination No.%d", i);
		out << str << endl;
		out << ("#Combination Name") << endl;
		out << com->GetName() << endl;
		out << ("#Combination Motion") << endl;
		out << com->select_first << endl;
		out << com->select_second << endl;
	}
	return true;
}
	
bool ComboPage::Load(wxTextInputStream &in) {
	int i, num, sel, max;
	wxString str;

	str = in.ReadLine();
	str = in.ReadLine();
	in >> sel;
	str = in.ReadLine();
	in >> max;
	str = in.ReadLine();
	in >> num;
	speed->SetValue(num);

	if (max > 200 || max < 1) {
		wxLogError("Ϣο۾Ǥ");
		return false;
	}

	while(MaxCombo() > max) {
		Del();	
	}
	while(MaxCombo() < max) {
		Add();	
	}
	max_combos = combos.GetCount();

	for (i = 0; i < max; i++) {
		myCombo *com = combos[i];
		str = in.ReadLine();
		str = in.ReadLine();
		str = in.ReadLine();
		com->SetName(str);
		str = in.ReadLine();
		in >> num;
		com->SetSelect1(num);
		in >> num;
		com->SetSelect2(num);
	}
	if (sel >= MaxCombo()) {
		wxLogError("Ϣǡ۾Ǥ");
		return false;
	}
	SetSelectCombo(sel);   
	ReLayout();
	return true;
}
