#include "frameslider.h"
#include "myglcanvas.h"
#include "motionpage.h"

BEGIN_EVENT_TABLE(FrameSlider, wxPanel)
	EVT_SLIDER(-1, FrameSlider::OnSlider)
	EVT_TEXT_ENTER(-1, FrameSlider::OnText)
	EVT_SPINCTRL(-1, FrameSlider::OnSpin)
END_EVENT_TABLE()

FrameSlider::FrameSlider(wxWindow* parent, int cnumber) : wxPanel(parent, -1) {
	motionpage = (MotionPage *) parent;
	wxString name;
	name.Printf("FRAME %d", cnumber);
	tbutton = new wxToggleButton(this, -1, name);			
    SetProperties();
    DoLayout();		
}

void FrameSlider::SetProperties() {
	slider = new wxSlider(this, -1, 10, 1, 50, wxDefaultPosition, wxSize(100, 20));
	slider->SetRange(1, 50);
	spin = new wxSpinCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(80, 20));	
	spin->SetRange(1, 50);
	spin->SetValue(slider->GetValue());
	tbutton->SetSize(80, 20);
//	slider->SetSize(100, 20);
//	spin->SetSize(80, 20);
}

void FrameSlider::DoLayout() {
	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add(tbutton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(slider, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(spin, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);	
  	sizer->SetSizeHints(this);
 	SetSizer(sizer);
	Layout();
}

void FrameSlider::OnSlider(wxCommandEvent &event) {
	spin->SetValue(slider->GetValue());
	motionpage->SetRange();
}

void FrameSlider::OnText(wxCommandEvent &event) {
	SetSlider();
}

void FrameSlider::OnSpin(wxSpinEvent &event) {
	SetSlider();
}

void FrameSlider::SetSlider() {
	slider->SetValue(spin->GetValue());
	motionpage->SetRange();
}

void FrameSlider::SetValue(int value) {
	spin->SetValue(value);
	slider->SetValue(value);
}


BEGIN_EVENT_TABLE(ToggleSlider, wxPanel)
	EVT_SLIDER(ON_TOGGLE_SLIDER_ID, ToggleSlider::OnSlider)
	EVT_TEXT_ENTER(ON_TOGGLE_SPINCTRL_ID, ToggleSlider::OnText)
	EVT_SPINCTRL(ON_TOGGLE_SPINCTRL_ID, ToggleSlider::OnSpin)
	EVT_SPIN_UP(ON_TOGGLE_SPINCTRL_ID, ToggleSlider::OnSpinCtrlUp)
	EVT_SPIN_DOWN(ON_TOGGLE_SPINCTRL_ID, ToggleSlider::OnSpinCtrlDown)  
	EVT_TOGGLEBUTTON(ON_TOGGLE_SLIDER_ID, ToggleSlider::OnToggle)
END_EVENT_TABLE()

ToggleSlider::ToggleSlider(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, int min_value, int max_value) : wxPanel(parent, -1) {
	canvas = ccanvas;
    SetProperties(name, min_value, max_value);
    DoLayout();		
}

void ToggleSlider::SetProperties(const wxString& name, int min_value, int max_value) {
	tbutton= new wxToggleButton(this, ON_TOGGLE_SLIDER_ID, name, wxDefaultPosition, wxSize(60, 20));
	slider = new wxSlider(this, ON_TOGGLE_SLIDER_ID, 0, 0, 255, wxDefaultPosition, wxSize(150, 20));
	slider->SetRange(min_value, max_value);
	spin = new wxSpinCtrl(this, ON_TOGGLE_SPINCTRL_ID, wxEmptyString, wxDefaultPosition, wxSize(80, 20));	
	spin->SetRange(min_value, max_value);
	spin->SetValue(slider->GetValue());
//	tbutton->SetSize(60, 20);
//	slider->SetSize(150, 20);
//	spin->SetSize(80, 20);
}

void ToggleSlider::DoLayout() {
	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add(tbutton, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(slider,  0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(spin, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5);	
  	sizer->SetSizeHints(this);
 	SetSizer(sizer);
	Layout();
}

void ToggleSlider::OnToggle(wxCommandEvent &event) {
}

void ToggleSlider::OnSlider(wxCommandEvent &event) {
	tbutton->SetValue(true);
	spin->SetValue(slider->GetValue());
	canvas->Refresh();
}

void ToggleSlider::OnText(wxCommandEvent &event) {
	SetSlider();
}

void ToggleSlider::OnSpin(wxSpinEvent &event) {
	SetSlider();
}

void ToggleSlider::SetSlider() {
	tbutton->SetValue(true);
	slider->SetValue(slider->GetValue());
	canvas->Refresh();
}

float ToggleSlider::GetValueF(int base) {
	return (float)spin->GetValue()/(float)base;
}

void ToggleSlider::SetValue(int value) {
	spin->SetValue(value);
	slider->SetValue(value);
}

void ToggleSlider::SetValueF(float value, int base) {
	SetValue((int)(value * base + 0.05 / (float)base));
}

void ToggleSlider::OnSpinCtrlUp(wxSpinEvent &event) {
	SetValue(GetValue() + 9);
}

void ToggleSlider::OnSpinCtrlDown(wxSpinEvent &event) {
	SetValue(GetValue() - 9);
}
