#include "glcamera.h"
#include "glshapes.h"

/*  */
GLCamera::GLCamera() {
	DefPosition();
	DefDirection();
	DefHeadUp();	
}

GLCamera::~GLCamera() {
}

void GLCamera::DefPosition(GLfloat x , GLfloat y , GLfloat z) {
	if (x != NO_MOVE) {px = x;}
	if (y != NO_MOVE) {py = y;}
	if (z != NO_MOVE) {pz = z;}
}

void GLCamera::Position(GLfloat x , GLfloat y , GLfloat z) {
	DefPosition(x, y, z);
	Set();
}

void GLCamera::SetPosition() {
	Set();
}

void GLCamera::DefDirection(GLfloat x , GLfloat y , GLfloat z) {
	if (x != NO_MOVE) {dx = x;}
	if (y != NO_MOVE) {dy = y;}
	if (z != NO_MOVE) {dz = z;}
}

void GLCamera::Direction(GLfloat x , GLfloat y , GLfloat z) {
	DefDirection(x, y, z);
	Set();
}

void GLCamera::SetDirection() {
	Set();
}

void GLCamera::DefHeadUp(GLfloat x , GLfloat y , GLfloat z) {
	if (x != NO_MOVE) {hx = x;}
	if (y != NO_MOVE) {hy = y;}
	if (z != NO_MOVE) {hz = z;}

	float radius, nx, ny;
	nx = dx - px;
	ny = dy - py;

	radius = hx * hx + hy * hy + hz * hz;
	if (radius != 0.0) {
		radius = sqrt(radius);
		if (nx == 0.0 && ny == 0.0) {
			rot = asin(-hy/radius);			
		} else {
			rot = acos(hz/radius);			
		}
	} else {		
		rot = 0.0;
	}		
}

void GLCamera::HeadUp(GLfloat x , GLfloat y , GLfloat z) {
	DefHeadUp(x, y, z);
	Set();
}

void GLCamera::SetHeadUp() {
	Set();
}

void GLCamera::DefRotation(GLfloat r) {
	float radius, cf, sf, nx, ny;
	nx = dx - px;
	ny = dy - py;
	rot = RAD(r);		
	if (nx == 0.0 && ny == 0.0) {
		hz = 0.0;
		hy = -cos(rot);					
		hx = -sin(rot);			
	} else {
		radius = sqrt(nx*nx + ny*ny);
		cf = -nx/radius;
		sf =  ny/radius;
				
		hz = cos(rot);
		hy = sin(rot) * cf;					
		hx = sin(rot) * sf;			
	}		
}

void GLCamera::Rotation(GLfloat r) {
	DefRotation(r);
	Set();
}

void GLCamera::SetRotation() {
	Set();
}

void GLCamera::Set() {
	gluLookAt(px, py, pz, dx, dy, dz, hx, hy, hz);
}


void GLCamera::Show() {
	GLfloat pos[3], angle;
	GLdouble absv, absv2, t, f;

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);

  	glPushMatrix();

	pos[0] = px;
	pos[1] = py;
	pos[2] = pz;
	angle = rot;

	glTranslatef(pos[0], pos[1], pos[2]);
	glScalef(0.1, 0.1, 0.1);	

	pos[0] -= dx;
	pos[1] -= dy;
	pos[2] -= dz;

	absv  = pos[0] * pos[0] + pos[1] * pos[1] + pos[2] * pos[2];
	absv2 = pos[0] * pos[0] + pos[1] * pos[1];
	if (absv > 0.0) {
		t = asin(sqrt(absv2/absv));
		if (pos[2] >= 0.0) {
			glRotatef(DEG(t), -pos[1], pos[0], 0.0);
		} else {
			glRotatef(180.0 - DEG(t), -pos[1], pos[0], 0.0);
		}		
		if (absv2) {
			f = asin(pos[1]/sqrt(absv2));
			if (pos[0] >= 0.0 && pos[1] >= 0.0) {
				glRotatef(DEG(f), 0.0, 0.0, 1.0);
   		 	}
			if (pos[0] <  0.0 && pos[1] >= 0.0) {
				glRotatef(180.0 - DEG(f), 0.0, 0.0, 1.0);
    		}
			if (pos[0] >= 0.0 && pos[1] < 0.0) {
				glRotatef(DEG(f), 0.0, 0.0, 1.0);
    		}
			if (pos[0] < 0.0  && pos[1] < 0.0) {
				glRotatef(180.0 - DEG(f), 0.0, 0.0, 1.0);
    		}
		}
	}
	glRotatef(angle, 0.0, 0.0, 1.0);
	
	camera();

	glPopMatrix();
	glPopAttrib();
}

GLCamera& GLCamera::operator=(const GLCamera &glcamera) {
	this->px = glcamera.px;
	this->py = glcamera.py;
	this->pz = glcamera.pz;
	this->dx = glcamera.dx;
	this->dy = glcamera.dy;
	this->dz = glcamera.dz;
	this->hx = glcamera.hx;
	this->hy = glcamera.hy;
	this->hz = glcamera.hz;
	this->rot = glcamera.rot;
	return *this;
} 


void GLCamera::Print() {
	fprintf(stdout, "--- Camera Properties ---\n");
	fprintf(stdout, "Position  x = %f y = %f z = %f\n", px, py, pz);
	fprintf(stdout, "Direction x = %f y = %f z = %f\n", dx, dy, dz);
	fprintf(stdout, "HeadUp    x = %f y = %f z = %f\n", hx, hy, hz);
	fprintf(stdout, "Rotation = %f(degree) %f(radian)\n", DEG(rot), rot);
}

/* 饺(ʣ) */
GLCameras::GLCameras() {
	cameras = NULL;
	DefPerspective();
}

GLCameras::GLCameras(int number) {
	DefPerspective();
	Init(number);
}

GLCameras::~GLCameras() {
	Free();
}

int GLCameras::Init(int number) {
	if (WasInit()) {
		Free();
	}
	mnumber = number;
	cameras = new GLCamera[mnumber];
	cnumber = 0;
	if (WasInit()) {
		return true;
	} else {
		return false;	
	}
}

void GLCameras::Free() {
	if (WasInit()) {
		delete [] cameras;
		cameras = NULL;
		mnumber = 0;
		cnumber = 0;	
	}
}

void GLCameras::DefPosition(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].DefPosition(x, y, z);
}

void GLCameras::Position(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].Position(x, y, z);
}

void GLCameras::SetPosition() {
	cameras[cnumber].SetPosition();
}

void GLCameras::DefPosition(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	DefPosition(x, y, z);	
}

void GLCameras::Position(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	Position(x, y, z);	
}

void GLCameras::SetPosition(int number) {
	Choice(number);
	SetPosition();
}

void GLCameras::DefDirection(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].DefDirection(x, y, z);
}

void GLCameras::Direction(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].Direction(x, y, z);
}

void GLCameras::SetDirection() {
	cameras[cnumber].SetDirection();
}

void GLCameras::DefDirection(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	DefDirection(x, y, z);	
}

void GLCameras::Direction(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	Direction(x, y, z);	
}

void GLCameras::SetDirection(int number) {
	Choice(number);
	SetDirection();
}

void GLCameras::DefHeadUp(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].DefHeadUp(x, y, z);
}

void GLCameras::HeadUp(GLfloat x, GLfloat y, GLfloat z) {
	cameras[cnumber].HeadUp(x, y, z);
}

void GLCameras::SetHeadUp() {
	cameras[cnumber].SetHeadUp();
}

void GLCameras::DefHeadUp(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	DefHeadUp(x, y, z);	
}

void GLCameras::HeadUp(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	HeadUp(x, y, z);	
}

void GLCameras::SetHeadUp(int number) {
	Choice(number);
	SetHeadUp();
}

void GLCameras::DefRotation(GLfloat r) {
	cameras[cnumber].DefRotation(r);
}

void GLCameras::Rotation(GLfloat r) {
	cameras[cnumber].Rotation(r);
}

void GLCameras::SetRotation() {
	cameras[cnumber].SetRotation();
}

void GLCameras::DefRotation(int number, GLfloat r) {
	Choice(number);
	DefRotation(r);	
}

void GLCameras::Rotation(int number, GLfloat r) {
	Choice(number);
	Rotation(r);	
}

void GLCameras::SetRotation(int number) {
	Choice(number);
	SetRotation();
}

void GLCameras::DefPerspective(GLfloat a, GLfloat zn, GLfloat zf, GLfloat w, GLfloat h) {
	angle = a;
	znear = zn;
	zfar = zf;
	width = w;
	height = h;
}

void GLCameras::Perspective(GLfloat a, GLfloat zn, GLfloat zf, GLfloat w, GLfloat h) {
	DefPerspective(a, zn, zf, w, h);
	SetPerspective();
}

void GLCameras::SetPerspective() {
	float ratio = 1.0;
	if (height > 0) {
		ratio = width / height;
	}
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(angle, ratio, znear, zfar);
    glMatrixMode(GL_MODELVIEW);
}

void GLCameras::DefOrtho(GLfloat zn, GLfloat zf, GLfloat w, GLfloat h) {
	znear = zn;
	zfar = zf;
	width = w;
	height = h;
}

void GLCameras::Ortho(GLfloat zn, GLfloat zf, GLfloat w, GLfloat h) {
	DefOrtho(zn, zf, w, h);
	SetOrtho();
}

void GLCameras::SetOrtho() {
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-width/160.0, width/160.0, -height/160.0, height/160.0, znear, zfar);
    glMatrixMode(GL_MODELVIEW);
}

void GLCameras::Set() {
	cameras[cnumber].Set();
}	
	
void GLCameras::Set(int number) {
	Choice(number);
	Set();
}

void GLCameras::Show() {
	cameras[cnumber].Show();
}

void GLCameras::Show(int number) {
	Choice(number);
	Show();
}

void GLCameras::Add(int number) {
	if (number == CURRENT_CAMERA) {
		number = mnumber;	
	}
	int i, onumber;
	onumber = mnumber;
	GLCameras buf;
	buf.Init(mnumber);
	for (i = 0; i < mnumber; i++) {
		buf[i] = cameras[i];
	}
	Free();
	Init(onumber + 1);
	for (i = 0; i < onumber; i++) {
		if (i < number) {
			cameras[i] = buf[i];
		} else {
			cameras[i+1] = buf[i];		
		}
	}
}

void GLCameras::Del(int number) {
	if (number == CURRENT_CAMERA) {
		number = mnumber;	
	}
	int i, onumber;
	onumber = mnumber;
	GLCameras buf;
	buf.Init(mnumber);
	for (i = 0; i < mnumber; i++) {
		buf[i] = cameras[i];
	}
	Free();
	Init(onumber - 1);
	for (i = 0; i < mnumber; i++) {
		if (i < number) {
			cameras[i] = buf[i];
		} else {
			cameras[i] = buf[i+1];		
		}
	}
}

GLCamera& GLCameras::operator[](int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
		return cameras[cnumber];
	} else {
		return cameras[0];
	}
}

int GLCameras::Count() {
	return mnumber;
}

void GLCameras::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

bool GLCameras::WasInit() {
	if (cameras != NULL) {
		return true;	
	} else {
		return false;	
	}
}
