#ifndef GLLIGHT_H
#define GLLIGHT_H

#include "myglheader.h"

class GLCamera;
class GLLight {
public:
	GLLight();
	~GLLight() {}
	
	void Init(int number);
	void DefDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse();

	void DefAmbient(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Ambient(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetAmbient();

	void DefSpecular(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Specular(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetSpecular();

	void DefPosition(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Position(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetPosition();

	void DefDirection(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void Direction(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void SetDirection();

	void DefAttenuation(GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void Attenuation(GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void SetAttenuation();

	void DefCutOff(GLfloat c = 0.0);
	void CutOff(GLfloat c = 0.0);
	void SetCutOff();

	void DefExponent(GLfloat e = 0.0);
	void Exponent(GLfloat e = 0.0);
	void SetExponent();

	void FlatLight();
	bool IsFlatLight();

	void SpotLight();
	bool IsSpotLight();
	
	void Set();
	void SetForCamera(GLCamera *camera);
	void Get();

	void On();
	void Off();
	bool IsOn();
	
	void Show();
	void Print();
	GLLight& operator=(const GLLight &gllightin); 
	
	bool WasInit();
private:
	GLfloat gl_dif[4];
	GLfloat gl_amb[4];
	GLfloat gl_spe[4];
	GLfloat gl_pos[4];
	GLfloat gl_dir[4];
	GLfloat gl_att[3];
	GLfloat cutoff;
	GLfloat exponent;
	GLenum gllight;
};

class GLLights {
public:
	GLLights();
	~GLLights();

 	int Init();
	void Free();
	int GetMaxLights();

	void DefDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse();

	void DefDiffuse(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse(int number);

	void DefAmbient(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Ambient(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetAmbient();

	void DefAmbient(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Ambient(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetAmbient(int number);

	void DefSpecular(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Specular(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetSpecular();

	void DefSpecular(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Specular(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetSpecular(int number);

	void DefPosition(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Position(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetPosition();

	void DefPosition(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Position(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetPosition(int number);

	void DefDirection(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void Direction(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void SetDirection();

	void DefDirection(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void Direction(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = -1.0);
	void SetDirection(int number);

	void DefAttenuation(GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void Attenuation(GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void SetAttenuation();

	void DefAttenuation(int number, GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void Attenuation(int number, GLfloat c = 1.0, GLfloat l = 0.0, GLfloat q = 0.0);
	void SetAttenuation(int number);

	void DefCutOff(GLfloat c = 0.0);
	void CutOff(GLfloat c = 0.0);
	void SetCutOff();

	void DefCutOff(int number, GLfloat c = 0.0);
	void CutOff(int number, GLfloat c = 0.0);
	void SetCutOff(int number);

	void DefExponent(GLfloat e = 0.0);
	void Exponent(GLfloat e = 0.0);
	void SetExponent();

	void DefExponent(int number, GLfloat e = 0.0);
	void Exponent(int number, GLfloat e = 0.0);
	void SetExponent(int number);

	void FlatLight();
	bool IsFlatLight();

	void FlatLight(int number);
	bool IsFlatLight(int number);

	void SpotLight();
	bool IsSpotLight();

	void SpotLight(int number);
	bool IsSpotLight(int number);
	
	void DefModelDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void ModelDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetModelDiffuse();

	void LocalViewOn();
	void LocalViewOff();

	void SetModel();

	void Set();	
	void Set(int number);

	void Get();
	void Get(int number);

	void On();
	void On(int number);
	
	void Off();
	void Off(int number);

	bool IsOn();
	bool IsOn(int number);

	void Show();
	void Show(int number);

	void Enable();
	void Disable();
	bool IsEnable();

	int Count();
	void Choice(int number);
 	GLLight& operator[](int number);
	bool WasInit();
private:
	GLLight *lights;
	int mnumber;
	int cnumber;
	GLfloat gl_dif[4];
	int localview;
}; 

#endif // GLLIGHT_H
