#ifndef GLMATERIAL_H
#define GLMATERIAL_H

#include "myglheader.h"

class GLMaterial {
public:
	GLMaterial();
	~GLMaterial();

	float DifR() const {return gl_dif[0];}
	float DifG() const {return gl_dif[1];}
	float DifB() const {return gl_dif[2];}
	float DifA() const {return gl_dif[3];}
	float AmbR() const {return gl_amb[0];}
	float AmbG() const {return gl_amb[1];}
	float AmbB() const {return gl_amb[2];}
	float AmbA() const {return gl_amb[3];}
	float SpeR() const {return gl_spe[0];}
	float SpeG() const {return gl_spe[1];}
	float SpeB() const {return gl_spe[2];}
	float SpeA() const {return gl_spe[3];}
	float EmiR() const {return gl_emi[0];}
	float EmiG() const {return gl_emi[1];}
	float EmiB() const {return gl_emi[2];}
	float EmiA() const {return gl_emi[3];}
	float Shi() const {return gl_shi;}

	void DefDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse();

	void DefAmbient(GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void Ambient(GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void SetAmbient();

	void DefSpecular(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Specular(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetSpecular();

	void DefEmission(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Emission(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetEmission();

	void DefShininess(GLfloat p = 0.0);
	void Shininess(GLfloat p = 0.0);
	void SetShininess();

	void Get();
	void Set();
	GLMaterial& operator=(const GLMaterial &glmat); 
	
	void Print();
private:
	GLfloat gl_dif[4];
	GLfloat gl_amb[4];
	GLfloat gl_spe[4];
	GLfloat gl_emi[4];
	GLfloat gl_shi;
};

class GLMaterials {
public:
	GLMaterials();
	GLMaterials(int number);
	~GLMaterials();

 	int Init(int number);
	void Free();

	void DefDiffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse();

	void DefDiffuse(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void Diffuse(int number, GLfloat r = 1.0, GLfloat g = 1.0, GLfloat b = 1.0, GLfloat a = 1.0);
	void SetDiffuse(int number);

	void DefAmbient(int number, GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void Ambient(int number, GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void SetAmbient(int number);

	void DefAmbient(GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void Ambient(GLfloat r = 0.2, GLfloat g = 0.2, GLfloat b = 0.2, GLfloat a = 1.0);
	void SetAmbient();

	void DefSpecular(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Specular(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetSpecular();

	void DefSpecular(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Specular(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetSpecular(int number);

	void DefEmission(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Emission(GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetEmission();

	void DefEmission(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void Emission(int number, GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 1.0);
	void SetEmission(int number);

	void DefShininess(GLfloat p = 0.0);
	void Shininess(GLfloat p = 0.0);
	void SetShininess();

	void DefShininess(int number, GLfloat p = 0.0);
	void Shininess(int number, GLfloat p = 0.0);
	void SetShininess(int number);

	void Get();
	void Get(int number);

	void Set();	
	void Set(int number);

	void Add(int number = 65536);
	void Del(int number = 65536);
 	GLMaterial& operator[](int number);
	
	int Count();
	void Choice(int number);
	bool WasInit();
private:
	GLMaterial *materials;
	int mnumber;
	int cnumber;
};

#endif // GLMATERIAL_H
