#include "jointpage.h"
#include "myglcanvas.h"
#include "myslider.h"
#include "boxer.h"

myJoint::myJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name) : wxPanel(parent, -1) {
	canvas = ccanvas;
	tbutton = new wxToggleButton(parent, -1, name);			
	Hide();
}

myArmJoint::myArmJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright)
	: myJoint(parent, ccanvas, name) {
	right = cright;
	SetProperties();
	DoLayout();
}

void myArmJoint::SetProperties() {
	if (right) {
		wrist_g = new mySlider(this, canvas, wxT(" : "), -180, 0);
	} else {
		wrist_g = new mySlider(this, canvas, wxT(" : "), 0, 180);
	}
	elbow_a = new mySlider(this, canvas, wxT(" : "), 0, 150);
	arm_g = new mySlider(this, canvas, wxT(" : "), -90, 90);;
	shoulder_a = new mySlider(this, canvas, wxT(" : "), 0, 360);
	if (right) {
		shoulder_b = new mySlider(this, canvas, wxT(" : "), -45, 225);
	} else {
		shoulder_b = new mySlider(this, canvas, wxT(" : "), -225, 45);	
	}
	wrist_g->SetValue(0);
	elbow_a->SetValue(0);
	arm_g->SetValue(0);
	shoulder_a->SetValue(180);
	if (right) {
		shoulder_b->SetValue(90);
	} else {
		shoulder_b->SetValue(-90);	
	}
}

void myArmJoint::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ɪ")), wxVERTICAL);
  	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer4 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
  	tsizer->Add(sizer4, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(wrist_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(elbow_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(arm_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer4->Add(shoulder_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer4->Add(shoulder_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

myLegJoint::myLegJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright)
	: myJoint(parent, ccanvas, name) {
	right = cright;
	SetProperties();
	DoLayout();
}

void myLegJoint::SetProperties() {
	toe_a = new mySlider(this, canvas, wxT(" : "), -45, 0);
	ankle_a = new mySlider(this, canvas, wxT(" : "), -45, 45);
	ankle_b = new mySlider(this, canvas, wxT(" : "), -30, 30);
	ankle_g = new mySlider(this, canvas, wxT(" : "), -30, 30);
	knee_a = new mySlider(this, canvas, wxT(" : "), 0, 150);
	hip_a = new mySlider(this, canvas, wxT(" : "), 45, 315);
	if (right) {
		hip_b = new mySlider(this, canvas, wxT(" : "), -30, 150);
	} else {
		hip_b = new mySlider(this, canvas, wxT(" : "), -150, 30);	
	}
	hip_g = new mySlider(this, canvas, wxT(" : "), 90, 270);

	toe_a->SetValue(0);
	ankle_a->SetValue(0);
	ankle_b->SetValue(0);
	ankle_g->SetValue(0);
	knee_a->SetValue(0);
	hip_a->SetValue(180);
	hip_b->SetValue(0);
	hip_g->SetValue(180);
}

void myLegJoint::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("­")), wxVERTICAL);
  	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ɨ")), wxVERTICAL);
  	wxStaticBoxSizer *sizer4 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("Դ")), wxVERTICAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer4, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(toe_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(ankle_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(ankle_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(ankle_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(knee_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer4->Add(hip_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer4->Add(hip_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer4->Add(hip_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

myBodyJoint::myBodyJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name)
	: myJoint(parent, ccanvas, name) {
	SetProperties();
	DoLayout();
}

void myBodyJoint::SetProperties() {
	abdomen_a = new mySlider(this, canvas, wxT(" : "), -90, 90);
	abdomen_b = new mySlider(this, canvas, wxT(" : "), -90, 90);
	abdomen_g = new mySlider(this, canvas, wxT(" : "), -90, 90);
	breast_a = new mySlider(this, canvas, wxT(" : "), -90, 90);
	breast_b = new mySlider(this, canvas, wxT(" : "), -90, 90);
	breast_g = new mySlider(this, canvas, wxT(" : "), -90, 90);

	abdomen_a->SetValue(0);
	abdomen_b->SetValue(0);
	abdomen_g->SetValue(0);
	breast_a->SetValue(0);
	breast_b->SetValue(0);
	breast_g->SetValue(0);
}

void myBodyJoint::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ʢ")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ʢȶ")), wxVERTICAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(abdomen_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(abdomen_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(abdomen_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(breast_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(breast_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(breast_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

myHeadJoint::myHeadJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name)
	: myJoint(parent, ccanvas, name) {
	SetProperties();
	DoLayout();
}

void myHeadJoint::SetProperties() {
	neck_a = new mySlider(this, canvas, wxT(" : "), -45, 45);
	neck_b = new mySlider(this, canvas, wxT(" : "), -45, 45);
	neck_g = new mySlider(this, canvas, wxT(" : "), -90, 90);
	face_a = new mySlider(this, canvas, wxT(" : "), -45, 45);
	face_b = new mySlider(this, canvas, wxT(" : "), -45, 45);
	face_g = new mySlider(this, canvas, wxT(" : "), -90, 90);

	neck_a->SetValue(0);
	neck_b->SetValue(0);
	neck_g->SetValue(0);
	face_a->SetValue(0);
	face_b->SetValue(0);
	face_g->SetValue(0);
}

void myHeadJoint::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
  	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);

	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(neck_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(neck_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(face_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(face_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

BEGIN_EVENT_TABLE(JointPage, wxScrolledWindow)
	EVT_TOGGLEBUTTON(-1, JointPage::OnToggle) 
END_EVENT_TABLE()

JointPage::JointPage(wxWindow* parent, myGLCanvas *ccanvas) : wxScrolledWindow(parent, -1) {
	canvas = ccanvas;
	SetProperties();
	DoLayout();
}

void JointPage::SetProperties() {
	SetScrollRate(10,10);

	myJoint *right_arm = new myArmJoint(this, canvas, wxT(""));
	myJoint *left_arm = new myArmJoint(this, canvas, wxT(""), false);
	myJoint *right_leg = new myLegJoint(this, canvas, wxT(""));
	myJoint *left_leg = new myLegJoint(this, canvas, wxT(""), false);
	myJoint *body = new myBodyJoint(this, canvas, wxT("ƹ")); 
	myJoint *head = new myHeadJoint(this, canvas, wxT("Ƭ"));

	joints.Add(right_arm);
	joints.Add(right_leg);
	joints.Add(body);
	joints.Add(left_arm);
	joints.Add(left_leg);
	joints.Add(head);
	max_joints = joints.Count();
	select_joint = 0;

	joints[0]->tbutton->SetValue(true);
	pos_x = new mySlider(this, canvas, wxT("X : "), -1000, 1000);
	pos_y = new mySlider(this, canvas, wxT("Y : "), -1000, 1000);
	pos_z = new mySlider(this, canvas, wxT("Z : "),     0, 2000);
	arg_a = new mySlider(this, canvas, wxT(" : "), -180, 180);
	arg_b = new mySlider(this, canvas, wxT(" : "), -180, 180);
	arg_g = new mySlider(this, canvas, wxT(" : "), -180, 180);
	pos_x->SetValue(0);
	pos_y->SetValue(0);
	pos_z->SetValue(0);
	arg_a->SetValue(0);
	arg_b->SetValue(0);
	arg_g->SetValue(0);    
	gsizer = new wxGridSizer(max_joints/3, 3, 0, 0);
    tsizer = new wxBoxSizer(wxVERTICAL);
}

void JointPage::DoLayout() {
	int i;
 	wxStaticBoxSizer *sizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
 	wxStaticBoxSizer *sizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
 	wxStaticBoxSizer *sizer3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
 
	tsizer->Add(sizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(sizer3, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	sizer1->Add(pos_x, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(pos_y, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer1->Add(pos_z, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer2->Add(arg_a, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(arg_b, 0, wxGROW |wxALIGN_CENTER_VERTICAL);
	sizer2->Add(arg_g, 0, wxGROW |wxALIGN_CENTER_VERTICAL);

	sizer3->Add(gsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	for (i = 0; i < max_joints; i++) {
		gsizer->Add(joints[i]->tbutton,  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}

	GetJoint()->Show();	
	tsizer->Add(GetJoint(), 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	SetSizer(tsizer);
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
    Layout();
}

void JointPage::SetSelectJoint(int new_select) {
	joints[new_select]->tbutton->SetValue(true);

	if (new_select != SelectJoint()) {
		GetJoint()->tbutton->SetValue(false);
		tsizer->Remove(GetJoint());
		GetJoint()->Hide();

		select_joint = new_select;
		GetJoint()->Show();	
		tsizer->Add(GetJoint(), 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	}
	
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
	Layout();
}

void JointPage::OnToggle(wxCommandEvent &event){
	int i, new_select = 0;
	wxObject *obj = event.GetEventObject();

	for (i = 0; i < max_joints; i++) {
		if(obj == (wxObject*) joints[i]->tbutton) {
			new_select = i;
			break;
		}
	}
	SetSelectJoint(new_select); 
}

void JointPage::GetJoint(Boxer &boxer) {
	pos_x->SetValueF(boxer.X(), 1000);
	pos_y->SetValueF(boxer.Y(), 1000);
	pos_z->SetValueF(boxer.Z(), 1000);
	arg_a->SetValue((int)boxer.Alpha());
	arg_b->SetValue((int)boxer.Beta());
	arg_g->SetValue((int)boxer.Gamma());

	myArmJoint *arm = (myArmJoint*)joints[0];
	arm->wrist_g->SetValue((int)boxer.right_arm.fist.Gamma());
	arm->elbow_a->SetValue((int)boxer.right_arm.forearm.Alpha());
	arm->arm_g->SetValue((int)boxer.right_arm.upperarm.Gamma());
	arm->shoulder_a->SetValue((int)boxer.right_arm.shoulder.Alpha());
	arm->shoulder_b->SetValue((int)boxer.right_arm.shoulder.Beta());

	arm = (myArmJoint*)joints[3];
	arm->wrist_g->SetValue((int)boxer.left_arm.fist.Gamma());
	arm->elbow_a->SetValue((int)boxer.left_arm.forearm.Alpha());
	arm->arm_g->SetValue((int)boxer.left_arm.upperarm.Gamma());
	arm->shoulder_a->SetValue((int)boxer.left_arm.shoulder.Alpha());
	arm->shoulder_b->SetValue((int)boxer.left_arm.shoulder.Beta());

	myLegJoint *leg = (myLegJoint*)joints[1];
	leg->toe_a->SetValue((int)boxer.right_leg.toe.Alpha());
	leg->ankle_a->SetValue((int)boxer.right_leg.foot.Alpha());
	leg->ankle_b->SetValue((int)boxer.right_leg.foot.Beta());
	leg->ankle_g->SetValue((int)boxer.right_leg.foot.Gamma());
	leg->knee_a->SetValue((int)boxer.right_leg.shank.Alpha());
	leg->hip_a->SetValue((int)boxer.right_leg.thigh.Alpha());
	leg->hip_b->SetValue((int)boxer.right_leg.thigh.Beta());
	leg->hip_g->SetValue((int)boxer.right_leg.thigh.Gamma());

	leg = (myLegJoint*)joints[4];
	leg->toe_a->SetValue((int)boxer.left_leg.toe.Alpha());
	leg->ankle_a->SetValue((int)boxer.left_leg.foot.Alpha());
	leg->ankle_b->SetValue((int)boxer.left_leg.foot.Beta());
	leg->ankle_g->SetValue((int)boxer.left_leg.foot.Gamma());
	leg->knee_a->SetValue((int)boxer.left_leg.shank.Alpha());
	leg->hip_a->SetValue((int)boxer.left_leg.thigh.Alpha());
	leg->hip_b->SetValue((int)boxer.left_leg.thigh.Beta());
	leg->hip_g->SetValue((int)boxer.left_leg.thigh.Gamma());

	myBodyJoint *body = (myBodyJoint*)joints[2];
	body->abdomen_a->SetValue((int)boxer.body.abdomen.Alpha());
	body->abdomen_b->SetValue((int)boxer.body.abdomen.Beta());
	body->abdomen_g->SetValue((int)boxer.body.abdomen.Gamma());
	body->breast_a->SetValue((int)boxer.body.breast.Alpha());
	body->breast_b->SetValue((int)boxer.body.breast.Beta());
	body->breast_g->SetValue((int)boxer.body.breast.Gamma());

	myHeadJoint *head = (myHeadJoint*)joints[5];
	head->neck_a->SetValue((int)boxer.head.neck.Alpha());
	head->neck_b->SetValue((int)boxer.head.neck.Beta());
	head->neck_g->SetValue((int)boxer.head.neck.Gamma());
	head->face_a->SetValue((int)boxer.head.face.Alpha());
	head->face_b->SetValue((int)boxer.head.face.Beta());
	head->face_g->SetValue((int)boxer.head.face.Gamma());
}

void JointPage::SetJoint(Boxer &boxer) {
	boxer.SetPos(
		pos_x->GetValueF(1000),
		pos_y->GetValueF(1000),
		pos_z->GetValueF(1000));
	
	boxer.SetAngle(
		(float)arg_a->GetValue(),
		(float)arg_b->GetValue(),
		(float)arg_g->GetValue());

	myArmJoint *arm = (myArmJoint*)joints[0];
	boxer.right_arm.fist.SetAngle(
		0.0, 0.0,
		(float)arm->wrist_g->GetValue());
	boxer.right_arm.forearm.SetAngle(
		(float)arm->elbow_a->GetValue(),
		0.0, 0.0);
	boxer.right_arm.upperarm.SetAngle(
		0.0, 0.0,
		(float)arm->arm_g->GetValue());		
	boxer.right_arm.shoulder.SetAngle(
		(float)arm->shoulder_a->GetValue(),
		(float)arm->shoulder_b->GetValue(),
		0.0);

	arm = (myArmJoint*)joints[3];
	boxer.left_arm.fist.SetAngle(
		0.0, 0.0,
		(float)arm->wrist_g->GetValue());
	boxer.left_arm.forearm.SetAngle(
		(float)arm->elbow_a->GetValue(),
		0.0, 0.0);
	boxer.left_arm.upperarm.SetAngle(
		0.0, 0.0,
		(float)arm->arm_g->GetValue());		
	boxer.left_arm.shoulder.SetAngle(
		(float)arm->shoulder_a->GetValue(),
		(float)arm->shoulder_b->GetValue(),
		0.0);

	myLegJoint *leg = (myLegJoint*)joints[1];
	boxer.right_leg.toe.SetAngle(
		(float)leg->toe_a->GetValue(),
		0.0, 0.0);
	boxer.right_leg.foot.SetAngle(
		(float)leg->ankle_a->GetValue(),
		(float)leg->ankle_b->GetValue(),
		(float)leg->ankle_g->GetValue());
	boxer.right_leg.shank.SetAngle(
		(float)leg->knee_a->GetValue(),
		0.0, 0.0);
	boxer.right_leg.thigh.SetAngle(
		(float)leg->hip_a->GetValue(),
		(float)leg->hip_b->GetValue(),
		(float)leg->hip_g->GetValue());
	
	leg = (myLegJoint*)joints[4];
	boxer.left_leg.toe.SetAngle(
		(float)leg->toe_a->GetValue(),
		0.0, 0.0);
	boxer.left_leg.foot.SetAngle(
		(float)leg->ankle_a->GetValue(),
		(float)leg->ankle_b->GetValue(),
		(float)leg->ankle_g->GetValue());
	boxer.left_leg.shank.SetAngle(
		(float)leg->knee_a->GetValue(),
		0.0, 0.0);
	boxer.left_leg.thigh.SetAngle(
		(float)leg->hip_a->GetValue(),
		(float)leg->hip_b->GetValue(),
		(float)leg->hip_g->GetValue());

	myBodyJoint *body = (myBodyJoint*)joints[2];
	boxer.body.abdomen.SetAngle(
		(float)body->abdomen_a->GetValue(),
		(float)body->abdomen_b->GetValue(),
		(float)body->abdomen_g->GetValue());
	boxer.body.breast.SetAngle(
		(float)body->breast_a->GetValue(),
		(float)body->breast_b->GetValue(),
		(float)body->breast_g->GetValue());

	myHeadJoint *head = (myHeadJoint*)joints[5];
	boxer.head.neck.SetAngle(
		(float)head->neck_a->GetValue(),
		(float)head->neck_b->GetValue(),
		(float)head->neck_g->GetValue());
	boxer.head.face.SetAngle(
		(float)head->face_a->GetValue(),
		(float)head->face_b->GetValue(),
		(float)head->face_g->GetValue());
}
