#ifndef	JOINTPAGE_H
#define JOINTPAGE_H

#include <wx/wx.h>
#include <wx/tglbtn.h>

class myGLCanvas;
class mySlider;
class JointPage;
class Boxer;

class myJoint : public wxPanel {
public:
	myJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
protected:
    virtual void SetProperties() = 0;
    virtual void DoLayout() = 0;
	wxToggleButton *tbutton;	
	myGLCanvas *canvas;
	friend class JointPage;
};

class myArmJoint : public myJoint {
public:
	myArmJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright = true);
private:
    void SetProperties();
    void DoLayout();
	mySlider *wrist_g;
	mySlider *elbow_a;
	mySlider *arm_g;
	mySlider *shoulder_a;
	mySlider *shoulder_b;
	bool right;
	friend class JointPage;
};

class myLegJoint : public myJoint {
public:
	myLegJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, bool cright = true);
private:
    void SetProperties();
    void DoLayout();
	mySlider *toe_a;
	mySlider *ankle_a;
	mySlider *ankle_b;
	mySlider *ankle_g;
	mySlider *knee_a;
	mySlider *hip_a;
	mySlider *hip_b;
	mySlider *hip_g;
	bool right;
	friend class JointPage;
};

class myBodyJoint : public myJoint {
public:
	myBodyJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
private:
    void SetProperties();
    void DoLayout();
	mySlider *abdomen_a;
	mySlider *abdomen_b;
	mySlider *abdomen_g;
	mySlider *breast_a;
	mySlider *breast_b;
	mySlider *breast_g;
	friend class JointPage;
};

class myHeadJoint : public myJoint {
public:
	myHeadJoint(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name);
private:
    void SetProperties();
    void DoLayout();
	mySlider *neck_a;
	mySlider *neck_b;
	mySlider *neck_g;
	mySlider *face_a;
	mySlider *face_b;
	mySlider *face_g;
	friend class JointPage;
};

WX_DEFINE_ARRAY(myJoint*, myJoints);

class JointPage : public wxScrolledWindow {
public:
	JointPage(wxWindow* parent, myGLCanvas *ccanvas);
	myJoint* GetJoint()  const {return joints[select_joint];}
	int MaxJoint() const {return max_joints;}
	int SelectJoint() const {return select_joint;}
	void SetSelectJoint(int new_select);
	void GetJoint(Boxer &boxer);
	void SetJoint(Boxer &boxer);
private:
    void SetProperties();
    void DoLayout();
	void OnToggle(wxCommandEvent &event);		

	int max_joints;
	int select_joint;
	myJoints joints;
	mySlider *pos_x;
	mySlider *pos_y;
	mySlider *pos_z;
	mySlider *arg_a;
	mySlider *arg_b;
	mySlider *arg_g;
	wxBoxSizer *tsizer;
	wxGridSizer *gsizer;
	myGLCanvas *canvas;
	DECLARE_EVENT_TABLE()
};

#endif // JOINTPAGE_H
