#ifndef	MOTIONPAGE_H
#define MOTIONPAGE_H

#include <wx/wx.h>
#include <wx/tglbtn.h>
#include <wx/slider.h>
#include <wx/textctrl.h>
#include <wx/datetime.h>
#include <wx/combobox.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include <wx/arrimpl.cpp>

#include "bframe.h"
#include "frameslider.h"

class myGLCanvas;
class myNotebook;
class MotionPage;
class ComboPage;
class AssignPage;
class RangePage;

class FrameInfo {
private:
	wxString name;
	int number;
	int rfist_fix, lfist_fix, rtoe_fix, ltoe_fix, head_fix, waist_fix;
	BFrame bframe;
	friend class MotionPage;
	friend class ComboPage;
	friend class RangePage;
};

WX_DECLARE_OBJARRAY(FrameInfo, FrameInfos);

class MotionInfo {
private:
	wxString name;
	FrameInfos frames;	
	int speed;
	int select_frame;
	int max_frames;
	friend class MotionPage;
	friend class ComboPage;
	friend class AssignPage;
	friend class RangePage;
};   

WX_DECLARE_OBJARRAY(MotionInfo, MotionInfos);

class MotionPanel : public wxPanel {
public:
	MotionPanel(wxWindow* parent, myGLCanvas *ccanvas);
	int RFistFixState();
	int LFistFixState();
	int RToeFixState();
	int LToeFixState();
	int WaistFixState();
	int HeadFixState();
private:
    void SetProperties();
    void DoLayout();
 	void OnRFistFix1(wxCommandEvent &event); 
 	void OnRFistFix2(wxCommandEvent &event); 
 	void OnRFistFix3(wxCommandEvent &event); 
 	void OnRFistFix4(wxCommandEvent &event); 
 	void OnLFistFix1(wxCommandEvent &event); 
 	void OnLFistFix2(wxCommandEvent &event); 
 	void OnLFistFix3(wxCommandEvent &event); 
 	void OnLFistFix4(wxCommandEvent &event); 
 	void OnRToeFix1(wxCommandEvent &event); 
 	void OnRToeFix2(wxCommandEvent &event); 
 	void OnRToeFix3(wxCommandEvent &event); 
 	void OnRToeFix4(wxCommandEvent &event); 
 	void OnLToeFix1(wxCommandEvent &event); 
 	void OnLToeFix2(wxCommandEvent &event); 
 	void OnLToeFix3(wxCommandEvent &event); 
 	void OnLToeFix4(wxCommandEvent &event); 
 	void OnWaistFix1(wxCommandEvent &event); 
 	void OnWaistFix2(wxCommandEvent &event); 
 	void OnWaistFix3(wxCommandEvent &event); 
 	void OnWaistFix4(wxCommandEvent &event); 
 	void OnHeadFix1(wxCommandEvent &event); 
 	void OnHeadFix2(wxCommandEvent &event); 
 	void OnHeadFix3(wxCommandEvent &event); 
 	void OnHeadFix4(wxCommandEvent &event); 
	void OnToggle(wxCommandEvent &event);		

	ToggleSlider *rfist_x;
	ToggleSlider *rfist_y;
	ToggleSlider *rfist_z;
	ToggleSlider *rfist_a;
	ToggleSlider *rfist_b;
	ToggleSlider *rfist_g;
	wxButton *rfist_fix1;
	wxButton *rfist_fix2;
	wxButton *rfist_fix3;
	wxButton *rfist_fix4;

	ToggleSlider *lfist_x;
	ToggleSlider *lfist_y;
	ToggleSlider *lfist_z;
	ToggleSlider *lfist_a;
	ToggleSlider *lfist_b;
	ToggleSlider *lfist_g;
	wxButton *lfist_fix1;
	wxButton *lfist_fix2;
	wxButton *lfist_fix3;
 	wxButton *lfist_fix4;

	ToggleSlider *rtoe_x;
	ToggleSlider *rtoe_y;
	ToggleSlider *rtoe_z;
	ToggleSlider *rtoe_a;
	ToggleSlider *rtoe_b;
	ToggleSlider *rtoe_g;
	wxButton *rtoe_fix1;
	wxButton *rtoe_fix2;
	wxButton *rtoe_fix3;
	wxButton *rtoe_fix4;

	ToggleSlider *ltoe_x;
	ToggleSlider *ltoe_y;
	ToggleSlider *ltoe_z;
	ToggleSlider *ltoe_a;
	ToggleSlider *ltoe_b;
	ToggleSlider *ltoe_g;
	wxButton *ltoe_fix1;
	wxButton *ltoe_fix2;
	wxButton *ltoe_fix3;
	wxButton *ltoe_fix4;

	ToggleSlider *head_x;
	ToggleSlider *head_y;
	ToggleSlider *head_z;
	ToggleSlider *head_a;
	ToggleSlider *head_b;
	ToggleSlider *head_g;
	wxButton *head_fix1;
	wxButton *head_fix2;
	wxButton *head_fix3;
	wxButton *head_fix4;

	ToggleSlider *waist_x;
	ToggleSlider *waist_y;
	ToggleSlider *waist_z;
	ToggleSlider *waist_a;
	ToggleSlider *waist_b;
	ToggleSlider *waist_g;
	wxButton *waist_fix1;
	wxButton *waist_fix2;
	wxButton *waist_fix3;
	wxButton *waist_fix4;

	myGLCanvas *canvas;
	friend class MotionPage;
	DECLARE_EVENT_TABLE()
};

enum {
	ON_RFIST_FIX1_ID,
	ON_RFIST_FIX2_ID,
	ON_RFIST_FIX3_ID,
	ON_RFIST_FIX4_ID,
	ON_LFIST_FIX1_ID,
	ON_LFIST_FIX2_ID,
	ON_LFIST_FIX3_ID,
	ON_LFIST_FIX4_ID,
	ON_RTOE_FIX1_ID,
	ON_RTOE_FIX2_ID,
	ON_RTOE_FIX3_ID,
	ON_RTOE_FIX4_ID,
	ON_LTOE_FIX1_ID,
	ON_LTOE_FIX2_ID,
	ON_LTOE_FIX3_ID,
	ON_LTOE_FIX4_ID,
	ON_WAIST_FIX1_ID,
	ON_WAIST_FIX2_ID,
	ON_WAIST_FIX3_ID,
	ON_WAIST_FIX4_ID,
	ON_HEAD_FIX1_ID,
	ON_HEAD_FIX2_ID,
	ON_HEAD_FIX3_ID,
	ON_HEAD_FIX4_ID
};

class MotionFrame : public FrameSlider {
public:
	MotionFrame(wxWindow* parent, myGLCanvas *ccanvas, int cnumber) : FrameSlider(parent, cnumber) {canvas = ccanvas;}
	void Get(Boxer *boxer) {bframe.Get(boxer);}
	void Set(Boxer *boxer) {bframe.Set(boxer);}
 private:
	BFrame bframe;
	int rfist_fix, lfist_fix, rtoe_fix, ltoe_fix, head_fix, waist_fix;
	myGLCanvas *canvas;
	friend class MotionPage;
	friend class ComboPage;
	friend class RangePage;
};

WX_DEFINE_ARRAY(MotionFrame*, MotionFrames);

class MotionPage : public wxScrolledWindow {
public:
	MotionPage(wxWindow* parent, myGLCanvas *ccanvas);
	MotionFrame* GetFrame() const {return frames[select_frame];}

	MotionInfo* GetMotionInfo() const {return &infos[select_motion];}
	wxString GetName() const {return name;}
    void SetName(const wxString& cname) {name = cname;}
    void Rename();
	int MaxFrame() const {return max_frames;}
	int SelectFrame() const {return select_frame;}
	void SetSelectFrame(int new_select);
	int MaxMotion() const {return max_motions;}
	int SelectMotion() const {return select_motion;}
	void ClearMotion();		
	void AddMotion();		
	void DelMotion();		
	void Add();		
	void Insert();		
	void Del();		
	void CalcOn() {is_calc =true;}
	void CalcOff() {is_calc = false;}
	bool IsCalc() const {return is_calc;}
	bool IsDoingMotion() const {return do_motion->GetValue();}
	void MotionOn();
	void MotionOff();
	void MakeComboList();
	void SetInfo(int number);
	void GetInfo(int number);
	void SaveInfo(int number);
	void SetFrameInfo(FrameInfo *finfo, int i);
	void GetFrameInfo(FrameInfo *finfo, int i);
	void SetSelection(int select) {cbox->SetSelection(select);}
	int GetSelection() const {return cbox->GetSelection();} 
	void ReLayout();
	bool IsCutting() const {return cut->GetValue();}
	void SetCut();
	void CutOff();
 	void SetRange();
	void SelectCutNumber();
	void SetCutNumber(int number);
	int GetCutNumber() const {return cut_spin->GetValue();}
	void OffSet();
	void OffSetAll();
	void Reverse();
	void ReverseAll();
	void Interpolate();
	void LagrangeInterpolate();
	void FrameAll();
 	void FrameCopy();		
 	void FramePaste();		
 	void FrameUp();		
  	void FrameDown();		
	void MotionToggle();		
	void GetCoords(Boxer &boxer);
	void SetCoords(Boxer &boxer);
	void SetPanelRightDown(int which_pick);
	void SetPanelRightDoubleClick(int which_pick);
	void SetPanelViewX(int which_pick, float objx, float objy);
	void SetPanelViewY(int which_pick, float objy, float objz);
	void SetPanelViewZ(int which_pick, float objz, float objx);
	void SetPanelViewP(int which_pick, float objx, float objy, float objz);
	bool Save(wxTextOutputStream &out);
	bool Load(wxTextInputStream &in);
	bool SaveCurrent(wxTextOutputStream &out);
private:
    void SetProperties();
    void DoLayout();
  	void OnSelect(wxCommandEvent &event);		
  	void OnChangeName(wxCommandEvent &event);		
	void OnSpeed(wxCommandEvent &event);		
	void OnCutSlider(wxCommandEvent &event);
	void OnCutSpin(wxSpinEvent &event);
	void OnCut(wxCommandEvent &event);
	void OnFrameName(wxCommandEvent &event);		
	void OnAdd(wxCommandEvent &event);		
	void OnDel(wxCommandEvent &event);		
	void OnDoMotion(wxCommandEvent &event);		
	void OnCopy(wxCommandEvent &event);		
	void OnPaste(wxCommandEvent &event);		
	void OnToggle(wxCommandEvent &event);		
 	void SetFixState(int i);
	void GetFixState(int i);	
 	void OnMotionTimer(wxTimerEvent& event); 
	void TimerOn();	
	void TimerOff();

	MotionFrames frames;
	MotionInfos infos;
	MotionPanel *mpanel;
	wxToggleButton *cut;
	wxSlider *cut_slider;
	wxSpinCtrl *cut_spin;
	int max_motions;
	int select_motion;
	wxSlider *speed;
	wxString name;
	int max_frames;
	int select_frame;
	wxTextCtrl *framename;
	wxTextCtrl *fps;
	wxComboBox *cbox;
	bool is_calc;
	wxButton *add_frame;
	wxButton *del_frame;
	wxButton *copy_frame;
	wxButton *paste_frame;
	wxToggleButton *do_motion;
	FrameInfo copy;
	BFrame bframe, *base, *next;
	int current_frame, current_cut, cut_number;
	wxTimer motion_timer;
	wxBoxSizer *tsizer, *vsizer;
	myGLCanvas *canvas;
	myNotebook *note;
	friend class ComboPage;
	friend class AssignPage;
	friend class RangePage;
	DECLARE_EVENT_TABLE()
};

enum {
 	ON_SELECT_ID,
 	ON_SPEED_ID,
	ON_MOTION_LABEL_ID,
	ON_CUT_SLIDER_ID,
	ON_FRAME_NAME_ID,
	ON_ADD_FRAME_ID,
	ON_DEL_FRAME_ID,
	ON_COPY_FRAME_ID,
	ON_PASTE_FRAME_ID,
	ON_DO_MOTION_ID,
 	ON_MOTION_TIMER_ID,
};

#endif //  MOTIONPAGE_H
