#include "myglcanvas.h"
#include "myframe.h"
#include "mynotebook.h"
#include "glshapes.h"

BEGIN_EVENT_TABLE(myGLCanvas, wxGLCanvas)
	EVT_SIZE(myGLCanvas::OnSize)
	EVT_PAINT(myGLCanvas::OnPaint)
	EVT_ERASE_BACKGROUND(myGLCanvas::OnEraseBackGround)
	EVT_ENTER_WINDOW(myGLCanvas::OnEnterWindow)
	EVT_LEAVE_WINDOW(myGLCanvas::OnLeaveWindow) 
	EVT_MOTION(myGLCanvas::OnMouseMotion) 
	EVT_LEFT_DOWN(myGLCanvas::OnMouseLeftDown) 
	EVT_LEFT_UP(myGLCanvas::OnMouseLeftUp) 
	EVT_RIGHT_DOWN(myGLCanvas::OnMouseRightDown) 
	EVT_RIGHT_DCLICK(myGLCanvas::OnMouseRightDoubleClick)
END_EVENT_TABLE()

myGLCanvas::myGLCanvas(wxWindow *parent) 
	: wxGLCanvas(parent, -1), ringDL(false), sandbagDL(false), domeDL(false), sphereDL(false) {
	a = 1.0f;
	frame = (myFrame *)parent;
	note = new myNotebook(parent, this);
	InitGL();
	SetProperties();
}

myGLCanvas::~myGLCanvas() {
}

myNotebook* myGLCanvas::GetNote() {
	return note;
}

void myGLCanvas::Render() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	SetBoxer();
	if (frame->IsOptionAnotherBoxer()) {
		another_boxer = boxer;
		note->rpage->GetBoxerFrame()->Set(&another_boxer);
		another_boxer.SetPoint(0.0, note->rpage->BoxerDistance(), 180.0);
	}

	if (frame->IsOptionLight()) {
		light[1].On();	
	} else {
		light[1].Off();		
	}

	GetPerspective();

	if (frame->IsOptionMulti()) {
		ViewX();
		ShowObjects();
		ShowBoxer();

		ViewY();
		ShowObjects();
		ShowBoxer();

		ViewZ();
		ShowObjects();
		ShowBoxer();
	}

	ViewP();
	if (frame->IsOptionLight()) {
		light[1].Show();
	}
	if (frame->IsOptionCameras()) {
		int i;
		for (i = 0; i < note->MaxCamera(); i++) {
			note->SetCamera(viewp, i);
			viewp.Show();
		}
	}
	if (frame->IsOptionShadow() && frame->IsOptionRing()) {
		ShowShadow(&boxer);
		if (frame->IsOptionAnotherBoxer()) {
			ShowShadow(&another_boxer);
		}
	}
	ShowObjects();
	ShowBoxer();

	SwapBuffers();
	GLmes();
}

void myGLCanvas::OnPaint(wxPaintEvent& event) {
	wxPaintDC dc(this);
 	Render();
}

void myGLCanvas::OnSize(wxSizeEvent& event) {
    wxGLCanvas::OnSize(event);
	GetClientSize(&w, &h);
	if (h > 0) {
		a = (float) w / (float) h;
    } else {
		a = 1.0f;	
	}
	Render();
}

void myGLCanvas::OnEraseBackGround(wxEraseEvent& event) {
  // եåɻ
}

void myGLCanvas::InitGL() {
	SetCurrent();

	glEnable(GL_DEPTH_TEST);

	glCullFace(GL_BACK);
    glFrontFace(GL_CCW);
    glEnable(GL_CULL_FACE);
	
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);

    glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	light.Init();
	light[0].DefPosition(0.0, 0.0, 1.0);
	light[0].On();	
	light[1].DefDiffuse(0.2, 0.2, 0.2);
	light[1].DefPosition(0.0, 0.0, 3.0);
	GLmesDebug();
}

void myGLCanvas::InitCamera() {
	camera_distance = 5.0;
	pcamera_distance_x = 1.2;
	pcamera_distance_y = 1.2;
	pcamera_distance_z = -0.3;

	viewx.DefPosition(boxer.X(), boxer.Y(), boxer.OffSetZ() + camera_distance);
	viewx.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
	viewx.DefRotation(90.0f);
	viewy.DefPosition(boxer.X() + camera_distance, boxer.Y(), boxer.OffSetZ());
	viewy.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ() );
	viewy.DefRotation(0.0f);
	viewz.DefPosition(boxer.X(), boxer.Y() + camera_distance, boxer.OffSetZ());
	viewz.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
	viewz.DefRotation(0.0f);
	viewp.DefPosition(boxer.X() + pcamera_distance_x, boxer.Y() + pcamera_distance_x, boxer.OffSetZ() + pcamera_distance_z);
 	viewp.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
	viewp.DefRotation(0.0f);

	SetOrigin();
}

void myGLCanvas::ChangeDistance(float distance) {
	camera_distance = distance;
}

void myGLCanvas::SetUndo() {
	undo.Copy(&boxer);
}

void myGLCanvas::Undo() {
	if (!note->IsMoving()) {
		redo.Copy(&boxer);
		SetBoxer(&undo);
		boxer.MakeDLists();
		Refresh();
	}	
}

void myGLCanvas::Redo() {
	if (!note->IsMoving()) {
		undo.Copy(&boxer);
		SetBoxer(&redo);
		boxer.MakeDLists();
		Refresh();	
	} 
}


void myGLCanvas::SetProperties() {
	ringDL.Generate();
	ringDL.NewList();
	ring();	
	ringDL.EndList();

	sandbagDL.Generate();
	sandbagDL.NewList();
	glPushMatrix();
 	sandbag();	
	glPopMatrix();
	sandbagDL.EndList();

	domeDL.Generate();
	domeDL.NewList();
	dome();
	domeDL.EndList();

	sphereDL.Generate();
	sphereDL.NewList();
	glPushMatrix();
	sphere(1.0, 36);
	glPopMatrix();
	sphereDL.EndList();

	boxer.MakeDLists();
	SetJoint();
	boxer.SetPos(0.0, 0.0, boxer.LegHeight());
	boxer.SetRanges();
	boxer.MakeRanges();

	GetCoords();
	note->motionpage->Add();
	note->motionpage->Rename();

	InitCamera();   
}

void myGLCanvas::SetPerspective() {
    glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(angle, a, znear, zfar);  
	GLmesDebug();
}

void myGLCanvas::GetPerspective() {
	znear = note->ZNear();
	zfar  = note->ZFar();
	angle = note->Angle();
}	

void myGLCanvas::ShowShadow(Boxer *shodow_boxer){
	static GLfloat m[16] = {1.0, 0.0, 0.0, 0.0,
							0.0, 1.0, 0.0, 0.0,
							0.0, 0.0, 0.0, 0.0,
							0.0, 0.0, 0.005, 1.0};
	glPushMatrix();
	glMultMatrixf(m);
	glDisable(GL_LIGHTING);
//	glDisable(GL_DEPTH_TEST);
	glColor3f(0.0, 0.0, 0.0);
	shodow_boxer->CallLists();		
//	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glPopMatrix();
}

void myGLCanvas::ShowObjects() {
	if (frame->IsOptionBackGround()) {
		domeDL.CallList();
	}
	if (frame->IsOptionRing()) {
		ringDL.CallList();
	}
	if (frame->IsOptionSandbag()) {
		glPushMatrix();
		glTranslatef(0.0, note->rpage->SandbagDistance(), 2.2);
		if (note->rpage->IsShowRange() && IsHitSandbag()) {	
			glDisable(GL_LIGHTING);
			glColor3f(1.0, 1.0, 1.0);
			sandbagDL.CallList();
			glEnable(GL_LIGHTING);				
		} else {
			sandbagDL.CallList();		
		}
		glPopMatrix();
	}
}

bool myGLCanvas::IsHitSandbag() {
	float distance = note->rpage->SandbagDistance();
	boxer.CalcCoords();
	return IsHitPillar(
		boxer.right_arm.fist.X(),
		boxer.right_arm.fist.Y(),
		boxer.right_arm.fist.Z(),
		boxer.right_arm.fist.R(),
		0.0, distance, 0.60,
		0.0, distance, 2.35, 0.25)
	 || IsHitPillar(
		boxer.left_arm.fist.X(),
		boxer.left_arm.fist.Y(),
		boxer.left_arm.fist.Z(),
		boxer.left_arm.fist.R(),
		0.0, distance, 0.60,
		0.0, distance, 2.35, 0.25);
}

void myGLCanvas::ShowBoxer() {
	if (frame->IsOptionAnotherBoxer()) {
		if (note->rpage->IsShowRange()) {
		 	another_boxer.CallHitRanges(boxer);
		} else {
		 	another_boxer.CallLists();	
		}
	}

	if (frame->IsOptionCenter()) {
		ShowCenters();	
	} else if (frame->IsOptionRange()){
	 	boxer.CallRanges();
	} else {
		if (note->cpage->IsBackView()) {
			boxer.skin.DefDiffuse(NO_CHANGE, NO_CHANGE, NO_CHANGE, 0.5);
		}	
	 	boxer.CallLists();	
		if (note->cpage->IsBackView()) {
			SetMaterial();
		}	
	}
}


void myGLCanvas::ShowCenter(float x, float y, float z, float scale) {
	scale = pow(scale * 4.0 / 3.0 / PI, 0.3333) * 0.25;
	glPushMatrix();
	glTranslatef(x, y, z);
	glScalef(scale, scale, scale);
	sphereDL.CallList();
	glPopMatrix();
}

void myGLCanvas::ShowCenters() {
	boxer.CalcCoords();
	boxer.CalcCenter();
	GLMaterial mat;
	mat.DefDiffuse(1.0, 0.0, 0.0);
	mat.Set();
	ShowCenter(boxer.GX(), boxer.GY(), boxer.GZ(), boxer.Weight());
	mat.DefDiffuse(0.0, 0.0, 1.0);
	mat.Set();
	ShowCenter(boxer.right_arm.fist.GX(), boxer.right_arm.fist.GY(), boxer.right_arm.fist.GZ(), boxer.right_arm.fist.Weight());
	ShowCenter(boxer.right_arm.forearm.GX(), boxer.right_arm.forearm.GY(), boxer.right_arm.forearm.GZ(), boxer.right_arm.forearm.Weight());
	ShowCenter(boxer.right_arm.upperarm.GX(), boxer.right_arm.upperarm.GY(), boxer.right_arm.upperarm.GZ(), boxer.right_arm.upperarm.Weight());
	ShowCenter(boxer.right_arm.shoulder.GX(), boxer.right_arm.shoulder.GY(), boxer.right_arm.shoulder.GZ(), boxer.right_arm.shoulder.Weight());
	ShowCenter(boxer.left_arm.fist.GX(), boxer.left_arm.fist.GY(), boxer.left_arm.fist.GZ(), boxer.left_arm.fist.Weight());
	ShowCenter(boxer.left_arm.forearm.GX(), boxer.left_arm.forearm.GY(), boxer.left_arm.forearm.GZ(), boxer.left_arm.forearm.Weight());
	ShowCenter(boxer.left_arm.upperarm.GX(), boxer.left_arm.upperarm.GY(), boxer.left_arm.upperarm.GZ(), boxer.left_arm.upperarm.Weight());
	ShowCenter(boxer.left_arm.shoulder.GX(), boxer.left_arm.shoulder.GY(), boxer.left_arm.shoulder.GZ(), boxer.left_arm.shoulder.Weight());
	ShowCenter(boxer.right_leg.toe.GX(), boxer.right_leg.toe.GY(), boxer.right_leg.toe.GZ(), boxer.right_leg.toe.Weight());
	ShowCenter(boxer.right_leg.foot.GX(), boxer.right_leg.foot.GY(), boxer.right_leg.foot.GZ(), boxer.right_leg.foot.Weight());
	ShowCenter(boxer.right_leg.shank.GX(), boxer.right_leg.shank.GY(), boxer.right_leg.shank.GZ(), boxer.right_leg.shank.Weight());
	ShowCenter(boxer.right_leg.thigh.GX(), boxer.right_leg.thigh.GY(), boxer.right_leg.thigh.GZ(), boxer.right_leg.thigh.Weight());
	ShowCenter(boxer.left_leg.toe.GX(), boxer.left_leg.toe.GY(), boxer.left_leg.toe.GZ(), boxer.left_leg.toe.Weight());
	ShowCenter(boxer.left_leg.foot.GX(), boxer.left_leg.foot.GY(), boxer.left_leg.foot.GZ(), boxer.left_leg.foot.Weight());
	ShowCenter(boxer.left_leg.shank.GX(), boxer.left_leg.shank.GY(), boxer.left_leg.shank.GZ(), boxer.left_leg.shank.Weight());
	ShowCenter(boxer.left_leg.thigh.GX(), boxer.left_leg.thigh.GY(), boxer.left_leg.thigh.GZ(), boxer.left_leg.thigh.Weight());
	ShowCenter(boxer.body.waist.GX(), boxer.body.waist.GY(), boxer.body.waist.GZ(), boxer.body.waist.Weight());
	ShowCenter(boxer.body.abdomen.GX(), boxer.body.abdomen.GY(), boxer.body.abdomen.GZ(), boxer.body.abdomen.Weight());
	ShowCenter(boxer.body.breast.GX(), boxer.body.breast.GY(), boxer.body.breast.GZ(), boxer.body.breast.Weight());
	ShowCenter(boxer.head.neck.GX(), boxer.head.neck.GY(), boxer.head.neck.GZ(), boxer.head.neck.Weight());
	ShowCenter(boxer.head.face.GX(), boxer.head.face.GY(), boxer.head.face.GZ(), boxer.head.face.Weight());
}

void myGLCanvas::ViewPortX() {
	glViewport(0, h/2, w/2, h/2);
}

void myGLCanvas::ProjectionX() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-a, a, -1.0, 1.0, znear, zfar);  
}

void myGLCanvas::ModelViewX() {
 	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	if (frame->IsOptionFix()) {
		viewx.DefPosition(0.0, 0.0, 1.0 + camera_distance);
		viewx.DefDirection(0.0, 0.0, 1.0);
		viewx.DefRotation(90.0f);
//		light[0].DefPosition(0.0, 0.0, 1.0 + camera_distance);
	} else {
		viewx.DefPosition(boxer.X(), boxer.Y(), boxer.OffSetZ() + camera_distance);
		viewx.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
		viewx.DefRotation(90.0f);	
//		light[0].DefPosition(boxer.X(), boxer.Y(), boxer.OffSetZ() + camera_distance);
	}
}

void myGLCanvas::ViewX() {
	ViewPortX();
	ProjectionX();
	ModelViewX();

	glDisable(GL_LIGHTING);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINES);
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glEnd();
	glEnable(GL_LIGHTING);

	light[0].Set();
	viewx.Set();
	light[1].Set();
	GLmesDebug();
}

void myGLCanvas::ViewPortY() {
    glViewport(w/2, 0, w/2, h/2);
}

void myGLCanvas::ProjectionY() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-a, a, -1.0, 1.0, znear, zfar);  
}

void myGLCanvas::ModelViewY() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	if (frame->IsOptionFix()) {
		viewy.DefPosition(camera_distance, 0.0, 1.0);
		viewy.DefDirection(0.0, 0.0, 1.0);
//		light[0].DefPosition(camera_distance, 0.0, 1.0);
	} else {
		viewy.DefPosition(boxer.X() + camera_distance, boxer.Y(), boxer.OffSetZ());
		viewy.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
//		light[0].DefPosition(boxer.X() + camera_distance, boxer.Y(), boxer.OffSetZ());
	}
}

void myGLCanvas::ViewY() {
	ViewPortY();
	ProjectionY();
	ModelViewY();

	glDisable(GL_LIGHTING);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINES);
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glEnd();
	glEnable(GL_LIGHTING);

	light[0].Set();
	viewy.Set();
	light[1].Set();
	GLmesDebug();
}

void myGLCanvas::ViewPortZ() {
   glViewport(w/2, h/2, w/2, h/2);
}

void myGLCanvas::ProjectionZ() {
 	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-a, a, -1.0, 1.0, znear, zfar);  
}

void myGLCanvas::ModelViewZ() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	if (frame->IsOptionFix()) {
		viewz.DefPosition(0.0, camera_distance, 1.0);
		viewz.DefDirection(0.0, 0.0, 1.0);
//		light[0].DefPosition(0.0, camera_distance, 1.0);
	} else {
		viewz.DefPosition(boxer.X(), boxer.Y() + camera_distance, boxer.OffSetZ());
		viewz.DefDirection(boxer.X(), boxer.Y(), boxer.OffSetZ());
//		light[0].DefPosition(boxer.X(), boxer.Y() + camera_distance, boxer.OffSetZ());
	}
}

void myGLCanvas::ViewZ() {
	ViewPortZ();
	ProjectionZ();
	ModelViewZ();

	glDisable(GL_LIGHTING);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINES);
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f( a*0.99, -1.0*0.99, -0.1);	
	glVertex3f( a*0.99,  1.0*0.99, -0.1);	
	glVertex3f(-a*0.99, -1.0*0.99, -0.1);	
	glVertex3f(-a*0.99,  1.0*0.99, -0.1);	
	glEnd();
	glEnable(GL_LIGHTING);

	light[0].Set();
	viewz.Set();
	light[1].Set();
	GLmesDebug();
}


void myGLCanvas::ViewPortP() {
	if (frame->IsOptionMulti()) {
	    glViewport(0, 0, w/2, h/2);
	} else {
	    glViewport(0, 0, w, h);	
	}
}

void myGLCanvas::ProjectionP() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (frame->IsOptionOrtho()) {
		glOrtho(-a, a, -1.0, 1.0, znear, zfar);  
	} else {
		gluPerspective(angle, a, znear, zfar);  
	}
}

void myGLCanvas::ModelViewP() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if (note->cpage->IsBoxerView() || note->cpage->IsBackView()) {
		boxer.CalcCoords();

		float px, py, pz, dx, dy, dz, hx, hy, hz;
		if (note->cpage->IsBackView()) {
			px = boxer.OX() - note->cpage->GetDistance() * sin(boxer.OT());
			py = boxer.OY() - note->cpage->GetDistance() * cos(boxer.OT());			
			pz = boxer.LegHeight();			
			dx = boxer.OX();
			dy = boxer.OY();			
			dz = boxer.LegHeight();			
			hx = 0.0;
			hy = 0.0;			
			hz = 1.0;  
		} else {
			px = boxer.head.face.X();
			py = boxer.head.face.Y();			
			pz = boxer.head.face.Z();   
			dx = boxer.head.face.DirX();
			dy = boxer.head.face.DirY();			
			dz = boxer.head.face.DirZ();   
			hx = boxer.head.face.HeadX();
			hy = boxer.head.face.HeadY();			
			hz = boxer.head.face.HeadZ();   
		}
		viewp.DefPosition(px, py, pz);
	    viewp.DefDirection(dx, dy, dz);
		viewp.DefHeadUp(hx, hy, hz);
//		light[0].DefPosition(px, py, pz);
	} else {
		note->SetCamera(viewp);
/*		light[0].DefPosition(
			camera->pos_x->GetValueF(1000), 
			camera->pos_y->GetValueF(1000), 
			camera->pos_z->GetValueF(1000)); */
	}
}

void myGLCanvas::ViewP() {
	ViewPortP();
	ProjectionP();
	ModelViewP();

	light[0].Set();
	viewp.Set();
	light[1].Set();
	GLmesDebug();
}

void myGLCanvas::SetBoxer(Boxer *cboxer) {
	boxer.Copy(cboxer);
	GetBody();
	GetJoint();
	GetMaterial();
	SetBody();
	SetJoint();
	SetMaterial();
	GetCoords();
}

void myGLCanvas::SetBoxer() {
	static int select;
	select = note->GetSelection();
	if (!note->IsMoving()) {
#ifndef CUT_PAGES
		switch (select) {
			case BODY_PAGE:
				SetBody();	
				GetJoint();
				GetCoords();		
				boxer.MakeDLists();
				note->SetLabel(boxer);
			break;
			case MATERIAL_PAGE:
				SetMaterial();			
				boxer.MakeDLists();
			break;
			case JOINT_PAGE:
				SetJoint();		
				GetCoords();		
			break;
			case MOTION_PAGE:
				SetMotion();	
			break;
			case RANGE_PAGE:
    			SetRanges();
			break;
			default:
			break;
		}				
#else
		switch (select) {
			case BODY_PAGE:
			break;
			case MATERIAL_PAGE:
			break;
			case JOINT_PAGE:
				SetJoint();		
				GetCoords();		
			break;
			case MOTION_PAGE:
				SetMotion();	
			break;
			case RANGE_PAGE:
    			SetRanges();
			break;
			default:
			break;
		}				
#endif
	}
}	  

void myGLCanvas::SetBody() {
	note->SetBody(boxer);
}

void myGLCanvas::SetMaterial() {
	note->SetMaterial(boxer);
}

void myGLCanvas::SetJoint() {
	note->SetJoint(boxer);
}

void myGLCanvas::SetOrigin() {
	note->SetOrigin(boxer);
	Refresh();
}

void myGLCanvas::GetCoords() {
	boxer.CalcCoords();
	note->GetCoords(boxer);
	note->GetFrame()->Get(&boxer);
}

void myGLCanvas::GetRanges() {
	boxer.SetRanges();
	note->rpage->GetRange(boxer);
}

void myGLCanvas::GetBody() {
	note->GetBody(boxer);
}

void myGLCanvas::GetMaterial() {
	note->GetMaterial(boxer);
}

void myGLCanvas::GetJoint() {
	note->GetJoint(boxer);
}

void myGLCanvas::SetMotion() {
	if (note->IsCalc()) {
		SetMotionCalc();
	}
}

void myGLCanvas::SetRanges() {
	note->rpage->SetRange(boxer);
	boxer.MakeRanges();
}

void myGLCanvas::SetMotionCalc() {
	note->SetCoords(boxer);
	GetJoint();
	GetCoords();
}

void myGLCanvas::SetMotionNoCalc() {
	if (note->IsCalc()) {
		note->CalcOff();
		GetJoint();
		GetCoords();
		Render();	
		note->CalcOn();
	}
}

void myGLCanvas::DoMotion() {
	Render();	
}

void myGLCanvas::OnEnterWindow(wxMouseEvent& event) {
	SetFocus();
}

void myGLCanvas::OnLeaveWindow(wxMouseEvent& event) {
	left_down = 0;	
}

void myGLCanvas::OnMouseMotion(wxMouseEvent& event){
	static int new_mouse_x, new_mouse_y, dx, dy, distance;
	new_mouse_x = event.GetX();
	new_mouse_y = event.GetY();
	dx = mouse_x - new_mouse_x;
	dy = mouse_y - new_mouse_y;
	distance = dx * dx + dy * dy;
	
	if (distance > 0 && left_down) {
		switch (which_view)  {
			case 1:
				MoveViewX(mouse_x, mouse_y);
			break;
			case 2:
				MoveViewY(mouse_x, mouse_y);
			break;
			case 3:
				MoveViewZ(mouse_x, mouse_y);
			break;
			case 4:
				MoveViewP(mouse_x, mouse_y);
			break;
		}
	} 	
	mouse_x = new_mouse_x;
	mouse_y = new_mouse_y;
}

void myGLCanvas::OnMouseLeftDown(wxMouseEvent& event){
	mouse_x = event.GetX();
	mouse_y = event.GetY();
	left_down = 1;

	GetPerspective();
	if (frame->IsOptionMulti()) {
		if (mouse_x < w/2 && mouse_y < h/2) {
			which_pick = PickViewX(mouse_x, mouse_y);
			which_view = 1;
		}
		if (mouse_x >= w/2 && mouse_y >= h/2) {
			which_pick = PickViewY(mouse_x, mouse_y);
			which_view = 2;
		}
		if (mouse_x >= w/2 && mouse_y < h/2) {
			which_pick = PickViewZ(mouse_x, mouse_y);
			which_view = 3;
		}
		if (mouse_x < w/2 && mouse_y >= h/2) {
			which_pick = PickViewP(mouse_x, mouse_y);		
			which_view = 4;
		}
	} else {
		which_pick = PickViewP(mouse_x, mouse_y);
		which_view = 4;
	}
}

void myGLCanvas::OnMouseLeftUp(wxMouseEvent& event) {
	left_down = 0;
	which_pick = 0;
	which_view = 0;
}

void myGLCanvas::OnMouseRightDown(wxMouseEvent& event){
	mouse_x = event.GetX();
	mouse_y = event.GetY();
	if (frame->IsOptionMulti()) {
		if (mouse_x < w/2 && mouse_y < h/2) {
			which_pick = PickViewX(mouse_x, mouse_y);
		}
		if (mouse_x >= w/2 && mouse_y >= h/2) {
			which_pick = PickViewY(mouse_x, mouse_y);
		}
		if (mouse_x >= w/2 && mouse_y < h/2) {
			which_pick = PickViewZ(mouse_x, mouse_y);
		}
		if (mouse_x < w/2 && mouse_y >= h/2) {
			which_pick = PickViewP(mouse_x, mouse_y);		
		}
	} else {
		which_pick = PickViewP(mouse_x, mouse_y);
	}

	note->motionpage->SetPanelRightDown(which_pick);
}

void myGLCanvas::OnMouseRightDoubleClick(wxMouseEvent& event){
	mouse_x = event.GetX();
	mouse_y = event.GetY();
	if (frame->IsOptionMulti()) {
		if (mouse_x < w/2 && mouse_y < h/2) {
			which_pick = PickViewX(mouse_x, mouse_y);
		}
		if (mouse_x >= w/2 && mouse_y >= h/2) {
			which_pick = PickViewY(mouse_x, mouse_y);
		}
		if (mouse_x >= w/2 && mouse_y < h/2) {
			which_pick = PickViewZ(mouse_x, mouse_y);
		}
		if (mouse_x < w/2 && mouse_y >= h/2) {
			which_pick = PickViewP(mouse_x, mouse_y);		
		}
	} else {
		which_pick = PickViewP(mouse_x, mouse_y);
	}

	note->motionpage->SetPanelRightDoubleClick(which_pick);
}

void myGLCanvas::SetPick() {
	glSelectBuffer(SELECT_BUFFER_SIZE, select_buf);
	glRenderMode(GL_SELECT);
	glInitNames();
	glPushName(0);
	SetBoxer();
}

int myGLCanvas::PickViewX(int x, int y) {
	SetPick();
	ViewPortX();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPickMatrix((GLdouble) x, (GLdouble)(h - y), 3.0, 3.0, viewport);
	glOrtho(-a, a, -1.0, 1.0, 0.1, 10.0);  

	ModelViewX();
	viewx.Set();
	boxer.CallListsSelect();

	glFlush();
	
	hits = glRenderMode(GL_RENDER);
	GLmesDebug(); 
    if (hits > 0) {
		return WhichHit();	
	} else {
		return 0;
	}
}

int myGLCanvas::PickViewY(int x, int y) {
	SetPick();
	ViewPortY();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPickMatrix((GLdouble) x, (GLdouble)(h - y), 3.0, 3.0, viewport);
	glOrtho(-a, a, -1.0, 1.0, 0.1, 10.0);  

	ModelViewY();
	viewy.Set();
	boxer.CallListsSelect();
	glFlush();

	hits = glRenderMode(GL_RENDER);
	GLmesDebug(); 
    if (hits > 0) {
		return WhichHit();	
	} else {
		return 0;	
	}
}

int myGLCanvas::PickViewZ(int x, int y) {
	SetPick();
	ViewPortZ();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPickMatrix((GLdouble) x, (GLdouble)(h - y), 3.0, 3.0, viewport);
	glOrtho(-a, a, -1.0, 1.0, 0.1, 10.0);  

	ModelViewZ();
	viewz.Set();
	boxer.CallListsSelect();

	ProjectionZ();
	ModelViewZ();

	glFlush();

	hits = glRenderMode(GL_RENDER);
	GLmesDebug(); 
    if (hits > 0) {
		return WhichHit();	
	} else {
		return 0;
	}
}

int myGLCanvas::PickViewP(int x, int y) {
	SetPick();
	ViewPortP();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPickMatrix((GLdouble) x, (GLdouble)(h - y), 3.0, 3.0, viewport);

	if (frame->IsOptionOrtho()) {
		glOrtho(-a, a, -1.0, 1.0, znear, zfar);  
	} else {
		gluPerspective(angle, a, znear, zfar);  
	}

	ModelViewP();
	viewp.Set();
	boxer.CallListsSelect();
	glFlush();

	hits = glRenderMode(GL_RENDER);
	GLmesDebug(); 
    if (hits > 0) {
		return WhichHit();	
	} else {
		return 0;
	}
}

void myGLCanvas::MoveViewX(int x, int y) {
	ViewPortX();
	glGetIntegerv(GL_VIEWPORT, viewport);

	ProjectionX();
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	ModelViewX();
	viewx.Set();
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	gluUnProject((GLdouble)x, (GLdouble)(h - y), 0.0, modelview, projection, viewport, &objx, &objy, &objz);						

	note->motionpage->SetPanelViewX(which_pick, objx, objy);
	note->SetSelection(4);
	Refresh();
}

void myGLCanvas::MoveViewY(int x, int y) {
	ViewPortY();
	glGetIntegerv(GL_VIEWPORT, viewport);

	ProjectionY();
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	ModelViewY();
	viewy.Set();
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	gluUnProject((GLdouble)x, (GLdouble)(h - y), 0.0, modelview, projection, viewport, &objx, &objy, &objz);						

	note->motionpage->SetPanelViewY(which_pick, objy, objz);
	note->SetSelection(4);
	Refresh();
}

void myGLCanvas::MoveViewZ(int x, int y) {
	ViewPortZ();
	glGetIntegerv(GL_VIEWPORT, viewport);

	ProjectionZ();
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	ModelViewZ();
	viewz.Set();
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	gluUnProject((GLdouble)x, (GLdouble)(h - y), 0.0, modelview, projection, viewport, &objx, &objy, &objz);						

	note->motionpage->SetPanelViewZ(which_pick, objz, objx);
	note->SetSelection(4);
	Refresh();
}

void myGLCanvas::MoveViewP(int x, int y) {
//	glDepthRange(0.0, 1.0);
	ViewPortP();
	glGetIntegerv(GL_VIEWPORT, viewport);

	ProjectionP();
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	ModelViewP();
	viewp.Set();
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	gluUnProject((GLdouble)x, (GLdouble)(h - y), (GLfloat)select_buf[1 + (hits - 1)*4]/0xffffffff, modelview, projection, viewport, &objx, &objy, &objz);						

//	wxLogError("%f %f %f %f", (GLfloat)select_buf[1]/0xffffffff, objx, objy, objz);
	note->motionpage->SetPanelViewP(which_pick, objx, objy, objz);
	note->SetSelection(4);
	Refresh();
}

int myGLCanvas::WhichHitTest() {
	int i, j;
	GLuint names, *ptr = select_buf;

	wxLogError("Hits = %d\n", hits);
  	for (i = 0; i < hits; i++) {
		names = *ptr;
		wxLogError("Names = %d\n", names);
		ptr++;
		wxLogError("z1 = %f", (float)*ptr/0xffffffff);
		ptr++;                  
		wxLogError("z2 = %f", (float)*ptr/0xffffffff);
		ptr++;
		for (j = 0; j < names; j++) {
			wxLogError("name = %d" , *ptr);
			ptr++;
		}
	}
	return select_buf[3 + (hits - 1)*4];
}
