#ifndef MYGLCANVAS_H
#define MYGLCANVAS_H

#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <GL/glu.h>

#include "myglheader.h"
#include "boxer.h"
#include "bframe.h"
#include "gllight.h"
#include "glcamera.h"
#include "glmaterial.h"

class myFrame;
class myNotebook;

class myGLCanvas: public wxGLCanvas {
public:
	myGLCanvas(wxWindow *parent);
	~myGLCanvas(void);
	myNotebook* GetNote();
	Boxer* GetBoxer() {return &boxer;}
	void GetBody();	
	void GetMaterial();
	void GetJoint();	
	void GetCoords();	
    void GetRanges();
	void SetBoxer(Boxer *cboxer);
	void SetBoxer();	
	void SetBody();	
	void SetMaterial();	
	void SetJoint();	
    void SetMotionCalc();
    void SetMotionNoCalc();
    void SetMotion();
    void SetRanges();
	void DoMotion();
    void SetOrigin();
	void InitCamera();
    void ChangeDistance(float distance);
	void SetUndo();
	void Undo();
	void Redo();
private:
	void SetProperties();	
	void InitGL();
	void Render();
	void ShowCenter(float x, float y, float z, float scale);
	void ShowCenters();
	void ShowObjects();
	void ShowShadow(Boxer* shadow_boxer);
	void ShowBoxer();
	void SetPerspective();	
	void GetPerspective();
	bool IsHitSandbag();	
	void ViewX();	
	void ViewY();	
	void ViewZ();	
	void ViewP();	
	void ViewPortX();	
	void ViewPortY();	
	void ViewPortZ();	
	void ViewPortP();	
	void ProjectionX();	
	void ProjectionY();	
	void ProjectionZ();	
	void ProjectionP();	
	void ModelViewX();	
	void ModelViewY();	
	void ModelViewZ();	
	void ModelViewP();	
	void OnPaint(wxPaintEvent& event);
 	void OnSize(wxSizeEvent& event);
 	void OnEraseBackGround(wxEraseEvent& event);
 	void OnEnterWindow(wxMouseEvent& event); 
 	void OnLeaveWindow(wxMouseEvent& event); 
	void OnMouseMotion(wxMouseEvent& event);
	void OnMouseLeftDown(wxMouseEvent& event);
	void OnMouseLeftUp(wxMouseEvent& event);
	void OnMouseRightDown(wxMouseEvent& event);
	void OnMouseRightDoubleClick(wxMouseEvent& event);
	void SetPick();
	int PickViewX(int x, int y);
	int PickViewY(int x, int y);
	int PickViewZ(int x, int y);
	int PickViewP(int x, int y);
	int WhichHit() const {return select_buf[3 + (hits - 1)*4];}
	int WhichHitTest();
	void MoveViewX(int x, int y);
	void MoveViewY(int x, int y);
	void MoveViewZ(int x, int y);
	void MoveViewP(int x, int y);

	myFrame *frame;
	myNotebook *note;
	Boxer boxer, undo, redo, another_boxer;
	GLDList ringDL, sandbagDL, domeDL, sphereDL;
 	GLLights light;
	GLCamera viewp, viewx, viewy, viewz;
	enum {
		SELECT_BUFFER_SIZE = 128	
	};
	int which_view, which_pick, mouse_x, mouse_y, left_down;
	GLint w, h, hits, viewport[4];
	GLuint select_buf[SELECT_BUFFER_SIZE];
	GLfloat a, angle, zfar, znear;
	GLfloat camera_distance, pcamera_distance_x, pcamera_distance_y, pcamera_distance_z;
	GLdouble objx, objy, objz;
	GLdouble projection[16], modelview[16];	
	DECLARE_EVENT_TABLE()
};

#endif // MYGLCANVAS_H
