#include "mynotebook.h"
#include "myglcanvas.h"
#include "boxer.h"

BEGIN_EVENT_TABLE(myNotebook, wxNotebook)
	EVT_NOTEBOOK_PAGE_CHANGED(ON_NOTEBOOK_ID, myNotebook::OnSelChange) 
END_EVENT_TABLE()  

myNotebook::myNotebook(wxWindow *parent, myGLCanvas *ccanvas) : wxNotebook(parent, ON_NOTEBOOK_ID) {
	canvas = ccanvas;
	SetProperties();
	DoLayout();		   
}

void myNotebook::SetProperties() {
#ifndef CUT_PAGES
	bpage = new BodyPage(this, canvas);
	mpage = new MatPage(this, canvas);
#else
	page[0] = new myPage(this);
	page[1] = new myPage(this);
	page[2] = new myPage(this);
#endif
	cpage = new CameraPage(this, canvas);
	jpage = new JointPage(this, canvas);
	motionpage = new MotionPage(this, canvas);
	combopage = new ComboPage(this, canvas, motionpage);
	apage = new AssignPage(this, motionpage);
	rpage = new RangePage(this, canvas, motionpage);
	tpage = new ThinkPage(this, apage);
	opage = new OtherPage(this);
}		

void myNotebook::DoLayout() {
#ifndef CUT_PAGES
	AddPage(bpage, "体格");
	AddPage(mpage, "材質");
#else
	AddPage(page[0], "ダミー");
	AddPage(page[1], "ダミー");
#endif
	AddPage(cpage, "カメラ");
	AddPage(jpage, "関節");
	AddPage(motionpage, "動き");
	AddPage(combopage, "連結");
	AddPage(apage, "割当");
	AddPage(rpage, "接触");
	AddPage(tpage, "思考");
	AddPage(opage, "その他");
}		

void myNotebook::OnSelChange(wxNotebookEvent& event) {
#ifdef	 __WXMSW__	
	wxNotebook::OnSelChange(event);
#endif
	int select = event.GetSelection();
	if (!IsMoving()) {
		GetFrame()->Set(canvas->GetBoxer());
		if (select == MOTION_PAGE) {
			canvas->SetMotionNoCalc();
		} else {
			canvas->Refresh();		
		}
	} else {
		canvas->Refresh();		
	}
	if (select == COMBO_PAGE) {
		combopage->MakeComboLists();
    }
	if (select == ASSIGN_PAGE) {
		apage->MakeComboLists();
    }
	if (select == THINK_PAGE) {
		tpage->MakeComboLists();
    }
	canvas->SetUndo();
}

void myNotebook::Clear() {
	MovingOff();
	Boxer new_boxer;
	canvas->SetBoxer(&new_boxer);
	canvas->GetBoxer()->MakeDLists();
	canvas->GetBoxer()->SetPos(0.0, 0.0, canvas->GetBoxer()->LegHeight());
	canvas->GetRanges();		
	canvas->SetRanges();

	motionpage->ClearMotion();
	motionpage->SetInfo(0);
	motionpage->SetSelectFrame(0);
	while (motionpage->MaxFrame() > 2) {
		motionpage->Del();	
	}
	motionpage->SetSelectFrame(1);
//	motionpage->MakeComboList();
//	motionpage->SetSelection(0);
	motionpage->Rename();
	motionpage->ReLayout();
	MakeComboLists();

	canvas->InitCamera();
	SetSelection(BODY_PAGE);
	canvas->Refresh();
}
