#ifndef MYNOTEBOOK_H
#define MYNOTEBOOK_H

#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>

#include "myframe.h"
#include "mypage.h"
#include "bodypage.h"
#include "jointpage.h"
#include "camerapage.h"
#include "matpage.h"
#include "motionpage.h"
#include "combopage.h"
#include "assignpage.h"
#include "rangepage.h"
#include "thinkpage.h"
#include "otherpage.h"

class myGLCanvas;

enum {
	BODY_PAGE = 0,
	MATERIAL_PAGE,
	CAMERA_PAGE,
	JOINT_PAGE,
	MOTION_PAGE,
	COMBO_PAGE,
	ASSIGN_PAGE,
	RANGE_PAGE,
	THINK_PAGE,
	OTHER_PAGE
};

class myNotebook : public wxNotebook {
public:
    myNotebook(wxWindow* parent, myGLCanvas* ccanvas);
	void Clear();

	void SetBody(Boxer& boxer) {bpage->SetBody(boxer);}
	void GetBody(Boxer& boxer) {bpage->GetBody(boxer);}
	void SetLabel(Boxer& boxer) {bpage->SetLabel(boxer);}
	bool SaveBody(wxTextOutputStream &out) {return bpage->Save(out);}
	bool LoadBody(wxTextInputStream &in) {return bpage->Load(in);}

	void SetMaterial(Boxer& boxer) {mpage->SetMaterial(boxer);}
	void GetMaterial(Boxer& boxer) {mpage->GetMaterial(boxer);}
	bool SaveMaterial(wxTextOutputStream &out) {return mpage->Save(out);}
	bool LoadMaterial(wxTextInputStream &in) {return mpage->Load(in);}

	int MaxCamera() {return cpage->MaxCamera();}
	myCamera* GetCamera() {return cpage->GetCamera();}
	void SetCamera(GLCamera &glcamera, int i) {cpage->SetCamera(glcamera, i);}
	void SetCamera(GLCamera &glcamera) {cpage->SetCamera(glcamera);}
	void SetOrigin(Boxer &boxer) {cpage->SetOrigin(boxer);}
	float ZNear() {return cpage->ZNear();}
	float ZFar() {return cpage->ZFar();}
	float Angle() {return cpage->Angle();}
	bool SaveCamera(wxTextOutputStream &out) {return cpage->Save(out);}
	bool LoadCamera(wxTextInputStream &in) {return cpage->Load(in);}

	void SetJoint(Boxer& boxer) {jpage->SetJoint(boxer);}
	void GetJoint(Boxer& boxer) {jpage->GetJoint(boxer);}

	MotionInfo *GetMotionInfo() const {return motionpage->GetMotionInfo();}
	MotionFrame *GetFrame() const {return motionpage->GetFrame();}
	void MakeComboLists() {combopage->MakeComboLists(); apage->MakeComboLists(); rpage->MakeComboList(); tpage->MakeComboLists();}
	bool IsMoving() const {return motionpage->IsDoingMotion() || motionpage->IsCutting() ||combopage->IsDoingCombo();}	
	void MotionOn() {motionpage->MotionOn();}
	void MovingOff() {motionpage->MotionOff(); motionpage->CutOff(); combopage->ComboOff();}
	bool IsCalc() const {return motionpage->IsCalc();}	
	void CalcOn() {motionpage->CalcOn();}	
	void CalcOff() {motionpage->CalcOff();}	
	void GetCoords(Boxer& boxer) {motionpage->GetCoords(boxer);}
	void SetCoords(Boxer& boxer) {motionpage->SetCoords(boxer);}
	bool SaveMotion(wxTextOutputStream &out) {return motionpage->Save(out);}
	bool LoadMotion(wxTextInputStream &in) {return motionpage->Load(in);}
	bool SaveCurrentMotion(wxTextOutputStream &out) {return motionpage->SaveCurrent(out);}
	void FrameUp() {motionpage->FrameUp();}
	void FrameDown() {motionpage->FrameDown();}
	void FrameCopy() {motionpage->FrameCopy();}
	void FramePaste() {motionpage->FramePaste();}
	void MotionToggle() {motionpage->MotionToggle();}

	void ComboOn() {combopage->ComboOn();}
	bool SaveCombo(wxTextOutputStream &out) {return combopage->Save(out);}
	bool LoadCombo(wxTextInputStream &in) {return combopage->Load(in);}

	bool SaveAssign(wxTextOutputStream &out) {return apage->Save(out);}
	bool LoadAssign(wxTextInputStream &in) {return apage->Load(in);}
//	bool LoadAssignOld(wxTextInputStream &in) {return apage->LoadOld(in);}

	bool SaveRange(wxTextOutputStream &out) {return rpage->Save(out);}
	bool LoadRange(wxTextInputStream &in) {return rpage->Load(in);}

	bool SaveThink(wxTextOutputStream &out) {return tpage->Save(out);}
	bool LoadThink(wxTextInputStream &in) {return tpage->Load(in);}
	void ThinkPre() {tpage->ThinkPre();}
	void ThinkNext() {tpage->ThinkNext();}

	bool SaveOther(wxTextOutputStream &out) {return opage->Save(out);}
	bool LoadOther(wxTextInputStream &in) {return opage->Load(in);}

private:
    void SetProperties();
    void DoLayout();
	void OnSelChange(wxNotebookEvent& event);

	BodyPage *bpage;
	MatPage *mpage;
	CameraPage *cpage;
	JointPage *jpage;
	MotionPage *motionpage;
	ComboPage *combopage;
	AssignPage *apage;
	RangePage *rpage;
	ThinkPage *tpage;
	OtherPage *opage;
	myGLCanvas *canvas;
#ifdef CUT_PAGES
	myPage *page[4];
#endif
	friend class myFrame;
	friend class myGLCanvas;
	DECLARE_EVENT_TABLE()
};

enum {
	ON_NOTEBOOK_ID
};

#endif // MYFNOTEBOOK_H
