#include "myslider.h"
#include "myglcanvas.h"

BEGIN_EVENT_TABLE(mySlider, wxPanel)
	EVT_SLIDER(ON_MY_SLIDER_ID, mySlider::OnSlider)
	EVT_TEXT_ENTER(ON_MY_SPINCTRL_ID, mySlider::OnText)
	EVT_SPINCTRL(ON_MY_SPINCTRL_ID, mySlider::OnSpin)
	EVT_SPIN_UP(ON_MY_SPINCTRL_ID, mySlider::OnSpinCtrlUp)
	EVT_SPIN_DOWN(ON_MY_SPINCTRL_ID, mySlider::OnSpinCtrlDown)
END_EVENT_TABLE()

mySlider::mySlider(wxWindow* parent, myGLCanvas *ccanvas, const wxString& name, int min_value, int max_value) : wxPanel(parent, -1) {
	canvas = ccanvas;
    SetProperties(name, min_value, max_value);
    DoLayout();		
}

void mySlider::SetProperties(const wxString& name, int min_value, int max_value) {
	int osv, mav, miv;
	osv = wxGetOsVersion(&mav, &miv);
#if wxCHECK_VERSION(2, 6, 0)
	label= new wxStaticText(this, ON_MY_SLIDER_ID, name, wxDefaultPosition, wxSize(60, 20), wxALIGN_CENTRE);
	label->SetWindowStyle(wxSIMPLE_BORDER);
	label->SetFont(wxFont(10, wxDEFAULT, wxNORMAL, wxNORMAL));
#else
	label= new wxStaticText(this, ON_MY_SLIDER_ID, name, wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	label->SetWindowStyle(wxSIMPLE_BORDER);
	if (osv != wxWIN386 && osv != wxWIN95 && osv != wxWIN32S) {
		label->SetFont(wxFont(10, wxDEFAULT, wxNORMAL, wxNORMAL));
	}
	label->SetSize(60, 20);
#endif
	slider = new wxSlider(this, ON_MY_SLIDER_ID, 0, 0, 255, wxDefaultPosition, wxSize(150, -1));
	slider->SetRange(min_value, max_value);
	spin = new wxSpinCtrl(this, ON_MY_SPINCTRL_ID, wxEmptyString, wxDefaultPosition, wxSize(80, -1));	
	spin->SetRange(min_value, max_value);
	spin->SetValue(slider->GetValue());
//	slider->SetSize(150, 20);
//	spin->SetSize(80, 20);
}

void mySlider::DoLayout() {
	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add(label, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(slider, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 5);
	sizer->Add(spin, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 5);	
  	sizer->SetSizeHints(this);
 	SetSizer(sizer);
	Layout();
}

void mySlider::OnSlider(wxCommandEvent &event) {
	spin->SetValue(slider->GetValue());
	canvas->Refresh();
}

void mySlider::OnText(wxCommandEvent &event) {
	SetSlider();
}

void mySlider::OnSpin(wxSpinEvent &event) {
	SetSlider();
}

void mySlider::SetSlider() {
	slider->SetValue(spin->GetValue());
	canvas->Refresh();
}

float mySlider::GetValueF(int base) {
	return (float)spin->GetValue()/(float)base;
}

void mySlider::SetValue(int value) {
	spin->SetValue(value);
	slider->SetValue(value);
}

void mySlider::SetValueF(float value, int base) {
	SetValue((int)(value * base + 0.05 / (float)base));
}

void mySlider::ValueUp(int value) {
	SetValue((int)((float)GetValue() * (float)(100 + value) / 100.0)+1);
}

void mySlider::ValueDown(int value) {
	SetValue((int)((float)GetValue() * (float)(100 - value) / 100.0)-1);
}

void mySlider::OnSpinCtrlUp(wxSpinEvent &event) {
	SetValue(GetValue() + 4);
}

void mySlider::OnSpinCtrlDown(wxSpinEvent &event) {
	SetValue(GetValue() - 4);
}


BEGIN_EVENT_TABLE(wxSpinSlider, wxPanel)
	EVT_SLIDER(ON_SPINSLIDER_ID, wxSpinSlider::OnSlider)
	EVT_TEXT_ENTER(ON_SPINSLIDERCTRL_ID, wxSpinSlider::OnText)
	EVT_SPINCTRL(ON_SPINSLIDERCTRL_ID, wxSpinSlider::OnSpin)
	EVT_SPIN_UP(ON_SPINSLIDERCTRL_ID, wxSpinSlider::OnSpinCtrlUp)
	EVT_SPIN_DOWN(ON_SPINSLIDERCTRL_ID, wxSpinSlider::OnSpinCtrlDown)
END_EVENT_TABLE()

wxSpinSlider::wxSpinSlider(wxWindow* parent, int min_value, int max_value, int change_value) : wxPanel(parent, -1) {
	change = change_value - 1;
    SetProperties(min_value, max_value);
    DoLayout();		
}

void wxSpinSlider::SetProperties(int min_value, int max_value) {
	slider = new wxSlider(this, ON_SPINSLIDER_ID, 0, 0, 255, wxDefaultPosition, wxSize(140, -1));
	slider->SetRange(min_value, max_value);
	spin = new wxSpinCtrl(this, ON_SPINSLIDERCTRL_ID, wxEmptyString, wxDefaultPosition, wxSize(60, -1));	
	spin->SetRange(min_value, max_value);
	spin->SetValue(slider->GetValue());
//	slider->SetSize(140, -1);
//	spin->SetSize(60, -1);
}

void wxSpinSlider::DoLayout() {
	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add(slider, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 0);
	sizer->Add(spin, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 0);	
  	sizer->SetSizeHints(this);
 	SetSizer(sizer);
	Layout();
}

void wxSpinSlider::SetValue(int value) {
	spin->SetValue(value);
	slider->SetValue(value);
}

void wxSpinSlider::OnSlider(wxCommandEvent &event) {
	spin->SetValue(slider->GetValue());
	event.Skip();
}

void wxSpinSlider::OnText(wxCommandEvent &event) {
	SetSlider();
	event.Skip();
}

void wxSpinSlider::OnSpin(wxSpinEvent &event) {
	SetSlider();
	event.Skip();
}

void wxSpinSlider::SetSlider() {
	slider->SetValue(spin->GetValue());
}

void wxSpinSlider::OnSpinCtrlUp(wxSpinEvent &event) {
	SetValue(GetValue() + change);
	event.Skip();
}

void wxSpinSlider::OnSpinCtrlDown(wxSpinEvent &event) {
	SetValue(GetValue() - change);
	event.Skip();
}
