#include "thinkpage.h"
#include "myglcanvas.h"
#include "mynotebook.h"
#include "assignpage.h"
#include "boxer.h"

myThink::myThink(wxWindow* parent, int cnumber) : wxToggleButton(parent, -1, wxT(""), wxDefaultPosition, wxSize(80, -1)) {
	SetNameFromNumber(cnumber);
}

void myThink::Copy(myThink *cthink) {
	this->SetName(cthink->GetName());

 	this->con_restround_less = cthink->con_restround_less;
	this->restround_less = cthink->restround_less;	
	this->con_resttime_less = cthink->con_resttime_less;
	this->resttime_less = cthink->resttime_less;	

	this->con_mrestcount_less = cthink->con_mrestcount_less;
	this->mrestcount_less = cthink->mrestcount_less;	
	this->con_mrestdown_less = cthink->con_mrestdown_less;
	this->mrestdown_less = cthink->mrestdown_less;	

	this->con_distance_greater = cthink->con_distance_greater;
	this->distance_greater = cthink->distance_greater;
	this->con_distance_less = cthink->con_distance_less;
	this->distance_less = cthink->distance_less;
	this->con_mposition = cthink->con_mposition;
	this->mposition = cthink->mposition;	
	this->con_msituation = cthink->con_msituation;
	this->msituation = cthink->msituation;	

	this->con_mswing_greater = cthink->con_mswing_greater;
	this->mswing_greater = cthink->mswing_greater;
	this->con_mswing_less = cthink->con_mswing_less;
	this->mswing_less = cthink->mswing_less;
	this->con_mspeed_greater = cthink->con_mspeed_greater;
	this->mspeed_greater = cthink->mspeed_greater;
	this->con_mspeed_less = cthink->con_mspeed_less;
	this->mspeed_less = cthink->mspeed_less;
	this->con_mstate = cthink->con_mstate;
	this->mstate = cthink->mstate;

	this->con_oswing_greater = cthink->con_oswing_greater;
	this->oswing_greater = cthink->oswing_greater;
	this->con_oswing_less = cthink->con_oswing_less;
	this->oswing_less = cthink->oswing_less;
	this->con_ospeed_greater = cthink->con_ospeed_greater;
	this->ospeed_greater = cthink->ospeed_greater;
	this->con_ospeed_less = cthink->con_ospeed_less;
	this->ospeed_less = cthink->ospeed_less;
 	this->con_ostate = cthink->con_ostate;
	this->ostate = cthink->ostate;

	int i;
	for (i = 0; i < 5; i++) {
		this->action[i] = cthink->action[i];
		this->action_rate[i] = cthink->action_rate[i];
 	}
	
	this->max_cons = cthink->max_cons;
}

void myThink::SetNameFromNumber(int cnumber) {
	wxString name;
	name.Printf("THINK %d", cnumber);
	SetName(name);
}			

void myThink::SumConditions() {
	max_cons = 0;
 	if (con_restround_less) {max_cons++;}
	if (con_resttime_less) {max_cons++;}

	if (con_mrestcount_less) {max_cons++;}
	if (con_mrestdown_less) {max_cons++;}

	if (con_distance_greater) {max_cons++;}
	if (con_distance_less) {max_cons++;}
	if (con_mposition) {max_cons++;}
	if (con_msituation) {max_cons++;}

	if (con_mswing_greater) {max_cons++;}
	if (con_mswing_less) {max_cons++;}
	if (con_mspeed_greater) {max_cons++;}
	if (con_mspeed_less) {max_cons++;}
	if (con_mstate) {max_cons++;}

	if (con_oswing_greater) {max_cons++;}
	if (con_oswing_less) {max_cons++;}
	if (con_ospeed_greater) {max_cons++;}
	if (con_ospeed_less) {max_cons++;}
 	if (con_ostate) {max_cons++;}
}


BEGIN_EVENT_TABLE(ThinkPanel, wxPanel)
	EVT_SLIDER(ON_SPINSLIDER_ID, ThinkPanel::OnChangeSlider)
	EVT_TEXT_ENTER(ON_SPINSLIDERCTRL_ID, ThinkPanel::OnChangeText)
	EVT_SPINCTRL(ON_SPINSLIDERCTRL_ID, ThinkPanel::OnChangeSpin)
	EVT_SPIN_UP(ON_SPINSLIDERCTRL_ID, ThinkPanel::OnChangeSpin)
	EVT_SPIN_DOWN(ON_SPINSLIDERCTRL_ID, ThinkPanel::OnChangeSpin)
END_EVENT_TABLE()


ThinkPanel::ThinkPanel(wxWindow* parent) : wxPanel(parent, -1) {
	SetProperties();
	DoLayout();
}

void ThinkPanel::SetProperties() {
	int i;
	const static wxString position_str[] = {wxT(""), wxT("׺"), wxT("ʡ")};
	const static wxString situation_str[] = {wxT("Կ"), wxT(""), wxT(""), wxT("߳"), wxT("ͥ"), wxT("ͥ"), wxT("ɬ")};
   	const static wxString state_str[] = {wxT("ꥺ"), wxT("ư"), wxT(""), wxT(""), wxT("ɸ"), wxT(""), wxT(""), wxT("")};

	con_restround_less = new wxToggleButton(this, -1, wxT("Round <"));
	restround_less = new wxSpinSlider(this, 0, 15, 1);	
	con_resttime_less = new wxToggleButton(this, -1, wxT("Time <"));
	resttime_less = new wxSpinSlider(this, 0, 300, 10);	

	con_mrestcount_less = new wxToggleButton(this, -1, wxT("Count <"));
	mrestcount_less = new wxSpinSlider(this, 0, 10, 1);	
	con_mrestdown_less = new wxToggleButton(this, -1, wxT("Down <"));
	mrestdown_less = new wxSpinSlider(this, 0, 2, 1);	

	con_distance_greater = new wxToggleButton(this, -1, wxT("Υ >"));
	distance_greater = new wxSpinSlider(this, 0, 6000, 50);	
	con_distance_less = new wxToggleButton(this, -1, wxT("Υ <"));
	distance_less = new wxSpinSlider(this, 0, 6000, 50);	

	con_mposition = new wxToggleButton(this, -1, wxT(""));
	mposition = new wxComboBox(this, -1, wxT(""), wxDefaultPosition, wxSize(200, -1), 3, position_str, wxCB_READONLY);
	mposition->SetSelection(0);
	con_msituation = new wxToggleButton(this, -1, wxT(""));
	msituation = new wxComboBox(this, -1, wxT(""), wxDefaultPosition, wxSize(200, -1), 7, situation_str, wxCB_READONLY);	
	msituation->SetSelection(3);

	con_mswing_greater = new wxToggleButton(this, -1, wxT(" >"));
	mswing_greater= new wxSpinSlider(this, 0, 100, 5);
	con_mswing_less = new wxToggleButton(this, -1, wxT(" <"));
	mswing_less = new wxSpinSlider(this, 0, 100, 5);
	con_mspeed_greater = new wxToggleButton(this, -1, wxT("Damage >"));
	mspeed_greater = new wxSpinSlider(this, 0, 100, 5);
	con_mspeed_less = new wxToggleButton(this, -1, wxT("Damage <"));
	mspeed_less = new wxSpinSlider(this, 0, 100, 5);
	con_mstate = new wxToggleButton(this, -1, wxT(""));
	mstate = new wxComboBox(this, -1, wxT(""), wxDefaultPosition, wxSize(200, -1), 8, state_str, wxCB_READONLY);
	mstate->SetSelection(0);
 
	con_oswing_greater = new wxToggleButton(this, -1, wxT(" >"));
	oswing_greater= new wxSpinSlider(this, 0, 100, 5);
	con_oswing_less = new wxToggleButton(this, -1, wxT(" <"));
	oswing_less = new wxSpinSlider(this, 0, 100, 5);
	con_ospeed_greater = new wxToggleButton(this, -1, wxT("Damage >"));
	ospeed_greater = new wxSpinSlider(this, 0, 100, 5);
	con_ospeed_less = new wxToggleButton(this, -1, wxT("Damage <"));
	ospeed_less = new wxSpinSlider(this, 0, 100, 5);
	con_ostate = new wxToggleButton(this, -1, wxT(""));
	ostate = new wxComboBox(this, -1, wxT(""), wxDefaultPosition, wxSize(200, -1), 8, state_str, wxCB_READONLY);
	ostate->SetSelection(0);
	
	for (i = 0; i < 5; i++) {	
		action[i] = new wxComboBox(this, -1, wxT(""), wxDefaultPosition, wxSize(100, -1), 8, state_str, wxCB_READONLY);
 		action_rate[i] = new wxSpinSlider(this, 0, 100, 5);
		action[i]->SetSelection(0);
	}
 	action_rate[0]->SetValue(100);
}

void ThinkPanel::DoLayout() {
	wxBoxSizer *tsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer *bsizer11 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("")), wxVERTICAL);
    wxStaticBoxSizer *bsizer12 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ܥ־")), wxVERTICAL);
    wxStaticBoxSizer *bsizer13 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ܥ־")), wxVERTICAL);
    wxStaticBoxSizer *bsizer2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("ư")), wxVERTICAL);

	wxBoxSizer *hsizer111 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer112 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer113 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer114 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer115 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer116 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer117 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer118 = new wxBoxSizer(wxHORIZONTAL);

	wxBoxSizer *hsizer121 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer122 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer123 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer124 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer125 = new wxBoxSizer(wxHORIZONTAL);

	wxBoxSizer *hsizer131 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer132 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer133 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer134 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer135 = new wxBoxSizer(wxHORIZONTAL);

	wxBoxSizer *hsizer21 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer22 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer23 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer24 = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hsizer25 = new wxBoxSizer(wxHORIZONTAL);

	bsizer11->Add(hsizer111, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer112, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer113, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer114, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer115, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer116, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer117, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer11->Add(hsizer118, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);

	bsizer12->Add(hsizer121, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer12->Add(hsizer122, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer12->Add(hsizer123, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer12->Add(hsizer124, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer12->Add(hsizer125, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);

	bsizer13->Add(hsizer131, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer13->Add(hsizer132, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer13->Add(hsizer133, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer13->Add(hsizer134, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer13->Add(hsizer135, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);

	bsizer2->Add(hsizer21, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer2->Add(hsizer22, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer2->Add(hsizer23, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer2->Add(hsizer24, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);
	bsizer2->Add(hsizer25, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 0);

	tsizer->Add(bsizer2, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(bsizer11, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(bsizer12, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(bsizer13, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	hsizer111->Add(con_restround_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer111->Add(restround_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer112->Add(con_resttime_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer112->Add(resttime_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer113->Add(con_mrestcount_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer113->Add(mrestcount_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer114->Add(con_mrestdown_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer114->Add(mrestdown_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer115->Add(con_distance_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer115->Add(distance_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer116->Add(con_distance_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer116->Add(distance_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer117->Add(con_mposition, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer117->Add(mposition, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer118->Add(con_msituation, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer118->Add(msituation, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer121->Add(con_mswing_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer121->Add(mswing_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer122->Add(con_mswing_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer122->Add(mswing_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer123->Add(con_mspeed_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer123->Add(mspeed_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer124->Add(con_mspeed_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer124->Add(mspeed_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer125->Add(con_mstate, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer125->Add(mstate, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer131->Add(con_oswing_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer131->Add(oswing_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer132->Add(con_oswing_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer132->Add(oswing_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer133->Add(con_ospeed_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer133->Add(ospeed_greater, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer134->Add(con_ospeed_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer134->Add(ospeed_less, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer135->Add(con_ostate, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer135->Add(ostate, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

	hsizer21->Add(action[0], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer21->Add(action_rate[0], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer22->Add(action[1], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer22->Add(action_rate[1], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer23->Add(action[2], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer23->Add(action_rate[2], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer24->Add(action[3], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer24->Add(action_rate[3], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);
	hsizer25->Add(action[4], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxLEFT, 3);
	hsizer25->Add(action_rate[4], 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 3);

 	tsizer->SetSizeHints(this);
	SetSizer(tsizer);
    Layout();
}

int ThinkPanel::SumPercent() {
	int i, percent;
	percent = 0;
	for (i = 0; i < 5; i++) {	
		percent += action_rate[i]->GetValue();
	}
	return percent;	
}

void ThinkPanel::Get(myThink *cthink) {
	this->con_restround_less->SetValue(cthink->con_restround_less);
	this->restround_less->SetValue(cthink->restround_less);	
	this->con_resttime_less->SetValue(cthink->con_resttime_less);
	this->resttime_less->SetValue(cthink->resttime_less);	

	this->con_mrestcount_less->SetValue(cthink->con_mrestcount_less);
	this->mrestcount_less->SetValue(cthink->mrestcount_less);	
	this->con_mrestdown_less->SetValue(cthink->con_mrestdown_less);
	this->mrestdown_less->SetValue(cthink->mrestdown_less);	

	this->con_distance_greater->SetValue(cthink->con_distance_greater);
	this->distance_greater->SetValue(cthink->distance_greater);
	this->con_distance_less->SetValue(cthink->con_distance_less);
	this->distance_less->SetValue(cthink->distance_less);
	this->con_mposition->SetValue(cthink->con_mposition);
	this->mposition->SetSelection(cthink->mposition);	
	this->con_msituation->SetValue(cthink->con_msituation);
	this->msituation->SetSelection(cthink->msituation);	

	this->con_mswing_greater->SetValue(cthink->con_mswing_greater);
	this->mswing_greater->SetValue(cthink->mswing_greater);
	this->con_mswing_less->SetValue(cthink->con_mswing_less);
	this->mswing_less->SetValue(cthink->mswing_less);
	this->con_mspeed_greater->SetValue(cthink->con_mspeed_greater);
	this->mspeed_greater->SetValue(cthink->mspeed_greater);
	this->con_mspeed_less->SetValue(cthink->con_mspeed_less);
	this->mspeed_less->SetValue(cthink->mspeed_less);
	this->con_mstate->SetValue(cthink->con_mstate);
	this->mstate->SetSelection(cthink->mstate);

	this->con_oswing_greater->SetValue(cthink->con_oswing_greater);
	this->oswing_greater->SetValue(cthink->oswing_greater);
	this->con_oswing_less->SetValue(cthink->con_oswing_less);
	this->oswing_less->SetValue(cthink->oswing_less);
	this->con_ospeed_greater->SetValue(cthink->con_ospeed_greater);
	this->ospeed_greater->SetValue(cthink->ospeed_greater);
	this->con_ospeed_less->SetValue(cthink->con_ospeed_less);
	this->ospeed_less->SetValue(cthink->ospeed_less);
 	this->con_ostate->SetValue(cthink->con_ostate);
	this->ostate->SetSelection(cthink->ostate);

	int i;
	for (i = 0; i < 5; i++) {
		this->action[i]->SetSelection(cthink->action[i]);
		this->action_rate[i]->SetValue(cthink->action_rate[i]);
 	}

}

void ThinkPanel::Set(myThink *cthink) {
 	cthink->con_restround_less = this->con_restround_less->GetValue();
	cthink->restround_less = this->restround_less->GetValue();	
	cthink->con_resttime_less = this->con_resttime_less->GetValue();
	cthink->resttime_less = this->resttime_less->GetValue();	

	cthink->con_mrestcount_less = this->con_mrestcount_less->GetValue();
	cthink->mrestcount_less = this->mrestcount_less->GetValue();	
	cthink->con_mrestdown_less = this->con_mrestdown_less->GetValue();
	cthink->mrestdown_less = this->mrestdown_less->GetValue();	

	cthink->con_distance_greater = this->con_distance_greater->GetValue();
	cthink->distance_greater = this->distance_greater->GetValue();
	cthink->con_distance_less = this->con_distance_less->GetValue();
	cthink->distance_less = this->distance_less->GetValue();
	cthink->con_mposition = this->con_mposition->GetValue();
	cthink->mposition = this->mposition->GetSelection();	
	cthink->con_msituation = this->con_msituation->GetValue();
	cthink->msituation = this->msituation->GetSelection();	

	cthink->con_mswing_greater = this->con_mswing_greater->GetValue();
	cthink->mswing_greater = this->mswing_greater->GetValue();
	cthink->con_mswing_less = this->con_mswing_less->GetValue();
	cthink->mswing_less = this->mswing_less->GetValue();
	cthink->con_mspeed_greater = this->con_mspeed_greater->GetValue();
	cthink->mspeed_greater = this->mspeed_greater->GetValue();
	cthink->con_mspeed_less = this->con_mspeed_less->GetValue();
	cthink->mspeed_less = this->mspeed_less->GetValue();
	cthink->con_mstate = this->con_mstate->GetValue();
	cthink->mstate = this->mstate->GetSelection();

	cthink->con_oswing_greater = this->con_oswing_greater->GetValue();
	cthink->oswing_greater = this->oswing_greater->GetValue();
	cthink->con_oswing_less = this->con_oswing_less->GetValue();
	cthink->oswing_less = this->oswing_less->GetValue();
	cthink->con_ospeed_greater = this->con_ospeed_greater->GetValue();
	cthink->ospeed_greater = this->ospeed_greater->GetValue();
	cthink->con_ospeed_less = this->con_ospeed_less->GetValue();
	cthink->ospeed_less = this->ospeed_less->GetValue();
 	cthink->con_ostate = this->con_ostate->GetValue();
	cthink->ostate = this->ostate->GetSelection();

	int i;
	for (i = 0; i < 5; i++) {
		cthink->action[i] = this->action[i]->GetSelection();
		cthink->action_rate[i] = this->action_rate[i]->GetValue();
 	}

}

void ThinkPanel::Init() {
	this->con_restround_less->SetValue(false);
	this->restround_less->SetValue(0);	
	this->con_resttime_less->SetValue(false);
	this->resttime_less->SetValue(0);	

	this->con_mrestcount_less->SetValue(false);
	this->mrestcount_less->SetValue(0);	
	this->con_mrestdown_less->SetValue(false);
	this->mrestdown_less->SetValue(0);	

	this->con_distance_greater->SetValue(false);
	this->distance_greater->SetValue(0);
	this->con_distance_less->SetValue(false);
	this->distance_less->SetValue(0);
	this->con_mposition->SetValue(false);
	this->mposition->SetSelection(0);	
	this->con_msituation->SetValue(false);
	this->msituation->SetSelection(0);	

	this->con_mswing_greater->SetValue(false);
	this->mswing_greater->SetValue(0);
	this->con_mswing_less->SetValue(false);
	this->mswing_less->SetValue(0);
	this->con_mspeed_greater->SetValue(false);
	this->mspeed_greater->SetValue(0);
	this->con_mspeed_less->SetValue(false);
	this->mspeed_less->SetValue(0);
	this->con_mstate->SetValue(false);
	this->mstate->SetSelection(0);

	this->con_oswing_greater->SetValue(false);
	this->oswing_greater->SetValue(0);
	this->con_oswing_less->SetValue(false);
	this->oswing_less->SetValue(0);
	this->con_ospeed_greater->SetValue(false);
	this->ospeed_greater->SetValue(0);
	this->con_ospeed_less->SetValue(false);
	this->ospeed_less->SetValue(0);
 	this->con_ostate->SetValue(false);
	this->ostate->SetSelection(0);

	int i;
	this->action[0]->SetSelection(0);
	this->action_rate[0]->SetValue(100);
	for (i = 1; i < 5; i++) {
		this->action[i]->SetSelection(0);
		this->action_rate[i]->SetValue(0);
 	}

}

void ThinkPanel::NormPercent(int percent) {
	int i, value;
	if (percent > 100) {
		percent = percent - 100;
		for (i = 0; i < 5; i++) {
			value = action_rate[i]->GetValue();
			if (value >= percent) {
				value -= percent;
				action_rate[i]->SetValue(value);
				break;			
			}		
		}
	} else {	
		percent = 100 - percent;
		value = action_rate[0]->GetValue();
		action_rate[0]->SetValue(value + percent);
	}	
}

void ThinkPanel::OnChangeSlider(wxCommandEvent &event) {
	int percent;
	percent = SumPercent();	
	if (percent != 100) {
		NormPercent(percent);
	}
}

void ThinkPanel::OnChangeText(wxCommandEvent &event) {
	int percent;
	percent = SumPercent();	
	if (percent != 100) {
		NormPercent(percent);
	}
}

void ThinkPanel::OnChangeSpin(wxSpinEvent &event) {
	int percent;
	percent = SumPercent();	
	if (percent != 100) {
		NormPercent(percent);
	}
}


BEGIN_EVENT_TABLE(ThinkPage, wxScrolledWindow)
	EVT_TEXT(ON_THINK_NAME_ID, ThinkPage::OnThinkName) 
	EVT_BUTTON(ON_ADD_THINK_ID, ThinkPage::OnAdd) 
	EVT_BUTTON(ON_DEL_THINK_ID, ThinkPage::OnDel) 
	EVT_BUTTON(ON_COPY_THINK_ID, ThinkPage::OnCopy) 
	EVT_BUTTON(ON_PASTE_THINK_ID, ThinkPage::OnPaste) 
	EVT_BUTTON(ON_INIT_THINK_ID, ThinkPage::OnInit) 
	EVT_BUTTON(ON_UP_THINK_ID, ThinkPage::OnUp) 
	EVT_BUTTON(ON_DOWN_THINK_ID, ThinkPage::OnDown) 
	EVT_BUTTON(ON_SORT_THINK_ID, ThinkPage::OnSort) 
	EVT_TOGGLEBUTTON(-1, ThinkPage::OnToggle)
END_EVENT_TABLE()

ThinkPage::	ThinkPage(wxWindow* parent, AssignPage *cpage)
	: wxScrolledWindow(parent, -1) {
	page = cpage;
	SetProperties();
	DoLayout();
}

void ThinkPage::SetProperties() {
	SetScrollRate(10,10);

	copy = new myThink(this, 0);
	swap = new myThink(this, 1);
	copy->Hide();
	swap->Hide();

	max_thinks = 1;
	int i;
	for (i = 0; i < max_thinks; i++) {
		myThink *think = new myThink(this, i);
		thinks.Add(think);
	}
	select_think = 0;

	think_name = new wxTextCtrl(this, ON_THINK_NAME_ID, wxT(""), wxDefaultPosition, wxSize(80, -1));
	add_think = new wxButton(this, ON_ADD_THINK_ID, wxT("׹"));
	del_think = new wxButton(this, ON_DEL_THINK_ID, wxT("׹ͺ"));	
 	copy_think = new wxButton(this, ON_COPY_THINK_ID, wxT("ԡ"));
	paste_think = new wxButton(this, ON_PASTE_THINK_ID, wxT("ڡ"));	
	init_think = new wxButton(this, ON_INIT_THINK_ID, wxT(""));	
 	up_think = new wxButton(this, ON_UP_THINK_ID, wxT("UP"));
	down_think = new wxButton(this, ON_DOWN_THINK_ID, wxT("DOWN"));	
	sort_think = new wxButton(this, ON_SORT_THINK_ID, wxT("SORT"));
  
	tpanel = new ThinkPanel(this);
	tpanel->Set(GetThink());;
	think_name->SetValue(GetThink()->GetName());
	GetThink()->On();
	copy->Copy(GetThink());
	MakeComboLists();
}

void ThinkPage::DoLayout() {
    tsizer = new wxBoxSizer(wxVERTICAL);
	gsizer = new wxGridSizer(max_thinks/3, 3, 0, 0);

    wxStaticBoxSizer *bsizer1 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("׹ͥѥ")), wxVERTICAL);
    wxStaticBoxSizer *bsizer12 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("׹")), wxVERTICAL);
	wxGridSizer *gsizer12 = new wxGridSizer(3, 3, 0, 0);

	bsizer12->Add(gsizer12, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	bsizer1->Add(gsizer, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	bsizer1->Add(bsizer12, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	
	tsizer->Add(bsizer1, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	tsizer->Add(tpanel, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);

	gsizer12->Add(think_name, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(add_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(del_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(copy_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(paste_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(init_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(up_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(down_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
	gsizer12->Add(sort_think, 0, wxALIGN_CENTRE_HORIZONTAL |wxALIGN_CENTER_VERTICAL | wxALL, 5);
 
	int i;
	for (i = 0; i < max_thinks; i++) {
		gsizer->Add(thinks[i],  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
	
  	SetSizer(tsizer);
 	ReLayout();
}

void ThinkPage::ReLayout() {
 	tsizer->SetVirtualSizeHints(this);
	FitInside();
	Layout();
}

void ThinkPage::MakeComboLists() {
	int i, j, save_select;
	for (i = 0; i < 5; i++) {
		save_select = tpanel->action[i]->GetSelection();
		tpanel->action[i]->Clear();
		tpanel->action[i]->Append(wxT("̵ư"));
		tpanel->action[i]->Append(wxT("ư"));
		tpanel->action[i]->Append(wxT("ư"));
		tpanel->action[i]->Append(wxT("ư"));
		tpanel->action[i]->Append(wxT("ư"));
		for (j = 10; j < 37; j++) {
			tpanel->action[i]->Append(page->assigns[j]->GetStringSelection());
		}
		tpanel->action[i]->Append(wxT(""));
		tpanel->action[i]->Append(wxT("ɬӥ͡"));
		tpanel->action[i]->Append(wxT("ư"));
		tpanel->action[i]->Append(wxT(""));
		tpanel->action[i]->Append(wxT(""));
		tpanel->action[i]->Append(wxT("ɸ"));
		tpanel->action[i]->Append(wxT("ξ"));
		tpanel->action[i]->SetSelection(save_select);
	}
}


void ThinkPage::OnThinkName(wxCommandEvent &event) {
	GetThink()->SetName(think_name->GetValue());	
}		

void ThinkPage::OnAdd(wxCommandEvent &event) {
	Add();
	ReLayout();
}		

void ThinkPage::OnDel(wxCommandEvent &event) {
	Del();
	ReLayout();
}		

void ThinkPage::OnCopy(wxCommandEvent &event) {
	tpanel->Set(copy);
	copy->SetName(GetThink()->GetName());
}		

void ThinkPage::OnPaste(wxCommandEvent &event) {
	GetThink()->Copy(copy);
	GetThink()->SetName(copy->GetName());
	tpanel->Get(copy);
	think_name->SetValue(copy->GetName());
}		

void ThinkPage::OnInit(wxCommandEvent &event) {
	tpanel->Init();
}		

void ThinkPage::OnUp(wxCommandEvent &event) {
	if (SelectThink() == 0) {
		return;	
	}
	int i;
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Remove(thinks[i]);	
	}
	Set();
    GetThink()->Off();
	
	swap->Copy(GetThink());
	GetThink()->Copy(thinks[select_think-1]);
	thinks[select_think-1]->Copy(swap);
	
	select_think--;
	Get();	
    GetThink()->On();
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Add(thinks[i],  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
	ReLayout();
}		

void ThinkPage::OnDown(wxCommandEvent &event) {
	if (SelectThink() == MaxThink() -1) {
		return;	
	}
	int i;
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Remove(thinks[i]);	
	}
	Set();
    GetThink()->Off();
	
	swap->Copy(GetThink());
	GetThink()->Copy(thinks[select_think+1]);
	thinks[select_think+1]->Copy(swap);
	
	select_think++;
	Get();	
    GetThink()->On();
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Add(thinks[i],  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
	ReLayout();
}		

void ThinkPage::OnSort(wxCommandEvent &event) {
	int i, j;
	Set();
	for (i = select_think; i < MaxThink(); i++) {
		thinks[i]->SumConditions();
	}

	for (i = 0; i < MaxThink() - select_think; i++) {
		for (j = MaxThink() - 1; j > select_think + i; j--) {
			if (thinks[j]->Conditions() > thinks[j-1]->Conditions()) {
				Swap(thinks[j], thinks[j-1]);			
			}
		}
	}
	Get();
	think_name->SetValue(GetThink()->GetName());
}

void ThinkPage::ThinkPre() {
	int new_select = select_think - 1;
	if (new_select < 0) {
		new_select = MaxThink() - 1;	
	}
	SetSelectThink(new_select);
}

void ThinkPage::ThinkNext() {
	int new_select = select_think + 1;
	if (new_select >= MaxThink()) {
		new_select = 0;	
	}
	SetSelectThink(new_select);
}

void ThinkPage::OnToggle(wxCommandEvent &event) {
	int i, new_select = -1;
	wxObject *obj = event.GetEventObject();

	for (i = 0; i < MaxThink(); i++) {
		if(obj == thinks[i]->Object()) {
			new_select = i;
			break;
		}
	}
	if (new_select != -1) {
		SetSelectThink(new_select);
	}   
}		

void ThinkPage::Add() {
	if (MaxThink() >= 99) {
		return;	
	}
	int i;
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Remove(thinks[i]);	
	}
	myThink *think = new myThink(this, 0);
	think->Copy(GetThink());
	think->SetNameFromNumber(max_thinks);
	thinks.Insert(think, SelectThink());

	select_think++;
	max_thinks++;
	for (i = 0; i < MaxThink(); i++) {
		gsizer->Add(thinks[i],  0, wxGROW |wxALIGN_CENTRE_VERTICAL | wxALL, 5);
	}
}

void ThinkPage::Del() {
	if (max_thinks < 2) {
		return;	
	}
	max_thinks--;
	myThink *think = GetThink();
	gsizer->Remove(think);
	thinks.Remove(think);
	delete think;	
	gsizer->SetRows(max_thinks/3);
	if (select_think >= MaxThink() && MaxThink()) {
		select_think--;	
	}
	GetThink()->On();
	Get();
	think_name->SetValue(GetThink()->GetName());
}


void ThinkPage::SetSelectThink(int new_select) {
	if (new_select != SelectThink()) {
		GetThink()->Off();
		Set();

		select_think = new_select;
		think_name->SetValue(GetThink()->GetName());	
		GetThink()->On();
		Get();
	} else {
		GetThink()->On();	
	}
	ReLayout();
}

bool ThinkPage::Save(wxTextOutputStream &out) {
	int i, j;
	wxString str;
	Set();
	out << wxT("#Think Data made by") << wxT(BM_CAPTION_EN_TITLE) << endl;
	out << wxT("#Select Think") << endl;
	out << SelectThink() << endl;;
	out << wxT("#Max Think") << endl;
	out << MaxThink() << endl;;
	for (i = 0; i < MaxThink(); i++) {
		myThink *t = thinks[i];
		str.Printf("#Think No. %d", i);
		out << str << endl;
		out << wxT("#Think Name") << endl;
		out << t->GetName() << endl;		

		out << wxT("#Match Condition") << endl;
		out << (int)t->con_restround_less << endl;
		out << t->restround_less << endl;	
		out << (int)t->con_resttime_less << endl;
		out << t->resttime_less << endl;	
		out << (int)t->con_mrestcount_less << endl;
		out << t->mrestcount_less << endl;	
		out << (int)t->con_mrestdown_less << endl;
		out << t->mrestdown_less << endl;	
		out << (int)t->con_distance_greater << endl;
		out << t->distance_greater << endl;
		out << (int)t->con_distance_less << endl;
		out << t->distance_less << endl;
		out << (int)t->con_mposition << endl;
		out << t->mposition << endl;	
		out << (int)t->con_msituation << endl;
		out << t->msituation << endl;	

		out << wxT("#Boxer Condition") << endl;
		out << (int)t->con_mswing_greater<< endl;
		out << t->mswing_greater<< endl;
		out << (int)t->con_mswing_less<< endl;
		out << t->mswing_less<< endl;
		out << (int)t->con_mspeed_greater<< endl;
		out << t->mspeed_greater<< endl;
		out << (int)t->con_mspeed_less<< endl;
		out << t->mspeed_less<< endl;
		out << (int)t->con_mstate<< endl;
		out << t->mstate<< endl;

		out << wxT("#Opponent Boxer Condition") << endl;
		out << (int)t->con_oswing_greater<< endl;
		out << t->oswing_greater<< endl;
		out << (int)t->con_oswing_less<< endl;
		out << t->oswing_less<< endl;
		out << (int)t->con_ospeed_greater<< endl;
		out << t->ospeed_greater<< endl;
		out << (int)t->con_ospeed_less<< endl;
		out << t->ospeed_less<< endl;
	 	out << (int)t->con_ostate<< endl;
		out << t->ostate<< endl;

		out << wxT("#Select Action") << endl;
		for (j = 0; j < 5; j++) {
			out << t->action[j] << endl;
			out << t->action_rate[j] << endl;
		}
	}        


	return true;
}
	
bool ThinkPage::Load(wxTextInputStream &in) {
	int i, j, num, sel, max;
	wxString str;
	str = in.ReadLine();
	str = in.ReadLine();
	in >> sel;
	str = in.ReadLine();
	in >> max;

	if (max > 99 || max < 1) {
		wxLogError("׹ͤο۾Ǥ");
		return false;
	}
	while(MaxThink() > max) {
		Del();	
	}
	while(MaxThink() < max) {
		Add();	
	}

	if (sel >= MaxThink()) {
		wxLogError("׹ͥǡ۾Ǥ");
		return false;
	}
	SetSelectThink(sel);
	for (i = 0; i < max; i++) {
		myThink *t = thinks[i];
  		str = in.ReadLine();
  		str = in.ReadLine();
		str = in.ReadLine();
		t->SetName(str);

		str = in.ReadLine();
		in >> num;
		t->con_restround_less = (bool)num;
		in >> t->restround_less;	
		in >> num;
		t->con_resttime_less = (bool)num;
		in >> t->resttime_less;	
		in >> num;
		t->con_mrestcount_less = (bool)num;
		in >> t->mrestcount_less;	
		in >> num;
		t->con_mrestdown_less = (bool)num;
		in >> t->mrestdown_less;	

		in >> num;
		t->con_distance_greater = (bool)num;
		in >> t->distance_greater;
		in >> num;
		t->con_distance_less = (bool)num;
		in >> t->distance_less;
		in >> num;
		t->con_mposition = (bool)num;
		in >> t->mposition;	
		in >> num;
		t->con_msituation = (bool)num;
		in >> t->msituation;

		str = in.ReadLine();
		in >> num;
		t->con_mswing_greater = (bool)num;
		in >> t->mswing_greater;
		in >> num;
		t->con_mswing_less = (bool)num;
		in >> t->mswing_less;
		in >> num;
		t->con_mspeed_greater = (bool)num;
		in >> t->mspeed_greater;
		in >> num;
		t->con_mspeed_less = (bool)num;
		in >> t->mspeed_less;
		in >> num;
		t->con_mstate = (bool)num;
		in >> t->mstate;

		str = in.ReadLine();
		in >> num;
		t->con_oswing_greater = (bool)num;
		in >> t->oswing_greater;
		in >> num;
		t->con_oswing_less = (bool)num;
		in >> t->oswing_less;
		in >> num;
		t->con_ospeed_greater = (bool)num;
		in >> t->ospeed_greater;
		in >> num;
		t->con_ospeed_less = (bool)num;
		in >> t->ospeed_less;
	 	in >> num;
		t->con_ostate = (bool)num;
		in >> t->ostate;

		str = in.ReadLine();
		for (j = 0; j < 5; j++) {
			in >> t->action[j];
			in >> t->action_rate[j];
		}       
	}

	Get();
	ReLayout();
	return true;
}
