#ifndef	THINKPAGE_H
#define THINKPAGE_H

#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/treectrl.h>
#include <wx/button.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "myslider.h"

class AssignPage;
class ThinkPage;

class myThink : public wxToggleButton {
	myThink(wxWindow* parent, int cnumber);
	wxString GetName() const {return GetLabel();}
    void SetName(const wxString& name) {SetLabel(name);}
	void SetNameFromNumber(int cnumber);
	bool IsOn(){return GetValue();}
	void On() {SetValue(true);}
	void Off() {SetValue(false);}
	wxObject* Object() {return (wxObject*)this;}
	void SetToggle(bool value) {SetValue(value);}
	void Copy(myThink *cthink);
	void SumConditions();
	int Conditions() const {return max_cons;}
private:
	int max_cons;

	bool con_restround_less;
	int restround_less;	
	bool con_resttime_less;
	int resttime_less;	

	bool con_mrestcount_less;
	int mrestcount_less;	
	bool con_mrestdown_less;
	int mrestdown_less;	
	bool con_distance_greater;
	int distance_greater;
	bool con_distance_less;
	int distance_less;
	bool con_mposition;
	int mposition;	
	bool con_msituation;
	int msituation;	

	bool con_mswing_greater;
	int mswing_greater;
	bool con_mswing_less;
	int mswing_less;
	bool con_mspeed_greater;
	int mspeed_greater;
	bool con_mspeed_less;
	int mspeed_less;
	bool con_mstate;
	int mstate;

	bool con_oswing_greater;
	int oswing_greater;
	bool con_oswing_less;
	int oswing_less;
	bool con_ospeed_greater;
	int ospeed_greater;
	bool con_ospeed_less;
	int ospeed_less;
 	bool con_ostate;
	int ostate;

	int action[5];
	int action_rate[5];

	friend class ThinkPanel;
	friend class ThinkPage;
};
WX_DEFINE_ARRAY(myThink*, myThinks);


class ThinkPanel : public wxPanel {
public:
	ThinkPanel(wxWindow* parent);
	void OnChangeSlider(wxCommandEvent &event);
	void OnChangeText(wxCommandEvent &event);
	void OnChangeSpin(wxSpinEvent &event);
	void Set(myThink *cthink);
	void Get(myThink *cthink);
	void Init();
	int SumPercent();
	void NormPercent(int percent);
private:
    void SetProperties();
    void DoLayout();

	wxToggleButton *con_restround_less;
	wxSpinSlider *restround_less;	
	wxToggleButton *con_resttime_less;
	wxSpinSlider *resttime_less;	

	wxToggleButton *con_mrestcount_less;
	wxSpinSlider *mrestcount_less;	
	wxToggleButton *con_mrestdown_less;
	wxSpinSlider *mrestdown_less;	
	wxToggleButton *con_distance_greater;
	wxSpinSlider *distance_greater;
	wxToggleButton *con_distance_less;
	wxSpinSlider *distance_less;
	wxToggleButton *con_mposition;
	wxComboBox *mposition;	
	wxToggleButton *con_msituation;
	wxComboBox *msituation;	

	wxToggleButton *con_mswing_greater;
	wxSpinSlider *mswing_greater;
	wxToggleButton *con_mswing_less;
	wxSpinSlider *mswing_less;
	wxToggleButton *con_mspeed_greater;
	wxSpinSlider *mspeed_greater;
	wxToggleButton *con_mspeed_less;
	wxSpinSlider *mspeed_less;
	wxToggleButton *con_mstate;
	wxComboBox *mstate;

	wxToggleButton *con_oswing_greater;
	wxSpinSlider *oswing_greater;
	wxToggleButton *con_oswing_less;
	wxSpinSlider *oswing_less;
	wxToggleButton *con_ospeed_greater;
	wxSpinSlider *ospeed_greater;
	wxToggleButton *con_ospeed_less;
	wxSpinSlider *ospeed_less;
 	wxToggleButton *con_ostate;
	wxComboBox *ostate;

	wxComboBox *action[5];
	wxSpinSlider *action_rate[5];

	friend class ThinkPage;
	DECLARE_EVENT_TABLE()
};


class ThinkPage : public wxScrolledWindow {
public:
	ThinkPage(wxWindow* parent, AssignPage *cpage);
	void MakeComboLists();
	
	int MaxThink() const {return max_thinks;}
	int SelectThink() const {return select_think;}
	myThink* GetThink() const {return thinks[select_think];}
	void Set() {tpanel->Set(GetThink());}
	void Get() {tpanel->Get(GetThink());}
	void Add();
	void Del();
	void SetSelectThink(int new_select);
 	void Swap(myThink *t1, myThink *t2) {
		swap->Copy(t1);
		t1->Copy(t2);
		t2->Copy(swap);		
	}

	void ThinkPre();
	void ThinkNext();	
	bool Save(wxTextOutputStream &out);
	bool Load(wxTextInputStream &in);
private:
    void SetProperties();
    void DoLayout();
	void ReLayout();

	void OnThinkName(wxCommandEvent &event);
	void OnAdd(wxCommandEvent &event);		
	void OnDel(wxCommandEvent &event);		
	void OnCopy(wxCommandEvent &event);		
	void OnPaste(wxCommandEvent &event);		
	void OnInit(wxCommandEvent &event);		
	void OnUp(wxCommandEvent &event);		
	void OnDown(wxCommandEvent &event);		
	void OnSort(wxCommandEvent &event);		
	void OnToggle(wxCommandEvent &event);		

	myThink *copy, *swap;
	myThinks thinks;
	int select_think;
	int max_thinks;

	wxTextCtrl *think_name;
	wxButton *add_think;
	wxButton *del_think;
	wxButton *copy_think;
	wxButton *paste_think;
	wxButton *init_think;
	wxButton *up_think;
	wxButton *down_think;
	wxButton *sort_think;

	ThinkPanel *tpanel;
	wxGridSizer *gsizer;
	wxBoxSizer *tsizer;
   	AssignPage* page;
	DECLARE_EVENT_TABLE()
};

enum {
 	ON_THINK_NAME_ID,
	ON_ADD_THINK_ID,
	ON_DEL_THINK_ID,
	ON_COPY_THINK_ID,
	ON_PASTE_THINK_ID,
	ON_INIT_THINK_ID,
	ON_UP_THINK_ID,
	ON_DOWN_THINK_ID,
	ON_SORT_THINK_ID
};

#endif // THINKPAGE_H
