#include "bframe.h"
#include "boxer.h"

const float LengthUnit = 1000000.0;

BFrame::BFrame() {
}

BFrame::~BFrame() {
}
/*
BFrame& BFrame::operator=(const BFrame &cbframe) {
	int static i;
	for (i = 0; i < 44; i++) {
		this->joints[i] = cbframe.joints[i];
	}
	return *this; 
}
*/
void BFrame::Copy(const BFrame &cbframe) {
	int static i;
	for (i = 0; i < 44; i++) {
		this->joints[i] = cbframe.joints[i];
	}
}

void BFrame::Set(Boxer *boxer) {
	boxer->SetPos(joints[0], joints[1], joints[2]);
	boxer->SetAngle(joints[3], joints[4], joints[5]);

	boxer->right_arm.fist.SetAngle(0.0, 0.0, joints[6]);
	boxer->right_arm.forearm.SetAngle(joints[7], 0.0,  0.0);
	boxer->right_arm.upperarm.SetAngle(0.0, 0.0, joints[8]);
	boxer->right_arm.shoulder.SetAngle(joints[9], joints[10], 0.0);

	boxer->left_arm.fist.SetAngle(0.0, 0.0, joints[11]);
	boxer->left_arm.forearm.SetAngle(joints[12], 0.0,  0.0);
	boxer->left_arm.upperarm.SetAngle(0.0, 0.0, joints[13]);
	boxer->left_arm.shoulder.SetAngle(joints[14], joints[15], 0.0);

	boxer->right_leg.toe.SetAngle(joints[16], 0.0, 0.0);
	boxer->right_leg.foot.SetAngle(joints[17], joints[18], joints[19]);
	boxer->right_leg.shank.SetAngle(joints[20], 0.0, 0.0);
	boxer->right_leg.thigh.SetAngle(joints[21], joints[22], joints[23]);

	boxer->left_leg.toe.SetAngle(joints[24], 0.0, 0.0);
	boxer->left_leg.foot.SetAngle(joints[25], joints[26], joints[27]);
	boxer->left_leg.shank.SetAngle(joints[28], 0.0, 0.0);
	boxer->left_leg.thigh.SetAngle(joints[29], joints[30], joints[31]);

	boxer->body.abdomen.SetAngle(joints[32], joints[33], joints[34]);
	boxer->body.breast.SetAngle(joints[35], joints[36], joints[37]);

	boxer->head.neck.SetAngle(joints[38], joints[39], joints[40]);
	boxer->head.face.SetAngle(joints[41], joints[42], joints[43]);
}


void BFrame::Get(Boxer *boxer) {
	joints[0] = boxer->X();
	joints[1] = boxer->Y();
	joints[2] = boxer->Z();
	joints[3] = boxer->Alpha();
	joints[4] = boxer->Beta();
	joints[5] = boxer->Gamma();
	
	joints[6] = boxer->right_arm.fist.Gamma();
	joints[7] = boxer->right_arm.forearm.Alpha();
	joints[8] = boxer->right_arm.upperarm.Gamma();
	joints[9] = boxer->right_arm.shoulder.Alpha();
	joints[10] = boxer->right_arm.shoulder.Beta();
	
	joints[11] = boxer->left_arm.fist.Gamma();
	joints[12] = boxer->left_arm.forearm.Alpha();
	joints[13] = boxer->left_arm.upperarm.Gamma();
	joints[14] = boxer->left_arm.shoulder.Alpha();
	joints[15] = boxer->left_arm.shoulder.Beta();
	
	joints[16] = boxer->right_leg.toe.Alpha();
	joints[17] = boxer->right_leg.foot.Alpha();
	joints[18] = boxer->right_leg.foot.Beta();
	joints[19] = boxer->right_leg.foot.Gamma();
	joints[20] = boxer->right_leg.shank.Alpha();
	joints[21] = boxer->right_leg.thigh.Alpha();
	joints[22] = boxer->right_leg.thigh.Beta();
	joints[23] = boxer->right_leg.thigh.Gamma();

	joints[24] = boxer->left_leg.toe.Alpha();
	joints[25] = boxer->left_leg.foot.Alpha();
	joints[26] = boxer->left_leg.foot.Beta();
	joints[27] = boxer->left_leg.foot.Gamma();
	joints[28] = boxer->left_leg.shank.Alpha();
	joints[29] = boxer->left_leg.thigh.Alpha();
	joints[30] = boxer->left_leg.thigh.Beta();
	joints[31] = boxer->left_leg.thigh.Gamma();

	joints[32] = boxer->body.abdomen.Alpha();
	joints[33] = boxer->body.abdomen.Beta();
	joints[34] = boxer->body.abdomen.Gamma();
	joints[35] = boxer->body.breast.Alpha();
	joints[36] = boxer->body.breast.Beta();
	joints[37] = boxer->body.breast.Gamma();

	joints[38] = boxer->head.neck.Alpha();
	joints[39] = boxer->head.neck.Beta();
	joints[40] = boxer->head.neck.Gamma();
	joints[41] = boxer->head.face.Alpha();
	joints[42] = boxer->head.face.Beta();
	joints[43] = boxer->head.face.Gamma();
}

void BFrame::Interpolate(BFrame *base, BFrame *next, int current_cut, int cut_number) {
	static int i;
	static float rate;
	rate = (float) current_cut / (float) cut_number;	
#if 1
	for (i = 0; i < 44; i++) {
		joints[i] = base->joints[i] + rate * (next->joints[i] - base->joints[i]);
	}
#else
	static float djoint;
	djoint = next->joints[0] - base->joints[0];
	joints[0] = base->joints[0] + rate * djoint;
	djoint = next->joints[1] - base->joints[1];
	joints[1] = base->joints[1] + rate * djoint;
	djoint = next->joints[2] - base->joints[2];
	joints[2] = base->joints[2] + rate * djoint;
	for (i = 3; i < 6; i++) {
		djoint = next->joints[i] - base->joints[i];
		if (djoint > 180.0) {
			djoint = djoint - 360.0;	
		}
		if (djoint < -180.0) {
			djoint = djoint + 360.0;	
		}
		joints[i] = base->joints[i] + rate * djoint;
	}
	for (i = 6; i < 44; i++) {
		djoint = next->joints[i] - base->joints[i];
		joints[i] = base->joints[i] + rate * djoint;
	}
#endif
}

void BFrame::LagrangeInterpolate(BFrame *pre, BFrame *base, BFrame *next, BFrame *nearnext, int current_cut, int pre_number, int base_number, int next_number) {
	static int i, x, x0, x1, x2, x3;
	static float r0, r1, r2, r3;
	x0 = 0;
	x1 = pre_number;
	x2 = x1 + base_number;
	x3 = x2 + next_number;
	x = x1 + current_cut;	 

	r0 = (float)((x - x1) * (x - x2) * (x - x3))/ (float)((x0 - x1) * (x0 - x2) * (x0 - x3));
	r1 = (float)((x - x0) * (x - x2) * (x - x3))/ (float)((x1 - x0) * (x1 - x2) * (x1 - x3));
	r2 = (float)((x - x0) * (x - x1) * (x - x3))/ (float)((x2 - x0) * (x2 - x1) * (x2 - x3));
	r3 = (float)((x - x0) * (x - x1) * (x - x2))/ (float)((x3 - x0) * (x3 - x1) * (x3 - x2));

//	wxLogError("r0 = %f r1 = %f r2 = %f r3 = %f", r0, r1, r2, r3);
	
	for (i = 0; i < 44; i++) {
		joints[i] =r0 * pre->joints[i] + r1 * base->joints[i] + r2 * next->joints[i] + r3 * nearnext->joints[i];
	}
}

BCoords::BCoords() {
}

BCoords::~BCoords() {
}

void BCoords::SetCoords() {
	rf_x = boxer->right_arm.fist.X();
	rf_y = boxer->right_arm.fist.Y();
	rf_z = boxer->right_arm.fist.Z();
	rf_a = boxer->right_arm.fist.AbsAlpha();
	rf_b = boxer->right_arm.fist.AbsBeta();
	rf_g = boxer->right_arm.fist.AbsGamma();

	lf_x = boxer->left_arm.fist.X();
	lf_y = boxer->left_arm.fist.Y();
	lf_z = boxer->left_arm.fist.Z();
	lf_a = boxer->left_arm.fist.AbsAlpha();
	lf_b = boxer->left_arm.fist.AbsBeta();
	lf_g = boxer->left_arm.fist.AbsGamma();

	rt_x = boxer->right_leg.toe.X();
	rt_y = boxer->right_leg.toe.Y();
	rt_z = boxer->right_leg.toe.Z();
	rt_a = boxer->right_leg.toe.AbsAlpha();
	rt_b = boxer->right_leg.toe.AbsBeta();
	rt_g = boxer->right_leg.toe.AbsGamma();

	lt_x = boxer->left_leg.toe.X();
	lt_y = boxer->left_leg.toe.Y();
	lt_z = boxer->left_leg.toe.Z();
	lt_a = boxer->left_leg.toe.AbsAlpha();
	lt_b = boxer->left_leg.toe.AbsBeta();
	lt_g = boxer->left_leg.toe.AbsGamma();

	w_x = boxer->body.waist.X();
	w_y = boxer->body.waist.Y();
	w_z = boxer->body.waist.Z();
	w_a = boxer->body.waist.AbsAlpha();
	w_b = boxer->body.waist.AbsBeta();
	w_g = boxer->body.waist.AbsGamma();

	h_x = boxer->head.face.X();
	h_y = boxer->head.face.Y();
	h_z = boxer->head.face.Z();
	h_a = boxer->head.face.AbsAlpha();
	h_b = boxer->head.face.AbsBeta();
	h_g = boxer->head.face.AbsGamma();
//	wxString str;
//	str.Printf("a = %f b = %f g = %f", boxer->head.face.AbsAlpha(), boxer->head.face.AbsBeta(), boxer->head.face.AbsGamma());
//	wxLogError(str);
}

float BCoords::CalcFunc() {
	boxer->CalcCoords();
	
	rfc = 0.0;
	rfa = 0.0;
	if (rfist_fix & 0x0001) {
		drf_x = boxer->right_arm.fist.X() - rf_x;
		rfc += drf_x * drf_x;
	}
	if (rfist_fix & 0x0002) {
		drf_y = boxer->right_arm.fist.Y() - rf_y;
		rfc += drf_y * drf_y;
	}
	if (rfist_fix & 0x0004) {
		drf_z = boxer->right_arm.fist.Z() - rf_z;
		rfc += drf_z * drf_z;
	}
	if (rfist_fix & 0x0010) {
		drf_a = boxer->right_arm.fist.AbsAlpha() - rf_a;
		rfa += drf_a * drf_a;
	}
	if (rfist_fix & 0x0020) {
		drf_b = boxer->right_arm.fist.AbsBeta()  - rf_b;
		rfa += drf_b * drf_b;	
	}
	if (rfist_fix & 0x0040) {
		drf_g = boxer->right_arm.fist.AbsGamma() - rf_g;
		rfa += drf_g * drf_g;	
	}
	rfc *= LengthUnit; 

	lfc = 0.0;
	lfa = 0.0;
	if (lfist_fix & 0x0001) {
		dlf_x = boxer->left_arm.fist.X() - lf_x;
        lfc += dlf_x * dlf_x;
	}
	if (lfist_fix & 0x0002) {
		dlf_y = boxer->left_arm.fist.Y() - lf_y;
        lfc += dlf_y * dlf_y;
	}
	if (lfist_fix & 0x0004) {
		dlf_z = boxer->left_arm.fist.Z() - lf_z;
 	    lfc += dlf_z * dlf_z;
	}
	if (lfist_fix & 0x0010) {
		dlf_a = boxer->left_arm.fist.AbsAlpha() - lf_a;
 	    lfa += dlf_a * dlf_a;
	}
	if (lfist_fix & 0x0020) {
		dlf_b = boxer->left_arm.fist.AbsBeta()  - lf_b;
 	    lfa += dlf_b * dlf_b;
	}		
	if (lfist_fix & 0x0040) {
		dlf_g = boxer->left_arm.fist.AbsGamma() - lf_g;
 	    lfa += dlf_g * dlf_g;
	}		
	lfc *= LengthUnit; 

	rtc = 0.0;
	rta = 0.0;
	if (rtoe_fix & 0x0001) {
		drt_x = boxer->right_leg.toe.X() - rt_x;
		rtc += drt_x * drt_x;
	}
	if (rtoe_fix & 0x0002) {
		drt_y = boxer->right_leg.toe.Y() - rt_y;
		rtc += drt_y * drt_y;
	}
	if (rtoe_fix & 0x0004) {
		drt_z = boxer->right_leg.toe.Z() - rt_z;
		rtc += drt_z * drt_z;
	}
	if (rtoe_fix & 0x0010) {
		drt_a = boxer->right_leg.toe.AbsAlpha() - rt_a;
		rta += drt_a * drt_a;
	}
	if (rtoe_fix & 0x0020) {
		drt_b = boxer->right_leg.toe.AbsBeta()  - rt_b;
		rta += drt_b * drt_b;	
	}
	if (rtoe_fix & 0x0040) {
		drt_g = boxer->right_leg.toe.AbsGamma() - rt_g;
		rta += drt_g * drt_g;	
	}
	rtc *= LengthUnit; 

	ltc = 0.0;
	lta = 0.0;
	if (ltoe_fix & 0x0001) {
		dlt_x = boxer->left_leg.toe.X() - lt_x;
		ltc += dlt_x * dlt_x;
	}
	if (ltoe_fix & 0x0002) {
		dlt_y = boxer->left_leg.toe.Y() - lt_y;
		ltc += dlt_y * dlt_y;
	}
	if (ltoe_fix & 0x0004) {
		dlt_z = boxer->left_leg.toe.Z() - lt_z;
		ltc += dlt_z * dlt_z;
	}
	if (ltoe_fix & 0x0010) {
		dlt_a = boxer->left_leg.toe.AbsAlpha() - lt_a;
		lta += dlt_a * dlt_a;
	}
	if (ltoe_fix & 0x0020) {
		dlt_b = boxer->left_leg.toe.AbsBeta()  - lt_b;
		lta += dlt_b * dlt_b;	
	}
	if (ltoe_fix & 0x0040) {
		dlt_g = boxer->left_leg.toe.AbsGamma() - lt_g;
		lta += dlt_g * dlt_g;	
	}
	ltc *= LengthUnit; 

	wc = 0.0;
	wa = 0.0;
	if (waist_fix & 0x0001) {
		dw_x = boxer->body.waist.X() - w_x;
		wc += dw_x * dw_x;
	}
	if (waist_fix & 0x0002) {
		dw_y = boxer->body.waist.Y() - w_y;
		wc += dw_y * dw_y;
	}
	if (waist_fix & 0x0004) {
		dw_z = boxer->body.waist.Z() - w_z;
		wc += dw_z * dw_z;
	}
	if (waist_fix & 0x0010) {
		dw_a = boxer->body.waist.AbsAlpha() - w_a;
		wa += dw_a * dw_a;
	}
	if (waist_fix & 0x0020) {
		dw_b = boxer->body.waist.AbsBeta()  - w_b;
		wa += dw_b * dw_b;	
	}
	if (waist_fix & 0x0040) {
		dw_g = boxer->body.waist.AbsGamma() - w_g;
		wa += dw_g * dw_g;	
	}
	wc *= LengthUnit; 

	hc = 0.0;
	ha = 0.0;
	if (head_fix & 0x0001) {
		dh_x = boxer->head.face.X() - h_x;
		hc += dh_x * dh_x;
	}
	if (head_fix & 0x0002) {
		dh_y = boxer->head.face.Y() - h_y;
		hc += dh_y * dh_y;
	}
	if (head_fix & 0x0004) {
		dh_z = boxer->head.face.Z() - h_z;
		hc += dh_z * dh_z;
	}
	if (head_fix & 0x0010) {
		dh_a = boxer->head.face.AbsAlpha() - h_a;
		ha += dh_a * dh_a;
	}
	if (head_fix & 0x0020) {
		dh_b = boxer->head.face.AbsBeta()  - h_b;
		ha += dh_b * dh_b;	
	}
	if (head_fix & 0x0040) {
		dh_g = boxer->head.face.AbsGamma() - h_g;
		ha += dh_g * dh_g;	
	}
	hc *= LengthUnit; 

	f = rfc + rfa + lfc + lfa + rtc + rta + ltc + lta + wc + wa + hc + ha;	

/*
#if 1
	FILE *fp = fopen("log.txt", "w");
	fprintf(fp,"rfc = %f\n", rfc);
	fprintf(fp,"rfa = %f\n", rfa);
	fprintf(fp,"lfc = %f\n", lfc);
	fprintf(fp,"lfa = %f\n", lfa);
	fprintf(fp,"rtc = %f\n", rtc);
	fprintf(fp,"rta = %f\n", rta);
	fprintf(fp,"ltc = %f\n", ltc);
	fprintf(fp,"lta = %f\n", lta);
	fprintf(fp,"hc = %f\n", hc);
	fprintf(fp,"ha = %f\n", ha);
	fprintf(fp,"wc = %f\n", wc);
	fprintf(fp,"wa = %f\n", wa);
	fprintf(fp,"f = %f\n", f);
	fprintf(fp,"---------\n");
	fclose(fp); 
#endif
*/ 

	if (boxer->X() < -2.0 || boxer->X() > 2.0) {
		f += VERY_BIG;	
	}
	if (boxer->Y() < -2.0 || boxer->Y() > 2.0) {
		f += VERY_BIG;	
	}
	if (boxer->Z() < -2.0 || boxer->Z() > 2.0) {
		f += VERY_BIG;	
	}


	if (boxer->right_arm.fist.Gamma() < -180.0 || boxer->right_arm.fist.Gamma() > 0.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_arm.forearm.Alpha() < 0.0 || boxer->right_arm.forearm.Alpha() > 150.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_arm.upperarm.Gamma() < -90.0 || boxer->right_arm.upperarm.Gamma() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_arm.shoulder.Beta() < -45.0 || boxer->right_arm.upperarm.Beta() > 225.0) {
		f += VERY_BIG;	
	}


	if (boxer->left_arm.fist.Gamma() < 0.0 || boxer->left_arm.fist.Gamma() > 180.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_arm.forearm.Alpha() < 0.0 || boxer->left_arm.forearm.Alpha() > 150.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_arm.upperarm.Gamma() < -90.0 || boxer->left_arm.upperarm.Gamma() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_arm.shoulder.Beta() < -225.0 || boxer->left_arm.upperarm.Beta() > 45.0) {
		f += VERY_BIG;	
	}


	if (boxer->right_leg.toe.Alpha() < -45.0 || boxer->right_leg.toe.Alpha() > 0.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.foot.Alpha() < -45.0 || boxer->right_leg.foot.Alpha() > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.foot.Beta() < -30.0 || boxer->right_leg.foot.Beta() > 30.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.foot.Gamma() < -30.0 || boxer->right_leg.foot.Gamma() > 30.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.shank.Alpha() < 0.0 || boxer->right_leg.shank.Alpha() > 150.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.thigh.Alpha() < 45.0 || boxer->right_leg.thigh.Alpha() > 315.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.thigh.Beta() < -30.0 || boxer->right_leg.thigh.Beta() > 150.0) {
		f += VERY_BIG;	
	}
	if (boxer->right_leg.thigh.Gamma() < 90.0 || boxer->right_leg.thigh.Gamma() > 270.0) {
		f += VERY_BIG;	
	}


	if (boxer->left_leg.toe.Alpha() < -45.0 || boxer->left_leg.toe.Alpha() > 0.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.foot.Alpha() < -45.0 || boxer->left_leg.foot.Alpha() > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.foot.Beta() < -30.0 || boxer->left_leg.foot.Beta() > 30.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.foot.Gamma() < -30.0 || boxer->left_leg.foot.Gamma() > 30.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.shank.Alpha() < 0.0 || boxer->left_leg.shank.Alpha() > 150.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.thigh.Alpha() < 45.0 || boxer->left_leg.thigh.Alpha() > 315.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.thigh.Beta() <-150.0 || boxer->left_leg.thigh.Beta()  > 30.0) {
		f += VERY_BIG;	
	}
	if (boxer->left_leg.thigh.Gamma() < 90.0 || boxer->left_leg.thigh.Gamma() > 270.0) {
		f += VERY_BIG;	
	}


	if (boxer->body.abdomen.Alpha() < -90.0 || boxer->body.abdomen.Alpha() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->body.abdomen.Beta()  < -90.0 || boxer->body.abdomen.Beta()  > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->body.abdomen.Gamma() < -90.0 || boxer->body.abdomen.Gamma() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->body.breast.Alpha() < -90.0 || boxer->body.breast.Alpha() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->body.breast.Beta()  < -90.0 || boxer->body.breast.Beta()  > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->body.breast.Gamma() < -90.0 || boxer->body.breast.Gamma() > 90.0) {
		f += VERY_BIG;	
	}
	

	if (boxer->head.neck.Alpha() < -45.0 || boxer->head.neck.Alpha() > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->head.neck.Beta()  < -45.0 || boxer->head.neck.Beta()  > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->head.neck.Gamma() < -90.0 || boxer->head.neck.Gamma() > 90.0) {
		f += VERY_BIG;	
	}
	if (boxer->head.face.Alpha() < -45.0 || boxer->head.face.Alpha() > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->head.face.Beta()  < -45.0 || boxer->head.face.Beta()  > 45.0) {
		f += VERY_BIG;	
	}
	if (boxer->head.face.Gamma() < -90.0 || boxer->head.face.Gamma() > 90.0) {
		f += VERY_BIG;	
	}

	return f;
}

bool BCoords::CalcAngles(Boxer *cboxer, int crfist_fix, int clfist_fix, int crtoe_fix, int cltoe_fix, int cwaist_fix, int chead_fix) {
	int i, j;
	boxer = cboxer;
	rfist_fix = crfist_fix;
	lfist_fix = clfist_fix;
	rtoe_fix = crtoe_fix;
	ltoe_fix = cltoe_fix;
	waist_fix = cwaist_fix;
	head_fix = chead_fix;

	SetCoords();
	sfb = CalcFunc();
	if (sfb < VERY_SMALL) {
		return true;	
	}
	
	dargs[0] = 0.001;
	dargs[1] = 0.001;
	dargs[2] = 0.001;
	for (i = 3; i < 44; i++) {
		dargs[i] = 1.0;
    }

	j = 0;
	fb = sfb;
	fb_pre = sfb + 1.0;
	change.Get(boxer);
	while (fb < fb_pre) {
		fb_pre = fb;
		for (i = 0; i < 44; i++) {
			changep = change;
			changep.joints[i] += dargs[i];
			changep.Set(boxer);
			fp = CalcFunc();
			changem = change;
			changem.joints[i] -= dargs[i];
			changem.Set(boxer);
			fm = CalcFunc();

			if (fp < fm) {
				if (fp < fb) {
					fb = fp;
  					change = changep;							
				}
			} else {
				if (fm < fb) {
					fb = fm;
  					change = changem;							
				}			
			}
		}
 		if (j++ > 1000) {
			break;
		}
	}

	if (fb < 1200.0) {
//		wxLogError("true j = %d fb = %f", j , fb);
		change.Set(boxer);
		return true;
	} else {
//		wxLogError("false j = %d fb = %f", j , fb);
		return false;
    }
}

BFrameList::BFrameList() {
	next = NULL;
	pre = NULL;
}

BFrameList::~BFrameList() {
}

/*
BFrameList& BFrameList::operator=(const BFrameList &cbframelist) {
	Copy(cbframelist);
	this->next = cbframelist.next;
	this->pre = cbframelist.pre;
	this->cut_number = cbframelist.cut_number;
	return *this;		
}
*/

void BFrameList::From(const BFrame &cbframe, int number) {
	Copy(cbframe);
	this->next = NULL;
	this->pre = NULL;
	this->cut_number = number;
}
