#ifndef BFRAME_H
#define BFRAME_H

#include "myglheader.h"

#define CURRENT_FRAME 65536
#define CURRENT_MOTION 65536
#define MAX_MOTIONS 255

class Boxer;
class BCooords;
class BFrame;

class BFrame {
public:
	BFrame();
	~BFrame();
	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Interpolate(BFrame *base, BFrame *next, int current_cut, int cut_number);
	void LagrangeInterpolate(BFrame *pre, BFrame *base, BFrame *next, BFrame *nearnext, int current_cut, int pre_number, int base_number, int next_number);
//	BFrame& operator=(const BFrame &cbframe); 
 	void Copy(const BFrame &cbframe); 
 	float& operator[](int number) {return joints[number];}
protected:
	float joints[44];
	friend class BCoords;
};

class BFrameList : public BFrame {
public:
	BFrameList();
	~BFrameList();	
	void SetPre(BFrameList *cpre) {pre = cpre;}
	void SetNext(BFrameList *cnext) {next = cnext;}
	BFrameList& GetPre() const {return *pre;}
	BFrameList& GetNext() const {return *next;}
//	BFrameList& operator=(const BFrameList &cbframe); 
	void From(const BFrame &cbframe, int number); 
	void SetNumber(int cnumber) {cut_number = cnumber;}
	int GetNumber() const {return cut_number;}
	void ChangeNumber(float change) {cut_number = (int)(cut_number * change);}
private:
	int cut_number;
	BFrameList *pre;
	BFrameList *next;	
};

class BCoords {
public:
	BCoords();
	~BCoords();
	bool CalcAngles(Boxer *cboxer, int crfist_fix, int clfist_fix, int crtoe_fix, int cltoe_fix, int cwaist_fix, int chead_fix);
private:
	Boxer *boxer;
	void SetCoords();
	float CalcFunc();
	float dargs[44];
	BFrame change, changep, changem;
	int rfist_fix, lfist_fix, rtoe_fix, ltoe_fix, head_fix, waist_fix;
	float rf_x, rf_y, rf_z;
	float rf_a, rf_b, rf_g;
	float lf_x, lf_y, lf_z;
	float lf_a, lf_b, lf_g;
	
	float rt_x, rt_y, rt_z;
	float rt_a, rt_b, rt_g;
	float lt_x, lt_y, lt_z;
	float lt_a, lt_b, lt_g;

	float h_x, h_y, h_z;
	float h_a, h_b, h_g;
	float w_x, w_y, w_z;
	float w_a, w_b, w_g;

	float drf_x, drf_y, drf_z, rfc;
	float drf_a, drf_b, drf_g, rfa;
	float dlf_x, dlf_y, dlf_z, lfc;
	float dlf_a, dlf_b, dlf_g, lfa;
	
	float drt_x, drt_y, drt_z, rtc;
	float drt_a, drt_b, drt_g, rta;
	float dlt_x, dlt_y, dlt_z, ltc;
	float dlt_a, dlt_b, dlt_g, lta;

	float dh_x, dh_y, dh_z, hc;
	float dh_a, dh_b, dh_g, ha;
	float dw_x, dw_y, dw_z, wc;
	float dw_a, dw_b, dw_g, wa;
	float f, fb, sfb, fm, fp, fb_pre;
};

#endif // BFRAME_H
