#ifndef BOXERHANDLER_H
#define BOXERHANDLER_H

#include "csdl.h"
#include "creader.h"
#include "boxer.h"
#include "bframe.h"
#include "motionhandler.h"
#include "matchhandler.h"
#include "roundinfo.h"
#include "replayinfo.h"
#include "think.h"
#include "boxerdata.h"
#include "storyboxer.h"

#define CHANGE_SPECIFICATIONS 1
#define NAME_LENGTH 12

enum {
	NO_DAMAGE,
	SMALL_DAMAGE,	
	FACE_DAMAGE,	
	BODY_DAMAGE	
};

enum {
	SE_GONG,
	SE_KO,
	SE_SWING1,
	SE_SWING2,
	SE_DASH,
	SE_HIT_JAW,
	SE_HIT_FACE,
	SE_HIT_BODY,
	SE_BLOCK,
	SE_BOX,
	SE_BREAK,
	SE_DOWN,	
	MAX_SOUND_EFFECTS,
	NO_SOUND
};

enum {
	RHYTHM,
	MOVE,
	R_OFFENSE,
	L_OFFENSE,	
	DEFENSE,
	PULL,
	DAMAGED,	
	DOWNING,
	DOWN,
	RESTORE,
	WAIT,
	CLINCHING,
	READY
};

enum {
	DIR_NO,
	DIR_FORWARD,
	DIR_BACK,
	DIR_RIGHT,
	DIR_LEFT,
	DIR_NO_RELEASED = 0xf000
};

enum {
	NO_KEY      = 0x0000,
	FORWARD_KEY = 0x0001,
	BACK_KEY    = 0x0002,
	RIGHT_KEY   = 0x0004,
	LEFT_KEY    = 0x0008,
	LEFT_OFFENSE_KEY  = 0x0010,
	RIGHT_OFFENSE_KEY = 0x0020,
	DEFENSE_KEY       = 0x0040,
	INVALID_KEY       = 0x0080
};	

enum {
	FULL_CONTROL = 0,
	SEMI_AUTO    = 1,
	FULL_AUTO    = 2
};

enum {
	FILE_BODY     = 0x0001,
	FILE_MATERIAL = 0x0002,
	FILE_COMBO    = 0x0004,
	FILE_MOTION   = 0x0008,
	FILE_ASSIGN   = 0x0010,
	FILE_RANGE    = 0x0020,
	FILE_THINK    = 0x0040,
	FILE_OTHER    = 0x0080
};

#define FILE_SOTRY (FILE_BODY | FILE_COMBO | FILE_MOTION | FILE_ASSIGN | FILE_RANGE)
#define FILE_ALL   (FILE_STORY | FILE_MATERIAL | FILE_THINK | FILE_OTHER)

class BoxerHandler {
public:
	BoxerHandler();
	~BoxerHandler();
	bool Init(Boxer &boxer, MotionHandler &motion, MatchHandler &match);
	void SetInit(Boxer &boxer, MotionHandler &motion, MatchHandler &match);
	void Free();
	bool WasInit();

	void SaveInfo(int tr, Boxer &boxer, MotionHandler &motion); 
	void SaveIntervalInfo(Boxer &boxer, MotionHandler &motion); 
	void SaveEndInfo(Boxer &boxer, MotionHandler &motion); 
	void LoadInfo(int tr, Boxer &boxer, MotionHandler &motion);
	void LoadIntervalInfo(Boxer &boxer, MotionHandler &motion);
	void LoadEndInfo(Boxer &boxer, MotionHandler &motion);
	bool HasInfo(int tr) const;
	bool ExistIntervalInfo() const;
	bool ExistEndInfo() const;

	void SaveReplay(int latest_round, SDL_RWops *rwops);
	void LoadReplay(int latest_round, SDL_RWops *rwops);

	void SetRound(int cround);
	void SaveKeyStates(int tc);
	void LoadKeyStates(int tc);
	void SaveThrowInState(int tc, int throwin_state);
	int LoadThrowInState(int tc);
	Uint8 GetKeyState() const {return (Uint8)key_state;}
	void SetKeyState(Uint8 uc) {key_state = (int)uc;}
	bool HasKeyStates(int tc) const;
	int LastKeyStates();
	void SetJudges();
	void GetJudges();
	void SumJudge();

	void BoxerHandling(Boxer &boxer, MotionHandler &motion);
	void RhythmHandling(Boxer &boxer, MotionHandler &motion);
	void PullHandling(Boxer &boxer, MotionHandler &motion);
	void MoveHandling(Boxer &boxer, MotionHandler &motion);
	void Move(Boxer &boxer);
	void SetDistance(float cdistance) {distance = cdistance;}
	float Distance() const {return distance;}
	void SetAdjustDistance(float cadjust_distance) {adjust_distance = cadjust_distance;}
	float AdjustDistance() const {return adjust_distance;}
	int WhichState(int caction);
	void PlaySwingSE(int cstate);
	void InitOffense() {
		absorb = 1.0;
	    hit_first = 1;
		SwingInc();
	}

	void SetStateRhythm() {state = RHYTHM;}
	void SetStatePull();	
	void SetStateRestore() {state = RESTORE;}
	void SetStateClinch(MotionHandler &motin);
	void SetNoDamage() {hit_state = NO_DAMAGE;}	
	void SetSmallDamage() {hit_state = SMALL_DAMAGE;}	

	void CheckHitPunch(Boxer &m_boxer, MotionHandler &m_motion, Boxer &o_boxer, BoxerHandler &o_handler);
	void RestoreCyanosis(Boxer &boxer);
	void CalcCyanosis(Boxer &boxer);
	void CyanosisColor(float *r, float *g, float *b);
	void RestoreDamage();
	void RestoreDamageInterval(MatchHandler &match);
	void SetDamage(Boxer &boxer, MotionHandler &motion);			
	void CalcDamage();
	bool IsMinimamDamage();
	bool IsFirstHit() const {return hit_first;}
	void Blocked() {hit_first = 0;}
	bool CanBreathe() const {
		return (state != R_OFFENSE && state != L_OFFENSE && state != PULL && state != DAMAGED && state != DOWNING);			
	}	
	bool CanOffense();
	bool CanBlock() const {
		return (state != R_OFFENSE && state != L_OFFENSE && state != PULL && state != DAMAGED && state != DOWNING);			
	}
	bool CanClinch() {
		return (distance <= 0.76 && adjust_distance <= 0.75);	
	}
	void GetComStates(MatchHandler &match, Boxer& m_boxer, BoxerHandler& o_handler);		
	void Reverse();

	int LoadBoxer(Boxer &boxer, MotionHandler &motion, const char *filename);
	int LoadBoxerBody(Boxer &boxer, CReader &breader);
	int LoadBoxerMaterial(Boxer &boxer, CReader &breader);
	int LoadBoxerMotion(MotionHandler &motion, CReader &breader);
	int LoadBoxerCombo(MotionHandler &motion, CReader &breader);
	int LoadBoxerAssign(MotionHandler &motion, CReader &breader);
	int LoadBoxerRange(Boxer &boxer, CReader &breader);
	int LoadBoxerThink(CReader &breader);
	int LoadBoxerOther(Boxer &boxer, CReader &breader);

	int MakeBoxerSourceFile(Boxer &boxer, MotionHandler &motion, const char *filename, int which_data = 0);
	int SaveBinBoxerFile(Boxer &boxer, MotionHandler &motion, const char *filename);
	int SaveBinBoxer(Boxer &boxer, MotionHandler &motion, SDL_RWops *rwops);
	int SaveBinBoxerBodySetFile(Boxer &boxer, const char *filename);
	int SaveBinBoxerBodySet(Boxer &boxer, SDL_RWops *rwops);
	int SaveBinBoxerMotionSetFile(MotionHandler &motion, const char *filename);
	int SaveBinBoxerMotionSet(MotionHandler &motion, SDL_RWops *rwops);
	int SaveBinBoxerThinkFile(const char *filename);
	int SaveBinBoxerBody(Boxer &boxer, SDL_RWops *rwops);
	int SaveBinBoxerMaterial(Boxer &boxer, SDL_RWops *rwops);
	int SaveBinBoxerMotion(MotionHandler &motion, SDL_RWops *rwops);
	int SaveBinBoxerCombo(MotionHandler &motion, SDL_RWops *rwops);
	int SaveBinBoxerAssign(MotionHandler &motion, SDL_RWops *rwops);
	int SaveBinBoxerRange(Boxer &boxer, SDL_RWops *rwops);
	int SaveBinBoxerThink(SDL_RWops *rwops);
	int SaveBinBoxerOther(SDL_RWops *rwops);
	int CalcDataSize(MotionHandler &motion);

	int LoadBinBoxerFile(Boxer &boxer, MotionHandler &motion, const char *filename);
	int LoadBinBoxer(Boxer &boxer, MotionHandler &motion, SDL_RWops *rwops);
	int LoadBinBoxerBodySet(Boxer &boxer, SDL_RWops *rwops);
	int LoadBinBoxerMotionSet(MotionHandler &motion, SDL_RWops *rwops);
	int LoadBinBoxerBody(Boxer &boxer, SDL_RWops *rwops);
	int LoadBinBoxerMaterial(Boxer &boxer, SDL_RWops *rwops);
	int LoadBinBoxerMotion(MotionHandler &motion, SDL_RWops *rwops);
	int LoadBinBoxerCombo(MotionHandler &motion, SDL_RWops *rwops);
	int LoadBinBoxerAssign(MotionHandler &motion, SDL_RWops *rwops);
	int LoadBinBoxerRange(Boxer &boxer, SDL_RWops *rwops);
	int LoadBinBoxerThink(SDL_RWops *rwops);
	int LoadBinBoxerOther(Boxer &boxer, SDL_RWops *rwops);

	int LoadDefaultThink();
	int SaveDefaultThink();	

	void PrintBoxer(Boxer &boxer, MotionHandler &motion);
	void PrintBody(Boxer &boxer);
	void PrintCombo(MotionHandler &motion);
	void PrintAssign(MotionHandler &motion);
	void PrintRange(Boxer &boxer);
	void PrintOther();

	void LoadStoryBoxer(Boxer &boxer, MotionHandler &motion, StoryBoxer &sb);
	void SetStoryBoxer(Boxer &boxer, MotionHandler &motion, StoryBoxer &sb);
	void SetBoxerColor(Boxer &boxer, int c1, int c2, int c3, int c4);
	void SetGlovesColor(Boxer &boxer, int cn);
	void SetShoesColor(Boxer &boxer, int cn);
	void SetTrunksColor(Boxer &boxer, int cn);
	void SetBandageColor(Boxer &boxer, int cn);
	void SetSkinColor(Boxer &boxer, int cn);
	void SetHairColor(Boxer &boxer, int cn);
	void SetBoxerFace(Boxer &boxer, int c1, int c2, int c3, int c4, int c5);

	void SetBoxerBody(Boxer &boxer, int c);
	void SetBoxerMotion(MotionHandler &motion, int c);
	void SetBoxerCombo(MotionHandler &motion, int c);
	void SetBoxerThink(int c);
	void SetBoxerManip(StoryBoxer &sb);
	void CleanManip();

	void SetBoxerName(StoryBoxer &sb); 
	void SetCustomName(char *name, int number);
	void SetRingName(int c);
	void SetFamilyNameJapan(int c);
	void SetFamilyNameForeign(int c);

	void InitKey() {key_state = NO_KEY;}
	void InvalidKey() {key_state = INVALID_KEY;}
	void SetLeftOffense(int cleft_offense) {left_offense = cleft_offense;}
	void SetRightOffense(int cright_offense) {right_offense = cright_offense;}
	void SetDefense(int cdefense) {defense = cdefense;}
	void SetForward(int cforward) {forward = cforward;}
	void SetBack(int cback) {back = cback;}
	void SetRight(int cright) {right = cright;}
	void SetLeft(int cleft) {left = cleft;}

	int LeftOffenseKey() const {return left_offense;}
	int RightOffenseKey() const {return right_offense;}
	int DefenseKey() const {return defense;}
	int ForwardKey() const {return forward;}
	int BackKey() const {return back;}
	int RightKey() const {return right;}
	int LeftKey() const {return left;}
	void DeleteOffenseKey() {
		key_state &= ~LEFT_OFFENSE_KEY;
		key_state &= ~RIGHT_OFFENSE_KEY;			
	}

	void Forward() {key_state |= FORWARD_KEY;}	
	void Back()    {key_state |= BACK_KEY;}	
	void Right()   {key_state |= RIGHT_KEY;}	
	void Left()    {key_state |= LEFT_KEY;}	
	void LeftOffense()  {key_state |= LEFT_OFFENSE_KEY;}	
	void RightOffense() {key_state |= RIGHT_OFFENSE_KEY;}	
	void Defense()     {key_state |= DEFENSE_KEY;}	

	bool IsKeyValid() const {
		return key_state != INVALID_KEY;	
	}
	bool IsNoKey() const {
		return key_state == NO_KEY;	
	}
	bool IsForward() const {
		return (bool)(key_state & FORWARD_KEY);			
	}	
	bool IsBack() const {
		return (bool)(key_state & BACK_KEY);			
	}	
	bool IsRight() const {
		return (bool)(key_state & RIGHT_KEY);			
	}	
	bool IsLeft() const {
		return (bool)(key_state & LEFT_KEY);			
	}	
	bool IsRightForward() const {
		return IsRight() && IsForward();			
	}	
	bool IsRightBack() const {
		return IsRight() && IsBack();			
	}	
	bool IsLeftForward() const {
		return IsLeft() && IsForward();			
	}	
	bool IsLeftBack() const {
		return IsLeft() && IsBack();			
	}	
	bool IsAnyMove() const {
		return IsForward() || IsBack() || IsRight() || IsLeft();	
	}	

	bool IsLeftOffense() const {
		return (bool)(key_state & LEFT_OFFENSE_KEY);			
	}	
	bool IsRightOffense() const {
		return (bool)(key_state & RIGHT_OFFENSE_KEY);			
	}	
	bool IsDefense() const {
		return (bool)(key_state & DEFENSE_KEY);			
	}
	bool IsDeadly() const {
		return IsLeftOffense() && IsRightOffense();		
	}
	bool IsClinch() const {
		return IsDefense() && (IsLeftOffense() || IsRightOffense());		
	}

	bool IsAnyAction() const {
		return IsLeftOffense() || IsRightOffense() || IsDefense();	
	}
	bool IsAny() const {
		return IsAnyMove() || IsAnyAction();	
	}
	void SetMode(int cmode) {mode = cmode;}
	int WhichMode() const {return mode;}
	void NextMode() {
		if (++mode > 2) {
			mode = 0;						
		}
	}
	bool IsFullControl() const {return (mode == FULL_CONTROL);}
	bool IsSemiAuto() const {return (mode == SEMI_AUTO);}
	bool IsFullAuto() const {return (mode == FULL_AUTO);}

	bool IsHitFace() const {
		return (bool)(hit_parts & HIT_FACE);			
	}	
	bool IsHitJaw() const {
		return (bool)(hit_parts & HIT_JAW);			
	}	
	bool IsHitBody() const {
		return (bool)(hit_parts & HIT_BODY);			
	}	
	bool IsHitLiver() const {
		return (bool)(hit_parts & HIT_LIVER);			
	}	
	bool IsHitRShoul() const {
		return (bool)(hit_parts & HIT_RSHOUL);			
	}	
	bool IsHitRUarm() const {
		return (bool)(hit_parts & HIT_RUARM);			
	}	
	bool IsHitRFarm() const {
		return (bool)(hit_parts & HIT_RFARM);			
	}	
	bool IsHitRFist() const {
		return (bool)(hit_parts & HIT_RFIST);			
	}	
	bool IsHitLShoul() const {
		return (bool)(hit_parts & HIT_LSHOUL);			
	}	
	bool IsHitLUarm() const {
		return (bool)(hit_parts & HIT_LUARM);			
	}	
	bool IsHitLFarm() const {
		return (bool)(hit_parts & HIT_LFARM);			
	}	
	bool IsHitLFist() const {
		return (bool)(hit_parts & HIT_LFIST);			
	}	

	bool IsRhythmState() const {
		return (state == RHYTHM);			
	}	
	bool IsOffenseState() const {
		return (state == R_OFFENSE || state == L_OFFENSE);			
	}	
	bool IsRightOffenseState() const {
		return (state == R_OFFENSE);			
	}	
	bool IsLeftOffenseState() const {
		return (state == L_OFFENSE);			
	}	
	bool IsDefenseState() const {
		return (state == DEFENSE);			
	}	
	bool IsPullState() const {
		return (state == PULL);			
	}	
	bool IsMoveState() const {
		return (state == MOVE);			
	}	
	bool IsMoving() const {
		return (state == MOVE);			
	}	
	bool IsDowning() const {
		return (state == DOWNING);			
	}	
	bool IsDownState() const {
		return (state == DOWN);			
	}	
	bool IsWaitState() const {
		return (state == WAIT);			
	}	
	bool IsClinching() const {
		return (state == CLINCHING);			
	}	

	bool IsDamageFace() const {
		return (hit_state == FACE_DAMAGE);			
	}
	bool IsDamageBody() const {
		return (hit_state == BODY_DAMAGE);			
	}	
	bool IsDamaged() const {
		return (hit_state != NO_DAMAGE);			
	}	
	bool IsCounting() const {
		return (state == DOWN || state == RESTORE || state == WAIT);			
	}	

	float CyanosisRate() const {return cyanosis_rate;}
	void SetTraumRate(int i) {traum_rate = i;}
	void SetTraumDamage() {traum_damage = (float) traum_rate / 100.0 * max_damage;}
	float TraumDamage() const {return traum_damage;}
	float BodyDamage() const {return body_damage;}
	float FaceDamage() const {return face_damage;}
	float BrainDamage() const {return brain_damage;}
	float Damage() const {return damage;}
	float DamageRate() const {return damage/max_damage;}
	float SwingSpeed() const {return swing_speed;}
	float MoveSpeed() const {return move_speed;}
	float Stamina() const {return stamina;}
	int StaminaPercent() const {return (int)(100.0 * StaminaRate());}
	float StaminaRate() const {return stamina/max_stamina;}
	int State() const {return state;}
	int KeyState() const {return key_state;}

	bool IsSound() const {return select_sound != NO_SOUND;}
	int SelectSound() const {return select_sound;}

	void InitCount() {count = 0;}
	void CountInc() {count++;}
	int Count() const {return (count/50);}
	char *CountString() {	
	 	static char *count_str[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"};
		return count_str[Count()];
	}

	void InitSwing() {swing = 0;}
	void SwingInc() {swing++;}
	int Swing() const {return swing;}
	int SwingInfo() const {return info[round].swing;}
	int HitPercent(int o_hit) const {
		if (SwingInfo() == 0) {
		 	return 0;
		} else {
			return o_hit * 100 / SwingInfo();		
		}
	}	

	void InitDown() {down = 0;}
	void DownInc() {down++;}
	void DownDec() {down--;}
	int Down() const {return down;}
	int DownInfo() const {return info[round].down;}

	void InitHit() {hit = 0;}
	void HitInc() {hit++;}
	int Hit() const {return hit;}
	int HitInfo() const {return info[round].hit;}

	void InitCritical() {critical = 0;}
	void CriticalInc() {critical++;}
	int Critical() const {return critical;}
	int CriticalInfo() const {return info[round].critical;}

	void InitRoundDamage() {round_damage = 0;}
	void RoundDamageInc(float inc_damage) {round_damage+=(int)inc_damage;}
	int RoundDamage() const {return round_damage;}
	int RoundDamageInfo() const {return info[round].round_damage;}
	
	void InitRound() {
		InitCount();
		InitSwing();
		InitDown();
		InitHit();
		InitCritical();
		InitRoundDamage();
	}

	void InitScore() {score = 0;}
	void SetScore(int rnd) {score = judge[rnd];}
	int Score() const {return score;}
	int ScoreInfo() const {return info[round].score;}

	void InitTotalScore() {total_score = 0;}
	void TotalScoreInc() {total_score+=score;}
	int TotalScore() const {return total_score;}

	void SetJudge(int i, int score) {judge[i] = score;}
	int Judge(int i) const {return judge[i];}
	int JudgeInfo(int i) const {return info[round].judge[i];}
	int JudgeInfo(int j, int i) const {return info[j].judge[i];}
	int TotalJudge(int i) const {return total_judge[i];}

	char* NickNameStrJp() {return nickname_jp;}
	char* NickNameStrEn() {return nickname_en;}
	char* FamilyNameStrJp() {return familyname_jp;}
	char* FamilyNameStrEn() {return familyname_en;}

private:
	int key_state;
	int left_offense;
	int right_offense;
	int defense;
	int forward;
	int back;
	int right;
	int left;
/*	int right_forward;
	int right_back;
	int left_foward;
	int left_back;  */
	int mode;
		
	float cyanosis;	
	float cyanosis_restore;	
	float max_cyanosis;	
	float cyanosis_rate;	
	float swing_speed;	

	float absorb;
	float damage;
	int traum_rate;
	float traum_damage;
	float brain_damage;
	float face_damage;
	float body_damage;		
	float damage_restore;	
	float max_damage;	
	float damage_rate;	
	float hit_damage;	
	float move_speed;	
	float distance;
  	float adjust_distance;

	float stamina;
	float max_stamina;
	float restore_rate;
	
	int state;
	int next_state;
	int moved;
	int dashing;
	int direct;
	int action;
	int next_action;
	int combination;
	int deadly;
	int hit_state;
	int hit_parts;
	int hit_first;

	int count;
	int swing;
	int down;
	int hit;
	int critical;
	int round_damage;

	int score;
	int total_score;
	int judge[3];
	int total_judge[3];

	RoundInfo *info;
	int round;
	int max_round;
	int select_sound;

    Think *think;
	int max_think;

    Think *dthink;
	int max_dthink;

	char nickname_jp[16], nickname_en[16];	
	char familyname_jp[16], familyname_en[16];	
	float punch_manip;
	float speed_manip;
	float stamina_manip;
	float damage_restore_manip;
	float max_damage_manip;
	float cyanosis_restore_manip;
	float max_cyanosis_manip;

	float defense_shoulder_absorb;
	float defense_upperarm_absorb;
	float defense_forearm_absorb;
	float defense_fist_absorb;

	float normal_shoulder_absorb;
	float normal_upperarm_absorb;
	float normal_forearm_absorb;
	float normal_fist_absorb;

	float offense_shoulder_absorb;
	float offense_upperarm_absorb;
	float offense_forearm_absorb;
	float offense_fist_absorb;

	float body_strength;
	float liver_strength;
	float face_strength;
	float jaw_strength;		

	friend class HandlerInfo;
};                      

#endif // BOXERHANDLER_H
