#include "boxerhandler.h"

BoxerHandler::BoxerHandler() {
	info = NULL;
	think = NULL;
	dthink = NULL;
}

BoxerHandler::~BoxerHandler() {
	Free();
	if (think != NULL) {
		delete [] think;		
	}
	if (dthink != NULL) {
		delete [] dthink;		
	}
}

bool BoxerHandler::Init(Boxer &boxer, MotionHandler &motion, MatchHandler&match) {
	SetInit(boxer, motion, match);

	if (WasInit()) {
		Free();	
	}

	int i;
	max_round = match.MaxRound()+1;
	info = new RoundInfo[max_round];
	for (i = 0; i < max_round; i++) {
		info[i].Init(match, motion);
 	}       
	
	return WasInit();
}

void BoxerHandler::SetInit(Boxer &boxer, MotionHandler &motion, MatchHandler&match) {
	float weight;
	weight = boxer.Weight();
	weight *= 1000.0;
	if (weight > 100.0) {
		weight = 100.0;	
	}

	state = RHYTHM;
	next_state = RHYTHM;
	moved = 0;
	dashing = 0;
	direct = DIR_NO;
	action = 0;
	next_action = 0;
	combination = 0;
	deadly = 0;
	hit_parts = HIT_NOTHING;
	hit_first = 0;
	hit_state = NO_DAMAGE;

	count = 0;
	swing = 0;
	down = 0;
	hit = 0;
	critical = 0;
	round_damage = 0;
	score = 0;
	total_score = 0;
	
	cyanosis = 0.0;
	max_cyanosis = weight * 2.5 * max_cyanosis_manip;
	cyanosis_restore = max_cyanosis/2.0/50.0 * cyanosis_restore_manip;
	swing_speed = 1.0;	

	damage = 0.0;
	traum_damage = 0.0;
	brain_damage = 0.0;
	face_damage = 0.0;
	body_damage = 0.0;		
	max_damage = weight*10.0 * max_damage_manip;
	damage_restore = max_damage/90.0/50.0 * damage_restore_manip;

	move_speed = 1.0;	
	absorb = 1.0;
	adjust_distance = 0.75;

	max_stamina = weight*100.0*6.0 * stamina_manip / match.Denominator();
	stamina = max_stamina;
	restore_rate = 1.0;

	key_state = 0x0000;

/*	right_forward = 0;
	right_back = 0;
	left_foward = 0;
	left_back = 0; */
}

void BoxerHandler::Free() {
	if(WasInit()) {
		delete [] info;
		info = NULL;		
	}	
}

bool BoxerHandler::WasInit() {
	if (info == NULL) {
		return false;		
	}	
	return true;		
}

void BoxerHandler::SaveInfo(int tr, Boxer &boxer, MotionHandler &motion) {
#ifdef DEBUG
	if (tr >= info[round].max_replay) {
		emes("oxerHandler::SaveInfo -> Overflow");		
	}
#endif 
	if (info[round].replay[tr].WasInit()) {
		info[round].replay[tr].GetInfo(*this, boxer, motion);
//		fprintf(stderr, "info[%d][%d] was gotten.\n", round, tr);
	} else {
		fprintf(stderr, "oxerHandler::SaveInfo -> info[%d][%d] wasn't initialized.\n", round, tr);
	}
}

void BoxerHandler::SaveIntervalInfo(Boxer &boxer, MotionHandler &motion) {
	SaveInfo(info[round].max_replay-2, boxer, motion);
}

void BoxerHandler::SaveEndInfo(Boxer &boxer, MotionHandler &motion) {
	SaveInfo(info[round].max_replay-1, boxer, motion);
}

void BoxerHandler::LoadInfo(int tr, Boxer &boxer, MotionHandler &motion) {
#ifdef DEBUG
	if (tr >= info[round].max_replay) {
		emes("BoxerHandler::LoadInfo -> Overflow");		
	}
#endif 
	if (info[round].replay[tr].HasInfo()) {
		info[round].replay[tr].SetInfo(*this, boxer, motion);
//		fprintf(stderr, "BoxerHandler::LoadInfo -> info[%d][%d] was set.\n", round, tr);
	}  else {
		fprintf(stderr, "BoxerHandler::LoadInfo -> info[%d][%d] doesn't have data.\n", round, tr);
	}
}

void BoxerHandler::LoadIntervalInfo(Boxer &boxer, MotionHandler &motion) {
	LoadInfo(info[round].max_replay-2, boxer, motion);
}

void BoxerHandler::LoadEndInfo(Boxer &boxer, MotionHandler &motion) {
	LoadInfo(info[round].max_replay-1, boxer, motion);
}

bool BoxerHandler::HasInfo(int tr) const {
#ifdef DEBUG
	if (tr >= info[round].max_replay) {
		emes("BoxerHandler::HasInfo -> Overflow");		
	}
#endif 
	return info[round].replay[tr].HasInfo();
}

bool BoxerHandler::ExistIntervalInfo() const {
	return HasInfo(info[round].max_replay-2);
}

bool BoxerHandler::ExistEndInfo() const {
	return HasInfo(info[round].max_replay-1);
}

void BoxerHandler::SetRound(int cround) {
	round = cround -1;
	if (round < 0) {
		round = 0;	
	    emes("BoxerHandler::SetRoun -> Round must be larger than 0.");
	}
	if (round >= max_round) {
		round = max_round - 1;	
	    emes("BoxerHandler::SetRoun -> Round is too big.");
	}
}

void BoxerHandler::SaveKeyStates(int tc) {
#ifdef DEBUG
	if (tc >= info[round].max_save_key) {
		emes("BoxerHandler::SaveKeyStates -> Overflow");		
	}
#endif 
	info[round].save_key[tc] = key_state;
}

void BoxerHandler::LoadKeyStates(int tc) {
#ifdef DEBUG
	if (tc >= info[round].max_save_key) {
		emes("BoxerHandler::LoadKeyStates -> Overflow");		
	}
#endif 
	key_state = info[round].save_key[tc];
}

bool BoxerHandler::HasKeyStates(int tc) const {
#ifdef DEBUG
	if (tc >= info[round].max_save_key) {
		emes("BoxerHandler::HasKeyStates -> Overflow");		
	}
#endif 
	return info[round].save_key[tc] != INVALID_KEY;
}

void BoxerHandler::SaveThrowInState(int tc, int throwin_state) {
#ifdef DEBUG
	if (tc+1 >= info[round].max_save_key) {
		emes("BoxerHandler::SaveThrowInState -> Overflow");		
	}
#endif 
	info[round].save_key[tc+1] = (Uint8) throwin_state;
}

int BoxerHandler::LoadThrowInState(int tc) {
#ifdef DEBUG
	if (tc >= info[round].max_save_key) {
		emes("BoxerHandler::LoadThrowInState -> Overflow");		
	}
#endif 
	return (int)info[round].save_key[tc];
}

int BoxerHandler::LastKeyStates() {
	int i;
	for (i = 0; i < info[round].max_save_key; i++) {
		if (info[round].save_key[i] == INVALID_KEY) {
			return i - 1;		
		}	
	}
	return info[round].max_save_key;
}

void BoxerHandler::SumJudge() {
	int i, j;
	for (j = 0; j < 3; j++) {
		total_judge[j] = 0;
	}	
	for (i = 0; i < max_round - 1; i++) {
		for (j = 0; j < 3; j++) {
			total_judge[j] += info[i].judge[j];
		}	
	}
}

void BoxerHandler::SetJudges() {
	info[round].swing = swing;
	info[round].down = down;
	info[round].hit = hit;
	info[round].critical = critical;
	info[round].round_damage = round_damage;
	info[round].judge[0] = judge[0];
	info[round].judge[1] = judge[1];
	info[round].judge[2] = judge[2];
	info[round].score = score;
}

void BoxerHandler::GetJudges() {
	swing = info[round].swing;
	down = info[round].down;
	hit = info[round].hit;
	critical = info[round].critical;
	round_damage = info[round].round_damage;
	judge[0] = info[round].judge[0];
	judge[1] = info[round].judge[1];
	judge[2] = info[round].judge[2];
	score = info[round].score;
}

void BoxerHandler::SaveReplay(int latest_round, SDL_RWops *rwops) {
	int i;
	for (i = 0; i < latest_round; i++) {
		info[i].SaveInfo(rwops);
 	}
}

void BoxerHandler::LoadReplay(int latest_round, SDL_RWops *rwops) {
	int i;
	for (i = 0; i < latest_round; i++) {
		info[i].LoadInfo(rwops);
 	}
}

