#include "boxerhandler.h"

void BoxerHandler::BoxerHandling(Boxer &boxer, MotionHandler &motion) {
	select_sound = NO_SOUND;
	switch (state) {
		case RHYTHM: /* Y */
			motion.NextRhythm();
			motion.MotionRhythm().Set(&boxer);
			RhythmHandling(boxer, motion);
		break;
		case R_OFFENSE: /* U */
		case L_OFFENSE:
			motion.NextAction();
			motion.MotionAction().Set(&boxer);
			if (motion.IsActionOverHalf()) {
				state = PULL;
			}
		break;
		case DEFENSE:
		case PULL: /* 菈 */
			motion.NextAction();
			motion.MotionAction().Set(&boxer);
			PullHandling(boxer, motion);
		break;
		case MOVE: /* ړ */
			motion.NextMove();
			motion.MotionMove().Set(&boxer);
			MoveHandling(boxer, motion);
		break;
		case DAMAGED: /* e */
			motion.NextDamaged();
			motion.MotionDamaged().Set(&boxer);
			if (motion.IsDamagedEnd()) {
				moved = 1;
				state = RHYTHM;
			}
		break;
		case DOWNING: /* _E */
			motion.NextDowning();
			motion.MotionDowning().Set(&boxer);
			if (motion.IsDowningEnd()) {
				state = DOWN;
				InitCount();
				DownInc();
			}
		break;
		case DOWN: /* _E */
			motion.MotionDowning().Set(&boxer);
			if ((damage < max_damage) && IsAnyAction()) {
				motion.SetRestore();		
				state = RESTORE;
			}
		break;
		case RESTORE: /* 񕜒 */
			motion.NextRestore();
			motion.MotionRestore().Set(&boxer);
			if (motion.IsRestoreEnd()) {
				state = WAIT;
			}
		break;
		case WAIT: /* ҋ@ */
			motion.MotionRestore().Set(&boxer);
			if (IsAnyAction()) {
				InitCount();
				state = DEFENSE;
				action = DEFENSE_N;
				motion.SetAction(action);	
			}
		break;
		case CLINCHING: /* N` */
			motion.NextClinching();
			motion.MotionClinching().Set(&boxer);
			if (motion.IsClinchingEnd()) {
				moved = 1;
				state = RHYTHM;
				adjust_distance = 3.0;
			}
		break;
	}
}

void BoxerHandler::RhythmHandling(Boxer &boxer, MotionHandler &motion) {
/* Deadly Combination */
	if (IsDeadly()) {
		action = motion.assign[DEADLY1];
		state = WhichState(action);
		deadly++;
		if (IsDefenseState()) {
			motion.SetAction(action);		
			boxer.MoveStart();
			return;
		} else if (CanOffense()) {
			InitOffense();
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			PlaySwingSE(state);
			return;
		}
		action = 0;
		state = RHYTHM;
		deadly = 0;
	}

/* Clinch */
	if (IsClinch()) {
		if (CanClinch()) {
			SetStateClinch(motion);
			boxer.MoveStart();
		}
		return;			
	}

/* Left Offense */
	if (IsLeftOffense() && IsRightForward()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_RF;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeftForward()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_LF;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsRightBack()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_RB;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeftBack()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_LB;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsForward()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_F;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsBack()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_B;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsRight()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_R;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeft()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_L;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense()) {
		if (CanOffense()) {
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_N;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING1;
			return;
		}
	}

/* RIGHT Offense */
	if (IsRightOffense() && IsRightForward()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_RF;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeftForward()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_LF;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsRightBack()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_RB;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeftBack()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_LB;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsForward()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_F;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsBack()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_B;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsRight()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_R;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeft()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_L;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense()) {
		if (CanOffense()) {
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_N;
			motion.SetAction(action, swing_speed);
			boxer.MoveStart();
			select_sound = SE_SWING2;
			return;
		}
	}

/* Defense */
	if (IsDefense() && IsRightForward()) {
		state = DEFENSE;
		action = DEFENSE_RF;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsLeftForward()) {
		state = DEFENSE;
		action = DEFENSE_LF;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsRightBack()) {
		state = DEFENSE;
		action = DEFENSE_RB;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsLeftBack()) {
		state = DEFENSE;
		action = DEFENSE_LB;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsForward()) {
		state = DEFENSE;
		action = DEFENSE_F;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsBack()) {
		state = DEFENSE;
		action = DEFENSE_B;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsRight()) {
		state = DEFENSE;
		action = DEFENSE_R;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense() && IsLeft()) {
		state = DEFENSE;
		action = DEFENSE_L;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

	if (IsDefense()) {
		state = DEFENSE;
		action = DEFENSE_N;
		motion.SetAction(action);
		boxer.MoveStart();
		return;
	}

/* Move */
	if (IsForward()) {
		if (boxer.CanForward(distance, adjust_distance)) {
			state = MOVE;
			action = MOVE_FORWARD;
			direct = DIR_FORWARD | DIR_NO_RELEASED;
			motion.SetMove(action, move_speed * speed_manip);
			boxer.MoveStart();
			return;
		}
	}

	if (IsBack()) {
		if (boxer.CanMove()) {
			state = MOVE;
			action = MOVE_BACK;
			direct = DIR_BACK | DIR_NO_RELEASED;
			motion.SetMove(action, move_speed * speed_manip);
			boxer.MoveStart();
			return;
		}
	}

	if (IsRight()) {
		if (boxer.CanMove()) {
			state = MOVE;
			action = MOVE_RIGHT;
			direct = DIR_RIGHT | DIR_NO_RELEASED;
			motion.SetMove(action, move_speed * speed_manip);
			boxer.MoveStart();
			return;
		}
	}

	if (IsLeft()) {
		if(boxer.CanMove()) {
			state = MOVE;
			action = MOVE_LEFT;
			direct = DIR_LEFT | DIR_NO_RELEASED;
			motion.SetMove(action, move_speed * speed_manip);
			boxer.MoveStart();
			return;
		}
	}
}

void BoxerHandler::PullHandling(Boxer &boxer, MotionHandler &motion) {
/* Left Offense */
 	if (IsLeftOffense()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_N)) {
				next_action = LEFT_N;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

 	if (IsLeftOffense() && IsForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_F)) {
				next_action = LEFT_F;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

 	if (IsLeftOffense() && IsBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_B)) {
				next_action = LEFT_B;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

 	if (IsLeftOffense() && IsRight()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_R)) {
				next_action = LEFT_R;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsLeftOffense() && IsLeft()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_L)) {
				next_action = LEFT_L;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsLeftOffense() && IsRightForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_RF)) {
				combination = 1;
				next_action = LEFT_RF;
				next_state = L_OFFENSE;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsLeftOffense() && IsLeftForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_LF)) {
				next_action = LEFT_LF;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsLeftOffense() && IsRightBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_RB)) {
				next_action = LEFT_RB;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsLeftOffense() && IsLeftBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, LEFT_LB)) {
				next_action = LEFT_LB;
				next_state = L_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

/* Right Offense */
 	if (IsRightOffense()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_N)) {
				next_action = RIGHT_N;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_F)) {
				next_action = RIGHT_F;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_B)) {
				next_action = RIGHT_B;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsRight()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_R)) {
				next_action = RIGHT_R;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsLeft()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_L)) {
				next_action = RIGHT_L;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsRightForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_RF)) {
				next_action = RIGHT_RF;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsLeftForward()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_LF)) {
				next_action = RIGHT_LF;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsRightBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_RB)) {
				next_action = RIGHT_RB;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

	if (IsRightOffense() && IsLeftBack()) {
		if (CanOffense()) {
			if (motion.IsCombo(action, RIGHT_LB)) {
				next_action = RIGHT_LB;
				next_state = R_OFFENSE;
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} 

/* Defense */
 	if (IsDefense()) {
		if (motion.IsCombo(action, DEFENSE_N)) {
			next_action = DEFENSE_N;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsForward()) {
		if (motion.IsCombo(action, DEFENSE_F)) {
			next_action = DEFENSE_F;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsBack()) {
		if (motion.IsCombo(action, DEFENSE_B)) {
			next_action = DEFENSE_B;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsRight()) {
		if (motion.IsCombo(action, DEFENSE_R)) {
			next_action = DEFENSE_R;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsLeft()) {
		if (motion.IsCombo(action, DEFENSE_L)) {
			next_action = DEFENSE_L;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsRightForward()) {
		if (motion.IsCombo(action, DEFENSE_RF)) {
			next_action = DEFENSE_RF;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsLeftForward()) {
		if (motion.IsCombo(action, DEFENSE_LF)) {
			next_action = DEFENSE_LF;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsRightBack()) {
		if (motion.IsCombo(action, DEFENSE_RB)) {
			next_action = DEFENSE_RB;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	} 

	if (IsDefense() && IsLeftBack()) {
		if (motion.IsCombo(action, DEFENSE_LB)) {
			next_action = DEFENSE_LB;
			next_state = DEFENSE;
			combination = 1;
		} else {
			combination = 0;		
		}
	}

/* Deadly Combination */
	if (deadly && deadly < 4 && CanOffense()) {
		if (motion.assign[DEADLY1 + deadly] != -1) {
			if (motion.IsCombo(action, motion.assign[DEADLY1 + deadly])) {
				next_action = motion.assign[DEADLY1 + deadly];
				next_state = WhichState(next_action);
				combination = 1;
			} else {
				combination = 0;		
			}
		}
	} else {
		deadly = 0;
	}   

/* Combination */
	if (motion.IsActionCombo() && combination) {
		if (deadly) {
			deadly++;					
		}    
		combination = 0;
		state = next_state;
		action = next_action;
		if (IsDefenseState()) {
			motion.SetComboAction(action);		
		} else {
			InitOffense();
			motion.SetComboAction(action, swing_speed);
			PlaySwingSE(state);
		}
		return;
	}

	if (motion.IsActionEnd()) {
		combination = 0;
		moved = 1;
		if (deadly && motion.assign[DEADLY1 + deadly] != -1) {
			action = motion.assign[DEADLY1 + deadly];
			state = WhichState(action);
			deadly++;
			if (IsDefenseState()) {
				motion.SetAction(action);		
			} else {
				InitOffense();
				motion.SetAction(action, swing_speed);					
				PlaySwingSE(state);
			}
		} else {
			deadly = 0;
			state = RHYTHM;
		}     
	}
}

void BoxerHandler::MoveHandling(Boxer &boxer, MotionHandler &motion) {
/* Deadly Combination */
	if (IsDeadly()) {
		action = motion.assign[DEADLY1];
		state = WhichState(action);
		deadly++;
		moved= 2;
		if (IsDefenseState()) {
			motion.SetActionDuringMove(action);	
			return;
		} else if (CanOffense()) {
			InitOffense();
			motion.SetActionDuringMove(action, swing_speed);		
			PlaySwingSE(state);
			return;
		}
		action = 0;
		state = MOVE;
		deadly = 0;
		moved = 0;
	}

/* Clinch */
	if (IsClinch()) {
		if (CanClinch()) {
			SetStateClinch(motion);
			return;			
		} else {
			DeleteOffenseKey();
		}
	}

/* Left Offense */
	if (IsLeftOffense() && IsRightForward()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
	 		state = L_OFFENSE;
			action = LEFT_RF;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeftForward()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_LF;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}
	
	if (IsLeftOffense() && IsRightBack()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_RB;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeftBack()) {
		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_LB;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsForward()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_F;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsBack()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_B;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsRight()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_R;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense() && IsLeft()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_L;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

	if (IsLeftOffense()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = L_OFFENSE;
			action = LEFT_N;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING1;
			return;
		}
	}

/* Right Offense */
	if (IsRightOffense() && IsRightForward()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_RF;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeftForward()) {
		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_LF;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}
	
	if (IsRightOffense() && IsRightBack()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_RB;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeftBack()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_LB;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsForward()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_F;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsBack()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_B;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsRight()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_R;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense() && IsLeft()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_L;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

	if (IsRightOffense()) {
 		if (CanOffense()) {
			moved = 2;
			InitOffense();
			state = R_OFFENSE;
			action = RIGHT_N;
			motion.SetActionDuringMove(action, swing_speed);
			select_sound = SE_SWING2;
			return;
		}
	}

/* Defense */
	if (IsDefense() && IsRightForward()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_RF;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsLeftForward()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_LF;
		motion.SetActionDuringMove(action);
		return;
	}
	
	if (IsDefense() && IsRightBack()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_RB;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsLeftBack()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_LB;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsForward()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_F;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsBack()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_B;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsRight()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_R;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense() && IsLeft()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_L;
		motion.SetActionDuringMove(action);
		return;
	}

	if (IsDefense()) {
		moved = 2;
		state = DEFENSE;
		action = DEFENSE_N;
		motion.SetActionDuringMove(action);
		return;
	}

/* Dashing */
	if (!dashing) { 
		if (direct == (DIR_FORWARD | DIR_NO_RELEASED)) {
			if (!IsForward()) {
				direct = DIR_FORWARD;				
			}
		} else if (IsForward() && direct == DIR_FORWARD) {
			dashing = 1;							
			select_sound = SE_DASH;
		}

		if (direct == (DIR_BACK | DIR_NO_RELEASED)) {
			if (!IsBack()) {
				direct = DIR_BACK;				
			}
		} else if (IsBack() && direct == DIR_BACK) {
			dashing = 1;							
			select_sound = SE_DASH;
		}

		if (direct == (DIR_RIGHT | DIR_NO_RELEASED)) {
			if (!IsRight()) {
				direct = DIR_RIGHT;				
			}
		} else if (IsRight() && direct == DIR_RIGHT) {
			dashing = 1;							
			select_sound = SE_DASH;
		}

		if (direct == (DIR_LEFT | DIR_NO_RELEASED)) {
			if (!IsLeft()) {
				direct = DIR_LEFT;				
			}
		} else if (IsLeft() && direct == DIR_LEFT) {
			dashing = 1;							
			select_sound = SE_DASH;
		}
	} else if (!motion.IsMoveEnd()) {
		motion.NextMove();
		motion.MotionMove().Set(&boxer);					
	}

	if (motion.IsMoveEnd()) {
		state = RHYTHM;
		direct = DIR_NO;
		moved = 1;
		dashing = 0;
	}
}
       
void BoxerHandler::Move(Boxer &boxer) {
	if (moved == 1) {
		boxer.MoveToRhythm();
		moved = 0;	
	}  
	if (moved == 2) {
		boxer.Move();
		moved = 0;	
	}  
}

void BoxerHandler::SetStatePull() {
	if (state == R_OFFENSE || state == L_OFFENSE) {	
		state = PULL;
	}
}

void BoxerHandler::SetStateClinch(MotionHandler &motion) {
	if (IsClinching() || !CanClinch() || IsDowning() || IsDownState()) {
		return;	
	} else {
	/*	if (IsMoving()) {
			moved = 2;	
		}  */
		motion.SetClinching(IsRhythmState());		
		state = CLINCHING;
		action = CLINCH_MOTION;
		deadly = 0;
		combination = 0;
	}
}

int BoxerHandler::WhichState(int caction) {
	if (caction >= RIGHT_N && caction <= RIGHT_LB) {
		return R_OFFENSE;
	} else if(caction >= LEFT_N && caction <= LEFT_LB) {
		return L_OFFENSE;
	} else {
		return DEFENSE;					
	}
}

void BoxerHandler::PlaySwingSE(int cstate) {
	if (cstate == R_OFFENSE) {
		select_sound = SE_SWING1;
	} else {
		select_sound = SE_SWING2;
	} 
}
