#include "boxerhandler.h"

int BoxerHandler::LoadBoxer(Boxer &boxer, MotionHandler &motion, const char *filename) {
 	CReader breader;
	int i, j, max_camera;

	if (breader.Open(filename) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}

/* Skip Optoin State */	
	for (i = 0; i < 13; i++) {
		breader.Skip();	
	}

	if (LoadBoxerBody(boxer, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBoxerMaterial(boxer, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}

/* Skip Camera Data */	
	breader.Skip();	
	breader.Skip();	
	breader.Skip();	
	breader.Skip();	
	max_camera = breader.NextInt();
	if (max_camera > 100) {
		emes("BoxerHandler::LoadBoxer -> Camera number is wrong.");
		return SDL_FAILURE;		
	}
	for (i = 0; i < max_camera; i++) {
		for (j = 0; j < 10; j++) {
			breader.Skip();
		}	
	}

	if (LoadBoxerMotion(motion, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBoxerCombo(motion, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBoxerAssign(motion, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBoxerRange(boxer, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBoxerThink(breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (SaveDefaultThink() == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
 	if (LoadBoxerOther(boxer, breader) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}

//	PrintBoxer(boxer, motion);

	return SDL_SUCCESS; 
}

int BoxerHandler::LoadBoxerBody(Boxer &boxer, CReader &breader) {
	float r, a1, a2, b1, b2, c1, c2; 
/* --- Load Boxer Body Data --- */	
	breader.Skip();	
	boxer.SetShoulderWidth(breader.NextIntF());
	boxer.SetWaistWidth(breader.NextIntF());
/* Load Boxer Data Right Arm */	
	r = breader.NextIntF();
	boxer.right_arm.fist.SetSize(r, r, r, r, r, r);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.right_arm.forearm.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.right_arm.upperarm.SetSize(a1, a2, b1, b2, c1, c2);
	r = breader.NextIntF();
	boxer.right_arm.shoulder.SetSize(r, r, r, r, r, r);
/* Load Boxer Data Left Arm */	
	r = breader.NextIntF();
	boxer.left_arm.fist.SetSize(r, r, r, r, r, r);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.left_arm.forearm.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.left_arm.upperarm.SetSize(a1, a2, b1, b2, c1, c2);
	r = breader.NextIntF();
	boxer.left_arm.shoulder.SetSize(r, r, r, r, r, r);
/* Load Boxer Data Right Leg */	
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	boxer.right_leg.toe.SetSize(a1, a2, b1, b2, c1, 0.0);
	boxer.right_leg.foot.SetSize(a1, a2, b1, b2, c1, 0.0);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.right_leg.shank.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.right_leg.thigh.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Left Leg */	
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	boxer.left_leg.toe.SetSize(a1, a2, b1, b2, c1, 0.0);
	boxer.left_leg.foot.SetSize(a1, a2, b1, b2, c1, 0.0);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.left_leg.shank.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.left_leg.thigh.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Body */	
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.body.waist.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.body.abdomen.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.body.breast.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Head */	
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.head.neck.SetSize(a1, a2, b1, b2, c1, c2);
	a1 = breader.NextIntF();
	a2 = breader.NextIntF();
	b1 = breader.NextIntF();
	b2 = breader.NextIntF();
	c1 = breader.NextIntF();
	c2 = breader.NextIntF();
	boxer.head.face.SetSize(a1, a2, b1, b2, c1, c2);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerMaterial(Boxer &boxer, CReader &breader) {
	float r, g, b, a, s;
/* --- Load Material Data --- */
	breader.Skip();		
/* Load Material Data Glove */
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.glove.DefDiffuse(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.glove.DefAmbient(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.glove.DefSpecular(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.glove.DefEmission(r, g, b, a);
	s = (float)breader.NextInt();
	boxer.glove.DefShininess(s);
/* Load Material Data Bandage */	
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.bandage.DefDiffuse(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.bandage.DefAmbient(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.bandage.DefSpecular(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.bandage.DefEmission(r, g, b, a);
	s = (float)breader.NextInt();
	boxer.bandage.DefShininess(s);
/* Load Material Data Skin */	
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.skin.DefDiffuse(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.skin.DefAmbient(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.skin.DefSpecular(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.skin.DefEmission(r, g, b, a);
	s = (float)breader.NextInt();
	boxer.skin.DefShininess(s);
/* Load Material Data Trunks */	
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.trunks.DefDiffuse(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.trunks.DefAmbient(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.trunks.DefSpecular(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.trunks.DefEmission(r, g, b, a);
	s = (float)breader.NextInt();
	boxer.trunks.DefShininess(s);
/* Load Material Data Shoes */	
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.shoes.DefDiffuse(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.shoes.DefAmbient(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.shoes.DefSpecular(r, g, b, a);
	r = breader.NextIntF(255);
	g = breader.NextIntF(255);
	b = breader.NextIntF(255);
	a = breader.NextIntF(255);
	boxer.shoes.DefEmission(r, g, b, a);
	s = (float)breader.NextInt();
	boxer.shoes.DefShininess(s);
/* Load Material Data Hair */	
	boxer.SetDefaultFace();
	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerMotion(MotionHandler &motion, CReader &breader) {
	int i, j, k, max_motion, max_handle, max_frame;
/* --- Load Motion Data --- */
	max_handle = 0;
	max_motion = breader.NextInt();
	motion.Init(max_motion);
	breader.Skip();	

	if (max_motion > 100) {
		emes("BoxerHandler::LoadBoxerMotion -> Motion number is wrong.");
		return SDL_FAILURE;
	}

	for (i = 0; i < max_motion; i++) {
		breader.Skip();	
		breader.Skip();	
		max_frame = breader.NextInt();	
		if (max_motion > 100) {
			return SDL_FAILURE;		
		}
		if (max_frame > 80 || max_frame < 2) {
			emes("BoxerHandler::LoadBoxerMotion -> Frame number is wrong.");
			return SDL_FAILURE;
		}
		motion[i].Init(max_frame);
  		breader.NextInt();	
		if (max_frame > max_handle) {
		 	max_handle = max_frame;
		}
		for (j = 0; j < max_frame; j++) {
			breader.Skip();	
			motion[i][j].SetNumber(breader.NextInt());	
 		
			for (k = 0; k < 44; k++) {
				motion[i][j][k] = breader.NextFloat();
			}
			for (k = 0; k < 6; k++) {
				breader.Skip();	
			}
			motion[i].Link();
	    } 
	} 
	motion.hmotion.Init(max_handle + 2);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerCombo(MotionHandler &motion, CReader &breader) {
	int i, max_combo, first, second;
/* --- Load Combination --- Data */	
	breader.Skip();
	max_combo = breader.NextInt();
	if (max_combo > 200 || max_combo < 1) {
		emes("BoxerHandler::LoadBoxerCombo");
		return SDL_FAILURE;
	}
	motion.combo.Init(max_combo);
	breader.Skip();
	for (i = 0; i < max_combo; i++) {
		breader.Skip();
		first = breader.NextInt();	
		second = breader.NextInt();	
		motion.combo.SetCombo(i, first, second);
	}

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerAssign(MotionHandler &motion, CReader &breader) {
	int i;
/* --- Load Assign Data --- */	
	breader.Skip();
	for (i = 0; i < MAX_ASSIGN; i++) {
		motion.assign[i] = breader.NextInt();
//        fprintf(stderr, "i = %d motion.assign[i] = %d\n", i, motion.assign[i]);
	}
	if (motion.assign[DEADLY1] == 0) {
		motion.assign[DEADLY1] = LEFT_N;	
	} else {
		motion.assign[DEADLY1] += 9;			
	}
	if (motion.assign[DEADLY2] == 0) {
		motion.assign[DEADLY2] = -1;	
	} else {
		motion.assign[DEADLY2] += 9;		
	}
	if (motion.assign[DEADLY3] == 0) {
		motion.assign[DEADLY3] = -1;	
	} else {
		motion.assign[DEADLY3] += 9;		
	}
	if (motion.assign[DEADLY4] == 0) {
		motion.assign[DEADLY4] = -1;	
	} else {
		motion.assign[DEADLY4] += 9;		
	}

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerRange(Boxer &boxer, CReader &breader) {
	float x, y, z, r, t, b;
/* --- Load Range Data --- */	
	breader.Skip();
	breader.Skip();
	breader.Skip();
	boxer.right_arm.fist.SetRange(breader.NextIntF());
	boxer.right_arm.forearm.SetRange(breader.NextIntF());
	boxer.right_arm.upperarm.SetRange(breader.NextIntF());
	boxer.right_arm.shoulder.SetRange(breader.NextIntF());
	boxer.left_arm.fist.SetRange(breader.NextIntF());
	boxer.left_arm.forearm.SetRange(breader.NextIntF());
	boxer.left_arm.upperarm.SetRange(breader.NextIntF());
	boxer.left_arm.shoulder.SetRange(breader.NextIntF());
	r = breader.NextIntF();
	t = breader.NextIntF();
	b = breader.NextIntF();
	boxer.body.abdomen.SetRange(r, t, b);
	r = breader.NextIntF();
	x = breader.NextIntF();
	y = breader.NextIntF();
	z = breader.NextIntF();
	boxer.body.abdomen.SetLiver(r, x, y, z);
	r = breader.NextIntF();
	t = breader.NextIntF();
	b = breader.NextIntF();
	boxer.head.face.SetRange(r, t, b);
	r = breader.NextIntF();
	x = breader.NextIntF();
	y = breader.NextIntF();
	z = breader.NextIntF();
	boxer.head.face.SetJaw(r, x, y, z);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBoxerThink(CReader &breader) {
	int i, j;
/* --- Load Think Data --- */	
	breader.Skip();	
	max_think = breader.NextInt();

	if (max_think > 99 || max_think < 1) {
		emes("BoxerHandler::LoadBoxerThink -> Routine number is wrong.");
		return SDL_FAILURE;		
	}
	if (think != NULL) {
		delete [] think;
	} 
	think = new Think[max_think];
	if (think == NULL) {
		emes("BoxerHandler::LoadBoxerThink -> Out of Memory");
		return SDL_FAILURE;		
	}
  
	for (i = 0; i < max_think; i++) {
		breader.Skip();
		think[i].con_restround_less = breader.NextInt();
		think[i].restround_less = breader.NextInt();	
		think[i].con_resttime_less = breader.NextInt();
		think[i].resttime_less = breader.NextInt() * 50;	

		think[i].con_mrestcount_less = breader.NextInt();
		think[i].mrestcount_less = breader.NextInt();	
		think[i].con_mrestdown_less = breader.NextInt();
		think[i].mrestdown_less = breader.NextInt();	

		think[i].con_distance_greater = breader.NextInt();
		think[i].distance_greater = breader.NextIntF();
		think[i].con_distance_less = breader.NextInt();
		think[i].distance_less = breader.NextIntF();
		think[i].con_mposition = breader.NextInt();
		think[i].mposition = breader.NextInt();	
		think[i].con_msituation = breader.NextInt();
		think[i].msituation = breader.NextInt();	

#if CHANGE_SPECIFICATIONS
		think[i].con_mswing_greater = breader.NextInt();
		think[i].mswing_greater = breader.NextIntF(100);
		think[i].con_mswing_less = breader.NextInt();
		think[i].mswing_less = breader.NextIntF(100);
		think[i].con_mspeed_greater = breader.NextInt();
		think[i].mspeed_greater = breader.NextIntF(100);
		think[i].con_mspeed_less = breader.NextInt();
		think[i].mspeed_less = breader.NextIntF(100);
		think[i].con_mstate = breader.NextInt();
		think[i].mstate = breader.NextInt();

		think[i].con_oswing_greater = breader.NextInt();
		think[i].oswing_greater = breader.NextIntF(100);
		think[i].con_oswing_less = breader.NextInt();
		think[i].oswing_less = breader.NextIntF(100);
		think[i].con_ospeed_greater = breader.NextInt();
		think[i].ospeed_greater = breader.NextIntF(100);
		think[i].con_ospeed_less = breader.NextInt();
		think[i].ospeed_less = breader.NextIntF(100);
 		think[i].con_ostate = breader.NextInt();
		think[i].ostate = breader.NextInt();
#else
		think[i].con_mswing_greater = breader.NextInt();
		think[i].mswing_greater = 1.0 + breader.NextIntF(100);
		think[i].con_mswing_less = breader.NextInt();
		think[i].mswing_less = 1.0 + breader.NextIntF(100);
		think[i].con_mspeed_greater = breader.NextInt();
		think[i].mspeed_greater = 1.0 + breader.NextIntF(100);
		think[i].con_mspeed_less = breader.NextInt();
		think[i].mspeed_less = 1.0 + breader.NextIntF(100);
		think[i].con_mstate = breader.NextInt();
		think[i].mstate = breader.NextInt();

		think[i].con_oswing_greater = breader.NextInt();
		think[i].oswing_greater = 1.0 + breader.NextIntF(100);
		think[i].con_oswing_less = breader.NextInt();
		think[i].oswing_less = 1.0 + breader.NextIntF(100);
		think[i].con_ospeed_greater = breader.NextInt();
		think[i].ospeed_greater = 1.0 + breader.NextIntF(100);
		think[i].con_ospeed_less = breader.NextInt();
		think[i].ospeed_less = 1.0 + breader.NextIntF(100);
 		think[i].con_ostate = breader.NextInt();
		think[i].ostate = breader.NextInt();
#endif

		for (j = 0; j < 5; j++) {	
			think[i].action[j] = breader.NextInt();
			think[i].action_rate[j] = breader.NextInt();
		}
		for (j = 1; j < 5; j++) {	
			think[i].action_rate[j] += think[i].action_rate[j-1];
		}
	}
	return SDL_SUCCESS;
}
	
int BoxerHandler::LoadBoxerOther(Boxer &boxer, CReader &breader) {
	int i;
	char buf[255];
/* --- Load Other Data --- */	
	breader.NextChar(buf);
	buf[NAME_LENGTH] = '\0';
#ifdef LINUX
    strcpy(nickname_jp, babel::euc_to_sjis(buf).c_str());
#else
    strcpy(nickname_jp, buf);
#endif

	breader.NextChar(buf);
	buf[NAME_LENGTH] = '\0';
#ifdef LINUX
    strcpy(familyname_jp, babel::euc_to_sjis(buf).c_str());
#else
    strcpy(familyname_jp, buf);
#endif

	breader.NextChar(buf);
	buf[NAME_LENGTH] = '\0';
    strcpy(nickname_en, buf);
	breader.NextChar(buf);
	buf[NAME_LENGTH] = '\0';
    strcpy(familyname_en, buf);

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Punch");
		i = 0;	
	}
	punch_manip = 1.0 + (float)i / 100.0;
	
	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Speed");
		i = 0;	
	}
	speed_manip = 1.0 / (1.0 + (float)i / 100.0);

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Stamina");
		i = 0;	
	}
	stamina_manip = 1.0 + (float)i / 100.0;
	
	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> DamageRestore");
		i = 0;	
	}
	damage_restore_manip = 1.0 + (float)i / 100.0;

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> DamageMax");
		i = 0;	
	}
	max_damage_manip = 1.0 + (float)i / 100.0;

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> CyanosisRstore");
		i = 0;	
	}
	cyanosis_restore_manip = 1.0 + (float)i / 100.0;

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> MaxCyanosis");
		i = 0;	
	}
	max_cyanosis_manip = 1.0 + (float)i / 100.0;

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Block");
		i = 0;	
	}

	defense_shoulder_absorb = 0.5 - (float)i / 1000.0;
	defense_upperarm_absorb = 0.6 - (float)i / 1000.0;
	defense_forearm_absorb  = 0.7 - (float)i / 1000.0;
	defense_fist_absorb     = 0.8 - (float)i / 1000.0;

	normal_shoulder_absorb = 0.75 - (float)i / 1000.0;
	normal_upperarm_absorb = 0.80 - (float)i / 1000.0;
	normal_forearm_absorb  = 0.85 - (float)i / 1000.0;
	normal_fist_absorb     = 0.90 - (float)i / 1000.0;

	offense_shoulder_absorb = 0.875 - (float)i / 1000.0;
	offense_upperarm_absorb = 0.900 - (float)i / 1000.0;
	offense_forearm_absorb  = 0.925 - (float)i / 1000.0;
	offense_fist_absorb     = 0.950 - (float)i / 1000.0;

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Body");
		i = 0;	
	}
	body_strength = 1.0 / (1.0 + (float)i / 100.0);

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Liver");
		i = 0;	
	}
	liver_strength = 1.5 - (float)i / 100.0;
	
	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Face");
		i = 0;	
	}
	face_strength = 1.0 / (1.0 + (float)i / 100.0);

	i = breader.NextInt();
	if (i > 50 || i < -50) {
		wmes("BoxerHandler::LoadBoxerOther -> Jaw");
		i = 0;	
	}
	jaw_strength = 1.5 - (float)i / 100.0;

	boxer.hair_style = 1;
	boxer.eyebrow = 1;
	boxer.eye = 1;
	boxer.nose = 1;
	boxer.mouth = 1;

	return SDL_SUCCESS;
}

int BoxerHandler::MakeBoxerSourceFile(Boxer &boxer, MotionHandler &motion, const char *filename, int which_data) {
	FILE *bin_fp;
	FILE *src_fp;
	char bin_file[260];
	char src_file[260];
	int j, k;
	Uint8 p[1];
	Uint32 v[1];

	sprintf(bin_file, "%s.bin", filename);

	switch (which_data) {
		case MAKE_BODY_SOURCE:
			sprintf(src_file, "%s_body.h", filename);
			SaveBinBoxerBodySetFile(boxer, bin_file);
		break;
		case MAKE_MOTION_SOURCE:
			sprintf(src_file, "%s_motion.h", filename);
			SaveBinBoxerMotionSetFile(motion, bin_file);
		break;
		case MAKE_THINK_SOURCE:
			sprintf(src_file, "%s_think.h", filename);
			SaveBinBoxerThinkFile(bin_file);
		break;
		case MAKE_BOXER_SOURCE:
		default:
			sprintf(src_file, "%s.h", filename);
			SaveBinBoxerFile(boxer, motion, bin_file);
		break;
	}
	
	bin_fp = fopen(bin_file, "rb") ;
	if (bin_fp == NULL)	 {
		return SDL_FAILURE;
	}
	src_fp = fopen(src_file, "wt") ;
	if (src_fp == NULL)	 {
		return SDL_FAILURE;
	}

	j = 0;	
	fprintf(src_fp, "/* This file was made from [%s]. */\n", filename);
	fprintf(src_fp, "\t\n");
	fprintf(src_fp, "#include <SDL.h>\n");
	fprintf(src_fp, "\t\n");
	switch (which_data) {
		case 1:
			fprintf(src_fp, "SDL_RWops* BoxerBody();\n\n");
		break;
		case 2:
			fprintf(src_fp, "SDL_RWops* BoxerMotion();\n\n");
		break;
		case 3:
			fprintf(src_fp, "SDL_RWops* BoxerThink();\n\n");
		break;
		default:
			fprintf(src_fp, "SDL_RWops* BoxerData();\n\n");
		break;
	}
	fprintf(src_fp, "#include \"boxerdata.h\"\n\n");
	switch (which_data) {
		case 1:
			fprintf(src_fp, "SDL_RWops* BoxerBody() {\n");
		break;
		case 2:
			fprintf(src_fp, "SDL_RWops* BoxerMotion() {\n");
		break;
		case 3:
			fprintf(src_fp, "SDL_RWops* BoxerThink() {\n");
		break;
		default:
			fprintf(src_fp, "SDL_RWops* BoxerData() {\n");
		break;
	}
	fprintf(src_fp, "\tSDL_RWops *rwops;\n");
	fprintf(src_fp, "\tstatic Uint32 p[] = {\n");
	
	while (1) {		
		if (fread(p, sizeof(Uint8), 1, bin_fp)) {
			j++;
		} else {
			break;		
		}
	}
	if (j % 4) {
		j = j / 4 + 1;	
	} else {
		j = j / 4;		
	}

	fseek(bin_fp, 0, 0);
	for (k = 0; k < j; k++) {
		fread(v, sizeof(Uint32), 1, bin_fp);
		if (k == j - 1) {
			if (k % 4 == 0){ 
				fprintf(src_fp, "\t\t0x%08x };\n", *v);
			} else {
				fprintf(src_fp, "0x%08x };\n", *v);
			}
		} else if (k % 4 == 0){
			fprintf(src_fp, "\t\t0x%08x, ", *v);
		} else if ((k + 1) % 4 == 0) {
			fprintf(src_fp, "0x%08x,\n", *v);
		} else {
			fprintf(src_fp, "0x%08x, ", *v);		
		}
	}
	fprintf(src_fp, "\t\n");
	fprintf(src_fp, "\trwops = SDL_RWFromMem((void *)p, %d);\n", j*4);
	fprintf(src_fp, "\treturn rwops;\n");
	fprintf(src_fp, "}\n");
	fclose(src_fp);

	return SDL_SUCCESS;
}


int BoxerHandler::SaveBinBoxerFile(Boxer &boxer, MotionHandler &motion, const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "wb");

	if (rwops == NULL) {
		emes("oxerHandler::SaveBinBoxerFile") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = SaveBinBoxer(boxer, motion, rwops);
	SDL_RWclose(rwops);
	return flag;
}


int BoxerHandler::SaveBinBoxer(Boxer &boxer, MotionHandler &motion, SDL_RWops *rwops) {	
	if (SaveBinBoxerBody(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (SaveBinBoxerMaterial(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (SaveBinBoxerMotion(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}   
	if (SaveBinBoxerCombo(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (SaveBinBoxerAssign(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (SaveBinBoxerRange(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (LoadDefaultThink() == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (SaveBinBoxerThink(rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
 	if (SaveBinBoxerOther(rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}              
  
	return SDL_SUCCESS; 
}


int BoxerHandler::SaveBinBoxerBodySetFile(Boxer &boxer, const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "wb");

	if (rwops == NULL) {
		emes("BoxerHandler::SaveBinBoxerBodySetFile") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = SaveBinBoxerBodySet(boxer, rwops);
	SDL_RWclose(rwops);
	return flag;
}

int BoxerHandler::SaveBinBoxerMotionSetFile(MotionHandler &motion, const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "wb");

	if (rwops == NULL) {
		emes("BoxerHandler::SaveBinBoxerMotionSetFile") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = SaveBinBoxerMotionSet(motion, rwops);
	SDL_RWclose(rwops);
	return flag;
}

int BoxerHandler::SaveBinBoxerBodySet(Boxer &boxer, SDL_RWops *rwops) {	
	if (SaveBinBoxerBody(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (SaveBinBoxerRange(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
  
	return SDL_SUCCESS; 
}

int BoxerHandler::SaveBinBoxerMotionSet(MotionHandler &motion, SDL_RWops *rwops) {	
	if (SaveBinBoxerMotion(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}   
	if (SaveBinBoxerCombo(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (SaveBinBoxerAssign(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
  
	return SDL_SUCCESS; 
}

int BoxerHandler::SaveBinBoxerThinkFile(const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "wb");

	if (rwops == NULL) {
		emes("BoxerHandler::SaveBinBoxerThinkFile") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = SaveBinBoxerThink(rwops);
	SDL_RWclose(rwops);
	return flag;
}

int BoxerHandler::SaveBinBoxerBody(Boxer &boxer, SDL_RWops *rwops) {
	float data[6]; 
/* --- Save Bin Boxer Body Data --- */	
	data[0] = boxer.ShoulderWidth();
	data[1] = boxer.WaistWidth();
	SDL_RWwrite(rwops, data, sizeof(float), 2);
/* Save Bin Boxer Data Right Arm */	
	data[0] = boxer.right_arm.fist.A1();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
	data[0] = boxer.right_arm.forearm.A1();
	data[1] = boxer.right_arm.forearm.A2();
	data[2] = boxer.right_arm.forearm.B1();
	data[3] = boxer.right_arm.forearm.B2();
	data[4] = boxer.right_arm.forearm.C1();
	data[5] = boxer.right_arm.forearm.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0]  = boxer.right_arm.upperarm.A1();
	data[1] = boxer.right_arm.upperarm.A2();
	data[2] = boxer.right_arm.upperarm.B1();
	data[3] = boxer.right_arm.upperarm.B2();
	data[4] = boxer.right_arm.upperarm.C1();
	data[5] = boxer.right_arm.upperarm.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.right_arm.shoulder.A1();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Bin Boxer Data Left Arm */	
	data[0] = boxer.left_arm.fist.A1();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
	data[0] = boxer.left_arm.forearm.A1();
	data[1] = boxer.left_arm.forearm.A2();
	data[2] = boxer.left_arm.forearm.B1();
	data[3] = boxer.left_arm.forearm.B2();
	data[4] = boxer.left_arm.forearm.C1();
	data[5] = boxer.left_arm.forearm.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.left_arm.upperarm.A1();
	data[1] = boxer.left_arm.upperarm.A2();
	data[2] = boxer.left_arm.upperarm.B1();
	data[3] = boxer.left_arm.upperarm.B2();
	data[4] = boxer.left_arm.upperarm.C1();
	data[5] = boxer.left_arm.upperarm.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.left_arm.shoulder.A1();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Bin Boxer Data Right Leg */	
	data[0] = boxer.right_leg.toe.A1();
	data[1] = boxer.right_leg.toe.A2();
	data[2] = boxer.right_leg.toe.B1();
	data[3] = boxer.right_leg.toe.B2();
	data[4] = boxer.right_leg.toe.C1();
	SDL_RWwrite(rwops, data, sizeof(float), 5);
	data[0] = boxer.right_leg.shank.A1();
	data[1] = boxer.right_leg.shank.A2();
	data[2] = boxer.right_leg.shank.B1();
	data[3] = boxer.right_leg.shank.B2();
	data[4] = boxer.right_leg.shank.C1();
	data[5] = boxer.right_leg.shank.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.right_leg.thigh.A1();
	data[1] = boxer.right_leg.thigh.A2();
	data[2] = boxer.right_leg.thigh.B1();
	data[3] = boxer.right_leg.thigh.B2();
	data[4] = boxer.right_leg.thigh.C1();
	data[5] = boxer.right_leg.thigh.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
/* Save Bin Boxer Data Left Leg */	
	data[0] = boxer.left_leg.toe.A1();
	data[1] = boxer.left_leg.toe.A2();
	data[2] = boxer.left_leg.toe.B1();
	data[3] = boxer.left_leg.toe.B2();
	data[4] = boxer.left_leg.toe.C1();
	SDL_RWwrite(rwops, data, sizeof(float), 5);
	data[0] = boxer.left_leg.shank.A1();
	data[1] = boxer.left_leg.shank.A2();
	data[2] = boxer.left_leg.shank.B1();
	data[3] = boxer.left_leg.shank.B2();
	data[4] = boxer.left_leg.shank.C1();
	data[5] = boxer.left_leg.shank.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.left_leg.thigh.A1();
	data[1] = boxer.left_leg.thigh.A2();
	data[2] = boxer.left_leg.thigh.B1();
	data[3] = boxer.left_leg.thigh.B2();
	data[4] = boxer.left_leg.thigh.C1();
	data[5] = boxer.left_leg.thigh.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
/* Save Bin Boxer Data Body */	
	data[0] = boxer.body.waist.A1();
	data[1] = boxer.body.waist.A2();
	data[2] = boxer.body.waist.B1();
	data[3] = boxer.body.waist.B2();
	data[4] = boxer.body.waist.C1();
	data[5] = boxer.body.waist.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.body.abdomen.A1();
	data[1] = boxer.body.abdomen.A2();
	data[2] = boxer.body.abdomen.B1();
	data[3] = boxer.body.abdomen.B2();
	data[4] = boxer.body.abdomen.C1();
	data[5] = boxer.body.abdomen.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.body.breast.A1();
	data[1] = boxer.body.breast.A2();
	data[2] = boxer.body.breast.B1();
	data[3] = boxer.body.breast.B2();
	data[4] = boxer.body.breast.C1();
	data[5] = boxer.body.breast.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
/* Save Bin Boxer Data Head */	
	data[0] = boxer.head.neck.A1();
	data[1] = boxer.head.neck.A2();
	data[2] = boxer.head.neck.B1();
	data[3] = boxer.head.neck.B2();
	data[4] = boxer.head.neck.C1();
	data[5] = boxer.head.neck.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);
	data[0] = boxer.head.face.A1();
	data[1] = boxer.head.face.A2();
	data[2] = boxer.head.face.B1();
	data[3] = boxer.head.face.B2();
	data[4] = boxer.head.face.C1();
	data[5] = boxer.head.face.C2();
	SDL_RWwrite(rwops, data, sizeof(float), 6);

	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerMaterial(Boxer &boxer, SDL_RWops *rwops) {
	float data[4]; 
/* --- Save Material Data --- */
/* Save Material Data Glove */
	data[0] = boxer.glove.DifR();
	data[1] = boxer.glove.DifG();
	data[2] = boxer.glove.DifB();
	data[3] = boxer.glove.DifA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.glove.AmbR();
	data[1] = boxer.glove.AmbG();
	data[2] = boxer.glove.AmbB();
	data[3] = boxer.glove.AmbA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.glove.SpeR();
	data[1] = boxer.glove.SpeG();
	data[2] = boxer.glove.SpeB();
	data[3] = boxer.glove.SpeA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.glove.EmiR();
	data[1] = boxer.glove.EmiG();
	data[2] = boxer.glove.EmiB();
	data[3] = boxer.glove.EmiA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.glove.Shi();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Material Data Bandage */	
	data[0] = boxer.bandage.DifR();
	data[1] = boxer.bandage.DifG();
	data[2] = boxer.bandage.DifB();
	data[3] = boxer.bandage.DifA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.bandage.AmbR();
	data[1] = boxer.bandage.AmbG();
	data[2] = boxer.bandage.AmbB();
	data[3] = boxer.bandage.AmbA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.bandage.SpeR();
	data[1] = boxer.bandage.SpeG();
	data[2] = boxer.bandage.SpeB();
	data[3] = boxer.bandage.SpeA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.bandage.EmiR();
	data[1] = boxer.bandage.EmiG();
	data[2] = boxer.bandage.EmiB();
	data[3] = boxer.bandage.EmiA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.bandage.Shi();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Material Data Skin */	
	data[0] = boxer.skin.DifR();
	data[1] = boxer.skin.DifG();
	data[2] = boxer.skin.DifB();
	data[3] = boxer.skin.DifA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.skin.AmbR();
	data[1] = boxer.skin.AmbG();
	data[2] = boxer.skin.AmbB();
	data[3] = boxer.skin.AmbA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.skin.SpeR();
	data[1] = boxer.skin.SpeG();
	data[2] = boxer.skin.SpeB();
	data[3] = boxer.skin.SpeA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.skin.EmiR();
	data[1] = boxer.skin.EmiG();
	data[2] = boxer.skin.EmiB();
	data[3] = boxer.skin.EmiA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.skin.Shi();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Material Data Trunks */	
	data[0] = boxer.trunks.DifR();
	data[1] = boxer.trunks.DifG();
	data[2] = boxer.trunks.DifB();
	data[3] = boxer.trunks.DifA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.trunks.AmbR();
	data[1] = boxer.trunks.AmbG();
	data[2] = boxer.trunks.AmbB();
	data[3] = boxer.trunks.AmbA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.trunks.SpeR();
	data[1] = boxer.trunks.SpeG();
	data[2] = boxer.trunks.SpeB();
	data[3] = boxer.trunks.SpeA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.trunks.EmiR();
	data[1] = boxer.trunks.EmiG();
	data[2] = boxer.trunks.EmiB();
	data[3] = boxer.trunks.EmiA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.trunks.Shi();
	SDL_RWwrite(rwops, data, sizeof(float), 1);
/* Save Material Data Shoes */	
	data[0] = boxer.shoes.DifR();
	data[1] = boxer.shoes.DifG();
	data[2] = boxer.shoes.DifB();
	data[3] = boxer.shoes.DifA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.shoes.AmbR();
	data[1] = boxer.shoes.AmbG();
	data[2] = boxer.shoes.AmbB();
	data[3] = boxer.shoes.AmbA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.shoes.SpeR();
	data[1] = boxer.shoes.SpeG();
	data[2] = boxer.shoes.SpeB();
	data[3] = boxer.shoes.SpeA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.shoes.EmiR();
	data[1] = boxer.shoes.EmiG();
	data[2] = boxer.shoes.EmiB();
	data[3] = boxer.shoes.EmiA();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.shoes.Shi();
	SDL_RWwrite(rwops, data, sizeof(float), 1);

	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerMotion(MotionHandler &motion, SDL_RWops *rwops) {
	int i, j, number, max_frame;
/* --- Save Motion Data --- */
	SDL_RWwrite(rwops, &motion.max_number, sizeof(int), 1);
	for (i = 0; i < motion.max_number; i++) {
		max_frame = motion.bmotions[i].MaxNumber();
		SDL_RWwrite(rwops, &max_frame, sizeof(int), 1);
		for (j = 0; j < max_frame; j++) {
			number = motion[i][j].GetNumber();
			SDL_RWwrite(rwops, &number, sizeof(int), 1);
			SDL_RWwrite(rwops, &motion[i][j][0], sizeof(float), 44);
		}
	}		
	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerCombo(MotionHandler &motion, SDL_RWops *rwops) {
	int max_combo;
/* --- Save Combination --- Data */	
	max_combo = motion.combo.MaxNumber();
	SDL_RWwrite(rwops, &max_combo, sizeof(int), 1);
	SDL_RWwrite(rwops, motion.combo.GetComboFirst(), sizeof(int), max_combo);
	SDL_RWwrite(rwops, motion.combo.GetComboSecond(), sizeof(int),  max_combo);

	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerAssign(MotionHandler &motion, SDL_RWops *rwops) {
	SDL_RWwrite(rwops, motion.assign, sizeof(int), MAX_ASSIGN);
	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerRange(Boxer &boxer, SDL_RWops *rwops) {
	float data[4];
/* --- Save Range Data --- */	
	data[0] = boxer.right_arm.fist.R();
	data[1] = boxer.right_arm.forearm.R();
	data[2] = boxer.right_arm.upperarm.R();
	data[3] = boxer.right_arm.shoulder.R();
	SDL_RWwrite(rwops, data, sizeof(float), 4);
	data[0] = boxer.left_arm.fist.R();
	data[1] = boxer.left_arm.forearm.R();
	data[2] = boxer.left_arm.upperarm.R();
	data[3] = boxer.left_arm.shoulder.R();
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = boxer.body.abdomen.R();
	data[1] = boxer.body.abdomen.Top();
	data[2] = boxer.body.abdomen.Bottom();
	SDL_RWwrite(rwops, data, sizeof(float), 3);
	data[0] = boxer.body.abdomen.LR();
	data[1] = boxer.body.abdomen.LRX();
	data[2] = boxer.body.abdomen.LRY();
	data[3] = boxer.body.abdomen.LRZ();
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = boxer.head.face.R();
	data[1] = boxer.head.face.Top();
	data[2] = boxer.head.face.Bottom();
	SDL_RWwrite(rwops, data, sizeof(float), 3);

	data[0] = boxer.head.face.JR();
	data[1] = boxer.head.face.JRX();
	data[2] = boxer.head.face.JRY();
	data[3] = boxer.head.face.JRZ();
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	return SDL_SUCCESS;
}

int BoxerHandler::SaveBinBoxerThink(SDL_RWops *rwops) {
/* --- Save Think Data --- */	
	SDL_RWwrite(rwops, &max_think, sizeof(int), 1);
	SDL_RWwrite(rwops, think, sizeof(Think), max_think);
	return SDL_SUCCESS;
}
	
int BoxerHandler::SaveBinBoxerOther(SDL_RWops *rwops) {
	float data[4];
/* --- Save Other Data --- */	
	SDL_RWwrite(rwops, nickname_jp, sizeof(char), 16);
	SDL_RWwrite(rwops, familyname_jp, sizeof(char), 16);
	SDL_RWwrite(rwops, nickname_en, sizeof(char), 16);
	SDL_RWwrite(rwops, familyname_en, sizeof(char), 16);
 
	data[0] = punch_manip;
	data[1] = speed_manip;
	data[2] = stamina_manip;
	SDL_RWwrite(rwops, data, sizeof(float), 3);
 
	data[0] = damage_restore_manip;
	data[1] = max_damage_manip;
	data[2] = cyanosis_restore_manip;
	data[3] = max_cyanosis_manip;
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = defense_shoulder_absorb;
	data[1] = defense_upperarm_absorb;
	data[2] = defense_forearm_absorb;
	data[3] = defense_fist_absorb;
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = normal_shoulder_absorb;
	data[1] = normal_upperarm_absorb;
	data[2] = normal_forearm_absorb;
	data[3] = normal_fist_absorb;
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = offense_shoulder_absorb;
	data[1] = offense_upperarm_absorb;
	data[2] = offense_forearm_absorb;
	data[3] = offense_fist_absorb;
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	data[0] = body_strength;
	data[1] = liver_strength;
	data[2] = face_strength;
	data[3] = jaw_strength;		
	SDL_RWwrite(rwops, data, sizeof(float), 4);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerFile(Boxer &boxer, MotionHandler &motion, const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "rb");
 
	if (rwops == NULL) {
		emes(" BoxerHandler::LoadBinBoxerFile") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = LoadBinBoxer(boxer, motion, rwops);
	SDL_RWclose(rwops);
	return flag;
}

int BoxerHandler::CalcDataSize(MotionHandler &motion) {
	int data_size, i, j, max_frame, max_combo;
	
	data_size = (2 + 14 + 14 + 17 + 17 + 18 + 12) * sizeof(float);
	data_size+= (17 * 5) * sizeof(float);

	data_size+= sizeof(int);
	for (i = 0; i < motion.max_number; i++) {
		max_frame = motion.bmotions[i].MaxNumber();
		data_size+= sizeof(int);
		for (j = 0; j < max_frame; j++) {
			data_size+= sizeof(int);
			data_size+= 44 * sizeof(float);
		}
	}
	
	max_combo = motion.combo.MaxNumber();
	data_size+= sizeof(int);
	data_size+= 2 * max_combo * sizeof(float);

	data_size+= MAX_ASSIGN * sizeof(int);
	
	data_size+= (4 + 4 + 3 + 4 + 3 + 4) * sizeof(float);

	data_size+= sizeof(int) + max_think * sizeof(Think);

	data_size+= 4 * 16 * sizeof(char);
	data_size+= (3 + 4 + 4 + 4 + 4 + 4) * sizeof(float);
	return data_size;		
}


int BoxerHandler::LoadBinBoxer(Boxer &boxer, MotionHandler &motion, SDL_RWops *rwops) {
	if (LoadBinBoxerBody(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (LoadBinBoxerMaterial(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}
	if (LoadBinBoxerMotion(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (LoadBinBoxerCombo(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}    
	if (LoadBinBoxerAssign(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (LoadBinBoxerRange(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (LoadBinBoxerThink(rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}       
	if (SaveDefaultThink() == SDL_FAILURE) {
		return SDL_FAILURE;	
	}       
	if (LoadBinBoxerOther(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}    
 
//	PrintBoxer(boxer, motion);
	return SDL_SUCCESS; 
}

int BoxerHandler::LoadBinBoxerBodySet(Boxer &boxer, SDL_RWops *rwops) {
	if (LoadBinBoxerBody(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
	if (LoadBinBoxerRange(boxer, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
 
	return SDL_SUCCESS; 
}

int BoxerHandler::LoadBinBoxerMotionSet(MotionHandler &motion, SDL_RWops *rwops) {
	if (LoadBinBoxerMotion(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}  
	if (LoadBinBoxerCombo(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}    
	if (LoadBinBoxerAssign(motion, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	} 
 
	return SDL_SUCCESS; 
}
 
int BoxerHandler::LoadBinBoxerBody(Boxer &boxer, SDL_RWops *rwops) {
	float r, a1, a2, b1, b2, c1, c2, data[6]; 
/* --- Load Bin Boxer Body Data --- */	
	SDL_RWread(rwops, data, sizeof(float), 2);
	boxer.SetShoulderWidth(data[0]);
	boxer.SetWaistWidth(data[1]);
/* Load Boxer Data Right Arm */	
	SDL_RWread(rwops, data, sizeof(float), 1);
	r = data[0];
	boxer.right_arm.fist.SetSize(r, r, r, r, r, r);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.right_arm.forearm.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.right_arm.upperarm.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 1);
	r = data[0];
	boxer.right_arm.shoulder.SetSize(r, r, r, r, r, r);
/* Load Boxer Data Left Arm */	
	SDL_RWread(rwops, data, sizeof(float), 1);
	r = data[0];
	boxer.left_arm.fist.SetSize(r, r, r, r, r, r);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.left_arm.forearm.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.left_arm.upperarm.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 1);
	r = data[0];
	boxer.left_arm.shoulder.SetSize(r, r, r, r, r, r);
/* Load Boxer Data Right Leg */	
	SDL_RWread(rwops, data, sizeof(float), 5);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	boxer.right_leg.toe.SetSize(a1, a2, b1, b2, c1, 0.0);
	boxer.right_leg.foot.SetSize(a1, a2, b1, b2, c1, 0.0);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.right_leg.shank.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.right_leg.thigh.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Left Leg */	
	SDL_RWread(rwops, data, sizeof(float), 5);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	boxer.left_leg.toe.SetSize(a1, a2, b1, b2, c1, 0.0);
	boxer.left_leg.foot.SetSize(a1, a2, b1, b2, c1, 0.0);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.left_leg.shank.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.left_leg.thigh.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Body */	
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.body.waist.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.body.abdomen.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.body.breast.SetSize(a1, a2, b1, b2, c1, c2);
/* Load Boxer Data Head */	
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.head.neck.SetSize(a1, a2, b1, b2, c1, c2);
	SDL_RWread(rwops, data, sizeof(float), 6);
	a1 = data[0];
	a2 = data[1];
	b1 = data[2];
	b2 = data[3];
	c1 = data[4];
	c2 = data[5];
	boxer.head.face.SetSize(a1, a2, b1, b2, c1, c2);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerMaterial(Boxer &boxer, SDL_RWops *rwops) {
	float r, g, b, a, s, data[4];
/* --- Load Material Data --- */
/* Load Material Data Glove */
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.glove.DefDiffuse(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.glove.DefAmbient(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.glove.DefSpecular(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.glove.DefEmission(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 1);
	s = data[0];
	boxer.glove.DefShininess(s);
/* Load Material Data Bandage */	
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.bandage.DefDiffuse(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.bandage.DefAmbient(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.bandage.DefSpecular(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.bandage.DefEmission(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 1);
	s = data[0];
	boxer.bandage.DefShininess(s);
/* Load Material Data Skin */	
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.skin.DefDiffuse(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.skin.DefAmbient(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.skin.DefSpecular(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.skin.DefEmission(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 1);
	s = data[0];
	boxer.skin.DefShininess(s);
/* Load Material Data Trunks */	
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.trunks.DefDiffuse(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.trunks.DefAmbient(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.trunks.DefSpecular(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.trunks.DefEmission(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 1);
	s = data[0];
	boxer.trunks.DefShininess(s);
/* Load Material Data Shoes */	
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.shoes.DefDiffuse(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.shoes.DefAmbient(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.shoes.DefSpecular(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	g = data[1];
	b = data[2];
	a = data[3];
	boxer.shoes.DefEmission(r, g, b, a);
	SDL_RWread(rwops, data, sizeof(float), 1);
	s = data[0];
	boxer.shoes.DefShininess(s);
/* Load Material Data Hair */	
/*	boxer.SetHairBlack(); */
	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerMotion(MotionHandler &motion, SDL_RWops *rwops) {
	int i, j, max_motion, max_handle, max_frame, number;
/* --- Load Motion Data --- */
	max_handle = 0;
	SDL_RWread(rwops, &max_motion, sizeof(int), 1);
	motion.Init(max_motion);

	if (max_motion > 100) {
		emes("BoxerHandler::LoadBinBoxerMotion -> Motion number is wrong.");
		return SDL_FAILURE;
	}

	for (i = 0; i < max_motion; i++) {
		SDL_RWread(rwops, &max_frame, sizeof(int), 1);
		if (max_frame > 80 || max_frame < 2) {
			emes("BoxerHandler::LoadBinBoxerMotion -> Frame number is wrong.");
			return SDL_FAILURE;
		}
		motion[i].Init(max_frame);
		if (max_frame > max_handle) {
		 	max_handle = max_frame;
		}
		for (j = 0; j < max_frame; j++) {
			SDL_RWread(rwops, &number, sizeof(int), 1);
			motion[i][j].SetNumber(number);	
 			SDL_RWread(rwops, &motion[i][j][0], sizeof(float), 44);
			motion[i].Link();
	    } 
	} 
	motion.hmotion.Init(max_handle + 2);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerCombo(MotionHandler &motion, SDL_RWops *rwops) {
	int max_combo;
/* --- Load Combination --- Data */	
	SDL_RWread(rwops, &max_combo, sizeof(int), 1);
	if (max_combo > 200 || max_combo < 1) {
		emes("BoxerHandler::LoadBinBoxerCombo");
		return SDL_FAILURE;
	}
	motion.combo.Init(max_combo);
	SDL_RWread(rwops, motion.combo.GetComboFirst(), sizeof(int), max_combo);
	SDL_RWread(rwops, motion.combo.GetComboSecond(), sizeof(int), max_combo);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerAssign(MotionHandler &motion, SDL_RWops *rwops) {
	SDL_RWread(rwops, motion.assign, sizeof(int), MAX_ASSIGN);
	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerRange(Boxer &boxer, SDL_RWops *rwops) {
	float x, y, z, r, t, b, data[4];
/* --- Load Range Data --- */	
	SDL_RWread(rwops, data, sizeof(float), 4);
	boxer.right_arm.fist.SetRange(data[0]);
	boxer.right_arm.forearm.SetRange(data[1]);
	boxer.right_arm.upperarm.SetRange(data[2]);
	boxer.right_arm.shoulder.SetRange(data[3]);
	SDL_RWread(rwops, data, sizeof(float), 4);
	boxer.left_arm.fist.SetRange(data[0]);
	boxer.left_arm.forearm.SetRange(data[1]);
	boxer.left_arm.upperarm.SetRange(data[2]);
	boxer.left_arm.shoulder.SetRange(data[3]);

	SDL_RWread(rwops, data, sizeof(float), 3);
	r = data[0];
	t = data[1];
	b = data[2];
	boxer.body.abdomen.SetRange(r, t, b);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	x = data[1];
	y = data[2];
	z = data[3];
	boxer.body.abdomen.SetLiver(r, x, y, z);

	SDL_RWread(rwops, data, sizeof(float), 3);
	r = data[0];
	t = data[1];
	b = data[2];
	boxer.head.face.SetRange(r, t, b);
	SDL_RWread(rwops, data, sizeof(float), 4);
	r = data[0];
	x = data[1];
	y = data[2];
	z = data[3];
	boxer.head.face.SetJaw(r, x, y, z);

	return SDL_SUCCESS;
}

int BoxerHandler::LoadBinBoxerThink(SDL_RWops *rwops) {
/* --- Load Think Data --- */	
	SDL_RWread(rwops, &max_think, sizeof(int), 1);
	if (max_think > 99 || max_think < 1) {
		emes("BoxerHandler::LoadBinBoxerThink -> Routine number is wrong.");
		return SDL_FAILURE;		
	}
	if (think != NULL) {
		delete [] think;
	} 
	think = new Think[max_think];
	if (think == NULL) {
		emes("BoxerHandler::LoadBinBoxerThink -> Out of Memory");
		return SDL_FAILURE;		
	}
	
	SDL_RWread(rwops, think, sizeof(Think), max_think);
#if 0
	int i;
	for (i = 0; i < max_think; i++) {
		think[i].mswing_greater -= 1.0;
		think[i].mswing_less -= 1.0;
		think[i].mspeed_greater -= 1.0;
		think[i].mspeed_less -= 1.0;

		think[i].oswing_greater -= 1.0;
		think[i].oswing_less -= 1.0;
		think[i].ospeed_greater -= 1.0;
		think[i].ospeed_less -= 1.0; *
	}  
#endif
	return SDL_SUCCESS;
}
	
int BoxerHandler::LoadBinBoxerOther(Boxer &boxer, SDL_RWops *rwops) {
	float data[4];
/* --- Load Other Data --- */	
	SDL_RWread(rwops, nickname_jp, sizeof(char), 16);
	SDL_RWread(rwops, familyname_jp, sizeof(char), 16);
	SDL_RWread(rwops, nickname_en, sizeof(char), 16);
	SDL_RWread(rwops, familyname_en, sizeof(char), 16);
 
	SDL_RWread(rwops, data, sizeof(float), 3);
	punch_manip = data[0];
	speed_manip = data[1];
	stamina_manip = data[2];
 
	SDL_RWread(rwops, data, sizeof(float), 4);
	damage_restore_manip = data[0];
	max_damage_manip = data[1];
	cyanosis_restore_manip = data[2];
	max_cyanosis_manip = data[3];

	SDL_RWread(rwops, data, sizeof(float), 4);
	defense_shoulder_absorb = data[0];
	defense_upperarm_absorb = data[1];
	defense_forearm_absorb = data[2];
	defense_fist_absorb = data[3];

	SDL_RWread(rwops, data, sizeof(float), 4);
	normal_shoulder_absorb = data[0];
	normal_upperarm_absorb = data[1];
	normal_forearm_absorb = data[2];
	normal_fist_absorb = data[3];

	SDL_RWread(rwops, data, sizeof(float), 4);
	offense_shoulder_absorb = data[0];
	offense_upperarm_absorb = data[1];
	offense_forearm_absorb = data[2];
	offense_fist_absorb = data[3];

	SDL_RWread(rwops, data, sizeof(float), 4);
	body_strength = data[0];
	liver_strength = data[1];
	face_strength = data[2];
	jaw_strength = data[3];		

	return SDL_SUCCESS;
}

int BoxerHandler::LoadDefaultThink() {
	int i;
/* --- Load Think Data --- */	
	max_think = max_dthink;
	if (think != NULL) {
		delete [] think;
	} 
	think = new Think[max_think];
	if (think == NULL) {
		emes("BoxerHandler::LoadDefaultThink -> Out of Memory");
		return SDL_FAILURE;		
	}
	for (i = 0; i < max_think; i++) {
		think[i] = dthink[i];	
	}
	return SDL_SUCCESS;		
}

int BoxerHandler::SaveDefaultThink() {
	int i;
/* --- Save Think Data --- */	
	max_dthink = max_think;
	if (dthink != NULL) {
		delete [] dthink;
	} 
	dthink = new Think[max_dthink];
	if (dthink == NULL) {
		emes("BoxerHandler::SaveDefaultThink -> Out of Memory");
		return SDL_FAILURE;		
	}
	for (i = 0; i < max_dthink; i++) {
		dthink[i] = think[i];	
	}
	return SDL_SUCCESS;		
}

void BoxerHandler::PrintBoxer(Boxer &boxer, MotionHandler &motion) {
	fprintf(stderr, "--- Boxer Data ---\n");
	PrintBody(boxer);
	PrintCombo(motion);
	PrintAssign(motion);
	PrintRange(boxer);
	PrintOther();
}

void BoxerHandler::PrintBody(Boxer &boxer) {
	fprintf(stderr, "%f\n", boxer.ShoulderWidth());
	fprintf(stderr, "%f\n\n", boxer.WaistWidth());

	fprintf(stderr, "%f\n", boxer.right_arm.fist.A1());
	fprintf(stderr, "%f\n", boxer.right_arm.fist.A2());
	fprintf(stderr, "%f\n", boxer.right_arm.fist.B1());
	fprintf(stderr, "%f\n", boxer.right_arm.fist.B2());
	fprintf(stderr, "%f\n", boxer.right_arm.fist.C1());
	fprintf(stderr, "%f\n\n", boxer.right_arm.fist.C2());

	fprintf(stderr, "%f\n", boxer.right_arm.forearm.A1());
	fprintf(stderr, "%f\n", boxer.right_arm.forearm.A2());
	fprintf(stderr, "%f\n", boxer.right_arm.forearm.B1());
	fprintf(stderr, "%f\n", boxer.right_arm.forearm.B2());
	fprintf(stderr, "%f\n", boxer.right_arm.forearm.C1());
	fprintf(stderr, "%f\n\n", boxer.right_arm.forearm.C2());

	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.A1());
	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.A2());
	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.B1());
	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.B2());
	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.C1());
	fprintf(stderr, "%f\n\n", boxer.right_arm.upperarm.C2());

	fprintf(stderr, "%f\n", boxer.right_arm.shoulder.A1());
	fprintf(stderr, "%f\n", boxer.right_arm.shoulder.A2());
	fprintf(stderr, "%f\n", boxer.right_arm.shoulder.B1());
	fprintf(stderr, "%f\n", boxer.right_arm.shoulder.B2());
	fprintf(stderr, "%f\n", boxer.right_arm.shoulder.C1());
	fprintf(stderr, "%f\n\n", boxer.right_arm.shoulder.C2());

	fprintf(stderr, "%f\n", boxer.left_arm.fist.A1());
	fprintf(stderr, "%f\n", boxer.left_arm.fist.A2());
	fprintf(stderr, "%f\n", boxer.left_arm.fist.B1());
	fprintf(stderr, "%f\n", boxer.left_arm.fist.B2());
	fprintf(stderr, "%f\n", boxer.left_arm.fist.C1());
	fprintf(stderr, "%f\n\n", boxer.left_arm.fist.C2());

	fprintf(stderr, "%f\n", boxer.left_arm.forearm.A1());
	fprintf(stderr, "%f\n", boxer.left_arm.forearm.A2());
	fprintf(stderr, "%f\n", boxer.left_arm.forearm.B1());
	fprintf(stderr, "%f\n", boxer.left_arm.forearm.B2());
	fprintf(stderr, "%f\n", boxer.left_arm.forearm.C1());
	fprintf(stderr, "%f\n\n", boxer.left_arm.forearm.C2());

	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.A1());
	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.A2());
	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.B1());
	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.B2());
	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.C1());
	fprintf(stderr, "%f\n\n", boxer.left_arm.upperarm.C2());

	fprintf(stderr, "%f\n", boxer.left_arm.shoulder.A1());
	fprintf(stderr, "%f\n", boxer.left_arm.shoulder.A2());
	fprintf(stderr, "%f\n", boxer.left_arm.shoulder.B1());
	fprintf(stderr, "%f\n", boxer.left_arm.shoulder.B2());
	fprintf(stderr, "%f\n", boxer.left_arm.shoulder.C1());
	fprintf(stderr, "%f\n\n", boxer.left_arm.shoulder.C2());

	fprintf(stderr, "%f\n", boxer.right_leg.toe.A1());
	fprintf(stderr, "%f\n", boxer.right_leg.toe.A2());
	fprintf(stderr, "%f\n", boxer.right_leg.toe.B1());
	fprintf(stderr, "%f\n", boxer.right_leg.toe.B2());
	fprintf(stderr, "%f\n", boxer.right_leg.toe.C1());
	fprintf(stderr, "%f\n\n", boxer.right_leg.toe.C2());

	fprintf(stderr, "%f\n", boxer.right_leg.foot.A1());
	fprintf(stderr, "%f\n", boxer.right_leg.foot.A2());
	fprintf(stderr, "%f\n", boxer.right_leg.foot.B1());
	fprintf(stderr, "%f\n", boxer.right_leg.foot.B2());
	fprintf(stderr, "%f\n", boxer.right_leg.foot.C1());
	fprintf(stderr, "%f\n\n", boxer.right_leg.foot.C2());

	fprintf(stderr, "%f\n", boxer.right_leg.shank.A1());
	fprintf(stderr, "%f\n", boxer.right_leg.shank.A2());
	fprintf(stderr, "%f\n", boxer.right_leg.shank.B1());
	fprintf(stderr, "%f\n", boxer.right_leg.shank.B2());
	fprintf(stderr, "%f\n", boxer.right_leg.shank.C1());
	fprintf(stderr, "%f\n\n", boxer.right_leg.shank.C2());

	fprintf(stderr, "%f\n", boxer.right_leg.thigh.A1());
	fprintf(stderr, "%f\n", boxer.right_leg.thigh.A2());
	fprintf(stderr, "%f\n", boxer.right_leg.thigh.B1());
	fprintf(stderr, "%f\n", boxer.right_leg.thigh.B2());
	fprintf(stderr, "%f\n", boxer.right_leg.thigh.C1());
	fprintf(stderr, "%f\n\n", boxer.right_leg.thigh.C2());

	fprintf(stderr, "%f\n", boxer.left_leg.toe.A1());
	fprintf(stderr, "%f\n", boxer.left_leg.toe.A2());
	fprintf(stderr, "%f\n", boxer.left_leg.toe.B1());
	fprintf(stderr, "%f\n", boxer.left_leg.toe.B2());
	fprintf(stderr, "%f\n", boxer.left_leg.toe.C1());
	fprintf(stderr, "%f\n\n", boxer.left_leg.toe.C2());

	fprintf(stderr, "%f\n", boxer.left_leg.foot.A1());
	fprintf(stderr, "%f\n", boxer.left_leg.foot.A2());
	fprintf(stderr, "%f\n", boxer.left_leg.foot.B1());
	fprintf(stderr, "%f\n", boxer.left_leg.foot.B2());
	fprintf(stderr, "%f\n", boxer.left_leg.foot.C1());
	fprintf(stderr, "%f\n\n", boxer.left_leg.foot.C2());

	fprintf(stderr, "%f\n", boxer.left_leg.shank.A1());
	fprintf(stderr, "%f\n", boxer.left_leg.shank.A2());
	fprintf(stderr, "%f\n", boxer.left_leg.shank.B1());
	fprintf(stderr, "%f\n", boxer.left_leg.shank.B2());
	fprintf(stderr, "%f\n", boxer.left_leg.shank.C1());
	fprintf(stderr, "%f\n\n", boxer.left_leg.shank.C2());

	fprintf(stderr, "%f\n", boxer.left_leg.thigh.A1());
	fprintf(stderr, "%f\n", boxer.left_leg.thigh.A2());
	fprintf(stderr, "%f\n", boxer.left_leg.thigh.B1());
	fprintf(stderr, "%f\n", boxer.left_leg.thigh.B2());
	fprintf(stderr, "%f\n", boxer.left_leg.thigh.C1());
	fprintf(stderr, "%f\n\n", boxer.left_leg.thigh.C2());

	fprintf(stderr, "%f\n", boxer.body.waist.A1());
	fprintf(stderr, "%f\n", boxer.body.waist.A2());
	fprintf(stderr, "%f\n", boxer.body.waist.B1());
	fprintf(stderr, "%f\n", boxer.body.waist.B2());
	fprintf(stderr, "%f\n", boxer.body.waist.C1());
	fprintf(stderr, "%f\n\n", boxer.body.waist.C2());

	fprintf(stderr, "%f\n", boxer.body.abdomen.A1());
	fprintf(stderr, "%f\n", boxer.body.abdomen.A2());
	fprintf(stderr, "%f\n", boxer.body.abdomen.B1());
	fprintf(stderr, "%f\n", boxer.body.abdomen.B2());
	fprintf(stderr, "%f\n", boxer.body.abdomen.C1());
	fprintf(stderr, "%f\n\n", boxer.body.abdomen.C2());

	fprintf(stderr, "%f\n", boxer.body.breast.A1());
	fprintf(stderr, "%f\n", boxer.body.breast.A2());
	fprintf(stderr, "%f\n", boxer.body.breast.B1());
	fprintf(stderr, "%f\n", boxer.body.breast.B2());
	fprintf(stderr, "%f\n", boxer.body.breast.C1());
	fprintf(stderr, "%f\n\n", boxer.body.breast.C2());

	fprintf(stderr, "%f\n", boxer.head.neck.A1());
	fprintf(stderr, "%f\n", boxer.head.neck.A2());
	fprintf(stderr, "%f\n", boxer.head.neck.B1());
	fprintf(stderr, "%f\n", boxer.head.neck.B2());
	fprintf(stderr, "%f\n", boxer.head.neck.C1());
	fprintf(stderr, "%f\n\n", boxer.head.neck.C2());

	fprintf(stderr, "%f\n", boxer.head.face.A1());
	fprintf(stderr, "%f\n", boxer.head.face.A2());
	fprintf(stderr, "%f\n", boxer.head.face.B1());
	fprintf(stderr, "%f\n", boxer.head.face.B2());
	fprintf(stderr, "%f\n", boxer.head.face.C1());
	fprintf(stderr, "%f\n\n", boxer.head.face.C2());

}

void BoxerHandler::PrintCombo(MotionHandler &motion) {
	int i;
	for (i = 0; i < motion.combo.MaxNumber(); i++) {
		fprintf(stderr, "%d %d\n", *(motion.combo.GetComboFirst() + i), *(motion.combo.GetComboSecond() + i));
	}
}

void BoxerHandler::PrintAssign(MotionHandler &motion) {
}

void BoxerHandler::PrintRange(Boxer &boxer) {
	fprintf(stderr, "%f\n", boxer.right_arm.fist.R());
	fprintf(stderr, "%f\n", boxer.right_arm.forearm.R());
	fprintf(stderr, "%f\n", boxer.right_arm.upperarm.R());
	fprintf(stderr, "%f\n\n", boxer.right_arm.shoulder.R());

	fprintf(stderr, "%f\n", boxer.left_arm.fist.R());
	fprintf(stderr, "%f\n", boxer.left_arm.forearm.R());
	fprintf(stderr, "%f\n", boxer.left_arm.upperarm.R());
	fprintf(stderr, "%f\n\n", boxer.left_arm.shoulder.R());

	fprintf(stderr, "%f\n", boxer.body.abdomen.R());
	fprintf(stderr, "%f\n", boxer.body.abdomen.Top());
	fprintf(stderr, "%f\n\n",boxer.body.abdomen.Bottom());

	fprintf(stderr, "%f\n", boxer.body.abdomen.LR());
	fprintf(stderr, "%f\n", boxer.body.abdomen.LRX());
	fprintf(stderr, "%f\n", boxer.body.abdomen.LRY());
	fprintf(stderr, "%f\n\n",boxer.body.abdomen.LRZ());

	fprintf(stderr, "%f\n", boxer.head.face.R());
	fprintf(stderr, "%f\n", boxer.head.face.Top());
	fprintf(stderr, "%f\n\n",boxer.head.face.Bottom());

	fprintf(stderr, "%f\n", boxer.head.face.JR());
	fprintf(stderr, "%f\n", boxer.head.face.JRX());
	fprintf(stderr, "%f\n", boxer.head.face.JRY());
	fprintf(stderr, "%f\n\n",boxer.head.face.JRZ());
}

void BoxerHandler::PrintOther() {
	fprintf(stderr, "%s\n", nickname_jp);
	fprintf(stderr, "%s\n", familyname_jp);
	fprintf(stderr, "%s\n", nickname_en);
	fprintf(stderr, "%s\n\n", familyname_en);

	fprintf(stderr, "%f\n", punch_manip);
	fprintf(stderr, "%f\n", speed_manip);
	fprintf(stderr, "%f\n\n", stamina_manip);

	fprintf(stderr, "%f\n", damage_restore_manip);
	fprintf(stderr, "%f\n", max_damage_manip);
	fprintf(stderr, "%f\n", cyanosis_restore_manip);
	fprintf(stderr, "%f\n\n", cyanosis_restore_manip);

	fprintf(stderr, "%f\n", defense_shoulder_absorb);
	fprintf(stderr, "%f\n", defense_upperarm_absorb);
	fprintf(stderr, "%f\n", defense_forearm_absorb);
	fprintf(stderr, "%f\n\n", defense_fist_absorb);

	fprintf(stderr, "%f\n", normal_shoulder_absorb);
	fprintf(stderr, "%f\n", normal_upperarm_absorb);
	fprintf(stderr, "%f\n", normal_forearm_absorb);
	fprintf(stderr, "%f\n\n", normal_fist_absorb);

	fprintf(stderr, "%f\n", offense_shoulder_absorb);
	fprintf(stderr, "%f\n", offense_upperarm_absorb);
	fprintf(stderr, "%f\n", offense_forearm_absorb);
	fprintf(stderr, "%f\n\n", offense_fist_absorb);

	fprintf(stderr, "%f\n", body_strength);
	fprintf(stderr, "%f\n", liver_strength);
	fprintf(stderr, "%f\n", face_strength);
	fprintf(stderr, "%f\n\n", jaw_strength);
}
