#include "creader.h"

CReader::CReader() {
	lines = NULL;
}

CReader::CReader(const char* filename) {
	Open(filename);
}

CReader::~CReader() {
	Close();
}

int CReader::Open(const char* filename) {
	int i;	
	SDL_RWops *rwops;

	if (WasOpen()) {
		Close();	
	}

	rwops = SDL_RWFromFile(filename, "rb");
	if (rwops == NULL) {
		emes("CReader::Open -> Can't opne file.");
		efile(filename) ;		
		return SDL_FAILURE;
	}

	SDL_RWseek(rwops, 0, SEEK_END);
	filesize = SDL_RWtell(rwops);
	SDL_RWseek(rwops, 0, SEEK_SET);
	buf = new char[filesize + 1];
	SDL_RWread(rwops, buf, sizeof(char), filesize);
	SDL_RWclose(rwops);
//	SDL_FreeRW(rwops);

	mnumber = 0;
	for (i = 0; i < FileSize() && buf[i]; i++) {
		if (buf[i] == 0x0A) {
			mnumber++;
	//		fprintf(stdout, "line =%d i = %d\n", mnumber, i);
		}
	}
	mnumber++;
	buf[i] = '\0';

	if (mnumber == 0) {
		emes("CReader::Open -> File is empty.");
		efile(filename) ;		
		return SDL_FAILURE;	
	}

	MakeLines();
	Rewind();
	if (!WasOpen()) {
		emes("CReader::Open -> Can't read file.");
		efile(filename);		
		return SDL_FAILURE;	
	}
	smes("CReader::Open");	
	sfile(filename);	
	return SDL_SUCCESS;	
}

int CReader::OpenFrom(char* data, int size) {
	int i;	

	filesize = size;
	buf = new char[filesize + 1];
	memcpy(buf, data, size);
	mnumber = 0;
	for (i = 0; i < FileSize() && buf[i]; i++) {
		if (buf[i] == 0x0A) {
			mnumber++;
		}
	}
	mnumber++;
	buf[i] = '\0';
	if (mnumber == 0) {
		return SDL_FAILURE;	
	}

	MakeLines();
	Rewind();
	if (!WasOpen()) {
		return SDL_FAILURE;	
	}
	return SDL_SUCCESS;	
}

void CReader::Close() {
	if (WasOpen()) {
		delete [] buf;
		delete [] lines;
		lines = NULL;
		smes("CReader::Close");	
	}
}

void CReader::MakeLines() {
	lines = new LINE[mnumber];
	int i, len = 0, line_n = 0;
	
	lines[line_n].cp = buf;	
	for (i = 0; i < FileSize() && buf[i]; i++) {
		len++;
		if (buf[i] == 0x0A) {
			lines[line_n].len = len;
			buf[i] = '\0';
			len = 0;
			line_n++;
			if (line_n < mnumber) {
				lines[line_n].cp = buf + i + 1;
			}
		}
	}
}

void CReader::NextChar(char* str){
	int i;
	cp = CurrentLine();	
	while (cp[0] == '#' || cp[0] == '\0' || cp[0] == 0x0D) {
		NextLine();
		cp = CurrentLine();	
	}
	for (i = 0; i < MAX_LINE_SIZE -1 &&
		cp[i] != ' '  && cp[i] != ';' &&
		cp[i] != '\0' && cp[i] != 0x0D; i++) {
		str[i] = cp[i];
	}
	str[i] = '\0';
	NextLine();
}	

void CReader::NextLine(char* str){
	int i;
	cp = CurrentLine();	
	while (cp[0] == '#' || cp[0] == '\0' || cp[0] == 0x0D) {
		NextLine();
		cp = CurrentLine();	
	}
	for (i = 0; i < MAX_LINE_SIZE -1 &&
		cp[i] != '\0' && cp[i] != 0x0D; i++) {
		str[i] = cp[i];
	}
	str[i] = '\0';
	NextLine();
}	

void CReader::Skip() {
	NextChar(line);
}	

int CReader::NextInt() {
	NextChar(line);
	return atoi(line);
}	

float CReader::NextIntF(int number) {
	return (float)NextInt()/(float)number;
}

float CReader::NextFloat() {
	NextChar(line);
	return atof(line);
}

void CReader::Rewind() {
	cnumber = 0;
	is_last = false;
}

void CReader::NextLine() {
	cnumber++;
	if (cnumber >= mnumber) {
		cnumber = 0;
		is_last = true;	
	}
}

char* CReader::CurrentLine() {
	return lines[cnumber].cp;
}	

char* CReader::GetLine(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
	return CurrentLine();
}

void CReader::Back(int number) {
	cnumber -= number;
	if (cnumber < 0) {
		cnumber = 0;	
	}
}	

void CReader::Print() {
	int i;
	for (i = 0; i < mnumber; i++) {
		fprintf(stdout, "%d : %s", i, lines[i].cp);	
	}
}

bool CReader::WasOpen() {
	if (lines == NULL) {
		return false;		
	}
	return true;	
}
