#include "csdl.h"

/* SDL_INIT_TIMER */
STimer::STimer() {
	Init();
}

STimer::~STimer() {
	Quit();
}

int STimer::Init() {
	if (WasInit()) {
		Quit();	
	}
	SDL_InitSubSystem(SDL_INIT_TIMER);
	if (!WasInit()) {
			emes("STimer::Init");
			rmes();	
			return SDL_FAILURE;
	}
	smes("STimer::Init");	
	SetFPS(60);
	return SDL_SUCCESS;
}

void STimer::Quit() {
	if (WasInit()) {
		SDL_QuitSubSystem(SDL_INIT_TIMER);
		smes("STimer::Quit");	
	} else {
		wmes("STimer::Quit");		
	}
}

bool STimer::WasInit() {
	return SDL_INIT_TIMER & SDL_WasInit(SDL_INIT_TIMER);
}

char *STimer::FPSString() {
	sprintf(fps_str,"Now %d FPS", GetFPS());
	return fps_str;
}

Uint32 STimer::GetTicks() {
	return SDL_GetTicks();
}

void STimer::Delay(Uint32 ms) {
	SDL_Delay(ms);
}

/* t[[ǧ */
void STimer::SetFPS(int input_fps) {
	fps = input_fps;
	fpsi = 1000 / fps;
	time_now = GetTicks();
	count = 0;
}

/* t[[g̎擾 */
int STimer::GetFPS() {
	if (time_now > time_pre) {
		return 1000 / (time_now - time_pre);
	} else {
		return 0;
	}
}

/* t[[gɂ */
Uint32 STimer::Wait() {
	time_pre = time_now;
	delay = fpsi + time_pre;
	time_now = GetTicks();
	if (delay > time_now) {
		delay -= time_now;
		Delay(delay);	
		time_now += delay; 
	} else if ((time_now - delay) > (fpsi / 5) ) {
		delay = 0;
		count++;
		if (count > (Uint32)fps) { // ܂xƂɂ͌x
#ifdef DEBUG
			wmes("Machine Power is low.");
			fprintf(stderr, "WARNING : Required : %dFPS Now : %dFPS \n", fps, 1000/(time_now - time_pre));
#endif
			count = 0;
		}
	}
	return delay;
}

/* SDL_INIT_VIDEO */
SVideo::SVideo() {
	Init();
}

SVideo::~SVideo() {
	Quit();
}

int SVideo::Init() {
	if (WasInit()) {
 		Quit();
	}
	SDL_InitSubSystem(SDL_INIT_VIDEO);
	if (!WasInit()) {
		emes("SVideo::Init");
		rmes();	
		return SDL_FAILURE;
	}
	smes("SVideo::Init");	
	return SDL_SUCCESS;
}

void SVideo::Quit() {
	if (WasInit()) {
		SDL_QuitSubSystem(SDL_INIT_VIDEO);
		smes("SVideo::Quit");	
	} else {
		wmes("SVideo::Quit");		
	}
}

bool SVideo::WasInit() {
	return SDL_INIT_VIDEO & SDL_WasInit(SDL_INIT_VIDEO);
}

/* SDL_INIT_AUDIO */
SAudio::SAudio() {
	Init();
}

SAudio::~SAudio() {
	Quit();
}

int SAudio::Init() {
	if (WasInit()) {
		Quit();
	}
	SDL_InitSubSystem(SDL_INIT_AUDIO);
	if (!WasInit()) {
		emes("SAudio::Init");	
		rmes();
		return SDL_FAILURE;
	}
	smes("SAudio::Init");	
	return SDL_SUCCESS;
}

void SAudio::Quit() {
	if (WasInit()) {
		SDL_QuitSubSystem(SDL_INIT_AUDIO);
		smes("SAudio::Quit");	
	}
}

bool SAudio::WasInit() {
	return SDL_INIT_AUDIO & SDL_WasInit(SDL_INIT_AUDIO);
}

/* Cxg */

SEvent::SEvent() {
	Init();
}

SEvent::~SEvent() {
	delete [] keys;	
	delete [] keys_pre;
	delete [] keys_count;	
}

void SEvent::Init() {
	int i;
//	SDL_EnableUNICODE(1);
	state = GAME_NOEVENT;
	keys_state = SDL_GetKeyState(&keys_number);
	keys = new int[keys_number];
	keys_pre = new int[keys_number];
	keys_count = new int[keys_number];
	for (i = 0; i < keys_number; i++) {
		keys[i] = keys_state[i];
		keys_pre[i] = keys_state[i];
		keys_count[i] = 0;
	}
	buttons = 0;
	buttons_pre = 0;
	Clear();
}

void SEvent::Clear() {
    while(SDL_PollEvent(&event)) {}
}
	
void SEvent::Poll() {
	state = GAME_CONTINUE;
    while(SDL_PollEvent(&event)) {
		Select();
		if (state == GAME_EXIT || state == GAME_EXPOSE) {
			break;
		}
	}
}

void SEvent::Wait() {
	Clear();
	SDL_WaitEvent(&event);
	Select();
}

void SEvent::Select() {
    switch(event.type) {
        case SDL_QUIT:
			state = GAME_EXIT;
        break;
		case SDL_KEYDOWN:
			state = KeyDown();
		break;
		case SDL_KEYUP:
			state = KeyUp();
		break;
		case SDL_MOUSEBUTTONDOWN:
			state = MouseDown();
		break; 
		case SDL_MOUSEBUTTONUP:
			state = MouseUp();
		break; 
		case SDL_MOUSEMOTION:
			state = MouseMotion();
		break; 
		case SDL_VIDEOEXPOSE:
			state = GAME_EXPOSE;
		break;     
		default:
			state = GAME_CONTINUE;
		break;
    }
}

int SEvent::KeyDown() {
	keys_pre[event.key.keysym.sym] = keys[event.key.keysym.sym];
	keys[event.key.keysym.sym] = SDL_PRESSED;
	if (keys_pre[event.key.keysym.sym] != keys[event.key.keysym.sym]) {
		keys_count[event.key.keysym.sym]++;
	}
	switch (event.key.keysym.sym) {
		case SDLK_ESCAPE:
    		return GAME_EXIT;
		break;
		default:
		 	return GAME_CONTINUE;
		break;
    }
}

int SEvent::KeyUp() {
	keys_pre[event.key.keysym.sym] = keys[event.key.keysym.sym];
	keys[event.key.keysym.sym] = SDL_RELEASED;
	switch (event.key.keysym.sym) {
		case SDLK_ESCAPE:
    		return GAME_EXIT;
		break;
		case SDLK_RETURN:
		case SDLK_SPACE:
			return GAME_NEXT;
		break;    
		default:
			return GAME_CONTINUE;
		break;
    }
}

int SEvent::MouseDown() {
	xo = MouseX();
	yo = MouseY();	
	buttons_pre = buttons;
	buttons |= SDL_BUTTON(event.button.button);
	return GAME_NEXT;	
}

int SEvent::MouseUp() {
	if (event.button.button == 4) {
		wheel = 1;	
	}
	if (event.button.button == 5) {
		wheel = 0;	
	}
	buttons_pre = buttons;
	buttons &= ~SDL_BUTTON(event.button.button);
	return GAME_CONTINUE;
}

int SEvent::MouseMotion() {
	x = event.motion.x;
	y = event.motion.y;
	xrel = event.motion.xrel;
	yrel = event.motion.yrel;
	return GAME_CONTINUE;
}

bool SEvent::IsNoEvent() {
	if (state == GAME_NOEVENT) {
		return true;
	}
	return false;	
}

bool SEvent::IsContinue() {
	if (state == GAME_CONTINUE) {
		return true;
	}
	return false;	
}

bool SEvent::IsQuit() {
	if (state == GAME_EXIT) {
		return true;
	}
	return false;	
}

bool SEvent::IsNext() {
	if (state == GAME_NEXT) {
		return true;
	}
	return false;	
}

bool SEvent::IsExpose() {
	if (state == GAME_EXPOSE) {
		return true;
	}
	return false;	
}

void SetMinMax(int *value, int minimum, int maximum) {
	if (*value < minimum) {
		*value = minimum;	
	}
	if (*value > maximum) {
		*value = maximum;	
	}
}

int GetSDLKey(char *key_name, int default_key) {
	int i, numkeys;
	SDL_GetKeyState(&numkeys);
	for (i = 0; i < numkeys; i++) {
		if (!strcmp(SDL_GetKeyName((SDLKey)i), key_name)) {
		 	return i;
		}
	}
	return default_key;
}

