#include "game.h"
#include "icon.h"

void Game::Config() {
	ReadConfig();
	ScreenConfig();
	SoundConfig();
	ReplayConfig();
	FontConfig();
	ImageConfig();
	BGMConfig();
	SEConfig();
	DataConfig();
	ThinkConfig();
//	WriteConfig();
}

void Game::ReadConfig() {
	char key_name[MAX_LINE_SIZE];
	if (reader.Open(CONFIG_FILE) == SDL_FAILURE) {
		ScreenSize = 0;
		ScreenMode = 0;

		SoundFreq = 0;
		SoundFormat = 0;
		SoundChannel = 0;
		SoundBuffer = 0;

		BGMVolume = -8;
		SEVolume = -8;

		lang = Japanese;
		MakeFromTTF = 1;
		UsePText = 1;

		Partition = 24;
		FaceStyle = 2;
		meterspec = 0.6;
		ShadowSwitch = 1;
		BackGroundSwitch = 2;
		AdsSwitch = 1;

		Mag_Linear = 1;
		Min_Linear = 0;
		SkipFrame = 1;

		strcpy(UserName, "user");
		strcpy(HostName, "localhost");
		strcpy(HostCGI, "hpcgi2.nifty.com");
		strcpy(MatchMakeCGI, "/FengDan/matchmake.cgi");
		strcpy(LobbyCGI, "/FengDan/lobby.cgi");
		Port = 2000;
		PortLobby = 3000;
		LobbyMode = 1;

		DefaultView = 0;
		Angle = 10;
		Rotation = 45;
		SetAngle();
		Brightness = 0;
		SkipOpening = 0;
		SkipReplay = 0;
		SkipDown = 0;
		
		Player1Mode = 0;
		Player2Mode = 0;
		UseMouse = 1;
		go_next = SDLK_SPACE;

		KB1LeftOffense = SDLK_z;
		KB1RightOffense = SDLK_x;
		KB1Defense = SDLK_c;
		KB1Forward = SDLK_RIGHT;
		KB1Back = SDLK_LEFT;
		KB1Right = SDLK_DOWN;
		KB1Left = SDLK_UP;

		KB2LeftOffense = SDLK_z;
		KB2RightOffense = SDLK_x;
		KB2Defense = SDLK_c;;
		KB2Forward = SDLK_m;
		KB2Back = SDLK_b;
		KB2Right = SDLK_n;
		KB2Left = SDLK_h;

		KB3LeftOffense = SDLK_PERIOD;
		KB3RightOffense = SDLK_SLASH;
		KB3Defense = SDLK_BACKSLASH;
		KB3Forward = SDLK_KP4;
		KB3Back = SDLK_KP6;
		KB3Right = SDLK_KP8;
		KB3Left = SDLK_KP2;
		
		MaxRound = 6;
		RoundTime = 60;
		TKOSystem = 3;
		JudgeSystem = 0;
		CountSystem = 0;
		ExtraRound = 0;
		KOCount = 10;
		Judge1Type = 0;
		Judge2Type = 0;
		Judge3Type = 0;
		ExtraInfo = 0;
		ReplayRate = 5;
		
		RedManipOn = 0;
		BlueManipOn = 0;

		strcpy(ReplayCf, REPLAYCF_FILE);
		strcpy(BGMCf, BGMCF_FILE);
		strcpy(FontCf, FONTCF_FILE);
		strcpy(ImageCf, IMAGECF_FILE);
		strcpy(SECf, SECF_FILE);
		strcpy(DataCf, DATACF_FILE);
		strcpy(ThinkCf, THINKCF_FILE);
	} else {
		ScreenSize = reader.NextInt();
		ScreenMode = reader.NextInt();
		SoundFreq = reader.NextInt();
		SoundFormat = reader.NextInt();
		SoundChannel = reader.NextInt();
		SoundBuffer = reader.NextInt();
		BGMVolume = reader.NextInt();
		SEVolume = reader.NextInt();
		switch (reader.NextInt()) {
			case 1 :
				lang = Kana;
			break; 
			case 2 : 
				lang = English;
			break;
			default :
				lang = Japanese;
			break;			 
		}
		MakeFromTTF = reader.NextInt();
		UsePText = reader.NextInt();

		Partition = reader.NextInt();
		SetMinMax(&Partition, 2, 10);
		meterspec = (float)Partition / 10.0;
		Partition*=4;
		FaceStyle = reader.NextInt();
		SetMinMax(&SkipFrame, 0, 2);

		ShadowSwitch = reader.NextInt();
		BackGroundSwitch = reader.NextInt();
		SetMinMax(&SkipFrame, 0, 2);
		AdsSwitch = reader.NextInt();
		Mag_Linear = reader.NextInt();
		Min_Linear = reader.NextInt();
		SkipFrame = reader.NextInt();
		SetMinMax(&SkipFrame, 1, 5);

		DefaultView = reader.NextInt();
		SetMinMax(&DefaultView, 0, 15);
		Angle = reader.NextInt();
		Rotation = 45;
		SetAngle();
		Brightness = reader.NextInt();
		SetMinMax(&Brightness, -100, 100);

		SkipOpening = reader.NextInt();
		SkipReplay = reader.NextInt();
   		SkipDown = reader.NextInt();

		reader.NextChar(UserName);
		if (!strcmp(UserName, "user") || !strcmp(UserName, "User") ||
			!strcmp(UserName, "[----------]") || !strcmp(UserName, "")){
			BoxerOM.CreateRandom(random, JAPAN);
			handler1.SetBoxerName(BoxerOM);
	 		strcpy(UserName, handler1.NickNameStrEn());	
		}
#ifdef LINUX
		strcpy(UserName, babel::euc_to_sjis(UserName).c_str());
#endif
		reader.NextChar(HostName);
		reader.NextChar(HostCGI);
		reader.NextChar(MatchMakeCGI);
		reader.NextChar(LobbyCGI);
		Port = (Uint16)reader.NextInt();
		PortLobby = (Uint16)reader.NextInt();
		LobbyMode = reader.NextInt();
	
		Player1Mode = reader.NextInt();
		SetMinMax(&Player1Mode, 0, 2);
		Player2Mode = reader.NextInt();
		SetMinMax(&Player2Mode, 0, 2);
		UseMouse = reader.NextInt();
		SetMinMax(&UseMouse, 1, 2);

		reader.NextLine(key_name);
		go_next = GetSDLKey(key_name, SDLK_SPACE);

		reader.NextLine(key_name);
		KB1LeftOffense = GetSDLKey(key_name, SDLK_z);
		reader.NextLine(key_name);
		KB1RightOffense = GetSDLKey(key_name, SDLK_x);
		reader.NextLine(key_name);
		KB1Defense = GetSDLKey(key_name, SDLK_c);
		reader.NextLine(key_name);
		KB1Forward = GetSDLKey(key_name, SDLK_RIGHT);
		reader.NextLine(key_name);
		KB1Back = GetSDLKey(key_name, SDLK_LEFT);
		reader.NextLine(key_name);
		KB1Right = GetSDLKey(key_name, SDLK_DOWN);
		reader.NextLine(key_name);
		KB1Left = GetSDLKey(key_name, SDLK_UP);

		reader.NextLine(key_name);
		KB2LeftOffense = GetSDLKey(key_name, SDLK_z);
		reader.NextLine(key_name);
		KB2RightOffense = GetSDLKey(key_name, SDLK_x);
		reader.NextLine(key_name);
		KB2Defense = GetSDLKey(key_name, SDLK_c);
		reader.NextLine(key_name);
		KB2Forward = GetSDLKey(key_name, SDLK_m);
		reader.NextLine(key_name);
		KB2Back = GetSDLKey(key_name, SDLK_b);
		reader.NextLine(key_name);
		KB2Right = GetSDLKey(key_name, SDLK_n);
		reader.NextLine(key_name);
		KB2Left = GetSDLKey(key_name, SDLK_h);

		reader.NextLine(key_name);
		KB3LeftOffense = GetSDLKey(key_name, SDLK_PERIOD);
		reader.NextLine(key_name);
		KB3RightOffense = GetSDLKey(key_name, SDLK_SLASH);
		reader.NextLine(key_name);
		KB3Defense = GetSDLKey(key_name, SDLK_BACKSLASH);
		reader.NextLine(key_name);
		KB3Forward = GetSDLKey(key_name, SDLK_KP4);
		reader.NextLine(key_name);
		KB3Back = GetSDLKey(key_name, SDLK_KP6);
		reader.NextLine(key_name);
		KB3Right = GetSDLKey(key_name, SDLK_KP8);
		reader.NextLine(key_name);
		KB3Left = GetSDLKey(key_name, SDLK_KP2);

		MaxRound = reader.NextInt();
		RoundTime = reader.NextInt();
		TKOSystem = reader.NextInt();
		JudgeSystem = reader.NextInt();
		CountSystem = reader.NextInt();
		ExtraRound = reader.NextInt();
		KOCount = reader.NextInt();
		Judge1Type = reader.NextInt();
		Judge2Type = reader.NextInt();
		Judge3Type = reader.NextInt();	
		ExtraInfo = reader.NextInt();
		ReplayRate = reader.NextInt();

		RedManipOn = reader.NextInt();
		RedManip.SetPunch(reader.NextInt());
		RedManip.SetSpeed(reader.NextInt());
        RedManip.SetStamina(reader.NextInt());
		RedManip.SetDamageRestore(reader.NextInt());
 		RedManip.SetMaxDamage(reader.NextInt());
 		RedManip.SetCyanosisRestore(reader.NextInt());
 		RedManip.SetMaxCyanosis(reader.NextInt());
	 	RedManip.SetBlockAbsorb(reader.NextInt());
 		RedManip.SetBodyStrength(reader.NextInt());
 		RedManip.SetLiverStrength(reader.NextInt());
 		RedManip.SetFaceStrength(reader.NextInt());
	 	RedManip.SetJawStrength(reader.NextInt());

		BlueManipOn = reader.NextInt();
		BlueManip.SetPunch(reader.NextInt());
		BlueManip.SetSpeed(reader.NextInt());
        BlueManip.SetStamina(reader.NextInt());
		BlueManip.SetDamageRestore(reader.NextInt());
 		BlueManip.SetMaxDamage(reader.NextInt());
 		BlueManip.SetCyanosisRestore(reader.NextInt());
 		BlueManip.SetMaxCyanosis(reader.NextInt());
	 	BlueManip.SetBlockAbsorb(reader.NextInt());
 		BlueManip.SetBodyStrength(reader.NextInt());
 		BlueManip.SetLiverStrength(reader.NextInt());
 		BlueManip.SetFaceStrength(reader.NextInt());
	 	BlueManip.SetJawStrength(reader.NextInt());   

		reader.NextChar(ReplayCf);
		reader.NextChar(FontCf);
		reader.NextChar(ImageCf);
		reader.NextChar(BGMCf);
		reader.NextChar(SECf);
		reader.NextChar(DataCf);
		reader.NextChar(ThinkCf);
	}
}

void Game::WriteConfig() {
	FILE *fp;
	
	fp = fopen(CONFIG_FILE, "wt");
	if (fp == NULL) {
		switch (lang) {
			case Kana:
				ttext.Pause("ݒt@CЂ炯܂", 0.25);
			break;
			case English:
				ttext.Pause("Can't open \"CONFIG_FILE\" file.", 0.25);
			break;
			default:
				ttext.Pause("ݒt@CJ܂B", 0.25);
				break;
		}								
		return;	
	}

	fprintf(fp, "#Config File for %s\n", PDB_CAPTION_EN_TITLE);

	fprintf(fp, "\n#Screen Config\n");
	fprintf(fp, "#Screeen Size  0 : 640x480  1 : 800x600  2 : 1024x768\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", ScreenSize);
	fprintf(fp, "#Screen Mode  0 : Window Mode  1 : Full Screen Mode\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", ScreenMode);

	fprintf(fp, "\n#Sound Config\n");
	fprintf(fp, "#Sound Frequency  0 : 44100  1 : 22050  2 : 11025  3 : 48000  4 : 32000\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", SoundFreq);
	fprintf(fp, "#Sound Format  0 : 16bit  1 : 8bit\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", SoundFormat);
	fprintf(fp, "#Sound Channel  0 : Stereo  1 : Mono\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", SoundChannel);
	fprintf(fp, "#Sound Buffer Size  0 : 4096  1 : 2048  2 : 1024  3 : 512  4 : 8182\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", SoundBuffer);

	fprintf(fp, "\n#BGM Volume  -8-128\n");
	fprintf(fp, "#Default -8\n");
	fprintf(fp, "%d\n", BGMVolume);

	fprintf(fp, "\n#Sound Effect  Volume -8-128\n");
	fprintf(fp, "#Default -8\n");
	fprintf(fp, "%d\n", SEVolume);

	fprintf(fp, "\n#Language  0 : Japanese Kanji  1 : Japanese Kana  2 : English\n");
	fprintf(fp, "#Default 0\n");
	switch (lang) {
		case Kana:
			fprintf(fp, "1\n");
		break;
		case English:
			fprintf(fp, "2\n");
		break;
		default:
			fprintf(fp, "0\n");
		break;
	}

	fprintf(fp, "\n#Make Alphabet Font from TTF  0 : No  1 : Yes\n");
	fprintf(fp, "#Default 1\n");
	if (MakeFromTTF) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Use Polygon Font  0 : No  1 : Yes\n");
	fprintf(fp, "#Default 1\n");
	if (UsePText) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}

	fprintf(fp, "\n#Expression Accuracy Config\n");
	fprintf(fp, "#Partition Number  2-10\n");
	fprintf(fp, "#Default 6\n");
	fprintf(fp, "%d\n", Partition/4);	
	fprintf(fp, "#Face Style  0 : None  1 : Texture  2 : Polygon\n");
	fprintf(fp, "#Default 2\n");
	fprintf(fp, "%d\n", FaceStyle);	
	fprintf(fp, "#Shadows  1 : ON  0 : OFF\n");
	fprintf(fp, "#Default 1\n");
	if (ShadowSwitch) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#BackGround  0 : OFF  1 : Gray  2 : Texture\n");
	fprintf(fp, "#Default 2\n");
	if (BackGroundSwitch) {
		fprintf(fp, "2\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Ring Ad's  1 : ON  0 : OFF\n");
	fprintf(fp, "#Default 1\n");
	if (AdsSwitch) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Texture Magnification Filter  0 : Nearest  1 : Linear\n");
	fprintf(fp, "#Default 1\n");
	if (Mag_Linear) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Texture Minification Filter  0 : Nearest  1 : Linear\n");
	fprintf(fp, "#Default 0\n");
	if (Min_Linear) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Skip Frame  1-5\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", SkipFrame);	

	fprintf(fp, "\n#Camera Config\n");
	fprintf(fp, "#Default View  0 : SideView1  1 : SideView2  2 : BackView1  3 : BackView2\n");
	fprintf(fp, "#              4 : TopView1  5 : Top View 2  6 : BoxerView1  7 : BoxerView2\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", DefaultView);	
	fprintf(fp, "#Angle  -90-90\n");
	fprintf(fp, "#Default 10\n");
	fprintf(fp, "%d\n", Angle);	
	fprintf(fp, "#Brightness  -100-100\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", Brightness);	

	fprintf(fp, "\n#Skip Opening  0 : No  1 : Yes\n");
	fprintf(fp, "#Default 0\n");
	if (SkipOpening) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Skip Replay  0 : No  1 : Yes\n");
	fprintf(fp, "#Default 0\n");
	if (SkipReplay) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}
	fprintf(fp, "#Skip Down  0 : No  1 : Yes\n");
	fprintf(fp, "#Default 0\n");
	if (SkipDown) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}

	fprintf(fp, "\n#Network Config\n");
	fprintf(fp, "#User Name\n");
	fprintf(fp, "#Default user\n");
	if (strlen(UserName)) {
		fprintf(fp, "%s\n", UserName);	
	} else {
		fprintf(fp, "user\n");		
	}
	fprintf(fp, "#Host Name\n");
	fprintf(fp, "#Default localhost\n");
	if (strlen(HostName)) {
		fprintf(fp, "%s\n", HostName);	
	} else {
		fprintf(fp, "localhost\n");		
	}
	fprintf(fp, "#CGI Host Name\n");
	fprintf(fp, "#Default hpcgi2.nifty.com\n");
	if (strlen(HostCGI)) {
		fprintf(fp, "%s\n", HostCGI);	
	} else {
		fprintf(fp, "hpcgi2.nifty.com\n");		
	}
	fprintf(fp, "#Match Make CGI\n");
	fprintf(fp, "#Default /FengDan/matchmake.cgi\n");
	if (strlen(MatchMakeCGI)) {
		fprintf(fp, "%s\n", MatchMakeCGI);	
	} else {
		fprintf(fp, "/FengDan/matchmake.cgi\n");		
	}
	fprintf(fp, "#Lobby CGI\n");
	fprintf(fp, "#Default /FengDan/lobby.cgi\n");
	if (strlen(LobbyCGI)) {
		fprintf(fp, "%s\n", LobbyCGI);	
	} else {
		fprintf(fp, "/FengDan/lobby.cgi\n");		
	}
	fprintf(fp, "#Port\n");
	fprintf(fp, "#Default 2000\n");
	fprintf(fp, "%d\n", Port);	
	fprintf(fp, "#Port(Lobby)\n");
	fprintf(fp, "#Default 3000\n");
	fprintf(fp, "%d\n", PortLobby);	
	fprintf(fp, "#Lobby Mode  0 : Local  1 : CGI\n");
	fprintf(fp, "#Default 1\n");
	if (LobbyMode) {
		fprintf(fp, "1\n");	
	} else {
		fprintf(fp, "0\n");	
	}

	fprintf(fp, "\n#Control Config\n");
	fprintf(fp, "#Player1 Mode  0 : Full Control  1 : Semi-Automatic  2 : Automatic\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", Player1Mode);
	fprintf(fp, "#Player2 Mode  0 : Full Control  1 : Semi-Automatic  2 : Automatic\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", Player2Mode);
	fprintf(fp, "#Use Mouse  1 : Player1  2 : Player2\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", UseMouse);

	fprintf(fp, "#For Go Next\n");
	fprintf(fp, "#Default \"space\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)go_next));

	fprintf(fp, "#For 1P Player1\n");
	fprintf(fp, "#1P Player1 Left Offense\n");
	fprintf(fp, "#Default \"z\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1LeftOffense));
	fprintf(fp, "#1P Player1 Right Offense\n");
	fprintf(fp, "#Default \"x\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1RightOffense));
	fprintf(fp, "#1P Player1 Defense\n");
	fprintf(fp, "#Default \"c\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1Defense));
	fprintf(fp, "#1P Player1 Forward\n");
	fprintf(fp, "#Default \"right\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1Forward));
	fprintf(fp, "#1P Player1 Back\n");
	fprintf(fp, "#Default \"left\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1Back));
	fprintf(fp, "#1P Player1 Right\n");
	fprintf(fp, "#Default \"down\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1Right));
	fprintf(fp, "#1P Player1 Left\n");
	fprintf(fp, "#Default \"up\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB1Left));

	fprintf(fp, "#For 2P Player1\n");
	fprintf(fp, "#2P Player1 Left Offense\n");
	fprintf(fp, "#Default \"z\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2LeftOffense));
	fprintf(fp, "#2P Player1 Right Offense\n");
	fprintf(fp, "#Default \"x\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2RightOffense));
	fprintf(fp, "#2P Player1 Defense\n");
	fprintf(fp, "#Default \"c\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2Defense));
	fprintf(fp, "#2P Player1 Forward\n");
	fprintf(fp, "#Default \"m\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2Forward));
	fprintf(fp, "#2P Player1 Back\n");
	fprintf(fp, "#Default \"b\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2Back));
	fprintf(fp, "#2P Player1 Right\n");
	fprintf(fp, "#Default \"n\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2Right));
	fprintf(fp, "#2P Player1 Left\n");
	fprintf(fp, "#Default \"h\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB2Left));

	fprintf(fp, "#For 2P Player2\n");
	fprintf(fp, "#2P Player2 Left Offense\n");
	fprintf(fp, "#Default \".\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3LeftOffense));
	fprintf(fp, "#2P Player2 Right Offense\n");
	fprintf(fp, "#Default \"/\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3RightOffense));
	fprintf(fp, "#2P Player2 Defense\n");
	fprintf(fp, "#Default \"\\\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3Defense));
	fprintf(fp, "#2P Player2 Forward\n");
	fprintf(fp, "#Default \"[4]\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3Forward));
	fprintf(fp, "#2P Player2 Back\n");
	fprintf(fp, "#Default \"[6]\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3Back));
	fprintf(fp, "#2P Player2 Right\n");
	fprintf(fp, "#Default \"[8]\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3Right));
	fprintf(fp, "#2P Player2 Left\n");
	fprintf(fp, "#Default \"[2]\"\n");
	fprintf(fp, "%s\n", SDL_GetKeyName((SDLKey)KB3Left));

	fprintf(fp, "\n#Match Rules Config\n");
	fprintf(fp, "#Max Round\n");
	fprintf(fp, "#Default 6\n");
	fprintf(fp, "%d\n", MaxRound);
	fprintf(fp, "#Round Time\n");
	fprintf(fp, "#Default 60\n");
	fprintf(fp, "%d\n", RoundTime);
	fprintf(fp, "#TKO System\n");
	fprintf(fp, "#Default 3\n");
	fprintf(fp, "%d\n", TKOSystem);
	fprintf(fp, "#Judging System 0 : Round Must  1 : Not Round Must\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", JudgeSystem);
	fprintf(fp, "#Count System 0 : Last Round Stop  1 : Last Round Continue\n");
	fprintf(fp, "#             2 : All Round Stop   3 : All Round Continue\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", CountSystem);
	fprintf(fp, "#Extra Round 0 : OFF 1 : ON\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", ExtraRound);
	fprintf(fp, "#KO Count\n");
	fprintf(fp, "#Default 10\n");
	fprintf(fp, "%d\n", KOCount);
	fprintf(fp, "#Judge Type 0 : Random  1 : Hit  2 : Critical  3 : Damage\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", Judge1Type);
	fprintf(fp, "%d\n", Judge2Type);
	fprintf(fp, "%d\n", Judge3Type);
	fprintf(fp, "#Extra Information 0 : OFF 1 : ON\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", ExtraInfo);
	fprintf(fp, "##Replay Rate  1-10\n");
	fprintf(fp, "#Default 5\n");
	fprintf(fp, "%d\n", ReplayRate);

	fprintf(fp, "\n#Match Manipulation Config\n");
	fprintf(fp, "#Red Corner Manipulation 0 : OFF 1 : ON\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManipOn);
	fprintf(fp, "#Red Punch Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.Punch());
	fprintf(fp, "#Red Speed Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.Speed());
	fprintf(fp, "#Red Stamina Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.Stamina());
	fprintf(fp, "#Red Damage Restore Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.DamageRestore());
	fprintf(fp, "#Red Max Damage Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.MaxDamage());
	fprintf(fp, "#Red Cyanosis Restore Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.CyanosisRestore());
	fprintf(fp, "#Red Max Cyanosis Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.MaxCyanosis());
	fprintf(fp, "#Red Block Absorb Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.BlockAbsorb());
	fprintf(fp, "#Red Body Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.BodyStrength());
	fprintf(fp, "#Red Liver Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.LiverStrength());
	fprintf(fp, "#Red Face Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.FaceStrength());
	fprintf(fp, "#Red Jaw Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.JawStrength());

	fprintf(fp, "#Blue Corner Manipulation 0 : OFF 1 : ON\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManipOn);
	fprintf(fp, "#Blue Punch Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.Punch());
	fprintf(fp, "#Blue Speed Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.Speed());
	fprintf(fp, "#Blue Stamina Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.Stamina());
	fprintf(fp, "#Blue Damage Restore Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.DamageRestore());
	fprintf(fp, "#Blue Max Damage Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.MaxDamage());
	fprintf(fp, "#Blue Cyanosis Restore Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.CyanosisRestore());
	fprintf(fp, "#Blue Max Cyanosis Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.MaxCyanosis());
	fprintf(fp, "#Blue Block Absorb Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.BlockAbsorb());
	fprintf(fp, "#Blue Body Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.BodyStrength());
	fprintf(fp, "#Blue Liver Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.LiverStrength());
	fprintf(fp, "#Blue Face Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.FaceStrength());
	fprintf(fp, "#Blue Jaw Strength Manipulation -50-50\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.JawStrength());

	fprintf(fp, "\n#Config Files\n");
	fprintf(fp, "#Replay Config File\n");
	fprintf(fp, "#Default ./Replay/replaycf.txt\n");
	fprintf(fp, "%s\n", ReplayCf);
	fprintf(fp, "#Font Config File\n");
	fprintf(fp, "#Default ./Font/fontcf.txt\n");
	fprintf(fp, "%s\n", FontCf);
	fprintf(fp, "#Image Config File\n");
	fprintf(fp, "#Default ./Image/imagecf.txt\n");
	fprintf(fp, "%s\n", ImageCf);
	fprintf(fp, "#BGM Config File\n");
	fprintf(fp, "#Default ./BGM/bgmcf.txt\n");
	fprintf(fp, "%s\n", BGMCf);
	fprintf(fp, "#Sound Effect Config File\n");
	fprintf(fp, "#Default ./SE/secf.txt\n");
	fprintf(fp, "%s\n", SECf);
	fprintf(fp, "#Data Config File\n");
	fprintf(fp, "#Default ./Data/datacf.txt\n");
	fprintf(fp, "%s\n", DataCf);
	fprintf(fp, "#Think Config File\n");
	fprintf(fp, "#Default ./Data/thinkcf.txt\n");
	fprintf(fp, "%s\n", ThinkCf);

	fprintf(fp, "\n#End of File\n");

	fclose(fp);
}

void Game::ScreenConfig() {
	Uint32 flags = SDL_OPENGL;// | SDL_RESIZABLE;
	switch (ScreenSize) {
		case 1: 
			width = 800;
			height = 600;
		break;
		case 2: 
			width = 1024;
			height = 768;
		break;
		case 3: 
			width = 320;
			height = 240;
		break;
		case 4: 
			width = 400;
			height = 300;
		break;
		case 5: 
			width = 512;
			height = 384;
		break;
		default:
			width = 640;
			height = 480;
		break;
	}
				
	switch (ScreenMode) {
		case 1: 
			flags |= SDL_FULLSCREEN;
		break;
		default:
		break;
	}

/* CXN[̏ */
	icon.Set(Icon());
	screen.SetIcon(icon.Get(), NULL);
	if (lang == English) {
		screen.SetCaption(PDB_CAPTION_EN_TITLE, PDB_CAPTION_EN_ICON);
	} else {
		screen.SetCaption(PDB_CAPTION_TITLE, PDB_CAPTION_ICON);
	}
	screen.SetVideoMode(width, height, SDL_GetVideoInfo()->vfmt->BitsPerPixel, flags);
	if (!screen.WasSet()) {
		exit(-1);
	}
/*@fobOo */
	fprintf(stdout, "Version     : %s\n", PDB_CAPTION_EN_TITLE);
	screen.PrintScreenInfo();
	screen.PrintGLInfo();
}

void Game::ScreenReconfig() {
	int i;
	Uint32 flags = SDL_OPENGL;// | SDL_RESIZABLE;
	switch (ScreenSize) {
		case 1: 
			width = 800;
			height = 600;
		break;
		case 2: 
			width = 1024;
			height = 768;
		break;
		case 3: 
			width = 320;
			height = 240;
		break;
		case 4: 
			width = 400;
			height = 300;
		break;
		case 5: 
			width = 512;
			height = 384;
		break;
		default:
			width = 640;
			height = 480;
		break;
	}
				
	switch (ScreenMode) {
		case 1: 
			flags |= SDL_FULLSCREEN;
		break;
		default:
		break;
	}

	screen.Reset();
	ttext.Close();
	ptext.Close();
	texs.Free();

/* CXN[̍ď */	
	if (lang == English) {
		screen.SetCaption(PDB_CAPTION_EN_TITLE, PDB_CAPTION_EN_ICON);
	} else {
		screen.SetCaption(PDB_CAPTION_TITLE, PDB_CAPTION_ICON);
	}
	screen.SetVideoMode(width, height, SDL_GetVideoInfo()->vfmt->BitsPerPixel, flags);
	if (!screen.WasSet()) {
		exit(-1);
	}

	screen.GLInit();
/* eNX`(|S)eLXg̍ď */
	ttext.Init(&ttf);
	ptext.Init(&glp, &ttf);
	if (MakeFromTTF) {
		ttf.Choice(1);
		ttext.MakeAlphabetFromTTF();
		ptext.MakeAlphabetFromTTF();
	}
	ttext.Clear();
	ptext.Clear();

	texs.Init(MAX_IMAGES);
	for (i = 0; i < MAX_IMAGES; i++) {
		texs.Generate(i, surfaces.Get(i));
	}
}

void  Game::SoundConfig() {
	switch (SoundFreq) {
		case 1: 
			frequency = 22050;
		break;
		case 2: 
			frequency = 11025;
		break;
		case 3: 
			frequency = 48000;
		break;
		case 4: 
			frequency = 32000;
		break;
		default:
			frequency = 44100;
		break;
	}

	switch (SoundFormat) {
		case 1: 
			format = AUDIO_U8;
		break;
		default:
			format = AUDIO_U16;
		break;
	}

	switch (SoundChannel) {
		case 1: 
			channel = 1;
		break;
		default:
			channel = 2;
		break;
	}

	switch (SoundBuffer) {
		case 1: 
			buffer = 2048;
		break;
		case 2: 
			buffer = 1024;
		break;
		case 3: 
			buffer = 512;
		break;
		case 4: 
			buffer = 8182;
		break;
		default:
			buffer = 4096;
		break;
	}

/* ~LT[()̏ */
	mixer.Open(frequency, format, channel, buffer); 
	if (!mixer.WasOpen()) {
		mixer.SetMusicVolume(0);
		mixer.SetChunkVolume(0); 
		return;
	}
	
	if (BGMVolume > 128) {
		BGMVolume = 128;
	}
	if (BGMVolume < -8) {
		BGMVolume = -8;
	}

	if (SEVolume > 128) {
		SEVolume = 128;
	}
	if (SEVolume < -8) {
		SEVolume = -8;
	}
	
	mixer.SetMusicVolume(BGMVolume);
	mixer.SetChunkVolume(SEVolume); 
}

void Game::FontConfig() {
	int i;		
/* TTF̏ */
	ttf.Init(MAX_FONTS);
	if (!ttf.WasInit()) {
		exit(-1);
	}

/* eNX`eLXg̏ */
	ttext.Init(&ttf);
	if (!ttext.WasInit()) {
		emes("Game::FontConfig -> Can't initialize texture text.");
		exit(-1);
	}

/* |SeLXg̏ */
	ptext.Init(&glp, &ttf);
	if (!ptext.WasInit()) {
		ttext.Pause("Can't Initialize Polygon Text", 0.25);
		exit(-1);
	}

/* tHg̓ǂݍ */
	int	fonts[MAX_FONT_NUMBER];
	char fontn[MAX_FONT_NUMBER][MAX_LINE_SIZE];

	if (reader.Open(FontCf) == SDL_FAILURE) {
		fonts[FONT_TITLE] = FONT_SIZE1;
		fonts[FONT_SYSTEM] = FONT_SIZE2;
		strcpy(fontn[FONT_TITLE], FONT_NAME1);
		strcpy(fontn[FONT_SYSTEM], FONT_NAME2);
	} else {
		for (i = 0; i < MAX_FONT_NUMBER; i++) {
			reader.NextChar(fontn[i]);
			fonts[i] = reader.NextInt();
			if (fonts[i] < 8) {
				fonts[i] = 8;
			}
			if (fonts[i] > 100) {
				fonts[i] = 100;
			}
		}
	}

	if (ttf.Open(FONT_TITLE, fontn[FONT_TITLE], fonts[FONT_TITLE]) == SDL_FAILURE) {
			ttext.Pause("Can't Open Title Font", 0.25);
			exit(-1);	
	}
	if (lang == Japanese || MakeFromTTF) {
		if (ttf.Open(FONT_SYSTEM, fontn[FONT_SYSTEM], fonts[FONT_SYSTEM]) == SDL_FAILURE) {
			switch (lang) {
				case English:
					ttext.Pause("Can't open system font.", 0.25);
				break;
				case Japanese:
					lang = Kana;
					wmes("Hiragame Message");
				default:
					ttf.Choice(lang);
					ttext.Pause("VXetHgЂ炯܂", 0.25);
				break;
			}								
			if (MakeFromTTF) {
				wmes("Game::FontConfig -> Can't use TTF by alphabet.");
				MakeFromTTF = 0;
			}					
		} else {
			switch (lang) {
				case English:
					smes("English Message");
				break;
				case Kana:
					smes("Hiraganae Messeage");
				break;
				default:
					smes("Kanji Message");
				break;
			}
			if (fonts[FONT_SYSTEM] < 23) {
				ttf.SetRender(RENDER_SOLID);			
			}
			if (MakeFromTTF) {
				smes("Use Alphabet TTF");
				ptext.MakeAlphabetFromTTF();
				ttext.MakeAlphabetFromTTF();
			}					
		}
	}
}

void Game::ImageConfig() {
	int i;
	char fn[MAX_LINE_SIZE];
/* C[W̓ǂݍ */
	if (surfaces.Init(MAX_IMAGES) == SDL_FAILURE) {
		switch (lang) {
			case Kana:
				ttext.Pause("Ԃ񃁃sĂ܂B", 0.25);		
			break;
			case English:
				ttext.Pause("Out of Memory", 0.25);
			break;
			default:
				ttext.Pause("Ԃ񃁃ӂĂ܂", 0.25);		
			break;
		}								
		exit(-1);		
	}

	if (reader.Open(ImageCf) == SDL_FAILURE) {
		surfaces.Load(0, "./Image/canvas.jpg");
		surfaces.Load(1, "./Image/henoheno.png");
		surfaces.Load(2, "./Image/background.jpg");
		surfaces.Load(3, "./Image/floor.jpg");
		surfaces.Load(4, "./Image/roof.jpg");
		surfaces.Load(5, "./Image/ads.png");
	} else {
		for (i = 0; i < MAX_IMAGES; i++) {
			reader.NextChar(fn);
			surfaces.Load(i, fn);
		}
	}

	for (i = 0; i < MAX_IMAGES; i++) {	
		if (!surfaces.WasCreate(i)) {
			switch (lang) {
				case Kana:
					ttext.Pause("C[W݂߂܂", 0.25);
				break;
				case English:
					ttext.Pause("Can't read image files.", 0.25);
				break;
				default:
					ttext.Pause("C[Wǂݍ߂܂B", 0.25);
				break;
			}								
			exit(-1);	
		}
	}
	
/* eNX`̍쐬 */
	if (texs.Init(MAX_IMAGES) == SDL_FAILURE) {
		switch (lang) {
			case Kana:
				ttext.Pause("Ԃ񃁃sĂ܂B", 0.25);		
			break;
			case English:
				ttext.Pause("Out of Memory", 0.25);
			break;
			default:
				ttext.Pause("Ԃ񃁃ӂĂ܂", 0.25);		
			break;
		}								
		exit(-1);		
	}
	texs.SetMagFilter(Mag_Linear);
	texs.SetMinFilter(Min_Linear);
	for (i = 0; i < MAX_IMAGES; i++) {
		texs.Generate(i, surfaces.Get(i));
	}
	texs.Choice(TE_CANVAS);
}

void Game::BGMConfig() {
	int i;
	char fn[MAX_LINE_SIZE];
/* BGM̓ǂݍ */
	if (mixer.InitMusic(MAX_BGMS) == SDL_FAILURE) {
		switch (lang) {
			case Kana:
				ttext.Pause("Ԃ񃁃sĂ܂B", 0.25);		
			break;
			case English:
				ttext.Pause("Out of Memory", 0.25);
			break;
			default:
				ttext.Pause("Ԃ񃁃ӂĂ܂", 0.25);		
			break;
		}								
		exit(-1);		
	}
	if (reader.Open(BGMCf) == SDL_FAILURE) {
		mixer.LoadMusic(0, "./BGM/selectgm.mid");
		mixer.LoadMusic(1, "./BGM/kokorogm.mid");
		mixer.LoadMusic(2, "./BGM/cumugm.mid");
//		mixer.LoadMusic(3, "./BGM/test.mp3");
	} else {
		for (i = 0; i < MAX_BGMS; i++) {
			reader.NextChar(fn);
			mixer.LoadMusic(i, fn);
		}
	}
	for (i = 0; i < MAX_BGMS; i++) {
		if (!mixer.MusicWasLoad(i)) {
			switch (lang) {
				case Kana:
					ttext.Pause("r[W[G݂߂܂", 0.25);
				break;
				case English:
					ttext.Pause("Can't read BGM files.", 0.25);
				break;
				default:
					ttext.Pause("aflǂݍ߂܂B", 0.25);
				break;
			}
		}
	}
}

void Game::SEConfig() {
	int i;
	char fn[MAX_LINE_SIZE];
/* ʉ̓ǂݍ */
	if (mixer.InitChunk(MAX_SOUND_EFFECTS) == SDL_FAILURE) {
		switch (lang) {
			case Kana:
				ttext.Pause("Ԃ񃁃sĂ܂B", 0.25);		
			break;
			case English:
				ttext.Pause("Out of Memory", 0.25);
			break;
			default:
				ttext.Pause("Ԃ񃁃ӂĂ܂", 0.25);		
			break;
		}								
		exit(-1);		
	}
	if (reader.Open(SECf) == SDL_FAILURE) {
	  	mixer.LoadChunk(0, "./SE/gong.wav");
  		mixer.LoadChunk(1, "./SE/ko.wav");
 	 	mixer.LoadChunk(2, "./SE/sha00.wav");
  		mixer.LoadChunk(3, "./SE/swing29.wav");
 	 	mixer.LoadChunk(4, "./SE/qu.ogg");
  		mixer.LoadChunk(5, "./SE/gun08.wav");
  		mixer.LoadChunk(6, "./SE/hit_p01_c.wav");
 	 	mixer.LoadChunk(7, "./SE/hit_s07_a.wav");
  		mixer.LoadChunk(8, "./SE/bosu00.wav");
 	 	mixer.LoadChunk(9, "./SE/box.ogg");
 	 	mixer.LoadChunk(10, "./SE/break.ogg");
 	 	mixer.LoadChunk(11, "./SE/down.ogg");
 	} else {
		for (i = 0; i < MAX_SOUND_EFFECTS; i++) {
			reader.NextChar(fn);
		  	mixer.LoadChunk(i, fn);
		}
	}
	for (i = 0; i < MAX_SOUND_EFFECTS; i++) {
		if (!mixer.ChunkWasLoad(i)) {
			switch (lang) {
				case Kana:
					ttext.Pause("TEhGtFNg݂߂܂", 0.25);
				break;
				case English:
					ttext.Pause("Can't read sound effect files.", 0.25);
				break;
				default:
					ttext.Pause("TEhGtFNgǂݍ߂܂B", 0.25);
				break;
			}								
		}
	}
}

void Game::DataConfig() {
	int i;
/* {NT[t@C */
	if (reader.Open(DataCf) == SDL_FAILURE) {
		MembersNumber = 2;
		strcpy(GymMembers[0], "./Data/light.box");
		strcpy(GymMembers[1], "./Data/general.box");		

 		RedManip.Init();
		BlueManip.Init();
 	} else {
 		MembersNumber = reader.NextInt();
		if (MembersNumber < 0) {
			MembersNumber = 0;		
		}
		if (MembersNumber > MAX_MEMBERS) {
			MembersNumber = MAX_MEMBERS;		
		}
		for (i = 0; i < MembersNumber; i++) {
			reader.NextChar(GymMembers[i]);		
		}

		RedManip.SetHeight(reader.NextInt());
		RedManip.SetReach(reader.NextInt());
        RedManip.SetStyle(reader.NextInt());
		RedManip.SetStance(reader.NextInt());
 		RedManip.SetComb(reader.NextInt());
 		RedManip.SetThought(reader.NextInt());
 		RedManip.SetGloves(reader.NextInt());
	 	RedManip.SetTrunks(reader.NextInt());
 		RedManip.SetSkin(reader.NextInt());

		BlueManip.SetHeight(reader.NextInt());
		BlueManip.SetReach(reader.NextInt());
        BlueManip.SetStyle(reader.NextInt());
		BlueManip.SetStance(reader.NextInt());
 		BlueManip.SetComb(reader.NextInt());
 		BlueManip.SetThought(reader.NextInt());
 		BlueManip.SetGloves(reader.NextInt());
	 	BlueManip.SetTrunks(reader.NextInt());
 		BlueManip.SetSkin(reader.NextInt());

/*		BoxerOM1.SetHeight(reader.NextInt());
		BoxerOM1.SetReach(reader.NextInt());
        BoxerOM1.SetStyle(reader.NextInt());
		BoxerOM1.SetStance(reader.NextInt());
 		BoxerOM1.SetComb(reader.NextInt());
 		BoxerOM1.SetThought(reader.NextInt());
 		BoxerOM1.SetGloves(reader.NextInt());
	 	BoxerOM1.SetTrunks(reader.NextInt());
 		BoxerOM1.SetSkin(reader.NextInt());
*/
/*
		RedManip.Init();
		BlueManip.Init();
		DataConfigSave();
*/
	}
}

void Game::DataConfigSave() {
	FILE *fp;
	int i;
	
	fp = fopen(DataCf, "wt");
	if (fp == NULL) {
		return;	
	}

	fprintf(fp, "#Config File for Boxer Data\n");
	fprintf(fp, "\n#Gym Member Number\n");
	fprintf(fp, "%d\n\n", MembersNumber);
	for (i = 0; i < MembersNumber; i++) {
		fprintf(fp, "#Data No. %d\n", i+1);
		fprintf(fp, "%s\n", GymMembers[i]);
	}

	/* JX^f[^̕ۑ */
	fprintf(fp, "\n#Sparring Custom Config\n");
	fprintf(fp, "#No.1 Height 1-7\n");
	fprintf(fp, "#Default 4\n");
	fprintf(fp, "%d\n", RedManip.Height());
	fprintf(fp, "#No.1 Reach 1-7\n");
	fprintf(fp, "#Default 4\n");
	fprintf(fp, "%d\n", RedManip.Reach());
	fprintf(fp, "#No.1 Style 1-7\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", RedManip.Style());
	fprintf(fp, "#No.1 Stance  0 : Orthodox  1 : Southpaw\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", RedManip.Stance());
	fprintf(fp, "#No.1 Combination 1-20\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", RedManip.Comb());
	fprintf(fp, "#No.1 Think 1-14\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", RedManip.Thought());
	fprintf(fp, "#No.1 Gloves 1-8\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", RedManip.Gloves());
	fprintf(fp, "#No.1 Trunks 1-64\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", RedManip.Trunks());
	fprintf(fp, "#No.1 Skin 1-10\n");
	fprintf(fp, "#Default 3\n");
	fprintf(fp, "%d\n", RedManip.Skin());

	fprintf(fp, "#No.2 Height 1-7\n");
	fprintf(fp, "#Default 4\n");
	fprintf(fp, "%d\n", BlueManip.Height());
	fprintf(fp, "#No.2 Reach 1-7\n");
	fprintf(fp, "#Default 4\n");
	fprintf(fp, "%d\n", BlueManip.Reach());
	fprintf(fp, "#No.2 Style 1-7\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", BlueManip.Style());
	fprintf(fp, "#No.2 Stance  0 : Orthodox  1 : Southpaw\n");
	fprintf(fp, "#Default 0\n");
	fprintf(fp, "%d\n", BlueManip.Stance());
	fprintf(fp, "#No.2 Combination 1-20\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", BlueManip.Comb());
	fprintf(fp, "#No.2 Think 1-14\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", BlueManip.Thought());
	fprintf(fp, "#No.2 Gloves 1-8\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", BlueManip.Gloves());
	fprintf(fp, "#No.2 Trunks 1-64\n");
	fprintf(fp, "#Default 1\n");
	fprintf(fp, "%d\n", BlueManip.Trunks());
	fprintf(fp, "#No.2 Skin 1-10\n");
	fprintf(fp, "#Default 3\n");
	fprintf(fp, "%d\n", BlueManip.Skin());

	fprintf(fp, "\n#End of File\n");
	fclose(fp);
}

void Game::ThinkConfig() {
	int i;
/* vlt@C */
	strcpy(ThinkFiles[0], "Default");
	if (reader.Open(ThinkCf) == SDL_FAILURE) {
		ThinkNumber = 2;
		strcpy(ThinkFiles[1], "./Think/light.thk");
		strcpy(ThinkFiles[2], "./Think/general.thk");		
 	} else {
		ThinkNumber = reader.NextInt();
		if (ThinkNumber < 0) {
			ThinkNumber = 0;		
		}
		if (ThinkNumber > MAX_MEMBERS) {
			ThinkNumber = MAX_MEMBERS;		
		}
		for (i = 0; i < ThinkNumber; i++) {
			reader.NextChar(ThinkFiles[i+1]);		
		}
	}
}

void Game::ReplayConfig() {
	int i;
/* vCt@C */
	if (reader.Open(ReplayCf) == SDL_FAILURE) {
		ChoiceSaveReplay = 0;
		ReplayNumber = 100;
		CurrentReplay = 100;
		SlotReplayNumber = 0;
 	} else {
		ChoiceSaveReplay = reader.NextInt();
		ReplayNumber = reader.NextInt();
		if (ReplayNumber < 1) {
			ReplayNumber = 1;		
		}
		CurrentReplay = reader.NextInt();
		if (CurrentReplay < 1) {
			CurrentReplay = 1;		
		}
		if (CurrentReplay > ReplayNumber) {
			CurrentReplay = ReplayNumber;		
		}
		SlotReplayNumber = reader.NextInt();
		if (SlotReplayNumber < 0) {
			SlotReplayNumber = 0;		
		}
		if (SlotReplayNumber > MAX_REPLAYS) {
			SlotReplayNumber = MAX_REPLAYS;		
		}
		for (i = 0; i < SlotReplayNumber; i++) {
			reader.NextChar(SlotReplay[i]);		
		}
	}
}

void Game::ReplayConfigSave() {
	FILE *fp;
	int i;
	
	fp = fopen(ReplayCf, "wt");
	if (fp == NULL) {
		return;	
	}

	/* vCt@C̕ۑ */
	fprintf(fp, "#Config File for Replay\n");

 	fprintf(fp, "\n#Choice Save Replay File  0 : DO 1 : NOT DO\n");
	fprintf(fp, "##Default 0\n");
	fprintf(fp, "%d\n", ChoiceSaveReplay);

 	fprintf(fp, "\n#Maximum Replay Number\n");
	fprintf(fp, "%d\n", ReplayNumber);
 	fprintf(fp, "#Current Replay Number\n");
	fprintf(fp, "%d\n", CurrentReplay);
	
 	fprintf(fp, "\n#Slot Replay Number\n");
	fprintf(fp, "%d\n\n", SlotReplayNumber);
	for (i = 0; i < SlotReplayNumber; i++) {
		fprintf(fp, "#Replay No.%d\n", i);
		fprintf(fp, "%s\n", SlotReplay[i]);
	}

	fprintf(fp, "\n#End of File\n");

	fclose(fp);
}
