#include "game.h"

bool Game::SetUpEventBoxer(StoryHandler &story) {
	if (story.Mode() == ORIGINAL_MODE) {
		if (handler1.LoadBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
        	LoadErrorMessage();
			return false;
		}
		boxer2.SetDefaultFace();
		SetUpBoxer2();
	} else {
		SetUpStoryBoxer2(story.Boxer(MY_BOXER));
	}
	handler2.LoadBinBoxerMotionSet(motion2, BoxerMotionEvent());	

	InitSelectBoxer();
	motion2.OffSet(boxer2);
	return true;
}

void Game::InitEnding() {
	MakeObjects(Partition);
	camera.Init(2);

	light.Init(0);
	light2.Init(1);
	light.DefDiffuse(0.8, 0.8, 0.8);
	light.DefPosition(0.0, 0.0, 1.0);
	light2.DefPosition(0.0, 0.0, 3.0);
	light2.DefDiffuse(0.8, 0.8, 0.8);
	light.On();
	light2.On();
}

void Game::Ending(StoryHandler &story) {
	mixer.PlayMusic(BGM_TRAINING);
	char buf[255];
	float size, angle;
	int i, j, k;

	j = 0;
	size = 0.0;
	angle = 0.0;

	if (!SetUpEventBoxer(story)) {
		return;	
	}
 	boxer2.SetPoint(0.0, 0.0, 180.0);
 	InitEnding();

	for (i = 0; i < 50; i++) {
		handler2.RightOffense();
		handler2.Left();
		handler2.Back();
		handler2.BoxerHandling(boxer2, motion2);
	}
	camera[0].DefPosition(2.0*sin(RAD(angle)), 2.0*cos(RAD(angle)), 0.5f);
	camera[0].DefDirection(0.0f, 0.0f, 1.0f);
	camera[0].DefRotation(0.0f);
	i = 0;

	NextOff();
	timer.SetFPS(50);	
	do {
	//	SelectOptions();

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		light.Set();
		camera[0].Set();
		light2.Set();

		if (ShadowSwitch) {
			ShowShadow(boxer2);
	    }
		if (BackGroundSwitch) {
			glbg.Show();
		}
		glring.ShowAll();

		texs.Choice(TE_FACE);
		handler2.RightOffense();
		handler2.Left();
		handler2.Back();
		handler2.BoxerHandling(boxer2, motion2);
		boxer2.CalcCoords();
		boxer2.CalcCenter();
		boxer2.CallLists();

		ttext.MessageEnter();

		k = j / 2;
		for (i = 0; i < story.RecordNumber(); i++) {
			RecordDate(story, i, buf);
			ttext.SetColor(1.0, 1.0, 1.0);	
			ttext.Print(buf, 0.20, -95, k - i * 7);		

			if (lang == Japanese) {
				ttext.Print(ResultMarkJp(story.RecordMark(i)), 0.20, 40, k - i * 7);		
			} else {
				ttext.Symbol(ResultMark(story.RecordMark(i)), 0.25, 40, k - i * 7);		
			}

			if (story.RecordRound(i) == 255) {
				RecordJudge(story, i, buf);
				ttext.Print(buf, 0.15, 85, k - i * 7);		
			} else {
				RecordTime(story, i, buf);
				ttext.Print(buf, 0.20, 85, k - i * 7);		
			}
			RecordName(story, i, buf);
			ttext.Print(buf, 0.20, -10, k - i * 7);
		}

		ttext.SetColor(0.9, 0.9, 0.0);
		switch (lang) {
			case Kana:
				ttext.Print("̂", size, 0, 100);
            break;
			case English:
				ttext.Print("Glorious Record", size, 0, 100);
            break;
			default:
				ttext.Print("h̋O", size, 0, 100);
            break;			
		}
		ttext.SetColor(1.0, 1.0, 1.0);
	  	ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();
		timer.Wait();

		if (size < 1.0) {
			size += 0.01;		
		}
		angle += 0.5;
		camera[0].DefPosition(2.0*sin(RAD(angle)), 2.0*cos(RAD(angle)), 0.5f);	
		camera[0].DefDirection(0.0f, 0.0f, 1.0f);
		camera[0].DefRotation(0.0f);
		j++;
		if (j % 100 == 0 && j < 1100) {
			mixer.PlayChunk(SE_GONG);		
		}
/*
		if (j % 200 == 0) {
			i++;
			SetUpStoryBoxer1(story.RecordBoxer(i));
			boxer1.MakeDLists(Partition);
			boxer1.MakeRanges(Partition);
			motion1.OffSet(boxer1);
		 	boxer1.SetPoint(0.0, -1.0, 0.0);
			if (i >= story.RecordNumber()) {
				i = 0;	
			}
		}  */

		event.Poll();
		if (event.IsExpose() && ScreenMode) {
			ScreenReconfig();
			InitEnding();
			event.Clear();
			wmes("Change Resolution");
		} 
		if (IsAnyKeyDown() || event.IsMouse1Down()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
}

void Game::InitBurnOut() {
	MakeObjects(Partition);
	camera.Init(2);
	camera.Perspective(60.0, 0.05, 20.0);
	camera[0].DefPosition(-0.75f, 0.75f, 0.5f);
	camera[0].DefDirection(-2.5f, 2.5f, 0.75f);
	camera[0].DefRotation(0.0f);

	light.Init(0);
	light2.Init(1);
	light.DefDiffuse(0.4, 0.4, 0.4);
	light.DefPosition(0.0, 0.0, 1.0);
	light2.DefPosition(0.0, 0.0, 3.0);
	light2.DefDiffuse(0.4, 0.4, 0.4);
	light.On();
	light2.On();
}

void Game::BurnOut(StoryHandler &story) {
	mixer.PlayMusic(BGM_TRAINING);
	float size;
	size = 0.0;

	if (!SetUpEventBoxer(story)) {
		return;	
	}
 	boxer2.SetPoint(-2.2, 2.2, -135.0);
 	InitBurnOut();

	NextOff();
	timer.SetFPS(50);	

	do {
	//	SelectOptions();

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		light.Set();
		camera[0].Set();
		light2.Set();

		if (ShadowSwitch) {
			ShowShadow(boxer2);
	    }
		if (BackGroundSwitch) {
			glbg.Show();
		}
		glring.ShowAll();

		texs.Choice(TE_FACE);
		handler2.Defense();
		handler2.BoxerHandling(boxer2, motion2);
		boxer2.CalcCoords();
		boxer2.CalcCenter();
		boxer2.CallLists();

		ttext.MessageEnter();
		ttext.SetColor(0.6, 0.6, 0.6);
		switch (lang) {
			case Kana:
				ttext.Print("܂", size, 0, 80);
            break;
			case English:
				ttext.Print("You were burnout.", size, 0, 80);
            break;
			default:
				ttext.Print("Rs܂", size, 0, 80);
            break;			
		}
		ttext.SetColor(1.0, 1.0, 1.0);
	  	ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();
		timer.Wait();

		if (size < 0.8) {
			size += 0.008;		
		}

		event.Poll();
		if (event.IsExpose() && ScreenMode) {
			ScreenReconfig();
			InitBurnOut();
			event.Clear();
			wmes("Change Resolution");
		} 
		if (IsAnyKeyDown() || event.IsMouse1Down()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
}

void Game::InitTournamentVictor() {
	MakeObjects(Partition);
	camera.Init(2);
	camera.Perspective(60.0, 0.05, 20.0);
	camera[0].DefPosition(0.0f, -4.0f, 2.0f);
	camera[0].DefDirection(0.0f, 0.0f, 1.0f);
	camera[0].DefRotation(0.0f);

	light.Init(0);
	light2.Init(1);
	light.DefDiffuse(0.8, 0.8, 0.8);
	light.DefPosition(0.0, 0.0, 1.0);
	light2.DefPosition(0.0, 0.0, 3.0);
	light2.DefDiffuse(0.8, 0.8, 0.8);
	light.On();
	light2.On();
}

void Game::TournamentVictor(StoryHandler &story) {
	mixer.PlayMusic(BGM_OPENING);
	float size;
	size = 0.0;

	if (!SetUpEventBoxer(story)) {
		return;	
	}
 	boxer2.SetPoint(0.0, -2.0, 180.0); 
	InitTournamentVictor();

	NextOff();
	timer.SetFPS(50);	
	do {
	//	SelectOptions();

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		light.Set();
		camera[0].Set();
		light2.Set();

		if (ShadowSwitch) {
			ShowShadow(boxer2);
	    }
		if (BackGroundSwitch) {
			glbg.Show();
		}
		glring.ShowAll();

		texs.Choice(TE_FACE);
		handler2.LeftOffense();
		handler2.BoxerHandling(boxer2, motion2);
		boxer2.CalcCoords();
		boxer2.CalcCenter();
		boxer2.CallLists();
	
		ttext.MessageEnter();
		switch (lang) {
			case Kana:
				ttext.Print("g[ig", size, 0, 80);
				ttext.Print("䂤傤", size, 0, 40);
            break;
			case English:
				ttext.Print("The Victor of", size, 0, 80);
				ttext.Print("the Tournament", size, 0, 40);
            break;
			default:
				ttext.Print("g[ig", size, 0, 80);
				ttext.Print("D", size, 0, 40);
            break;			
		}
	  	ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();
		timer.Wait();

		if (size < 1.0) {
			size += 0.01;		
		}

		event.Poll();
		if (event.IsExpose() && ScreenMode) {
			ScreenReconfig();
			InitTournamentVictor();
			event.Clear();
			wmes("Change Resolution");
		} 
		if (IsAnyKeyDown() || event.IsMouse1Down()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
}


void Game::InitWonTitle() {
	MakeObjects(Partition);
	camera.Init(2);
	camera.Perspective(60.0, 0.05, 20.0);
	camera[0].DefPosition(0.0f, -4.0f, 2.0f);
	camera[0].DefDirection(0.0f, 0.0f, 1.0f);
	camera[0].DefRotation(0.0f);

	light.Init(0);
	light2.Init(1);
	light.DefDiffuse(0.8, 0.8, 0.8);
	light.DefPosition(0.0, 0.0, 1.0);
	light2.DefPosition(0.0, 0.0, 3.0);
	light2.DefDiffuse(0.8, 0.8, 0.8);
	light.On();
	light2.On();
}

void Game::WonTitle(StoryHandler &story) {
	mixer.PlayMusic(BGM_OPENING);
	float size;
	size = 0.0;

	if (!SetUpEventBoxer(story)) {
		return;	
	}
 	boxer2.SetPoint(0.0, -2.0, 180.0); 
	InitWonTitle();

	NextOff();
	timer.SetFPS(50);	
	do {
	//	SelectOptions();

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		light.Set();
		camera[0].Set();
		light2.Set();

		if (ShadowSwitch) {
			ShowShadow(boxer2);
	    }
		if (BackGroundSwitch) {
			glbg.Show();
		}
		glring.ShowAll();

		texs.Choice(TE_FACE);
		handler2.RightOffense();
		handler2.BoxerHandling(boxer2, motion2);
		boxer2.CalcCoords();
		boxer2.CalcCenter();
		boxer2.CallLists();

		ttext.MessageEnter();
		switch (lang) {
			case Kana:
				ttext.Print("^CgƂ", size, 0, 80);
            break;
			case English:
				ttext.Print("Won the Title", size, 0, 80);
            break;
			default:
				ttext.Print("^Cgl", size, 0, 80);
            break;			
		}
	  	ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();
		timer.Wait();

		if (size < 1.0) {
			size += 0.01;		
		}

		event.Poll();
		if (event.IsExpose() && ScreenMode) {
			ScreenReconfig();
			InitWonTitle();
			event.Clear();
			wmes("Change Resolution");
		} 
		if (IsAnyKeyDown() || event.IsMouse1Down()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
}


