#include "game.h"

bool Game::Match(int select_type) {
	mixer.PlayMusic(BGM_MATCH);

	handler1.Init(boxer1, motion1, match);
	handler2.Init(boxer2, motion2, match);

	treble_mode = 0;
	can_change_mode1 = 1;
	can_change_mode2 = 1;
	which_select = 0;

	if (select_type == SELECT_1PSPARRING) {
		handler1.SetMode(FULL_AUTO);
		handler2.SetMode(Player1Mode);		
	} else if (select_type == SELECT_2PMATCH) {
		handler1.SetMode(Player2Mode);
		handler2.SetMode(Player1Mode);	
	} else {
		handler1.SetTraumDamage();		
		handler2.SetTraumDamage();		
		if (select_type == SELECT_STORY_RED) {
			handler1.SetMode(FULL_AUTO);
			handler2.SetMode(Player1Mode);
			can_change_mode1 = 0;
		} else if (select_type == SELECT_STORY_BLUE) {
			handler1.SetMode(Player1Mode);
			handler2.SetMode(FULL_AUTO);	
			can_change_mode2 = 0;
		} else {
			handler1.SetMode(FULL_AUTO);
			handler2.SetMode(FULL_AUTO);
			can_change_mode1 = 0;
			can_change_mode2 = 0;
		}
	}
	
	SetCamera();
	SetLights();
	SetDefaultView();	
	SetBoxerPosition(1.6);
	SetBoxerName();

	handler1.SetDistance(distance);	
	handler2.SetDistance(distance);	
	handler1.SetRound(1);
	handler2.SetRound(1);	
	handler1.SaveInfo(0, boxer1, motion1);
	handler2.SaveInfo(0, boxer2, motion2);	 
    match.SetCounter(50);

	StopOff();
	NextOff();
	timer.SetFPS(50);	
	do {
		MatchOptions();

	 	if (event.IsKeyPressed(SDLK_BACKSPACE)) {
			match.CalcModeOn();
		} else {
			if (treble_mode && (match.TimeCount() % treble_mode)) {
				match.CalcModeOn();		
			} else {
				match.CalcModeOff();	
			}
		}

	 	if (event.IsKeyDown(SDLK_2) && can_change_mode1) {
			handler1.NextMode();
		}
		if (event.IsKeyDown(SDLK_1) && can_change_mode2) {
			handler2.NextMode();
		}	

		if (match.IsShowRound()) {
			if (match.RingGong()) {
				mixer.PlayChunk(SE_GONG);
			}
			if (match.HasShownRound()) {
				match.Matching();
				handler1.SaveInfo(0, boxer1, motion1);
				handler2.SaveInfo(0, boxer2, motion2);	
				match.SetCounter(-50);		
			}
		} 
	
		handler1.SetDistance(distance);	
		handler2.SetDistance(distance);	

		switch (handler1.WhichMode()) {
			case SEMI_AUTO:
				handler1.GetComStates(match, boxer1, handler2);
				handler1.DeleteOffenseKey();
				GetOffenseKeyStates(handler1);
				GetOffenseMouseStates();
				ChangeThink1();
			break;		
			case FULL_AUTO:
				handler1.GetComStates(match, boxer1, handler2);
				ChangeThink1();
			break;
			default:
				GetKeyStates(handler1);
				GetOffenseMouseStates();
			break;
		}

		switch (handler2.WhichMode()) {
			case SEMI_AUTO:
				handler2.GetComStates(match, boxer2, handler1);
				handler2.DeleteOffenseKey();
				GetOffenseKeyStates(handler2);
				GetOffenseMouseStates();
				ChangeThink2();
			break;		
			case FULL_AUTO:
				handler2.GetComStates(match, boxer2, handler1);
				ChangeThink2();
			break;
			default:
				GetKeyStates(handler2);
				GetOffenseMouseStates();
			break;
		}

		if (which_select != 3 && match.IsMatching() && (event.IsKeyDown(go_next) || IsStop())) {
 			StopOff();
 			which_select = 3;
		}

		handler1.SaveKeyStates(match.TimeCount());
		handler2.SaveKeyStates(match.TimeCount());

		if (match.IsEnd()) {
			match.CalcModeOff();
		}		
		if (match.IsCalcMode() && !match.IsMatching()) {
			handler1.Defense();			
			handler2.Defense();				
		}

		if (match.IsBeforeMatch() && match.IsCounterZero()) {	
			if ((handler2.IsAnyAction() || handler2.IsFullAuto()) &&
				(handler1.IsAnyAction() || handler1.IsFullAuto()) ||
				event.IsKeyDown(go_next) || event.IsMouse1Pressed()) {
				match.ShowRound();
			}
		}

		if (match.IsRoundInterval()) {	
			if ((handler2.IsFullControl() && handler2.IsNoKey()) ||
				(handler1.IsFullControl() && handler1.IsNoKey()) ||
				(handler2.IsSemiAuto() && !handler2.IsAnyAction()) ||
				(handler1.IsSemiAuto() && !handler1.IsAnyAction())) {
				match.SkipIntervalCount();			
			}
			if (match.IsSkipInterval() &&
				(handler2.IsAnyAction() || handler2.IsFullAuto()) &&
				(handler1.IsAnyAction() || handler1.IsFullAuto()) ||
				event.IsKeyPressed(go_next) || event.IsMouse1Pressed()) {
				match.CalcModeOn();
			}
			if (distance > 6.0 &&
				(handler2.IsAnyAction() || handler2.IsFullAuto()) &&
				(handler1.IsAnyAction() || handler1.IsFullAuto())) {
				if (match.IsLastRound()) {
					match.Decision(handler1, handler2);
					if (match.IsExtraRoundSystem() && match.IsDraw() && !match.IsExtraRound()) {
						RoundStart();
					} else {
						match.End();
						mixer.PlayChunk(SE_KO);
						handler1.SaveEndInfo(boxer1, motion1);
						handler2.SaveEndInfo(boxer2, motion2);			
					}
				} else {
					RoundStart();
				}
			}
		}

		if (!match.IsMatching()) {
			handler1.InitKey();		
			handler2.InitKey();	
		} else {
			if (handler2.IsCounting() && !handler1.IsCounting()) {
				handler1.InitKey();				
				if (SkipDown && handler2.Count() > 0 && handler2.Count() < match.KOCount() -3) {
					match.CalcModeOn();				
				}
			}
			if (handler1.IsCounting() && !handler2.IsCounting()) {
				handler2.InitKey();		
				if (SkipDown && handler1.Count() > 0 && handler1.Count() < match.KOCount() -3) {
					match.CalcModeOn();				
				}
			}
		}

		if (match.IsCalcMode()) {
			CalcRoutine();		
		} else {
			switch (which_select) {
				case 1:
					SelectThink(handler1, 1);		
				break;
				case 2:
					SelectThink(handler2, 2);		
				break;
				case 3:
					switch (SelectStop(select_type)) {
						case 1 :
							ThrowIn(select_type);					
						break;
						case 2 :
							return false;					
						break;
					}
				break;
				default :
					ShowRoutine();		
				break;
			}
		}

		event.Poll();
		ExposeScreen();
		if (match.IsEnd() && (event.IsKeyDown(go_next) || IsNext())) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}

	return true;	
}

void Game::ShowRoutine() {
	static int loop = 0;

	handler1.BoxerHandling(boxer1, motion1);
	handler2.BoxerHandling(boxer2, motion2);
	if (handler1.IsSound()) {
		mixer.PlayChunk(handler1.SelectSound());	
	}  
	if (handler2.IsSound()) {
		mixer.PlayChunk(handler2.SelectSound());	
	}  

	if (handler1.IsClinching()) {
		handler2.SetStateClinch(motion2);	
	}
	if (handler2.IsClinching()) {
		handler1.SetStateClinch(motion1);	
	}  

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	SelectView();
	light2.Set();

	boxer1.CalcCoords();
	boxer1.CalcCenter();
	boxer2.CalcCoords();
	boxer2.CalcCenter();

	if (ShadowSwitch) {
		ShowShadow(boxer1);
		ShowShadow(boxer2);
    }
	if (BackGroundSwitch) {
		glbg.Show();
	}
	glring.Show(cxp, cyp, czp);

	if (handler1.IsOffenseState()) {
		handler1.CheckHitPunch(boxer1, motion1, boxer2, handler2);			
		if (handler1.IsSound()) {
			mixer.PlayChunk(handler1.SelectSound());	
		} 
	}
	if (handler2.IsOffenseState()) {
		handler2.CheckHitPunch(boxer2, motion2, boxer1, handler1);			
		if (handler2.IsSound()) {
			mixer.PlayChunk(handler2.SelectSound());	
		}
	}

	texs.Choice(TE_FACE);
	if (IsBackView1()) {
		if (show1)  {
			boxer1.CallHitRanges(boxer2);
		} else {
			boxer1.CallLists();
		}
		if (show2)  {
			boxer2.CallHitRanges(boxer1);
		} else {
			boxer2.CallLists();
		}
	} else {
		if (show2)  {
			boxer2.CallHitRanges(boxer1);
		} else {
			boxer2.CallLists(IsBoxerView1());
		}	
		if (show1)  {
			boxer1.CallHitRanges(boxer2);
		} else {
			boxer1.CallLists(IsBoxerView2());
		}
	}

	handler1.Move(boxer1);
	handler2.Move(boxer2);

	if (handler1.IsDamaged()) {
		handler1.SetDamage(boxer1, motion1);			
		handler1.SetNoDamage();
		handler2.SetStatePull();
	}
	if (handler2.IsDamaged()) {
		handler2.SetDamage(boxer2, motion2);			
		handler2.SetNoDamage();
		handler1.SetStatePull();
	}

	if (match.IsMatching()) {
		handler1.RestoreDamage();
		handler2.RestoreDamage();
    }

	ShowMeters();
	CalcAdjust();
	if (!match.IsEnd()) {
		CheckResult();	
		if (match.IsEnd()){
			mixer.PlayChunk(SE_KO);	
			match.SetLatestRound();	
			handler1.SaveEndInfo(boxer1, motion1);
			handler2.SaveEndInfo(boxer2, motion2);		
		}	
	}
	
	if (match.IsRoundInterval()) {
		if (handler1.IsDownState()) {
			handler1.SetStateRestore();		
			motion1.SetRestore();		
        }
		if (handler2.IsDownState()) {
			handler2.SetStateRestore();		
			motion2.SetRestore();		
        }
		if (handler1.IsWaitState()) {
			handler1.InitCount();					
			handler1.SetStateRhythm();				
		}
		if (handler2.IsWaitState()) {
			handler2.InitCount();					
			handler2.SetStateRhythm();				
		}
		boxer1.GoBlueCorner();	
		boxer2.GoRedCorner();	
	}

	ttext.MessageEnter();

 	ttext.Print(match.RoundString(), 0.4, -25, 100);
 	ttext.Print(match.RestTimeString(), 0.4, 25, 100);
  	ttext.Print(timer.FPSString(), 0.25, 100, -100);
	ShowBoxerNames();
	if (match.IsMatching()) {
		ShowStop();
	} else if (match.IsEnd()) {
		ShowNext();	
	}

	ttext.MessageLeave();

	if (match.IsMatching()) {
		ShowRefereeMessages();
	} else {
		ShowRound();
		ShowMessages();
	}

 	if (loop++ % SkipFrame == 0) SDL_GL_SwapBuffers(); 
	GLmes();
	timer.Wait();

	if (match.IsMatching()) {
		match.TickTime();
		if (match.IsGong() && !(match.CountMustbeContinuing() &&
			(handler1.IsCounting() || handler2.IsCounting()))) {
			RoundEnd();
		}
	}

	if (match.IsReplayTime() && match.IsMatching()) {
		handler1.SaveInfo(match.ReplayNumber(), boxer1, motion1);
		handler2.SaveInfo(match.ReplayNumber(), boxer2, motion2);	
	} 
}

void Game::CalcRoutine() {
	handler1.BoxerHandling(boxer1, motion1);
	handler2.BoxerHandling(boxer2, motion2);

	if (handler1.IsClinching()) {
		handler2.SetStateClinch(motion2);	
	}
	if (handler2.IsClinching()) {
		handler1.SetStateClinch(motion1);	
	}  

	CalcDistance();

	boxer1.CalcCoords();
	boxer1.CalcCenter();
	boxer2.CalcCoords();
	boxer2.CalcCenter();

	if (handler1.IsOffenseState()) {
		handler1.CheckHitPunch(boxer1, motion1, boxer2, handler2);			
	}
	if (handler2.IsOffenseState()) {
		handler2.CheckHitPunch(boxer2, motion2, boxer1, handler1);			
	}

	handler1.Move(boxer1);
	handler2.Move(boxer2);

	if (handler1.IsDamaged()) {
		handler1.SetDamage(boxer1, motion1);			
		handler1.SetNoDamage();
		handler2.SetStatePull();
	}
	if (handler2.IsDamaged()) {
		handler2.SetDamage(boxer2, motion2);			
		handler2.SetNoDamage();
		handler1.SetStatePull();
	}

	if (match.IsMatching()) {
		handler1.RestoreDamage();
		handler2.RestoreDamage();
    }

	CalcMeters();
	CalcAdjust();
	if (!match.IsEnd()) {
		CheckResult();
		if (match.IsEnd()){
			mixer.PlayChunk(SE_KO);	
			match.SetLatestRound();	
			handler1.SaveEndInfo(boxer1, motion1);
			handler2.SaveEndInfo(boxer2, motion2);		
		}	
	}

	if (match.IsRoundInterval()) {
		if (handler1.IsDownState()) {
			handler1.SetStateRestore();		
			motion1.SetRestore();		
        }
		if (handler2.IsDownState()) {
			handler2.SetStateRestore();		
			motion2.SetRestore();		
        }
		if (handler1.IsWaitState()) {
			handler1.InitCount();					
			handler1.SetStateRhythm();				
		}
		if (handler2.IsWaitState()) {
			handler2.InitCount();					
			handler2.SetStateRhythm();				
		}
		boxer1.GoBlueCorner();	
		boxer2.GoRedCorner();	
	}

	if (match.IsMatching() &&
		(handler1.IsCounting() || handler2.IsCounting()) &&
		(handler1.IsWaitState() || handler2.IsWaitState())) {
		match.SetCounter(-50);		
	} else {
		match.IsCounterPlus();
	}	

	if (match.IsMatching()) {
		match.TickTime();
		if (match.IsGong() && !(match.CountMustbeContinuing() &&
			(handler1.IsCounting() || handler2.IsCounting()))) {
			RoundEnd();
		}
	}
	  
	if (match.IsReplayTime() && match.IsMatching()) {
		handler1.SaveInfo(match.ReplayNumber(), boxer1, motion1);
		handler2.SaveInfo(match.ReplayNumber(), boxer2, motion2);	
	} 
}

void Game::CheckResult() {
	static bool is_match_end;
	is_match_end = false;

	if (handler1.IsCounting()) {
		handler1.CountInc();
		if (handler1.Down() >= match.TKODown() && match.IsMatching()) {
			match.RedWonbyTKO();		
			is_match_end = true;
		}		
		if (handler1.Count() >= match.KOCount() && match.IsMatching()) {
			match.RedWonbyKO();		
			is_match_end = true;
		}
	} else {
		boxer1.SetPoint(nox1, noy1, not1);		
		if (handler2.IsCounting()) {
			boxer1.GoNeutralCorner(boxer2);	
		}
	}

	if (handler2.IsCounting()) {
		handler2.CountInc();
		if (handler2.Down() >= match.TKODown() && match.IsMatching()) {
			if (match.IsRedWonbyTKO()) {
				match.DKODraw();			
			} else {
				match.BlueWonbyTKO();				
			}
			is_match_end = true;
		}		
		if (handler2.Count() >= match.KOCount() && match.IsMatching()) {
			if (match.IsRedWonbyKO()) {
				match.DKODraw();			
			} else {
				match.BlueWonbyKO();				
			}
			is_match_end = true;
		}
	} else {
		boxer2.SetPoint(nox2, noy2, not2);		
		if (handler1.IsCounting()) {
			boxer2.GoNeutralCorner(boxer1);	
		}
	}

	if (is_match_end) {
		match.End();
	}
}

void Game::SetBoxerPosition(float c_distance) {
	distance = c_distance;
	boxer1.SetPoint(-c_distance/sqrt(2.0)*0.5, c_distance/sqrt(2.0)*0.5, 225.0);
	motion1.SetRhythm();
	motion1.MotionRhythm().Set(&boxer1);
	boxer1.InitCenter();

	boxer2.SetPoint(c_distance/sqrt(2.0)*0.5, -c_distance/sqrt(2.0)*0.5, 45.0);
	motion2.SetRhythm();
	motion2.MotionRhythm().Set(&boxer2);
	boxer2.InitCenter();
}

void Game::SetBoxerName() {
	switch (lang) {
		case English:
			strcpy(player1_name, handler1.NickNameStrEn());
			strcpy(player2_name, handler2.NickNameStrEn());
		break;
		case Kana:
		default:
			strcpy(player1_name, handler1.NickNameStrJp());
			strcpy(player2_name, handler2.NickNameStrJp());
		break;
	}
}

void Game::RoundStart() {
	handler1.RestoreDamageInterval(match);	
	handler2.RestoreDamageInterval(match);	

	match.ShowRound();
	match.NextRound();
	match.SetTimeCount(0);

	handler1.InitRound();
	handler2.InitRound();				
	handler1.SetRound(match.Round());
	handler2.SetRound(match.Round());	
}

void Game::RoundEnd() {
	mixer.PlayChunk(SE_GONG);
	match.RoundInterval();
	match.SetLatestRound();
	match.Judgement(handler1, handler2);

	handler1.SetJudges();
	handler2.SetJudges();
	handler1.SaveIntervalInfo(boxer1, motion1);
	handler2.SaveIntervalInfo(boxer2, motion2);		
}

void Game::ShowRound() {
	if (match.IsShowRound()) {
		if (UsePText) {
		 	ptext.MessageEnter();
		 	ptext.SetColor(1.0, 0.0, 0.0);
		 	ptext.SetSpecular(1.0, 1.0, 1.0);
		 	ptext.SetShininess(128.0);
			ptext.Print(match.RoundString(), match.ShowRoundSize(), 0, 10);
		 	ptext.MessageLeave();
		} else {
		 	ttext.MessageEnter();
		 	ttext.SetColor(1.0, 0.0, 0.0);
			ttext.Print(match.RoundString(), match.ShowRoundSize(), 0, 10);
		 	ttext.SetColor(1.0, 1.0, 1.0);
		 	ttext.MessageLeave();		
		}
	}
} 

void Game::ShowRefereeMessages() {
	static bool is_down, is_break;
	ttext.MessageEnter();
	if (handler1.IsCounting() || handler2.IsCounting()) {
	  	ttext.Print("DOWN", 0.5, 0, 80);
		if (handler1.IsWaitState() || handler2.IsWaitState()) {
			match.SetCounter(-50);		
		}
		if (is_down) {
			mixer.PlayChunk(SE_DOWN);			
		}
		is_down = false;
	} else {
		if(match.IsCounterPlus()) {
			if (match.IsBox()) {
				mixer.PlayChunk(SE_BOX);			
			}
	  		ttext.Print("BOX", 0.5, 0, 80);	
		}
		is_down = true;
	}	
	if (handler1.IsClinching() || handler2.IsClinching()) {
	  	ttext.Print("BREAK", 0.5, 0, 80);
		if (is_break) {
			mixer.PlayChunk(SE_BREAK);			
		}
		is_break = false;
	} else {
		is_break = true;
	}
	ttext.MessageLeave();		
}

void Game::ShowBoxerNames() {
	if (IsView1()) { 
		ttext.Print(player1_name, 0.3, 85, 98);
		ttext.Print(player2_name, 0.3,-85, 98);
		if (handler1.Count()) {
			ttext.Print(handler1.CountString(), 0.75, 80, 40);
		}
		if (handler2.Count()) {
			ttext.Print(handler2.CountString(), 0.75, -80, 40);
		}
	} else {
		ttext.Print(player1_name, 0.3,-85, 98);	
		ttext.Print(player2_name, 0.3, 85, 98);
		if (handler1.Count()) {
			ttext.Print(handler1.CountString(), 0.75, -80, 40);
		}
		if (handler2.Count()) {
			ttext.Print(handler2.CountString(), 0.75, 80, 40);
		}
	}
}


void Game::ShowMessages() {
	static char judge_type[] = {'H', 'C', 'D'};
	static char buf[255];

	ttext.MessageEnter();
	if (match.IsBeforeMatch()) {
		ttext.SetColor(1.0, 0.0, 0.0);
		if (MatchInfo1Str(match.MatchRegion(), match.MatchType(), buf)) {
			ttext.Print(buf, 0.40, 0, 70);
			MatchInfo2Str(match.MatchType(), buf);
			ttext.Print(buf, 0.50, 0, 50);
		} else {
			ttext.Print(buf, 0.45, 0, 50);
		}
		ttext.SetColor(1.0, 1.0, 1.0);
		MatchRoundsStr(buf);
		ttext.Print(buf, 1.0, 0, 0);
	}

	if (match.IsRoundInterval()) {
		ResultStr(buf);
		ttext.Print(buf, 0.5, 0, 70);

		SituationStr(buf);
		ttext.Print(buf, 0.5, 0, -50);

		if (IsView1()) {
			sprintf(buf, "Down  %3d   %3d", handler1.DownInfo(), handler2.DownInfo());
			ttext.Print(buf, 0.5, 0, 40);
			sprintf(buf, "Hit   %3d   %3d", handler1.HitInfo(), handler2.HitInfo());
			ttext.Print(buf, 0.5, 0, 20);
			sprintf(buf, "Score %3d   %3d", handler2.ScoreInfo(), handler1.ScoreInfo());
			ttext.Print(buf, 0.5, 0, 0);
			sprintf(buf, "Total %3d   %3d", handler2.TotalScore(), handler1.TotalScore());
			ttext.Print(buf, 0.5, 0, -20);

			if (match.ShowExtraInfo()) {
				sprintf(buf, "Swing   %4d %4d", handler2.SwingInfo(), handler1.SwingInfo());
				ttext.Print(buf, 0.25, -50, -62);
				sprintf(buf, "Percent %4d %4d", handler2.HitPercent(handler1.HitInfo()), handler1.HitPercent(handler2.HitInfo()));
				ttext.Print(buf, 0.25, -50, -70);
				sprintf(buf, "Critical%4d %4d", handler1.CriticalInfo(), handler2.CriticalInfo());
				ttext.Print(buf, 0.25, -50, -78);
				sprintf(buf, "Damage  %4d %4d", handler1.RoundDamageInfo(), handler2.RoundDamageInfo());
				ttext.Print(buf, 0.25, -50, -86);
				sprintf(buf, "Stamina %4d %4d", handler2.StaminaPercent(), handler1.StaminaPercent());
				ttext.Print(buf, 0.25, -50, -94);

				sprintf(buf, "Judge1(%c)%4d %4d", judge_type[match.Judge1Type()], handler2.JudgeInfo(0), handler1.JudgeInfo(0));
				ttext.Print(buf, 0.25, 50, -62);
				sprintf(buf, "Judge2(%c)%4d %4d", judge_type[match.Judge2Type()], handler2.JudgeInfo(1), handler1.JudgeInfo(1));
				ttext.Print(buf, 0.25, 50, -70);
				sprintf(buf, "Judge3(%c)%4d %4d", judge_type[match.Judge3Type()], handler2.JudgeInfo(2), handler1.JudgeInfo(2));
				ttext.Print(buf, 0.25, 50, -78);
			}
		} else {
			sprintf(buf, "Down  %3d   %3d", handler2.DownInfo(), handler1.DownInfo());
			ttext.Print(buf, 0.5, 0, 40);
			sprintf(buf, "Hit   %3d   %3d", handler2.HitInfo(), handler1.HitInfo());
			ttext.Print(buf, 0.5, 0, 20);
			sprintf(buf, "Score %3d   %3d", handler1.ScoreInfo(), handler2.ScoreInfo());
			ttext.Print(buf, 0.5, 0, 0);
			sprintf(buf, "Total %3d   %3d", handler1.TotalScore(), handler2.TotalScore());
			ttext.Print(buf, 0.5, 0, -20);		

			if (match.ShowExtraInfo()) {
				sprintf(buf, "Swing   %4d %4d", handler1.SwingInfo(), handler2.SwingInfo());
				ttext.Print(buf, 0.25, -50, -62);
				sprintf(buf, "Percent %4d %4d", handler1.HitPercent(handler2.HitInfo()), handler2.HitPercent(handler1.HitInfo()));
				ttext.Print(buf, 0.25, -50, -70);
				sprintf(buf, "Critical%4d %4d", handler2.CriticalInfo(), handler1.CriticalInfo());
				ttext.Print(buf, 0.25, -50, -78);
				sprintf(buf, "Damage  %4d %4d", handler2.RoundDamageInfo(), handler1.RoundDamageInfo());
				ttext.Print(buf, 0.25, -50, -86);
				sprintf(buf, "Stamina %4d %4d", handler1.StaminaPercent(), handler2.StaminaPercent());
				ttext.Print(buf, 0.25, -50, -94);

				sprintf(buf, "Judge1(%c)%4d %4d", judge_type[match.Judge1Type()], handler1.JudgeInfo(0), handler2.JudgeInfo(0));
				ttext.Print(buf, 0.25, 50, -62);
				sprintf(buf, "Judge2(%c)%4d %4d", judge_type[match.Judge2Type()], handler1.JudgeInfo(1), handler2.JudgeInfo(1));
				ttext.Print(buf, 0.25, 50, -70);
				sprintf(buf, "Judge3(%c)%4d %4d", judge_type[match.Judge3Type()], handler1.JudgeInfo(2), handler2.JudgeInfo(2));
				ttext.Print(buf, 0.25, 50, -78);
			}
		}
	}      

	if (match.IsEnd()) {
		ShowResults();
	}
	ttext.MessageLeave();
}

void Game::ShowResults() {
	static char line[] = "-------------------------------------------";
	static char buf[255];
	int i;
	if (match.IsBlueWon()) {
		BlueWinStr(buf);
		ttext.Print(buf, 0.5, 0, 80);
	}

	if (match.IsRedWon()) {
		RedWinStr(buf);
		ttext.Print(buf, 0.5, 0, 80);
	}

	if (match.IsDraw()) {
		DrawStr(buf);
		ttext.Print(buf, 0.5, 0, 80);
	}

	if (match.IsJudge()) {
		JudgeStr(buf);
		ttext.Print(buf, 0.5, 0, 60);
		if (IsView1()) {
			for (i = 0; i < match.MaxRound(); i++) {
				sprintf(buf, "Round%2d   %3d %3d %3d   %3d %3d %3d", i+1, 
				handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2),
				handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - i*8);
			}
			 ttext.Print(line, 0.25, 0, 40 - i*8);
			 sprintf(buf, " Total    %3d %3d %3d   %3d %3d %3d", 
			 handler2.TotalJudge(0), handler2.TotalJudge(1), handler2.TotalJudge(2),
			 handler1.TotalJudge(0), handler1.TotalJudge(1), handler1.TotalJudge(2));
			 ttext.Print(buf, 0.25, 0, 40 - (i+1)*8);
		} else {
			for (i = 0; i < match.MaxRound(); i++) {
				sprintf(buf, "Round%2d   %3d %3d %3d   %3d %3d %3d", i+1, 
				handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2),
				handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - i*8);
			}
			ttext.Print(line, 0.25, 0, 40 - i*8);
			sprintf(buf, " Total    %3d %3d %3d   %3d %3d %3d", 
			handler1.TotalJudge(0), handler1.TotalJudge(1), handler1.TotalJudge(2),
			handler2.TotalJudge(0), handler2.TotalJudge(1), handler2.TotalJudge(2));
			ttext.Print(buf, 0.25, 0, 40 - (i+1)*8);		
		}		
	} else {
		if (match.IsNoContest()) {
			ttext.Print("No Contset", 0.5, 0, 60);
		} else if (match.IsDKODraw()) {
			ttext.Print("Double KO", 0.5, 0, 60);
		} else if (match.IsTKO()) {
			ttext.Print("TKO", 0.5, 0, 60);		
		} else {
			ttext.Print("KO", 0.5, 0, 60);		
		}
		if (IsView1()) {
			for (i = 0; i < match.Round()-1; i++) {
				sprintf(buf, "Round%2d   %3d %3d %3d   %3d %3d %3d", i+1, 
				 handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2),
				 handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - i*8);
			}
		} else {
			for (i = 0; i < match.Round()-1; i++) {
				sprintf(buf, "Round%2d   %3d %3d %3d   %3d %3d %3d", i+1, 
				 handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2),
				 handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - i*8);
			}		
		}		
	}

	if (match.IsExtraRoundSystem()) {
		if (match.ExtraResultState() == JUDGE_DRAW) {
			if (IsView1()) {
				sprintf(buf, " Extra    %3d %3d %3d   %3d %3d %3d", 
				handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2),
				handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - (i+2)*8);
			} else {
				sprintf(buf, " Extra    %3d %3d %3d   %3d %3d %3d", 
				handler1.JudgeInfo(i, 0), handler1.JudgeInfo(i, 1), handler1.JudgeInfo(i, 2),
				handler2.JudgeInfo(i, 0), handler2.JudgeInfo(i, 1), handler2.JudgeInfo(i, 2));
				ttext.Print(buf, 0.25, 0, 40 - (i+2)*8);
			}
		}

		if (match.IsDraw()){
			if (match.WhichWin(handler1, handler2) == 2) {
				ExtraRedWinStr(buf);
			} else {
				ExtraBlueWinStr(buf);
			}
			ttext.Print(buf, 0.5, 50, 60);
		}
	}
}

void Game::MatchRoundsStr(char *buf) {		
	static char *round_name[] = {
		"", "", "O", "l", "܉", "Z", 
		"", "", "", "\x8f\\", "\x8f\\", "\x8f\\",
		"\x8f\\O", "\x8f\\l", "\x8f\\܉", ""};
	static char *round_kana[] = {
		"", "ɂ", "񂩂", "񂩂", "", "", 
		"ȂȂ", "͂", "イ", "", "PP", "PQ",
		"PR", "PS", "PT", "ނ"};
	static char *round_english[] = {
		"1-Round Match", "2-Round Match", "3-Round Match", "4-Round Match", "5-Round Match", "6-Round Match", 
		"7-Round Match", "8-Round Match", "9-Round Match", "10-Round Match", "11-Round Match", "12-Round Match",
		"13-Round Match", "14-Round Match", "15-Round Match", "Unlimited Match"};
	switch (lang) {
		case Kana:
			sprintf(buf, "%s", round_kana[match.LimitRound()]);
		break;
		case English:
			sprintf(buf, "%s", round_english[match.LimitRound()]);
		break;
		default:
			sprintf(buf, "%s", round_name[match.LimitRound()]);
		break;
	}								
}

void Game::ResultStr(char *buf) {		
	switch (lang) {
		case Kana:
			sprintf(buf, "%s ", match.RoundString());
		break;
		case English:
			sprintf(buf, "%s Results", match.RoundString());
		break;
		default:
			sprintf(buf, "%s ", match.RoundString());
		break;
	}
}

void Game::SituationStr(char *buf) {		
/*
	static char *situation1p[] = {"K", "D", "D", "݊p", "", "", "sF"};
	static char *situation1p_kana[] = {"Ђ傤", "䂤", "䂤", "", "", "", "͂傭"};
	static char *situation1p_english[] = {"Best", "Better", "Good", "Even", "Bad", "Worse", "Worst"};
*/
	static char *situation[] = {"ԃR[i[K", "ԃR[i[D", "ԃR[i[D", "݊p", "R[i[D", "R[i[D", "R[i[K"};
	static char *situation_kana[] = {"Ђ傤", "䂤", "䂤", "", "䂤", "䂤", "Ђ傤"};
	static char *situation_english[] = {"Red is Winning", "Red is Leading", "Red is Advantaged", "Even", "Blue is Advantaged", "Blue is Leading", "Blue is Winning"};
	switch (lang) {
		case Kana:
			sprintf(buf, "傤傤 : %s", situation_kana[match.Situation(handler1, handler2)]);
		break;
		case English:
			sprintf(buf, "Situation : %s", situation_english[match.Situation(handler1, handler2)]);
		break;
		default:
			sprintf(buf, " : %s", situation[match.Situation(handler1, handler2)]);
		break;
	}
}

void Game::RedWinStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "R[i[̂");
		break;
		case English:
			strcpy(buf, "Winner is Red Corner");
		break;
		default:
			strcpy(buf, "ԃR[i[");
		break;
	}
}

void Game::BlueWinStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "R[i[̂");
		break;
		case English:
			strcpy(buf, "Winner is Blue Corner");
		break;
		default:
			strcpy(buf, "R[i[");
		break;
	}
}

void Game::DrawStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "Ђ킯");
		break;
		case English:
			strcpy(buf, "This Match is Draw");
		break;
		default:
			strcpy(buf, "");
		break;
	}
}

void Game::JudgeStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "͂Ă");
		break;
		case English:
			strcpy(buf, "Decision");
		break;
		default:
			strcpy(buf, "");
		break;
	}
}

void Game::ExtraRedWinStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "()");
		break;
		case English:
			strcpy(buf, "(Red)");
		break;
		default:
			strcpy(buf, "()");
		break;
	}
}

void Game::ExtraBlueWinStr(char *buf) {		
	switch (lang) {
		case Kana:
			strcpy(buf, "()");
		break;
		case English:
			strcpy(buf, "(Blue)");
		break;
		default:
			strcpy(buf, "()");
		break;
	}
}
