#include "game.h"

void Game::Replay() {
	mixer.PlayMusic(BGM_MATCH);

	replay_pause = 0;
	can_change_mode1 = 0;
	can_change_mode2 = 0;

	SetCamera();
	SetLights();
	SetDefaultView();	
	SetBoxerPosition(1.6);
	SetBoxerName();

	match.SetRound(1);
	match.SetTimeCount(0);

	handler1.SetRound(match.Round());
	handler2.SetRound(match.Round());	
	handler1.LoadInfo(0, boxer1, motion1);
	handler2.LoadInfo(0, boxer2, motion2);	
	handler1.SumJudge();
	handler2.SumJudge();	
	distance = handler2.Distance();
	match.BeforeMatch();	
	match.SetCounter(50);

	NextOff();
	timer.SetFPS(50);
	do {
		ReplayInterface();
		
		MatchOptions();
  	 	if (event.IsKeyDown(SDLK_i)) {
			match.ToggleExtraInfo();
		}

		if (match.IsShowRound()) {
			if (match.RingGong()) {
				mixer.PlayChunk(SE_GONG);
			}
			if (match.HasShownRound()) {
				match.Matching();
				match.PopCalcMode();
				handler1.LoadInfo(0, boxer1, motion1);
				handler2.LoadInfo(0, boxer2, motion2);	
				distance = handler1.Distance();
				match.SetCounter(-50);		
			}
		} 
	
		handler1.SetDistance(distance);	
		handler2.SetDistance(distance);	

		if (handler1.HasKeyStates(match.TimeCount())) {
			handler1.LoadKeyStates(match.TimeCount());
		} else {
			match.NoData();
		}
		if (handler2.HasKeyStates(match.TimeCount())) {
			handler2.LoadKeyStates(match.TimeCount());
		} else {
			match.NoData();				
		}

		if (match.IsBeforeMatch()) {	
			if (handler2.IsAnyAction() || match.IsCounterZero()) {
				match.ShowRound();
			}
		}

		if (match.IsRoundInterval()) {	
//			match.SkipIntervalCount();			
			if (match.IsSkipInterval()) {
				match.CalcModeOn();		
			}
			if (distance > 6.0) {
				if (match.IsLastRound()) {
					match.Decision(handler1, handler2);
					if (match.IsExtraRoundSystem() && match.IsDraw() && !match.IsExtraRound()) {
						RoundStart();
					} else {
						match.End();
						mixer.PlayChunk(SE_KO);
						handler1.SaveEndInfo(boxer1, motion1);
						handler2.SaveEndInfo(boxer2, motion2);			
					}
				} else {
					RoundStart();
				}
			}
		}

		if (!match.IsMatching()) {
			handler1.InitKey();		
			handler2.InitKey();	
		} else {
			if (handler2.IsCounting() && !handler1.IsCounting()) {
				handler1.InitKey();		
			}
			if (handler1.IsCounting() && !handler2.IsCounting()) {
				handler2.InitKey();		
			}
		}

		if (match.IsCalcMode()) {
			CalcReplayRoutine();		
		} else {
			ShowReplayRoutine();		
		}

		if (!handler1.HasKeyStates(match.TimeCount()+1)) {
			if (handler2.HasKeyStates(match.TimeCount()+1)) {
				if (!match.IsEnd()){
					mixer.PlayChunk(SE_KO);
					match.SetResultState(handler2.LoadThrowInState(match.TimeCount()+1));
					match.End();
				}
        	}
		}
		event.Poll();
		ExposeScreen();
		if (event.IsKeyDown(go_next) || IsNext()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
	mixer.SetChunkVolume(SEVolume);
}

void Game::ShowReplayRoutine() {
	static int loop = 0;

	handler1.BoxerHandling(boxer1, motion1);
	handler2.BoxerHandling(boxer2, motion2);
	if (handler1.IsSound()) {
		mixer.PlayChunk(handler1.SelectSound());	
	}
	if (handler2.IsSound()) {
		mixer.PlayChunk(handler2.SelectSound());	
	}

	if (handler1.IsClinching()) {
		handler2.SetStateClinch(motion2);	
	}
	if (handler2.IsClinching()) {
		handler1.SetStateClinch(motion1);	
	}  

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	SelectView();
	light2.Set();

	boxer1.CalcCoords();
	boxer1.CalcCenter();
	boxer2.CalcCoords();
	boxer2.CalcCenter();

	if (ShadowSwitch) {
		ShowShadow(boxer1);
		ShowShadow(boxer2);
    }
	if (BackGroundSwitch) {
		glbg.Show();
	}
	glring.Show(cxp, cyp, czp);

	if (handler1.IsOffenseState()) {
		handler1.CheckHitPunch(boxer1, motion1, boxer2, handler2);			
		if (handler1.IsSound()) {
			mixer.PlayChunk(handler1.SelectSound());	
		}
	}
	if (handler2.IsOffenseState()) {
		handler2.CheckHitPunch(boxer2, motion2, boxer1, handler1);			
		if (handler2.IsSound()) {
			mixer.PlayChunk(handler2.SelectSound());	
		}
	}

	texs.Choice(TE_FACE);
	if (IsBackView1()) {
		if (show1)  {
			boxer1.CallHitRanges(boxer2);
		} else {
			boxer1.CallLists();
		}
		if (show2)  {
			boxer2.CallHitRanges(boxer1);
		} else {
			boxer2.CallLists();
		}
	} else {
		if (show2)  {
			boxer2.CallHitRanges(boxer1);
		} else {
			boxer2.CallLists(IsBoxerView1());
		}	
		if (show1)  {
			boxer1.CallHitRanges(boxer2);
		} else {
			boxer1.CallLists(IsBoxerView2());
		}
	}

	handler1.Move(boxer1);
	handler2.Move(boxer2);

	if (handler1.IsDamaged()) {
		handler1.SetDamage(boxer1, motion1);			
		handler1.SetNoDamage();
		handler2.SetStatePull();
	}
	if (handler2.IsDamaged()) {
		handler2.SetDamage(boxer2, motion2);			
		handler2.SetNoDamage();
		handler1.SetStatePull();
	}

	if (match.IsMatching()) {
		handler1.RestoreDamage();
		handler2.RestoreDamage();
    }

	ShowMeters();
	CalcAdjust();
	if (!match.IsEnd()) {
		CheckResult();	
		if (match.IsEnd()){
			mixer.PlayChunk(SE_KO);		
		}	
	}

	if (match.IsRoundInterval()) {
		if (handler1.IsDownState()) {
			handler1.SetStateRestore();		
			motion1.SetRestore();		
        }
		if (handler2.IsDownState()) {
			handler2.SetStateRestore();		
			motion2.SetRestore();		
        }
		if (handler1.IsWaitState()) {
			handler1.InitCount();					
			handler1.SetStateRhythm();				
		}
		if (handler2.IsWaitState()) {
			handler2.InitCount();					
			handler2.SetStateRhythm();				
		}
		boxer1.GoBlueCorner();	
		boxer2.GoRedCorner();	
	}

	ttext.MessageEnter();

 	ttext.Print(match.RoundString(), 0.4, -25, 100);
 	ttext.Print(match.RestTimeString(), 0.4, 25, 100);
  	ttext.Print(timer.FPSString(), 0.25, 100, -100);

	ShowReplayMessages();
	ShowBoxerNames();
	ShowNext();
	ttext.MessageLeave();

	if (match.IsMatching()) {
		ShowRefereeMessages();
	} else {
		ShowRound();
		ShowMessages();
	}
	ShowReplayInterface();

 	if (loop++ % SkipFrame == 0) SDL_GL_SwapBuffers(); 
	GLmes();
	timer.Wait();

	if (match.IsMatching()) {
		match.TickTime();
		if (match.IsGong() && !(match.CountMustbeContinuing() &&
			(handler1.IsCounting() || handler2.IsCounting()))) {
			RoundReplayEnd();
		}
	}
}

void Game::CalcReplayRoutine() {
	handler1.BoxerHandling(boxer1, motion1);
	handler2.BoxerHandling(boxer2, motion2);

	if (handler1.IsClinching()) {
		handler2.SetStateClinch(motion2);	
	}
	if (handler2.IsClinching()) {
		handler1.SetStateClinch(motion1);	
	}  

	CalcDistance();

	boxer1.CalcCoords();
	boxer1.CalcCenter();
	boxer2.CalcCoords();
	boxer2.CalcCenter();

	if (handler1.IsOffenseState()) {
		handler1.CheckHitPunch(boxer1, motion1, boxer2, handler2);			
	}
	if (handler2.IsOffenseState()) {
		handler2.CheckHitPunch(boxer2, motion2, boxer1, handler1);			
	}

	handler1.Move(boxer1);
	handler2.Move(boxer2);

	if (handler1.IsDamaged()) {
		handler1.SetDamage(boxer1, motion1);			
		handler1.SetNoDamage();
		handler2.SetStatePull();
	}
	if (handler2.IsDamaged()) {
		handler2.SetDamage(boxer2, motion2);			
		handler2.SetNoDamage();
		handler1.SetStatePull();
	}

	if (match.IsMatching()) {
		handler1.RestoreDamage();
		handler2.RestoreDamage();
    }

	CalcMeters();
	CalcAdjust();
	if (!match.IsEnd()) {
		CheckResult();
		if (match.IsEnd()){
			mixer.PlayChunk(SE_KO);		
		}	
	}

	if (match.IsRoundInterval()) {
		if (handler1.IsDownState()) {
			handler1.SetStateRestore();		
			motion1.SetRestore();		
        }
		if (handler2.IsDownState()) {
			handler2.SetStateRestore();		
			motion2.SetRestore();		
        }
		if (handler1.IsWaitState()) {
			handler1.InitCount();					
			handler1.SetStateRhythm();				
		}
		if (handler2.IsWaitState()) {
			handler2.InitCount();					
			handler2.SetStateRhythm();				
		}
		boxer1.GoBlueCorner();	
		boxer2.GoRedCorner();	
	}

	if (match.IsMatching() &&
		(handler1.IsCounting() || handler2.IsCounting()) &&
		(handler1.IsWaitState() || handler2.IsWaitState())) {
		match.SetCounter(-50);		
	} else {
		match.IsCounterPlus();
	}	

	if (match.IsMatching()) {
		match.TickTime();
		if (match.IsGong() && !(match.CountMustbeContinuing() &&
			(handler1.IsCounting() || handler2.IsCounting()))) {
			RoundReplayEnd();
		}
	}
}

void Game::RoundReplayEnd() {
	mixer.PlayChunk(SE_GONG);
	match.RoundInterval();
	match.SetCounter(50);

	handler1.GetJudges();
	handler2.GetJudges();
	handler1.TotalScoreInc();
	handler2.TotalScoreInc();
}

void Game::ShowReplayMessages() {
	if (match.IsNoData()) {
		ttext.SetColor(1.0, 0.0, 0.0);
		switch (lang) {
			case Kana:
				ttext.Print("f[^Ȃ", 0.5, 0, -70);
			break;
			case English:
				ttext.Print("No Data", 0.5, 0, -70);
			break;
			default:
				ttext.Print("f[^܂", 0.5, 0, -70);
			break;
		}
		ttext.SetColor(1.0, 1.0, 1.0);
	} else if (show_interface) {
		switch (lang) {
			case Kana:
				ttext.Print("vCł", 0.5, 0, -70);
			break;
			case English:
				ttext.Print("Replaying", 0.5, 0, -70);
			break;
			default:
				ttext.Print("vC", 0.5, 0, -70);
			break;
		}
	}
}

void Game::ReplayInterface() {
	static int x, y, save_time_count, save_round;

	replay_select = -1;
	SDL_GetMouseState(&x, &y);
	if (height*0.900 < y && height*0.950 > y) {
		if (width*0.125 < x && width*0.150 > x) {
			replay_select = INTERFACE_TOGGLE;		
		}
		if (width*0.175 < x && width*0.225 > x) {
			replay_select = NEXT_ROUND;				
		}				
		if (width*0.225 < x && width*0.275 > x) {
			replay_select = PREVIOUS_ROUND;				
		}						
		if (width*0.275 < x && width*0.350 > x) {
			replay_select = ROUND_START;						
		}
		if (width*0.350 < x && width*0.450 > x) {
			replay_select = PREVIOUS_TIME;				
		}
		if (width*0.450 < x && width*0.550 > x) {
			replay_select = MOVE_TOGGLE;				
		}
		if (width*0.550 < x && width*0.650 > x) {
			replay_select = NEXT_TIME;				
		}
		if (width*0.650 < x && width*0.725 > x) {
			replay_select = ROUND_END;				
		}		
		if (width*0.725 < x && width*0.775 > x) {
			replay_select = MOVESPEED_UP;						
		}
		if (width*0.775 < x && width*0.825 > x) {
			replay_select = MOVESPEED_DOWN;								
		}
	}

	if (replay_select == INTERFACE_TOGGLE && event.IsMouse1Down()) {
		if (show_interface) {
			show_interface = 0;		
		} else {
			show_interface = 1;				
		}
	}

	if ((replay_select == MOVE_TOGGLE && event.IsMouse1Down()) ||
		event.IsKeyDown(kb2_defense)) {
		if (match.IsMatching() || match.IsRoundInterval() || match.IsEnd()) {
			if (replay_pause) {
			 	replay_move = 1;
				mixer.SetChunkVolume(SEVolume);
			} else {
			 	replay_pause = 1;			
			 	replay_move = 0;
				pause_time_count = match.TimeCount();			
				mixer.SetChunkVolume(0);
			}
		}
	}

	if ((replay_select == NEXT_TIME && event.IsMouse1Down()) ||
		event.IsKeyDown(kb2_forward)) {			
		if (replay_pause) {
			pause_time_count++;
			if (pause_time_count > handler1.LastKeyStates()-2) {
				pause_time_count = handler1.LastKeyStates()-2;
			}
		} else { 
			if(match.IsRoundInterval()) {
				match.SkipIntervalOn();		
			}   
			save_time_count = match.TimeCount();			
			match.NextReplay();
			if (handler1.HasInfo(match.ReplayNumber()) &&
				handler1.HasKeyStates(match.TimeCount())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);
				distance = handler2.Distance();
				match.Matching();
			} else {
				match.SetTimeCount(save_time_count);					
			}
		}
	}
	if ((replay_select == PREVIOUS_TIME && event.IsMouse1Down()) ||
		event.IsKeyDown(kb2_back)) {			
		if (replay_pause) {
			if (pause_time_count > 1) {
				pause_time_count--;
				if(match.IsRoundInterval() || match.IsEnd()) {
					pause_time_count--;
					match.Matching();
				}
			}
		} else {		
			save_time_count = match.TimeCount();			
			match.PreReplay();
			if (handler1.HasInfo(match.ReplayNumber())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
				distance = handler2.Distance();
				match.Matching();
			} else {
				match.SetTimeCount(save_time_count);					
			}
		}
	}

	if (replay_select == ROUND_START && event.IsMouse1Down()) {
		if (replay_pause) {
			if (pause_time_count >= match.ReplayRate()) {
				pause_time_count-= match.ReplayRate();			
				match.Matching();
            }
		} else {
			save_time_count = match.TimeCount();			
			match.SetTimeCount(0);
			match.SetReplay();
			if (handler1.HasInfo(match.ReplayNumber())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
				distance = handler2.Distance();
				match.Matching();
			} else {
				match.SetTimeCount(save_time_count);					
			}
		}
	}
	if (replay_select == ROUND_END && event.IsMouse1Down()) {
		if (replay_pause) {
			pause_time_count+= match.ReplayRate();
			if (pause_time_count > handler1.LastKeyStates()-2) {
				pause_time_count = handler1.LastKeyStates()-2;
			}
		} else {
			pause_time_count = handler1.LastKeyStates()-2;
			match.SetTimeCount(pause_time_count);
			match.Matching();
			replay_pause = 1;
			replay_move = 1;
			match.SetReplay();
			if (handler1.HasInfo(match.ReplayNumber())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
				distance = handler2.Distance();
				match.Matching();
			} else {
				match.SetTimeCount(save_time_count);					
			}			
		}
	}

	if ((replay_select == NEXT_ROUND && event.IsMouse1Down()) ||
		event.IsKeyDown(kb2_left_offense)) {			
		save_round = match.Round();
		match.NextRound();
		if (match.Round() > match.LatestRound()) {
			match.SetRound(save_round);		
		}
		if (replay_pause) {
			handler1.SetRound(match.Round());
			handler2.SetRound(match.Round());	
			match.SetTimeCount(handler1.LastKeyStates());
			if (handler1.ExistIntervalInfo()) {
				match.RoundInterval();				
			} else {
				if (match.IsExtraRound()) {
					match.SetResultState(match.ExtraResultState());	
				}
				match.End();
			}		
		} else {
			save_time_count = match.TimeCount();			
			match.SetReplay();
   	
			handler1.SetRound(match.Round());
			handler2.SetRound(match.Round());	
			match.SetTimeCount(0);
			match.SetReplay();
			if (handler1.HasInfo(match.ReplayNumber())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
				distance = handler2.Distance();
				match.ShowRound();
			} else {
				match.SetTimeCount(save_time_count);					
				match.SetRound(save_round);
				handler1.SetRound(match.Round());
				handler2.SetRound(match.Round());				
			}
		}
	}
	if ((replay_select == PREVIOUS_ROUND && event.IsMouse1Down())
		|| event.IsKeyDown(kb2_right_offense)) {			
		save_round = match.Round();
		match.PreRound();
		if (replay_pause) {
			handler1.SetRound(match.Round());
			handler2.SetRound(match.Round());	
			match.SetTimeCount(handler1.LastKeyStates());
			if (handler1.ExistIntervalInfo()) {
				match.RoundInterval();				
			} else {
				match.End();							
			}		
		} else {
			save_time_count = match.TimeCount();			
			match.SetReplay();
				
			handler1.SetRound(match.Round());
			handler2.SetRound(match.Round());	
			match.SetTimeCount(0);
			match.SetReplay();
			if (handler1.HasInfo(match.ReplayNumber())) {
				handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
				handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
				distance = handler2.Distance();
				match.ShowRound();
			} else {
				match.SetTimeCount(save_time_count);					
				match.SetRound(save_round);
				handler1.SetRound(match.Round());
				handler2.SetRound(match.Round());				
			}
		}
	}

	if ((replay_select == MOVESPEED_UP && event.IsMouse1Pressed()) ||
		event.IsKeyPressed(kb2_left)) {			
		replay_speed = timer.FPS();
		if (++replay_speed > 1000) {
			replay_speed = 1000;
		}
		timer.SetFPS(replay_speed);	
	}
	if ((replay_select == MOVESPEED_DOWN && event.IsMouse1Pressed()) ||
		event.IsKeyPressed(kb2_right)) {
		replay_speed = timer.FPS();
		if (--replay_speed < 1) {
			replay_speed = 1;
		}
		timer.SetFPS(replay_speed);	
	}

	if (replay_pause) {			
		if (match.IsMatching()) {
            if (match.TimeCount() < pause_time_count) {
				match.CalcModeOn();
			} else if (match.TimeCount() == pause_time_count) { 
				if (replay_move) {
				 	replay_pause = 0;
				}		
				match.CalcModeOff();
			} else {
				match.PreReplay();
				if (handler1.HasInfo(match.ReplayNumber())) {
					handler1.LoadInfo(match.ReplayNumber(), boxer1, motion1);
					handler2.LoadInfo(match.ReplayNumber(), boxer2, motion2);	
					distance = handler2.Distance();
				} else {
					match.SetTimeCount(save_time_count);					
				}
				match.CalcModeOn();
			}
		} else {
			if (match.IsRoundInterval()) {
				handler1.LoadIntervalInfo(boxer1, motion1);
				handler2.LoadIntervalInfo(boxer2, motion2);	
				handler1.GetJudges();
				handler2.GetJudges();
			} else {
				handler1.LoadEndInfo(boxer1, motion1);
				handler2.LoadEndInfo(boxer2, motion2);	
			}
			distance = handler2.Distance();			
			if (replay_move) {
				replay_pause = 0;
			}		
			match.CalcModeOff();
		}
	} else {
		if (!match.IsEnd() &&
			(timer.FPS() > 60 && match.TimeCount() % (1+timer.FPS()/60))) {	
			match.CalcModeOn();
		} else {
			match.CalcModeOff();
		}
	}
}	


void Game::ShowReplayInterface() {
	static char buf[10];

	if (UsePText) {
		ptext.MessageEnter();
		ptext.SetSpecular(0.0, 0.0, 0.0);
		ptext.SetShininess(0.0);
	
		ptext.SelectColor(replay_select == INTERFACE_TOGGLE);
		ptext.Symbol(SYMBOL_CIRCLE2, 0.5, -75, -90);

		if (show_interface) {
			ptext.SelectColor(replay_select == NEXT_ROUND);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.5, -60, -90);

			ptext.SelectColor(replay_select == PREVIOUS_ROUND);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.5,-50, -90);
	
			ptext.SelectColor(replay_select == ROUND_START);
			ptext.Print("<<", 0.5,-35, -90);
   	
			ptext.SelectColor(replay_select == PREVIOUS_TIME);
			ptext.Print("<-", 0.5,-15, -90);

			ptext.SelectColor(replay_select == MOVE_TOGGLE);
			ptext.Print("=", 0.5, 0, -90);

			ptext.SelectColor(replay_select == NEXT_TIME);
			ptext.Print("->", 0.5, 15, -90);

			ptext.SelectColor(replay_select == ROUND_END);
			ptext.Print(">>", 0.5, 35, -90);

			ptext.SelectColor(replay_select == MOVESPEED_UP);
			ptext.Symbol(SYMBOL_TRIANGLE_UP1, 0.5, 50, -90);

			ptext.SelectColor(replay_select == MOVESPEED_DOWN);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN1, 0.5, 60, -90);

			ptext.SetColor(1.0, 1.0, 1.0);	
			sprintf(buf, "%4.2f", (float)timer.FPS()/50.0);
			ptext.Print(buf, 0.5, 60, -70);
		}
		ptext.MessageLeave();
	} else {
		ttext.MessageEnter();
		ttext.SelectColor(replay_select == INTERFACE_TOGGLE);
		ttext.Symbol(SYMBOL_CIRCLE2, 0.5, -75, -90);

		if (show_interface) {
			ttext.SelectColor(replay_select == NEXT_ROUND);
			ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.5, -60, -90);

			ttext.SelectColor(replay_select == PREVIOUS_ROUND);
			ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.5,-50, -90);
	
			ttext.SelectColor(replay_select == ROUND_START);
			ttext.Print("<<", 0.5,-35, -90);
   	
			ttext.SelectColor(replay_select == PREVIOUS_TIME);
			ttext.Print("<-", 0.5,-15, -90);

			ttext.SelectColor(replay_select == MOVE_TOGGLE);
			ttext.Print("=", 0.5, 0, -90);

			ttext.SelectColor(replay_select == NEXT_TIME);
			ttext.Print("->", 0.5, 15, -90);

			ttext.SelectColor(replay_select == ROUND_END);
			ttext.Print(">>", 0.5, 35, -90);

			ttext.SelectColor(replay_select == MOVESPEED_UP);
			ttext.Symbol(SYMBOL_TRIANGLE_UP1, 0.5, 50, -90);

			ttext.SelectColor(replay_select == MOVESPEED_DOWN);
			ttext.Symbol(SYMBOL_TRIANGLE_DOWN1, 0.5, 60, -90);

			ttext.SetColor(1.0, 1.0, 1.0);	
			sprintf(buf, "%4.2f", (float)timer.FPS()/50.0);
			ttext.Print(buf, 0.5, 60, -70);
		}
		ttext.SetColor(1.0, 1.0, 1.0);
		ttext.MessageLeave();
	}
}

int Game::SaveReplayFile(const char *filename) {
	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "wb");
 
	if (rwops == NULL) {
		emes("Game::SaveReplayFile -> Can't open replay file.") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = SaveReplay(rwops);
	SDL_RWclose(rwops);
	return flag;
}

int Game::SaveReplay(SDL_RWops *rwops) {
	if (match.SaveMatch(rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}	
	boxer1.skin.DefDiffuse(NO_CHANGE, NO_CHANGE, NO_CHANGE, 1.0);
	boxer2.skin.DefDiffuse(NO_CHANGE, NO_CHANGE, NO_CHANGE, 1.0);
	boxer1.hair.DefDiffuse(NO_CHANGE, NO_CHANGE, NO_CHANGE, 1.0);
	boxer2.hair.DefDiffuse(NO_CHANGE, NO_CHANGE, NO_CHANGE, 1.0);
	if (handler1.SaveBinBoxer(boxer1, motion1, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}	
	if (handler2.SaveBinBoxer(boxer2, motion2, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}      
	handler1.SaveReplay(match.LatestRound(), rwops);
	handler2.SaveReplay(match.LatestRound(), rwops);

	return SDL_SUCCESS;
}

int Game::LoadReplayFile(const char* filename, int round_data) {
 	int flag;
 	SDL_RWops *rwops;
	rwops = SDL_RWFromFile(filename, "rb");
 
	if (rwops == NULL) {
		emes("LoadReplayFile -> Can't open replay file.") ;
		efile(filename) ;		
		return SDL_FAILURE;
	}

	flag = LoadReplay(rwops, round_data);
	SDL_RWclose(rwops);
	return flag;
}

int Game::LoadReplay(SDL_RWops *rwops, int round_data) {
	if (match.LoadMatch(rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}	
	if (handler1.LoadBinBoxer(boxer1, motion1, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}	
	if (handler2.LoadBinBoxer(boxer2, motion2, rwops) == SDL_FAILURE) {
		return SDL_FAILURE;	
	}     

	handler1.Init(boxer1, motion1, match);
	handler2.Init(boxer2, motion2, match);

	if (round_data) {
		handler1.LoadReplay(match.LatestRound(), rwops);
		handler2.LoadReplay(match.LatestRound(), rwops);
	}

	return SDL_SUCCESS;
}	

void Game::SaveReplaySelect() {
	if (ChoiceSaveReplay) {
		int choice;
		choice = 0;
		do {
			switch (SaveReplayMenu()) {
				case 1 :
					choice = SaveSlotReplay();
				break;		
				case 2 :
					choice = 1;
				break;	
				default :
					SaveReplayCycle();
					choice = 1;
				break;
			}		
		} while (!choice);
	} else {
		SaveReplayCycle();	
	}
}

void Game::SaveReplayCycle() {
	static char replay_file[MAX_LINE_SIZE];
	if (++CurrentReplay > ReplayNumber) {
		CurrentReplay = 1;
	}
	sprintf(replay_file, "./Replay/cycle%03d.rep", CurrentReplay);
	SaveReplayFile(replay_file);
	ReplayConfigSave();
}

int Game::SaveSlotReplay() {
	int select_slot;
	select_slot = SelectSlotReplayMenu();
	switch (select_slot) {
		case - 1 :
			return 0;
		break;
		default :
			sprintf(SlotReplay[select_slot], "./Replay/slot%03d.rep", select_slot+1);
			SaveReplayFile(SlotReplay[select_slot]);
			ReplayConfigSave();
			return 1;
		break;
	}
}
