#include "game.h"

void Game::InitSelectBoxer() {
	MakeObjects(Partition);

	camera.Init(2);
	camera.Perspective(60.0, 0.05, 20.0);
	camera[0].DefPosition(0.5f, -0.5f, 1.5f);
	camera[0].DefDirection(2.5f, -2.5f, 1.0f);
	camera[0].DefRotation(0.0f);

	camera[1].DefPosition(-0.5f, 0.5f, 1.5f);
	camera[1].DefDirection(-2.5f, 2.5f, 1.0f);
	camera[1].DefRotation(0.0f);

	light.Init(0);
	light2.Init(1);
	light.DefDiffuse(0.8, 0.8, 0.8);
	light.DefPosition(0.0, 0.0, 1.0);
	light2.DefPosition(0.0, 0.0, 3.0);
	light2.DefDiffuse(0.8, 0.8, 0.8);
	light.On();
	light2.On();
}

void Game::ExposeSelectBoxer() {
	if (event.IsExpose() && ScreenMode) {
		ScreenReconfig();	
		InitSelectBoxer();
		event.Clear();
		wmes("ExposeSelectBoxer -> Change Resolution");
	} 
}

void Game::SetUpBoxer1() {
	handler1.SetInit(boxer1, motion1, match);
	boxer1.SetPoint(-2.0, 2.0, 225.0);
	motion1.SetRhythm();
	motion1.MotionRhythm().Set(&boxer1);
	boxer1.InitCenter();
}

void Game::SetUpBoxer2() {
	handler2.SetInit(boxer2, motion2, match);	
	boxer2.SetPoint(2.0, -2.0, 45.0);
	motion2.SetRhythm();
	motion2.MotionRhythm().Set(&boxer2);
	boxer2.InitCenter();
}


bool Game::SelectBoxer1File(int file_number) {
	if (file_number == -1) {
		handler1.LoadStoryBoxer(boxer1, motion1, RedManip);
		handler1.SetCustomName(UserName, 1);
		handler1.CleanManip();
	} else if (file_number == -2) {
		handler1.LoadStoryBoxer(boxer1, motion1, BlueManip);
		handler1.SetCustomName(UserName, 2);
		handler1.CleanManip();
	} else {
		strcpy(boxer1_file, GymMembers[file_number]);
		if (!LoadBoxer(handler1, boxer1, motion1, boxer1_file)) {
//			handler1.SaveBinBoxerFile(boxer1, motion1, "boxer1.bin");
//			handler1.LoadBinBoxerFile(boxer1, motion1, "boxer1.bin");
			return false;	
		}
	} 
	SetUpBoxer1();
	return true;
}

bool Game::SelectBoxer2File(int file_number) {
	if (file_number == -1) {
		handler2.LoadStoryBoxer(boxer2, motion2, RedManip);
		handler2.SetCustomName(UserName, 1);
		handler2.CleanManip();
	} else if (file_number == -2) {
		handler2.LoadStoryBoxer(boxer2, motion2, BlueManip);
		handler2.SetCustomName(UserName, 2);
		handler2.CleanManip();
	} else {
		strcpy(boxer2_file, GymMembers[file_number]);
		if (!LoadBoxer(handler2, boxer2, motion2, boxer2_file)) {
//			handler2.SaveBinBoxerFile(boxer2, motion2, "boxer2.bin");
//			handler2.LoadBinBoxerFile(boxer2, motion2, "boxer2.bin");
			return false;
		}
	}	
	SetUpBoxer2();		
	return true;
}

void Game::SetUpStoryBoxer1(StoryBoxer &sb) {
	handler1.SetStoryBoxer(boxer1, motion1, sb);
	SetUpBoxer1();
}

void Game::SetUpStoryBoxer2(StoryBoxer &sb) {
	handler2.SetStoryBoxer(boxer2, motion2, sb);
	SetUpBoxer2();
}

void Game::MouseSelectGoNext(int &mouse_select, int x, int y) {
	if (width * 0.00 < x && width * 0.07 > x) {
		if (height* 0.95 < y && height* 1.00 > y) {
			mouse_select = GO_NEXT;			
		}
	}
}

void Game::SelectMatchType(int match_type, StoryBoxer &sb) {
	int region;
	switch (match_type) {
		case WORLD_TITLE_MATCH :
		case AREA_TITLE_MATCH :
			match.Init(12, 60, 3, 0, 0, 0, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case WORLD_CHAMPIONSHIP :
		case AREA_CHAMPIONSHIP :
			match.Init(12, 60, 3, 0, 0, 1, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case NATION_TITLE_MATCH :
		case ROUND10_MATCH :	
			match.Init(10, 60, 3, 0, 0, 0, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case WORLD_RANKER_FINAL :
		case WORLD_RANKER_SEMIFINAL :
		case WORLD_RANKER_QUARTERFINAL :
		case AREA_RANKER_FINAL :
		case NATION_CHAMPIONSHIP :
			match.Init(10, 60, 3, 0, 0, 1, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case ROUND8_MATCH :	
			match.Init( 8, 60, 3, 0, 3, 0, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case AREA_RANKER_SEMIFINAL :
		case AREA_RANKER_QUARTERFINAL :
		case NATION_RANKER_FINAL :
			match.Init( 8, 60, 3, 0, 3, 1, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case ROUND6_MATCH :
			match.Init( 6, 60, 3, 0, 3, 0, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case NATION_RANKER_SEMIFINAL :
		case NATION_RANKER_QUARTERFINAL :
		case NATION_RANKER_1ST :
		case ROOKIE_KING_FINAL :
			match.Init( 6, 60, 3, 0, 3, 1, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case ROUND4_MATCH :
		case DEBUT_MATCH :
			match.Init( 4, 60, 2, 0, 3, 0, 10, 0, 0, 0, 0, ReplayRate);	
		break;
		case ROOKIE_KING_SEMIFINAL :
		case ROOKIE_KING_QUARTERFINAL :
		case ROOKIE_KING_1ST :
		case ROOKIE_KING_2ND :
		case ROOKIE_KING_3RD :	
			match.Init( 4, 60, 2, 0, 3, 1, 10, 0, 0, 0, 0, ReplayRate);	
		break;
	}

	switch (match_type) {
		case WORLD_TITLE_MATCH :
		case WORLD_CHAMPIONSHIP :
		case WORLD_RANKER_FINAL :
		case WORLD_RANKER_SEMIFINAL :
		case WORLD_RANKER_QUARTERFINAL :
			region = WORLD;
		break;

		case AREA_TITLE_MATCH :
		case AREA_CHAMPIONSHIP :
		case AREA_RANKER_FINAL :
		case AREA_RANKER_SEMIFINAL :
		case AREA_RANKER_QUARTERFINAL :
			region = sb.Area();
		break;
		
		case NATION_TITLE_MATCH :
		case NATION_CHAMPIONSHIP :
		case NATION_RANKER_FINAL :
		case NATION_RANKER_SEMIFINAL :
		case NATION_RANKER_QUARTERFINAL :
		case NATION_RANKER_1ST :
		case ROOKIE_KING_FINAL :
		case ROOKIE_KING_SEMIFINAL :
		case ROOKIE_KING_QUARTERFINAL :
		case ROOKIE_KING_3RD :	
		case ROOKIE_KING_2ND :
		case ROOKIE_KING_1ST :
		case ROUND8_MATCH :	
		case ROUND6_MATCH :
		case ROUND4_MATCH :
		case DEBUT_MATCH :
			region = sb.Nation();
		break;

		case ROUND10_MATCH :	
			region = OUT_OF_REGION;
		break;
	}
	match.SetMatchInfo(region, match_type);
}
/*
void Game::SelectStory(int select_type, StoryHandler &story) {
	static char buf[255];
	static int select_boxer1 = MAX_BOXER/2, select_boxer2 = 10;
	static int pre_select_boxer1 = select_boxer1;
	static int pre_select_boxer2 = select_boxer2;
	static int x, y, loop, mouse_select = -1;
	static int number1, number2, region1, region2, rank1, rank2;
	static bool red_select = true, blue_select = false;
	static float size, csize;
	 
	if (select_type == SELECT_STORY_RED) {
		number2= MY_BOXER;
	} else {
		story.SortRegion(WORLD);
		number2 = story.Sort(select_boxer2);
	}
	story.MaxColumn(number2);
	region2 = story.SelectRegion();
	rank2 = story.Boxer(number2).RegionRank();

	SelectMatchType(NATION_TITLE_MATCH, story.Boxer(number2));

	if (select_type == SELECT_STORY_BLUE) {
		number1 = MY_BOXER;
	} else {
		story.SortRegion(WORLD);
		number1 = story.Sort(select_boxer1);
	}			
	story.MaxColumn(number1);
	region1 = story.SelectRegion();
	rank1 = story.Boxer(number1).RegionRank();

	if (select_type == SELECT_STORY_BLUE) {
		if (story.Mode() == ORIGINAL_MODE) {
			if (handler1.LoadBinBoxerFile(boxer1, motion1, "original.sav") == SDL_FAILURE) {
        		LoadErrorMessage();
		        exit(-1);
			}
			SetUpBoxer1();
		} else {
			SetUpStoryBoxer1(story.Boxer(number1));		
		} 
		SetUpStoryBoxer2(story.Boxer(number2));		
	} else if (select_type == SELECT_STORY_RED){
		if (story.Mode() == ORIGINAL_MODE) {
			if (handler2.LoadBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
        		LoadErrorMessage();
		        exit(-1);
			}
			SetUpBoxer2();
    	} else {
			SetUpStoryBoxer2(story.Boxer(number2));				
		}
		SetUpStoryBoxer1(story.Boxer(number1));	
	} else {
		SetUpStoryBoxer1(story.Boxer(number1));	
		SetUpStoryBoxer2(story.Boxer(number2));	
	}		

	switch (select_type) {
		case SELECT_STORY_BLUE :
			blue_select = false;
			red_select = true;		
		break;				
		default :
			blue_select = true;
			red_select = false;		
		break;				
	}

	InitSelectBoxer();
	if (select_type == SELECT_STORY_BLUE) {
		if (story.Mode() == CUSTOM_MODE) {
			motion1.OffSet(boxer1);
		} 
		motion2.OffSet(boxer2);
	} else if (select_type == SELECT_STORY_RED){
		if (story.Mode() == CUSTOM_MODE) {
			motion2.OffSet(boxer2);
		}
		motion1.OffSet(boxer1);
	} else {
		motion1.OffSet(boxer1);
		motion2.OffSet(boxer2);
	}
	
	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);

		if (width * 0.00 < x && width * 0.07 > x && select_type != SELECT_STORY_RED) {
			if (height* 0.57 < y && height* 0.64 > y) {
				mouse_select = REDCORNER_DOWN;			
			}
			if (height* 0.50 < y && height* 0.57 > y) {
				mouse_select = REDCORNER_UP;			
			}
		}
		if (width * 0.50 < x && width * 0.57 > x && select_type != SELECT_STORY_BLUE) {
			if (height* 0.57 < y && height* 0.64 > y) {
				mouse_select = BLUECORNER_DOWN;			
			}
			if (height* 0.50 < y && height* 0.57 > y) {
				mouse_select = BLUECORNER_UP;			
			}
		}

		if (event.IsKeyDown(kb2_back)) {
			if (blue_select && select_type == SELECT_STORY_COM) {
				blue_select = false;
				red_select = true;				
			} 		
		}
		if (event.IsKeyDown(kb2_forward)) {
			if (red_select && select_type == SELECT_STORY_COM) {
				blue_select = true;
				red_select = false;				
			} 		
		}

		if ((mouse_select == BLUECORNER_UP && event.IsMouse1Down()) ||
			event.IsKeyDown(kb1_right) ||
			(blue_select && event.IsKeyDown(kb2_left))) {
			pre_select_boxer1 = select_boxer1;
			select_boxer1++;		
			if (select_boxer1 >= MAX_BOXER) {
				select_boxer1 = 0;			
			}
		}  
		if ((mouse_select == BLUECORNER_DOWN && event.IsMouse1Down()) ||
			event.IsKeyDown(kb1_left) ||
			(blue_select && event.IsKeyDown(kb2_right))) {
			pre_select_boxer1 = select_boxer1;
			select_boxer1--;		
			if (select_boxer1 < 0) {
				select_boxer1 = MAX_BOXER - 1;			
			}
		}  
		if ((red_select && event.IsKeyDown(kb2_left)) ||
			(mouse_select == REDCORNER_UP && event.IsMouse1Down())) {
			pre_select_boxer2 = select_boxer2;
			select_boxer2++;		
			if (select_boxer2 >= MAX_BOXER) {
				select_boxer2 = 0;			
			}
		}  
		if ((red_select && event.IsKeyDown(kb2_right)) ||
			(mouse_select == REDCORNER_DOWN && event.IsMouse1Down())) {
			pre_select_boxer2 = select_boxer2;
			select_boxer2--;		
			if (select_boxer2 < 0) {
				select_boxer2 = MAX_BOXER - 1;			
			}
		}  

		if (pre_select_boxer1 != select_boxer1) {
			if (select_type == SELECT_STORY_BLUE) {
				if (SelectBoxer1File(select_boxer1)) {
					boxer1.MakeDLists(Partition, FaceStyle);
					boxer1.MakeRanges(Partition);
					pre_select_boxer1 = select_boxer1;
				} else {
					select_boxer1 = pre_select_boxer1;				
					SelectBoxer1File(select_boxer1);				
				}			
			} else {
				story.SortRegion(WORLD);
				number1 = story.Sort(select_boxer1);
				story.MaxColumn(number1);
				region1 = story.SelectRegion();
				rank1 = story.Boxer(number1).RegionRank();

				SetUpStoryBoxer1(story.Boxer(number1));
				boxer1.MakeDLists(Partition, FaceStyle);
				boxer1.MakeRanges(Partition);
				motion1.OffSet(boxer1);
				pre_select_boxer1 = select_boxer1;
			}
		}
		if (pre_select_boxer2 != select_boxer2) {
			if (select_type != SELECT_STORY_RED) {
				story.SortRegion(WORLD);
				number2 = story.Sort(select_boxer2);
				story.MaxColumn(number2);
				region2 = story.SelectRegion();
				rank2 = story.Boxer(number2).RegionRank();

				SetUpStoryBoxer2(story.Boxer(number2));
				boxer2.MakeDLists(Partition, FaceStyle);
				boxer2.MakeRanges(Partition);
				motion2.OffSet(boxer2);
				pre_select_boxer2 = select_boxer2;			
			} else {
				if (SelectBoxer2File(select_boxer2)) {
					boxer2.MakeDLists(Partition, FaceStyle);
					boxer2.MakeRanges(Partition);
					pre_select_boxer2 = select_boxer2;
				} else {
					select_boxer2 = pre_select_boxer2;				
					SelectBoxer2File(select_boxer2);				
				}
			}
		}

		RedCornerBoxer();
		RedCornerInfo(-1, buf);
		BoxerRecordInfo(story.Boxer(number2), region2, rank2, buf);

		if (select_type != SELECT_STORY_RED) {
			ptext.MessageEnter();
			red_select ? size = csize : size = 1.0;
			ptext.SetSpecular(0.0, 0.0, 0.0);
			ptext.SelectColor(mouse_select == REDCORNER_UP);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
			ptext.SelectColor(mouse_select == REDCORNER_DOWN);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   			ptext.MessageLeave();
		}

		BlueCornerBoxer();
		BlueCornerInfo(-1, buf);		
		BoxerRecordInfo(story.Boxer(number1), region1, rank1, buf);

		if (select_type != SELECT_STORY_BLUE) {
			ptext.MessageEnter();
			blue_select ? size = csize : size = 1.0;
			ptext.SelectColor(mouse_select == BLUECORNER_UP);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
			ptext.SelectColor(mouse_select == BLUECORNER_DOWN);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   			ptext.MessageLeave();
		}
		
		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();		

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	
}
*/
bool Game::ShowMatchProfile(StoryHandler &story, int number1, int number2, int watch_mode) {
	static char buf[255];
	static int x, y, mouse_select = -1;
	static int region1, region2, rank1, rank2;
	static bool show_profile;

	StoryBoxer & sb1 = story.Boxer(number1);
	StoryBoxer & sb2 = story.Boxer(number2);
	 
	region2 = story.SelectRegion();
	rank2 = sb2.RegionRank();
	region1 = story.SelectRegion();
	rank1 = sb1.RegionRank();

	if (rank2 == OUT_OF_REGION) {
		story.MaxColumn(number2);
		region2 = story.SelectRegion();
		rank2 = sb2.RegionRank();
	}
	if (rank1 == OUT_OF_REGION) {
		story.MaxColumn(number1);
		region1 = story.SelectRegion();
		rank1 = sb1.RegionRank();
	}

	if (story.Mode() == ORIGINAL_MODE && number1 == MY_BOXER) {
		if (handler1.LoadBinBoxerFile(boxer1, motion1, "original.sav") == SDL_FAILURE) {
        	LoadErrorMessage();
			return false;
		}
		boxer1.SetDefaultFace();
		SetUpBoxer1();
		handler1.SetTraumRate(sb1.Traum());
	} else {
		SetUpStoryBoxer1(sb1);	
	}

	if (story.Mode() == ORIGINAL_MODE && number2 == MY_BOXER) {
		if (handler2.LoadBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
        	LoadErrorMessage();
			return false;
		}
		boxer2.SetDefaultFace();
		SetUpBoxer2();
		handler2.SetTraumRate(sb2.Traum());
	} else {
		SetUpStoryBoxer2(sb2);	
	}

	InitSelectBoxer();
	motion1.OffSet(boxer1);
	motion2.OffSet(boxer2);
	
	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);

		RedCornerBoxer();
		RedCornerInfo(-1, buf);
		BoxerRecordInfo(sb2, region2, rank2, buf);

		BlueCornerBoxer();
		BlueCornerInfo(-1, buf);		
		BoxerRecordInfo(sb1, region1, rank1, buf);

		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
		ShowMatchInfo(buf);
		if (match.ResultState() != NO_RESULT) {
			ShowMatchResult(buf);	
		}
		ttext.SelectColor(mouse_select == GO_NEXT);
		if (watch_mode) {
	   		ttext.Print("Watch", 0.25, -100, -100);
		} else {
	   		ttext.Print("Next", 0.25, -100, -100);		
		}
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();		

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next)|| 
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			show_profile = true;
			break;		
		}
		if (watch_mode) {
			if (event.IsKeyDown(kb2_right_offense) || event.IsMouse3Down()) {
				show_profile = false;
				break;
			}
		}

		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	
	return show_profile;
}

bool Game::SelectRecord(StoryHandler &story, int number) {
	static char buf[255];
	static int x, y, mouse_select = -1;
	static bool watch_match;

	StoryBoxer sb1, sb2;

	if (story.RecordCorner(number)) {	
		sb1 = story.RecordBoxer(number);	 
		sb2 = story.Boxer(MY_BOXER);	 
		if (story.Mode() == ORIGINAL_MODE) {
			if (handler2.LoadBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
        		LoadErrorMessage();
				return false;
			}
			boxer2.SetDefaultFace();
			SetUpBoxer2();
		} else {
			SetUpStoryBoxer2(sb2);	
		}
		SetUpStoryBoxer1(sb1);	
    } else {
		sb1 = story.Boxer(MY_BOXER);	 
		sb2 = story.RecordBoxer(number);	 
		if (story.Mode() == ORIGINAL_MODE) {
			if (handler1.LoadBinBoxerFile(boxer1, motion1, "original.sav") == SDL_FAILURE) {
        		LoadErrorMessage();
				return false;
			}
			boxer1.SetDefaultFace();
			SetUpBoxer1();
		} else {
			SetUpStoryBoxer1(sb1);	
		}
		SetUpStoryBoxer2(sb2);	
	}

	InitSelectBoxer();
	motion1.OffSet(boxer1);
	motion2.OffSet(boxer2);
	
	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);

		RedCornerBoxer();
		RedCornerInfo(-1, buf);
		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		ttext.Print(RegionStr(sb2.Nation()), 0.5, 0, -45);
		ttext.MessageLeave();
	
		BlueCornerBoxer();
		BlueCornerInfo(-1, buf);		
		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		ttext.Print(RegionStr(sb1.Nation()), 0.5, 0, -45);
		ttext.MessageLeave();
	
		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
		ShowMatchInfo(buf);
		if (match.ResultState() != NO_RESULT) {
			ShowMatchResult(buf);	
		}
		ttext.SelectColor(mouse_select == GO_NEXT);
	   	ttext.Print("Watch", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		sprintf(buf, "./Replay/cycle%03d.rep", story.RecordMatch(number).replay);
		ttext.Print(buf, 0.25, 0, -85);
		ttext.MessageLeave();		

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next)|| 
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			watch_match = true;
			break;		
		}
		if (event.IsKeyDown(kb2_right_offense) || event.IsMouse3Down()) {
			watch_match = false;
			break;
		}

		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	
	return watch_match;
}

void Game::MouseSelectCustom(int &mouse_select, int x, int y) {
	int i;
	if (width * 0.675 < x && width * 0.775 > x) {
		for (i = 0; i < 10; i++) {
			if (height* (0.10 + i * 0.075) < y &&
				height* (0.10 + (i+1) * 0.075) > y) {
				mouse_select = i*2;
			}
		}
	}
  	if (width * 0.90 < x && width * 1.00 > x) {
		for (i = 0; i < 10; i++) {
			if (height* (0.10 + i * 0.075) < y &&
				height* (0.10 + (i+1) * 0.075) > y) {
				mouse_select = i*2 + 1;
			}
		}
	}
}

void Game::CustomizeBoxer(StoryHandler &story) {
	static char buf[255];
	static int i, key_select = 0, key_pressed;
	static int x, y, loop, mouse_select = -1;
	static float size, csize;

	match.Init(4, 60, 2, 0, 3, 0, 10, 0, 0, 0, 1, ReplayRate);
	SetUpStoryBoxer1(story.Boxer(MY_BOXER));				
	SetUpStoryBoxer2(story.Boxer(MY_BOXER));				
	InitSelectBoxer();
  	motion2.OffSet(boxer2);

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		MouseSelectCustom(mouse_select, x, y);

		if (event.IsKeyDown(kb2_left)) {
			key_select--;
			if (key_select < 0) {
			 	key_select = 0;
			}
		}
		if (event.IsKeyDown(kb2_right)) {
			key_select++;
			if (key_select > 9) {
			 	key_select = 9;
			}
		}

		key_pressed = -1;
		if (event.IsKeyDown(kb2_back)) {
			key_pressed = key_select*2;
		}
		if (event.IsKeyDown(kb2_forward)) {
			key_pressed = key_select*2 + 1;
		}
		if ((mouse_select >= 0 && mouse_select < 20)
			&& event.IsMouse1Down()) {
			key_pressed = mouse_select;
		}

		if (key_pressed >= 0) {
			switch (key_pressed) {
				case 0 :
					story.Boxer(MY_BOXER).HeightM();
				break;									
				case 1:
					story.Boxer(MY_BOXER).HeightP();
				break;									
				case 2:
					story.Boxer(MY_BOXER).ReachM();
				break;									
				case 3:
					story.Boxer(MY_BOXER).ReachP();
				break;									
				case 4:
					story.Boxer(MY_BOXER).StyleM();
				break;									
				case 5:
					story.Boxer(MY_BOXER).StyleP();
				break;									
				case 6:
					story.Boxer(MY_BOXER).StanceM();
				break;									
				case 7:
					story.Boxer(MY_BOXER).StanceP();
				break;									
				case 8:
					story.Boxer(MY_BOXER).CombM();
				break;									
				case 9:
					story.Boxer(MY_BOXER).CombP();
				break;									
				case 10:
					story.Boxer(MY_BOXER).ThoughtM();
				break;									
				case 11:
					story.Boxer(MY_BOXER).ThoughtP();
				break;									
				case 12:
					story.Boxer(MY_BOXER).GlovesM();
				break;									
				case 13:
					story.Boxer(MY_BOXER).GlovesP();
				break;									
				case 14:
					story.Boxer(MY_BOXER).TrunksM();
				break;									
				case 15:
					story.Boxer(MY_BOXER).TrunksP();
				break;									
				case 16:
					story.Boxer(MY_BOXER).SkinM();
				break;									
				case 17:
					story.Boxer(MY_BOXER).SkinP();
				break;									
				case 18:
					story.Boxer(MY_BOXER).NationM();
				break;									
				case 19:
					story.Boxer(MY_BOXER).NationP();
				break;									
			}
			if (story.Boxer(MY_BOXER).Nation() == JAPAN) {
				story.Boxer(MY_BOXER).SetDefaultFace();	 
    		}
			SetUpStoryBoxer2(story.Boxer(MY_BOXER));
			boxer2.MakeDLists(Partition, FaceStyle);
			boxer2.MakeRanges(Partition);
			motion2.OffSet(boxer2);
		}

		RedCornerBoxer();
		RedCornerInfo(-1, buf);

		glViewport(0, 0, width, height);
		SelectOptions();

		if (UsePText) {
			ptext.MessageEnter();
			for (i = 0; i < 10; i++) {
				key_select == i ? size = csize : size = 1.0;
				ptext.SetSpecular(0.0, 0.0, 0.0);
				ptext.SelectColor(mouse_select == i*2);
				ptext.Symbol('<', 0.4*size, 45, 80 - i*16);
				ptext.SelectColor(mouse_select == i*2 + 1);
				ptext.Symbol('>', 0.4*size, 95, 80 - i*16);
   			}
			ptext.MessageLeave();
		} else {
			ttext.MessageEnter();
			for (i = 0; i < 10; i++) {
				key_select == i ? size = csize : size = 1.0;
				ttext.SelectColor(mouse_select == i*2);
				ttext.Symbol('<', 0.4*size, 45, 80 - i*16);
				ttext.SelectColor(mouse_select == i*2 + 1);
				ttext.Symbol('>', 0.4*size, 95, 80 - i*16);
   			}
			ttext.MessageLeave();		
		}

		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		for (i = 0; i < 10; i++) {
			key_select == i ? size = csize : size = 1.0;
			SelectCustomStr(i, buf);
			ttext.Print(buf, 0.3*size, 25, 80 - i*16);
   		}

		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Height());
		ttext.Print(buf, 0.3, 86, 80);
		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Reach());
		ttext.Print(buf, 0.3, 86, 80 - 16);
		CustomStyleStr(story.Boxer(MY_BOXER).Style(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 2 * 16);
		CustomStanceStr(story.Boxer(MY_BOXER).Stance(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 3 * 16);
		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Comb());
		ttext.Print(buf, 0.3, 86, 80 - 4 * 16);
		CustomThoughtStr(story.Boxer(MY_BOXER).Thought(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 5 * 16);

		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Gloves());
		ttext.Print(buf, 0.3, 86, 80 - 6 * 16);
		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Trunks());
		ttext.Print(buf, 0.3, 86, 80 - 7 * 16);
		sprintf(buf, "%2d", story.Boxer(MY_BOXER).Skin());
		ttext.Print(buf, 0.3, 86, 80 - 8 * 16);
		CustomNationStr(story.Boxer(MY_BOXER).Nation(), buf);
		switch (lang) {
			case Kana:
				ttext.Print(buf, 0.18, 86, 80 - 9 * 16 + 2);
			break;
			case English:
				ttext.Print(buf, 0.25, 86, 80 - 9 * 16 + 1);
			break;
			default:
				ttext.Print(buf, 0.25, 86, 80 - 9 * 16);
			break;
		}

		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();				

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	story.Boxer(MY_BOXER).SetArea();
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	
}

void Game::CustomizeSparBoxer(int number) {
	static char buf[255];
	static int i, key_select = 0, key_pressed;
	static int x, y, loop, mouse_select = -1, max_select, max_select2;
	static float size, csize;

	StoryBoxer sb;
	if (number < 100) {
		number = 1;
		max_select = 9;
		sb = RedManip;
	} else {
		number = 100;
		max_select = 8;
		sb.Init();
	}
	max_select2 = (max_select + 1) * 2;

	match.Init(4, 60, 2, 0, 3, 0, 10, 0, 0, 0, 1, ReplayRate);
	SetUpStoryBoxer1(sb);				
	SetUpStoryBoxer2(sb);				
	InitSelectBoxer();
	handler2.SetCustomName(UserName, number);
  	motion2.OffSet(boxer2);

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		MouseSelectCustom(mouse_select, x, y);

		if (event.IsKeyDown(kb2_left)) {
			key_select--;
			if (key_select < 0) {
			 	key_select = 0;
			}
		}
		if (event.IsKeyDown(kb2_right)) {
			key_select++;
			if (key_select > max_select) {
			 	key_select = max_select;
			}
		}

		key_pressed = -1;
		if (event.IsKeyDown(kb2_back)) {
			key_pressed = key_select*2;
		}
		if (event.IsKeyDown(kb2_forward)) {
			key_pressed = key_select*2 + 1;
		}
		if ((mouse_select >= 0 && mouse_select < max_select2)
			&& event.IsMouse1Down()) {
			key_pressed = mouse_select;
		}

		if (key_pressed >= 0) {
			switch (key_pressed) {
				case 0 :
					sb.HeightM();
				break;									
				case 1:
					if ((sb.Height() + sb.Reach()) < 8 || number < 100) {
						sb.HeightP();
					}
				break;									
				case 2:
					sb.ReachM();
				break;									
				case 3:
					if ((sb.Height() + sb.Reach()) < 8 || number < 100) {
						sb.ReachP();
					}
				break;									
				case 4:
					sb.StyleM();
				break;									
				case 5:
					sb.StyleP();
				break;									
				case 6:
					sb.StanceM();
				break;									
				case 7:
					sb.StanceP();
				break;									
				case 8:
					sb.CombM();
				break;									
				case 9:
					sb.CombP();
				break;									
				case 10:
					sb.ThoughtM();
				break;									
				case 11:
					sb.ThoughtP();
				break;									
				case 12:
					sb.GlovesM();
				break;									
				case 13:
					sb.GlovesP();
				break;									
				case 14:
					sb.TrunksM();
				break;									
				case 15:
					sb.TrunksP();
				break;									
				case 16:
					sb.SkinM();
				break;									
				case 17:
					sb.SkinP();
				break;									
				case 18:
					if (number == 2) {
						number = 1;
						BlueManip = sb;
						sb = RedManip;
					}
				break;									
				case 19:
					if (number == 1) {
						number = 2;
						RedManip = sb;
						sb = BlueManip;
					}
				break;									
			}
			SetUpStoryBoxer2(sb);
			handler2.SetCustomName(UserName, number);
			boxer2.MakeDLists(Partition, FaceStyle);
			boxer2.MakeRanges(Partition);
			motion2.OffSet(boxer2);
		}

		RedCornerBoxer();
		RedCornerInfo(-1, buf);

		glViewport(0, 0, width, height);
		SelectOptions();

		if (UsePText) {
			ptext.MessageEnter();
			for (i = 0; i < max_select+1; i++) {
				key_select == i ? size = csize : size = 1.0;
				ptext.SetSpecular(0.0, 0.0, 0.0);
				ptext.SelectColor(mouse_select == i*2);
				ptext.Symbol('<', 0.4*size, 45, 80 - i*16);
				ptext.SelectColor(mouse_select == i*2 + 1);
				ptext.Symbol('>', 0.4*size, 95, 80 - i*16);
   			}
			ptext.MessageLeave();
		} else {
			ttext.MessageEnter();
			for (i = 0; i < max_select+1; i++) {
				key_select == i ? size = csize : size = 1.0;
				ttext.SelectColor(mouse_select == i*2);
				ttext.Symbol('<', 0.4*size, 45, 80 - i*16);
				ttext.SelectColor(mouse_select == i*2 + 1);
				ttext.Symbol('>', 0.4*size, 95, 80 - i*16);
   			}
			ttext.MessageLeave();		
		}

		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		for (i = 0; i < 9; i++) {
			key_select == i ? size = csize : size = 1.0;
			SelectCustomStr(i, buf);
			ttext.Print(buf, 0.3*size, 25, 80 - i*16);
   		}
		if (number < 100) {
			key_select == 9 ? size = csize : size = 1.0;
			sprintf(buf, "No.%d", number);
			ttext.Print(buf, 0.3*size, 25, 80 - i*16);
		}

		sprintf(buf, "%2d", sb.Height());
		ttext.Print(buf, 0.3, 86, 80);
		sprintf(buf, "%2d", sb.Reach());
		ttext.Print(buf, 0.3, 86, 80 - 16);
		CustomStyleStr(sb.Style(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 2 * 16);
		CustomStanceStr(sb.Stance(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 3 * 16);
		sprintf(buf, "%2d", sb.Comb());
		ttext.Print(buf, 0.3, 86, 80 - 4 * 16);
		CustomThoughtStr(sb.Thought(), buf);
		ttext.Print(buf, 0.25, 86, 80 - 5 * 16);

		sprintf(buf, "%2d", sb.Gloves());
		ttext.Print(buf, 0.3, 86, 80 - 6 * 16);
		sprintf(buf, "%2d", sb.Trunks());
		ttext.Print(buf, 0.3, 86, 80 - 7 * 16);
		sprintf(buf, "%2d", sb.Skin());
		ttext.Print(buf, 0.3, 86, 80 - 8 * 16);

		ttext.SelectColor(mouse_select == GO_NEXT);
		if (number < 100) { 
	   		ttext.Print("Back", 0.25, -100, -100);
		} else {
	 		ttext.Print("Next", 0.25, -100, -100);	
		}
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();				

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(event.IsMouse3Down() && number < 100) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
	if (number == 1) {
		RedManip = sb;
	} else if (number == 2) {
		BlueManip = sb;	
	} else {
		BoxerOM = sb;	
	}
	DataConfigSave();
}


void Game::ShowRecentResults(StoryBoxer& sb, char* buf) {
	ttext.MessageEnter();
	ttext.SetColor(1.0, 1.0, 1.0);	

	SelectCustomStr(2, buf);
	ttext.Print(buf, 0.6, -90, 90);
	SelectCustomStr(3, buf);
	ttext.Print(buf, 0.6, -90, 65);
	SelectCustomStr(5, buf);
	ttext.Print(buf, 0.6, -90, 40);
  
  	CustomStyleStr(sb.Style(), buf);
	ttext.Print(buf, 0.5, 90, 90);
	CustomStanceStr(sb.Stance(), buf);
	ttext.Print(buf, 0.5, 90, 65);
	CustomThoughtStr(sb.Thought(), buf);
	ttext.Print(buf, 0.5, 90, 40);   

	switch (lang) {
		case Kana :
			strcpy(buf, "̂");				
		break;
		case English :
			strcpy(buf, "Recent Results");				
		break;
		case Japanese :		
			strcpy(buf, "ŋ߂̌");				
		break;		
	}
	ttext.Print(buf, 0.6, 0, 10);   
 	RecentStr(sb, buf);
	ttext.MessageLeave();				

	if (UsePText) {
		ptext.MessageEnter();				
		ptext.SetColor(1.0, 1.0, 1.0);
		glPushMatrix();
		glScalef(1.0, 0.5, 0.5);
 		glTranslatef(-2.7, -1.0, 0.0);
 		ptext.Text(buf);
  		glPopMatrix();
		ptext.MessageLeave();
	} else {
		ttext.MessageEnter();				
		ttext.SetColor(1.0, 1.0, 1.0);
		glPushMatrix();
		glScalef(0.5, 0.5, 0.5);
 		glTranslatef(-5.4, -1.0, 0.0);
 		ttext.TextPitch(buf, 1.0);
  		glPopMatrix();
		ttext.MessageLeave();	
	}
}				

void Game::ShowHeldTitles(StoryBoxer &sb, char* buf) {
	int i;
	float size;
	ttext.MessageEnter();
	switch (lang) {
		case Kana :
			strcpy(buf, "Ƃ^Cg");				
			size = 0.8;
		break;
		case English :
			strcpy(buf, "Titles");				
			size = 1.0;
		break;
		case Japanese :		
			strcpy(buf, "l^Cg");				
			size = 1.0;
		break;		
	}
	ttext.Print(buf, 0.6, 0, 100);   	
	i = 1;
	if (WorldTitleStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (AreaTitleStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (NationTitleStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (WorldRankerStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (AreaRankerStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (NationRankerStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i++ * 25);
	}
	if (RookieKingStr(sb, buf)) {
		ttext.Print(buf, 0.5*size, 0, 95 - i * 25);
	}
    ttext.MessageLeave();
}				

int Game::ShowChallengers(StoryHandler &story, int *challengers, int max_select) {
	static char buf[255];
	static int loop, region, rank;
	static int pre_blue_select, blue_select; 
	static int x, y, mouse_select = -1;
	static float csize;

	blue_select = 0;
	pre_blue_select = blue_select;
	StoryBoxer sb = story.SortBoxer(challengers[blue_select]);
	region = story.SelectRegion();
	rank = sb.RegionRank();
		 
	match.Init(4, 60, 2, 0, 3, 0, 10, 0, 0, 0, 1, ReplayRate);

	SetUpStoryBoxer2(sb);	
	SetUpStoryBoxer1(sb);				

	InitSelectBoxer();
	motion2.OffSet(boxer2);

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);

		if (width * 0.50 < x && width * 0.57 > x) {
			if (height* 0.57 < y && height* 0.64 > y) {
				mouse_select = BLUECORNER_DOWN;			
			}
			if (height* 0.50 < y && height* 0.57 > y) {
				mouse_select = BLUECORNER_UP;			
			}
		}

		if (event.IsKeyDown(kb2_left) || 
			(mouse_select == BLUECORNER_UP && event.IsMouse1Down())) {
			if (--blue_select < 0) {
				blue_select = max_select;			
			}
		} 
		if (event.IsKeyDown(kb2_right) ||
			(mouse_select == BLUECORNER_DOWN && event.IsMouse1Down())){
			if (++blue_select > max_select) {
				blue_select = 0;
			}
		}     

		if (pre_blue_select != blue_select) {
			sb = story.SortBoxer(challengers[blue_select]);
			rank = sb.RegionRank();
			region = story.SelectRegion();

			SetUpStoryBoxer1(sb);
			boxer1.MakeDLists(Partition, FaceStyle);
			boxer1.MakeRanges(Partition);
			motion1.OffSet(boxer1);
			pre_blue_select = blue_select;
		}

		BlueCornerBoxer();
		BlueCornerInfo(-1, buf);
 		BoxerRecordInfo(sb, region, rank, buf);

		if (UsePText) {
			ptext.MessageEnter();
			ptext.SetSpecular(0.0, 0.0, 0.0);
			ptext.SetShininess(0.0);
			ptext.SetColor(1.0, 1.0, 1.0);
			ptext.SelectColor(mouse_select == BLUECORNER_UP);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*csize, -92, 95);
			ptext.SelectColor(mouse_select == BLUECORNER_DOWN);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*csize, -92, 70);
   			ptext.MessageLeave();
		} else {
			ttext.MessageEnter();
			ttext.SetColor(1.0, 1.0, 1.0);
			ttext.SelectColor(mouse_select == BLUECORNER_UP);
			ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*csize, -92, 95);
			ttext.SelectColor(mouse_select == BLUECORNER_DOWN);
			ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*csize, -92, 70);
   			ttext.MessageLeave();		
		}

		glViewport(0, height/2, width/2, height/2);
		ShowRecentResults(sb, buf);

		glViewport(0, height/6, width/2, height/2);
		ShowHeldTitles(sb, buf);

		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();
	
	return challengers[blue_select];
}

void Game::ShowBoxerProfile(StoryHandler &story, int boxer_number) {
	static char buf[255];
	static int region, rank;
	static int x, y, mouse_select = -1;
	
	StoryBoxer &sb = story.Boxer(boxer_number);
	region = story.SelectRegion();
	rank = sb.RegionRank();
		 
	match.Init(4, 60, 2, 0, 3, 0, 10, 0, 0, 0, 1, ReplayRate);

	if (story.Mode() == ORIGINAL_MODE && boxer_number == MY_BOXER) {
		if (handler2.LoadBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
        	LoadErrorMessage();
			return;
		}
		boxer2.SetDefaultFace();
		SetUpBoxer2();
	} else {
		SetUpStoryBoxer2(sb);	
	}
	SetUpStoryBoxer1(sb);				

	InitSelectBoxer();
	motion2.OffSet(boxer2);

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
//		MouseSelectGoNext(mouse_select, x, y);

		RedCornerBoxer();
		RedCornerInfo(-1, buf);
 		BoxerRecordInfo(sb, region, rank, buf);

		glViewport(width/2, height/2, width/2, height/2);
		ShowRecentResults(sb, buf);

		glViewport(width/2, height/6, width/2, height/2);
		ShowHeldTitles(sb, buf);

		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
//		ttext.SelectColor(mouse_select == GO_NEXT);
//		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (IsAnyKeyDown() || event.IsMouse1Down() || event.IsMouse3Down()) {
			break;		
		}
		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	
}

bool Game::SelectBoxer(int select_type) {
	static char buf[255];
	static int select_boxer1 = 1, select_boxer2 = 0;
	static int pre_select_boxer1 = select_boxer1;
	static int pre_select_boxer2 = select_boxer2;
	static int x, y, loop, mouse_select = -1;
	static bool select_ok, red_select = true, blue_select = false;
	static float size, csize;
	 
	match.Init(MaxRound, RoundTime,
		TKOSystem, JudgeSystem, CountSystem, ExtraRound, KOCount,
		Judge1Type, Judge2Type, Judge3Type, 
		ExtraInfo, ReplayRate);
	if (!SelectBoxer1File(select_boxer1) ||
		!SelectBoxer2File(select_boxer2)) {
		return false;
	}

	switch (select_type) {
		case SELECT_1PSPARRING:
		case SELECT_TRAINING:
			blue_select = false;
			red_select = true;
		break;
		case SELECT_2PMATCH:
			blue_select = true;
			red_select = true;
		break;				
	}

	InitSelectBoxer();
	if (select_boxer1 < 0) {
		motion1.OffSet(boxer1);
	}
	if (select_boxer2 < 0) {
		motion2.OffSet(boxer2);
	}	

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		if (width * 0.00 < x && width * 0.07 > x) {
			if (height* 0.57 < y && height* 0.64 > y) {
				mouse_select = REDCORNER_DOWN;			
			}
			if (height* 0.50 < y && height* 0.57 > y) {
				mouse_select = REDCORNER_UP;			
			}
		}
		if (width * 0.50 < x && width * 0.57 > x) {
			if (height* 0.57 < y && height* 0.64 > y) {
				mouse_select = BLUECORNER_DOWN;			
			}
			if (height* 0.50 < y && height* 0.57 > y) {
				mouse_select = BLUECORNER_UP;			
			}
		}

		if (event.IsKeyDown(kb2_back)) {
			if (select_type == SELECT_1PSPARRING && blue_select) {
				blue_select = false;
				red_select = true;				
			} 		
		}
		if (event.IsKeyDown(kb2_forward)) {
			if (select_type == SELECT_1PSPARRING && red_select) {
				blue_select = true;
				red_select = false;				
			} 		
		}

		if ((mouse_select == BLUECORNER_UP && event.IsMouse1Down()) ||
			event.IsKeyDown(kb1_right) ||
			(select_type == SELECT_1PSPARRING && blue_select && event.IsKeyDown(kb2_left))) {
			pre_select_boxer1 = select_boxer1;
			select_boxer1++;		
			if (select_boxer1 >= MembersNumber) {
				select_boxer1 = -2;			
			}
		}  
		if ((mouse_select == BLUECORNER_DOWN && event.IsMouse1Down()) ||
			event.IsKeyDown(kb1_left) ||
			(select_type == SELECT_1PSPARRING && blue_select && event.IsKeyDown(kb2_right))) {
			pre_select_boxer1 = select_boxer1;
			select_boxer1--;		
			if (select_boxer1 < -2) {
				select_boxer1 = MembersNumber - 1;			
			}
		}  
		if ((red_select && event.IsKeyDown(kb2_left)) ||
			(mouse_select == REDCORNER_UP && event.IsMouse1Down())) {
			pre_select_boxer2 = select_boxer2;
			select_boxer2++;		
			if (select_boxer2 >= MembersNumber) {
				select_boxer2 = -2;			
			}
		}  
		if ((red_select && event.IsKeyDown(kb2_right)) ||
			(mouse_select == REDCORNER_DOWN && event.IsMouse1Down())) {
			pre_select_boxer2 = select_boxer2;
			select_boxer2--;		
			if (select_boxer2 < -2) {
				select_boxer2 = MembersNumber - 1;			
			}
		}  

		if (pre_select_boxer1 != select_boxer1) {
			if (SelectBoxer1File(select_boxer1)) {
				boxer1.MakeDLists(Partition, FaceStyle);
				boxer1.MakeRanges(Partition);
				if (select_boxer1 < 0) {
					motion1.OffSet(boxer1);
				}
				pre_select_boxer1 = select_boxer1;
			} else {
				select_boxer1 = pre_select_boxer1;				
				SelectBoxer1File(select_boxer1);				
			}
		}
		if (pre_select_boxer2 != select_boxer2) {
			if (SelectBoxer2File(select_boxer2)) {
				boxer2.MakeDLists(Partition, FaceStyle);
				boxer2.MakeRanges(Partition);
				if (select_boxer2 < 0) {
					motion2.OffSet(boxer2);
				}
				pre_select_boxer2 = select_boxer2;
			} else {
				select_boxer2 = pre_select_boxer2;				
				SelectBoxer2File(select_boxer2);				
			}
		}

		RedCornerBoxer();
		RedCornerInfo(select_boxer2, buf);

		if (UsePText) {
			ptext.MessageEnter();
			red_select ? size = csize :size = 1.0;
			ptext.SetSpecular(0.0, 0.0, 0.0);
			ptext.SelectColor(mouse_select == REDCORNER_UP);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
			ptext.SelectColor(mouse_select == REDCORNER_DOWN);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   			ptext.MessageLeave();
		} else {
			ttext.MessageEnter();
			red_select ? size = csize :size = 1.0;
			ttext.SelectColor(mouse_select == REDCORNER_UP);
			ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
			ttext.SelectColor(mouse_select == REDCORNER_DOWN);
			ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   			ttext.MessageLeave();		
		}

		if (select_type != SELECT_TRAINING) {
    		BlueCornerBoxer();
			BlueCornerInfo(select_boxer1, buf);
			
			if (UsePText) {
				ptext.MessageEnter();
				blue_select ? size = csize : size = 1.0;
				ptext.SelectColor(mouse_select == BLUECORNER_UP);
				ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
				ptext.SelectColor(mouse_select == BLUECORNER_DOWN);
				ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   				ptext.MessageLeave();
			} else {
				ttext.MessageEnter();
				blue_select ? size = csize : size = 1.0;
				ttext.SelectColor(mouse_select == BLUECORNER_UP);
				ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.8*size, -92, 95);
				ttext.SelectColor(mouse_select == BLUECORNER_DOWN);
				ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.8*size, -92, 70);
   				ttext.MessageLeave();			
			}
		}
		
		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();
		if (select_type != SELECT_TRAINING) {
			ShowMatchInfo(buf);
		}
		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();		

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			select_ok = true;
			break;
		}
		if (event.IsKeyDown(kb2_right_offense) || event.IsMouse3Down()) {
			select_ok = false;
			break;
		}

		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	

	if (RedManipOn) {
		handler2.SetBoxerManip(RedManip);	
	}
	if (BlueManipOn) {
		handler1.SetBoxerManip(BlueManip);		
	}
	return select_ok;
}

void Game::MouseSelectReplay(int &mouse_select, int x, int y) {
	if (height* 0.94 < y && height* 1.00 > y) {
		if (width * 0.30 < x && width * 0.45 > x) {
			mouse_select = REPLAY_LEFT;			
		}
		if (width * 0.55 < x && width * 0.70 > x) {
			mouse_select = REPLAY_RIGHT;			
		}
	}
	if (width * 0.07 < x && width * 0.15 > x) {
		if (height* 0.95 < y && height* 1.00 > y) {
			mouse_select = REPLAY_DOWN;			
		}
		if (height* 0.90 < y && height* 0.95 > y) {
			mouse_select = REPLAY_UP;			
		}
	}
}

bool Game::SelectReplay(int replay_type, char* filename) {
	static char buf[255], replay_file[255];
	static int i, select_replay1, select_replay2 = 0;
	static int pre_select_replay1, pre_select_replay2 = select_replay2;
	static int x, y, loop, mouse_select = -1;
	static int pre_replay_select, replay_select;
	static bool replay_ok;
	static float size, csize;

	replay_select = NO_REPLAY;
	if (replay_type == SELECT_REPLAY_ARGUMENT) {
		strcpy(replay_file, filename);
		if (LoadReplayFile(replay_file, 0) == SDL_SUCCESS) {
		    replay_select = SELECT_REPLAY_ARGUMENT;
		} else {
		 	return false;
		}
	}

	for (i = 0; i < ReplayNumber && replay_select == NO_REPLAY; i++) {
		select_replay1 = CurrentReplay + i;
		if (select_replay1 > ReplayNumber) {
			select_replay1-= ReplayNumber;
		}
		sprintf(replay_file, "./Replay/cycle%03d.rep", select_replay1);
		if (LoadReplayFile(replay_file, 0) == SDL_SUCCESS) {
			replay_select = CYCLE_REPLAY;
			pre_select_replay1 = select_replay1;
			break;
		}
	}
	if (replay_select == NO_REPLAY) {
		for (i = 0; i < SlotReplayNumber; i++) {
			select_replay2 = i;
			strcpy(replay_file, SlotReplay[select_replay2]);		
			if (LoadReplayFile(replay_file, 0) == SDL_SUCCESS) {
				pre_select_replay2 = select_replay2;
				break;
			}
		}
	}
	if (replay_select == NO_REPLAY) {
		return false;	
	} else {			
		pre_replay_select = replay_select;
	}
	
	InitSelectBoxer();
	SetUpBoxer1();
	SetUpBoxer2();
	boxer1.SetDefaultFace();
	boxer2.SetDefaultFace();

	timer.SetFPS(50);	
	do {
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		csize =	(0.9 + 0.1 * sin(RAD(5*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		MouseSelectReplay(mouse_select, x, y);

		if ((mouse_select == REPLAY_LEFT && event.IsMouse1Down()) ||
			event.IsKeyDown(kb2_back)) {
			if (replay_select == SLOT_REPLAY) {
     			pre_replay_select = replay_select;
     			replay_select = CYCLE_REPLAY;
			} 		
		}
		if ((mouse_select == REPLAY_RIGHT && event.IsMouse1Down()) || 
			event.IsKeyDown(kb2_forward)) {
			if (replay_select == CYCLE_REPLAY) {
     			pre_replay_select = replay_select;
     			replay_select = SLOT_REPLAY;
			} 		
		}

		if ((mouse_select == REPLAY_UP && event.IsMouse1Down()) ||
			event.IsKeyDown(kb2_left)) {
			if (replay_select == CYCLE_REPLAY) {
				pre_select_replay1 = select_replay1;			
        		select_replay1++;
				if (select_replay1 > ReplayNumber) {
					select_replay1 = 1;				
				}
			} else {
				pre_select_replay2 = select_replay2;			
	     		select_replay2++;					
				if (select_replay2 > SlotReplayNumber - 1) {
					select_replay2 = 0;				
				}
			} 		
		}  
		if ((mouse_select == REPLAY_DOWN && event.IsMouse1Down()) ||
			event.IsKeyDown(kb2_right)) {					
			if (replay_select == CYCLE_REPLAY) {
				pre_select_replay1 = select_replay1;			
	     		select_replay1--;
				if (select_replay1 < 1) {
					select_replay1 = ReplayNumber;				
				}
			} else {
				pre_select_replay2 = select_replay2;			
	     		select_replay2--;			
				if (select_replay2 < 0) {
					select_replay2 = SlotReplayNumber-1;				
				}
			} 		
		}  

		if ((replay_type == SELECT_REPLAY_MENU) &&
			(pre_select_replay1 != select_replay1 ||
			pre_select_replay2 != select_replay2 ||
			pre_replay_select != replay_select)) {
			if (replay_select == CYCLE_REPLAY) {
				sprintf(buf, "./Replay/cycle%03d.rep", select_replay1);
			} else {
				strcpy(buf, SlotReplay[select_replay2]);		
			}
			
			if (LoadReplayFile(buf, 0) == SDL_SUCCESS) {
				pre_select_replay1 = select_replay1;
				pre_select_replay2 = select_replay2;
				pre_replay_select = replay_select;
				sprintf(replay_file, buf);			
				boxer1.MakeDLists(Partition, FaceStyle);
				boxer1.MakeRanges(Partition);
				boxer2.MakeDLists(Partition, FaceStyle);
				boxer2.MakeRanges(Partition);
			} else {
				select_replay1 = pre_select_replay1;
				select_replay2 = pre_select_replay2;
				replay_select = pre_replay_select;
				LoadReplayFile(replay_file, 0);	
			}
		}

		RedCornerBoxer();
		RedCornerInfo(-1, buf);

		BlueCornerBoxer();
		BlueCornerInfo(-1, buf);
			
		glViewport(0, 0, width, height);
		SelectOptions();
		ttext.MessageEnter();

		if (replay_type == SELECT_REPLAY_MENU) {
			if (lang == English) {
				ttext.SelectColor(mouse_select == REPLAY_LEFT);
				(replay_select == CYCLE_REPLAY)? size = csize : size = 1.0;
				ttext.Print("Cycle", 0.25*size, -25, -96);
				ttext.SelectColor(mouse_select == REPLAY_RIGHT);
				(replay_select == SLOT_REPLAY) ? size = csize : size = 1.0;
				ttext.Print("Slot", 0.25*size, 25, -96);		
			} else {
				ttext.SelectColor(mouse_select == REPLAY_LEFT);
				(replay_select == CYCLE_REPLAY)? size = csize : size = 1.0;
				ttext.Print("TCN", 0.25*size, -25, -96);
				ttext.SelectColor(mouse_select == REPLAY_RIGHT);
				(replay_select == SLOT_REPLAY) ? size = csize : size = 1.0;
				ttext.Print("Xbg", 0.25*size, 25, -96);
			}
		}

		ttext.SetColor(1.0, 1.0, 1.0);	
		ShowMatchInfo(buf);
		ShowMatchResult(buf);
		ttext.Print(replay_file, 0.25, 0, -85);
		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();		

		if (replay_type == SELECT_REPLAY_MENU) {
			if (UsePText) {
				ptext.MessageEnter();
				ptext.SetSpecular(0.0, 0.0, 0.0);
				ptext.SelectColor(mouse_select == REPLAY_UP);
				ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.4*csize, -80, -87);
				ptext.SelectColor(mouse_select == REPLAY_DOWN);
				ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.4*csize, -80, -100);
 	  			ptext.MessageLeave();
			} else {
				ttext.MessageEnter();
				ttext.SelectColor(mouse_select == REPLAY_UP);
				ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.4*csize, -80, -87);
				ttext.SelectColor(mouse_select == REPLAY_DOWN);
				ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.4*csize, -80, -100);
 	  			ttext.MessageLeave();			
			}
		}

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(go_next) ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			replay_ok = true;
			break;		
		}
		if (event.IsKeyDown(kb2_right_offense) || event.IsMouse3Down()) {
			replay_ok = false;
			break;
		}

		ExposeSelectBoxer();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	

	LoadReplayFile(replay_file, 1);

	return replay_ok;
}

void Game::ShowMatchInfo(char *buf) {
	ttext.SetColor(1.0, 1.0, 1.0);
	if (MatchInfo1Str(match.MatchRegion(), match.MatchType(), buf)) {
		ttext.Print(buf, 0.20, 0, 78);
		MatchInfo2Str(match.MatchType(), buf);
		ttext.Print(buf, 0.25, 0, 70);
	} else {
		ttext.Print(buf, 0.25, 0, 70);
	}
	MatchRoundsStr(buf);
	ttext.Print(buf, 0.25, 0, 60);
}

void Game::ShowMatchResult(char *buf) {
	if (match.IsBlueWon()) {
		BlueWinStr(buf);
	} else if (match.IsRedWon()) {
		RedWinStr(buf);
	} else if (match.IsDraw()) {
		DrawStr(buf);
	}
	ttext.Print(buf, 0.25, 0, 50);
	if (match.IsJudge()) {
		JudgeStr(buf);
	} else {
		if (match.IsNoContest()) {
			strcpy(buf, "No Contest");				
		} else if (match.IsDKODraw()) {
			strcpy(buf, "Double KO");
		} else if (match.IsTKO()) {
			strcpy(buf, "TKO");		
		} else {
			strcpy(buf, "KO");				
		}
	}
	ttext.Print(buf, 0.25, 0, 40);
}

void Game::RedCornerBoxer() {
	glViewport(0, height/2, width/2, height/2);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	camera[0].Set();
	light2.Set();
	glring.ShowAll();

	texs.Choice(TE_FACE);
	handler2.RightOffense();
	handler2.LeftOffense();		
	handler2.BoxerHandling(boxer2, motion2);
	if (handler2.IsSound()) {
		mixer.PlayChunk(handler2.SelectSound());	
	}  
	boxer2.CalcCoords();
	boxer2.CalcCenter();
	if (ShadowSwitch) {
		ShowShadow(boxer2);
	}
	boxer2.CallLists();
	boxer2.SetPoint(
		 boxer2.OX() + ( 2.0 - boxer2.OX()) * 0.05,
		 boxer2.OY() + (-2.0 - boxer2.OY()) * 0.05,
		 boxer2.OT() - 0.5);
	handler2.Move(boxer2);
}

void Game::RedCornerInfo(int select_boxer2, char *buf) {
	glViewport(0, 0, width/2, height/2);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	camera[0].Set();
	light2.Set();

	ttext.MessageEnter();
	ttext.SetColor(1.0, 1.0, 1.0);	
   	BoxerNumberStr(select_boxer2, buf);
  	ttext.Print(buf, 0.5, -0, 100);
   	BoxerNameStr(handler2, buf);
  	ttext.Print(buf, 0.5, 0, 80);
   	BoxerClassStr(boxer2, buf);
  	ttext.Print(buf, 0.5, 0, 60);

	BoxerHeightStr(boxer2, buf);
  	ttext.Print(buf, 0.5, 0, 30);
   	BoxerWeightStr(boxer2, buf);
  	ttext.Print(buf, 0.5, 0, 10);
   	BoxerReachStr(boxer2, buf);
  	ttext.Print(buf, 0.5, 0, -10);
   	ttext.MessageLeave();
}

void Game::BlueCornerBoxer() {
	glViewport(width/2, height/2, width/2, height/2);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	camera[1].Set();
	light2.Set();
	glring.ShowAll();

	texs.Choice(TE_FACE);
	handler1.RightOffense();
	handler1.LeftOffense();		
	handler1.BoxerHandling(boxer1, motion1);
	if (handler1.IsSound()) {
		mixer.PlayChunk(handler1.SelectSound());	
	}  
	boxer1.CalcCoords();
	boxer1.CalcCenter();
	if (ShadowSwitch) {
		ShowShadow(boxer1);
	}
	boxer1.CallLists();
	boxer1.SetPoint(
		 boxer1.OX() + (-2.0 - boxer1.OX()) * 0.05,
		 boxer1.OY() + ( 2.0 - boxer1.OY()) * 0.05,
		 boxer1.OT() - 0.5);
	handler1.Move(boxer1);
}

void Game::BlueCornerInfo(int select_boxer1, char *buf) {
	glViewport(width/2, 0, width/2, height/2);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	light.Set();
	camera[1].Set();
	light2.Set();

	ttext.MessageEnter();
	ttext.SetColor(1.0, 1.0, 1.0);	

	BoxerNumberStr(select_boxer1, buf);
  	ttext.Print(buf, 0.5, 0, 100);
   	BoxerNameStr(handler1, buf);
 	ttext.Print(buf, 0.5, 0, 80);
   	BoxerClassStr(boxer1, buf);
  	ttext.Print(buf, 0.5, 0, 60);

	BoxerHeightStr(boxer1, buf);
  	ttext.Print(buf, 0.5, 0, 30);
   	BoxerWeightStr(boxer1, buf);
  	ttext.Print(buf, 0.5, 0, 10);
  	BoxerReachStr(boxer1, buf);
 	ttext.Print(buf, 0.5, 0, -10);

	ttext.MessageLeave();
}

void Game::BoxerRecordInfo(StoryBoxer &sb, int region, int rank, char *buf) {	
	ttext.MessageEnter();
	ttext.SetColor(1.0, 1.0, 1.0);	
	CareerStr(sb, buf);
 	ttext.Print(buf, 0.5, 0, -35);
	RecordStr(sb, buf);
 	ttext.Print(buf, 0.5, 0, -55);
	RankStr(region, rank, buf);
 	ttext.Print(buf, 0.5, 0, -75);
	ttext.MessageLeave();
}	

void Game::BoxerNumberStr(int i, char* buf) {
	if (i < 0) {
		switch (lang) {
			case English:
				strcpy(buf, "Boxer Profile");	
			break;	
			case Kana:
			default:
				strcpy(buf, "{NT[vtB[");	
			break;
		}
	} else {
		switch (lang) {
			case Kana:
			case English:
				sprintf(buf, "Boxer No.%d", i + 1);
			break;	
			default:
				sprintf(buf, "{NT[ No.%d", i + 1);
			break;
		}
	}
}

void Game::BoxerNameStr(BoxerHandler &handler, char* buf) {
	switch (lang) {
		case Kana:
			sprintf(buf, "%s %s", handler.NickNameStrJp(), handler.FamilyNameStrEn());
		break;	
		case English:
			sprintf(buf, "%s %s", handler.NickNameStrEn(), handler.FamilyNameStrEn());
		break;	
		default:
			sprintf(buf, "%s %s", handler.NickNameStrJp(), handler.FamilyNameStrJp());
		break;
	}
}

void Game::BoxerHeightStr(Boxer &boxer, char* buf) {
	switch (lang) {
		case Kana:
	   		sprintf(buf, " : %5.1f cm  ", boxer.Height()*100.0);
		break;	
		case English:
	   		sprintf(buf, "Height : %5.1f in. ", boxer.Height()/0.0254);
		break;	
		default:
	   		sprintf(buf, " g  : %5.1f cm  ", boxer.Height()*100.0);
		break;
	}
}

void Game::BoxerWeightStr(Boxer &boxer, char* buf) {
	switch (lang) {
		case Kana:
	   		sprintf(buf, " : %5.1f kg  ", boxer.Weight()*1000.0);
		break;	
		case English:
	   		sprintf(buf, "Weight : %5.1f lbs.", boxer.Weight()/0.0004536);
		break;	
		default:
	   		sprintf(buf, " ̏d  : %5.1f kg  ", boxer.Weight()*1000.0);
		break;
	}
}

void Game::BoxerReachStr(Boxer &boxer, char* buf) {
	switch (lang) {
		case Kana:
	  		sprintf(buf, "[` : %5.1f cm  ", boxer.Reach()*100.0);
		break;	
		case English:
	  		sprintf(buf, "Reach  : %5.1f in. ", boxer.Reach()/0.0254);
		break;	
		default:
	  		sprintf(buf, "[` : %5.1f cm  ", boxer.Reach()*100.0);
		break;
	}
}

void Game::BoxerClassStr(Boxer &boxer, char* buf) {
	static char *class_jp[] = {
		"NGX`", 
		"~j}", 
		"Cg tC", 
		"tC", 
		"X[p[ tC",
		"o^",
		"X[p[ o^",
		"tFU[",
		"X[p[ tFU[",
		"Cg",
		"X[p[ Cg",
		"EF^[",
		"X[p[ EF^[",
		"~h",
		"X[p[ ~h",
		"Cg wr[",
		"N[U[",
		"wr["
	};
	static char *class_en[] = {
		"Unknown", 
		"Minimam", 
		"Light Fly", 
		"Fly", 
		"Super Fly",
		"Bantam",
		"Super Bantam",
		"Feather",
		"Super Feather",
		"Light",
		"Super Light",
		"Welter",
		"Super Welter",
		"Middle",
		"Super Middle",
		"Light Heavy",
		"Cruiser",
		"Heavy"
	};
	int cl;
	float pounds;
	cl = 0;
	pounds = boxer.Weight() / 0.0004536;
	
	if (pounds <= 105.0) {
		cl = 1;	
	} else if (pounds <= 108.0) {
		cl = 2;	 	
	} else if (pounds <= 112.0) {
		cl = 3;	 	
	} else if (pounds <= 115.0) {
		cl = 4;	 	
	} else if (pounds <= 118.0) {
		cl = 5;	 	
	} else if (pounds <= 122.0) {
		cl = 6;	 	
	} else if (pounds <= 126.0) {
		cl = 7;	 	
	} else if (pounds <= 130.0) {
		cl = 8;	 	
	} else if (pounds <= 135.0) {
		cl = 9;	 	
	} else if (pounds <= 140.0) {
		cl =10;	 	
	} else if (pounds <= 147.0) {
		cl =11;	 	
	} else if (pounds <= 154.0) {
		cl =12;	 	
	} else if (pounds <= 160.0) {
		cl =13;	 	
	} else if (pounds <= 168.0) {
		cl =14;	 	
	} else if (pounds <= 175.0) {
		cl =15;	 	
	} else if (pounds <= 190.0) {
		cl =16;	 	
	} else {
		cl =17;	 	
	}
	switch (lang) {
		case Kana:
	  		sprintf(buf, "%s NX", class_jp[cl]);
		break;	
		case English:
	  		sprintf(buf, "%s Class", class_en[cl]);
		break;	
		default:
	  		sprintf(buf, "%s", class_jp[cl]);
		break;
	}
}

void Game::SelectCustomStr(int i, char* buf) {
	static char *custom_kanji[] = {
		"g",
		"[`",
		"X^C",
		"\\",
		"Rr",
		"vl",
		"O[u",
		"gNX",
		"",
		""
	};
	static char *custom_kana[] = {
		"",
		"[`",
		"X^C",
		"܂",
		"Rr",
		"񂪂",
		"O[u",
		"gNX",
		"͂",
		""
	};
	static char *custom_english[] = {
		"Height",
		"Reach",
		"Style",
		"Stance",
		"Comb.",
		"Thought",
		"Gloves",
		"Trunks",
		"Skin",
		"Country"
	};

	switch (lang) {
		case Kana:
			strcpy(buf, custom_kana[i]);
		break;	
		case English:
			strcpy(buf, custom_english[i]);
		break;	
		default:
			strcpy(buf, custom_kanji[i]);
		break;
	}
}

void Game::CustomStyleStr(int i, char* buf) {
	static char *style_jp[] = {
		"AbvCg",
		"s[Ju[",
		"fgCg",
		"n[hK[h",
		"m[K[h",
		"nCXs[h",
		"gbL["
	};
	static char *style_en[] = {
		"Upright",
		"Peekaboo",
		"Detroit",
		"Hard Guard",
		"No Gurad",
		"High Speed",
		"Tricky"
	};
	switch (lang) {
		case English:
			strcpy(buf, style_en[i-1]);
		break;	
		default:
			strcpy(buf, style_jp[i-1]);
		break;
	}
}

void Game::CustomStanceStr(int i, char* buf) {
	static char *stance_jp[] = {
		"I[\x83\\hbNX",
		"TEX|["
	};
	static char *stance_en[] = {
		"Orthodox",
		"Southpaw"
	};
	switch (lang) {
		case English:
			strcpy(buf, stance_en[i]);
		break;	
		default:
			strcpy(buf, stance_jp[i]);
		break;
	}
}

void Game::CustomThoughtStr(int i, char* buf) {
	static char *thought_kanji[] = {
		"hd",
		"ϋɓI",
		"s"
	};
	static char *thought_kana[] = {
		"ڂĂ",
		"Ă",
		"ӂ߂"
	};
	static char *thought_english[] = {
		"Defensive",
		"Aggressive",
		"Unknown"
	};
	if (i > 3) {
		i = 3;	
	}
	switch (lang) {
		case Kana:
			strcpy(buf, thought_kana[i-1]);
		break;	
		case English:
			strcpy(buf, thought_english[i-1]);
		break;	
		default:
			strcpy(buf, thought_kanji[i-1]);
		break;
	}
}

void Game::CustomNationStr(int i, char* buf) {
	strcpy(buf, RegionStr(i));
}

int Game::MatchInfo1Str(int region, int match_type, char *buf) {
	static char title_match_jp[] = "^Cg}b`";
	static char title_match_en[] = "Title Match";	

	static char championship_kanji[] = "";
	static char championship_kana[] = "Ă";
	static char championship_english[] = "Championship Match";

	static char tournament_jp[] = "g[ig";
	static char tournament_en[] = "Tournament";
	
	switch (match_type) {
		case WORLD_TITLE_MATCH :
		case AREA_TITLE_MATCH :
		case NATION_TITLE_MATCH :
			switch (lang) {
				case Kana:	
					sprintf(buf, "%s%s", RegionStr(region), title_match_jp);
				break;					
				case English:
					sprintf(buf, "%s %s", RegionStr(region), title_match_en);
				break;
				default:	
					sprintf(buf, "%s%s",  RegionStr(region), title_match_jp);
				break;					
			}
			return 0;	
		break;

		case WORLD_CHAMPIONSHIP :
		case AREA_CHAMPIONSHIP :
		case NATION_CHAMPIONSHIP :
			switch (lang) {
				case Kana:	
					sprintf(buf, "%s%s", RegionStr(region), championship_kana);
				break;					
				case English:
					sprintf(buf, "%s %s", RegionStr(region), championship_english);
				break;
				default:	
					sprintf(buf, "%s%s",  RegionStr(region), championship_kanji);
				break;					
			}	
			return 0;	
		break;

		case WORLD_RANKER_FINAL :
		case AREA_RANKER_FINAL :
		case NATION_RANKER_FINAL :
		case WORLD_RANKER_SEMIFINAL :
		case AREA_RANKER_SEMIFINAL :
		case NATION_RANKER_SEMIFINAL :
		case WORLD_RANKER_QUARTERFINAL :
		case AREA_RANKER_QUARTERFINAL :
		case NATION_RANKER_QUARTERFINAL :
		case NATION_RANKER_1ST :
			switch (lang) {
				case Kana:	
					sprintf(buf, "%s%s%s", RegionStr(region), RankerStr(), tournament_jp);
				break;					
				case English:
					sprintf(buf, "%s %s %s", RegionStr(region), RankerStr(), tournament_en);
				break;
				default:	
					sprintf(buf, "%s%s%s",  RegionStr(region), RankerStr(), tournament_jp);
				break;					
			}	
			return 1;	
		break;

		case ROOKIE_KING_FINAL :
		case ROOKIE_KING_SEMIFINAL :
		case ROOKIE_KING_QUARTERFINAL :
		case ROOKIE_KING_1ST :
		case ROOKIE_KING_2ND :
		case ROOKIE_KING_3RD :	
			switch (lang) {
				case Kana:	
					sprintf(buf, "%s%s%s", RegionStr(region), RookieStr(), tournament_jp);
				break;					
				case English:
					sprintf(buf, "%s %s %s", RegionStr(region), RookieStr(), tournament_en);
				break;
				default:	
					sprintf(buf, "%s%s%s",  RegionStr(region), RookieStr(), tournament_jp);
				break;					
			}	
			return 1;	
		break;

		case DEBUT_MATCH :	
			switch (lang) {
				case Kana:	
					strcpy(buf, "fr[");
				break;					
				case English:
					strcpy(buf, "Debut Match");
				break;
				default:	
					sprintf(buf, "%s", "fr[");
				break;					
			}	
			return 0;	
		break;

		default:
			strcpy(buf, "");
			return 0;	
		break;
 	}
}

void Game::MatchInfo2Str(int match_type, char *buf) {
	static char final_kanji[] = "";
	static char semifinal_kanji[] ="";
	static char quarterfinal_kanji[] ="X";
	static char third_qualifying_kanji[] = "O";
	static char second_qualifying_kanji[] = "";
	static char first_qualifying_kanji[] = "";

	static char final_kana[] = "傤";
	static char semifinal_kana[] ="񂯂傤";
	static char quarterfinal_kana[] ="񂶂񂯂傤";
	static char third_qualifying_kana[] = "񂩂";
	static char second_qualifying_kana[] = "ɂ";
	static char first_qualifying_kana[] = "";

	static char final_english[] = "Final Match";
	static char semifinal_english[] ="Semifinal Match";
	static char quarterfinal_english[] ="Quarterfinal Match";
	static char third_qualifying_english[] = "Third Qualifying Match";
	static char second_qualifying_english[] = "Second Qualifying Match";
	static char first_qualifying_english[] = "First Qualifying Match";

	switch (match_type) {
		case WORLD_RANKER_FINAL :
		case AREA_RANKER_FINAL :
		case NATION_RANKER_FINAL :
		case ROOKIE_KING_FINAL :
			switch (lang) {
				case Kana:	
					strcpy(buf, final_kana);
				break;					
				case English:
					strcpy(buf, final_english);
				break;
				default:	
					strcpy(buf, final_kanji);
				break;					
			}	
		break;

		case WORLD_RANKER_SEMIFINAL :
		case AREA_RANKER_SEMIFINAL :
		case NATION_RANKER_SEMIFINAL :
		case ROOKIE_KING_SEMIFINAL :
			switch (lang) {
				case Kana:	
					strcpy(buf, semifinal_kana);
				break;					
				case English:
					strcpy(buf, semifinal_english);
				break;
				default:	
					strcpy(buf, semifinal_kanji);
				break;					
			}	
		break;

		case WORLD_RANKER_QUARTERFINAL :
		case AREA_RANKER_QUARTERFINAL :
		case NATION_RANKER_QUARTERFINAL :
		case ROOKIE_KING_QUARTERFINAL :
			switch (lang) {
				case Kana:	
					strcpy(buf, quarterfinal_kana);
				break;					
				case English:
					strcpy(buf, quarterfinal_english);
				break;
				default:	
					strcpy(buf, quarterfinal_kanji);
				break;					
			}	
		break;

		case ROOKIE_KING_3RD :	
			switch (lang) {
				case Kana:	
					strcpy(buf, third_qualifying_kana);
				break;					
				case English:
					strcpy(buf, third_qualifying_english);
				break;
				default:	
					strcpy(buf, third_qualifying_kanji);
				break;					
			}	
		break;

		case ROOKIE_KING_2ND :
			switch (lang) {
				case Kana:	
					strcpy(buf, second_qualifying_kana);
				break;					
				case English:
					strcpy(buf, second_qualifying_english);
				break;
				default:	
					strcpy(buf, second_qualifying_kanji);
				break;					
			}	
		break;

		case NATION_RANKER_1ST :
		case ROOKIE_KING_1ST :
			switch (lang) {
				case Kana:	
					strcpy(buf, first_qualifying_kana);
				break;					
				case English:
					strcpy(buf, first_qualifying_english);
				break;
				default:	
					strcpy(buf, first_qualifying_kanji);
				break;					
			}	
		break;

		case WORLD_TITLE_MATCH :
		case AREA_TITLE_MATCH :
		case NATION_TITLE_MATCH :
		case WORLD_CHAMPIONSHIP :
		case AREA_CHAMPIONSHIP :
		case NATION_CHAMPIONSHIP :
		default:
			strcpy(buf, "");
		break;
 	}
}	

void Game::CareerStr(StoryBoxer &sb, char *buf) {
	switch (lang) {
		case Kana:
			sprintf(buf, "%2d %2d˂", sb.Age(), sb.CareerYear());
		break;		
		case English:
			sprintf(buf, "Age %d  Career %d. Year", sb.Age(), sb.CareerYear());
		break;		
		default:	
			sprintf(buf, "%2d %2dN", sb.Age(), sb.CareerYear());
		break;			
	}
}

void Game::RecordStr(StoryBoxer &sb, char *buf) {
	switch (lang) {
		case Kana:
		case English:
			sprintf(buf, "T%d  W%d  L%d  D%d  %dKO's", sb.Match(), sb.Win(), sb.Loss(), sb.Draw(), sb.KO());
		break;		
		default:	
			sprintf(buf, "%3d%3d%3ds%3d%3dKO ", sb.Match(), sb.Win(), sb.Loss(), sb.Draw(), sb.KO());
		break;			
	}
} 

void Game::RankStr(int region, int rank, char *buf) {
	if (rank) { 		
		switch (lang) {
			case Kana:
			case English:
				sprintf(buf, "%s %d.", RegionStr(region), rank);
			break;		
			default:	
				sprintf(buf, "%s %d", RegionStr(region), rank);
			break;			
		}
	} else {
		TitleStr(region, WORLD_TITLE, buf);	
	}
}

void Game::TitleStr(int region, int title, char *buf) {
	switch (title) {
		case WORLD_TITLE:
		case AREA_TITLE:
		case NATION_TITLE:
			switch (lang) {
				case Kana:
					sprintf(buf, "%s`v", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "%s Champion", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "%s`sI", RegionStr(region));			
				break;
			}
		break;

		case EX_WORLD_TITLE:
		case EX_AREA_TITLE:
		case EX_NATION_TITLE:
			switch (lang) {
				case Kana:
					sprintf(buf, "܂%s`v", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "Ex-%s Champion", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "O%s`sI", RegionStr(region));			
				break;
			}
		break;					

		case FORMER_WORLD_TITLE:
		case FORMER_AREA_TITLE:
		case FORMER_NATION_TITLE:
			switch (lang) {
				case Kana:
					sprintf(buf, "%s`v", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "Former %s Champion", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "%s`sI", RegionStr(region));			
				break;
			}
		break;					

		case ROOKIE_KING:
			switch (lang) {
				case Kana:
					sprintf(buf, "%s񂶂񂨂", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "%s Rookie King", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "%sVl", RegionStr(region));			
				break;
			}
		break;

		case EX_ROOKIE_KING:
			switch (lang) {
				case Kana:
					sprintf(buf, "܂%s񂶂񂨂", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "Ex-%s Rookie King", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "O%sVl", RegionStr(region));			
				break;
			}
		break;					

		case FORMER_ROOKIE_KING:
			switch (lang) {
				case Kana:
					sprintf(buf, "%s񂶂񂨂", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "Former %s Rookie King", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "%sVl", RegionStr(region));			
				break;
			}
		break;					

		case WORLD_RANKER_WINNER:
		case AREA_RANKER_WINNER:
		case NATION_RANKER_WINNER:
			switch (lang) {
				case Kana:
					sprintf(buf, "%sJ[T䂤傤", RegionStr(region));			
				break;		
				case English:
					sprintf(buf, "%s Ranker T Winner", RegionStr(region));			
				break;		
				default:	
					sprintf(buf, "%sJ[TD", RegionStr(region));			
				break;
			}
		break;
	}
}

void Game::RecentStr(StoryBoxer &sb, char *buf) {
	int i, j;
	char c;
	
	j = 0;
	for (i = 0; i < 5; i++) {
    	c = ResultMark(sb.GetRecent(9-i));
		if (c != '\0') {
			buf[j++] = c;
		}
		if (j == 5) {
			buf[j++] = ' ';
		}  
	}
	for (i = 5; i < 10; i++) {
    	c = ResultMark(sb.GetRecent(9-i));
		if (c != '\0') {
			buf[j++] = c;
		}
		if (j == 5) {
			buf[j++] = ' ';
		}   
	}
	buf[j] = '\0';
}

char Game::ResultMark(int i) {
	static char result_mark[] = {
		'\0',
		SYMBOL_DOUBLE_CIRCLE,
		SYMBOL_CIRCLE1,
		SYMBOL_TRIANGLE_UP1,
		SYMBOL_INFINITY,
		SYMBOL_CROSS,
		SYMBOL_FOOTNOTE,
		SYMBOL_STAR1
	};
	return result_mark[i];
}

char* Game::ResultMarkJp(int i) {
	static char *result_mark[] = {"", "", "", "", "", "~", "", ""};
	return result_mark[i];
}

char* Game::ResultMarkEn(int i) {
	static char *result_mark[] = {"", "O", "o", "=", "-", "x", "X", "*"};
	return result_mark[i];
}

int Game::WorldTitleStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(WORLD_TITLE)) {
		TitleStr(WORLD, WORLD_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(EX_WORLD_TITLE)) {
		TitleStr(WORLD, EX_WORLD_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(FORMER_WORLD_TITLE)) {
		TitleStr(WORLD, FORMER_WORLD_TITLE, buf);	
		return 1;
	}
	return 0;
}

int Game::AreaTitleStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(AREA_TITLE)) {
		TitleStr(sb.Area(), AREA_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(EX_AREA_TITLE)) {
		TitleStr(sb.Area(), EX_AREA_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(FORMER_AREA_TITLE)) {
		TitleStr(sb.Area(), FORMER_AREA_TITLE, buf);	
		return 1;
	}
	return 0;
}

int Game::NationTitleStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(NATION_TITLE)) {
		TitleStr(sb.Nation(), NATION_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(EX_NATION_TITLE)) {
		TitleStr(sb.Nation(), EX_NATION_TITLE, buf);	
		return 1;
	}
	if (sb.HaveTitle(FORMER_NATION_TITLE)) {
		TitleStr(sb.Nation(), FORMER_NATION_TITLE, buf);	
		return 1;
	}
	return 0;
}

int Game::WorldRankerStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(WORLD_RANKER_WINNER)) {
		TitleStr(WORLD, WORLD_RANKER_WINNER, buf);	
		return 1;
	}
	return 0;
}

int Game::AreaRankerStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(AREA_RANKER_WINNER)) {
		TitleStr(sb.Area(), AREA_RANKER_WINNER, buf);	
		return 1;
	}
	return 0;
}

int Game::NationRankerStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(NATION_RANKER_WINNER)) {
		TitleStr(sb.Nation(), NATION_RANKER_WINNER, buf);	
		return 1;
	}
	return 0;
}

int Game::RookieKingStr(StoryBoxer &sb, char *buf) {
	if (sb.HaveTitle(ROOKIE_KING)) {
		TitleStr(sb.Nation(), ROOKIE_KING, buf);	
		return 1;
	}
	if (sb.HaveTitle(EX_ROOKIE_KING)) {
		TitleStr(sb.Nation(), EX_ROOKIE_KING, buf);	
		return 1;
	}
	if (sb.HaveTitle(FORMER_ROOKIE_KING)) {
		TitleStr(sb.Nation(), FORMER_ROOKIE_KING, buf);	
		return 1;
	}
	return 0;
}
