#include "game.h"

/* Xg[[[h */
void Game::Story() {
	int i, skip_month;
    StoryHandler story;

	if (!story.Load()) {/* f[^t@C쐬 */
		WaitMessage(0);
		story.Init();
		story.InitTitle();
		skip_month = random.Rand(100, 120);
		for (i = 0; i <= skip_month; i++){
			WaitMessage(10+ 90 * i / skip_month);
			story.NextMonth(*this, 0);
			story.SetEvents(*this, 0);
		}
		story.QuitBoxer(MY_BOXER);
		story.Boxer(MY_BOXER).CreateRandom(random, 0);
		story.Boxer(MY_BOXER).Init();
		SelectGameMode(story);
		story.Save();
	}
	story.SortRank();

//	BurnOut(story);
//	Ending(story);
//	WonTitle(story);
//	TournamentVictor(story);
//	ShowScore(story);
 	while(1) {
		switch (StoryMenu(story.Month())) {
			case 0:	/* ̎ */
				StoryMatches(story, 0);
			break;	
			case 1:	/*  */
				story.NextMonth(*this, 1);
				story.SetEvents(*this, 1);
				story.Save();
			break;
			case 2:	/* LO\ */
				Ranking(story);
			break;
			case 3:	/* g[ig\ */
				Tournament(story);
			break;	
			case 4:	/* 挎̎ */
				StoryMatches(story, 1);
			break;	
			case 5:	/*  */
				Record(story);
			break;
			case 6:	/* ߂ */
				return;
			break;	
			case 7:	/*  */
				if (YesNoMenu() == 0) {
					Retire(story);	
					ShowScore(story);
					story.QuitBoxer(MY_BOXER);
					story.Boxer(MY_BOXER).Inherit(random);
					SelectGameMode(story);
					story.SetNewRanking();
					story.ClearRecord();
					story.Save();
				}
			break;	
		}	
	}
}

void Game::SelectGameMode(StoryHandler &story) {
	SetKey1P();
	switch (ChoiceMenu()) {
		case ORIGINAL_MODE :
			SelectBoxer(SELECT_TRAINING);			
			if (handler2.SaveBinBoxerFile(boxer2, motion2, "original.sav") == SDL_FAILURE) {
				SaveErrorMessage();
				return;				
			}
			boxer2.SetDefaultFace();
			story.SetMode(ORIGINAL_MODE);
		break;
		case RANKING_MODE :
			story.Boxer(MY_BOXER).CreateRandom(random, random.Rand(JAPAN, SA_COUNTRIES));
			story.SwapBoxer(Ranking(story, 1));
			CustomizeBoxer(story);
			story.SetMode(RANKING_MODE);						
		break;
		default :
			CustomizeBoxer(story);
			story.SetMode(CUSTOM_MODE);
		break;	
	}
}

void Game::MyMatch(StoryHandler &story, int number) {
	static bool is_matched;
	ShowMatchProfile(story, story.SelectMatch(number)->low_ranked, story.SelectMatch(number)->high_ranked, 0);
	if (story.SelectMatch(number)->high_ranked == MY_BOXER) { 
		SetKeyRedCorner();
		is_matched = Match(SELECT_STORY_RED);
	} else {
		static int default_view_pre;
		default_view_pre = DefaultView;
		if (DefaultView == 2) {
			DefaultView = 3;
		} else if (DefaultView == 6) {
			DefaultView = 7;
		}

		SetKeyBlueCorner();
		is_matched = Match(SELECT_STORY_BLUE);
		SetKey1P();

		if (DefaultView == 3 && default_view_pre == 2) {
			DefaultView = 2;
		} else if (DefaultView == 7 && default_view_pre == 6) {
			DefaultView = 6;
		}
	}

	if (is_matched) {
		SaveReplayCycle();
	}

	match.WhichWin(handler1, handler2);
	story.SelectMatch(number)->result = match.ResultState();		
	story.SelectMatch(number)->replay = CurrentReplay;		
	story.MatchRecord(story.SelectMatch(number));
	story.AddRecord(number, match, handler1, handler2);
	story.AddScore();

	if (story.Boxer(MY_BOXER).GetRecent(0) == 7) {
		switch (story.SelectMatch(number)->match_type) {
			case WORLD_TITLE_MATCH : 
			case AREA_TITLE_MATCH :
			case NATION_TITLE_MATCH :
			case WORLD_CHAMPIONSHIP :
			case AREA_CHAMPIONSHIP :
			case NATION_CHAMPIONSHIP :
				WonTitle(story);						
			break;		
			case WORLD_RANKER_FINAL :
			case AREA_RANKER_FINAL :
			case NATION_RANKER_FINAL :
			case ROOKIE_KING_FINAL :
				TournamentVictor(story);						
			break;				
		}	
	}

	if (is_matched && !SkipReplay) {
		Replay();
	}
}

void Game::MouseSelectRecord(int &mouse_record, int x, int y) {
	int i;
/* ёI */
	if (width * 0.25 < x && width * 0.75 > x) {
		for (i = 0; i < 20; i++) {
			if (height* (0.17 + i * 0.035) < y &&
				height* (0.17 + (i+1) * 0.035) > y) {
				mouse_record = i;
			}
		}
	}
}

void Game::RecordDate(StoryHandler &story, int number, char *buf) {
	static char *month_eng[] = {"Jan.", "Feb.", "Mar.", "Apr.", "May.", "Jun.",
			"Jul.", "Aug.", "Sep.", "Oct.", "Nov.", "Dec."};
	switch (lang) {
		case Kana :
			sprintf(buf, "%3d. %2d˂ %2d", number+1, story.RecordYear(number), story.RecordMonth(number));
		break;
		case English :
			sprintf(buf, "%3d. %2d. Year %4s    ", number+1, story.RecordYear(number), month_eng[story.RecordMonth(number)-1]);
		break;
		default :                             
			sprintf(buf, "%3d. %2dN %2d    ", number+1, story.RecordYear(number), story.RecordMonth(number));
		break;				
	}
}

void Game::RecordName(StoryHandler &story, int number, char *buf) {
	static char name[30];
	
	handler1.SetBoxerName(story.RecordBoxer(number));
	BoxerNameStr(handler1, name);
	sprintf(buf, "vs %-25s", name);
}

void Game::RecordJudge(StoryHandler &story, int number, char *buf) {
	sprintf(buf, "%d-%d,%d-%d,%d-%d",
		story.RecordMyJudge1(number),
		story.RecordJudge1(number),
		story.RecordMyJudge2(number),
		story.RecordJudge2(number),
		story.RecordMyJudge3(number),
		story.RecordJudge3(number));
}

void Game::RecordTime(StoryHandler &story, int number, char *buf) {
	sprintf(buf, "%2dR %2d:%02d:%02d", 
		story.RecordRound(number),
		story.RecordMinute(number),
		story.RecordSecond(number),
		story.RecordMSecond(number));			
}

void Game::Record(StoryHandler &story) {
	static char line[] = "---------------------------------------------------------------------------";
	static char buf[255];
	static int key_record = 0;
	static int mouse_select = -1, mouse_record = -1;
	static int x, y, i, loop, show_info;
	static int column_number = 0, max_column;	
	static int record_number;
	static float size, ssize;
	 
	InitRanking();

 	column_number = (story.RecordNumber()-1) / 20;
	max_column    = column_number;
	key_record	  = (story.RecordNumber()-1) % 20;

	timer.SetFPS(50);	
	do {
		RankingOptions();
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		ssize =	(0.9 + 0.1 * sin(RAD(3*loop++)));

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		if (width * 0.95 < x && width * 1.0 > x) {
			if (height* 0.40 < y && height* 0.50 > y) {
				mouse_select = RANKING_UP2;
			}
			if (height* 0.50 < y && height* 0.60 > y) {
				mouse_select = RANKING_DOWN2;
			}
		}
		mouse_record = -1;
		MouseSelectRecord(mouse_record, x, y);

/* \[gI(L[{[h) */
		if (mouse_select == RANKING_UP1 && event.IsMouse1Down()) {
			column_number = 0;
		}
 		if (mouse_select == RANKING_DOWN1 && event.IsMouse1Down()){
			column_number = max_column;
		}
		if (mouse_select == RANKING_UP2 && event.IsMouse1Down()) {
			if (--column_number < 0) {
				column_number = 0;
            }
		}
 		if (mouse_select == RANKING_DOWN2 && event.IsMouse1Down()) {
			if (++column_number > max_column) {
				column_number = max_column;
			}
		}

 		if (event.IsKeyDown(kb2_left)) {
			if (event.IsKeyPressed(kb2_right_offense)) {
				if (--column_number < 0) {
					column_number = 0;
 	        	}			
			} else {
				if (--key_record < 0) {
					if (--column_number < 0) {
						column_number = 0;
					 	key_record = 0;
					} else {
					 	key_record = 19;			
					}	 
				}
			}
		}
		if (event.IsKeyDown(kb2_right)) {
			if (event.IsKeyPressed(kb2_right_offense)) {
				if (++column_number > max_column) {
					column_number = max_column;
				}
			} else {
				if (++key_record > 19) {
					if (++column_number > max_column) {
						column_number = max_column;
					 	key_record = 19;
					} else {
					 	key_record = 0;			
					} 
				}
			}
		}
		
		show_info = 0;
		if (mouse_record >= 0 && mouse_record <= 20 && event.IsMouse1Down()) {
			record_number = column_number * 20 + mouse_record;
			if (record_number < story.RecordNumber()) {
				key_record = mouse_record;
			    show_info = 1;
			}
		}
		if (event.IsKeyDown(kb2_left_offense)) {
		    show_info = 1;
		}
		if (show_info) {
			record_number = column_number * 20 + key_record;
			if (record_number < story.RecordNumber()) {
				SelectMatchType(story.RecordMatch(record_number).match_type, story.Boxer(MY_BOXER));
				match.SetResultState(story.RecordMatch(record_number).result);
				if (SelectRecord(story, record_number)) {
					sprintf(buf, "./Replay/cycle%03d.rep", story.RecordMatch(record_number).replay);
					if (LoadReplayFile(buf, 1) == SDL_SUCCESS) {
						Replay();
					}
				}
			} else {
				key_record = (story.RecordNumber()-1) % 20;
			}
		}

		if (UsePText) {
			ptext.MessageEnter();
			ptext.SetSpecular(0.0, 0.0, 0.0);
			ptext.SelectColor(mouse_select == RANKING_UP2);
			ptext.Symbol(SYMBOL_TRIANGLE_UP2, 0.4, 97, 7);
			ptext.SelectColor(mouse_select == RANKING_DOWN2);
			ptext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.4, 97, -7);
			ptext.MessageLeave();
		} else {
			ttext.MessageEnter();
			ttext.SelectColor(mouse_select == RANKING_UP2);
			ttext.Symbol(SYMBOL_TRIANGLE_UP2, 0.4, 97, 7);
			ttext.SelectColor(mouse_select == RANKING_DOWN2);
			ttext.Symbol(SYMBOL_TRIANGLE_DOWN2, 0.4, 97, -7);
			ttext.MessageLeave();		
		}
	
		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		switch (lang) {
			case Kana :
		        ttext.Print("񂹂", 0.5, 0, 90);
			break;
			case English :
		        ttext.Print("Record", 0.5, 0, 90);
			break;
			default :                             
		        ttext.Print("", 0.5, 0, 90);
			break;				
		}
        ttext.Print(line, 0.20, -90, 70);

		for (i = 0; i < 20; i++) {
			record_number = column_number * 20 + i;
			if (record_number < story.RecordNumber()) {
				RecordDate(story, record_number, buf);
				ttext.SetColor(1.0, 1.0, 1.0);	
		        ttext.Print(buf, 0.20, -95, 65 - i * 7);		

				if (lang == Japanese) {
					ttext.Print(ResultMarkJp(story.RecordMark(record_number)), 0.20, 40, 65 - i * 7);		
				} else {
					ttext.Symbol(ResultMark(story.RecordMark(record_number)), 0.25, 40, 65 - i * 7);		
				}

				if (story.RecordRound(record_number) == 255) {
					RecordJudge(story, record_number, buf);
			        ttext.Print(buf, 0.15, 85, 65 - i * 7);		
				} else {
					RecordTime(story, record_number, buf);
			        ttext.Print(buf, 0.20, 85, 65 - i * 7);		
				}

				RecordName(story, record_number, buf);
				ttext.SelectColor(mouse_record == i);
				key_record == i ? size = ssize : size = 1.0;
				ttext.Print(buf, 0.20 * size, -10, 65 - i * 7);
			} else {
				if (key_record >= i) {
					key_record = i - 1;
				}						
			}
		}
		ttext.SetColor(1.0, 1.0, 1.0);	
        ttext.Print(line, 0.20, -90, -74);		
		RecordStr(story.Boxer(MY_BOXER), buf);
	 	ttext.Print(buf, 0.25, 0, -85);

		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Back", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();				

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(SDLK_o)) {
			PrintScore(story);
		}
		if (event.IsKeyDown(go_next) || event.IsMouse3Down() ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeRanking();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	

	story.SortRegion(WORLD);
}

void Game::ShowScore(StoryHandler &story) {
	static char buf[255];	
	static int mouse_select = -1;
	static int x, y, i, loop, new_entry;
	static float size, csize;
	 
	InitRanking();

	new_entry = -1;
	if (story.Mode() == CUSTOM_MODE) {
		for (i = 0; i < 10; i++) {
			if (story.Score() >= story.HighScores(i)) {
				new_entry = i;
				story.SetHighScores(new_entry, story.Score());
				break;
			}
		}
	}

	timer.SetFPS(50);	
	do {
		csize =	(1.1 + 0.1 * sin(RAD(5*loop++)));

		RankingOptions();
		glClearColor(0.33, 0.33, 0.33, 0.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		SDL_GetMouseState(&x, &y);
		mouse_select = -1;
		MouseSelectGoNext(mouse_select, x, y);
		if (width * 0.85 < x && width * 0.97 > x) {
			if (height* 0.87 < y && height* 0.94 > y) {
				mouse_select = OUTPUT_SCORE;			
			}
		}

		ttext.MessageEnter();
		ttext.SetColor(1.0, 1.0, 1.0);	
		ttext.Print("HIGH SCORES", 0.5, 0, 90);

		for (i = 0; i < 10; i++) {
			ttext.SelectColor(i == new_entry);
			sprintf(buf, "%2d. %12s$", i+1, ScoreStr(story.HighScores(i)));
			ttext.Print(buf, 0.4, 0, 65 - i * 15);
		}

		ttext.SetColor(1.0, 1.0, 1.0);	
		sprintf(buf, "GAME SCORE = %s$", ScoreStr(story.Score()));
		ttext.Print(buf, 0.4, 0, -90);

		ttext.SelectColor(mouse_select == OUTPUT_SCORE);
		ttext.Print("Output", 0.25*csize, 90, -85);			
		ttext.SelectColor(mouse_select == GO_NEXT);
   		ttext.Print("Next", 0.25, -100, -100);
		ttext.SetColor(1.0, 1.0, 1.0);	
 		ttext.Print(timer.FPSString(), 0.25, 100, -100);
		ttext.MessageLeave();				

		SDL_GL_SwapBuffers(); 
		GLmes();

		timer.Wait();
		event.Poll();
		if (event.IsKeyDown(kb2_left_offense) || event.IsKeyDown(SDLK_o)||
			(mouse_select == OUTPUT_SCORE && event.IsMouse1Down())) {
			PrintScore(story);
		}
		if (event.IsKeyDown(go_next) || event.IsMouse3Down() ||
			(mouse_select == GO_NEXT && event.IsMouse1Down())) {
			break;		
		}
		ExposeRanking();
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}
 
 	glViewport(0, 0, width, height);
 	glClearColor(0.0, 0.0, 0.0, 0.0);
//	ttext.Clear();	 	

	story.SortRegion(WORLD);
}

void Game::PrintScore(StoryHandler &story) {
	char line[] = "---------------------------------------------------------------------------";
	char buf[255];	
	static int i;
	FILE *fp;

	fp = fopen("score.txt", "wt");
	if (fp == NULL) {
		return;	
	}

	fprintf(fp, "   %s   \n\n", "HIGH SCORES");
	for (i = 0; i < 10; i++) {
		fprintf(fp, "%2d. %12s$\n", i+1, ScoreStr(story.HighScores(i)));
	}

	fprintf(fp, "\nGAME SCORE = %s$\n\n\n", ScoreStr(story.Score()));

	switch (lang) {
		case Kana :
			fprintf(fp, "%s", "񂹂");
		break;
		case English :
			fprintf(fp, "%s", "Record");
		break;
		default :                             
			fprintf(fp, "%s", "");
		break;				
	}

	RecordStr(story.Boxer(MY_BOXER), buf);
	fprintf(fp, "  %s\n", buf);
	fprintf(fp, "%s\n", line);
	for (i = 0; i < story.RecordNumber(); i++) {
		RecordDate(story, i, buf);
		fprintf(fp, "%s ", buf);
		RecordName(story, i, buf);
		fprintf(fp, "%s ", buf);

		if (lang != English) {
			fprintf(fp, "%s ", ResultMarkJp(story.RecordMark(i)));		
		} else {
			fprintf(fp, "%s ", ResultMarkEn(story.RecordMark(i)));		
		}

		if (story.RecordRound(i) == 255) {
			RecordJudge(story, i, buf);
			fprintf(fp, "%s\n", buf);
		} else {
			RecordTime(story, i, buf);
			fprintf(fp, "%s\n", buf);
		}
	}
	fprintf(fp, "%s", line);

	switch (lang) {
		case Kana :
			fprintf(fp, "\n\n%s\n", "Ƃ^Cg");				
		break;
		case English :
			fprintf(fp, "\n\n%s\n", "Titles");				
		break;
		case Japanese :		
			fprintf(fp, "\n\n%s\n", "l^Cg");				
		break;		
	}
	if (story.Boxer(MY_BOXER).HaveTitle(WORLD_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(EX_WORLD_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(FORMER_WORLD_TITLE)) {
		TitleStr(WORLD, WORLD_TITLE, buf);	
		fprintf(fp, "%s\n", buf);
	}
	if (story.Boxer(MY_BOXER).HaveTitle(AREA_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(EX_AREA_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(FORMER_AREA_TITLE)) {
		TitleStr(story.Boxer(MY_BOXER).Area(), AREA_TITLE, buf);	
		fprintf(fp, "%s\n", buf);
	}
	if (story.Boxer(MY_BOXER).HaveTitle(NATION_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(EX_NATION_TITLE) || 
		story.Boxer(MY_BOXER).HaveTitle(FORMER_NATION_TITLE)) {
		TitleStr(story.Boxer(MY_BOXER).Nation(), NATION_TITLE, buf);	
		fprintf(fp, "%s\n", buf);
	}
	if (WorldRankerStr(story.Boxer(MY_BOXER), buf)) {
		fprintf(fp, "%s\n", buf);
	}
	if (AreaRankerStr(story.Boxer(MY_BOXER), buf)) {
		fprintf(fp, "%s\n", buf);
	}
	if (NationRankerStr(story.Boxer(MY_BOXER), buf)) {
		fprintf(fp, "%s\n", buf);
	}
	if (story.Boxer(MY_BOXER).HaveTitle(ROOKIE_KING) || 
		story.Boxer(MY_BOXER).HaveTitle(EX_ROOKIE_KING) || 
		story.Boxer(MY_BOXER).HaveTitle(FORMER_ROOKIE_KING)) {
		TitleStr(story.Boxer(MY_BOXER).Nation(), ROOKIE_KING, buf);	
		fprintf(fp, "%s\n", buf);
	}

	fclose(fp);
}

char* Game::ScoreStr(int number) {
	static char score[15];
	int t1, t2, t3;
	t1 = number / 1000000;
	number -= t1 * 1000000;
	t2 = number / 1000;
	number -= t2 * 1000;
	t3 = number;

	if (t1) {
		sprintf(score, "%3d,%03d,%03d", t1, t2, t3);
		return score;
	}	
	if (t2) {
		sprintf(score, "%3d,%03d", t2, t3);
		return score;
	}	
	sprintf(score, "%3d", t3);
	return score;
}    


void Game::Retire(StoryHandler &story) {
	if (story.Mode() == CUSTOM_MODE &&
		(story.Boxer(MY_BOXER).HaveTitle(WORLD_TITLE) ||
		story.Boxer(MY_BOXER).HaveTitle(EX_WORLD_TITLE) ||
		story.Boxer(MY_BOXER).HaveTitle(FORMER_WORLD_TITLE))) { 
		Ending(story);
    } else {
		BurnOut(story);
    }
}

/*
void Game::MyMatch(StoryHandler &story) {
	int i;
	story.MaxColumn(MY_BOXER);
	story.RegionMatch(story.SelectRegion());
	for (i = 0; i < story.MaxMatch(); i++) {
		if (story.MatchExists(i)) {

			if (story.SelectMatch(i)->high_ranked == MY_BOXER ||
				story.SelectMatch(i)->low_ranked == MY_BOXER) {
				SelectMatchType(story.SelectMatch(i)->match_type, story.Boxer(story.SelectMatch(i)->high_ranked));
				story.SortRegion(match.MatchRegion());
				break;
			}	 			
		}	
	}
} */

bool Game::MatchSilent(int select_type, int match_number) {
//	char buf[255];

	match.Init(MaxRound, RoundTime,
		TKOSystem, JudgeSystem, CountSystem, ExtraRound, KOCount,
		Judge1Type, Judge2Type, Judge3Type, 
		ExtraInfo, ReplayRate);
	handler1.Init(boxer1, motion1, match);
	handler2.Init(boxer2, motion2, match);

	handler1.SetMode(FULL_AUTO);
	handler2.SetMode(FULL_AUTO);

	SetCamera();
	SetLights();
	SetDefaultView();	
	SetBoxerPosition(1.6);
	SetBoxerName();

	handler1.SetDistance(distance);	
	handler2.SetDistance(distance);	
	handler1.SetRound(1);
	handler2.SetRound(1);	
	handler1.SaveInfo(0, boxer1, motion1);
	handler2.SaveInfo(0, boxer2, motion2);	 

/*	NextOff();
	timer.SetFPS(1000);	*/
	do {
		if (match.IsShowRound()) {
			if (match.HasShownRound()) {
				match.Matching();
				handler1.SaveInfo(0, boxer1, motion1);
				handler2.SaveInfo(0, boxer2, motion2);	
				match.SetCounter(-50);		
			}
		} 
	
		handler1.SetDistance(distance);	
		handler2.SetDistance(distance);	

		handler1.GetComStates(match, boxer1, handler2);
		handler2.GetComStates(match, boxer2, handler1);

		handler1.SaveKeyStates(match.TimeCount());
		handler2.SaveKeyStates(match.TimeCount());

		if (match.IsBeforeMatch()) {	
			match.ShowRound();
		}

		if (match.IsRoundInterval()) {	
			if (distance > 6.0) {
				if (match.IsLastRound()) {
					match.Decision(handler1, handler2);
					if (match.IsExtraRoundSystem() && match.IsDraw() && !match.IsExtraRound()) {
						RoundStart();
					} else {
						match.End();
						handler1.SaveEndInfo(boxer1, motion1);
						handler2.SaveEndInfo(boxer2, motion2);			
					}
				} else {
					RoundStart();
				}
			}
		}

		if (!match.IsMatching()) {
			handler1.InitKey();		
			handler2.InitKey();	
		} else {
			if (handler2.IsCounting() && !handler1.IsCounting()) {
				handler1.InitKey();		
			}
			if (handler1.IsCounting() && !handler2.IsCounting()) {
				handler2.InitKey();		
			}
		}

		CalcRoutineSilent();		
//		if ((match.TimeCount() + 1) % 100 == 0) {
//			ShowRoutine();		
//			fprintf(stderr, "Match %d  Round %d  Time %2d\n", match_number, match.Round(), match.TimeCount()/50);
//		} else {
//			fprintf(stderr, "Time %2d\n", match.TimeCount());
//		}

/*		event.Poll();
		ExposeScreen();
*/
		if (match.IsEnd()) {
			break;		
		}
	} while(!event.IsQuit());
	if (event.IsQuit()) {
		Quit();
	}

	return true;	
}

void Game::CalcRoutineSilent() {
	handler1.BoxerHandling(boxer1, motion1);
	handler2.BoxerHandling(boxer2, motion2);

	if (handler1.IsClinching()) {
		handler2.SetStateClinch(motion2);	
	}
	if (handler2.IsClinching()) {
		handler1.SetStateClinch(motion1);	
	}  

	CalcDistance();

	boxer1.CalcCoords();
	boxer1.CalcCenter();
	boxer2.CalcCoords();
	boxer2.CalcCenter();

	if (handler1.IsOffenseState()) {
		handler1.CheckHitPunch(boxer1, motion1, boxer2, handler2);			
	}
	if (handler2.IsOffenseState()) {
		handler2.CheckHitPunch(boxer2, motion2, boxer1, handler1);			
	}

	if (handler1.IsDamaged()) {
		handler1.SetDamage(boxer1, motion1);			
		handler1.SetNoDamage();
		handler2.SetStatePull();
	}
	if (handler2.IsDamaged()) {
		handler2.SetDamage(boxer2, motion2);			
		handler2.SetNoDamage();
		handler1.SetStatePull();
	}

	handler1.Move(boxer1);
	handler2.Move(boxer2);

	if (match.IsMatching()) {
		handler1.RestoreDamage();
		handler2.RestoreDamage();
    }

	CalcMeters();
	CalcAdjust();
	if (!match.IsEnd()) {
		CheckResult();
		if (match.IsEnd()){
			match.SetLatestRound();	
			handler1.SaveEndInfo(boxer1, motion1);
			handler2.SaveEndInfo(boxer2, motion2);		
		}	
	}

	if (match.IsRoundInterval()) {
		if (handler1.IsDownState()) {
			handler1.SetStateRestore();		
			motion1.SetRestore();		
        }
		if (handler2.IsDownState()) {
			handler2.SetStateRestore();		
			motion2.SetRestore();		
        }
		if (handler1.IsWaitState()) {
			handler1.InitCount();					
			handler1.SetStateRhythm();				
		}
		if (handler2.IsWaitState()) {
			handler2.InitCount();					
			handler2.SetStateRhythm();				
		}
		boxer1.GoBlueCorner();	
		boxer2.GoRedCorner();	
	}

	if (match.IsMatching()) {
		match.TickTime();
		if (match.IsGong()) {
			if(match.CountMustbeContinuing()) {
				if (!handler1.IsCounting() && !handler2.IsCounting()) {
					RoundEndSilent();		
				}
			} else {
				RoundEndSilent();					
			}
		}
	}
	  
	if (match.IsReplayTime() && match.IsMatching()) {
		handler1.SaveInfo(match.ReplayNumber(), boxer1, motion1);
		handler2.SaveInfo(match.ReplayNumber(), boxer2, motion2);	
	} 
}

void Game::RoundEndSilent() {
	match.RoundInterval();
	match.SetLatestRound();
	match.Judgement(handler1, handler2);

	handler1.SetJudges();
	handler2.SetJudges();
	handler1.SaveIntervalInfo(boxer1, motion1);
	handler2.SaveIntervalInfo(boxer2, motion2);		
}

