#ifndef GLCAMERA_H
#define GLCAMERA_H

#include "csdl.h"

#define CURRENT_CAMERA 65536

class GLCamera {
public:
	GLCamera();
	~GLCamera();

	void DefPosition(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void Position(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void SetPosition();

	void DefDirection(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Direction(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetDirection();

	void DefHeadUp(GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void HeadUp(GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void SetHeadUp();

	void DefRotation(GLfloat r = 0.0);
	void Rotation(GLfloat r = 0.0);
	void SetRotation();

	float PosX() const {return px;}
	float PosY() const {return py;}
	float PosZ() const {return pz;}
	float DirX() const {return dx;}
	float DirY() const {return dy;}
	float DirZ() const {return dz;}
	float Rot() const {return rot;}
		
	GLCamera& operator=(const GLCamera &glcamera); 
	void Show();
	void Print();
		
	void Set();
private:
	GLfloat px, py, pz;
	GLfloat dx, dy, dz;
	GLfloat hx, hy, hz;
	GLfloat rot;	
};


class GLCameras {
public:
	GLCameras();
	GLCameras(int number);
	~GLCameras();

	int Init(int number);
	void Free();

	void DefPosition(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void Position(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void SetPosition();

	void DefPosition(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void Position(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0);
	void SetPosition(int number);

	void DefDirection(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Direction(GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetDirection();

	void DefDirection(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void Direction(int number, GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 1.0);
	void SetDirection(int number);

	void DefHeadUp(GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void HeadUp(GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void SetHeadUp();

	void DefHeadUp(int number, GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void HeadUp(int number, GLfloat x = 0.0, GLfloat y = 1.0, GLfloat z = 0.0);
	void SetHeadUp(int number);

	void DefRotation(GLfloat r = 0.0);
	void Rotation(GLfloat r = 0.0);
	void SetRotation();

	void DefRotation(int number, GLfloat r = 0.0);
	void Rotation(int number, GLfloat r);
	void SetRotation(int number);

	void DefPerspective(GLfloat a = 60.0, GLfloat zn = 1.0, GLfloat zf = 1024.0, GLfloat w = 640.0, GLfloat h = 480.0);
	void Perspective(GLfloat a = 60.0, GLfloat zn = 1.0, GLfloat zf = 1024.0, GLfloat w = 640.0, GLfloat h = 480.0);
	void SetPerspective();

	void DefOrtho(GLfloat zn = 1.0, GLfloat zf = 1024.0, GLfloat w = 640.0, GLfloat h = 480.0);
	void Ortho(GLfloat zn = 1.0, GLfloat zf = 1024.0, GLfloat w = 640.0, GLfloat h = 480.0);
	void SetOrtho();

	void Show();
	void Show(int number);
		
	void Set();
	void Set(int number);

	void Add(int number = CURRENT_CAMERA);
	void Del(int number = CURRENT_CAMERA);
 	GLCamera& operator[](int number);

	int Count();
	void Choice(int number);
	bool WasInit();
private:
	GLCamera *cameras;
	int mnumber;
	int cnumber;
	GLfloat angle;
	GLfloat znear;
	GLfloat zfar;
	GLfloat width;
	GLfloat height;	
};

#endif // GLCAMERA_H
