#include "gllight.h"
#include "glshapes.h"

/*@Cg */

GLLight::GLLight() {
	gllight = 0;
	gl_pos[3] = 0.0;
	DefDiffuse();
	DefAmbient();
	DefSpecular();
	DefPosition();
	DefDirection();
	DefAttenuation();
	DefCutOff();
	DefExponent();	
}

void GLLight::Init(int number) {
	static const GLenum lightenum[]= {
	GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3,
	GL_LIGHT4, GL_LIGHT5, GL_LIGHT6, GL_LIGHT7};
	gllight = lightenum[number];
}

void GLLight::DefDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_dif[0] = r;}
	if (g >= 0.0) {gl_dif[1] = g;}
	if (b >= 0.0) {gl_dif[2] = b;}
	if (a >= 0.0) {gl_dif[3] = a;}
}

void GLLight::Diffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefDiffuse(r, g, b, a);
	SetDiffuse();
}
	
void GLLight::SetDiffuse() {
	glLightfv(gllight, GL_DIFFUSE, gl_dif);
}

void GLLight::DefAmbient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_amb[0] = r;}
	if (g >= 0.0) {gl_amb[1] = g;}
	if (b >= 0.0) {gl_amb[2] = b;}
	if (a >= 0.0) {gl_amb[3] = a;}
}

void GLLight::Ambient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefAmbient(r, g, b, a);
	SetAmbient();
}
	
void GLLight::SetAmbient() {
	glLightfv(gllight, GL_AMBIENT, gl_amb);
}

void GLLight::DefSpecular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_spe[0] = r;}
	if (g >= 0.0) {gl_spe[1] = g;}
	if (b >= 0.0) {gl_spe[2] = b;}
	if (a >= 0.0) {gl_spe[3] = a;}
}

void GLLight::Specular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefAmbient(r, g, b, a);
	SetAmbient();
}
	
void GLLight::SetSpecular() {
	glLightfv(gllight, GL_SPECULAR, gl_spe);
}

void GLLight::DefPosition(GLfloat x, GLfloat y, GLfloat z) {
	if (x != NO_MOVE) {gl_pos[0] = x;}
	if (y != NO_MOVE) {gl_pos[1] = y;}
	if (z != NO_MOVE) {gl_pos[2] = z;}
}

void GLLight::Position(GLfloat x, GLfloat y, GLfloat z) {
	DefPosition(x, y, z);
	SetPosition();
}
	
void GLLight::SetPosition() {
	glLightfv(gllight, GL_POSITION, gl_pos);
}

void GLLight::DefDirection(GLfloat x, GLfloat y, GLfloat z) {
	if (x != NO_MOVE) {gl_dir[0] = x;}
	if (y != NO_MOVE) {gl_dir[1] = y;}
	if (z != NO_MOVE) {gl_dir[2] = z;}
}

void GLLight::Direction(GLfloat x, GLfloat y, GLfloat z) {
	DefDirection(x, y, z);
	SetDirection();
}
	
void GLLight::SetDirection() {
	glLightfv(gllight, GL_SPOT_DIRECTION, gl_dir);
}

void GLLight::DefAttenuation(GLfloat c, GLfloat l, GLfloat q) {
	if (c != NO_MOVE) {gl_att[0] = c;}
	if (l != NO_MOVE) {gl_att[1] = l;}
	if (q != NO_MOVE) {gl_att[2] = q;}
}

void GLLight::Attenuation(GLfloat c, GLfloat l, GLfloat q) {
	DefAttenuation(c, l, q);
	SetAttenuation();
}

void GLLight::SetAttenuation() {
	glLightf(gllight, GL_CONSTANT_ATTENUATION, gl_att[0]);
	glLightf(gllight, GL_LINEAR_ATTENUATION, gl_att[1]);
	glLightf(gllight, GL_QUADRATIC_ATTENUATION, gl_att[2]);
}

void GLLight::DefCutOff(GLfloat c) {
	cutoff = c;
}

void GLLight::CutOff(GLfloat c) {
	DefCutOff(c);
	SetCutOff();
}

void GLLight::SetCutOff() {
	glLightf(gllight, GL_SPOT_CUTOFF, cutoff);		
}

void GLLight::DefExponent(GLfloat e) {
	exponent = e;
}

void GLLight::Exponent(GLfloat e) {
	DefExponent(e);
	SetExponent();
}

void GLLight::SetExponent() {
	glLightf(gllight, GL_SPOT_EXPONENT, exponent);		
}

void GLLight::FlatLight() {
	gl_pos[3] = 0.0;
}

bool GLLight::IsFlatLight() {
	if (gl_pos[3] == 0.0) {
		return true;
	}
	return false;	
}

void GLLight::SpotLight() {
	gl_pos[3] = 1.0;
}

bool GLLight::IsSpotLight() {
	if (gl_pos[3] == 1.0) {
		return true;
	}
	return false;
}
	
void GLLight::Set() {
	SetDiffuse();
	SetAmbient();
	SetSpecular();
	SetPosition();
	if (IsSpotLight()) {
		SetDirection();
		SetAttenuation();
		SetCutOff();
		SetExponent();	
	}
}

void GLLight::Get() {
	glGetLightfv(gllight, GL_DIFFUSE, gl_dif);
	glGetLightfv(gllight, GL_AMBIENT, gl_amb);
	glGetLightfv(gllight, GL_SPECULAR, gl_spe);
	glGetLightfv(gllight, GL_POSITION, gl_pos);
	glGetLightfv(gllight, GL_SPOT_DIRECTION, gl_dir);
	GLfloat glfloat4[4];
	glGetLightfv(gllight, GL_CONSTANT_ATTENUATION, glfloat4);
	gl_att[0] = glfloat4[0];
	glGetLightfv(gllight, GL_LINEAR_ATTENUATION, glfloat4);
	gl_att[1] = glfloat4[0];
	glGetLightfv(gllight, GL_QUADRATIC_ATTENUATION, glfloat4);
	gl_att[2] = glfloat4[0];
	glGetLightfv(gllight, GL_SPOT_CUTOFF, glfloat4);
	cutoff = glfloat4[0];
	glGetLightfv(gllight, GL_SPOT_EXPONENT, glfloat4);
	exponent = glfloat4[0];
}

void GLLight::On() {
	glEnable(gllight);
}
	
void GLLight::Off() {
	glDisable(gllight);
}

bool GLLight::IsOn() {
	return glIsEnabled(gllight);
}

void GLLight::Show() {
	GLfloat pos[3];
	GLdouble absv, absv2, t;

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);

  	glPushMatrix();

	pos[0] = gl_pos[0];
	pos[1] = gl_pos[1];
	pos[2] = gl_pos[2];		
	
	glTranslatef(pos[0], pos[1], pos[2]);
	
	if (gl_pos[3]) {
		glScalef(0.1, 0.1, 0.1);	
		pos[0] -= gl_dir[0];
		pos[1] -= gl_dir[1];
		pos[2] -= gl_dir[2];
	} else {
		glScalef(0.15, 0.15, 0.067);	
	}

	absv  = pos[0] * pos[0] + pos[1] * pos[1] + pos[2] * pos[2];
	absv2 = pos[0] * pos[0] + pos[1] * pos[1];
	if (absv > 0.0) {
		t = asin(sqrt(absv2/absv));
		if (pos[2] >= 0.0) {
			glRotatef(DEG(t), -pos[1], pos[0], 0.0);
		} else {
			glRotatef(180.0 - DEG(t), -pos[1], pos[0], 0.0);
		}		
	}
	light();

	glPopMatrix();
	glPopAttrib();
}

void GLLight::Print() {
	fprintf(stdout, "--- Light %d Properties ---\n", gllight);
	if (gl_pos[3] == 0.0) {
		fprintf(stdout, "This is flat light.\n");			
	} else {
		fprintf(stdout, "This is spot light.\n");				
	}
	fprintf(stdout, "Diffuse   r = %f g = %f b = %f a = %f\n", gl_dif[0], gl_dif[1], gl_dif[2], gl_dif[3]);
	fprintf(stdout, "Ambient   r = %f g = %f b = %f a = %f\n", gl_amb[0], gl_amb[1], gl_amb[2], gl_amb[3]);
	fprintf(stdout, "Specular  r = %f g = %f b = %f a = %f\n", gl_spe[0], gl_spe[1], gl_spe[2], gl_spe[3]);
	fprintf(stdout, "Position  x = %f y = %f z = %f\n", gl_pos[0], gl_pos[1], gl_pos[2]);
	fprintf(stdout, "Direction x = %f y = %f z = %f\n", gl_dir[0], gl_dir[1], gl_dir[2]);
	fprintf(stdout, "Attenuation constant = %f liener = %f quadratic = %f\n", gl_att[0], gl_att[1], gl_att[2]);
	fprintf(stdout, "Cut Off = %f  Exponent = %f\n", cutoff, exponent);
}

GLLight& GLLight::operator=(const GLLight &gllightin) {
	this->gl_dif[0] = gllightin.gl_dif[0];
	this->gl_dif[1] = gllightin.gl_dif[1];
	this->gl_dif[2] = gllightin.gl_dif[2];
	this->gl_dif[3] = gllightin.gl_dif[3];
	this->gl_amb[0] = gllightin.gl_amb[0];
	this->gl_amb[1] = gllightin.gl_amb[1];
	this->gl_amb[2] = gllightin.gl_amb[2];
	this->gl_amb[3] = gllightin.gl_amb[3];
	this->gl_spe[0] = gllightin.gl_spe[0];
	this->gl_spe[1] = gllightin.gl_spe[1];
	this->gl_spe[2] = gllightin.gl_spe[2];
	this->gl_spe[3] = gllightin.gl_spe[3];
	this->gl_pos[0] = gllightin.gl_pos[0];
	this->gl_pos[1] = gllightin.gl_pos[1];
	this->gl_pos[2] = gllightin.gl_pos[2];
	this->gl_pos[3] = gllightin.gl_pos[3];
	this->gl_att[0] = gllightin.gl_att[0];
	this->gl_att[1] = gllightin.gl_att[1];
	this->gl_att[2] = gllightin.gl_att[2];
	this->cutoff = gllightin.cutoff;
	this->exponent = gllightin.exponent;
	this->gllight = gllightin.gllight;
	return *this;
} 

bool GLLight::WasInit() {
	if (gllight == 0) {
		return false;	
	}
	return true;		
}

/* Cc(`) */
GLLights::GLLights() {
	lights = NULL;
}

GLLights::~GLLights() {
	Free();
}

int GLLights::Init() {
	if (WasInit()) {
		Free();
	}
	mnumber = GetMaxLights();
	lights = new GLLight[mnumber];
	cnumber = 0;
	int i;
	for (i = 0; i < mnumber; i++) {
		lights[i].Init(i);	
	}
	if (!WasInit()) {
		return false;	
	}
	return true;
}

void GLLights::Free() {
	if (WasInit()) {
		delete [] lights;
		lights = NULL;
		mnumber = 0;
		cnumber = 0;	
	}
}

int GLLights::GetMaxLights() {
	GLint glint[4];
	glGetIntegerv(GL_MAX_LIGHTS, glint);
	GLmes();
	return glint[0];
}

void GLLights::DefDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].DefDiffuse(r, g, b, a);
}

void GLLights::Diffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].Diffuse(r, g, b, a);
}

void GLLights::SetDiffuse() {
	lights[cnumber].SetDiffuse();
}

void GLLights::DefDiffuse(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefDiffuse(r, g, b, a);	
}

void GLLights::Diffuse(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Diffuse(r, g, b, a);	
}

void GLLights::SetDiffuse(int number) {
	Choice(number);
	SetDiffuse();
}
	
void GLLights::DefAmbient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].DefAmbient(r, g, b, a);
}

void GLLights::Ambient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].Ambient(r, g, b, a);
}

void GLLights::SetAmbient() {
	lights[cnumber].SetAmbient();
}

void GLLights::DefAmbient(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefAmbient(r, g, b, a);	
}

void GLLights::Ambient(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Ambient(r, g, b, a);	
}

void GLLights::SetAmbient(int number) {
	Choice(number);
	SetAmbient();
}

void GLLights::DefSpecular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].DefSpecular(r, g, b, a);
}

void GLLights::Specular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	lights[cnumber].Specular(r, g, b, a);
}

void GLLights::SetSpecular() {
	lights[cnumber].SetSpecular();
}

void GLLights::DefSpecular(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefSpecular(r, g, b, a);	
}

void GLLights::Specular(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Specular(r, g, b, a);	
}

void GLLights::SetSpecular(int number) {
	Choice(number);
	SetSpecular();
}

void GLLights::DefPosition(GLfloat x, GLfloat y, GLfloat z) {
	lights[cnumber].DefPosition(x, y, z);
}

void GLLights::Position(GLfloat x, GLfloat y, GLfloat z) {
	lights[cnumber].Position(x, y, z);
}

void GLLights::SetPosition() {
	lights[cnumber].SetPosition();
}

void GLLights::DefPosition(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	DefPosition(x, y, z);	
}

void GLLights::Position(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	Position(x, y, z);	
}

void GLLights::SetPosition(int number) {
	Choice(number);
	SetPosition();
}

void GLLights::DefDirection(GLfloat x, GLfloat y, GLfloat z) {
	lights[cnumber].DefDirection(x, y, z);
}

void GLLights::Direction(GLfloat x, GLfloat y, GLfloat z) {
	lights[cnumber].Direction(x, y, z);
}

void GLLights::SetDirection() {
	lights[cnumber].SetDirection();
}

void GLLights::DefDirection(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	DefDirection(x, y, z);	
}

void GLLights::Direction(int number, GLfloat x, GLfloat y, GLfloat z) {
	Choice(number);
	Direction(x, y, z);	
}

void GLLights::SetDirection(int number) {
	Choice(number);
	SetDirection();
}

void GLLights::DefAttenuation(GLfloat c, GLfloat l, GLfloat q) {
	lights[cnumber].DefAttenuation(c, l, q);
}

void GLLights::Attenuation(GLfloat c, GLfloat l, GLfloat q) {
	lights[cnumber].Attenuation(c, l, q);
}

void GLLights::SetAttenuation() {
	lights[cnumber].SetAttenuation();
}

void GLLights::DefAttenuation(int number, GLfloat c, GLfloat l, GLfloat q) {
	Choice(number);
	DefAttenuation(c, l, q);	
}

void GLLights::Attenuation(int number, GLfloat c, GLfloat l, GLfloat q) {
	Choice(number);
	Attenuation(c, l, q);	
}

void GLLights::DefCutOff(GLfloat c) {
	lights[cnumber].DefCutOff(c);
}

void GLLights::CutOff(GLfloat c) {
	lights[cnumber].CutOff(c);
}

void GLLights::SetCutOff() {
	lights[cnumber].SetCutOff();
}

void GLLights::DefCutOff(int number, GLfloat c) {
	Choice(number);
	DefCutOff(c);	
}

void GLLights::CutOff(int number, GLfloat c) {
	Choice(number);
	CutOff(c);	
}

void GLLights::SetCutOff(int number) {
	Choice(number);
	SetCutOff();
}

void GLLights::DefExponent(GLfloat e) {
	lights[cnumber].DefExponent(e);
}

void GLLights::Exponent(GLfloat e) {
	lights[cnumber].Exponent(e);
}

void GLLights::SetExponent() {
	lights[cnumber].SetExponent();
}

void GLLights::DefExponent(int number, GLfloat e) {
	Choice(number);
	DefExponent(e);	
}

void GLLights::Exponent(int number, GLfloat e) {
	Choice(number);
	Exponent(e);	
}

void GLLights::SetExponent(int number) {
	Choice(number);
	SetExponent();
}

void GLLights::FlatLight() {
	lights[cnumber].FlatLight();
}

bool GLLights::IsFlatLight() {
	return lights[cnumber].IsFlatLight();
}

void GLLights::FlatLight(int number) {
	Choice(number);
	FlatLight();
}

bool GLLights::IsFlatLight(int number) {
	Choice(number);
	return IsFlatLight();
}

void GLLights::SpotLight() {
	lights[cnumber].SpotLight();
}

bool GLLights::IsSpotLight() {
	return lights[cnumber].IsSpotLight();
}

void GLLights::SpotLight(int number) {
	Choice(number);
	SpotLight();
}

bool GLLights::IsSpotLight(int number) {
	Choice(number);
	return IsSpotLight();
}

void GLLights::DefModelDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	gl_dif[0] = r;
	gl_dif[1] = g;
	gl_dif[2] = b;
	gl_dif[3] = a;	
}

void GLLights::ModelDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefModelDiffuse(r, g, b, a);
	SetModelDiffuse();
}

void GLLights::SetModelDiffuse() {
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, gl_dif);
}

void GLLights::LocalViewOn() {
	GLfloat glfloat = 1.0;
	glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, glfloat);	
}

void GLLights::LocalViewOff() {
	GLfloat glfloat = 0.0;
	glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, glfloat);	
}

void GLLights::SetModel() {
	SetModelDiffuse();
}

void GLLights::Set() {
	lights[cnumber].Set();
}	
	
void GLLights::Set(int number) {
	Choice(number);
	Set();
}

void GLLights::Get() {
	lights[cnumber].Get();
}	
	
void GLLights::Get(int number) {
	Choice(number);
	Get();
}

void GLLights::On() {
	lights[cnumber].On();
}	
	
void GLLights::On(int number) {
	Choice(number);
	On();
}

void GLLights::Off() {
	lights[cnumber].Off();
}	
	
void GLLights::Off(int number) {
	Choice(number);
	Off();
}

bool GLLights::IsOn() {
	return lights[cnumber].IsOn();
}

bool GLLights::IsOn(int number) {
	Choice(number);
	return IsOn();
}

void GLLights::Show() {
	lights[cnumber].Show();
}

void GLLights::Show(int number) {
	Choice(number);
	Show();
}


void GLLights::Enable() {
	glEnable(GL_LIGHTING);	
}

void GLLights::Disable() {
	glDisable(GL_LIGHTING);
}	

bool GLLights::IsEnable() {
	return glIsEnabled(GL_LIGHTING);
}

void GLLights::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

int GLLights::Count() {
	return mnumber;
}


GLLight& GLLights::operator[](int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
		return lights[cnumber];
	} else {
		return lights[0];
	}
}

bool GLLights::WasInit() {
	if (lights == NULL) {
		return false;	
	}
	return true;	
}
