#include "glmaterial.h"

/* }eA */
GLMaterial::GLMaterial() {
	DefDiffuse();
	DefAmbient();
	DefSpecular();
	DefEmission();
	DefShininess();
}

GLMaterial::~GLMaterial() {
}

void GLMaterial::DefDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_dif[0] = r;}
	if (g >= 0.0) {gl_dif[1] = g;}
	if (b >= 0.0) {gl_dif[2] = b;}
	if (a >= 0.0) {gl_dif[3] = a;}
}

void GLMaterial::Diffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefDiffuse(r, g, b, a);
	SetDiffuse();
}

void GLMaterial::SetDiffuse() {
	glMaterialfv(GL_FRONT, GL_DIFFUSE, gl_dif);
}

void GLMaterial::DefAmbient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_amb[0] = r;}
	if (g >= 0.0) {gl_amb[1] = g;}
	if (b >= 0.0) {gl_amb[2] = b;}
	if (a >= 0.0) {gl_amb[3] = a;}
}

void GLMaterial::Ambient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefAmbient(r, g, b, a);
	SetAmbient();
}

void GLMaterial::SetAmbient() {
	glMaterialfv(GL_FRONT, GL_AMBIENT, gl_amb);
}

void GLMaterial::DefSpecular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_spe[0] = r;}
	if (g >= 0.0) {gl_spe[1] = g;}
	if (b >= 0.0) {gl_spe[2] = b;}
	if (a >= 0.0) {gl_spe[3] = a;}
}

void GLMaterial::Specular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefSpecular(r, g, b, a);
	SetSpecular();
}

void GLMaterial::SetSpecular() {
	glMaterialfv(GL_FRONT, GL_SPECULAR, gl_spe);
}

void GLMaterial::DefEmission(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	if (r >= 0.0) {gl_emi[0] = r;}
	if (g >= 0.0) {gl_emi[1] = g;}
	if (b >= 0.0) {gl_emi[2] = b;}
	if (a >= 0.0) {gl_emi[3] = a;}
}

void GLMaterial::Emission(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	DefEmission(r, g, b, a);
	SetEmission();
}

void GLMaterial::SetEmission() {
	glMaterialfv(GL_FRONT, GL_EMISSION, gl_emi);
}

void GLMaterial::DefShininess(GLfloat p) {
	if (p >= 0.0) {gl_shi = p;}
}

void GLMaterial::Shininess(GLfloat p) {
	DefShininess(p);
	SetShininess();
}

void GLMaterial::SetShininess() {
	glMaterialf(GL_FRONT, GL_SHININESS, gl_shi);
}

void GLMaterial::Get() {
	GLfloat glfloat4[4];

	glGetMaterialfv(GL_FRONT, GL_DIFFUSE, glfloat4);
	DefDiffuse(glfloat4[0], glfloat4[1], glfloat4[2], glfloat4[3]);

	glGetMaterialfv(GL_FRONT, GL_AMBIENT, glfloat4);
	DefAmbient(glfloat4[0], glfloat4[1], glfloat4[2], glfloat4[3]);

	glGetMaterialfv(GL_FRONT, GL_SPECULAR, glfloat4);
	DefSpecular(glfloat4[0], glfloat4[1], glfloat4[2], glfloat4[3]);

	glGetMaterialfv(GL_FRONT, GL_EMISSION, glfloat4);
	DefEmission(glfloat4[0], glfloat4[1], glfloat4[2], glfloat4[3]);

	glGetMaterialfv(GL_FRONT, GL_SHININESS, glfloat4);
	DefShininess(glfloat4[0]);
}

void GLMaterial::Set() {
	SetDiffuse();
	SetAmbient();
	SetEmission();
	SetSpecular();
	SetShininess();
}

GLMaterial& GLMaterial::operator=(const GLMaterial &glmat) {
	this->gl_dif[0] = glmat.gl_dif[0];
	this->gl_dif[1] = glmat.gl_dif[1];
	this->gl_dif[2] = glmat.gl_dif[2];
	this->gl_dif[3] = glmat.gl_dif[3];
	this->gl_amb[0] = glmat.gl_amb[0];
	this->gl_amb[1] = glmat.gl_amb[1];
	this->gl_amb[2] = glmat.gl_amb[2];
	this->gl_amb[3] = glmat.gl_amb[3];
	this->gl_spe[0] = glmat.gl_spe[0];
	this->gl_spe[1] = glmat.gl_spe[1];
	this->gl_spe[2] = glmat.gl_spe[2];
	this->gl_spe[3] = glmat.gl_spe[3];
	this->gl_emi[0] = glmat.gl_emi[0];
	this->gl_emi[1] = glmat.gl_emi[1];
	this->gl_emi[2] = glmat.gl_emi[2];
	this->gl_emi[3] = glmat.gl_emi[3];
	this->gl_shi = glmat.gl_shi;
	return *this;
} 

void GLMaterial::Print() {
	fprintf(stdout, "--- Material Properties ---\n");
	fprintf(stdout, "Diffuse   r = %f g = %f b = %f a = %f\n", gl_dif[0], gl_dif[1], gl_dif[2], gl_dif[3]);
	fprintf(stdout, "Ambient   r = %f g = %f b = %f a = %f\n", gl_amb[0], gl_amb[1], gl_amb[2], gl_amb[3]);
	fprintf(stdout, "Specular  r = %f g = %f b = %f a = %f\n", gl_spe[0], gl_spe[1], gl_spe[2], gl_spe[3]);
	fprintf(stdout, "Emission  r = %f g = %f b = %f a = %f\n", gl_emi[0], gl_emi[1], gl_emi[2], gl_emi[3]);
	fprintf(stdout, "Shininess = %f\n", gl_shi);
}

/* }eAY */
GLMaterials::GLMaterials() {
	materials = NULL;
}


GLMaterials::GLMaterials(int number) {
	Init(number);
}

GLMaterials::~GLMaterials() {
	Free();
}

int GLMaterials::Init(int number) {
	if (WasInit()) {
		Free();
	}
	materials = new GLMaterial[number];
	mnumber = number;
	cnumber = 0;
	if (!WasInit()) {
		return false;	
	}
	return true;
}

void GLMaterials::Free() {
	if (WasInit()) {
		delete [] materials;
		materials = NULL;
		mnumber = 0;
		cnumber = 0;
	}
}

void GLMaterials::DefDiffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].DefDiffuse(r, g, b, a);
}

void GLMaterials::Diffuse(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].Diffuse(r, g, b, a);
}

void GLMaterials::SetDiffuse() {
	materials[cnumber].SetDiffuse();
}

void GLMaterials::DefDiffuse(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefDiffuse(r, g, b, a);	
}

void GLMaterials::Diffuse(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Diffuse(r, g, b, a);	
}

void GLMaterials::SetDiffuse(int number) {
	Choice(number);
	SetDiffuse();
}
	
void GLMaterials::DefAmbient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].DefAmbient(r, g, b, a);
}

void GLMaterials::Ambient(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].Ambient(r, g, b, a);
}

void GLMaterials::SetAmbient() {
	materials[cnumber].SetAmbient();
}

void GLMaterials::DefAmbient(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefAmbient(r, g, b, a);	
}

void GLMaterials::Ambient(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Ambient(r, g, b, a);	
}

void GLMaterials::SetAmbient(int number) {
	Choice(number);
	SetAmbient();
}

void GLMaterials::DefSpecular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].DefSpecular(r, g, b, a);
}

void GLMaterials::Specular(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].Specular(r, g, b, a);
}

void GLMaterials::SetSpecular() {
	materials[cnumber].SetSpecular();
}

void GLMaterials::DefSpecular(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefSpecular(r, g, b, a);	
}

void GLMaterials::Specular(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Specular(r, g, b, a);	
}

void GLMaterials::SetSpecular(int number) {
	Choice(number);
	SetSpecular();
}

void GLMaterials::DefEmission(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].DefEmission(r, g, b, a);
}

void GLMaterials::Emission(GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	materials[cnumber].Emission(r, g, b, a);
}

void GLMaterials::SetEmission() {
	materials[cnumber].SetEmission();
}

void GLMaterials::DefEmission(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	DefEmission(r, g, b, a);	
}

void GLMaterials::Emission(int number, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
	Choice(number);
	Emission(r, g, b, a);	
}

void GLMaterials::SetEmission(int number) {
	Choice(number);
	SetEmission();
}

void GLMaterials::DefShininess(GLfloat p) {
	materials[cnumber].DefShininess(p);
}

void GLMaterials::Shininess(GLfloat p) {
	materials[cnumber].Shininess(p);
}

void GLMaterials::SetShininess() {
	materials[cnumber].SetShininess();
}

void GLMaterials::DefShininess(int number, GLfloat p) {
	Choice(number);
	DefShininess(p);	
}

void GLMaterials::Shininess(int number, GLfloat p) {
	Choice(number);
	Shininess(p);	
}

void GLMaterials::SetShininess(int number) {
	Choice(number);
	SetShininess();
}

void GLMaterials::Get() {
	materials[cnumber].Get();
}	
	
void GLMaterials::Get(int number) {
	Choice(number);
	Get();
}

void GLMaterials::Set() {
	materials[cnumber].Set();
}	
	
void GLMaterials::Set(int number) {
	Choice(number);
	Set();
}
	
void GLMaterials::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

GLMaterial& GLMaterials::operator[](int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
		return materials[cnumber];
	} else {
		return materials[0];
	}
}

void GLMaterials::Add(int number) {
	if (number == 65536) {
		number = mnumber;	
	}
	int i, onumber;
	onumber = mnumber;
	GLMaterials buf;
	buf.Init(mnumber);
	for (i = 0; i < mnumber; i++) {
		buf[i] = materials[i];
	}
	Free();
	Init(onumber + 1);
	for (i = 0; i < onumber; i++) {
		if (i < number) {
			materials[i] = buf[i];
		} else {
			materials[i+1] = buf[i];		
		}
	}
}

void GLMaterials::Del(int number) {
	if (number == 65536) {
		number = mnumber;	
	}
	int i, onumber;
	onumber = mnumber;
	GLMaterials buf;
	buf.Init(mnumber);
	for (i = 0; i < mnumber; i++) {
		buf[i] = materials[i];
	}
	Free();
	Init(onumber - 1);
	for (i = 0; i < mnumber; i++) {
		if (i < number) {
			materials[i] = buf[i];
		} else {
			materials[i] = buf[i+1];		
		}
	}
}

int GLMaterials::Count() {
	return mnumber;
}

bool GLMaterials::WasInit() {
	if (materials == NULL) {
		return false;	
	}
	return true;	
}
