#include "sttf.h"
#include "gldlist.h"
#include "glpolygonize.h"
#include "glptext.h"

GLPText::GLPText() {
	wasinit = false;
	mnumber = 0;
	cnumber = 0;
}

GLPText::~GLPText() {
	Close();
}

int GLPText::Init(GLPolygonize *glp_in, STTF *ttf_in) {
	if (WasInit()) {
		Close();
	}
	glp = glp_in;
	ttf = ttf_in;
	alphabet.Generate(CHAR_END - CHAR_START + 1);
	surface.Generate(MAX_GLDLISTS);
	if (!alphabet.WasGenerate() && !surface.WasGenerate()) {
		return SDL_FAILURE;	
	}
/* At@xbg̃fBXvCXg̍쐬 */
	int i;
	for (i = 0; i <= CHAR_END - CHAR_START; i++) {
		alphabet.NewList(i);	
		glp->Alphabet(CHAR_START + i);
		alphabet.EndList();	
//		fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", CHAR_START + i);
	}
	camera.DefPosition(0.0, 0.0, (float)YSSIZE * 1.732);
	color.DefDiffuse(1.0, 1.0, 1.0);
	light_save.Init(0);	
	light.Init(0);	
	
	wasinit = true;
	return SDL_SUCCESS;	
}

void GLPText::Close() {
	if (WasInit()) {
		alphabet.Delete();
		surface.Delete();
		wasinit = false;
	}
}

void GLPText::Text(char *str) {
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			Char(str[i]);
			glTranslatef(0.5, 0.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2(str+i);		
				glTranslatef(1.0, 0.0, 0.0);
				i++;
			}
		}
	}
}

void GLPText::TextVertical(char *str) {
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			glTranslatef(0.25, 0.0, 0.0);
			Char(str[i]);
			glTranslatef(-0.25, -1.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2(str+i);		
				glTranslatef(0.0, -1.0, 0.0);
				i++;
			}
		}
	}
}

void GLPText::Char(char c) {
	alphabet.CallList(c - CHAR_START);
}

void GLPText::Char2(char *str) {
	char c[3];
	int i, call_flag = 1;
	c[0] = str[0];
	c[1] = str[1];
	c[2] = '\0';
	for (i = 0; i < mnumber; i++) {
		if (c[0] == code2[i][0] && c[1] == code2[i][1]) {
			surface.CallList(i);
			call_flag = 0;
			break;			
		}
	}
	
	if (call_flag) {
		if (mnumber < MAX_GLDLISTS) {
			code2[mnumber][0] = c[0];
			code2[mnumber][1] = c[1];
			code2[mnumber][2] = c[2];
			surface.NewList(mnumber, GL_COMPILE_AND_EXECUTE);	
			glp->Surface(ttf->Render(c));
			surface.EndList();
#ifdef DEBUG
			fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", mnumber);
#endif
			mnumber++;
		} else {
			code2[cnumber][0] = c[0];
			code2[cnumber][1] = c[1];
			code2[cnumber][2] = c[2];
			surface.NewList(cnumber, GL_COMPILE_AND_EXECUTE);	
			glp->Surface(ttf->Render(c));
			surface.EndList();
#ifdef DEBUG
			fprintf(stdout, "SUCCESS : Make Alphabet Font List  No. %d\n", cnumber);
#endif
            cnumber++;
			if (cnumber > MAX_GLDLISTS) {
				cnumber = 0;					
			}  
		}	
	}
}

void GLPText::MakeChar2(char *str) {
	char c[3];
	int i, call_flag = 1;
	c[0] = str[0];
	c[1] = str[1];
	c[2] = '\0';
	for (i = 0; i < mnumber; i++) {
		if (c[0] == code2[i][0] && c[1] == code2[i][1]) {
			surface.CallList(i);
			call_flag = 0;
			break;			
		}
	}
	
	if (call_flag) {
		if (mnumber < MAX_GLDLISTS) {
			code2[mnumber][0] = c[0];
			code2[mnumber][1] = c[1];
			code2[mnumber][2] = c[2];
			surface.NewList(mnumber);	
			glp->Surface(ttf->Render(c));
			surface.EndList();
#ifdef DEBUG
			fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", mnumber);
#endif
			mnumber++;
		} else {
			code2[cnumber][0] = c[0];
			code2[cnumber][1] = c[1];
			code2[cnumber][2] = c[2];
			surface.NewList(cnumber);	
			glp->Surface(ttf->Render(c));
			surface.EndList();
#ifdef DEBUG
			fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", cnumber);
#endif
            cnumber++;
			if (cnumber > MAX_GLDLISTS) {
				cnumber = 0;					
			}  
		}	
	}
}
	
void GLPText::MakeText(char *str) {
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (!isascii(str[i])) {
			if (str[i+1] == '\0') {
				break;		
			} else {
				MakeChar2(str+i);		
				i++;
			}
		}
	}
}

void GLPText::ClearAlphabet() {
	alphabet.Delete();
}

void GLPText::MakeAlphabetFromTTF() {
	int i;
	char ab[2];
	ab[1] = '\0';
/*	if (alphabet.WasGenerate()) {
		alphabet.Delete();
	}
	alphabet.Generate(CHAR_END - CHAR_START + 1);*/
	for (i = MAX_SYMBOL; i <= CHAR_END - CHAR_START; i++) {
		if (alphabet.WasGenerate(i)) {
			alphabet.Delete(i);		
		}
		alphabet.NewList(i);
		ab[0] = CHAR_START + i;	
		glp->Surface(ttf->Render(ab));
		alphabet.EndList();	
//		fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", CHAR_START + i);
	}
}

void GLPText::Clear() {
	surface.Delete();
	surface.Generate(MAX_GLDLISTS);
	mnumber = 0;
	cnumber = 0;
	int i;
	for (i = 0; i < MAX_GLDLISTS; i++) {
		code2[i][0] = '\0';
		code2[i][1] = '\0';
		code2[i][2] = '\0';
	}
}

void GLPText::Symbol(char symbol, float size, int xpos, int ypos) {
	static char text[2];
	text[0] = symbol;
	text[1] = '\0';
	Print(text, size, xpos, ypos);
}

void GLPText::Print(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -(float)strlen(str)*0.25;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - (float)strlen(str)*0.5) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -(float)strlen(str)*0.25;			
	}

	if (ypos) {
			y0 = -0.5;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (YSSIZE / size) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (YSSIZE / size - 1.0) * ((float)ypos/100.0);
		}
	} else {
			y0 = -0.5;			
	}
	glPushMatrix();
	glScalef(size, size, size);
	glTranslatef(x0, y0, 0.0);
	Text(str);
	glPopMatrix();
}

void GLPText::LongPrint(char *str, float size, int xpos, int ypos) {
	static char line[255];
	int i, charlen, linelen, linec = 0;
	float x0, y0;
	float xsize = (float)xpos/100.0, ysize = (float)ypos/100.0;
	charlen = strlen(str);
	linelen = (int)(2.0 * XSSIZE * xsize / size) * 2;
	if (linelen >= 255) {
		linelen = 255;	
	}
	if (linelen < 2) {
		linelen = 2;	
	}
	x0 =-XSSIZE * xsize / size;
	y0 = YSSIZE * ysize / size - 1.0;

	glPushMatrix();
	glScalef(size, size, size);
	for (i = 0; i< charlen;i ++) {
		if (isascii(str[i])) {
			line[linec++] = str[i];
		} else {
			if (linec > linelen-2) {
				line[linec++] = '\0';
				line[linec++] = '\0';
				i--;
			} else {
				line[linec++] = str[i++];
				line[linec++] = str[i];		
			}
		}

		if (linec > linelen-1) {
		 	line[linec] = '\0';
			linec = 0;

			glPushMatrix();
			glTranslatef(x0, y0, 0.0);
			Text(line);
			glPopMatrix();
			y0-=1.0;
		}
	}
	line[linec] = '\0';
	glTranslatef(x0, y0, 0.0);
	Text(line);

	glPopMatrix();
}


void GLPText::PrintVertical(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -0.5;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - 1.0) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -0.5;			
	}

	if (ypos) {
			y0 = (float)strlen(str)*0.25 - 1.0;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (-YSSIZE / size + 1.0) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (-YSSIZE / size - 1.0 + (float)strlen(str)*0.5) * ((float)ypos/100.0);
		}
	} else {
			y0 = (float)strlen(str)*0.25 - 1.0;			
	}
	glPushMatrix();
	glScalef(size, size, size);
	glTranslatef(x0, y0, 0.0);
	TextVertical(str);
	glPopMatrix();
}

void GLPText::Message(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -(float)strlen(str)*0.25;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - (float)strlen(str)*0.5) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -(float)strlen(str)*0.25;			
	}

	if (ypos) {
			y0 = -0.5;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (YSSIZE / size) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (YSSIZE / size - 1.0) * ((float)ypos/100.0);
		}
	} else {
			y0 = -0.5;			
	}
	MessageEnter();
	glScalef(size, size, size);
	glTranslatef(x0, y0, 0.0);
	Text(str);
	MessageLeave();
}

void GLPText::LongMessage(char *str, float size, int xpos, int ypos) {
	char line[255];
	int i, charlen, linelen, linec = 0;
	float x0, y0;
	float xsize = (float)xpos/100.0, ysize = (float)ypos/100.0;
	charlen = strlen(str);
	linelen = (int)(2.0 * XSSIZE * xsize / size) * 2;
	if (linelen >= 255) {
		linelen = 255;	
	}
	if (linelen < 2) {
		linelen = 2;	
	}
	x0 =-XSSIZE * xsize / size;
	y0 = YSSIZE * ysize / size - 1.0;

	MessageEnter();
	glScalef(size, size, size);
	for (i = 0; i< charlen;i ++) {
		if (isascii(str[i])) {
			line[linec++] = str[i];
		} else {
			if (linec > linelen-2) {
				line[linec++] = '\0';
				line[linec++] = '\0';
				i--;
			} else {
				line[linec++] = str[i++];
				line[linec++] = str[i];		
			}
		}
		if (linec > linelen-1) {
		 	line[linec] = '\0';
			linec = 0;
			glPushMatrix();
			glTranslatef(x0, y0, 0.0);
			Text(line);
			glPopMatrix();
			y0-=1.0;
		}
	}
	line[linec] = '\0';
	glTranslatef(x0, y0, 0.0);
	Text(line);

	MessageLeave();
}

void GLPText::Pause(char *str, float size, int xpos, int ypos) {
	SEvent event;
	do {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		Message(str, size);
		SDL_GL_SwapBuffers();
		event.Wait();
	} while(event.IsContinue());
	if (event.IsQuit()) {
		exit(0);		
	}
}

void GLPText::MessageEnter() {
//	SDL_Surface *screen = SDL_GetVideoSurface();

	light_save.Get();
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);

//	glViewport(0, 0, screen->w, screen->h);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

//	if (screen->h > 0) {
//		float ratio = (float) screen->w / (float) screen->h;
		gluPerspective(60.0, 1.33333, 0.05, 20.0);
//		glFrustum(-XSSIZE, XSSIZE, -YSSIZE, YSSIZE, 0.01, 10.0);
//		glOrtho(-XSSIZE, XSSIZE, -YSSIZE, YSSIZE, 0.01, 10.0);
//	}

    glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
    glLoadIdentity();

	glEnable(GL_LIGHTING);
	light.On();	
	light.Set();
	camera.Set();
	color.Set();
}

void GLPText::MessageLeave() {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	light_save.Set();
	glPopAttrib();
}

bool GLPText::WasInit() {
	return wasinit;
}
