#include "glsandbag.h"

void GLSandbag::Make(int partition) {
	x = -0.5;
	y =  0.5;
	z =  2.0;
	a =  0.0;
	b =  0.0;
	g = 45.0;
	da = 0.0;
	db = 0.0;
	aa = 0.0;
	ab = 0.0;
	count = 0;
	glsandbag.Generate();
	glsandbag.NewList();
	sandbag(partition);	
	glsandbag.EndList();
}

void GLSandbag::ShowShadow() {
	static GLfloat m[16] = {1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0,
							0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.005, 1.0};
	glPushMatrix();
	glMultMatrixf(m);
	glTranslatef(x, y, z);
	glRotatef(a, 1.0, 0.0, 0.0);
	glRotatef(b, 0.0, 1.0, 0.0);
	glDisable(GL_LIGHTING);
	glColor3f(0.0, 0.0, 0.0);
	glsandbag.CallList();		
	glEnable(GL_LIGHTING);
	glPopMatrix();
}

void GLSandbag::Show() {
	glPushMatrix();
	glTranslatef(x, y, z);
	glRotatef(a, 1.0, 0.0, 0.0);
	glRotatef(b, 0.0, 1.0, 0.0);
	glsandbag.CallList();
	glPopMatrix();
}

void GLSandbag::ShowHit() {
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glPushMatrix();
	glTranslatef(x, y, z);
	glRotatef(a, 1.0, 0.0, 0.0);
	glRotatef(b, 0.0, 1.0, 0.0);
	glColor3f(0.8, 0.8, 0.8);
	glsandbag.CallList();
	glPopMatrix();
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);				
}

void GLSandbag::CalcCoords() {
	x1 = x;
	y1 = y;
	z1 = z;
	
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(x, y, z);	
	glRotatef(a, 1.0, 0.0, 0.0);
	glRotatef(b, 0.0, 1.0, 0.0);
	matrix.Get();
	matrix.Calc(0.0, 0.0, -1.75);
	glPopMatrix();

	x2 = matrix.X();
	y2 = matrix.Y();
	z2 = matrix.Z();
}

void GLSandbag::Move() {
	da+= aa - sin(RAD(a));
	db+= ab - sin(RAD(b));
	da*= 0.98;
	db*= 0.98;
	a+= da;
	b+= db;
	count++;
	if (count > 5) {
		aa = 0.0;
		ab = 0.0;	
	}		
}

void GLSandbag::Hit(float px, float py, float damage) {
	float vx, vy, v;
	vx = px - x;
	vy = py - y;
	
	v = sqrt(vx*vx + vy*vy);

	aa =-damage * 0.0025 * vy/v;
	ab = damage * 0.0025 * vx/v;

	count = 0;	
}

bool GLSandbag::IsHit(float xp, float yp, float zp, float rp) {
	CalcCoords();
	if (IsHitPillar(xp, yp, zp, rp, x1, y1, z1, x2, y2, z2, 0.25)){
		return true;
	}
	return false;
}
