#include "gltexture.h"

/* OpenGLpT[tFCX */
GLSurface::GLSurface() {
	surface = NULL;	
}

GLSurface::~GLSurface() {
	Free();
}

int GLSurface::Create(int x, int y) {
	if (WasCreate()) {
		Free();
	}
/* OpenGLp̃T[tFCX */
	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, x, y, 32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* gCfBAp */
			0x000000FF, 
			0x0000FF00, 
			0x00FF0000, 
			0xFF000000
#else                              /* rbOCfBAp */
			0xFF000000,
			0x00FF0000, 
			0x0000FF00, 
			0x000000FF
#endif
	);
	if (!WasCreate()) {
		emes("GLSurface::Create");	
		return SDL_FAILURE;	
	}
//	smes("GLSurface::Create");	
	return SDL_SUCCESS;
}

int GLSurface::Create(SDL_Surface *image) {
	int w, h;
	SDL_Rect rect;
	Uint32 saved_flags;
	Uint8  saved_alpha;

/* ł߂2^n̒l߂ */
	w = power_of_two(image->w);
	h = power_of_two(image->h);

	Create(w, h);
	if (!WasCreate()) {
		return SDL_FAILURE;	
	}

/* At@̕ۑ */
	saved_flags = image->flags;
	saved_alpha = image->format->alpha;
	if (saved_flags & SDL_SRCALPHA) {
		SDL_SetAlpha(image, 0, 0);
	}

/* ͈͂w肵blit */
	rect.x = 0;
	rect.y = 0;
	rect.w = image->w;
	rect.h = image->h;
	SDL_BlitSurface(image, &rect, surface, &rect);

/* At@̉ */
	if (saved_flags & SDL_SRCALPHA) {
		SDL_SetAlpha(image, saved_flags, saved_alpha);
	} 
	return SDL_SUCCESS;
}

void GLSurface::Free() {
	if (WasCreate()) {
		SDL_FreeSurface(surface);
		surface = NULL;
//		smes("GLSurface::Free");	
	} else {
		wmes("GLSurface::Free");
	}
}

int GLSurface::power_of_two(int input) {
	int value = 1;
	while ( value < input ) {
		value <<= 1;
	}
	return value;
}

SDL_Surface* GLSurface::Get() {
	return surface;
}

void* GLSurface::Pixel() {
	return surface->pixels;
}

int GLSurface::Width() {
	return surface->w;
}

int GLSurface::Height() {
	return surface->h;
}

void GLSurface::Set(SDL_Surface* set_surf) {
	surface = set_surf;
}

void GLSurface::Cls() {
	SDL_FillRect(surface, NULL, SDL_MapRGBA(surface->format, 0, 0, 0, 0));
}

bool GLSurface::WasCreate() {
	if (surface == NULL) {
		return false;
	}
	return true;
}

/* eNX` */
GLTexture::GLTexture() {
	texname = 0;
}

GLTexture::~GLTexture() {
	Delete();
}

int GLTexture::Generate(SDL_Surface *surface, int mag_linear, int min_linear) {
	if (WasGenerate()) {
		Delete();
	}
	glGenTextures(1, &texname);
	Bind();

	GLSurface glsurface;
	glsurface.Create(surface);
	w = glsurface.Width();
	h = glsurface.Height();
	w_ratio = surface->w / (float) w;
	h_ratio = surface->h / (float) h;
			
	if (mag_linear) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	}
	if (min_linear) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	}

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h,
		          0, GL_RGBA, GL_UNSIGNED_BYTE, glsurface.Pixel());

	if(!WasGenerate()) {
		emes("GLTexture::Generate");	
		GLmes();
		return SDL_FAILURE;	
	}
//	smes("GLTexture::Generate");	
	return SDL_SUCCESS;
}

void GLTexture::Delete() {
	if(WasGenerate()) {
		glDeleteTextures(1, &texname);
//		smes("GLTexture::Delete");	
	}
}

int GLTexture::Width() {
	return w;
}

int GLTexture::Height() {
	return h;
}	

float GLTexture::WidthRatio() {
	return w_ratio;
}

float GLTexture::HeightRatio() {
	return h_ratio;
}

void GLTexture::Bind() {
	glBindTexture(GL_TEXTURE_2D, texname);
}

bool GLTexture::WasGenerate() {
	if (glIsTexture(texname)) {
		return true;
	}
	return false;
}

/* eNX`Y(`) */
GLTextures::GLTextures() {
	textures = NULL;
	mnumber = 0;
}

GLTextures::~GLTextures() {
	Free();
}

int GLTextures::Init(int number) {
	if(WasInit()) {
		Free();
	}
	textures = new GLTexture[number];
	cnumber = 0;
	mnumber = number;
	if(!WasInit()) {
		emes("GLTextures::Init");		
		GLmes();
		return SDL_FAILURE;	
	}
	smes("GLTextures::Init");		
	return SDL_SUCCESS;
}

void GLTextures::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
	textures[cnumber].Bind();
}

void GLTextures::Free() {
	if (WasInit()) {
		delete [] textures;
		textures = NULL;
		mnumber = 0;
		smes("GLTextures::Free");	
	} else {
		wmes("GLTextures::Free");
	}
}

int GLTextures::Generate(SDL_Surface *surface) {
	return textures[cnumber].Generate(surface, mag_filter, min_filter);
}

int GLTextures::Generate(int number, SDL_Surface *surface) {
	Choice(number);
	return Generate(surface);
}

int GLTextures::Width() {
	return textures[cnumber].Width();	
}

int GLTextures::Width(int number) {
	Choice(number);
	return Width();
}

int GLTextures::Height() {	
	return textures[cnumber].Height();	
}

int GLTextures::Height(int number) {	
	Choice(number);
	return Height();
}

float GLTextures::WidthRatio() {
	return textures[cnumber].WidthRatio();	
}

float GLTextures::WidthRatio(int number) {
 	Choice(number);
	return WidthRatio();
}

float GLTextures::HeightRatio() {
	return textures[cnumber].HeightRatio();	
}

float GLTextures::HeightRatio(int number) {
 	Choice(number);
	return HeightRatio();
}

GLTexture& GLTextures::operator[](int number) {
	Choice(number);
	return textures[cnumber];
}

bool GLTextures::WasInit() {
	if (textures == NULL) {
		return false;
	}
	return true;
}
