#include "sttf.h"
#include "gltexture.h"
#include "glttext.h"

#ifdef USE_GFX_FONT
#include <SDL_gfxPrimitives_font.h>
#else
#include "alphabetfont.h"
#endif

GLTText::GLTText() {
	wasinit = false;
	mnumber = 0;
	cnumber = 0;
	ShadowOn();
}

GLTText::~GLTText() {
	Close();
}

int GLTText::Init(STTF *ttf_in) {
	if (WasInit()) {
		Close();
	}
	ttf = ttf_in;
	alphabet.Init(TCHAR_END - TCHAR_START + 1);
	surface.Init(MAX_GLTEXTURES);
	if (!alphabet.WasInit() && !surface.WasInit()) {
		return SDL_FAILURE;	
	}
/* At@xbg̃eNX`̍쐬 */
#ifdef USE_GFX_FONT
	SDL_Surface *buf = SDL_CreateRGBSurface(SDL_SWSURFACE, 8, 8, 8, 0, 0, 0, 0);
#else
	SDL_Surface *buf = SDL_CreateRGBSurface(SDL_SWSURFACE, 11, 11, 8, 0, 0, 0, 0);
#endif
	if( buf == NULL ) {
		return SDL_FAILURE;
	}
	buf->format->palette->colors[0].r = 0;
	buf->format->palette->colors[0].g = 0;
	buf->format->palette->colors[0].b = 0;
	buf->format->palette->colors[1].r = 255;
	buf->format->palette->colors[1].g = 255;
	buf->format->palette->colors[1].b = 255;
	SDL_SetColorKey(buf, SDL_SRCCOLORKEY, 0);
	
	int i;
	for (i = 0; i <= TCHAR_END - TCHAR_START; i++) {
		Render(TCHAR_START + i, buf);		
		alphabet[i].Generate(buf); 
//		fprintf(stdout, "SUCCESS : Make Alphabet Font List No. %d\n", CHAR_START + i);
	}
	SDL_FreeSurface(buf);

	camera.DefPosition(0.0, 0.0, (float)YSSIZE * 1.732);
	color.DefDiffuse(1.0, 1.0, 1.0);
	shadow_color.DefDiffuse(0.1, 0.1, 0.1);
	light_save.Init(0);	
	light.Init(0);	
	
	wasinit = true;
	return SDL_SUCCESS;	
}

void GLTText::Close() {
	if (WasInit()) {
		wasinit = false;
	}
}

void GLTText::Text(char *str) {
	glPushAttrib(GL_ENABLE_BIT);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	save_rendertype = ttf->rendertype;
	ttf->SetRender(RENDER_BLENDED);

	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			CharD(str[i]);
			glTranslatef(0.5, 0.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2D(str+i);		
				glTranslatef(1.0, 0.0, 0.0);
				i++;
			}
		}
	}
	ttf->SetRender(save_rendertype);
	glPopAttrib();
}

void GLTText::TextPitch(char *str, float pitch) {
	glPushAttrib(GL_ENABLE_BIT);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	save_rendertype = ttf->rendertype;
	ttf->SetRender(RENDER_BLENDED);

	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			CharD(str[i]);
			glTranslatef(pitch, 0.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2D(str+i);		
				glTranslatef(pitch*2.0, 0.0, 0.0);
				i++;
			}
		}
	}
	ttf->SetRender(save_rendertype);
	glPopAttrib();
}

void GLTText::TextSingle(char *str) {
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			Char(str[i]);
			glTranslatef(0.5, 0.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2(str+i);		
				glTranslatef(1.0, 0.0, 0.0);
				i++;
			}
		}
	}
}

void GLTText::TextVertical(char *str) {
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (isascii(str[i])) {
			glTranslatef(0.25, 0.0, 0.0);
			Char(str[i]);
			glTranslatef(-0.25, -1.0, 0.0);
		} else {
			if (str[i+1] == '\0') {
				break;		
			} else {
				Char2(str+i);		
				glTranslatef(0.0, -1.0, 0.0);
				i++;
			}
		}
	}
}

void GLTText::Char(char c) {
	float w, h;
	alphabet.Choice(c - TCHAR_START);
	w = alphabet.WidthRatio();
	h = alphabet.HeightRatio();

	glBegin(GL_QUADS);
	glNormal3f(0.0, 0.0, 1.0);
	glTexCoord2f(0.0,  h ); glVertex2f(0.0, 0.0);
	glTexCoord2f( w ,  h ); glVertex2f(0.5, 0.0);
	glTexCoord2f( w , 0.0); glVertex2f(0.5, 1.0);
	glTexCoord2f(0.0, 0.0); glVertex2f(0.0, 1.0);
	glEnd();
}

void GLTText::CharD(char c) {
	float w, h;
	alphabet.Choice(c - TCHAR_START);
	w = alphabet.WidthRatio();
	h = alphabet.HeightRatio();

	glBegin(GL_QUADS);
	glNormal3f(0.0, 0.0, 1.0);
	glTexCoord2f(0.0,  h ); glVertex3f(0.0, 0.0, 0.125);
	glTexCoord2f( w ,  h ); glVertex3f(1.0, 0.0, 0.125);
	glTexCoord2f( w , 0.0); glVertex3f(1.0, 1.0, 0.125);
	glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 1.0, 0.125);
	glNormal3f(0.0, 0.0, -1.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 1.0,-0.125);
	glTexCoord2f( w , 0.0 );glVertex3f(1.0, 1.0,-0.125);
	glTexCoord2f( w ,  h ); glVertex3f(1.0, 0.0,-0.125);
	glTexCoord2f(0.0,  h ); glVertex3f(0.0, 0.0,-0.125);
	glEnd();
}

void GLTText::Char2D(char *str) {
	char c[3];
	int i, call_flag = 1;
	c[0] = str[0];
	c[1] = str[1];
	c[2] = '\0';
	for (i = 0; i < mnumber; i++) {
		if (c[0] == code2[i][0] && c[1] == code2[i][1]) {
			CallTextureD(i);
			call_flag = 0;
			break;			
		}
	}
	
	if (call_flag) {
		if (mnumber < MAX_GLTEXTURES) {
			code2[mnumber][0] = c[0];
			code2[mnumber][1] = c[1];
			code2[mnumber][2] = c[2];
			MakeTexture(mnumber, ttf->Render(c));
			mnumber++;
		} else {
			code2[cnumber][0] = c[0];
			code2[cnumber][1] = c[1];
			code2[cnumber][2] = c[2];
			MakeTexture(cnumber, ttf->Render(c));
            cnumber++;
			if (cnumber > MAX_GLTEXTURES) {
				cnumber = 0;					
			}  
		}	
	}
}

void GLTText::Char2(char *str) {
	char c[3];
	int i, call_flag = 1;
	c[0] = str[0];
	c[1] = str[1];
	c[2] = '\0';
	for (i = 0; i < mnumber; i++) {
		if (c[0] == code2[i][0] && c[1] == code2[i][1]) {
			CallTexture(i);
			call_flag = 0;
			break;			
		}
	}
	
	if (call_flag) {
		if (mnumber < MAX_GLTEXTURES) {
			code2[mnumber][0] = c[0];
			code2[mnumber][1] = c[1];
			code2[mnumber][2] = c[2];
			MakeTexture(mnumber, ttf->Render(c));
			mnumber++;
		} else {
			code2[cnumber][0] = c[0];
			code2[cnumber][1] = c[1];
			code2[cnumber][2] = c[2];
			MakeTexture(cnumber, ttf->Render(c));
            cnumber++;
			if (cnumber > MAX_GLTEXTURES) {
				cnumber = 0;					
			}  
		}	
	}
}

void GLTText::MakeChar2(char *str) {
	char c[3];
	int i, call_flag = 1;
	c[0] = str[0];
	c[1] = str[1];
	c[2] = '\0';
	for (i = 0; i < mnumber; i++) {
		if (c[0] == code2[i][0] && c[1] == code2[i][1]) {
			CallTexture(i);
			call_flag = 0;
			break;			
		}
	}
	
	if (call_flag) {
		if (mnumber < MAX_GLTEXTURES) {
			code2[mnumber][0] = c[0];
			code2[mnumber][1] = c[1];
			code2[mnumber][2] = c[2];
			MakeTexture(mnumber, ttf->Render(c));
			mnumber++;
		} else {
			code2[cnumber][0] = c[0];
			code2[cnumber][1] = c[1];
			code2[cnumber][2] = c[2];
			MakeTexture(cnumber, ttf->Render(c));
            cnumber++;
			if (cnumber > MAX_GLTEXTURES) {
				cnumber = 0;					
			}  
		}	
	}
}
	
void GLTText::MakeText(char *str) {
	ttf->SetRender(RENDER_BLENDED);
	int i;
	for (i = 0; str[i] !='\0'; i++) {
		if (!isascii(str[i])) {
			if (str[i+1] == '\0') {
				break;		
			} else {
				MakeChar2(str+i);		
				i++;
			}
		}
	}
}

void GLTText::ClearAlphabet() {
	alphabet.Free();
}

void GLTText::MakeAlphabetFromTTF() {
	int i;
	char ab[2];
	ab[1] = '\0';
//	alphabet.Init(TCHAR_END - TCHAR_START + 1);
	save_rendertype = ttf->rendertype;
	ttf->SetRender(RENDER_BLENDED);
	SDL_Surface *render;
	for (i = 31; i <= TCHAR_END - TCHAR_START; i++) {
		ab[0] = TCHAR_START + i;	
		render = ttf->Render(ab);
		if (alphabet[i].WasGenerate()) {
			alphabet[i].Delete();
		}
		alphabet[i].Generate(render);	
		SDL_FreeSurface(render);
	}
	ttf->SetRender(save_rendertype);
}

void GLTText::Clear() {
	surface.Free();
	surface.Init(MAX_GLTEXTURES);
	mnumber = 0;
	cnumber = 0;
	int i;
	for (i = 0; i < MAX_GLTEXTURES; i++) {
		code2[i][0] = '\0';
		code2[i][1] = '\0';
		code2[i][2] = '\0';
	}
}

void GLTText::Symbol(char symbol, float size, int xpos, int ypos) {
	static char text[2];
	text[0] = symbol;
	text[1] = '\0';
	Print(text, size, xpos, ypos);
}

void GLTText::Print(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -(float)strlen(str)*0.25;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - (float)strlen(str)*0.5) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -(float)strlen(str)*0.25;			
	}

	if (ypos) {
			y0 = -0.5;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (YSSIZE / size) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (YSSIZE / size - 1.0) * ((float)ypos/100.0);
		}
	} else {
			y0 = -0.5;			
	}
	glPushMatrix();
	glScalef(size, size, size);

	if (with_shadow) {
		color_save.Get();
		shadow_color.Set();
		glPushMatrix();
		glTranslatef(x0 + 0.1, y0 - 0.1, 0.0);
		TextSingle(str);
		glPopMatrix();
		color_save.Set();
	}

	glTranslatef(x0, y0, 0.0);
	TextSingle(str);
	glPopMatrix();
}


void GLTText::LongPrint(char *str, float size, int xpos, int ypos) {
	char line[255];
	int i, charlen, linelen, linec = 0;
	float x0, y0;
	float xsize = (float)xpos/100.0, ysize = (float)ypos/100.0;
	charlen = strlen(str);
	linelen = (int)(2.0 * XSSIZE * xsize / size) * 2;
	if (linelen >= 255) {
		linelen = 255;	
	}
	if (linelen < 2) {
		linelen = 2;	
	}
	x0 =-XSSIZE * xsize / size;
	y0 = YSSIZE * ysize / size - 1.0;

	glPushMatrix();
	glScalef(size, size, size);
	for (i = 0; i< charlen;i ++) {
		if (isascii(str[i])) {
			line[linec++] = str[i];
		} else {
			if (linec > linelen-2) {
				line[linec++] = '\0';
				line[linec++] = '\0';
				i--;
			} else {
				line[linec++] = str[i++];
				line[linec++] = str[i];		
			}
		}
		if (linec > linelen-1) {
			static const char *unallowable_head[] = {"B", "A", "v", ". ", ", "};
			int j;
			for (j = 0; j < 5; j++) {
				if (isascii(str[i+1])) {
					if (str[i+1] == unallowable_head[j][0]) {
						line[linec++] = str[++i];
						break;		
					}
				} else {
					if (str[i+1] == unallowable_head[j][0] && str[i+2] == unallowable_head[j][1]) {
						line[linec++] = str[++i];
						line[linec++] = str[++i];
						line[linec++] = '\0';
						line[linec++] = '\0';
						break;		
					}
				}
			}

		 	line[linec] = '\0';
			linec = 0;

			if (with_shadow) {
				color_save.Get();
				shadow_color.Set();
				glPushMatrix();
				glTranslatef(x0 + 0.1, y0 - 0.1, 0.0);
				TextSingle(line);
				glPopMatrix();
				color_save.Set();
			}

			glPushMatrix();
			glTranslatef(x0, y0, 0.0);
			TextSingle(line);
			glPopMatrix();
			y0-=1.0;
		}
	}
	line[linec] = '\0';

	if (with_shadow) {
		color_save.Get();
		shadow_color.Set();
		glPushMatrix();
		glTranslatef(x0 + 0.1, y0 - 0.1, 0.0);
		TextSingle(line);
		glPopMatrix();
		color_save.Set();
	}

	glTranslatef(x0, y0, 0.0);
	TextSingle(line);

	glPopMatrix();
}


void GLTText::PrintVertical(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -0.5;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - 1.0) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -0.5;			
	}

	if (ypos) {
			y0 = (float)strlen(str)*0.25 - 1.0;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (-YSSIZE / size + 1.0) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (-YSSIZE / size - 1.0 + (float)strlen(str)*0.5) * ((float)ypos/100.0);
		}
	} else {
			y0 = (float)strlen(str)*0.25 - 1.0;			
	}

	glPushMatrix();
	glScalef(size, size, size);

	if (with_shadow) {
		color_save.Get();
		shadow_color.Set();
		glPushMatrix();
		glTranslatef(x0 + 0.1, y0 - 0.1, 0.0);
		TextVertical(str);
		glPopMatrix();
		color_save.Set();
	}

	glTranslatef(x0, y0, 0.0);
	TextVertical(str);
	glPopMatrix();
}

void GLTText::Message(char *str, float size, int xpos, int ypos) {
	float x0, y0;
	if (xpos) {
		x0 = -(float)strlen(str)*0.25;			
		if (xpos < 0) {
			x0 *= ((float)(100 + xpos)/100.0);			
			x0 += (XSSIZE / size) * ((float)xpos/100.0);		
		} else {
			x0 *= ((float)(100 - xpos)/100.0);			
			x0 += (XSSIZE / size - (float)strlen(str)*0.5) * ((float)xpos/100.0);		
		}
	} else {
		x0 = -(float)strlen(str)*0.25;			
	}

	if (ypos) {
			y0 = -0.5;			
		if (ypos < 0) {
			y0 *= ((float)(100 + ypos)/100.0);
			y0 += (YSSIZE / size) * ((float)ypos/100.0);
		} else {
			y0 *= ((float)(100 - ypos)/100.0);			
			y0 += (YSSIZE / size - 1.0) * ((float)ypos/100.0);
		}
	} else {
			y0 = -0.5;			
	}
	MessageEnter();
	glScalef(size, size, size);
	glTranslatef(x0, y0, 0.0);
	TextSingle(str);
	MessageLeave();
}

void GLTText::LongMessage(char *str, float size, int xpos, int ypos) {
	char line[255];
	int i, charlen, linelen, linec = 0;
	float x0, y0;
	float xsize = (float)xpos/100.0, ysize = (float)ypos/100.0;
	charlen = strlen(str);
	linelen = (int)(2.0 * XSSIZE * xsize / size) * 2;
	if (linelen >= 255) {
		linelen = 255;	
	}
	if (linelen < 2) {
		linelen = 2;	
	}
	x0 =-XSSIZE * xsize / size;
	y0 = YSSIZE * ysize / size - 1.0;

	MessageEnter();
	glScalef(size, size, size);
	for (i = 0; i< charlen;i ++) {
		if (isascii(str[i])) {
			line[linec++] = str[i];
		} else {
			if (linec > linelen-2) {
				line[linec++] = '\0';
				line[linec++] = '\0';
				i--;
			} else {
				line[linec++] = str[i++];
				line[linec++] = str[i];		
			}
		}
		if (linec > linelen-1) {
		 	line[linec] = '\0';
			linec = 0;
			glPushMatrix();
			glTranslatef(x0, y0, 0.0);
			TextSingle(line);
			glPopMatrix();
			y0-=1.0;
		}
	}
	line[linec] = '\0';
	glTranslatef(x0, y0, 0.0);
	TextSingle(line);

	MessageLeave();
}

void GLTText::Pause(char *str, float size, int xpos, int ypos) {
	SEvent event;
	do {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		Message(str, size);
		SDL_GL_SwapBuffers();
		event.Poll();
	} while(event.IsContinue());
	if (event.IsQuit()) {
		exit(0);		
	}
}

void GLTText::MessageEnter() {
	light_save.Get();
	glPushAttrib(GL_ENABLE_BIT);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH_TEST);
	
	save_rendertype = ttf->rendertype;
	ttf->SetRender(RENDER_BLENDED);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(-XSSIZE, XSSIZE, -YSSIZE, YSSIZE, 0.01, 20.0);

    glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
    glLoadIdentity();

	glEnable(GL_LIGHTING);
	light.On();	
	light.Set();
	camera.Set();
	color.Set();
}

void GLTText::MessageLeave() {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	ttf->SetRender(save_rendertype);
	light_save.Set();
	glPopAttrib();
}

bool GLTText::WasInit() {
	return wasinit;
}

void GLTText::MakeTexture(int number, SDL_Surface *render) {
	surface.Generate(number, render); 
	SDL_FreeSurface(render);
}

void GLTText::CallTexture(int number) {
	float w, h;
	surface.Choice(number);
	w = surface.WidthRatio();
	h = surface.HeightRatio();

	glBegin(GL_QUADS);
	glNormal3f(0.0, 0.0, 1.0);
	glTexCoord2f(0.0,  h ); glVertex2f(0.0, 0.0);
	glTexCoord2f( w ,  h ); glVertex2f(1.0, 0.0);
	glTexCoord2f( w , 0.0); glVertex2f(1.0, 1.0);
	glTexCoord2f(0.0, 0.0); glVertex2f(0.0, 1.0);
	glEnd();
}

void GLTText::CallTextureD(int number) {
	float w, h;
	surface.Choice(number);
	w = surface.WidthRatio();
	h = surface.HeightRatio();

	glBegin(GL_QUADS);
	glNormal3f(0.0, 0.0, 1.0);
	glTexCoord2f(0.0,  h ); glVertex3f(0.0, 0.0, 0.125);
	glTexCoord2f( w ,  h ); glVertex3f(1.0, 0.0, 0.125);
	glTexCoord2f( w , 0.0); glVertex3f(1.0, 1.0, 0.125);
	glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 1.0, 0.125);
	glNormal3f(0.0, 0.0, -1.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 1.0,-0.125);
	glTexCoord2f( w , 0.0 );glVertex3f(1.0, 1.0,-0.125);
	glTexCoord2f( w ,  h ); glVertex3f(1.0, 0.0,-0.125);
	glTexCoord2f(0.0,  h ); glVertex3f(0.0, 0.0,-0.125);
	glEnd();
}

/* SDL_gfx̃tHgSurface쐬 */
#ifdef USE_GFX_FONT
int GLTText::Render(char ccode, SDL_Surface *surface) {
	int i, j, index_y, bpp = surface->format->BytesPerPixel;
    Uint8 *p;
	Uint8 index_x;

  	if (SDL_MUSTLOCK(surface)) {
		if (SDL_LockSurface(surface) < 0) {
		    return SDL_FAILURE;
		}
    }

	for (j = 0; j < 8; j++) {
		index_y = j;
		for (i = 0; i < 8; i++) {
			index_x = (128>>i);	
			p = (Uint8 *)surface->pixels + j * surface->pitch + i * bpp;
			if (gfxPrimitivesFontdata[8*ccode+index_y] & index_x) {
				*p = 1;						
			} else {
				*p = 0;									
			}
		}	
	}

 	if (SDL_MUSTLOCK(surface)) {
		SDL_UnlockSurface(surface);
	}
	return SDL_SUCCESS;
}
#else      
/* ƎҏW̃tHgSurface쐬 */ 
int GLTText::Render(char ccode, SDL_Surface *surface) {
	int i, j, index_y, bpp = surface->format->BytesPerPixel;
    Uint8 *p;
	Uint16 index_x;

  	if (SDL_MUSTLOCK(surface)) {
		if (SDL_LockSurface(surface) < 0) {
		    return SDL_FAILURE;
		}
    }

	for (j = 0; j < 11; j++) {
		index_y = j;
		for (i = 0; i < 11; i++) {
			index_x = (1<<i);
			p = (Uint8 *)surface->pixels + j * surface->pitch + i * bpp;
			if (alphabet_font[11*ccode+index_y] & index_x) {
				*p = 1;						
			} else {
				*p = 0;									
			}	
		}	
	}

	if (SDL_MUSTLOCK(surface)) {
		SDL_UnlockSurface(surface);
	}
	return SDL_SUCCESS;
}  
#endif
