#include "lobby.h"

TCP_Socket::TCP_Socket() {
	socket = NULL;
}

TCP_Socket::~TCP_Socket() {
	Close();
}

int TCP_Socket::Open(IP &ip) {
	if (WasOpen()) {
		wmes("TCP_Socket::Open -> Socket is already opened.");
		return SDL_FAILURE;
	}
	
	socket = ip.TCP_Open();
	if (!WasOpen()) {
		emes("TCP_Socket::Open -> Can't open socket.");
//		ehost(ip.HostName());
		rmes();
		return SDL_FAILURE;	
	}
	smes("TCP_Socket::Open");
	shost(ip.HostName());
}

void TCP_Socket::Close() {
	if (WasOpen()) {
		smes("TCP_Socket::Close");
		SDLNet_TCP_Close(socket);
		socket = NULL;
	}
}

bool TCP_Socket::WasOpen() {
	if (socket == NULL) {
		return false;	
	}
	return true;	
}

int TCP_Socket::GetOnce(void *data, int maxlen) {
	static int size;
	size = SDLNet_TCP_Recv(socket, (Uint8 *)data, maxlen);
	if (size <= 0) {
		return SDL_FAILURE;
	}
	return size;
}

int TCP_Socket::SendData(void *data, int maxlen) {
	if (SDLNet_TCP_Send(socket, data, maxlen) < maxlen) {
		wmes("TCP_Socket::SendData");		
		rmes();		
		return SDL_FAILURE;
	}
	return SDL_SUCCESS;
}

int TCP_Socket::GetData(void *data, int maxlen) {
	static int size, get_size, left;
	get_size = 0;
	left = maxlen;
	while(left > 0) {
		size = SDLNet_TCP_Recv(socket, (Uint8 *)data + get_size, left);
		if (size < 0) {
			wmes("TCP_Socket::GetData");		
			rmes();	
			return SDL_FAILURE;
		}
		get_size+= size;
		left = maxlen - get_size;	
	}
	return SDL_SUCCESS;
}

void TCP_Socket::Send8(Uint8 data) {
	if (SDLNet_TCP_Send(socket, &data, 1) < 1) {
		wmes("TCP_Socket::Send8");		
		rmes();		
	}
}

Uint8 TCP_Socket::Get8() {
	static int size;
	static Uint8 data;
	do {
		size = SDLNet_TCP_Recv(socket, &data, 1);
		if (size < 0) {	
			wmes("TCP_Socket::Get8");	
			rmes();
			return 0xff;
		}
	} while(size == 0); 
	return data;
}

void TCP_Socket::Send16(Uint16 data) {
	if (SDLNet_TCP_Send(socket, &data, 2) < 2) {
		wmes("TCP_Socket::Send16");		
		rmes();			
	}
}

Uint16 TCP_Socket::Get16() {
	static int size, get_size, left;
	static Uint16 data;
	get_size = 0;
	left = 2;
	while(left > 0) {
		size = SDLNet_TCP_Recv(socket, (Uint8 *)&data + get_size, left);
		if (size < 0) {
			wmes("TCP_Socket::Get16");	
			rmes();	
			return 0xffff;
		}
		get_size+= size;
		left = 2 - get_size;	
	}
	return data;
}

void TCP_Socket::Send32(Uint32 data) {
	if (SDLNet_TCP_Send(socket,  (Uint8 *)&data, 4) < 4) {
		wmes("TCP_Socket::Send32");		
		rmes();				
	}
}

Uint32 TCP_Socket::Get32() {
	static int size, get_size, left;
	static Uint32 data;
	get_size = 0;
	left = 4;
	while(left > 0) {
		size = SDLNet_TCP_Recv(socket, (Uint8 *)&data + get_size, left);
		if (size < 0) {
			wmes("TCP_Socket::Get32");	
			rmes();	
			return 0xffffffff;
		}
		get_size+= size;
		left = 4 - get_size;
	}
	return data;
}

UDP_Socket::UDP_Socket() {
	socket = NULL;
	packet = NULL;
}

UDP_Socket::~UDP_Socket() {
	Close();
}

int UDP_Socket::Open(Uint16 port) {
	if (WasOpen()) {
		wmes("UDP_Socket::Open -> Socket is already opened.");
		return SDL_FAILURE;
	}
	
	socket = SDLNet_UDP_Open(port);
	if (!WasOpen()) {
		emes("UDP_Socket::Open -> Can't open socket.");
		rmes();
		return SDL_FAILURE;	
	}
	smes("UDP_Socket::Open -> Open Socket");
	
	packet = SDLNet_AllocPacket(4);
	if (packet == NULL) {
		emes("UDP_Socket::Open -> Can't allocate packet.");
		rmes();
		return SDL_FAILURE;		
	}
	smes("UDP_Socket::Open -> Allocate Packet");	
}

void UDP_Socket::Close() {
	if (WasOpen()) {
		SDLNet_UDP_Close(socket);
		socket = NULL;
		if (packet != NULL) {
		 	SDLNet_FreePacket(packet);
			packet = NULL;
		}
		smes("UDP_Socket::Close");
	}
}

bool UDP_Socket::WasOpen() {
	if (socket == NULL) {
		return false;	
	}
	return true;	
}

/*
void UDP_Socket::SetDestination(IP &ip) {
	SDLNet_UDP_Bind(socket, 1, &ip.Address());
//	packet->address = ip.Address();
}
*/

void UDP_Socket::SetDestination(Uint32 host, Uint16 port) {
	IPaddress dest;
	dest.host = host;
	dest.port = port;
	SDLNet_UDP_Bind(socket, 1, &dest);	 
//	packet->address.host = host;
//	packet->address.port = port;   
}

int UDP_Socket::ResizePacket(int size) {
	SDLNet_ResizePacket(packet, size);
	if (packet == NULL) {
		emes("UDP_Socket::ResizePacket");		
		rmes();
		return SDL_FAILURE;	
	}
	return SDL_SUCCESS;		
}


int UDP_Socket::SendData(void *data, int maxlen) {
	if (packet->maxlen < maxlen) {
		ResizePacket(maxlen);
	}
	packet->len = maxlen;
	memcpy(packet->data, data, maxlen);	
	if (SDLNet_UDP_Send(socket, 1, packet) == 0) {
		wmes("UDP_Socket::SendData");		
		rmes();		
		return SDL_FAILURE;
	}
	return SDL_SUCCESS;
}

int UDP_Socket::GetData(void *data, int maxlen) {
	static int size;
	if (packet->maxlen < maxlen) {
		ResizePacket(maxlen);
	}
	do {
		size = SDLNet_UDP_Recv(socket, packet);		
		if (size < 0) {
			wmes("UDP_Socket::GetData");		
			rmes();	
			return SDL_FAILURE;
		}
	} while (size == 0);
 
	if (packet->len != maxlen) {
		wmes("UDP_Socket::GetData -> Size Error");
		fprintf(stderr, "Packet Size = %d  Required Size = %d", packet->len, maxlen);
		return SDL_FAILURE;
	}
	memcpy(data, packet->data, maxlen);	
	return SDL_SUCCESS;
}

void UDP_Socket::Send8(Uint8 data) {
	packet->len = 1;
	packet->data[0] = data;
	if (SDLNet_UDP_Send(socket, 1, packet) == 0) {
		wmes("UDP_Socket::Send8");		
		rmes();		
	}
}
 
Uint8 UDP_Socket::Get8() {
	static int size;
	static Uint8 data;
	do {
		size = SDLNet_UDP_Recv(socket, packet);
		if (size < 0) {	
			wmes("UDP_Socket::Get8");	
			rmes();
			return 0xff;
		}
	} while (size == 0);
	if (packet->len != 1) {
		wmes("UDP_Socket::Get8 -> Size Error");	
		return 0xff;
	}
	packet->data[0]; 
	return	data;
}

void UDP_Socket::Send16(Uint16 data) {
	packet->len = 2;
	*(packet->data)		= *((Uint8 *)&data);
	*(packet->data + 1)	= *((Uint8 *)&data + 1);
	if (SDLNet_UDP_Send(socket, 1, packet) == 0) {
		wmes("UDP_Socket::Send16");		
		rmes();			
	}
}

Uint16 UDP_Socket::Get16() {
	static int size;
	static Uint16 data;
	do {	
		size = SDLNet_UDP_Recv(socket, packet);
		if (size < 0) {
			wmes("UDP_Socket::Get16");	
			rmes();	
			return 0xffff;
		}
	} while (size == 0);
	if (packet->len != 2) {
		wmes("UDP_Socket::Get16 -> Size Error");	
		fprintf(stderr, "Packet Size = %d  Required Size = 2\n", packet->len);
		return 0xffff;
	}
	*((Uint8 *)&data)		= *(packet->data);
	*((Uint8 *)&data + 1)	= *(packet->data + 1);
	return data;
}

void UDP_Socket::Send32(Uint32 data) {
	packet->len = 4;
	*(packet->data)		= *((Uint8 *)&data);
	*(packet->data + 1)	= *((Uint8 *)&data + 1);
	*(packet->data + 2)	= *((Uint8 *)&data + 2);
	*(packet->data + 3)	= *((Uint8 *)&data + 3);
	if (SDLNet_UDP_Send(socket, 1, packet) == 0) {
		wmes("UDP_Socket::Send32");		
		rmes();				
	}
}

Uint32 UDP_Socket::Get32() {
	static int size;
	static Uint32 data;
	do {
		size = SDLNet_UDP_Recv(socket, packet);
		if (size < 0) {
			wmes("UDP_Socket::Get32");	
			rmes();	
			return 0xffffffff;
		}
	} while (size == 0);

	if (packet->len != 4) {
		wmes("UDP_Socket::Get32 -> Size Error");	
		fprintf(stderr, "Packet Size = %d  Required Sizev = 4\n", packet->len);
		return 0xffffffff;
	}
	*((Uint8 *)&data)		= *(packet->data);
	*((Uint8 *)&data + 1)	= *(packet->data + 1);
	*((Uint8 *)&data + 2)	= *(packet->data + 2);
	*((Uint8 *)&data + 3)	= *(packet->data + 3);
	return data;
}


SocketSet::SocketSet() {
	set = NULL;
}

SocketSet::~SocketSet() {
	Free();
}

int SocketSet::Create(int number) {
	if (WasCreate()) {
		wmes("SocketSet::Create -> Socket set is already created.");	
		return SDL_FAILURE;	
	}
	
	set = SDLNet_AllocSocketSet(number);
	if (!WasCreate()) {
		emes("SocketSet::Create -> Can't allocate socket set.");
		rmes();
		return SDL_FAILURE;	
	}
	smes("SocketSet::Create");	

	return SDL_SUCCESS;
}

void SocketSet::Free() {
	if (WasCreate()) {
		SDLNet_FreeSocketSet(set);
		set = NULL;	
	}
}

bool SocketSet::WasCreate() {
	if (set == NULL) {
		return false;	
	}
	return true;
}


/*struct _SDLNet_SocketSet {
	int numsockets;
	int maxsockets;
	struct SDLNet_Socket **sockets;
};

int SocketSet::Number() {
	return set->numsockets;
}

int SocketSet::Max() {
	return set->maxsockets;
}       */

void SocketSet::Add_TCP_Socket(TCP_Socket &tcp) {
	if (tcp.WasOpen()) {
#ifndef DEBUG
		SDLNet_TCP_AddSocket(set, tcp.socket);
#else
 		fprintf(stdout, "SUCCEESS : Socket Number %d SocketSet::Add_TCP_Socket\n", SDLNet_TCP_AddSocket(set, tcp.socket)); 
#endif		
	}
}

void SocketSet::Del_TCP_Socket(TCP_Socket &tcp) {
	if (tcp.WasOpen()) {
#ifndef DEBUG
		SDLNet_TCP_DelSocket(set, tcp.socket);
#else
 		fprintf(stdout, "SUCCESS : Socket Number %d SocketSet::Del_TCP_Socket\n", SDLNet_TCP_DelSocket(set, tcp.socket)); 
#endif		
	}
}

void SocketSet::Add_UDP_Socket(UDP_Socket &udp) {
	if (udp.WasOpen()) {
#ifndef DEBUG
		SDLNet_UDP_AddSocket(set, udp.socket);
#else
 		fprintf(stdout, "SUCCESS : Socket Number %d SocketSet::Add_UDP_Socket\n", SDLNet_UDP_AddSocket(set, udp.socket)); 
#endif		

	}
}

void SocketSet::Del_UDP_Socket(UDP_Socket &udp) {
	if (udp.WasOpen()) {
#ifndef DEBUG
		SDLNet_UDP_DelSocket(set, udp.socket);
#else
 		fprintf(stdout, "SUCCESS : Socket Number %d SocketSet::Del_UDP_Socket\n", SDLNet_UDP_DelSocket(set, udp.socket)); 
#endif		
	}
}

Lobby::Lobby() {
	int i;
	player_number = 0;
	for (i = 0; i < MAX_PLAYER; i++) {
		client_state[i] = NO_LOBBY;	
		client_name[i][NAME_LENGTH] = '\0';	
	}
	for (i = 0; i < MAX_ROOM; i++) {
		room_state[i] = EMPTY_ROOM;	
	}
}

Lobby::~Lobby() {
	Quit();
}

int Lobby::Init() {
	if (WasInit()) {
		Quit();	
	}	

	lobby_log = fopen("lobby_log.txt", "wt+");
	if (lobby_log == NULL) {
		wmes("Lobby::Init -> Can't open log file.");	
	}
	smes("Lobby::Init -> Open Log File");	

	if (SDLNet_Init() == SDL_FAILURE) {
		emes("Lobby::Init -> Can't open lobby.");
		rmes();
		return SDL_FAILURE;
	}
	smes("Lobby::Init -> Open Lobby");	

	socket_set.Create(MAX_PLAYER + 1);
	if (!socket_set.WasCreate()) {
		return SDL_FAILURE;	
	}

	return SDL_SUCCESS;   	
}

void Lobby::Quit() {
	int i;
    if (WasInit()) {
		if (lobby_log != NULL) {
			fclose(lobby_log);		
		}
		for (i = 0; i < MAX_PLAYER; i++) {
			socket_set.Del_TCP_Socket(client_socket[i]);
			client_socket[i] .Close();
		}
		socket_set.Del_TCP_Socket(server_socket);
		server_socket.Close();		
		socket_set.Free();				
		SDLNet_Quit();
		player_number = 0;	
		smes("Lobby::Quit");
	} else {
		wmes("Lobby::Quit");	
	}
}

bool Lobby::WasInit() {
	if (socket_set.WasCreate()) {
		return true;		
	}
	return false;	
}


int Lobby::ResolveHost(Uint16 port) {
	if (server_ip.ResolveHost(NULL, port)) {
		emes("Lobby::ResolveHost -> Can't resolve host.");
		rmes();
		return SDL_FAILURE;
	}
	smes("Lobby::ResolveHost");

	server_socket.Open(server_ip);
	if (!server_socket.WasOpen()) {
		emes("Lobby::ResolveHost -> Can't open server socket.");
		return SDL_FAILURE;	
	}
	smes("Lobby::ResolveHost -> Open Server Socket");	
	socket_set.Add_TCP_Socket(server_socket);

	fprintf(lobby_log, "Open Lobby\n");	
	return SDL_SUCCESS;
}

void Lobby::CheckSockets() {
	if (socket_set.Check(1000) < 0) {
		wmes("Lobby::CheckSockets");	
		rmes();
	}
}  

void Lobby::WaitClients() {
	if (server_socket.IsReady()) {
		AddClient();
	}
}

void Lobby::HandleClients() {
	static int i;
	static bool is_request;
	is_request = false;
	for (i = 0; i < MAX_PLAYER; i++){
		if (client_socket[i].IsReady()) {
			HandleClient(i);
			is_request = true;			
		} else {
		 	wait_count[i]++;
			if ((client_state[i] == LOBBY_CERTIFY && wait_count[i] > 50) ||
				(client_state[i] == LOBBY_WAIT && wait_count[i] > 5000)) {
		//		client_socket[i].Send8(KILL_CLIENT);
				socket_set.Del_TCP_Socket(client_socket[i]);
				client_socket[i].Close();
				player_number--;
				client_state[i] = NO_LOBBY;
				fprintf(lobby_log, "Delete Client %d.%d.%d.%d\n",
					client_ip[i].IP1(), client_ip[i].IP2(), client_ip[i].IP3(), client_ip[i].IP4());				
			}
		}
	}
	if (is_request) { 
		for (i = 0; i < MAX_PLAYER; i++){
			if (client_state[i] == LOBBY_WAIT || client_state[i] == LOBBY_ACCEPT) {
				client_socket[i].Send8(SEND_STATES);
				SendState(i);
			}
		}
	}
}

void Lobby::SendState(int number) {
	int i;
	client_socket[number].Send8(player_number);
	client_socket[number].SendData(room_state, MAX_ROOM);		
	for (i = 0; i < MAX_ROOM; i++) {
		client_socket[number].SendData(client_name[room_state[i]], NAME_LENGTH);		
	}

}

int Lobby::AddClient() {
	TCP_Socket new_socket;
	IP new_ip;
	int i, socket_number;

	server_socket.Accept(new_socket);
	if (!new_socket.WasOpen()) {
		emes("Lobby::AddClient -> Can't accept client.");
		return SDL_FAILURE;
	}
	smes("Lobby::AddClient -> Accept Client");
	new_ip.TCP_GetAddress(new_socket);
	fprintf(lobby_log, "Connet Start\n");

	for (i = 0; i < MAX_PLAYER; i++) {
		if (!client_socket[i].WasOpen()) {
			socket_number = i;		
			break;
		}
	}
	
	if (i == MAX_PLAYER) {
		new_socket.Send8(LOBBY_FULL);
		new_socket.Close();
		fprintf(lobby_log, "Lobby is full.\n");	
		return SDL_FAILURE;
	}

	new_socket.Send8(LOBBY_CONNECT);
	client_socket[socket_number] = new_socket;
	client_ip[socket_number] = new_ip;
	client_state[socket_number] = LOBBY_CERTIFY;
	wait_count[socket_number] = 0;
	socket_set.Add_TCP_Socket(client_socket[socket_number]);
	player_number++;
	fprintf(lobby_log, "Connet new player%s\n", new_ip.HostName());
	new_socket.Destroy();
	return SDL_SUCCESS;
}

void Lobby::HandleClient(int number) {
	int i;
	Uint8 data, room_number, room_client, o_number;
	TCP_Socket &socket = client_socket[number];
	IP &ip = client_ip[number];
	Uint8 &state = client_state[number];
	Uint8 *name = client_name[number];
	Uint16 &port = client_port[number];
	Uint32 &count = wait_count[number];

	socket.GetOnce(&data, 1);
	switch (data) {
		case LEAVE_LOBBY:
			if (state == LOBBY_ACCEPT) {
				for (i = 0; i < MAX_ROOM; i++) {
					if (room_state[i] == number) {
						room_state[i] = EMPTY_ROOM;	 	
						break;
					}
				}
			}
			socket_set.Del_TCP_Socket(socket);
			socket.Close();
			player_number--;
			state = NO_LOBBY;
			fprintf(lobby_log, "Player leaves.%s\n", ip.HostName());
		break;			
		case SEND_CODE:
			Uint32 lobby_code;
			socket.GetOnce(&lobby_code, 4);
			socket.GetOnce(&port, 2);
			socket.GetOnce(name, NAME_LENGTH);
			if (lobby_code != LOBBY_CODE) {
				socket.Send8(CODE_ERROR);
				socket_set.Del_TCP_Socket(socket);
				socket.Close();
				player_number--;
				state = NO_LOBBY;
				fprintf(lobby_log, "Certified code is different. %d.%d.%d.%d\n",
					ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4());	
			} else {
				socket.Send8(CODE_SUCCESS);
				fprintf(lobby_log, "Certified Code OK %d.%d.%d.%d\n",
					ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4());	
				state = LOBBY_WAIT;
				count = 0;
    		}
		break;
		case REQUEST_STATES:
			SendState(i);
		break;
		case REQUEST_ROOM:
			socket.GetOnce(&room_number, 1);
			if (room_number < MAX_ROOM) {
				o_number = room_state[room_number];
				if (state == LOBBY_WAIT) {
					if (o_number== EMPTY_ROOM) {
						room_state[room_number] = number;	 	
						state = LOBBY_ACCEPT;
					}  else {
						state = LOBBY_MATCH;				
                     	socket.Send8(MATCH_BLUE);
						socket.Send32(client_ip[o_number].Host());
						socket.Send16(client_port[o_number]);						
	
						client_state[o_number] = LOBBY_MATCH;
						client_socket[o_number].Send8(MATCH_RED);	
						client_socket[o_number].Send32(ip.Host());	
						client_socket[o_number].Send16(port);	

						room_state[room_number] = EMPTY_ROOM;
					}
				} else if (state == LOBBY_ACCEPT) {
					if (o_number == number) {
						room_state[room_number] = EMPTY_ROOM;	 	
						state = LOBBY_WAIT;				
					}
				}
			}
		break;
		case RETURN_LOBBY:
			state = LOBBY_WAIT;							
		break;	
		default:
		break;	
	}
}


LobbyClient::LobbyClient() {
	int i;
	for (i = 0; i < MAX_ROOM; i++) {
		room_name[i][NAME_LENGTH] = '\0';	
	}
}

LobbyClient::~LobbyClient() {
	Quit();
}

int LobbyClient::Init() {
	if (WasInit()) {
		Quit();	
	}	
	if (SDLNet_Init() == SDL_FAILURE) {
		emes("LobbyClient::Init");
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClient::Init");	

	socket_set.Create(1);
	if (!socket_set.WasCreate()) {
		return SDL_FAILURE;
	}
	return SDL_SUCCESS;
}

void LobbyClient::Quit() {
    if (WasInit()) {
		socket_set.Del_TCP_Socket(tcp_socket);
		tcp_socket.Close();
		socket_set.Free();
		SDLNet_Quit();
		smes("LobbyClient::Quit");	
	} else {
		emes("LobbyClient::Quit");	
	}
}

bool LobbyClient::WasInit() {
	if (socket_set.WasCreate()) {
		return true;		
	}
	return false;	
}

int LobbyClient::ConnectLobby(char* user_name, char* hostname, Uint16 port, Uint16 port_tcp) {
	IP server_ip;
	Uint8 lobby_state;
	int timeout_count;

	if (server_ip.ResolveHost(hostname, port)) {
		emes("LobbyClient::ConnectLobby -> Can't find lobby server.");
//		ehost(server_ip.HostName());
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClient::ConnectLobby -> Find Lobby Server");
	shost(server_ip.HostName());

	tcp_socket.Open(server_ip);
	if (!tcp_socket.WasOpen()) {
		return SDL_FAILURE;
	}
	socket_set.Add_TCP_Socket(tcp_socket);

	if (WaitData(500) == SDL_FAILURE) {
		emes("LobbyClient::ConnectLobby -> Time Out");
		return SDL_FAILURE;	
	}

	lobby_state = tcp_socket.Get8();
	switch (lobby_state) {
		case LOBBY_CONNECT :
			smes("LobbyClient::ConnectLobby -> Connet Lobby");
		break;	
		case LOBBY_FULL :
			emes("LobbyClient::ConnectLobby -> Lobby is full.");
			return SDL_FAILURE;				
		break;	
		default:
			emes("LobbyClient::ConnectLobby -> Unknown Error");
			return SDL_FAILURE;	
		break;	
	}
	SendLobbyCode();
		
	tcp_socket.Send16(port_tcp);	
	tcp_socket.SendData(user_name, NAME_LENGTH);

	if (tcp_socket.Get8() == CODE_ERROR) {
		emes("LobbyClient::ConnectLobby -> Code Error");
		return SDL_FAILURE;				
	}

	return SDL_SUCCESS;
}

Uint8 LobbyClient::Handle() {
	Uint8 data;

	tcp_socket.GetOnce(&data, 1);
	switch (data) {
		case SEND_STATES:
			GetState();
			return data;
		break;			
		case MATCH_RED:
		case MATCH_BLUE:
			host = tcp_socket.Get32();
			port = tcp_socket.Get16();
			ip.SetHost(host);
		break;			
		default:
		break;	
	}

	return data;
}

void LobbyClient::GetState() {
	int i;
	player_number = tcp_socket.Get8();
	tcp_socket.GetData(room_state, MAX_ROOM);
	for (i = 0; i < MAX_ROOM; i++) {
		tcp_socket.GetData(room_name[i], NAME_LENGTH);
	}
}

int LobbyClient::WaitData(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsReady()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}


LobbyClientUDP::LobbyClientUDP() {
	int i;
	for (i = 0; i < MAX_ROOM; i++) {
		room_name[i][NAME_LENGTH] = '\0';	
	}
}

LobbyClientUDP::~LobbyClientUDP() {
	Quit();
}

int LobbyClientUDP::Init() {
	if (WasInit()) {
		Quit();	
	}	
	if (SDLNet_Init() == SDL_FAILURE) {
		emes("LobbyClientUDP::Init");
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClientUDP::Init");	

	socket_set.Create(2);
	if (!socket_set.WasCreate()) {
		return SDL_FAILURE;
	}
	return SDL_SUCCESS;
}

void LobbyClientUDP::Quit() {
    if (WasInit()) {
		socket_set.Del_TCP_Socket(tcp_socket);
		tcp_socket.Close();
		socket_set.Del_UDP_Socket(udp_socket);
		udp_socket.Close();
		socket_set.Free();
		SDLNet_Quit();
		smes("LobbyClientUDP::Quit");	
	} else {
		emes("LobbyClientUDP::Quit");	
	}
}

bool LobbyClientUDP::WasInit() {
	if (socket_set.WasCreate()) {
		return true;		
	}
	return false;
}

int LobbyClientUDP::ConnectLobby(char* user_name, char* hostname, Uint16 port, Uint16 port_udp) {
	IP server_ip;
	Uint8 lobby_state;
	Uint16 try_port;
	int timeout_count;

	if (server_ip.ResolveHost(hostname, port)) {
		emes("LobbyClientUDP -> Can't find lobby server.");
//		ehost(server_ip.HostName());
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClientUDP -> Find Lobby Server");
	shost(server_ip.HostName());

	tcp_socket.Open(server_ip);
	if (!tcp_socket.WasOpen()) {
		return SDL_FAILURE;
	}
	socket_set.Add_TCP_Socket(tcp_socket);

	for (try_port = port_udp; try_port < port_udp + 10; try_port++) {
		udp_socket.Open(try_port);
		if (udp_socket.WasOpen()) {
//			fprintf(stderr, "Port = %d\n", try_port);
			break;	
		}
	}
	if (!udp_socket.WasOpen()) {
		return SDL_FAILURE;
	}
	my_ip.UDP_GetAddress(udp_socket);
	socket_set.Add_UDP_Socket(udp_socket);

	if (WaitData(500) == SDL_FAILURE) {
		emes("LobbyClientUDP -> Time Out");
		return SDL_FAILURE;	
	}

	lobby_state = tcp_socket.Get8();
	switch (lobby_state) {
		case LOBBY_CONNECT :
			smes("LobbyClientUDP -> Connect Lobby");
		break;	
		case LOBBY_FULL :
			emes("LobbyClientUDP -> Lobby is full.");
			return SDL_FAILURE;				
		break;	
		default:
			emes("LobbyClientUDP -> Unknown Error");
			return SDL_FAILURE;	
		break;	
	}
	SendLobbyCode();
		
	Uint16 this_port = my_ip.Port();
//	fprintf(stderr, "Port = %d\n", SDLNet_Read16(&this_port));
//	strcpy((char *)name, "Wait");
	tcp_socket.Send16(my_ip.Port());	
	tcp_socket.SendData(user_name, NAME_LENGTH);

	if (tcp_socket.Get8() == CODE_ERROR) {
		emes("LobbyClientUDP -> Code Error");
		return SDL_FAILURE;				
	}

	return SDL_SUCCESS;
}

Uint8 LobbyClientUDP::Handle() {
	Uint8 data;
	Uint16 port;
	Uint32 host;
//	IP ip;

	tcp_socket.GetOnce(&data, 1);
	switch (data) {
		case SEND_STATES:
			GetState();
		break;			
		case MATCH_RED:
			SetCornerRed();
			host = tcp_socket.Get32();
			port = tcp_socket.Get16();
			SetDestination(host, port);			
//			ip.SetHost(host);
//			fprintf(stderr, "red port = %d\n", SDLNet_Read16(&port));
//			fprintf(stderr, "red ip = %d.%d.%d.%d\n", ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4()); 
		break;			
		case MATCH_BLUE:
			SetCornerBlue();
			host = tcp_socket.Get32();
			port = tcp_socket.Get16();
			SetDestination(host, port);			
//			ip.SetHost(host);
//			fprintf(stderr, "blue port = %d\n", SDLNet_Read16(&port));
//			fprintf(stderr, "blue ip = %d.%d.%d.%d\n", ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4()); 
		break;			
		default:
		break;	
	}
	return data;
}

void LobbyClientUDP::GetState() {
	int i;
	player_number = tcp_socket.Get8();
	tcp_socket.GetData(room_state, MAX_ROOM);
	for (i = 0; i < MAX_ROOM; i++) {
		tcp_socket.GetData(room_name[i], NAME_LENGTH);
	}
}

int LobbyClientUDP::WaitData(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsReady()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}


int LobbyClientUDP::WaitUDPData(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsUDPReady()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}

/*
int LobbyClientUDP::SendLargeData(void *data, int maxlen) {
	static int size, send_size, left;
	left = maxlen;
	send_size = 0;
	while(left > 0) {
		size = MAX_PACKET_SIZE;
		if (left < size) {
			size = left;		
		}
		if (udp_socket.SendData((Uint8*)data + send_size, size) == SDL_FAILURE) {
			return SDL_FAILURE;
		}
		send_size += size;
		left = maxlen - send_size;	
		fprintf(stderr, "left send = %d\n", left);
	}
	return SDL_SUCCESS;
}

int LobbyClientUDP::GetLargeData(void *data, int maxlen) {
	static int size, get_size, left;
	left = maxlen;
	get_size = 0;
	while(left > 0) {
		size = MAX_PACKET_SIZE;
		if (left < size) {
			size = left;		
		}
		if (WaitUDPData(100) == SDL_FAILURE) {
			wmes("LobbyClientUDP::GetLargeData -> Time Out");
			return SDL_FAILURE;		
		}
		if (udp_socket.GetData((Uint8*)data + get_size, size) == SDL_FAILURE) {
			wmes("LobbyClientUDP::GetLargeData");
			return SDL_FAILURE;
		}
		get_size += size;
		left = maxlen - get_size;	
		fprintf(stderr, "left get = %d\n", left);
	}
	return SDL_SUCCESS;
}
*/


Lobby2::Lobby2() {
	int i;
	player_number = 0;
	for (i = 0; i < MAX_PLAYER; i++) {
		client_state[i] = NO_LOBBY;	
		client_name[i][NAME_LENGTH] = '\0';	
	}
	for (i = 0; i < MAX_ROOM; i++) {
		room_state[i] = EMPTY_ROOM;	
	}
}

Lobby2::~Lobby2() {
	Quit();
}

int Lobby2::Init() {
	if (WasInit()) {
		Quit();	
	}	

	lobby_log = fopen("lobby_log.txt", "wt+");
	if (lobby_log == NULL) {
		wmes("Lobby2::Init -> Can't open log file.");
	}
	smes("Lobby2::Init -> Open Log File");

	if (SDLNet_Init() == SDL_FAILURE) {
		emes("Lobby2::Init -> Can't initialize SDLNet");
		rmes();
		return SDL_FAILURE;
	}
	smes("Lobby2::Init");	

	socket_set.Create(MAX_PLAYER*2 + 2);
	if (!socket_set.WasCreate()) {
		return SDL_FAILURE;	
	}

	return SDL_SUCCESS;   	
}

void Lobby2::Quit() {
	int i;
    if (WasInit()) {
		if (lobby_log != NULL) {
			fclose(lobby_log);		
		}
		for (i = 0; i < MAX_PLAYER; i++) {
			socket_set.Del_TCP_Socket(client_socket[i]);
			client_socket[i] .Close();
			socket_set.Del_TCP_Socket(client_socket2[i]);
			client_socket2[i] .Close();
		}
		socket_set.Del_TCP_Socket(server_socket);
		server_socket.Close();		

		socket_set.Free();				
		SDLNet_Quit();
		player_number = 0;	
		smes("Lobby2::Quit");
	} else {
		wmes("Lobby2::Quit");	
	}
}

bool Lobby2::WasInit() {
	if (socket_set.WasCreate()) {
		return true;		
	}
	return false;	
}

int Lobby2::ResolveHost(Uint16 port) {
	if (server_ip.ResolveHost(NULL, port)) {
		emes("Lobby2::ResolveHost -> Can't resolver host.");
		rmes();
		return SDL_FAILURE;
	}
	smes("Lobby2::ResolveHost -> Resolve Host");

	server_socket.Open(server_ip);
	if (!server_socket.WasOpen()) {
		emes("Lobby2::ResolveHost -> Can't open server socket.");
		return SDL_FAILURE;	
	}
	smes("Lobby2::ResolveHost -> Open Server Socket");	
	socket_set.Add_TCP_Socket(server_socket);

	fprintf(lobby_log, "Open Lobby\n");	
	return SDL_SUCCESS;
}

void Lobby2::CheckSockets() {
	if (socket_set.Check(1000) < 0) {
		wmes("CheckSockets");	
		rmes();
	}
}  

void Lobby2::WaitClients() {
	if (server_socket.IsReady()) {
		AddClient();
	}
}

void Lobby2::HandleClients() {
	static int i, size;
	static bool is_request;
	static Uint8 packet[MAX_PACKET_SIZE+1];

	is_request = false;
	for (i = 0; i < MAX_PLAYER; i++){
		if (client_socket[i].IsReady()) {
			HandleClient(i);
			is_request = true;			
		} else {
		 	wait_count[i]++;
			if ((client_state[i] == LOBBY_CERTIFY && wait_count[i] > 50) ||
				(client_state[i] == LOBBY_WAIT && wait_count[i] > 5000)) {
		//		client_socket[i].Send8(KILL_CLIENT);
				socket_set.Del_TCP_Socket(client_socket[i]);
				client_socket[i].Close();
				player_number--;
				client_state[i] = NO_LOBBY;
				fprintf(lobby_log, "Delete client %d.%d.%d.%d\n",
					client_ip[i].IP1(), client_ip[i].IP2(), client_ip[i].IP3(), client_ip[i].IP4());				
			}
		}
		if (client_socket2[i].IsReady()) {
			size = client_socket2[i].GetOnce(packet, MAX_PACKET_SIZE);
			if (size > 0  && client_state[client_cont[i]] == LOBBY_MATCH) {
				client_socket2[client_cont[i]].SendData(packet, size);			
			}
		}
	}
	if (is_request) { 
		for (i = 0; i < MAX_PLAYER; i++){
			if (client_state[i] == LOBBY_WAIT || client_state[i] == LOBBY_ACCEPT) {
				client_socket[i].Send8(SEND_STATES);
				SendState(i);
			}
		}
	}
}

void Lobby2::SendState(int number) {
	int i;
	client_socket[number].Send8(player_number);
	client_socket[number].SendData(room_state, MAX_ROOM);		
	for (i = 0; i < MAX_ROOM; i++) {
		client_socket[number].SendData(client_name[room_state[i]], NAME_LENGTH);		
	}
}

int Lobby2::AddClient() {
	TCP_Socket new_socket, new_socket2;
	IP new_ip;
	int i, socket_number;

	server_socket.Accept(new_socket);
	if (!new_socket.WasOpen()) {
		emes("Lobby2::AddClient -> Can't accept client.");
		return SDL_FAILURE;
	}
	smes("Lobby2::AddClient -> Accept Client");
	new_ip.TCP_GetAddress(new_socket);
	fprintf(lobby_log, "Connect Start\n");

	for (i = 0; i < MAX_PLAYER; i++) {
		if (!client_socket[i].WasOpen()) {
			socket_number = i;		
			break;
		}
	}
	
	if (i == MAX_PLAYER) {
		new_socket.Send8(LOBBY_FULL);
		new_socket.Close();
		fprintf(lobby_log, "Lobby is full.\n");	
		return SDL_FAILURE;
	}

	for (i = 0; i < MAX_PLAYER; i++) {
		CheckSockets();
		if (server_socket.IsReady()) {
			break;
		}
		SDL_Delay(1);
	}
	server_socket.Accept(new_socket2);
	if (!new_socket2.WasOpen()) {
		new_socket.Send8(SOCKET2_ERROR);
		new_socket.Close();
		emes("Lobby2::AddClient -> Can't open data socket.");
		return SDL_FAILURE;
	}
	smes("Lobby2::AddClient -> Open Data Socket");
	client_socket2[socket_number] = new_socket2;
	socket_set.Add_TCP_Socket(client_socket2[socket_number]);

	new_socket.Send8(LOBBY_CONNECT);
	client_socket[socket_number] = new_socket;
	client_ip[socket_number] = new_ip;
	client_state[socket_number] = LOBBY_CERTIFY;
	wait_count[socket_number] = 0;
	socket_set.Add_TCP_Socket(client_socket[socket_number]);

	player_number++;

	fprintf(lobby_log, "Connet new player %s\n", new_ip.HostName());
	new_socket.Destroy();
	new_socket2.Destroy();
	return SDL_SUCCESS;
}

void Lobby2::HandleClient(int number) {
	int i;
	Uint8 data, room_number, room_client, o_number;
	TCP_Socket &socket = client_socket[number];
	IP &ip = client_ip[number];
	Uint8 &state = client_state[number];
	Uint8 *name = client_name[number];
//	Uint16 &port = client_port[number];
	Uint32 &count = wait_count[number];

	socket.GetOnce(&data, 1);
	
	switch (data) {
		case LEAVE_LOBBY:
			if (state == LOBBY_ACCEPT) {
				for (i = 0; i < MAX_ROOM; i++) {
					if (room_state[i] == number) {
						room_state[i] = EMPTY_ROOM;	 	
						break;
					}
				}
			}
			socket_set.Del_TCP_Socket(socket);
			socket.Close();
			player_number--;
			state = NO_LOBBY;
			fprintf(lobby_log, "Player leaves. %s\n", ip.HostName());
		break;			
		case SEND_CODE:
			Uint32 lobby_code;
			socket.GetOnce(&lobby_code, 4);
			socket.GetOnce(name, NAME_LENGTH);
			if (lobby_code != LOBBY_CODE) {
				socket.Send8(CODE_ERROR);
				socket_set.Del_TCP_Socket(socket);
				socket.Close();
				player_number--;
				state = NO_LOBBY;
				fprintf(lobby_log, "Certified code is diffrent. %d.%d.%d.%d\n",
					ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4());	
			} else {
				socket.Send8(CODE_SUCCESS);
				fprintf(lobby_log, "Certified Code OK %d.%d.%d.%d\n",
					ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4());	
				state = LOBBY_WAIT;
				count = 0;
    		}
		break;
		case REQUEST_STATES:
			SendState(i);
		break;
		case REQUEST_ROOM:
			socket.GetOnce(&room_number, 1);
			if (room_number < MAX_ROOM) {
				o_number = room_state[room_number];
				if (state == LOBBY_WAIT) {
					if (o_number== EMPTY_ROOM) {
						room_state[room_number] = number;	 	
						state = LOBBY_ACCEPT;
					}  else {
						state = LOBBY_MATCH;				
						client_cont[number] = o_number;
						client_cont[o_number] = number;

                     	socket.Send8(MATCH_BLUE);
//						socket.Send32(udp_host);
//						socket.Send16(udp_port[number]);						
	
						client_state[o_number] = LOBBY_MATCH;
						client_socket[o_number].Send8(MATCH_RED);	
//						client_socket[o_number].Send32(udp_host);	
//						client_socket[o_number].Send16(udp_port[o_number]);	

						room_state[room_number] = EMPTY_ROOM;
					}
				} else if (state == LOBBY_ACCEPT) {
					if (o_number == number) {
						room_state[room_number] = EMPTY_ROOM;	 	
						state = LOBBY_WAIT;				
					}
				}
			}
		break;
		case RETURN_LOBBY:
			state = LOBBY_WAIT;							
		break;	
		default:
		break;	
	}
}



LobbyClient2::LobbyClient2() {
	int i;
	for (i = 0; i < MAX_ROOM; i++) {
		room_name[i][NAME_LENGTH] = '\0';	
	}
}

LobbyClient2::~LobbyClient2() {
	Quit();
}

int LobbyClient2::Init() {
	if (WasInit()) {
		Quit();	
	}	
	if (SDLNet_Init() == SDL_FAILURE) {
		emes("LobbyClient2::Init");
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClient2::Init");	

	socket_set.Create(2);
	if (!socket_set.WasCreate()) {
		return SDL_FAILURE;
	}
	return SDL_SUCCESS;
}

void LobbyClient2::Quit() {
    if (WasInit()) {
		socket_set.Del_TCP_Socket(tcp_socket);
		tcp_socket.Close();
		socket_set.Del_TCP_Socket(tcp_socket2);
		tcp_socket2.Close();
		socket_set.Free();
		SDLNet_Quit();
		smes("LobbyClient2::Quit");	
	}
	emes("LobbyClient2::Quit");	
}

bool LobbyClient2::WasInit() {
	if (socket_set.WasCreate()) {
		return true;		
	}
	return false;	
}

int LobbyClient2::ConnectLobby(char* user_name, char* hostname, Uint16 port) {
	Uint8 lobby_state;
	int timeout_count;

	if (server_ip.ResolveHost(hostname, port)) {
		emes("LobbyClient2 -> Can't find lobby server.");
//		ehost(server_ip.HostName());
		rmes();
		return SDL_FAILURE;
	}
	smes("LobbyClient2 -> Find Lobby Server");
	shost(server_ip.HostName());

	tcp_socket.Open(server_ip);
	if (!tcp_socket.WasOpen()) {
		return SDL_FAILURE;
	}
	socket_set.Add_TCP_Socket(tcp_socket);

	tcp_socket2.Open(server_ip);
	if (!tcp_socket2.WasOpen()) {
		return SDL_FAILURE;
	}
	socket_set.Add_TCP_Socket(tcp_socket2);

	if (WaitData(500) == SDL_FAILURE) {
		emes("LobbyClient2 -> Time Out");
		return SDL_FAILURE;	
	}

	lobby_state = tcp_socket.Get8();
	switch (lobby_state) {
		case LOBBY_CONNECT :
			smes("LobbyClient2 -> Connect Lobby");
		break;	
		case LOBBY_FULL :
			emes("LobbyClient2 -> Lobby is full.");
			return SDL_FAILURE;				
		break;	
		case SOCKET2_ERROR :
			emes("LobbyClient2 -> Can't opne data socket.");
			return SDL_FAILURE;				
		break;	
		default:
			emes("LobbyClient2 -> Unknown Error");
			return SDL_FAILURE;	
		break;	
	}
	SendLobbyCode();
		
	tcp_socket.SendData(user_name, NAME_LENGTH);

	if (tcp_socket.Get8() == CODE_ERROR) {
		emes("LobbyClient2 -> Code Error");
		return SDL_FAILURE;				
	}

	return SDL_SUCCESS;
}

Uint8 LobbyClient2::Handle() {
	Uint8 data;
//	Uint16 port;
//	Uint32 host;
//	IP ip;

	tcp_socket.GetOnce(&data, 1);
	switch (data) {
		case SEND_STATES:
			GetState();
		break;			
		case MATCH_RED:
			SetCornerRed();
//			host = server_ip.Host();
//			port = tcp_socket.Get16();
//			SetDestination(host, port);			
//			ip.SetHost(host);
//			fprintf(stderr, "red port = %d\n", SDLNet_Read16(&port));
//			fprintf(stderr, "red ip = %d.%d.%d.%d\n", ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4()); 
		break;			
		case MATCH_BLUE:
			SetCornerBlue();
//			host = server_ip.Host();
//			port = tcp_socket.Get16();
//			SetDestination(host, port);			
//			ip.SetHost(host);
//			fprintf(stderr, "blue port = %d\n", SDLNet_Read16(&port));
//			fprintf(stderr, "blue ip = %d.%d.%d.%d\n", ip.IP1(), ip.IP2(), ip.IP3(), ip.IP4()); 
		break;			
		default:
		break;	
	}
	return data;
}

void LobbyClient2::GetState() {
	int i;
	player_number = tcp_socket.Get8();
	tcp_socket.GetData(room_state, MAX_ROOM);
	for (i = 0; i < MAX_ROOM; i++) {
		tcp_socket.GetData(room_name[i], NAME_LENGTH);
	}
}

int LobbyClient2::WaitData(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsReady()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}

int LobbyClient2::WaitData2(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsReady2()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}

/*
int LobbyClient2::WaitUDPData(int max_count) {
	int timeout_count;
	timeout_count = 0;
	while(1) {
		if (IsUDPReady()) {
			break;
		}
		if (timeout_count > max_count) {
			return SDL_FAILURE;
		}
		timeout_count++;
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}
 */
