#include "matchhandler.h"

void MatchHandler::Init(int MaxRound, int RoundTime,
		int TKOSystem, int JudgeSystem, int CountSystem, int ExtraRound, int KOCount,
		int Judge1Type, int Judge2Type, int Judge3Type,
		int ExtraInfo, int ReplayRate) {
	SetMinMax(&MaxRound, 1, 15);
	max_round = MaxRound;
	SetMinMax(&RoundTime, 10, 600);
	round_time = RoundTime*50;
	denominator = 180.0*50.0 / float(round_time);
	interval_time = (int)(180.0*50.0 / denominator);

	SetMinMax(&TKOSystem, 1, 255);
	tko_down = TKOSystem;

	if (JudgeSystem) {
		is_roundmust = false;
	} else {
		is_roundmust = true;
	}

	SetMinMax(&CountSystem, 0, 3);
	count_after_gong = CountSystem;

	if (ExtraRound) {
		is_extraround_system = true;	
	} else {
		is_extraround_system = false;
	}
	is_extraround = false;

	SetMinMax(&KOCount, 1, 10);
	ko_count = KOCount;
	
	judge1 = SelectJudge(Judge1Type);
	judge2 = SelectJudge(Judge2Type);
	judge3 = SelectJudge(Judge2Type);

	if (ExtraInfo) {
		show_extra_info = true;
	} else {
		show_extra_info = false;
	}

	SetMinMax(&ReplayRate, 1, 10);
	if (ReplayRate) {
		replay_rate = 50*ReplayRate;
	}
	if (replay_rate > round_time) {
		replay_rate = round_time;	
	}

	state = BEFORE_MATCH;
	result_state = NO_RESULT;
	extra_result_state = NO_RESULT;
	round = 1;
	time_count = 0;
	calc_mode = 0;
	save_calc_mode = 0;
	skip_interval = 0;
	match_info = 0;
}

int MatchHandler::SelectJudge(int c) {
	if (c < 0 || c > 3) {
		c = 0;	
	}
	if (c) {
		return c - 1;	
	} else {
		return 	random.Rand() % 3;
	}
}

void MatchHandler::PreReplay() {
	IsReplayTime();
	if (replay_number > 0) {
		replay_number--;
	}
	time_count = ReplayTime();
}	


void MatchHandler::NextReplay() {
	IsReplayTime();
	replay_number++;
	time_count = ReplayTime();
}	

void MatchHandler::SetReplay() {
	IsReplayTime();
	time_count = ReplayTime();
}	

bool MatchHandler::IsReplayTime() {
	replay_number = time_count/replay_rate;		
	return (time_count % replay_rate == 0);	
}

char* MatchHandler::RoundString() {
	if (is_extraround) {
		strcpy(round_str, "ExRound");	
	} else {
		sprintf(round_str, "Round%2d", round);
	}
	return round_str;
}

char* MatchHandler::MakeTimeString(int ctime) {
	static int minute, second, msecond;
	minute = ctime / 3000;
	second = (ctime - minute*3000) / 50;
	msecond = (ctime - minute*3000 - second*50) * 2;
	sprintf(time_str, "%2d:%02d:%02d", minute, second, msecond);
	return time_str;
}

char* MatchHandler::TimeString() {
	return MakeTimeString(time_count);
}

char* MatchHandler::RestTimeString() {
	return MakeTimeString(RestTime());
}

int MatchHandler::RestTime() {
	static int rest_time;
	rest_time = round_time - time_count;
	if (rest_time < 0) {
		rest_time = 0;	
	}
	return rest_time;
}

void MatchHandler::SetKOTime(Uint8 *ko_round, Uint8 *ko_minute, Uint8 *ko_second, Uint8 *ko_msecond) {
	if (is_extraround || IsJudge()) {
		*ko_round = 255;
	} else {
		*ko_round = round;	
	}

	static int ko_time;
	if (time_count > round_time) {
		ko_time = round_time;
	} else {
		ko_time = time_count;
	}

  	*ko_minute = ko_time / 3000;
	*ko_second = (ko_time - *ko_minute*3000) / 50;
	*ko_msecond = (ko_time - *ko_minute*3000 - *ko_second*50) * 2;
}

bool MatchHandler::IsExtraRound() {
	if (round == max_round + 1)	{
		is_extraround = true;
	} else {
		is_extraround = false;
	}
 	return is_extraround;
}

void MatchHandler::PreRound() {
	round--;
	if (round < 1) {
		round = 1;
	}	
	IsExtraRound();
}

void MatchHandler::NextRound() {
	round++;
	if (round > max_round + 1) {
		round = max_round + 1;
	}
	IsExtraRound();
}

void MatchHandler::SetRound(int cround) {
	round = cround;
	if (round < 1) {
		round = 1;		
	}
	if (round > max_round + 1) {
		round = max_round + 1;		
	}	
	IsExtraRound();
}		

void MatchHandler::SetTimeCount(int ctime_count) {
	if (ctime_count < 0) {
		ctime_count = 0;
	}
	time_count = ctime_count;
}
   
void MatchHandler::Judgement(BoxerHandler &handler1, BoxerHandler &handler2) {
	int score1, score2, rnd;
	Judge(&score1, &score2, judge1, handler1, handler2);
	handler1.SetJudge(0, score1);
	handler2.SetJudge(0, score2);	
	Judge(&score1, &score2, judge2, handler1, handler2);
	handler1.SetJudge(1, score1);
	handler2.SetJudge(1, score2);	
	Judge(&score1, &score2, judge3, handler1, handler2);
	handler1.SetJudge(2, score1);
	handler2.SetJudge(2, score2);	

	rnd = random.Rand() % 3;
	handler1.SetScore(rnd);
	handler2.SetScore(rnd);	
	handler1.TotalScoreInc();
	handler2.TotalScoreInc();
}

void MatchHandler::Judge(int *score1, int *score2, int judge,
		BoxerHandler &handler1, BoxerHandler &handler2) {
	switch (judge) {
		case JUDGE_CRITICAL:
			JudgeCritical(score1, score2, handler1, handler2);
		break;			
		case JUDGE_DAMAGE:
			JudgeDamage(score1, score2, handler1, handler2);
		break;		
		default:
			JudgeHit(score1, score2, handler1, handler2);
		break;	
	}
	if (*score1 < 0) {
		*score1 = 0;	
	}
	if (*score2 < 0) {
		*score2 = 0;	
	}
}      
   
void MatchHandler::JudgeHit(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2) {
	*score1 = 10;
	*score2 = 10;
	if (handler1.Down()) {
		*score1-= handler1.Down();
	}
	if (handler2.Down()) {
		*score2-= handler2.Down();
	}
	if (handler1.Down() == handler2.Down()) {
		if (handler1.Hit() == handler2.Hit()) {
			if (is_roundmust || is_extraround) {
				if (handler1.Critical() == handler2.Critical()) {
					if (handler1.RoundDamage() == handler2.RoundDamage()) {
						if (random.Rand() % 2) {
							(*score1)--;				
						} else {
							(*score2)--;			
						}			
					} else if (handler1.RoundDamage() > handler2.RoundDamage()) {
						(*score1)--;
					} else {
						(*score2)--;			
					}
				} else if (handler1.Critical() > handler2.Critical()) {
					(*score1)--;
				} else {
					(*score2)--;			
				}
			}
		} else if (handler1.Hit() > handler2.Hit()) {
			(*score1)--;
		} else {
			(*score2)--;
		}
	} else if (handler1.Down() > handler2.Down()) {
		(*score1)--;	
	} else {
		(*score2)--;
	}
}

void MatchHandler::JudgeCritical(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2) {
	*score1 = 10;
	*score2 = 10;
	if (handler1.Down()) {
		*score1-= handler1.Down();
	}
	if (handler2.Down()) {
		*score2-= handler2.Down();
	}
	if (handler1.Down() == handler2.Down()) {
		if (handler1.Critical() == handler2.Critical()) {
			if (is_roundmust || is_extraround) {
				if (handler1.RoundDamage() == handler2.RoundDamage()) {
					if (handler1.Hit() == handler2.Hit()) {
						if (random.Rand() % 2) {
							(*score1)--;				
						} else {
							(*score2)--;			
						}			
					} else if (handler1.Hit() > handler2.Hit()) {
						(*score1)--;
					} else {
						(*score2)--;			
					}
				} else if (handler1.RoundDamage() > handler2.RoundDamage()) {
					(*score1)--;
				} else {
					(*score2)--;			
				}
			}
		} else if (handler1.Critical() > handler2.Critical()) {
			(*score1)--;
		} else {
			(*score2)--;
		}
	} else if (handler1.Down() > handler2.Down()) {
		(*score1)--;	
	} else {
		(*score2)--;
	}
}

void MatchHandler::JudgeDamage(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2) {
	*score1 = 10;
	*score2 = 10;
	if (handler1.Down()) {
		*score1-= handler1.Down();
	}
	if (handler2.Down()) {
		*score2-= handler2.Down();
	}
	if (handler1.Down() == handler2.Down()) {
		if (handler1.RoundDamage()/100 == handler2.RoundDamage()/100) {
			if (is_roundmust || is_extraround) {
				if (handler1.Hit() == handler2.Hit()) {
					if (handler1.Critical() == handler2.Critical()) {
						if (random.Rand() % 2) {
							(*score1)--;				
						} else {
							(*score2)--;			
						}			
					} else if (handler1.Critical() > handler2.Critical()) {
						(*score1)--;
					} else {
						(*score2)--;			
					}
				} else if (handler1.Hit() > handler2.Hit()) {
					(*score1)--;
				} else {
					(*score2)--;			
				}
			}
		} else if (handler1.RoundDamage() > handler2.RoundDamage()) {
			(*score1)--;
		} else {
			(*score2)--;
		}
	}  else if (handler1.Down() > handler2.Down()) {
		(*score1)--;	
	} else {
		(*score2)--;
	}
}

void MatchHandler::Decision(BoxerHandler &handler1, BoxerHandler &handler2) {
	int i, vote1, vote2;
	vote1 = 0;
	vote2 = 0;
	handler1.SumJudge();
	handler2.SumJudge();
	
	for (i = 0; i < 3; i++) {
	 	if (handler1.TotalJudge(i) > handler2.TotalJudge(i)) {
			vote1++;		
		}
	 	if (handler2.TotalJudge(i) > handler1.TotalJudge(i)) {
			vote2++;		
		}	
	}	

	if (vote1 >= 2) {
		result_state = BLUE_WON_JUDGE;	
	} else if (vote2 >= 2) {
		result_state = RED_WON_JUDGE;
	} else {
		result_state = JUDGE_DRAW;	
	}
}


int MatchHandler::Situation(BoxerHandler &handler1, BoxerHandler &handler2) {
	int difference, rest_round;
	rest_round = max_round - round;	
	difference = handler2.TotalScore() - handler1.TotalScore();
	
	if (difference > 0) {
 		if (abs(difference) > rest_round * 2) {
			return 0;				
		} else if (abs(difference) > rest_round){
			return 1;						
		} else {
			return 2;								
		}
	}
	if (difference < 0) {
		if (abs(difference) > rest_round * 2) {
			return 6;						
		} else if (abs(difference) > rest_round) {
			return 5;								
		} else {
			return 4;										
		}
	}

	return 3;	
}

int MatchHandler::WhichWin(BoxerHandler &handler1, BoxerHandler &handler2) {
	int which_win;
	which_win = 0;

	if (IsRedWon()) {
		which_win = 2;
	}
	if (IsBlueWon()) {
		which_win = 1;
	}

	if (IsExtraRoundSystem()) {
		if (!which_win) {
			if (ExtraResultState() == RED_WON_KO ||
				ExtraResultState() == RED_WON_TKO) {
				which_win = 2;
			}
			if (ExtraResultState() == BLUE_WON_KO ||
				ExtraResultState() == BLUE_WON_TKO) { 
				which_win = 1;
			}
		}
		if (!which_win) {
			if (ExtraResultState() == DKO_DRAW || IsDKODraw()) {
				if (handler1.Damage() > handler2.Damage()) {
					which_win = 2;			
				} else if (handler2.Damage() > handler1.Damage()) {
					which_win = 1;
				} else {
					which_win = (random.Rand() % 2) + 1;			
				}
			}
		}
		if (!which_win) {
			if (handler2.JudgeInfo(MaxRound(), 0) +
				handler2.JudgeInfo(MaxRound(), 1) +
				handler2.JudgeInfo(MaxRound(), 2) >
				handler1.JudgeInfo(MaxRound(), 0) +
				handler1.JudgeInfo(MaxRound(), 1) +
				handler1.JudgeInfo(MaxRound(), 2)) {
				which_win = 2;
			} else {
				which_win = 1;			
			}
		}
		if (IsJudgeDraw()) {
			if (which_win == 2) {
				SetResultState(JUDGE_DRAW_RED);
			}
			if (which_win == 1) {
				SetResultState(JUDGE_DRAW_BLUE);
			}
		}
		if (IsDKODraw()) {
			if (which_win == 2) {
				SetResultState(DKO_DRAW_RED);
			}
			if (which_win == 1) {
				SetResultState(DKO_DRAW_BLUE);
			}
        }
	}

	return which_win;
}

void MatchHandler::SetMatchInfo(int region, int match_type) {
	match_info = (match_type << 8) | region;
}

int MatchHandler::LoadMatch(SDL_RWops *rwops) {
	if (SDL_RWread(rwops, this, sizeof(MatchHandler), 1)) {
		return SDL_SUCCESS;
	} else {
		return SDL_FAILURE;		
	}
}

int MatchHandler::SaveMatch(SDL_RWops *rwops) {
	if (SDL_RWwrite(rwops, this, sizeof(MatchHandler), 1)) {
		return SDL_SUCCESS;
	} else {
		return SDL_FAILURE;	
	}
}
 
