#ifndef MATCHHANDLER_H
#define MATCHHANDLER_H

#include "boxerhandler.h"
#include "random.h"

/* ̏ */	
enum {
	BEFORE_MATCH,
	SHOW_ROUND,
	MATCHING,
	ROUND_INTERVAL,
	END_MATCH,
	NO_DATA
};

/* ̌ */	
enum {
	NO_RESULT,
	RED_WON_KO,
	BLUE_WON_KO,
	RED_WON_TKO,
	BLUE_WON_TKO,
	DKO_DRAW,
	RED_WON_JUDGE,
	BLUE_WON_JUDGE,
	JUDGE_DRAW,
	JUDGE_DRAW_RED,
	JUDGE_DRAW_BLUE,
	DKO_DRAW_RED,
	DKO_DRAW_BLUE,
	NO_CONTEST_RED,
	NO_CONTEST_BLUE,
	NO_CONTEST	
};

/* EhĨJEg */
enum {
	ONLY_LASTROUND_STOP,
	ONLY_LASTROUND_CONTINUE,
	ALL_ROUND_STOP,
	ALL_ROUND_CONTINUE
};

/* WbW̃^Cv */
enum {
	JUDGE_HIT,
	JUDGE_CRITICAL,
	JUDGE_DAMAGE
};

class MatchHandler {
public:
	MatchHandler() {
		latest_round = 1;
//		Init();
	}
	void Init(int MaxRound, int RoundTime,
		int TKOSystem, int JudgeSystem, int CountSystem, int ExtraRound, int KOCount,
		int Judge1Type, int Judge2Type, int Judge3Type,
		int ExtraInfo, int ReplayRate);
	int State() const {return state;}
	void SetState(int cstate) {state = cstate;}
	void Matching() {state = MATCHING;}
	void RoundInterval() {
		state = ROUND_INTERVAL;
		InitSkipInterval();
		PushCalcMode();		
	}
	void ShowRound() {
		PopCalcMode();		
		state = SHOW_ROUND;
		show_round_count = 0;	
		InitSkipInterval();
	}
	void BeforeMatch() {state = BEFORE_MATCH;}
	void NoData() {state = NO_DATA;}
	bool IsBeforeMatch() const {return (state == BEFORE_MATCH);}
	bool IsShowRound() const {return (state == SHOW_ROUND);}
	bool IsMatching() const {return (state == MATCHING);}
	bool IsRoundInterval() const {return (state == ROUND_INTERVAL);}
	bool IsNoData() const {return (state == NO_DATA);}
	
	int ResultState() const {return result_state;}
	int ExtraResultState() const {return extra_result_state;}
	int RoundTime() const {return round_time;}
	int IntervalTime() const {return interval_time;}
	int ReplayRate() const {return replay_rate;}
	int KOCount() const {return ko_count;}
	int TKODown() const {return tko_down;}
	int MaxRound() const {return max_round;}
	int LimitRound() const {
		if (max_round-1 > 14) {
			return 14;		
		} else {
			return max_round-1;
		}
	}
	bool IsLastRound() const {return (round >= max_round);}
 	bool IsExtraRound();
 	bool IsExtraRoundSystem() const {return is_extraround_system;}
	void SetExtraResultState(int cresult_state){extra_result_state = cresult_state;}

	void NoResult(){result_state = NO_RESULT;}
	void SetResultState(int cresult_state){result_state = cresult_state;}
	void RedWonbyKO() {result_state = RED_WON_KO;}
	void RedWonbyTKO() {result_state = RED_WON_TKO;}
	void BlueWonbyKO() {result_state = BLUE_WON_KO;}
	void BlueWonbyTKO() {result_state = BLUE_WON_TKO;}
	void DKODraw() {result_state = DKO_DRAW;}
	void NoContest() {result_state = NO_CONTEST;}
	bool IsRedWonbyKO() const {return (result_state == RED_WON_KO);}
	bool IsRedWonbyTKO() const {return (result_state == RED_WON_TKO);}
	bool IsBlueWonbyKO() const {return (result_state == BLUE_WON_KO);}
	bool IsBlueWonbyTKO() const {return (result_state == BLUE_WON_TKO);}
	bool IsJudgeDraw() const {return (result_state == JUDGE_DRAW || result_state == JUDGE_DRAW_RED || result_state == JUDGE_DRAW_BLUE);}
	bool IsRedWonbyJudge() const {return (result_state == RED_WON_JUDGE);}
	bool IsBlueWonbyJudge() const {return (result_state == BLUE_WON_JUDGE);}
	bool IsDKODraw() const {return (result_state == DKO_DRAW || result_state == DKO_DRAW_RED || result_state == DKO_DRAW_BLUE);}
	bool IsRedWon() const {return (IsRedWonbyKO() || IsRedWonbyTKO() || IsRedWonbyJudge());}
	bool IsBlueWon() const {return (IsBlueWonbyKO() || IsBlueWonbyTKO() || IsBlueWonbyJudge());}
	bool IsDraw() const {return (IsDKODraw() || IsJudgeDraw());}
	bool IsJudge() const {return (IsRedWonbyJudge() || IsBlueWonbyJudge() || IsJudgeDraw());}
	bool IsKO() const {return !IsJudge();}
	bool IsTKO() const {return IsRedWonbyTKO() || IsBlueWonbyTKO();}
	bool IsNoContest() const {return (result_state == NO_CONTEST);}
	int WhichWin(BoxerHandler &handler1, BoxerHandler &handler2);

	void End() {
		state = END_MATCH;
		if (IsExtraRound()) {
			SetExtraResultState(ResultState());	
			SetResultState(JUDGE_DRAW);	
		}		
	}
	bool IsEnd() const {return (state == END_MATCH);}

	bool IsReplayTime();
	void SetReplay();
	void PreReplay();
	void NextReplay();
	int ReplayNumber() const {return replay_number;}
	int ReplayTime() const {return replay_number*replay_rate;}
	int Round() const {return round;}
	void PreRound();
	void NextRound();
	void SetRound(int cround);
	void SetLatestRound() {latest_round = round;}
	int LatestRound() const {return latest_round;}
	char* RoundString();
	
	char* MakeTimeString(int ctime);
	char* TimeString();
	char* RestTimeString();
	void SetKOTime(Uint8 *ko_round, Uint8 *ko_minute, Uint8 *ko_second, Uint8 *msecond);
	int TimeCount() const {return time_count;}
	int RestTime();
	void SetTimeCount(int ctime_count);
	void TickTime() {time_count++;}
	bool IsGong() const {return (time_count >= round_time);}

	int ShowRoundCount() const {return show_round_count;}
	bool RingGong() {
		return (show_round_count == 80);
	}
	bool HasShownRound() {
		return (show_round_count++ > 100);
	}
	float ShowRoundSize() {
		float size;
		size = ShowRoundCount()/25.0;
		if (size > 1.8) {
			size = 1.8;
		}
		return size;
	}
	
	void SetCounter(int count) {
		show_round_count = count;	
	} 
	bool IsCounterZero() {
		return (show_round_count-- < 0);
	} 
	bool IsCounterPlus() {
		return (show_round_count++ < 0);
	} 
	bool IsBox() {
		return (show_round_count == -48);
	} 
	
	void InitSkipInterval() {skip_interval = 0;}
	void SkipIntervalOn() {skip_interval = 25;}
	void SkipIntervalCount() {skip_interval++;}
	bool IsSkipInterval() const {return skip_interval>24;} 
	void InitCalcMode() {calc_mode = 0;}
	void CalcModeOn() {calc_mode = 1;}
	void CalcModeOff() {calc_mode = 0;}
	bool IsCalcMode() const {return (bool) calc_mode;} 
	void PushCalcMode() {save_calc_mode = calc_mode;}
	void PopCalcMode() {calc_mode = save_calc_mode;}

	int Situation(BoxerHandler &handler1, BoxerHandler &handler2);
	void Decision(BoxerHandler &handler1, BoxerHandler &handler2);
	void Judgement(BoxerHandler &handler1, BoxerHandler &handler2);   
	void Judge(int *score1, int *score2, int judge,
		BoxerHandler &handler1, BoxerHandler &handler2);
	void JudgeHit(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2);
	void JudgeCritical(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2);
	void JudgeDamage(int *score1, int *score2,
		BoxerHandler &handler1, BoxerHandler &handler2);
	int Judge1Type() const {return judge1;}
	int Judge2Type() const {return judge2;}
	int Judge3Type() const {return judge3;}	
	int SelectJudge(int c);
	bool ShowExtraInfo() const {return show_extra_info;}
	void ToggleExtraInfo() {
		if (show_extra_info) {
			show_extra_info = false;					
		} else {
			show_extra_info = true;							
		}	
	}

	bool CountOnlyLastRoundStop() const {return (count_after_gong == ONLY_LASTROUND_STOP);}
	bool CountOnlyLastRoundContinue() const {return (count_after_gong == ONLY_LASTROUND_CONTINUE);}
	bool CountAllRoundStop() const {return (count_after_gong == ALL_ROUND_STOP);}
	bool CountAllRoundContinue() const {return (count_after_gong == ALL_ROUND_CONTINUE);}
	bool CountMustbeContinuing() const {
		return CountAllRoundContinue() ||
			(CountOnlyLastRoundStop() && !IsLastRound()) || 
			(CountOnlyLastRoundContinue() && IsLastRound());
	}

	float Denominator() const {return denominator;}
	int Percent() const {return random.Percent();}

	void SetMatchInfo(int region, int match_type);
	int MatchRegion() const {return match_info & 0x00ff;}
	int MatchType() const {return (match_info & 0xff00) >> 8;}
	
	void Reset() {return random.Reset();}

	int LoadMatch(SDL_RWops *rwops);
	int SaveMatch(SDL_RWops *rwops);
private:
	int round_time;
	int interval_time;
	int max_round;
	int ko_count;
	int tko_down;
	int count_after_gong;
	float denominator;
		 
	bool is_roundmust;
	bool is_extraround;
	bool is_extraround_system;
	bool show_extra_info;
	int judge1;
	int judge2;
	int judge3;

	Random random;

	int round;
	int time_count;
	int replay_rate;

	int state;
	int result_state;
	int extra_result_state;
	int replay_number;
	int latest_round;
	int show_round_count;
	int skip_interval;
	int calc_mode, save_calc_mode; 
	int replay_pause_time;
	int match_info;

	char time_str[20];
	char round_str[20];
};

#endif //MATCHHANDLER_H
