#include "boxerhandler.h"

/* BCombo */

BCombo::BCombo() {
	first = NULL;
	second = NULL;
}

BCombo::BCombo(int number) {
	Init(number);
}

BCombo::~BCombo() {
	Free();
}

bool BCombo::Init(int number) {
	if (WasInit()) {
		Free();
	}
	max_number = number;
	current_number = 0;
	first = new int[max_number];
	second = new int[max_number];

	return WasInit();
}

void BCombo::Free() {
	if (WasInit()) {
		delete [] first;
		delete [] second;
		first = NULL;
		second = NULL;
		max_number = 0;
		current_number = 0;	
	}
}

void BCombo::SetCombo(int number, int first_number, int second_number) {
	first[number] = first_number;
	second[number] = second_number;
}


bool BCombo::IsCombo(int number, int number_next) {
	int i;
	for (i = 0; i < max_number; i++) {
		if ((first[i] == number) && (second[i] == number_next)) {
			return true;
		}	
	}
	return false;
}

bool BCombo::WasInit() {
	if (first != NULL && second != NULL) {
		return true;	
	}
	return false;
}

/* BMotion */
BMotion::BMotion() {
	bframes = NULL;
}

BMotion::BMotion(int number) {
	Init(number);
}

BMotion::~BMotion() {
	Free();
}

bool BMotion::Init(int number) {
	if (WasInit()) {
		Free();
	}
	max_number = number;
	current_number = 0;
	bframes = new BFrameList[max_number];

	return WasInit();
}

void BMotion::Free() {
	if (WasInit()) {
		delete[] bframes;
		bframes = NULL;
		max_number = 0;
		current_number = 0;	
	}
}

void BMotion::Set(Boxer *boxer) {
	bframes[current_number].Set(boxer);
}	
	
void BMotion::Set(int number, Boxer *boxer) {
	Choice(number);
	Set(boxer);
}

void BMotion::Get(Boxer *boxer) {
	bframes[current_number].Get(boxer);
}	
	
void BMotion::Get(int number, Boxer *boxer) {
	Choice(number);
	Get(boxer);
}

void BMotion::Add() {
	Insert(max_number);
}

void BMotion::Insert(int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	int i, max_now;
	BMotion buf;
	buf.Init(max_number);
	for (i = 0; i < max_number; i++) {
		buf[i] = bframes[i];
	}
	max_now = max_number;
	Init(max_now + 1);

	for (i = 0; i < max_now; i++) {
		if (i < number) {
			bframes[i] = buf[i];
		} else {
			bframes[i+1] = buf[i];		
		}
	}
}

void BMotion::Del(int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	int i, max_now;
	BMotion buf;
	buf.Init(max_number);
	for (i = 0; i < max_number; i++) {
		buf[i] = bframes[i];
	}
	max_now = max_number;
	Init(max_now - 1);

	for (i = 0; i < max_number; i++) {
		if (i < number) {
			bframes[i] = buf[i];
		} else {
			bframes[i] = buf[i+1];		
		}
	}
}

BMotion& BMotion::operator=(const BMotion &cbmotion) {
	int i;
	if (this->Count() != cbmotion.Count()) {
		this->Free();
		this->Init(cbmotion.Count());
	}
	for (i = 0; i < cbmotion.Count(); i++) {
		this->bframes[i] = cbmotion.bframes[i];	
	}
	return *this;
} 

BFrameList& BMotion::operator[](int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	if (number >= 0 && number < max_number) {
		current_number = number;
		return bframes[current_number];
	} else {
		return bframes[0];
	}
}

void BMotion::Choice(int number) {
	if (number >= 0 && number < max_number) {
		current_number = number;
	}
}

void BMotion::Link() {
	int i;
	total_cut = 0;

	bframes[0].SetPre(bframes + Count() -1);	
	for (i = 1; i < Count(); i++) {
		bframes[i].SetPre(bframes + i - 1);	
	}
	for (i = 0; i < Count() - 1; i++) {
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}
	bframes[Count() - 1].SetNext(bframes);	

	total_cut += bframes[Count() - 1].GetNumber();
	SetFrame(0);
}

void BMotion::SetFrame(int number) {
	int i, sum_cuts = 0;
	for (i = 0; i < Count(); i++) {
		if (sum_cuts <= number && number < sum_cuts + bframes[i].GetNumber()) {
			current_frame = i;
			current_cut = number - sum_cuts;		
			cut_number = number;
			NextFrame();
			return ;
		} else {
			sum_cuts += bframes[i].GetNumber();		
		}	
	}
	current_frame = 0;
	current_cut = 0;
	cut_number = 0;
	NextFrame();
}
	
void BMotion::NextFrame() {
	cut_number++;
	if (current_cut >= bframes[current_frame].GetNumber()){
		current_cut = 0;	
		current_frame++;
		if (current_frame >= Count()) {
			current_frame = 0;
			cut_number = 1;
		}
	}
	frame.Interpolate((BFrame*)(&bframes[current_frame]), (BFrame*)(&bframes[current_frame].GetNext()), current_cut, bframes[current_frame].GetNumber());
	current_cut++;
}

void BMotion::PreFrame() {
	cut_number--;
	if (current_cut < 0){
		current_frame--;
		if (current_frame < 0) {
			current_frame = Count() - 1;
			cut_number = total_cut;
		}
		current_cut = bframes[current_frame].GetNumber();	
	}
	frame.Interpolate((BFrame*)(&bframes[current_frame]), (BFrame*)(&bframes[current_frame].GetNext()), current_cut, bframes[current_frame].GetNumber());
	current_cut--;
}


bool BMotion::WasInit() {
	if (bframes == NULL) {
		return false;	
	}
	return true;	
}

void BMotion::Reverse() {
	int i;
	float swap;
	for (i = 0; i < MaxNumber(); i++) {
		bframes[i][0] *= -1.0;
		bframes[i][4] *= -1.0;
		bframes[i][5] *= -1.0;

        swap = bframes[i][6];
		bframes[i][6] = -bframes[i][11];
		bframes[i][11] = -swap;

        swap = bframes[i][7];
		bframes[i][7] = bframes[i][12];
		bframes[i][12] = swap;

        swap = bframes[i][8];
		bframes[i][8] = -bframes[i][13];
		bframes[i][13] = -swap;

        swap = bframes[i][9];
		bframes[i][9] = bframes[i][14];
		bframes[i][14] = swap;

        swap = bframes[i][10];
		bframes[i][10] = -bframes[i][15];
		bframes[i][15] = -swap;
	
        swap = bframes[i][16];
		bframes[i][16] = bframes[i][24];
		bframes[i][24] = swap;

		swap = bframes[i][17];
		bframes[i][17] = bframes[i][25];
		bframes[i][25] = swap;

        swap = bframes[i][18];
		bframes[i][18] = -bframes[i][26];
		bframes[i][26] = -swap;

        swap = bframes[i][19];
		bframes[i][19] = -bframes[i][27];
		bframes[i][27] = -swap;

		swap = bframes[i][20];
		bframes[i][20] = bframes[i][28];
		bframes[i][28] = swap;

		swap = bframes[i][21];
		bframes[i][21] = bframes[i][29];
		bframes[i][29] = swap;

        swap = bframes[i][22];
		bframes[i][22] = -bframes[i][30];
		bframes[i][30] = -swap;

        swap = bframes[i][23];
		bframes[i][23] = 360 - bframes[i][31];
		bframes[i][31] = 360 - swap;

		bframes[i][33] *= -1.0;
		bframes[i][34] *= -1.0;
		bframes[i][36] *= -1.0;
		bframes[i][37] *= -1.0;
		bframes[i][39] *= -1.0;
		bframes[i][40] *= -1.0;
		bframes[i][42] *= -1.0;
		bframes[i][43] *= -1.0;
	}
}

void BMotion::OffSet(Boxer &calc) {
	int i;
	float offset, minz = 1.0e10;

	for (i = 0; i < MaxNumber(); i++) {
		minz = 1.0e10;
		Set(i, &calc);
		calc.CalcCoords();
		if (calc.right_arm.fist.Z() < minz) {
			minz = calc.right_arm.fist.Z();		
		}
		if (calc.left_arm.fist.Z() < minz) {
			minz = calc.left_arm.fist.Z();		
		}
		if (calc.right_leg.toe.Z() < minz) {
			minz = calc.right_leg.toe.Z();		
		}
		if (calc.left_leg.toe.Z() < minz) {
			minz = calc.left_leg.toe.Z();		
		}
		if (calc.head.face.Z() < minz) {
			minz = calc.head.face.Z();		
		}
		if (calc.Z() < minz) {
			minz = calc.Z();		
		}
		offset = 0.0 - minz;
		bframes[i][2] += offset;
	}
}


/* HandleMotion */
void HandleMotion::Action(const BFrame &base, BMotion &bmotion, float speed) {
	int i, last;
	last = bmotion.Count();

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);	
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);	
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(base, bmotion[0].GetNumber());
	bframes[last].ChangeNumber(speed);	
	bframes[last].SetPre(bframes + last - 1);	
	bframes[last].SetNext(bframes);	
	bframes[last][0] = bframes[last - 1][0];	
	bframes[last][1] = bframes[last - 1][1];	

	first_cut = bframes[0].GetNumber();
	half_cut = (total_cut + 1) / 2;
	combo_cut = total_cut - bframes[last-1].GetNumber();
}

void HandleMotion::ActionDuringMove(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed) {
	int i, last;
	last = bmotion.Count();

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);	
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	bframes[0][0] = bmotion[0][0];
	bframes[0][1] = bmotion[0][1];
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);	
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(finish, 0);
	bframes[last].ChangeNumber(speed);	
	bframes[last].SetPre(bframes + last - 1);	
	bframes[last].SetNext(bframes);	
	bframes[last][0] = bframes[last-1][0];	
	bframes[last][1] = bframes[last-1][1];	

	first_cut = bframes[0].GetNumber();
	half_cut = (total_cut + 1) / 2;
	combo_cut = total_cut - bframes[last-1].GetNumber();
}

void HandleMotion::ComboAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed) {
	int i, last;
	last = bmotion.Count();

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(finish, 0);
	bframes[last].ChangeNumber(speed);
	bframes[last].SetPre(bframes + last - 1);	
	bframes[last].SetNext(bframes);	
	bframes[last][0] = bframes[last-1][0];	
	bframes[last][1] = bframes[last-1][1];	

	first_cut = bframes[0].GetNumber();
	half_cut = (total_cut + 1) / 2;
	combo_cut = total_cut - bframes[last-1].GetNumber();
}

void HandleMotion::Move(const BFrame &base, BMotion &bmotion, float speed) {
	int i, last;
	last = bmotion.Count() -1;

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(base, bmotion[last].GetNumber());
	bframes[last].ChangeNumber(speed);
	bframes[last].SetPre(bframes + last - 1);
	bframes[last].SetNext(bframes);
	bframes[last][0] = bmotion[last][0];
	bframes[last][1] = bmotion[last][1];
}

void HandleMotion::ActionDuringAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed) {
	int i, last;
	last = bmotion.Count();

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);	
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);	
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(finish, 0);
	bframes[last].ChangeNumber(speed);	
	bframes[last].SetPre(bframes + last - 1);	
	bframes[last].SetNext(bframes);	

	first_cut = bframes[0].GetNumber();
	half_cut = (total_cut + 1) / 2;
	combo_cut = total_cut - bframes[last-1].GetNumber();
}

void HandleMotion::DamageDuringAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float damage, float speed) {
	int i, last;
	last = bmotion.Count();

	bframes[0].From(base, bmotion[0].GetNumber());
	bframes[0].ChangeNumber(speed);	
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	total_cut = bframes[0].GetNumber();

	for (i = 1; i < last; i++) {
		bframes[i] = bmotion[i];
		bframes[i].ChangeNumber(speed);	
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
		bframes[i][1] += bframes[0][1] - damage * (float) i / (float) last;	
		total_cut += bframes[i].GetNumber();
	}

	bframes[last].From(finish, 0);
	bframes[last].ChangeNumber(speed);	
	bframes[last].SetPre(bframes + last - 1);	
	bframes[last].SetNext(bframes);	
	bframes[last][1] += bframes[0][1] - damage;	

	first_cut = bframes[0].GetNumber();
	half_cut = (total_cut + 1) / 2;
	combo_cut = total_cut - bframes[last-1].GetNumber();
}


HandleMotion& HandleMotion::operator=(const HandleMotion &chmotion) {
	int i;
	if (this->Count() != chmotion.Count()) {
		this->Free();
		this->Init(chmotion.Count());
	}
	for (i = 0; i < chmotion.Count(); i++) {
		this->bframes[i] = chmotion.bframes[i];	
	}
	this->frame = chmotion.frame;

	this->total_cut = chmotion.total_cut;	
	this->cut_number = chmotion.cut_number;	
	this->current_cut = chmotion.current_cut;	
	this->current_frame = chmotion.current_frame;	
	this->max_number = chmotion.max_number;
	this->current_number = chmotion.current_number;
	this->first_cut = chmotion.first_cut;
	this->half_cut = chmotion.half_cut;	
	this->combo_cut = chmotion.combo_cut;
	return *this;
} 

void HandleMotion::Link() {
	int i, last;
	last = Count() - 1;
	bframes[0].SetPre(bframes + last);	
	bframes[0].SetNext(bframes + 1);	
	for (i = 1; i < last; i++) {
		bframes[i].SetPre(bframes + i - 1);	
		bframes[i].SetNext(bframes + i + 1);	
	}
	bframes[last].SetPre(bframes + last -1);	
	bframes[last].SetNext(bframes);
}

void HandleMotion::Save(SDL_RWops *rwops) {
	SDL_RWwrite(rwops, bframes, sizeof(BFrameList), Count());
	SDL_RWwrite(rwops, &frame, sizeof(BFrame), 1);

	SDL_RWwrite(rwops, &total_cut, sizeof(int), 1);
	SDL_RWwrite(rwops, &cut_number, sizeof(int), 1);
	SDL_RWwrite(rwops, &current_cut, sizeof(int), 1);
	SDL_RWwrite(rwops, &current_frame, sizeof(int), 1);
	SDL_RWwrite(rwops, &max_number, sizeof(int), 1);
	SDL_RWwrite(rwops, &current_number, sizeof(int), 1);
	SDL_RWwrite(rwops, &first_cut, sizeof(int), 1);
	SDL_RWwrite(rwops, &half_cut, sizeof(int), 1);
	SDL_RWwrite(rwops, &combo_cut, sizeof(int), 1);
}

void HandleMotion::Load(SDL_RWops *rwops) {
	SDL_RWread(rwops, bframes, sizeof(BFrameList), Count());
	SDL_RWread(rwops, &frame, sizeof(BFrame), 1);

	SDL_RWread(rwops, &total_cut, sizeof(int), 1);
	SDL_RWread(rwops, &cut_number, sizeof(int), 1);
	SDL_RWread(rwops, &current_cut, sizeof(int), 1);
	SDL_RWread(rwops, &current_frame, sizeof(int), 1);
	SDL_RWread(rwops, &max_number, sizeof(int), 1);
	SDL_RWread(rwops, &current_number, sizeof(int), 1);
	SDL_RWread(rwops, &first_cut, sizeof(int), 1);
	SDL_RWread(rwops, &half_cut, sizeof(int), 1);
	SDL_RWread(rwops, &combo_cut, sizeof(int), 1);
}	 

// MotionHandler
MotionHandler::MotionHandler() {
	bmotions = NULL;
}

MotionHandler::MotionHandler(int number) {
	Init(number);
}

MotionHandler::~MotionHandler() {
	Free();
}

int MotionHandler::Init(int number) {
	if (WasInit()) {
		Free();
	}
	current_number = 0;
	max_number = number;
	bmotions = new BMotion[max_number];

	return WasInit();
}

void MotionHandler::Free() {
	if (WasInit()) {
		delete [] bmotions;
		bmotions = NULL;
		max_number = 0;
		current_number = 0;	
	}
}

void MotionHandler::Set(Boxer *boxer) {
	bmotions[current_number].Set(boxer);
}	
	
void MotionHandler::Set(int number, Boxer *boxer) {
	Choice(number);
	Set(boxer);
}

void MotionHandler::Get(Boxer *boxer) {
	bmotions[current_number].Get(boxer);
}	
	
void MotionHandler::Get(int number, Boxer *boxer) {
	Choice(number);
	Get(boxer);
}

void MotionHandler::Add() {
	Insert(max_number);
}

void MotionHandler::Insert(int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	int i, max_now;
	MotionHandler buf;
	buf.Init(max_number);
	for (i = 0; i < max_number; i++) {
		buf[i] = bmotions[i];
	}
	max_now = max_number;
	Free();
	Init(max_now + 1);

	for (i = 0; i < max_now; i++) {
		if (i < number) {
			bmotions[i] = buf[i];
		} else {
			bmotions[i+1] = buf[i];		
		}
	}
	max_number++;
}

void MotionHandler::Del(int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	int i, max_now;
	MotionHandler buf;
	buf.Init(max_number);
	for (i = 0; i < max_number; i++) {
		buf[i] = bmotions[i];
	}
	max_now = max_number;
	Free();
	Init(max_now - 1);
	
	for (i = 0; i < max_number; i++) {
		if (i < number) {
			bmotions[i] = buf[i];
		} else {
			bmotions[i] = buf[i+1];		
		}
	}
	max_number--;
}

MotionHandler& MotionHandler::operator=(const MotionHandler &cbmotions) {
	int i;
	if (this->Count() != cbmotions.Count()) {
		this->Free();
		this->Init(cbmotions.Count());
	}
	for (i = 0; i < cbmotions.Count(); i++) {
		bmotions[i] = cbmotions.bmotions[i];	
	}
	return *this;
} 

BMotion& MotionHandler::operator[](int number) {
	if (number == CURRENT_FRAME) {
		number = current_number;	
	}
	if (number >= 0 && number < max_number) {
		current_number = number;
		return bmotions[current_number];
	} else {
		return bmotions[0];
	}
}

void MotionHandler::Choice(int number) {
	if (number >= 0 && number < max_number) {
		current_number = number;
	}
}

bool MotionHandler::WasInit() {
	if (bmotions == NULL) {
		return false;	
	}
	return true;	
}

void MotionHandler::Reverse() {
	int i, swap;
	for (i = 0; i < MaxNumber(); i++) {
		bmotions[i].Reverse();
	}
	
	swap = assign[RIGHT_N];
	assign[RIGHT_N] = assign[LEFT_N];
	assign[LEFT_N] = swap;

	swap = assign[RIGHT_F];
	assign[RIGHT_F] = assign[LEFT_F];
	assign[LEFT_F] = swap;

	swap = assign[RIGHT_B];
	assign[RIGHT_B] = assign[LEFT_B];
	assign[LEFT_B] = swap;

	swap = assign[RIGHT_R];
	assign[RIGHT_R] = assign[LEFT_R];
	assign[LEFT_R] = swap;

	swap = assign[RIGHT_L];
	assign[RIGHT_L] = assign[LEFT_L];
	assign[LEFT_L] = swap;
	
	swap = assign[RIGHT_RF];
	assign[RIGHT_RF] = assign[LEFT_RF];
	assign[LEFT_RF] = swap;

	swap = assign[RIGHT_LF];
	assign[RIGHT_LF] = assign[LEFT_LF];
	assign[LEFT_LF] = swap;

	swap = assign[RIGHT_RB];
	assign[RIGHT_RB] = assign[LEFT_RB];
	assign[LEFT_RB] = swap;

	swap = assign[RIGHT_LB];
	assign[RIGHT_LB] = assign[LEFT_LB];
	assign[LEFT_LB] = swap;

	swap = assign[DEFENSE_R];
	assign[DEFENSE_R] = assign[DEFENSE_L];
	assign[DEFENSE_L] = swap;

	swap = assign[MOVE_RIGHT];
	assign[MOVE_RIGHT] = assign[MOVE_LEFT];
	assign[MOVE_LEFT] = swap;
}

void MotionHandler::OffSet(Boxer &calc) {
	int i;
	for (i = 0; i < MaxNumber(); i++) {
		bmotions[i].OffSet(calc);
	}
}
