#ifndef MOTIONHANDLER_H
#define MOTIONHANDLER_H

#include "bframe.h"

enum {
	BASIC_PAUSE,
	RHYTHM_MOTION,
	MOVE_FORWARD,
	MOVE_BACK,
	MOVE_RIGHT,
	MOVE_LEFT,
	GET_HIT_BODY,
	GET_HIT_FACE,
	DOWNING_BODY,
	DOWNING_FACE,	
	RIGHT_N,
	RIGHT_F,
	RIGHT_B,
	RIGHT_R,
	RIGHT_L,
	RIGHT_RF,
	RIGHT_LF,
	RIGHT_RB,
	RIGHT_LB,
	LEFT_N,
	LEFT_F,
	LEFT_B,
	LEFT_R,
	LEFT_L,
	LEFT_RF,
	LEFT_LF,
	LEFT_RB,
	LEFT_LB,
	DEFENSE_N,
	DEFENSE_F,
	DEFENSE_B,
	DEFENSE_R,
	DEFENSE_L,
	DEFENSE_RF,
	DEFENSE_LF,
	DEFENSE_RB,
	DEFENSE_LB,
	CLINCH_MOTION,
	DEADLY1,
	DEADLY2,
	DEADLY3,
	DEADLY4,
	MAX_ASSIGN		
};

enum {
	MAKE_BOXER_SOURCE,
	MAKE_BODY_SOURCE,
	MAKE_MOTION_SOURCE,
	MAKE_THINK_SOURCE
};

class BoxerHandler;

class BCombo {
public:
	BCombo();
	BCombo(int number);
	~BCombo();

	bool Init(int number);
	void Free();

	bool IsCombo(int number, int number_next);
	bool WasInit();
	void SetCombo(int number, int first_number, int second_number);
 	int *GetComboFirst() const {return first;}
 	int *GetComboSecond() const {return second;}	
	int MaxNumber() const {return max_number;} 
private:
	int *first, *second;
	int max_number, current_number;
};


class BMotion {
public:
	BMotion();
	BMotion(int number);
	~BMotion();

	bool Init(int number);
	void Free();

	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Set(int number, Boxer *boxer);	
	void Get(int number, Boxer *boxer);	
	void Link();	
	
	void Add();
	void Del(int number = CURRENT_FRAME);
	void Insert(int number = CURRENT_FRAME);

	int TotalCut() const {return total_cut;}
	int CutNumber() const {return cut_number;}
	int Count() const {return max_number;}
	void Choice(int number);
	bool WasInit();

	BFrameList& operator[](int number);
	void SetFrame(int number);
	void PreFrame();
	void NextFrame();
	BFrame& CurrentFrame() {return frame;}
	BMotion& operator=(const BMotion &cbmotion);
	int MaxNumber() const {return max_number;} 

	void Reverse();
	void OffSet(Boxer &calc);
protected:
	BFrame frame;
	BFrameList *bframes;
	int total_cut;	
	int cut_number;	
	int current_cut;	
	int current_frame;	
	int max_number, current_number;
};


class HandleMotion : public BMotion {
public:
	void Action(const BFrame &fbase, BMotion &bmotion, float speed = 1.0);
	void ActionDuringMove(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed = 1.0);
	void ComboAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed = 1.0);
	void ActionDuringAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float speed = 1.0);
	void DamageDuringAction(const BFrame &base, const BFrame &finish, BMotion &bmotion, float damage, float speed = 1.0);
	void Move(const BFrame &base, BMotion &bmotion, float speed = 1.0);
	
	int FirstCut() const {return first_cut;}
	int HalfCut() const {return half_cut;}
	int ComboCut() const {return combo_cut;}
	HandleMotion& operator=(const HandleMotion &chmotion); 
	void Link();
	void Save(SDL_RWops *rwops);
	void Load(SDL_RWops *rwops);	 
private:
	int first_cut;
	int half_cut;	
	int combo_cut;
};


class MotionHandler {
public:
	MotionHandler();
	MotionHandler(int number);
	~MotionHandler();

	int Init(int number);
	void Free();

	void Set(Boxer *boxer);	
	void Get(Boxer *boxer);	
	void Set(int number, Boxer *boxer);	
	void Get(int number, Boxer *boxer);	
	
	void Add();
	void Del(int number = CURRENT_FRAME);
	void Insert(int number = CURRENT_FRAME);

	int Count() const {return max_number;}
	int MaxNumber() const {return max_number;}
	void Choice(int number);
	bool WasInit();
	BMotion& operator[](int number);
	MotionHandler& operator=(const MotionHandler &cbmotions);

	void SetRhythm() {bmotions[assign[RHYTHM_MOTION]].SetFrame(0);}
	void NextRhythm() {bmotions[assign[RHYTHM_MOTION]].NextFrame();}
	BFrame& MotionRhythm() {return bmotions[assign[RHYTHM_MOTION]].CurrentFrame();}

	void SetAction(int number, float speed = 1.0) {
		hmotion.Action(MotionRhythm(), bmotions[assign[number]], speed);
		hmotion.SetFrame(0);
	}
	void SetActionDuringMove(int number, float speed = 1.0) {
		hmotion.ActionDuringMove(MotionMove(), MotionRhythm(), bmotions[assign[number]], speed);
		hmotion.SetFrame(0);
	}
	void SetComboAction(int number, float speed = 1.0) {
		hmotion.ComboAction(MotionAction(), MotionRhythm(), bmotions[assign[number]], speed);
		hmotion.SetFrame(0);
	}

	void NextAction() {hmotion.NextFrame();}
	BFrame& MotionAction() {return hmotion.CurrentFrame();}
	bool IsActionEnd() {
		if (hmotion.CutNumber() == hmotion.TotalCut()){ 
			return true;
		}
		return false;		
	}
	bool IsActionHalf() {
		if (hmotion.CutNumber() == hmotion.HalfCut()){ 
			return true;
		}
		return false;		
	}
	bool IsActionOverHalf() {
		if (hmotion.CutNumber() > hmotion.HalfCut()){ 
			return true;
		}
		return false;		
	}
	bool IsActionOverFirst() {
		if (hmotion.CutNumber() > hmotion.FirstCut()){ 
			return true;
		}
		return false;		
	}
	bool IsActionCombo() {
		if (hmotion.CutNumber() == hmotion.ComboCut()){ 
			return true;
		}
		return false;		
	}

	void NextActionBlocked() {hmotion.PreFrame();}
	bool IsActionBack() {
		if (hmotion.CutNumber() == 0){ 
			return true;
		}
		return false;		
	}

	void SetMove(int number, float speed = 1.0) {
		hmotion.Move(MotionRhythm(), bmotions[assign[number]], speed);
		hmotion.SetFrame(0);
	}
	void NextMove() {hmotion.NextFrame();}
	BFrame& MotionMove() {return hmotion.CurrentFrame();}
	bool IsMoveEnd() {
		if (hmotion.CutNumber() == hmotion.TotalCut()){ 
			return true;
		}
		return false;		
	}

	bool IsCombo(int number1, int number2) {
		if (combo.IsCombo(assign[number1], assign[number2])) {
			return true;
		}
		return false;		
	}

	void SetDamagedFace(float damage, bool isrhythm) {
		if (isrhythm) {
			hmotion.DamageDuringAction(MotionRhythm(), MotionRhythm(), bmotions[assign[GET_HIT_FACE]], damage);
		} else {
			hmotion.DamageDuringAction(hmotion.CurrentFrame(), MotionRhythm(), bmotions[assign[GET_HIT_FACE]], damage);
		}
		hmotion.SetFrame(0);
	}
	void SetDamagedBody(float damage, bool isrhythm) {
		if (isrhythm) {
			hmotion.DamageDuringAction(MotionRhythm(), MotionRhythm(), bmotions[assign[GET_HIT_BODY]], damage);
		} else {
			hmotion.DamageDuringAction(hmotion.CurrentFrame(), MotionRhythm(), bmotions[assign[GET_HIT_BODY]], damage);
		}
		hmotion.SetFrame(0);
	}
	void NextDamaged() {hmotion.NextFrame();}
	BFrame& MotionDamaged() {return hmotion.CurrentFrame();}
	bool IsDamagedEnd() {
		if (hmotion.CutNumber() == hmotion.TotalCut()){ 
			return true;
		}
		return false;		
	}

	void SetDowningFace(bool isrhythm) {
		if (isrhythm) {
			hmotion.Action(MotionRhythm(), bmotions[assign[DOWNING_FACE]]);		
		} else {
			hmotion.ActionDuringAction(hmotion.CurrentFrame(), MotionRhythm(), bmotions[assign[DOWNING_FACE]]);
		}
		hmotion.SetFrame(0);
	}
	void SetDowningBody(bool isrhythm) {
		if (isrhythm) {
			hmotion.Action(MotionRhythm(), bmotions[assign[DOWNING_BODY]]);		
		} else {
			hmotion.ActionDuringAction(hmotion.CurrentFrame(), MotionRhythm(), bmotions[assign[DOWNING_BODY]]);
		}
		hmotion.SetFrame(0);
	}
	void NextDowning() {hmotion.NextFrame();}
	BFrame& MotionDowning() {return hmotion.CurrentFrame();}
	bool IsDowningEnd() {
		if (hmotion.CutNumber() == hmotion.ComboCut()){ 
			return true;
		}
		return false;		
	}

	void SetRestore() {
		hmotion.ComboAction(MotionDowning(), MotionRhythm(), bmotions[assign[DOWNING_BODY]]);
		hmotion.SetFrame(0);
	}
	void NextRestore() {hmotion.NextFrame();}
	BFrame& MotionRestore() {return hmotion.CurrentFrame();}
	bool IsRestoreEnd() {
		if (hmotion.CutNumber() == hmotion.TotalCut()){ 
			return true;
		}
		return false;		
	}

	void SetClinching(bool isrhythm) {
		if (isrhythm) {
			hmotion.Action(MotionRhythm(), bmotions[assign[CLINCH_MOTION]]);		
		} else {
			hmotion.ActionDuringAction(hmotion.CurrentFrame(), MotionRhythm(), bmotions[assign[CLINCH_MOTION]]);
		}
		hmotion.SetFrame(0);
	}
	void NextClinching() {hmotion.NextFrame();}
	BFrame& MotionClinching() {return hmotion.CurrentFrame();}
	bool IsClinchingEnd() {
		if (hmotion.CutNumber() == hmotion.TotalCut()){ 
			return true;
		}
		return false;		
	}

	void Reverse();
	void OffSet(Boxer &calc);
	
private:
	int assign[MAX_ASSIGN];
	BCombo combo;
	BMotion *bmotions;
	HandleMotion hmotion;
	int max_number, current_number;
	friend class BoxerHandler;
	friend class MotionInfo;
};

#endif // MOTIONHANDLER_H
