#include "csdl.h"
#include "creader.h"
#include "boxer.h"
#include "boxerhandler.h"
#include "bframe.h"
#include "motionhandler.h"
#include "replayinfo.h"

bool MotionInfo::Init(MotionHandler &motion) {
	if (WasInit()) {
		Free();	
	}
	hmotion.Init(motion.hmotion.Count());
	return WasInit();
}

void MotionInfo::Free() {
	if (WasInit()) {
		hmotion.Free();	
	}
}	

void MotionInfo::GetInfo(MotionHandler &motion) {
	hmotion = motion.hmotion;
	current_rhythm = motion.bmotions[motion.assign[RHYTHM_MOTION]].CutNumber();
}

void MotionInfo::SetInfo(MotionHandler &motion) {
	motion.hmotion = hmotion;
	motion.hmotion.Link();
	motion.bmotions[motion.assign[RHYTHM_MOTION]].SetFrame(current_rhythm-1);
//	fprintf(stderr, "cr = %d  cn = %d\n", current_rhythm, motion.bmotions[motion.assign[RHYTHM_MOTION]].CutNumber());
}

void MotionInfo::SaveInfo(SDL_RWops *rwops) {
	SDL_RWwrite(rwops, &current_rhythm, sizeof(int), 1);	
	hmotion.Save(rwops);
}

void MotionInfo::LoadInfo(SDL_RWops *rwops) {
	SDL_RWread(rwops, &current_rhythm, sizeof(int), 1);	
	hmotion.Load(rwops);
}

void BoxerInfo::GetInfo(Boxer &boxer) {
	x[0] = boxer.right_arm.fist.XPre();
	y[0] = boxer.right_arm.fist.YPre();
	z[0] = boxer.right_arm.fist.ZPre();
	x[1] = boxer.right_arm.forearm.XPre();
	y[1] = boxer.right_arm.forearm.YPre();
	z[1] = boxer.right_arm.forearm.ZPre();
	x[2] = boxer.right_arm.upperarm.XPre();
	y[2] = boxer.right_arm.upperarm.YPre();
	z[2] = boxer.right_arm.upperarm.ZPre();
	x[3] = boxer.right_arm.shoulder.XPre();
	y[3] = boxer.right_arm.shoulder.YPre();
	z[3] = boxer.right_arm.shoulder.ZPre();
	x[4] = boxer.left_arm.fist.XPre();
	y[4] = boxer.left_arm.fist.YPre();
	z[4] = boxer.left_arm.fist.ZPre();
	x[5] = boxer.left_arm.forearm.XPre();
	y[5] = boxer.left_arm.forearm.YPre();
	z[5] = boxer.left_arm.forearm.ZPre();
	x[6] = boxer.left_arm.upperarm.XPre();
	y[6] = boxer.left_arm.upperarm.YPre();
	z[6] = boxer.left_arm.upperarm.ZPre();
	x[7] = boxer.left_arm.shoulder.XPre();
	y[7] = boxer.left_arm.shoulder.YPre();
	z[7] = boxer.left_arm.shoulder.ZPre();
	x[8] = boxer.right_leg.toe.XPre();
	y[8] = boxer.right_leg.toe.YPre();
	z[8] = boxer.right_leg.toe.ZPre();
	x[9] = boxer.right_leg.foot.XPre();
	y[9] = boxer.right_leg.foot.YPre();
	z[9] = boxer.right_leg.foot.ZPre();
	x[10] = boxer.right_leg.shank.XPre();
	y[10] = boxer.right_leg.shank.YPre();
	z[10] = boxer.right_leg.shank.ZPre();
	x[11] = boxer.right_leg.thigh.XPre();
	y[11] = boxer.right_leg.thigh.YPre();
	z[11] = boxer.right_leg.thigh.ZPre();
	x[12] = boxer.left_leg.toe.XPre();
	y[12] = boxer.left_leg.toe.YPre();
	z[12] = boxer.left_leg.toe.ZPre();
	x[13] = boxer.left_leg.foot.XPre();
	y[13] = boxer.left_leg.foot.YPre();
	z[13] = boxer.left_leg.foot.ZPre();
	x[14] = boxer.left_leg.shank.XPre();
	y[14] = boxer.left_leg.shank.YPre();
	z[14] = boxer.left_leg.shank.ZPre();
	x[15] = boxer.left_leg.thigh.XPre();
	y[15] = boxer.left_leg.thigh.YPre();
	z[15] = boxer.left_leg.thigh.ZPre();
	x[16] = boxer.body.waist.XPre();
	y[16] = boxer.body.waist.YPre();
	z[16] = boxer.body.waist.ZPre();
	x[17] = boxer.body.abdomen.XPre();
	y[17] = boxer.body.abdomen.YPre();
	z[17] = boxer.body.abdomen.ZPre();
	x[18] = boxer.body.breast.XPre();
	y[18] = boxer.body.breast.YPre();
	z[18] = boxer.body.breast.ZPre();
	x[19] = boxer.head.neck.XPre();
	y[19] = boxer.head.neck.YPre();
	z[19] = boxer.head.neck.ZPre();
	x[20] = boxer.head.face.XPre();
	y[20] = boxer.head.face.YPre();
	z[20] = boxer.head.face.ZPre();
	x[21] = boxer.GXPre();
	y[21] = boxer.GYPre();
	z[21] = boxer.GZPre();
	x[22] = boxer.OX();
	y[22] = boxer.OY();
	z[22] = boxer.OT();
	x[23] = boxer.OXS();
	y[23] = boxer.OYS();
}

void BoxerInfo::SetInfo(Boxer &boxer) {
	boxer.right_arm.fist.SetPosPre(x[0], y[0], z[0]);
	boxer.right_arm.forearm.SetPosPre(x[1], y[1], z[1]);
	boxer.right_arm.upperarm.SetPosPre(x[2], y[2], z[2]);
	boxer.right_arm.shoulder.SetPosPre(x[3], y[3], z[3]);
	boxer.left_arm.fist.SetPosPre(x[4], y[4], z[4]);
	boxer.left_arm.forearm.SetPosPre(x[5], y[5], z[5]);
	boxer.left_arm.upperarm.SetPosPre(x[6], y[6], z[6]);
	boxer.left_arm.shoulder.SetPosPre(x[7], y[7], z[7]);
	boxer.right_leg.toe.SetPosPre(x[8], y[8], z[8]);
	boxer.right_leg.foot.SetPosPre(x[9], y[9], z[9]);
	boxer.right_leg.shank.SetPosPre(x[10], y[10], z[10]);
	boxer.right_leg.thigh.SetPosPre(x[11], y[11], z[11]);
	boxer.left_leg.toe.SetPosPre(x[12], y[12], z[12]);
	boxer.left_leg.foot.SetPosPre(x[13], y[13], z[13]);
	boxer.left_leg.shank.SetPosPre(x[14], y[14], z[14]);
	boxer.left_leg.thigh.SetPosPre(x[15], y[15], z[15]);
	boxer.body.waist.SetPosPre(x[16], y[16], z[16]);
	boxer.body.abdomen.SetPosPre(x[17], y[17], z[17]);
	boxer.body.breast.SetPosPre(x[18], y[18], z[18]);
	boxer.head.neck.SetPosPre(x[19], y[19], z[19]);
	boxer.head.face.SetPosPre(x[20], y[20], z[20]);
	boxer.SetGPosPre(x[21], y[21], z[21]);
	boxer.SetPoint(x[22], y[22], z[22]);
	boxer.SetSPoint(x[23], y[23]);
}

void BoxerInfo::SaveInfo(SDL_RWops *rwops) {
	SDL_RWwrite(rwops, this, sizeof(BoxerInfo), 1);	
}

void BoxerInfo::LoadInfo(SDL_RWops *rwops) {
	SDL_RWread(rwops, this, sizeof(BoxerInfo), 1);	
}

void HandlerInfo::GetInfo(BoxerHandler &handler) {
	finfo[0] = handler.cyanosis;	
	finfo[1] = handler.cyanosis_restore;	
	finfo[2] = handler.max_cyanosis;	
	finfo[3] = handler.cyanosis_rate;	
	finfo[4] = handler.swing_speed;	

	finfo[5] = handler.absorb;
	finfo[6] = handler.damage;
	finfo[7] = handler.traum_damage;
	finfo[8] = handler.brain_damage;
	finfo[9] = handler.face_damage;
	finfo[10] = handler.body_damage;		
	finfo[11] = handler.damage_restore;	
	finfo[12] = handler.max_damage;	
	finfo[13] = handler.damage_rate;	
	finfo[14] = handler.hit_damage;	
	finfo[15] = handler.move_speed;	
	finfo[16] = handler.distance;
	finfo[17] = handler.adjust_distance;

	finfo[18] = handler.stamina;
	finfo[19] = handler.max_stamina;
	finfo[20] = handler.restore_rate;

	iinfo[0] = handler.state;
	iinfo[1] = handler.next_state;
	iinfo[2] = handler.moved;
	iinfo[3] = handler.dashing;
	iinfo[4] = handler.direct;
	iinfo[5] = handler.action;
	iinfo[6] = handler.next_action;
	iinfo[7] = handler.combination;
	iinfo[8] = handler.deadly;
	iinfo[9] = handler.hit_state;
	iinfo[10] = handler.hit_parts;
	iinfo[11] = handler.hit_first;
	
	iinfo[12] = handler.count;
	iinfo[13] = handler.swing;
	iinfo[14] = handler.down;
	iinfo[15] = handler.hit;
	iinfo[16] = handler.critical;
	iinfo[17] = handler.round_damage;
	iinfo[18] = handler.score;
	iinfo[19] = handler.total_score;
 }

void HandlerInfo::SetInfo(BoxerHandler &handler) {
	handler.cyanosis = finfo[0] ;	
	handler.cyanosis_restore = finfo[1];	
	handler.max_cyanosis = finfo[2];	
	handler.cyanosis_rate = finfo[3];	
	handler.swing_speed= finfo[4];	

	handler.absorb = finfo[5];
	handler.damage = finfo[6];
	handler.traum_damage = finfo[7];
	handler.brain_damage = finfo[8];
	handler.face_damage = finfo[9];
	handler.body_damage = finfo[10];		
	handler.damage_restore = finfo[11];	
	handler.max_damage = finfo[12];	
	handler.damage_rate = finfo[13];	
	handler.hit_damage = finfo[14];	
	handler.move_speed = finfo[15];	
	handler.distance = finfo[16];
	handler.adjust_distance = finfo[17];

	handler.stamina = finfo[18];
	handler.max_stamina = finfo[19];
	handler.restore_rate = finfo[20];

	handler.state = iinfo[0];
	handler.next_state = iinfo[1];
	handler.moved = iinfo[2];
	handler.dashing = iinfo[3];
	handler.direct = iinfo[4];
	handler.action = iinfo[5];
	handler.next_action = iinfo[6];
	handler.combination = iinfo[7];
	handler.deadly = iinfo[8];
	handler.hit_state = iinfo[9];
	handler.hit_parts = iinfo[10];
	handler.hit_first = iinfo[11];

	handler.count = iinfo[12];
	handler.swing = iinfo[13];
	handler.down = iinfo[14];
	handler.hit = iinfo[15]; 
	handler.critical = iinfo[16];
	handler.round_damage = iinfo[17];
	handler.score = iinfo[18];
	handler.total_score = iinfo[19];
 }


void HandlerInfo::SaveInfo(SDL_RWops *rwops) {
	SDL_RWwrite(rwops, this, sizeof(HandlerInfo), 1);	
}

void HandlerInfo::LoadInfo(SDL_RWops *rwops) {
	SDL_RWread(rwops, this, sizeof(HandlerInfo), 1);	
}

bool ReplayInfo::Init(MotionHandler &motion) {
	if (WasInit()) {
		Free();	
	}
	has_info = false;
	minfo.Init(motion);
	return WasInit();	
}

void ReplayInfo::Free() {
	if (WasInit()) {
		minfo.Free();
	}
}

void ReplayInfo::GetInfo(BoxerHandler &handler, Boxer &boxer, MotionHandler &motion) {
	has_info = true;
	hinfo.GetInfo(handler);
	binfo.GetInfo(boxer);
	minfo.GetInfo(motion);
}

void ReplayInfo::SetInfo(BoxerHandler &handler, Boxer &boxer, MotionHandler &motion) {
	if (has_info) {
		hinfo.SetInfo(handler);
		binfo.SetInfo(boxer);
		minfo.SetInfo(motion);
	}
}

void ReplayInfo::SaveInfo(SDL_RWops *rwops) {
	SDL_RWwrite(rwops, &has_info, sizeof(bool), 1);
	hinfo.SaveInfo(rwops);
	binfo.SaveInfo(rwops);
	minfo.SaveInfo(rwops);
}

void ReplayInfo::LoadInfo(SDL_RWops *rwops) {
	SDL_RWread(rwops, &has_info, sizeof(bool), 1);
	hinfo.LoadInfo(rwops);
	binfo.LoadInfo(rwops);
	minfo.LoadInfo(rwops);
}
	
