#include "smixer.h"

/* Mix_Music */
MMusic::MMusic() {
	music = NULL;
}

MMusic::~MMusic() {
	Free();
}

int MMusic::Load(const char *file) {
	if (WasLoad()) {
		Free();
	}
	music = Mix_LoadMUS(file);
	if (!WasLoad()) {
		emes("MMusic::Load");	
		efile(file);	
		return SDL_FAILURE;	
	}
	smes("MMusic::Load");	
	sfile(file);	
	return SDL_SUCCESS;
}

void MMusic::Free() {
	if (WasLoad()) {
		Mix_FreeMusic(music);
		music = NULL;
		smes("MMusic::Free");	
	} else {
		wmes("MMusic::Free");
	}
}

void MMusic::Play() {
	if (WasLoad()) {
		Mix_PlayMusic(music, -1);
	}
}

void MMusic::Stop() {
	if (WasLoad()) {
		Mix_HaltMusic();
	}
}

bool MMusic::WasLoad() {
	if (music == NULL) {
		return false;
	}
	return true;	
}

MMusics::MMusics() {
	musics = NULL;
}

MMusics::~MMusics() {
	Free();
}

int MMusics::Init(int number) {
	if(WasInit()) {
		Free();
	}
	musics = new MMusic[number];
	cnumber = 0;
	mnumber = number;
	if (!WasInit()) {
		emes("MMusics::Init");		
		return SDL_FAILURE;	
	}
	smes("MMusics::Init");		
	return SDL_SUCCESS;
}

void MMusics::Free() {
	if (WasInit()) {
		delete [] musics;		
		musics = NULL;
		smes("MMusics::Free");	
	} else {
		wmes("MMusics::Free");
	}
}

void MMusics::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

int MMusics::Load(const char * file) {
	if (WasInit()) {
		return musics[cnumber].Load(file);
	}
	return SDL_FAILURE;
}

void MMusics::Play() {
	musics[cnumber].Play();
}

void MMusics::Stop() {
	musics[cnumber].Stop();
}

bool MMusics::WasLoad() {
	return musics[cnumber].WasLoad();
}

bool MMusics::WasInit() {
	if (musics == NULL) {
		return false;
	}
	return true;	
}

/* Mix_Chunk */
MChunk::MChunk() {
	chunk = NULL;
}

MChunk::~MChunk() {
	Free();
}

int MChunk::Load(const char *file) {
	if (WasLoad()) {
		Free();
	}
	chunk = Mix_LoadWAV(file);
	if (!WasLoad()) {
		emes("MChunk::Load");	
		efile(file);	
		return SDL_FAILURE;	
	}
	smes("MChunk::Load");	
	sfile(file);	
	return SDL_SUCCESS;
}

void MChunk::Free() {
	if (WasLoad()) {
		Mix_FreeChunk(chunk);
		chunk = NULL;
		smes("MChunk::Free");	
	} else {
		wmes("MChunk::Free");
	}
}

void MChunk::Play() {
	if (WasLoad()) {
		Mix_PlayChannel(-1, chunk, 0);
	}
}

void MChunk::Stop() {
	if (WasLoad()) {
		Mix_HaltChannel(-1);
	}
}

bool MChunk::WasLoad() {
	if (chunk == NULL) {
		return false;
	}
	return true;	
}

MChunks::MChunks() {
	chunks = NULL;
}

MChunks::~MChunks() {
	Free();
}

int MChunks::Init(int number) {
	if(WasInit()) {
		Free();	
	}
	chunks = new MChunk[number];
	cnumber = 0;
	mnumber = number;
	if(!WasInit()) {
		emes("MChunks::Init");		
		return SDL_FAILURE;	
	}
	smes("MChunks::Init");		
	return SDL_SUCCESS;
}

void MChunks::Free() {
	if (WasInit()) {
		delete [] chunks;
		chunks = NULL;
		smes("MChunks::Free");	
	} else {
		wmes("MChunks::Free");
	}
}

void MChunks::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

int MChunks::Load(const char * file) {
	if (WasInit()) {
		return chunks[cnumber].Load(file);
	}
	return SDL_FAILURE;
}

void MChunks::Play() {
	chunks[cnumber].Play();
}

void MChunks::Stop() {
	chunks[cnumber].Stop();
}

bool MChunks::WasLoad() {
	return chunks[cnumber].WasLoad();
}

bool MChunks::WasInit() {
	if (chunks == NULL) {
		return false;
	}
	return true;	
}

/* ~LT[ */
SMixer::SMixer() {
	was_open = false;
}

SMixer::~SMixer() {
	Close();
}

int SMixer::Open(int frequency, Uint16 format, int channels, int chunksize) {
	if (WasOpen()) {
		Close();	
	}	
	if (Mix_OpenAudio(frequency, format, channels, chunksize) != SDL_SUCCESS) {
		emes("SMixer::Open");
		rmes();
		return SDL_FAILURE;
	}
	smes("SMixer::Open");	
 	was_open = true; 
	return SDL_SUCCESS;
}

void SMixer::Close() {
    if (WasOpen()) {
		Mix_CloseAudio();
		was_open = false;
		smes("SMixer::Close");	
	} else {
		wmes("SMixer::Close");	
	}
}

int SMixer::InitMusic(int number) {
	return musics.Init(number);
}

int SMixer::InitChunk(int number) {
	return chunks.Init(number);
}

void SMixer::ChoiceMusic(int number) {
	musics.Choice(number);
}

void SMixer::ChoiceChunk(int number) {
	chunks.Choice(number);
}

int SMixer::LoadMusic(const char *file) {
	return musics.Load(file);
}

int SMixer::LoadMusic(int number, const char *file) {
	ChoiceMusic(number);
	return LoadMusic(file);
}

int SMixer::LoadChunk(const char *file) {
	return chunks.Load(file);
}

int SMixer::LoadChunk(int number, const char *file) {
	ChoiceChunk(number);
	return LoadChunk(file);
}

void SMixer::PlayMusic() {
	if (current_music_volume) {
		musics.Play();
	}
}

void SMixer::PlayMusic(int number) {
	ChoiceMusic(number);
	PlayMusic();
}

void SMixer::PlayChunk() {
	if (current_chunk_volume) {
		chunks.Play();
	}
}

void SMixer::PlayChunk(int number) {
 	ChoiceChunk(number);
	PlayChunk();
}

void SMixer::StopMusic() {
	musics.Stop();
}

void SMixer::StopChunk() {
	chunks.Stop();
}

bool SMixer::MusicWasLoad() {
	return musics.WasLoad();
}

bool SMixer::MusicWasLoad(int number) {
 	ChoiceMusic(number);
	return MusicWasLoad();
}

bool SMixer::ChunkWasLoad() {
	return chunks.WasLoad();
}

bool SMixer::ChunkWasLoad(int number) {
 	ChoiceChunk(number);
	return chunks.WasLoad();
}

bool SMixer::WasOpen() {
	return was_open;
}

