#include "snet.h"

SData::SData() {
	size = 0;
	data = NULL;
	rwops = NULL;
}

SData::~SData() {
	Delete();
}

int SData::Init(int csize) {
	if (WasInit()) {
		Delete();
	}
	size = csize;
	data = new Uint8[size];
	if (data == NULL) {
		emes("SData::Init -> Out of Memory");
		return SDL_FAILURE;	
	}
	rwops = SDL_RWFromMem((void *)data, size);	
	if (rwops == NULL) {
		emes("SData::Init -> Out of Memory");
		return SDL_FAILURE;	
	}
	return SDL_SUCCESS;		
}

void SData::Delete() {
	if (WasInit()) {
		SDL_FreeRW(rwops);
		delete [] data;
		size = 0;
		rwops = NULL;
		data = NULL;	
	}
}

bool SData::WasInit() {
	if (data == NULL) {
		return false;	
	}
	return true;
}


SNet::SNet() {
	was_init = 0;
	server_socket = NULL;
	client_socket = NULL;
	socket_set = NULL;
}

SNet::~SNet() {
	Quit();
}

int SNet::Init() {
	if (WasInit()) {
		Quit();	
	}	
	if (SDLNet_Init() == SDL_FAILURE) {
		emes("SNet::Init -> Can't initialize SDL_Net.");
		rmes();
		return SDL_FAILURE;
	}
	
	was_init = INIT_SDL_NET;
 	smes("SNet::Init");	

	socket_set = SDLNet_AllocSocketSet(1);
	if (socket_set == NULL) {
		emes("SNet::Init -> Can't allocate socket set.");
		rmes();
		return SDL_FAILURE;	
	}
	smes("SNet::Init -> Allocate socket set.");	
	return SDL_SUCCESS;   	
}

void SNet::CloseSockets() {
	if (client_socket != NULL) {
		SDLNet_TCP_Close(client_socket);
		SDLNet_TCP_DelSocket(socket_set, client_socket);
		client_socket = NULL;				
	}
	if (server_socket != NULL) {
		SDLNet_TCP_Close(server_socket);		
		server_socket = NULL;
	}
	was_init = 0;
}

void SNet::Quit() {
    if (WasInit()) {
		CloseSockets();
		SDLNet_Quit();
		smes("SNet::Quit");	
	} else {
		wmes("SNet::Quit");	
	}
}

int SNet::WasInit() {
	return was_init;
}

int SNet::Server(Uint16 port) {
	if (SDLNet_ResolveHost(&server_ip, NULL, port)) {
		emes("SNet::Server -> Can't resolve host.");
		ehost(HostName());
		rmes();
		return SDL_FAILURE;
	}
	smes("SNet::Server");
	shost(HostName());

	server_socket = SDLNet_TCP_Open(&server_ip);
	if (server_socket == NULL) {
		emes("SNet::Server -> Can't open server socket.");
		ehost(HostName());
		rmes();
		return SDL_FAILURE;	
	}
	smes("SNet::Server -> Open Server Socket.");	
	shost(HostName());

	was_init = SERVER;
	return SDL_SUCCESS;
}

int SNet::WaitClient() {
	if (IsClient()) {
		emes("SNet::WaitClient -> This isn't server.");
		return SDL_FAILURE;			
	}
	client_socket = SDLNet_TCP_Accept(server_socket);
	if (client_socket == NULL) {
		return SDL_FAILURE;	
	}
	client_ip = *SDLNet_TCP_GetPeerAddress(client_socket);		
	smes("SNet::WaitClient -> Connet Client");
	shost(RemoteName());
	
	SDLNet_TCP_AddSocket(socket_set, client_socket);
	return SDL_SUCCESS;
}

int SNet::Client(char* hostname, Uint16 port) {
	if (SDLNet_ResolveHost(&server_ip, hostname, port)) {
		emes("SNet::Client -> Can't connect to server.");
//		ehost(hostname);
		rmes();
		return SDL_FAILURE;
	}
	smes("SNet::Client -> Connet Server");
	shost(HostName());

	client_socket = SDLNet_TCP_Open(&server_ip);
	if (client_socket == NULL) {
		emes("SNet::Client -> Can't opne client socket.");
		ehost(HostName());
		rmes();
		return SDL_FAILURE;	
	}
	
	SDLNet_TCP_AddSocket(socket_set, client_socket);
	smes("SNet::Client -> Open Client Socket");
	shost(HostName());
	was_init = CLIENT;
	return SDL_SUCCESS;
}

int SNet::IsSocketReady() {
	SDLNet_CheckSockets(socket_set, 0);
	return SDLNet_SocketReady(client_socket);
}

int SNet::SendData(void *data, int maxlen) {
	if (SDLNet_TCP_Send(client_socket, data, maxlen) < maxlen) {
		if (IsClient()) {
			wmes("SNet::SendData -> Error by Client");
		} else {
			wmes("SNet::SendData -> Error by Server");
		}
		rmes();
		return SDL_FAILURE;		
	}
	return SDL_SUCCESS;		
}

int SNet::GetData(void *data, int maxlen) {
	static int size, get_size, left, timeout_count;
	timeout_count = 0;
	get_size = 0;
	left = maxlen;
	while(left > 0) {
		size = SDLNet_TCP_Recv(client_socket, (Uint8 *)data + get_size, left);
		if (size < 0) {
			if (IsClient()) {
				wmes("SNet::GetData -> Error by Client");		
			} else {
				wmes("SNet::GetData -> Error by Server");		
			}
			rmes();	
			return SDL_FAILURE;		
		}
		get_size+= size;
		left = maxlen - get_size;	

		timeout_count++;
		if (timeout_count > 500) {
			if (IsClient()) {
				wmes("SNet::GetData -> Time Out by Client");		
			} else {
				wmes("SNet::GetData -> Time Out by Server");		
			}
			rmes();	
			return SDL_FAILURE;		
		}
		SDL_Delay(10);
	}
	return SDL_SUCCESS;		
}

void SNet::Send8(Uint8 data) {
	if (SDLNet_TCP_Send(client_socket, &data, 1) < 1) {
		if (IsClient()) {
			wmes("SNet::Send8 -> Error by Client");		
		} else {
			wmes("SNet::Send8 -> Error by Server");		
		}
		rmes();		
	}
}

Uint8 SNet::Get8() {
	static int size;
	do {
		size = SDLNet_TCP_Recv(client_socket, (Uint8 *)data8, 1);
		if (size < 0) {	
			if (IsClient()) {
				wmes("SNet::Get8 -> Error by Client");	
			} else {
				wmes("SNet::Get8 -> Error by Server");	
			}
			rmes();
			return 0xff;
		}	
	} while(size == 0); 
	return data8[0];
}

void SNet::Send16(Uint16 data) {
	if (SDLNet_TCP_Send(client_socket, &data, 2) < 2) {
		if (IsClient()) {
			wmes("SNet::Get8 -> Error by Clinent");		
		} else {
			wmes("SNet::Get8 -> Error by Server");		
		}
		rmes();			
	}
}

Uint16 SNet::Get16() {
	static int size, get_size, left;
	get_size = 0;
	left = 2;
	while(left > 0) {
		size = SDLNet_TCP_Recv(client_socket, (Uint8 *)data16 + get_size, left);
		if (size < 0) {
			if (IsClient()) {
				wmes("SNet::Get16 -> Error by Client");	
			} else {
				wmes("SNet::Get16 -> Error by Server");	
			}
			rmes();	
			return 0xffff;
		}
		get_size+= size;
		left = 2 - get_size;	
	}
	return data16[0];
}

void SNet::Send32(Uint32 data) {
	if (SDLNet_TCP_Send(client_socket, &data, 4) < 4) {
		if (IsClient()) {
			wmes("SNet::Send32 -> Erro by Client");		
		} else {
			wmes("SNet::Send32 -> Erro by Server");		
		}
		rmes();				
	}
}

Uint32 SNet::Get32() {
	static int size, get_size, left;
	get_size = 0;
	left = 4;
	while(left > 0) {
		size = SDLNet_TCP_Recv(client_socket, (Uint8 *)data32 + get_size, left);
		if (size < 0) {
			if (IsClient()) {
				wmes("SNet::Get32 -> Error by Client");	
			} else {
				wmes("SNet::Get32 -> Error by Server");
			}
			rmes();	
			return 0xffffffff;
		}
		get_size+= size;
		left = 4 - get_size;	
	}
	return data32[0];
}

