#ifndef SNET_H
#define SNET_H

#include "csdl.h"
#include "creader.h"
#include <SDL_net.h>

class SData {
public:
	SData();
	~SData();
	
	int Init(int csize);
	void Delete();
	bool WasInit();

	int Size() const {return size;}
	SDL_RWops* RWops() const {return rwops;}
	Uint8* Data() const {return data;}
	
private:
	int size;
	Uint8 *data;
	SDL_RWops *rwops;
};

enum {
	SERVER = 1,
	CLIENT,
	QUIT_SERVER,
	INIT_SDL_NET
};

class SNet {
public :
	SNet();
	~SNet();
	int Init();
	int Server(Uint16 port);
	int Client(char* hostname, Uint16 port);	

	void CloseSockets();
	void Quit();
	int WaitClient();
	int WasInit();
	bool IsServer() const {return (was_init == SERVER);}
	bool IsClient() const {return (was_init == CLIENT);}
		
	int IsSocketReady();

	int SendData(void *data, int maxlen);
	int GetData(void *data, int maxlen);

	void Send8(Uint8 data);
	Uint8 Get8();

	void Send16(Uint16 data);
	Uint16 Get16();

	void Send32(Uint32 data);
	Uint32 Get32();

	const char* HostName() {return SDLNet_ResolveIP(&server_ip);};
	Uint32 ServerIP() {return SDLNet_Read32(&server_ip.host);}

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	int ServerIP1() {return 0xff & (server_ip.host >> 24);}
	int ServerIP2() {return 0xff & (server_ip.host >> 16);}
	int ServerIP3() {return 0xff & (server_ip.host >> 8);}
	int ServerIP4() {return 0xff & (server_ip.host);}
#else
	int ServerIP1() {return 0xff & (server_ip.host);}
	int ServerIP2() {return 0xff & (server_ip.host >> 8);}
	int ServerIP3() {return 0xff & (server_ip.host >> 16);}
	int ServerIP4() {return 0xff & (server_ip.host >> 24);}
#endif
	Uint16 ServerPort() {return SDLNet_Read16(&server_ip.port);};

	const char* RemoteName() {return SDLNet_ResolveIP(&client_ip);};
	Uint32 ClientIP() {return SDLNet_Read32(client_ip.host);};
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	int ClientIP1() {return 0xff & (client_ip.host >> 24);}
	int ClientIP2() {return 0xff & (client_ip.host >> 16);}
	int ClientIP3() {return 0xff & (client_ip.host >> 8);}
	int ClientIP4() {return 0xff & (client_ip.host);}
#else
	int ClientIP1() {return 0xff & (client_ip.host);}
	int ClientIP2() {return 0xff & (client_ip.host >> 8);}
	int ClientIP3() {return 0xff & (client_ip.host >> 16);}
	int ClientIP4() {return 0xff & (client_ip.host >> 24);}
#endif
	Uint16 ClientPort() {return SDLNet_Read16(&server_ip.port);};

private :
	TCPsocket server_socket, client_socket;
	IPaddress server_ip, client_ip;
	SDLNet_SocketSet socket_set;

	Uint8 data8[1];
	Uint16 data16[1];
	Uint32 data32[1];

	int was_init;
};

#endif // SNET_H

