#ifndef STORYBOXER_H
#define STORYBOXER_H

#include "random.h"

#define OUT_OF_REGION 9999
#define OUT_OF_RANK   9999

enum {
	JAPAN,
	KOREA,
	THAI,
	PHILIPPINE,
	AUSTRALIA,
	ASIAN_COUNTRIES,
	SOUTH_AFRICA,
	NIGERIA,
	KENYA,
	GHANA,
	CAMEROON,
	AFRICAN_COUNTRIES,
	UK,
	FRANCE,
	GERMANY,
	ROSSIA,
	ITALIA,
	SPAIN,
	EUROPEAN_COUNTRIES,
	USA_EAST,
	USA_WEST,
	CANADA,
	MEXICO,
	PANAMA,
	CUBA,
	NA_COUNTRIES,
	VENEZUELA,
	COLOMBIA,
	BRAZIL,
	ARGENTINA,
	ECUADOR,
	SA_COUNTRIES,
	ASIA_OCEANIA,
	AFRICA,
	EUROPE,
	NORTH_AMERICA,
	SOUTH_AMERICA,
	WORLD,
	RANK,
	AGE,
	CAREER,
	MATCH,
	WIN,
	WIN_RATE,
	LOSS,
	DRAW,
	KO,
	KO_RATE,
	COUNTRY,
	STATE,
	RINGNAME
};

enum {
	ROOKIE = RANK,
	RANKER = AGE
};

enum {
	WAIT_STATE,
	MATCH_MONTH0,
	MATCH_MONTH11 = MATCH_MONTH0 + 11,
	RANKER_T0,
	RANKER_T15 = RANKER_T0 + 15,
	ROOKIE_T0,
	ROOKIE_T63 = ROOKIE_T0 + 63,
	NO_MATCHING,
	REST_MONTH0,
	REST_MONTH6 = REST_MONTH0 + 6,
	DEBUT,
	ROUND4,
	ROUND6,
	ROUND8,
	ROUND10,		
	RETIRE_STATE
};

enum {
	WORLD_TITLE 		= 0x00001,
	EX_WORLD_TITLE		= 0x00002,
	FORMER_WORLD_TITLE	= 0x00004,
	AREA_TITLE			= 0x00010,
	EX_AREA_TITLE		= 0x00020,
	FORMER_AREA_TITLE	= 0x00040,
	NATION_TITLE		= 0x00100,
	EX_NATION_TITLE		= 0x00200,
	FORMER_NATION_TITLE	= 0x00400,
	ROOKIE_KING			= 0x01000,
	EX_ROOKIE_KING		= 0x02000,
	FORMER_ROOKIE_KING	= 0x04000,
	WORLD_RANKER_WINNER	= 0x10000,
	AREA_RANKER_WINNER	= 0x20000,
	NATION_RANKER_WINNER= 0x40000
};	

enum {
	SORT_POINT,
	SORT_RANK,
	SORT_REGIONRANK,
	SORT_AGE,
	SORT_CAREER,
	SORT_MATCH,
	SORT_WIN,
	SORT_LOSS,
	SORT_DRAW,
	SORT_KO,
	SORT_KO_RATE,
	SORT_WIN_RATE,
	SORT_TRAUM,
	SORT_HEIGHT,
	SORT_REACH,
	SORT_STYLE,
	SORT_STANCE,
	SORT_COMB,
	SORT_THOUGHT,
	SORT_GLOVES,
	SORT_TRUNKS,
	SORT_SKIN,
	SORT_NATION,
	SORT_AREA,
	SORT_PUNCH,
	SORT_SPEED,
	SORT_STAMINA,
	SORT_DAMAGE_RESTORE,
	SORT_MAX_DAMAGE,
	SORT_CYANOSIS_RESOTRE,
	SORT_MAX_CYANOSIS,
	SORT_BLOCK_ABSORB,
	SORT_BODY_STRENGTH,
	SORT_LIVER_STRENGTH,
	SORT_FACE_STRENGTH,
	SORT_JAW_STRENGTH,
	SORT_TOTAL
};

class BoxerHandler;

class StoryBoxer {
public:
	StoryBoxer();
	~StoryBoxer() {};

	void Init();	
	void CreateRandom(Random &random, int c_nation = JAPAN);	
	void Inherit(Random &random);	
	void SetNation(Uint8 c_nation);
	void SetArea();

	void PowerUp(StoryBoxer &boxer);
	void PowerDown();
	int WhichTournament();

	void HeightP();
	void HeightM();
	void ReachP();
	void ReachM();
	void StyleP();
	void StyleM();
	void StanceP();
	void StanceM();
	void CombP();
	void CombM();
 	void ThoughtP();
	void ThoughtM();

	void GlovesP();
	void GlovesM();
	void TrunksP();
	void TrunksM();
	void SkinP();
	void SkinM();
	void NationP();
	void NationM();

	int RingName() const {return (int)nickname;}
	int FamilyName() const {return (int)familyname;}
	void ClearName() {nickname = 0; familyname= 0;}

	int Rank() const {return (int)rank;}
	int RegionRank() const {return (int)region_rank;}
	int Age() const {return (int)age;}
	int Career() const {return (int)career;}
	int CareerYear() const {return 1 + Career()/12;}
	int Match() const {return (int)match;}
	int Win() const {return (int)win;}
	int Loss() const {return (int)loss;}
	int Draw() const {return (int)draw;}
	int KO() const {return (int)ko;}
	int KORate();
	int WinRate();
	int Traum() const {return (int)traum;}
	void ClearTraum() {traum = 0;}
	void AddTraum(int i) {traum += (Uint8)i;}

	void AgeP() {age++;}
	void CareerP() {career++;}
	void MatchP() {match++;}
	void WinP() {win++;}
	void LossP() {loss++;}
	void DrawP() {draw++;}
	void KOP() {ko++;}
	void StateM() {state--;}
	void SetState(int i) {state = (Uint8)i;}

	int Height() const {return (int)body;}
	int Reach() const {return (int)reach;}
	int Style() const {return (int)motion;}
	int Stance() const {return (int)is_southpaw;}
	int Comb() const {return (int)combination;}
	int Thought() const {return (int)think;}
	int Gloves() const {return (int)gloves_shoes;}
	int Trunks() const {return (int)trunks;}
	int Skin() const {return (int)skin;}
	int HairColor() const {return (int)hair_color;}
	Uint8 HairStyle() const {return hair_style;}
	Uint8 Eyebrow() const {return eyebrow;}
	Uint8 Eye() const {return eye;}
	Uint8 Nose() const {return nose;}	
	Uint8 Mouth() const {return mouth;}	
	int Nation() const {return (int)nation;}
	int Area() const {return (int)area;}
	int State() const {return (int)state;}

	void SetHeight(int i) {SetMinMax(&i, 1, 7); body = (Uint8)i;}
 	void SetReach(int i) {SetMinMax(&i, 1, 7); reach = (Uint8)i;}
 	void SetStyle(int i) {SetMinMax(&i, 1, 7); motion = (Uint8)i;}
	void SetStance(int i)  {SetMinMax(&i, 0, 1); is_southpaw = (Uint8)i;}
 	void SetComb(int i)  {SetMinMax(&i, 1, 20); combination =  (Uint8)i;}
 	void SetThought(int i)  {SetMinMax(&i, 1, 14); think = (Uint8)i;}
 	void SetGloves(int i)  {SetMinMax(&i, 1, 8); gloves_shoes =  (Uint8)i;}
 	void SetTrunks(int i)  {SetMinMax(&i, 1, 64); trunks =  (Uint8)i;}
 	void SetSkin(int i)  {SetMinMax(&i, 1, 10); skin =  (Uint8)i;}
 	void SetHairColor(int i)  {SetMinMax(&i, 1, 10); hair_color =  (Uint8)i;}
	void SetDefaultFace();

	int Punch() const {return (int)punch;}
 	int Speed() const {return (int)speed;}
 	int Stamina() const {return (int)stamina;}

	int DamageRestore() const {return (int)damage_restore;}
 	int MaxDamage() const {return (int)max_damage;}
 	int CyanosisRestore() const {return (int)cyanosis_restore;}
 	int MaxCyanosis() const {return (int)max_cyanosis;}

 	int BlockAbsorb() const {return (int)block_absorb;}
 	int BodyStrength() const {return (int)body_strength;}
 	int LiverStrength() const {return (int)liver_strength;}
 	int FaceStrength() const {return (int)face_strength;}
 	int JawStrength() const {return (int)jaw_strength;}

	void SetPunch(int i) {SetMinMax(&i, -50, 50); punch = (Uint8)i;}
 	void SetSpeed(int i) {SetMinMax(&i, -50, 50); speed = (Uint8)i;}
 	void SetStamina(int i) {SetMinMax(&i, -50, 50); stamina = (Uint8)i;}

	void SetDamageRestore(int i)  {SetMinMax(&i, -50, 50); damage_restore = (Uint8)i;}
 	void SetMaxDamage(int i)  {SetMinMax(&i, -50, 50); max_damage = (Uint8)i;}
 	void SetCyanosisRestore(int i)  {SetMinMax(&i, -50, 50); cyanosis_restore = (Uint8)i;}
 	void SetMaxCyanosis(int i)  {SetMinMax(&i, -50, 50); max_cyanosis  = (Uint8)i;}

 	void SetBlockAbsorb(int i)  {SetMinMax(&i, -50, 50); block_absorb =  (Uint8)i;}
 	void SetBodyStrength(int i)  {SetMinMax(&i, -50, 50); body_strength =  (Uint8)i;}
 	void SetLiverStrength(int i)  {SetMinMax(&i, -50, 50); liver_strength =  (Uint8)i;}
 	void SetFaceStrength(int i)  {SetMinMax(&i, -50, 50); face_strength =  (Uint8)i;}
 	void SetJawStrength(int i)  {SetMinMax(&i, -50, 50); jaw_strength =  (Uint8)i;}   

	int Total() {
		return 300 + Punch() + Speed() + Stamina() + 
			(DamageRestore() + MaxDamage())/2 + (CyanosisRestore() + MaxCyanosis())/2 + 
		 	(BlockAbsorb() + (BodyStrength() + LiverStrength())/2 + (FaceStrength() + JawStrength())/2)/3;
	}

	void SetRank(int i) {rank = (short)i;}
	void SetRegionRank(int i) {region_rank = (short)i;}

	void SetData(int i = SORT_RANK);
	int Data() const {return data;}
	void SetPoint(int i) {point = i;}
	int Point() const {return point;}
	void AddPoint(int i) {point += i;}
	void DelPoint(int i) {point -= i;}

	void AddRecent(int i) {recent = recent << 3 | i;}
	void ChangeRecent(int i) {recent = recent | i;}
	int GetRecent(int i) const {return (int)(recent >> (3*i) & 0x0007);}
	void ClearRecent() {recent = 0x0000;}
	Uint32 Recent() const {return recent;}
	int DefeatSeries();

	bool HaveTitle(long t) {return (bool)(title & t);}
	void GetTitle(long t) {title |= t;}
	void LoseTitle(long t) {title ^= t;}
	int WhichTitle();

	bool IsInRegion() const {return (RegionRank() != OUT_OF_REGION);}
	bool IsWaiting() const {return (State() == WAIT_STATE);}
private:
	int data;
	int point;

	Uint32 title;
	Uint32 recent;
	
	short nickname;
	short familyname;

	short rank;
	short region_rank;
	Uint8 age;
	Uint16 career;
	Uint8 match;
	Uint8 win;
	Uint8 loss;
	Uint8 draw;
	Uint8 ko;
	Uint8 traum;	
	
	Uint8 body;
	Uint8 reach;
	Uint8 motion;
	Uint8 is_southpaw;
	Uint8 combination;
	Uint8 think;
	
	Uint8 gloves_shoes;
	Uint8 trunks;
	Uint8 skin;

	Uint8 hair_style;
	Uint8 hair_color;
	Uint8 eyebrow;
	Uint8 eye;
	Uint8 nose;
	Uint8 mouth;

	Uint8 area;
	Uint8 nation;
	Uint8 state;

	char punch;
	char speed;
	char stamina;
	
	char damage_restore;
	char max_damage;
	char cyanosis_restore;
	char max_cyanosis;
	
	char block_absorb;
	char body_strength;
	char liver_strength;
	char face_strength;
	char jaw_strength;
};


#endif		// STORYBOXER_H
