#include "storyhandler.h"
#include "game.h"

StoryHandler::StoryHandler() {
	int i;
	sb = new StoryBoxer[MAX_BOXER];
	sort = new int[MAX_BOXER];
	for (i = 0; i < MAX_BOXER; i++) {
		sort[i]  = i;
	}
	record_number = 0;
	score = 0;
	for (i = 0; i < 10; i++) {
		high_scores[i]  = 0;
	}
}

StoryHandler::~StoryHandler() {
	delete [] sb;
	delete [] sort;
}

void StoryHandler::Init() {
	int i, j;
	for (i = 0; i < MAX_MATCH; i++) {
		match[i].match_type = NO_MATCH;
		match[i].result = NO_RESULT;
		match[i].replay = 0;
		result[i].match_type = NO_MATCH;	
		region_match[i] = 0;	
	}
	for (i = JAPAN; i <= WORLD; i++) {
		title[i].defending = 0;	
		title[i].holder = NO_BOXER;
		title[i].ex_holder = NO_BOXER;
		title[i].challengers[0] = NO_BOXER;
	}
	for (i = JAPAN; i <= WORLD; i++) {
		for (j = 0; j < 16; j++) {
			ranker[i][j] = NO_BOXER;
		}
	}
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		for (j = 0; j < 64; j++) {
			rookie[i][j] = NO_BOXER;
		}
	}

	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		for (j = 0; j < 128; j++) {
	 		Boxer(i * 128 + j).CreateRandom(random, i);
		}	
	}
	InitPoint();
	SortData(SORT_POINT);
	InitRank();
	SortRank();
	month = 3;
}

StoryBoxer & StoryHandler::SortBoxer(int i) {
	if (i > 0 && i < MAX_BOXER) {
		return Boxer(sort[i]);	
	} else {
		return Boxer(sort[0]);	
	}
}

StoryBoxer & StoryHandler::Boxer(int i) {
	if (i > 0 && i < MAX_BOXER) {
		return sb[i];	
	} else {
		return sb[0];		
	}
}

int StoryHandler::Sort(int i) const {
	if (i > 0 && i < MAX_BOXER) {
		return sort[i];	
	} else {
		return sort[0];		
	}
}

int StoryHandler::Order(int number) {
	static int i;
	for (i = 0; i < MAX_BOXER; i++) {
		if (sort[i] == number) {
			return i;
		}		
	}
	return 0;
}

void StoryHandler::SetData(int data_type) {
	static int i;
	for (i = 0; i < MAX_BOXER; i++) {
		Boxer(i).SetData(data_type);	
	}
}

void StoryHandler::InitRank() {
	static int i;
	for (i = 0; i < MAX_BOXER; i++) {
		SortBoxer(i).SetRank(i);	
		SortBoxer(i).SetRegionRank(0);	
	}
}

void StoryHandler::InitPoint() {
	static int i;
	for (i = 0; i < MAX_BOXER; i++) {
		Boxer(i).SetPoint(700 + Boxer(i).Total());	
	}
}

int StoryHandler::SetRegion(int n) {
	static int i, j, k;
	select_region = n;
	j = 0;
	k = 2;
	SortRank();
	if (n <= SA_COUNTRIES) {
		for (i = 0; i < MAX_BOXER; i++) {
			if (SortBoxer(i).Nation() == n &&
				!SortBoxer(i).HaveTitle(WORLD_TITLE) &&
				!SortBoxer(i).HaveTitle(AREA_TITLE)) {
				if (SortBoxer(i).HaveTitle(NATION_TITLE)) {
					SortBoxer(i).SetRegionRank(0);
					k--;
				} else {
					SortBoxer(i).SetRegionRank(++j);			
				} 
			} else {
				SortBoxer(i).SetRegionRank(OUT_OF_REGION);		
			}
		}
	} else if (n <= SOUTH_AMERICA) {
		for (i = 0; i < MAX_BOXER; i++) {
			if (SortBoxer(i).Area() == n &&
				!SortBoxer(i).HaveTitle(WORLD_TITLE)) {
				if (SortBoxer(i).HaveTitle(AREA_TITLE)) {
					SortBoxer(i).SetRegionRank(0);
					k--;
				} else {
					SortBoxer(i).SetRegionRank(++j);			
				}
			} else {
				SortBoxer(i).SetRegionRank(OUT_OF_REGION);		
			}
		}
	} else {
		for (i = 0; i < MAX_BOXER; i++) {
			if (SortBoxer(i).HaveTitle(WORLD_TITLE)) {
				SortBoxer(i).SetRegionRank(0);
				k--;
			} else {
				SortBoxer(i).SetRegionRank(++j);			
			}
		}
	}
	return (j - k) / 16;     
}

/***********************************************************
 F搶́wbɂŐVASYTx]p
	http://oku.edu.mie-u.ac.jp/~okumura/algo/
	qsort2.c -- NCbN\[g
	bubsort.c -- ou\[g
***********************************************************/

void StoryHandler::InsSort() {
	int i, j, x;

	for (i = 1; i < MAX_BOXER; i++) {
		x = sort[i];
		for (j = i - 1; j >= 0 && Boxer(sort[j]).Data() > Boxer(x).Data() ; j--) {
			sort[j + 1] = sort[j];
		}
		sort[j + 1] = x;
	}
}

#define THRESHOLD 10
#define STACKSIZE 32  /*  int ̃rbgx */
  
void StoryHandler::QuickSort() {
	int i, j, x, t, left, right, p;
	int leftstack[STACKSIZE], rightstack[STACKSIZE];

	left = 0;
	right = MAX_BOXER - 1;
	p = 0;
	for ( ; ; ) {
		if (right - left <= THRESHOLD) {
			if (p == 0) {
				break;
			}
			p--;
			left = leftstack[p];
			right = rightstack[p];
		}
		x = sort[(left + right) / 2];
		i = left;
		j = right;
		for ( ; ; ) {
			while (Boxer(sort[i]).Data() < Boxer(x).Data()) {
				i++;
			}
			while (Boxer(x).Data() < Boxer(sort[j]).Data()) {
				j--;
			}
			if (i >= j) {
				break;
			}
			t = sort[i];
			sort[i] = sort[j];
			sort[j] = t;
			i++;
			j--;
		}
		if (i - left > right - j) {
			if (i - left > THRESHOLD) {
				leftstack[p] = left;
				rightstack[p] = i - 1;
				p++;
			}
			left = j + 1;
		} else {
			if (right - j > THRESHOLD) {
				leftstack[p] = j + 1;
				rightstack[p] = right;
				p++;
			}
			right = i - 1;
		}
	}
	InsSort();
}

void StoryHandler::BubbleSort() {
	int i, j, k, x;

	k = MAX_BOXER - 1;
	while (k >= 0) {
		j = -1;
		for (i = 1; i <= k; i++)
			if (Boxer(sort[i - 1]).Data() > Boxer(sort[i]).Data()) {
				j = i - 1;
				x = sort[j];
				sort[j] = sort[i];
				sort[i] = x;
			}
		k = j;
	}
}
/*------------------- ܂ -------------------*/ 

void StoryHandler::NextMonth(Game &game, int player) {
	int i;

	RetireBoxers(game, player);
	SetNewRanking();				

	RegionMatch(WORLD);
	for (i = 0; i < MaxMatch(); i++) {
		if (SelectMatch(i)->result == NO_RESULT) {
			if (player && (SelectMatch(i)->high_ranked == MY_BOXER || SelectMatch(i)->low_ranked == MY_BOXER)) {
				game.SelectMatchType(SelectMatch(i)->match_type, Boxer(SelectMatch(i)->high_ranked));
				game.MatchRefer().SetResultState(SelectMatch(i)->result);
				SortRegion(game.MatchRefer().MatchRegion());
				game.MyMatch(*this, i);			
				SortRegion(WORLD);
			} else { 
				SelectMatch(i)->result = SimpleMatch(SelectMatch(i)->high_ranked, SelectMatch(i)->low_ranked);
				MatchRecord(SelectMatch(i));
			}
		}
/*		switch (SelectMatch(i)->match_type) {
			case WORLD_CHAMPIONSHIP :
				SelectDesignatedChallenger(WORLD, game, player);
			break;
			case AREA_CHAMPIONSHIP : 
				SelectDesignatedChallenger(Boxer(SelectMatch(i)->high_ranked).Area(), game, player);
			break;
			case NATION_CHAMPIONSHIP : 
				SelectDesignatedChallenger(Boxer(SelectMatch(i)->high_ranked).Nation(), game, player);
			break;
		} */
		result[i] = match[region_match[i]];
	}
	for (i = 0; i < MaxMatch(); i++) {
		match[i].match_type = NO_MATCH;
	}   
	for (i = MaxMatch(); i < MAX_MATCH; i++) {
		result[i].match_type = NO_MATCH;
	}

	if (++month > 12) {
		month = 1;
	}
	for (i = 0; i < MAX_BOXER; i++) {
		Boxer(i).CareerP();	
		if (Boxer(i).State() >= MATCH_MONTH0 &&  Boxer(i).State() <= MATCH_MONTH11) {
			Boxer(i).StateM();
		} else if (Boxer(i).State() == REST_MONTH0 || Boxer(i).State() == NO_MATCHING) {
			Boxer(i).SetState(WAIT_STATE);		
		} else if (Boxer(i).State() > REST_MONTH0 &&  Boxer(i).State() <= REST_MONTH6) {
			Boxer(i).StateM();
		}
	}
}

void StoryHandler::SetEvents(Game &menu, int player) {
	switch (Month()) {
		case 4:
			AgeBoxers(menu, player);
			SelectChallengers(menu, player);

			SortRegion(WORLD);
			RookieKing1stQualifing();
			MatchMaking(menu, player);
		break;	
		case 5:
			SortRegion(WORLD);
			SetChallengers();
			NationRanker1stQualifing();
			AreaRankerQuarterFinal();
			MatchMaking(menu, player);
		break;	
		case 6:
			SortRegion(WORLD);
			SetChallengers();
			RookieKing2ndQualifing();
			WorldRankerQuarterFinal();
			MatchMaking(menu, player);
		break;	
		case 7:
			SortRegion(WORLD);
			SetChallengers();
			NationRankerQuarterFinal();
			RookieKingShrink();
			MatchMaking(menu, player);
		break;	
		case 8:
			SortRegion(WORLD);
			SetChallengers();
			RookieKing3rdQualifing();
			AreaRankerSemiFinal();
			MatchMaking(menu, player);
		break;	
		case 9:
			SortRegion(WORLD);
			SetChallengers();
			NationRankerSemiFinal();
			WorldRankerSemiFinal();
			MatchMaking(menu, player);
		break;	
		case 10:
			SortRegion(WORLD);
			SetChallengers();
			RookieKingQuarterFinal();
			MatchMaking(menu, player);
		break;	
		case 11:
			SortRegion(WORLD);
			SetChallengers();
			NationRankerFinal();
			AreaRankerFinal();
			MatchMaking(menu, player);
		break;	
		case 12:
			SortRegion(WORLD);
			SetChallengers();
			WorldRankerFinal();
			RookieKingSemiFinal();
			MatchMaking(menu, player);
		break;	
		case 1:
			SortRegion(WORLD);
			SetChallengers();
			MatchMaking(menu, player);
		break;	
		case 2:
			SortRegion(WORLD);
			SetChallengers();
		break;	
		case 3:
			SortRegion(WORLD);
			ExRookieKing();
			RookieKingFinal();
			SetChallengers();
		break;	
	}

}

void StoryHandler::WillAccept(int *accept, int number, Game &menu) {
	int choice;
	do {
		choice = menu.GetOfferMenu();
		switch (choice) {
			case 1:	/* 󂯂 */
				*accept = 1;	
			break;
			case 2:	/* f */
			break;
			default :	/* vtB[ */
				menu.ShowBoxerProfile(*this, number);
			break;
		}
	} while (!choice);
}

void StoryHandler::SetMatch(StoryMatch *set_match, int high, int low, int type, int state) {
	set_match->high_ranked = high;
	set_match->low_ranked = low;
	set_match->match_type = type;
	set_match->result = NO_RESULT;
	set_match->replay = 0;
	Boxer(high).SetState(state);
	Boxer(low).SetState(state);
} 

void StoryHandler::WillOffer(int *offer, Game &menu) {
	switch (menu.SendOfferMenu()) {
		case 1 :
			*offer = 2; /* It@[I */
		break;									
		case 2 :
		break;			/* It@[Ȃ */						
		default :
			*offer = 1; /* It@[ */
		break;
	}
}

int StoryHandler::ChoiceLowRanked(int *accept, int start, Game &menu) {
	static int j, max_select, candidates[51];
	max_select = 0;
	for (j = start + 1; j < MAX_BOXER; j++) {
		if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
			candidates[max_select++] = j;		
			if (max_select == 50) {
				break;
			} 
		}
	}
  
	if (max_select > 0) {
		*accept = 1;
		return menu.ShowChallengers(*this, candidates, max_select-1);
	}
	return 0;
}


void StoryHandler::MatchMaking(Game &menu, int player) {
	static int i, j, k, offer, accept, match_number;
	static StoryMatch match_buf[MAX_MATCH];

	match_number = 0;
	SortRegion(WORLD);
/* E}b`CN */
	for (i = 1; i <= 50; i++) {
		offer = 0;
		if (SortBoxer(i).IsWaiting()) {
			if (player && Sort(i) == MY_BOXER) {
				WillOffer(&offer, menu);	/* vC[It@[ */
			} else {
				if (random.Rand(100) > 20) {
	            	offer = 1;				/* COMIt@[ */			
				}
			}
			if (!offer) {	/* ̓It@[Ȃ */
				SortBoxer(i).SetState(NO_MATCHING);				
			}
		}
		if (offer) {
			accept = 0;
			if (offer == 1) {	/* IɑI */
				for (j = i + 1; j < MAX_BOXER; j++) {
					if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
						if (player && Sort(j) == MY_BOXER) {
							WillAccept(&accept, Sort(i), menu);
						} else {
							accept = 1;
						}
					}
					if (accept) {
							break;
					}
				}
			} else {	/* őI */
				j = ChoiceLowRanked(&accept, i, menu);
			}			
			if (accept) {
				SetMatch(match_buf + match_number++, Sort(i), Sort(j), ROUND10_MATCH, ROUND10);
			}			
		}
	}        

	for (k = 0; k < 5; k++) {
		SortRegion(ASIA_OCEANIA+k);
/* en}b`CN */
		for (i = 1; i <= 50; i++) {
			offer = 0;
			if (SortBoxer(i).IsWaiting()) {
				if (player && Sort(i) == MY_BOXER) {
					WillOffer(&offer, menu);
				} else {
					if (random.Rand(100) > 20) {
	               		offer = 1;			
					}
				}
				if (!offer) {
					SortBoxer(i).SetState(NO_MATCHING);				
				}
			}
			if (offer) {
				accept = 0;
				if (offer == 1) {
					for (j = i + 1; j < MAX_BOXER; j++) {
						if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
							if (player && Sort(j) == MY_BOXER) {
								WillAccept(&accept, Sort(i), menu);
							} else {
								accept = 1;
							}
						}
						if (accept) {
							break;
						}
					}			
				} else {
					j = ChoiceLowRanked(&accept, i, menu);
				}
				if (accept) {
					SetMatch(match_buf + match_number++, Sort(i), Sort(j), ROUND10_MATCH, ROUND10);
				}			
			}
		}  
	}
  
 	for (k = JAPAN; k <= SA_COUNTRIES; k++) {
		SortRegion(k);
/* eJ[}b`CN */
		for (i = 1; i <= 15; i++) {
			offer = 0;
			if (SortBoxer(i).IsWaiting()) {
				if (player && Sort(i) == MY_BOXER) {
					WillOffer(&offer, menu);
				} else {
					if (random.Rand(100) > 20) {
	               		offer = 1;			
					}
				}
				if (!offer) {
					SortBoxer(i).SetState(NO_MATCHING);				
				}
			}
			if (offer) {
				accept = 0;
				if (offer == 1) {
					for (j = i + 1; j < MAX_BOXER; j++) {
						if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
							if (player && Sort(j) == MY_BOXER) {
								WillAccept(&accept, Sort(i), menu);
							} else {
								accept = 1;
							}
						}
						if (accept) {
							break;
						}
					}			
				} else {
					j = ChoiceLowRanked(&accept, i, menu);
				}
				if (accept) {
					SetMatch(match_buf + match_number++, Sort(i), Sort(j), ROUND8_MATCH, ROUND8);
				}			
			}
		}   
/* eZ}b`CN */
		for (i = 16; i <= 50; i++) {
			offer = 0;
			if (SortBoxer(i).IsWaiting()) {
				if (player && Sort(i) == MY_BOXER) {
					WillOffer(&offer, menu);
				} else {
					if (random.Rand(100) > 20) {
	               		offer = 1;			
					}
				}
				if (!offer) {
					SortBoxer(i).SetState(NO_MATCHING);				
				}
			}
			if (offer) {
				accept = 0;
				if (offer == 1) {
					for (j = i + 1; j < MAX_BOXER; j++) {
						if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
							if (player && Sort(j) == MY_BOXER) {
								WillAccept(&accept, Sort(i), menu);
							} else {
								accept = 1;
							}
						}
						if (accept) {
							break;
						}
					}			
				} else {
					j = ChoiceLowRanked(&accept, i, menu);
				}
				if (accept) {
					SetMatch(match_buf + match_number++, Sort(i), Sort(j), ROUND6_MATCH, ROUND6);
				}			
			}
		}  
/* el}b`CN */
		for (i = 51; i < MAX_BOXER && SortBoxer(i).IsInRegion(); i++) {
			offer = 0;
			if (SortBoxer(i).IsWaiting()) {
				if (player && Sort(i) == MY_BOXER) {
					WillOffer(&offer, menu);
				} else {
					if (random.Rand(100) > 20) {
	               		offer = 1;			
					}
				}
				if (!offer) {
					SortBoxer(i).SetState(NO_MATCHING);				
				}
			}
			if (offer) {
				accept = 0;
				if (offer == 1) {
					for (j = i + 1; j < MAX_BOXER; j++) {
						if (SortBoxer(j).IsWaiting() && SortBoxer(j).IsInRegion()) {
							if (player && Sort(j) == MY_BOXER) {
								WillAccept(&accept, Sort(i), menu);
							} else {
								accept = 1;
							}
						}
						if (accept) {
							break;
						}
					}			
				} else {
					j = ChoiceLowRanked(&accept, i, menu);
				}
				if (accept) {
					if (SortBoxer(i).Match() == 0 || SortBoxer(j).Match() == 0) {					
						SetMatch(match_buf + match_number++, Sort(i), Sort(j), DEBUT_MATCH, ROUND4);
					} else {
						SetMatch(match_buf + match_number++, Sort(i), Sort(j), ROUND4_MATCH, ROUND4);						
					}
					if (SortBoxer(i).Match() == 0) {
						SortBoxer(i).SetState(DEBUT);
					}
					if (SortBoxer(j).Match() == 0) {
						SortBoxer(j).SetState(DEBUT);
					}
				}
			}
		} 
	}

	SortRegion(WORLD);
	for (i = 0; i < match_number; i++) {
		AddMatch(match_buf[i]);	
	}
}

void StoryHandler::QuitBoxer(int number) {
	int i, j;
	SetTitleHolder(Boxer(number).WhichTitle(), NO_BOXER);
	LeaveTournament(number);
	for (i = JAPAN; i <= WORLD; i++) {
		for (j = 0; j < 4; j++) {
			if (title[i].challengers[j] == number) {
				title[i].challengers[j] = NO_BOXER;			
			}
		}
	}
	for (i = 0; i < MAX_MATCH && match[i].match_type != NO_RESULT; i++) {
		if (match[i].high_ranked == number) {
			match[i].high_ranked = NO_BOXER;
			match[i].result = NO_CONTEST_BLUE;
			MatchRecord(match + i);

		} else if (match[i].low_ranked == number) {
			match[i].low_ranked = NO_BOXER;
			match[i].result = NO_CONTEST_RED;
			MatchRecord(match + i);
 	    }
	}
}

void StoryHandler::SwapBoxer(int number) {
	int i, j;
	StoryBoxer swap;
	swap = Boxer(MY_BOXER);
	Boxer(MY_BOXER) = Boxer(number);	
	Boxer(number) = swap;
    Boxer(MY_BOXER).ClearName();
	
	for (i = JAPAN; i <= WORLD; i++) {
		if (title[i].holder == number) {
			title[i].holder = MY_BOXER;		
		}	
		if (title[i].ex_holder == number) {
			title[i].ex_holder = MY_BOXER;		
		}	
		for (j = 0; j < 4; j++) {
			if (title[i].challengers[j] == number) {
				title[i].challengers[j] = MY_BOXER;
			}	
		}
		for (j = 0; j < 16; j++) {
			if (ranker[i][j] == number) {
			 	ranker[i][j] = MY_BOXER;
			}		
		}
		if (i >= JAPAN && i <= SA_COUNTRIES) {
			for (j = 0; j < 64; j++) {
				if (rookie[i][j] == number) {
				 	rookie[i][j] = MY_BOXER;
				}		
			}	
		}
	}
	for (i = 0; i < MAX_MATCH; i++) {
		if (match[i].high_ranked == number) {
			match[i].high_ranked = MY_BOXER;		
		}
		if (match[i].low_ranked == number) {
			match[i].low_ranked = MY_BOXER;		
		}
		if (result[i].high_ranked == number) {
			result[i].high_ranked = MY_BOXER;		
		}
		if (result[i].low_ranked == number) {
			result[i].low_ranked = MY_BOXER;		
		}
	}
}


void StoryHandler::AgeBoxers(Game &menu, int player) {
	int i;
	for (i = 0; i < MAX_BOXER; i++) {
		Boxer(i).AgeP();	
		if (Boxer(i).Age() >= 30) {
			Boxer(i).PowerDown();		
		}
		if (Boxer(i).Age() == 50) {
			QuitBoxer(i);
			if (player && i == MY_BOXER) {
				menu.RetireMenu();
				menu.Retire(*this);	
				menu.ShowScore(*this);
				Boxer(0).Inherit(random);
				menu.SelectGameMode(*this);
				ClearRecord();
				SetNewRanking();
			} else {
				Boxer(i).CreateRandom(random, Boxer(i).Nation());	
			}
		}
/* Ȃg[igoȂꍇ̂Ŕi */
		if (Boxer(i).State() >= RANKER_T0 && Boxer(i).State() <= RANKER_T15) {
		 	Boxer(i).SetState(WAIT_STATE);
		}
		if (Boxer(i).State() >= ROOKIE_T0 && Boxer(i).State() <= ROOKIE_T63) {
		 	Boxer(i).SetState(WAIT_STATE);
		}
	}
}


void StoryHandler::RetireBoxers(Game &menu, int player) {
	static int i, j, numbers[SA_COUNTRIES+1];
	static int number, defeats, age, point;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		numbers[i] = 0;	
	} 
	for (i = 0; i < MAX_BOXER; i++) {
		numbers[Boxer(i).Nation()]++;	
	} 

	RegionResult(WORLD);
	for (i = 0; i < MaxMatch(); i++) {
		number = NO_BOXER;
		switch(SelectResult(i)->result) {
			case RED_WON_KO :
			case RED_WON_TKO :
			case RED_WON_JUDGE :
				number = SelectResult(i)->low_ranked;
			break;				
			case BLUE_WON_TKO :
			case BLUE_WON_KO :
			case BLUE_WON_JUDGE :
				number = SelectResult(i)->high_ranked;
			break;
		}

		if (number != NO_BOXER) {
			defeats = Boxer(number).DefeatSeries();
			age = Boxer(number).Age();
			point = Boxer(number).Point();
	/*		if (defeats == 0) {
				fprintf(stderr, "high defeats = %d\n", Boxer(SelectResult(i)->high_ranked).DefeatSeries());
				fprintf(stderr, "low defeats = %d\n", Boxer(SelectResult(i)->low_ranked).DefeatSeries());
				fprintf(stderr, "match_type = %d\n", SelectResult(i)->match_type);
			
			}    */
			if ((point < 0) || 
				(defeats > 0 && age >= 40) ||
				(defeats > 1 && age >= 30) ||
				(defeats > 2 && age >= 20) ||
				(defeats > 3 && age >= 10)) {
//				fprintf(stderr, "defeats = %d age = %d\n", defeats, age);
				if (player && number == MY_BOXER) {
				 	menu.RetireMenu();
					menu.Retire(*this);	
					menu.ShowScore(*this);
					Boxer(number).Inherit(random);
					menu.SelectGameMode(*this);
					ClearRecord();
				} else {
					numbers[Boxer(number).Nation()]--;
					for (j = JAPAN; j <= SA_COUNTRIES; j++) {
						if (numbers[j] < 64) {
							Boxer(number).CreateRandom(random, j);	
							numbers[j]++;
							break;				
						}
					}
					if (j > SA_COUNTRIES) {
						Boxer(number).CreateRandom(random, random.Rand(JAPAN, SA_COUNTRIES));					
						numbers[Boxer(number).Nation()]++;
					}
				}
			}		
		}
	} 
}

int StoryHandler::SimpleMatch(int number1, int number2) {
	StoryBoxer &sb1 = Boxer(number1);
	StoryBoxer &sb2 = Boxer(number2);
	static int total1, total2, power1, power2, dp, result;

	total1 = sb1.Total();
	total2 = sb2.Total(); 

	power1 = random.Rand(total1*total1/100) * (100 - sb1.Traum());
	power2 = random.Rand(total2*total2/100) * (100 - sb2.Traum());

	dp = power1 - power2;

//	fprintf(stderr, "%d vs %d = %d\n", power1, power2, dp);
	
	if (dp > 0) {
		if (dp > 25000) {
			result = RED_WON_KO;
		} else if (dp > 500){
			result = RED_WON_JUDGE;
		} else {	
			result = JUDGE_DRAW_RED;
		}
	} else if (dp < 0) {
		if (dp < -25000) {
			result = BLUE_WON_KO;
		} else if (dp < -500){
			result = BLUE_WON_JUDGE;
		} else {	
			result = JUDGE_DRAW_BLUE;
		}
	} else {
		if (random.Rand() % 2) {
			result = JUDGE_DRAW_RED;
		} else {
			result = JUDGE_DRAW_BLUE;
		}
	}
	return result;    
}

void StoryHandler::MatchRecord(StoryMatch *match) {
	StoryBoxer &sb1 = Boxer(match->high_ranked);
	StoryBoxer &sb2 = Boxer(match->low_ranked);
	static int region, point1, point2, point_low, point_high;

	if (match->result != NO_CONTEST_RED && match->result != NO_CONTEST_BLUE) {
		sb1.MatchP();
		sb2.MatchP();
	}
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
			sb1.WinP();
			sb1.KOP();
			sb1.PowerUp(sb2);
			sb1.AddRecent(1);
			sb1.ClearTraum();
			sb2.LossP();
			sb2.AddRecent(6);
			sb2.AddTraum(25);
		break;
		case RED_WON_JUDGE :
			sb1.WinP();
			sb1.PowerUp(sb2);
			sb1.AddRecent(2);
			sb1.ClearTraum();
			sb2.LossP();
			sb2.AddRecent(5);
			sb2.AddTraum(10);
		break;
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
			sb2.WinP();
			sb2.KOP();
			sb2.PowerUp(sb1);
			sb2.AddRecent(1);
			sb2.ClearTraum();
			sb1.LossP();
			sb1.AddRecent(6);
			sb1.AddTraum(25);
		break;
		case BLUE_WON_JUDGE :
			sb2.WinP();
			sb2.PowerUp(sb1);
			sb2.AddRecent(2);
			sb2.ClearTraum();
			sb1.LossP();
			sb1.AddRecent(5);
			sb1.AddTraum(10);
		break;
    	case JUDGE_DRAW :
	  	case JUDGE_DRAW_RED :
  		case JUDGE_DRAW_BLUE :
			sb1.DrawP();
			sb1.AddRecent(3);
			sb2.DrawP();
			sb2.AddRecent(3);
		break;
    	case DKO_DRAW :
	  	case DKO_DRAW_RED :
  		case DKO_DRAW_BLUE :
			sb1.DrawP();	
			sb1.AddRecent(4);
			sb2.DrawP();
			sb2.AddRecent(4);
		break;
	}

	point1 = sb1.Point();
	point2 = sb2.Point();
	if (point1 > point2) {
		point_high = (point1 - point2)/2;	
		point_low  = 10000 / (point_high + 10);	
		point_high += point_low;
//		fprintf (stderr, "high = %d  low = %d\n", point_high, point_low); 
	} else {
		point_high = (point2 - point1)/2;	
		point_low  = 10000 / (point_high + 10);	
		point_high += point_low;
//		fprintf (stderr, "high = %d  low = %d\n", point_high, point_low); 
	}

	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
			if (point1 > point2) {
				sb1.AddPoint(point_low*2);
				sb2.DelPoint(point_low*2);
			} else {
				sb1.AddPoint(point_high*2);
				sb2.DelPoint(point_high*2);			
			}
		break;

		case RED_WON_JUDGE :
			if (point1 > point2) {
				sb1.AddPoint(point_low);
				sb2.DelPoint(point_low);
			} else {
				sb1.AddPoint(point_high);
				sb2.DelPoint(point_high);			
			}
		break;

		case BLUE_WON_KO :
		case BLUE_WON_TKO :
			if (point2 > point1) {
				sb2.AddPoint(point_low*2);
				sb1.DelPoint(point_low*2);
			} else {
				sb2.AddPoint(point_high*2);
				sb1.DelPoint(point_high*2);			
			}
		break;

		case BLUE_WON_JUDGE :
			if (point2 > point1) {
				sb2.AddPoint(point_low);
				sb1.DelPoint(point_low);
			} else {
				sb2.AddPoint(point_high);
				sb1.DelPoint(point_high);			
			}
		break;
	}

	switch (match->match_type) {
		case WORLD_TITLE_MATCH : 
		case AREA_TITLE_MATCH : 
		case NATION_TITLE_MATCH : 
			region = sb1.WhichTitle();			
			TitleMatch(region, match);			
		break;

		case WORLD_CHAMPIONSHIP :
			Championship(WORLD, match);			
		break;
		case AREA_CHAMPIONSHIP : 
			Championship(sb1.Area(), match);			
		break;
		case NATION_CHAMPIONSHIP : 
			Championship(sb1.Nation(), match);			
		break;

		case WORLD_RANKER_FINAL : 
			RankerFinal(WORLD, match);			
		break;
		case AREA_RANKER_FINAL : 
			RankerFinal(sb1.Area(), match);			
		break;
		case NATION_RANKER_FINAL : 
			RankerFinal(sb1.Nation(), match);			
		break;

		case ROOKIE_KING_FINAL : 
			RookieFinal(match);			
		break;

		case WORLD_RANKER_SEMIFINAL : 
		case WORLD_RANKER_QUARTERFINAL : 
		case AREA_RANKER_SEMIFINAL : 
		case AREA_RANKER_QUARTERFINAL : 
		case NATION_RANKER_SEMIFINAL : 
		case NATION_RANKER_QUARTERFINAL : 
		case NATION_RANKER_1ST : 
		case ROOKIE_KING_SEMIFINAL : 
		case ROOKIE_KING_QUARTERFINAL : 
		case ROOKIE_KING_3RD : 	
		case ROOKIE_KING_2ND : 
		case ROOKIE_KING_1ST : 
			WhichLeave(match);			
		break;

		case ROUND10_MATCH : 	
		case ROUND8_MATCH : 	
			SetRestRound8(match);
		break;
		case ROUND6_MATCH : 
		case ROUND4_MATCH : 
		case DEBUT_MATCH : 
			SetRestRound4(match);
		break;
	}
}

void StoryHandler::TitleMatch(int region, StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case RED_WON_JUDGE :
		case JUDGE_DRAW :
		case JUDGE_DRAW_RED :
  		case JUDGE_DRAW_BLUE :
		case DKO_DRAW :
		case DKO_DRAW_RED :
  		case DKO_DRAW_BLUE :
 		case NO_CONTEST_RED :
			TitleDefended(region);
			SetNextSchedule(region, match->high_ranked);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
		case BLUE_WON_JUDGE :
			SetTitleHolder(region, match->low_ranked);
			Boxer(match->low_ranked).ChangeRecent(7);
			SetNextSchedule(region, match->low_ranked);
		break;
	}
 	SetRest(match);
}

void StoryHandler::Championship(int region, StoryMatch *match) {
	LeaveTournament(match->low_ranked);
	LeaveTournament(match->high_ranked);
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case RED_WON_JUDGE :
		case JUDGE_DRAW_RED :
		case DKO_DRAW_RED :
 		case NO_CONTEST_RED :
			SetTitleHolder(region, match->high_ranked);
//			SetRankerWinner(region, match->high_ranked);
			SetNextSchedule(region, match->high_ranked);
			Boxer(match->high_ranked).ChangeRecent(7);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
		case BLUE_WON_JUDGE :
  		case JUDGE_DRAW_BLUE :
  		case DKO_DRAW_BLUE :
 		case NO_CONTEST_BLUE :
			SetTitleHolder(region, match->low_ranked);
//			SetRankerWinner(region, match->low_ranked);
			SetNextSchedule(region, match->low_ranked);
			Boxer(match->low_ranked).ChangeRecent(7);
		break;
	}
 	SetRest(match);
}

void StoryHandler::RankerFinal(int region, StoryMatch *match) {
	LeaveTournament(match->low_ranked);
	LeaveTournament(match->high_ranked);
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case RED_WON_JUDGE :
		case JUDGE_DRAW_RED :
		case DKO_DRAW_RED :
 		case NO_CONTEST_RED :
			SetRankerWinner(region, match->high_ranked);
			Boxer(match->high_ranked).ChangeRecent(7);
			WinnerToChallenger(region, match->high_ranked);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
		case BLUE_WON_JUDGE :
  		case JUDGE_DRAW_BLUE :
  		case DKO_DRAW_BLUE :
 		case NO_CONTEST_BLUE :
			SetRankerWinner(region, match->low_ranked);
			Boxer(match->low_ranked).ChangeRecent(7);
			WinnerToChallenger(region, match->low_ranked);
		break;
	}
 	SetRest(match);
}

void StoryHandler::RookieFinal(StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case RED_WON_JUDGE :
		case JUDGE_DRAW_RED :
		case DKO_DRAW_RED :
 		case NO_CONTEST_RED :
			Boxer(match->high_ranked).GetTitle(ROOKIE_KING);
			Boxer(match->high_ranked).ChangeRecent(7);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
		case BLUE_WON_JUDGE :
  		case JUDGE_DRAW_BLUE :
  		case DKO_DRAW_BLUE :
 		case NO_CONTEST_BLUE :
			Boxer(match->low_ranked).GetTitle(ROOKIE_KING);
			Boxer(match->low_ranked).ChangeRecent(7);
		break;
	}
	LeaveTournament(match->high_ranked);
	LeaveTournament(match->low_ranked);
 	SetRest(match);
}

void StoryHandler::WhichLeave(StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case RED_WON_JUDGE :
		case JUDGE_DRAW_RED :
		case DKO_DRAW_RED :
			LeaveTournament(match->low_ranked);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
		case BLUE_WON_JUDGE :
  		case JUDGE_DRAW_BLUE :
  		case DKO_DRAW_BLUE :
			LeaveTournament(match->high_ranked);
		break;
	}
	SetRest(match);
}

void StoryHandler::SetRest(StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case DKO_DRAW_RED :
			Boxer(match->low_ranked).SetState(REST_MONTH0+3);
		break;
		case RED_WON_JUDGE :
		case JUDGE_DRAW_RED :
			Boxer(match->low_ranked).SetState(REST_MONTH0+1);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
  		case DKO_DRAW_BLUE :
			Boxer(match->high_ranked).SetState(REST_MONTH0+3);
		break;
		case BLUE_WON_JUDGE :
  		case JUDGE_DRAW_BLUE :
			Boxer(match->high_ranked).SetState(REST_MONTH0+1);
		break;
	}
}

void StoryHandler::SetRestRound8(StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case DKO_DRAW_RED :
			Boxer(match->high_ranked).SetState(REST_MONTH0+2);
			Boxer(match->low_ranked).SetState(REST_MONTH0+3);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
  		case DKO_DRAW_BLUE :
			Boxer(match->low_ranked).SetState(REST_MONTH0+2);
			Boxer(match->high_ranked).SetState(REST_MONTH0+3);
		break;

		case RED_WON_JUDGE :
		case BLUE_WON_JUDGE :
		case JUDGE_DRAW :
		case JUDGE_DRAW_RED :
  		case JUDGE_DRAW_BLUE :
			Boxer(match->high_ranked).SetState(REST_MONTH0+2);
			Boxer(match->low_ranked).SetState(REST_MONTH0+2);
		break;

		case DKO_DRAW :
			Boxer(match->high_ranked).SetState(REST_MONTH0+3);
			Boxer(match->low_ranked).SetState(REST_MONTH0+3);
		break;

		case NO_CONTEST_RED :
			Boxer(match->high_ranked).SetState(REST_MONTH0);		
		break;
		case NO_CONTEST_BLUE :
			Boxer(match->low_ranked).SetState(REST_MONTH0);				
		break;
	}
}

void StoryHandler::SetRestRound4(StoryMatch *match) {
	switch(match->result) {
		case RED_WON_KO :
		case RED_WON_TKO :
		case DKO_DRAW_RED :
			Boxer(match->high_ranked).SetState(REST_MONTH0+1);
			Boxer(match->low_ranked).SetState(REST_MONTH0+3);
		break;
				
		case BLUE_WON_TKO :
		case BLUE_WON_KO :
  		case DKO_DRAW_BLUE :
			Boxer(match->low_ranked).SetState(REST_MONTH0+1);
			Boxer(match->high_ranked).SetState(REST_MONTH0+3);
		break;

		case RED_WON_JUDGE :
		case BLUE_WON_JUDGE :
		case JUDGE_DRAW :
		case JUDGE_DRAW_RED :
  		case JUDGE_DRAW_BLUE :
			Boxer(match->high_ranked).SetState(REST_MONTH0+1);
			Boxer(match->low_ranked).SetState(REST_MONTH0+1);
		break;

		case DKO_DRAW :
			Boxer(match->high_ranked).SetState(REST_MONTH0+3);
			Boxer(match->low_ranked).SetState(REST_MONTH0+3);
		break;

		case NO_CONTEST_RED :
			Boxer(match->high_ranked).SetState(REST_MONTH0);		
		break;
		case NO_CONTEST_BLUE :
			Boxer(match->low_ranked).SetState(REST_MONTH0);				
		break;
	}
}

void StoryHandler::SetRankerWinner(int region, int winner) {
	if (region != OUT_OF_REGION) {
		if (region >= JAPAN && region <= SA_COUNTRIES) {
			Boxer(winner).GetTitle(NATION_RANKER_WINNER);
		} else if (region >= ASIA_OCEANIA && region <= SOUTH_AMERICA) {
			Boxer(winner).GetTitle(AREA_RANKER_WINNER);	
		} else if (region == WORLD) {
			Boxer(winner).GetTitle(WORLD_RANKER_WINNER);
		}	
	}
}

void StoryHandler::WillDefend(int *offer, int region, Game &menu) {
	switch (menu.DefendMenu(region)) {
		case 1 :
			*offer = 2; /* hq(I) */
		break;
		case 2 :
			SetTitleHolder(region, NO_BOXER);  /* ԏシ */														
		break;
		default :
			*offer = 1; /* hq() */
		break;
	}
}

void StoryHandler::WillChallenge(int *accept, int number, int month, Game &menu) {
	int choice;
	do {
		choice = menu.ChallengerMenu(SelectRegion(), month);
		switch (choice) {
			case 1:		 /* 킷 */
				*accept = 1;
			break;
			case 2:		/* 킵Ȃ */
			break;
			default :	/* vtB[ */
				if (number != -1) {
					menu.ShowBoxerProfile(*this, number);
				} else {
					menu.UndecidedMenu();				
				}
			break;
		}
	} while (!choice);
}

int StoryHandler::SelectCandidates(int *candidates) {
	static int j, max_select;
	max_select = 0;
	for (j = 1; j <= 15; j++) {
		if (SortBoxer(j).IsWaiting()) {
			candidates[max_select++] = j;
		}
	}
	return max_select;
}

void StoryHandler::DeleteCandidate(int *candidates, int delete_challenger) {
	static int j, number;
	for (j = 0; j < 15; j++) {
		if (candidates[j] == delete_challenger) {
			number = j;
			break;
		}
	}
	for (j = number; j < 14; j++) {
		candidates[j] = candidates[j+1];
	}
}
       
void StoryHandler::SelectChallengers(Game &menu, int player) {
	static int schedule[4][4] = {{5, 0, 0, 0}, {4, 8, 0, 0}, {2, 5, 8, 0}, {2, 4, 6, 8}};
	static int candidates[15];
	int i, j, max_challenger, challenger_number, schedule_number, t_number;
	int offer, accept, challenger;

	for (i = 0; i < WORLD; i++) {
		title[i].challengers[0] = NO_BOXER;	
	}	

	SortRegion(WORLD);
/* E^Cghq? */
	offer = 0;
	if (SortBoxer(0).HaveTitle(WORLD_TITLE)) {
		if (player && Sort(0) == MY_BOXER) {
			WillDefend(&offer, WORLD, menu);
		} else {
	    	offer = 1;			
		}
	}
/* E^CgґI */
	max_challenger = SelectCandidates(candidates);
	if (max_challenger > 0 && offer) { /* Ҍ₠ */	
		if (offer == 1) { /* Ґ̌ */
			challenger_number = random.Rand(1, 3);
		} else {
			challenger_number = menu.HowManyMenu(3);		
		}
		if (challenger_number > max_challenger) {
			challenger_number = max_challenger;			
		}
		/* XPW[ݒ */
		schedule_number = challenger_number - 1;
		challenger_number = 0;		
		for (i = 0; i < 15; i++) {
			if (offer == 1) {	/*  */
				challenger = candidates[random.Rand(max_challenger-1)];					
			} else {			/* I */
				challenger = menu.ShowChallengers(*this, candidates, max_challenger-1);			
			}
			accept = 0;
			if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
				WillChallenge(&accept, Sort(0), 1 + schedule[schedule_number][challenger_number], menu);
			} else {
				accept = 1;
			}
			if (accept) {
				title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
				SortBoxer(challenger).SetState(1 + schedule[schedule_number][challenger_number]);
				SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
				if (++challenger_number > schedule_number) {
					break;				
				}
			}
			DeleteCandidate(candidates, challenger);
			max_challenger--;		
		}
		if (challenger_number > 0) {	/* ҂ */
			title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
			SortBoxer(0).SetState(1 + schedule[schedule_number][0]);
		} else {
			SortBoxer(0).SetState(11);	/* ҂Ȃ */	
		}
	} else if (max_challenger > 3) {/* ʂ̏ꍇ */
		challenger_number = 0;		
		for (i = 0; i < 4; i++) {
			challenger = candidates[i];					
			accept = 0;
			if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
				WillChallenge(&accept, -1, 1 + schedule[1][challenger_number > 0 ? challenger_number - 1 : 0], menu);
			} else {
				accept = 1;
			}
			if (accept) {
				title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
				SortBoxer(challenger).SetState(1 + schedule[1][challenger_number > 0 ? challenger_number - 1 : 0]);
				SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
				if (++challenger_number > 2) {
					break;				
				}
			}
		}
		title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
	}
	
/* EJ[g[igQ */
	for (i = 0; i < 16; i++) {
		ranker[WORLD][i] = NO_BOXER;	
	}	
	i = 0; 	challenger_number = 0;
	do {
		accept = 0;
		if (SortBoxer(i).IsWaiting() &&
			(SortBoxer(i).RegionRank() <= 10 ||
			SortBoxer(i).HaveTitle(AREA_TITLE))) {
			if (player && Sort(i) == MY_BOXER) {
				if (menu.ParticipateMenu(RANKER, WORLD) == 0) {
    	   	     	accept = 1;	/* Q */
				} else {
					accept = 0;  /* QȂ */			
				}
			} else {
	    		accept = 1;
			}
		}
		if (accept) {
			t_number = Pairing(challenger_number++, 8);
			ranker[WORLD][t_number] = Sort(i);
			SortBoxer(i).SetState(RANKER_T0 + t_number);
			SetTitleHolder(SortBoxer(i).WhichTitle(), NO_BOXER);
		}			
		i++;
	} while (challenger_number < 8 && i < MAX_BOXER);
	
	for (j = 0; j < 5; j++) {
		SortRegion(ASIA_OCEANIA+j);
/* en^Cghq */
		offer = 0;
		if (SortBoxer(0).HaveTitle(AREA_TITLE)) {
			if (player && Sort(0) == MY_BOXER) {
				WillDefend(&offer, SelectRegion(), menu);
			} else {
		    	offer = 1;			
			}
		}
/* en^CgґI */
		max_challenger = SelectCandidates(candidates);
		if (max_challenger > 0 && offer) { /* Ҍ₠ */	
			if (offer == 1) {	/* Ґ̌ */
				challenger_number = random.Rand(1, 3);
			} else {
				challenger_number = menu.HowManyMenu(3);		
			}
			if (challenger_number > max_challenger) {
				challenger_number = max_challenger;			
			}
			/* XPW[ݒ */
			schedule_number = challenger_number - 1;
			challenger_number = 0;		
			for (i = 0; i < 15; i++) {
				if (offer == 1) {	/*  */
					challenger = candidates[random.Rand(max_challenger-1)];					
				} else {			/* I */
					challenger = menu.ShowChallengers(*this, candidates, max_challenger-1);			
				}
				accept = 0;
				if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
					WillChallenge(&accept, Sort(0), schedule[schedule_number][challenger_number], menu);
				} else {
					accept = 1;
				}
				if (accept) {
					title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
					SortBoxer(challenger).SetState(schedule[schedule_number][challenger_number]);
					SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
					if (++challenger_number > schedule_number) {
						break;				
					}
				}
				DeleteCandidate(candidates, challenger);
				max_challenger--;		
			}
			if (challenger_number > 0) {	/* ҂ */
				title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
				SortBoxer(0).SetState(schedule[schedule_number][0]);
			} else {						/* ҂Ȃ */
				SortBoxer(0).SetState(10);		
			}     
		} else if (max_challenger > 3) {/* ʂ̏ꍇ */
			challenger_number = 0;		
			for (i = 0; i < 4; i++) {
				challenger = candidates[i];					
				accept = 0;
				if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
					WillChallenge(&accept, -1, schedule[1][challenger_number > 0 ? challenger_number - 1 : 0], menu);
				} else {
					accept = 1;
				}
				if (accept) {
					title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
					SortBoxer(challenger).SetState(schedule[1][challenger_number > 0 ? challenger_number - 1 : 0]);
					SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
					if (++challenger_number > 2) {
						break;				
					}
				}
			}
			title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
		}
/* en惉J[g[igQ */
		for (i = 0; i < 16; i++) {
			ranker[SelectRegion()][i] = NO_BOXER;	
		}	
		i = 0;	challenger_number = 0;
		do {
			accept = 0;
			if (SortBoxer(i).IsWaiting() && SortBoxer(i).IsInRegion() &&
				(SortBoxer(i).RegionRank() <= 10 ||
				SortBoxer(i).HaveTitle(NATION_TITLE))) {
				if (player && Sort(i) == MY_BOXER) {
					if (menu.ParticipateMenu(RANKER, SelectRegion()) == 0) {
	   	 	   	     	accept = 1;	/* Q */
					} else {
						accept = 0;	/* QȂ */			
					}
				} else {
		    		accept = 1;
				}
			}
			if (accept) {
				t_number = Pairing(challenger_number++, 8);
				ranker[SelectRegion()][t_number] = Sort(i);
				SortBoxer(i).SetState(RANKER_T0 + t_number);
				SetTitleHolder(SortBoxer(i).WhichTitle(), NO_BOXER);
			}			
			i++;
		} while (challenger_number < 8 && i < MAX_BOXER); 
	}

	for (j = 0; j <= SA_COUNTRIES; j++) {
		SortRegion(j);
/* e^Cghq */
		offer = 0;
		if (SortBoxer(0).HaveTitle(NATION_TITLE)) {
			if (player && Sort(0) == MY_BOXER) {
				WillDefend(&offer, SelectRegion(), menu);
			} else {
		    	offer = 1;			
			}
		}
/* e^CgґI */
		max_challenger = SelectCandidates(candidates);
		if (max_challenger > 0 && offer) { /* Ҍ₠ */	
			if (offer == 1) {	/* Ґ̌ */
				challenger_number = 1 + random.Rand(1, 3);
			} else {
				challenger_number = menu.HowManyMenu(4);		
			}
			if (challenger_number > max_challenger) {
				challenger_number = max_challenger;			
			}
			/* XPW[ݒ */
			schedule_number = challenger_number - 1;
			challenger_number = 0;		
			for (i = 0; i < 15; i++) {
				if (offer == 1) {	/*  */
					challenger = candidates[random.Rand(max_challenger-1)];					
				} else {			/* I */
					challenger = menu.ShowChallengers(*this, candidates, max_challenger-1);			
				}
				accept = 0;
				if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
					WillChallenge(&accept, Sort(0), schedule[schedule_number][challenger_number], menu);
				} else {
					accept = 1;
				}
				if (accept) {
					title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
					SortBoxer(challenger).SetState(schedule[schedule_number][challenger_number]);
					SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
					if (++challenger_number > schedule_number) {
						break;				
					}
				}
				DeleteCandidate(candidates, challenger);
				max_challenger--;		
			}
			if (challenger_number > 0) {	/* ҂ */
				title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
				SortBoxer(0).SetState(schedule[schedule_number][0]);
			} else {						/* ҂Ȃ */
				SortBoxer(0).SetState(10);		
			}     
		} else if (max_challenger > 4) {/* ʂ̏ꍇ */
			challenger_number = 0;		
			for (i = 0; i < 5; i++) {
				challenger = candidates[i];					
				accept = 0;
				if (player && Sort(challenger) == MY_BOXER) {/* 킷? */
					WillChallenge(&accept, -1, schedule[2][challenger_number > 0 ? challenger_number - 1 : 0], menu);
				} else {
					accept = 1;
				}
				if (accept) {
					title[SelectRegion()].challengers[challenger_number] = Sort(challenger);
					SortBoxer(challenger).SetState(schedule[2][challenger_number > 0 ? challenger_number - 1 : 0]);
					SetTitleHolder(SortBoxer(challenger).WhichTitle(), NO_BOXER);
					if (++challenger_number > 3) {
						break;				
					}
				}
			}
			title[SelectRegion()].challengers[challenger_number] = NO_BOXER;
		}
/* eg[igQ */
		for (i = 0; i < 16; i++) {
			ranker[j][i] = NO_BOXER;	
		}	
		i = 0;	challenger_number = 0;
		do {
			accept = 0;
			if (SortBoxer(i).IsWaiting() && SortBoxer(i).IsInRegion() &&
				(SortBoxer(i).RegionRank() <= 15 ||
				(SortBoxer(i).RegionRank() <= 20 && challenger_number < 13) ||
				SortBoxer(i).HaveTitle(ROOKIE_KING))) {
				if (player && Sort(i) == MY_BOXER) {
					if (menu.ParticipateMenu(RANKER, SelectRegion()) == 0) {
	   	 	   	     	accept = 1;	/* Q */
					} else {
						accept = 0;  /* QȂ */			
					}
				} else {
		    		accept = 1;
				}
			}
			if (accept) {
				t_number = Pairing(challenger_number++, 16);
				ranker[j][t_number] = Sort(i);
				SortBoxer(i).SetState(RANKER_T0 + t_number);
			}			
			i++;
		} while (challenger_number < 16 && i < MAX_BOXER);   

/* Vlg[igQ */
		for (i = 0; i < 64; i++) {
			rookie[j][i] = NO_BOXER;	
		}	
		i = 0;	challenger_number = 0;
		do {
			accept = 0;
			if (SortBoxer(i).IsWaiting() && SortBoxer(i).IsInRegion() &&
				(SortBoxer(i).RegionRank() >= 50 && SortBoxer(i).Career() <= 24)) {
				if (player && Sort(i) == MY_BOXER) {
					if (menu.ParticipateMenu(ROOKIE, SelectRegion()) == 0) {
	   	 	   	     	accept = 1;	/* Q */
					} else {
						accept = 0;  /* QȂ */			
					}
				} else {
		    		accept = 1;
				}
			}
			if (accept) {
				t_number = Pairing(challenger_number++, 64);
				rookie[j][t_number] = Sort(i);
				SortBoxer(i).SetState(ROOKIE_T0 + t_number);
			}			
			i++;
		} while (challenger_number < 64 && i < MAX_BOXER); 
	}   	
}

void StoryHandler::SelectDesignatedChallenger(int region, Game &menu, int player) {
	int i, challenger, accept;
	SortRegion(region);

	challenger = NO_BOXER;
	for (i = 1; i <= 15; i++) {
		accept = 0;
		if (SortBoxer(i).IsWaiting() || (SortBoxer(i).State() >= REST_MONTH0 && SortBoxer(i).State() <= REST_MONTH0 + 3)) {
			if (player && Sort(i) == MY_BOXER) {
				WillChallenge(&accept, Sort(0), 3, menu);
			} else {
				accept = 1;
			}
		}
		if (accept) {
			challenger = Sort(i);
			SetTitleHolder(Boxer(challenger).WhichTitle(), NO_BOXER);
			WinnerToChallenger(region, challenger);
			SortBoxer(0).SetState(MATCH_MONTH0 + 3);	
			break;
		}	
	}
}

void StoryHandler::WinnerToChallenger(int region, int challenger) {
	int i;
	for (i = 0; i < 4; i++) {
		if (title[region].challengers[i] == NO_BOXER) {
			title[region].challengers[i] = challenger;
			title[region].challengers[i+1] = NO_BOXER;
			break;				
		}	
	}
	Boxer(challenger).SetState(MATCH_MONTH0 + 3);	
}

void StoryHandler::SetChallengers() {
	int i, j, number1, number2;
	StoryMatch title_match;

	for (i = JAPAN; i <= WORLD; i++) {
		if (TitleHolder(i) != NO_BOXER) {
			number1 = TitleHolder(i);
			number2 = title[i].challengers[0];
			if (number2 != NO_BOXER && Boxer(number2).State() == MATCH_MONTH0) {
				if (i >= JAPAN && i <= SA_COUNTRIES) {
					title_match.match_type = NATION_TITLE_MATCH;
				} else if(i >= ASIA_OCEANIA && i <= SOUTH_AMERICA) {
					title_match.match_type = AREA_TITLE_MATCH;
				} else {
					title_match.match_type = WORLD_TITLE_MATCH;
				}
				title_match.high_ranked = number1;
				title_match.low_ranked = number2;
				title_match.result = NO_RESULT;
				title_match.replay = 0;
				AddMatch(title_match);
				for (j = 0; j < 4; j++) {
					title[i].challengers[j] = title[i].challengers[j+1];	
				}
			}
		} else {
			number1 = title[i].challengers[0];
			number2 = title[i].challengers[1];
			if (number1 != NO_BOXER && number2 != NO_BOXER && Boxer(number1).State() == MATCH_MONTH0) {
				if (i >= JAPAN && i <= SA_COUNTRIES) {
					title_match.match_type = NATION_CHAMPIONSHIP;
				} else if(i >= ASIA_OCEANIA && i <= SOUTH_AMERICA) {
					title_match.match_type = AREA_CHAMPIONSHIP;
				} else {
					title_match.match_type = WORLD_CHAMPIONSHIP;
				}		
				title_match.high_ranked = number1;
				title_match.low_ranked = number2;
				title_match.result = NO_RESULT;
				title_match.replay = 0;
				AddMatch(title_match);
				for (j = 0; j < 3; j++) {
					title[i].challengers[j] = title[i].challengers[j+2];	
				}
			}
		}
	}		
}

void StoryHandler::SetNextSchedule(int region, int number) {
	if (title[region].challengers[0] != NO_BOXER) {
		Boxer(number).SetState(Boxer(title[region].challengers[0]).State());	
//		fprintf(stderr, "state = %d\n", Boxer(title[region].challengers[0]).State());
	} else {
		if (Month() > 4) {
			if (region == WORLD) {
				Boxer(number).SetState(MATCH_MONTH0 + 15 - Month());		
			} else {
				Boxer(number).SetState(MATCH_MONTH0 + 14 - Month());			
			}
		}
	}
}

int StoryHandler::Pairing(int number, int max_number) {
	if (number % 2) {
		number = ~number + 1;
		number &= (max_number - 1); 
	}
	if (max_number == 8) {
		if (number% 2) {
			number= number*2 + 1;
		} else {
			number= number*2;		
		}
	}
	return number;
}

void StoryHandler::LeaveTournament(int number) {
	int i;
	if (Boxer(number).State() >= RANKER_T0 && Boxer(number).State() <= RANKER_T15) {
		for (i = JAPAN; i <= WORLD; i++) {
			if (ranker[i][Boxer(number).State() - RANKER_T0] == number) {
				ranker[i][Boxer(number).State() - RANKER_T0] = NO_BOXER;
			    break;
			}		
		}
		Boxer(number).SetState(WAIT_STATE);	
	}
	if (Boxer(number).State() >= ROOKIE_T0 && Boxer(number).State() <= ROOKIE_T63) {
		rookie[Boxer(number).Nation()][Boxer(number).State() - ROOKIE_T0] = NO_BOXER;
		Boxer(number).SetState(WAIT_STATE);	
	}
}

void StoryHandler::WorldRankerQuarterFinal() {
	SetRankerTourament(4, 16, WORLD_RANKER_QUARTERFINAL, WORLD);
}

void StoryHandler::WorldRankerSemiFinal() {
	SetRankerTourament(2, 16, WORLD_RANKER_SEMIFINAL, WORLD);
}

void StoryHandler::WorldRankerFinal() {
//	if (TitleHolder(WORLD) == NO_BOXER) {
//		SetRankerTourament(1, 16, WORLD_CHAMPIONSHIP, WORLD);
//	} else {
		SetRankerTourament(1, 16, WORLD_RANKER_FINAL, WORLD);	
//	}
}

void StoryHandler::AreaRankerQuarterFinal() {
	int i;
	for (i = ASIA_OCEANIA; i <= SOUTH_AMERICA; i++) {
		SetRankerTourament(4, 16, AREA_RANKER_QUARTERFINAL, i);
	}
}

void StoryHandler::AreaRankerSemiFinal() {
	int i;
	for (i = ASIA_OCEANIA; i <= SOUTH_AMERICA; i++) {
		SetRankerTourament(2, 16, AREA_RANKER_SEMIFINAL, i);
	}
}

void StoryHandler::AreaRankerFinal() {
	int i;
	for (i = ASIA_OCEANIA; i <= SOUTH_AMERICA; i++) {
	//	if (TitleHolder(i) == NO_BOXER) {
	//		SetRankerTourament(1, 16, AREA_CHAMPIONSHIP, i);
	//	} else {
		SetRankerTourament(1, 16, AREA_RANKER_FINAL, i);
	//	}
	}
}

void StoryHandler::NationRanker1stQualifing() {
	int i;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		SetRankerTourament(8, 16, NATION_RANKER_1ST, i);
	}
}

void StoryHandler::NationRankerQuarterFinal() {
	int i;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		SetRankerTourament(4, 16, NATION_RANKER_QUARTERFINAL, i);
	}
}

void StoryHandler::NationRankerSemiFinal() {
	int i;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		SetRankerTourament(2, 16, NATION_RANKER_SEMIFINAL, i);
	}
}

void StoryHandler::NationRankerFinal() {
	int i;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
//		if (TitleHolder(i) == NO_BOXER) {
//			SetRankerTourament(1, 16, NATION_CHAMPIONSHIP, i);
//		} else {
		SetRankerTourament(1, 16, NATION_RANKER_FINAL, i);		
//		}
	}
}

void StoryHandler::RookieKing1stQualifing() {
	SetRookieTourament(32, 64, ROOKIE_KING_1ST);
}

void StoryHandler::RookieKing2ndQualifing() {
	SetRookieTourament(16, 64, ROOKIE_KING_2ND);
}

void StoryHandler::RookieKing3rdQualifing() {
	SetRookieTourament(8, 16, ROOKIE_KING_3RD);
}

void StoryHandler::RookieKingQuarterFinal() {
	SetRookieTourament(4, 16, ROOKIE_KING_QUARTERFINAL);
}

void StoryHandler::RookieKingSemiFinal() {
	SetRookieTourament(2, 16, ROOKIE_KING_SEMIFINAL);
}

void StoryHandler::RookieKingFinal() {
	SetRookieTourament(1, 16, ROOKIE_KING_FINAL);
}

void StoryHandler::ExRookieKing() {
	int i;
	for (i = 0; i < MAX_BOXER; i++) {
		if (Boxer(i).HaveTitle(EX_ROOKIE_KING)) {
			Boxer(i).LoseTitle(EX_ROOKIE_KING);		
			Boxer(i).GetTitle(FORMER_ROOKIE_KING);		
		}
		if (Boxer(i).HaveTitle(ROOKIE_KING)) {
			Boxer(i).LoseTitle(ROOKIE_KING);		
			Boxer(i).GetTitle(EX_ROOKIE_KING);
		}	
	}
}


void StoryHandler::SetRankerTourament(int max_match, int max_number, int match_type, int region) {
	int i, j, k, number1, number2, skip;
	i = region;
	skip = max_number / max_match;
	StoryMatch tournament;
	for (j = 0; j < max_match; j++) {
		number1 = NO_BOXER;
		number2 = NO_BOXER;
		for (k = 0; k < skip/2; k++) {
			number1 = ranker[i][j*skip +k];
			if (number1 != NO_BOXER) {
				break;
			}
		}
		for (k = 0; k < skip/2; k++) {
			number2 = ranker[i][j*skip+skip/2 + k];
			if (number2 != NO_BOXER) {
				break;
			}
		}
		if (number1 != NO_BOXER && number2 != NO_BOXER) {
			if (Boxer(number1).RegionRank() < Boxer(number2).RegionRank()) {
				tournament.high_ranked = number1;
				tournament.low_ranked = number2;							
			} else {
				tournament.high_ranked = number2;
				tournament.low_ranked = number1;											
			}							
			tournament.match_type = match_type;											
			tournament.result = NO_RESULT;											
			tournament.replay = 0;											
			AddMatch(tournament);
		}
	}
}

void StoryHandler::RookieKingShrink() {
	int i, j, k, number, skip;
	skip = 64 / 16;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		for (j = 0; j < 16; j++) {
			number = NO_BOXER;
			for (k = 0; k < skip; k++) {
				number = rookie[i][j*skip +k];
				if (number != NO_BOXER) {
					break;
				}
			}
			rookie[i][j] = number;
			Boxer(number).SetState(ROOKIE_T0 + j);
		}
		for (j = 16; j < 64; j++) {
			rookie[i][j] = NO_BOXER;
		}
	}
}

void StoryHandler::SetRookieTourament(int max_match, int max_number, int match_type) {
	int i, j, k, number1, number2, skip;
	skip = max_number / max_match;
	StoryMatch tournament;
	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		for (j = 0; j < max_match; j++) {
			number1 = NO_BOXER;
			number2 = NO_BOXER;
			for (k = 0; k < skip/2; k++) {
				number1 = rookie[i][j*skip +k];
				if (number1 != NO_BOXER) {
					break;
				}
			}
			for (k = 0; k < skip/2; k++) {
				number2 = rookie[i][j*skip+skip/2 + k];
				if (number2 != NO_BOXER) {
					break;
				}
			}
			if (number1 != NO_BOXER && number2 != NO_BOXER) {
				if (Boxer(number1).RegionRank() < Boxer(number2).RegionRank()) {
					tournament.high_ranked = number1;
					tournament.low_ranked = number2;							
				} else {
					tournament.high_ranked = number2;
					tournament.low_ranked = number1;											
				}							
				tournament.match_type = match_type;											
				tournament.result = NO_RESULT;											
				tournament.replay = 0;											
				AddMatch(tournament);
			}
		}
	}
}

/*
void StoryHandler::FirstMatch() {
	current_match = 0;
	while (match[current_match].match_type != NO_MATCH && current_match < MAX_MATCH) {
		current_match++;	
	}
}

void StoryHandler::NextMatch() {
	while (match[current_match].match_type != NO_MATCH && current_match < MAX_MATCH) {
		current_match++;	
	}
}  

void StoryHandler::SetRigionMatch() {

} */

int StoryHandler::MaxColumn(int number) {
	int max_column = SetRegion(WORLD);
	SetData(SORT_REGIONRANK);
	QuickSort();
	if (Boxer(number).RegionRank() >= 16) {
		max_column = SetRegion(Boxer(number).Area());
		SetData(SORT_REGIONRANK);
		QuickSort();
		if (Boxer(number).RegionRank() >= 16) {
			max_column = SetRegion(Boxer(number).Nation());
			SetData(SORT_REGIONRANK);
			QuickSort();
		}
	}
	return max_column;
}

int StoryHandler::RankingOrder(int number) {
	if (Boxer(number).IsInRegion()) {
		return Order(number);
	} else {
		return 0;
	}
}	

void StoryHandler::InitTitle() {
	int i;

	SortRegion(WORLD);
	SetTitleHolder(WORLD, Sort(0));

	for (i = ASIA_OCEANIA; i <= SOUTH_AMERICA; i++) {
		SortRegion(i);
		SetTitleHolder(i, Sort(0));
	}

	for (i = JAPAN; i <= SA_COUNTRIES; i++) {
		SortRegion(i);
		SetTitleHolder(i, Sort(0));
		SortBoxer(20).GetTitle(ROOKIE_KING);		
	}
/*
	for (i = 0; i < MAX_BOXER; i++) {
		if (SortBoxer(i).HaveTitle(WORLD_TITLE)) {
			fprintf(stderr, "%d I have World Title\n", i);
		}
		if (SortBoxer(i).HaveTitle(AREA_TITLE)) {
			fprintf(stderr, "%d I have Area Title\n", i);
		}
		if (SortBoxer(i).HaveTitle(NATION_TITLE)) {
			fprintf(stderr, "%d I have Nation Title\n", i);
		}		
	}   */
}

void StoryHandler::SetTitleHolder(int region, int number) {
	int new_holder, holder, ex_holder;

	if (region != OUT_OF_REGION) {
		new_holder = number;
		if (new_holder != NO_BOXER) {
			if (region >= JAPAN && region <= SA_COUNTRIES) {
				Boxer(new_holder).GetTitle(NATION_TITLE);
			} else if (region >= ASIA_OCEANIA && region <= SOUTH_AMERICA) {
				Boxer(new_holder).GetTitle(AREA_TITLE);	
			} else if (region == WORLD) {
				Boxer(new_holder).GetTitle(WORLD_TITLE);
			}	
		}

		holder = title[region].holder;
		if (holder != NO_BOXER) {
			if (region >= JAPAN && region <= SA_COUNTRIES) {
				if (Boxer(holder).HaveTitle(NATION_TITLE)) {
					Boxer(holder).LoseTitle(NATION_TITLE);
					Boxer(holder).GetTitle(EX_NATION_TITLE);	
				}
			} else if (region >= ASIA_OCEANIA && region <= SOUTH_AMERICA) {
				if (Boxer(holder).HaveTitle(AREA_TITLE)) {
					Boxer(holder).LoseTitle(AREA_TITLE);
					Boxer(holder).GetTitle(EX_AREA_TITLE);	
				}
			} else if (region == WORLD) {
				if (Boxer(holder).HaveTitle(WORLD_TITLE)) {
					Boxer(holder).LoseTitle(WORLD_TITLE);
					Boxer(holder).GetTitle(EX_WORLD_TITLE);	
				}
			}
		}	

		ex_holder = title[region].ex_holder;
		if (ex_holder != NO_BOXER) {
			if (region >= JAPAN && region <= SA_COUNTRIES) {
				if (Boxer(ex_holder).HaveTitle(EX_NATION_TITLE)) {
					Boxer(ex_holder).LoseTitle(EX_NATION_TITLE);
					Boxer(ex_holder).GetTitle(FORMER_NATION_TITLE);	
				}
			} else if (region >= ASIA_OCEANIA && region <= SOUTH_AMERICA) {
				if (Boxer(ex_holder).HaveTitle(EX_AREA_TITLE)) {
					Boxer(ex_holder).LoseTitle(EX_AREA_TITLE);
					Boxer(ex_holder).GetTitle(FORMER_AREA_TITLE);	
				}
			} else if (region == WORLD) {
				if (Boxer(ex_holder).HaveTitle(EX_WORLD_TITLE)) {
					Boxer(ex_holder).LoseTitle(EX_WORLD_TITLE);
					Boxer(ex_holder).GetTitle(FORMER_WORLD_TITLE);	
				}
			}	
		}

		title[region].holder = new_holder;	
		title[region].ex_holder = holder;	
		title[region].defending = 0;	
	}
}

void StoryHandler::TitleDefended(int region) {
	if (region != OUT_OF_REGION) {
		title[region].defending++;
	}
}

int StoryHandler::TournamentBlock(int number) {
	int i;
	if (Boxer(number).WhichTournament() == RANKER) {
		/* EJ[g[igQ? */
		select_region = WORLD;
		for (i = 0; i < 16; i++) {
			if (ranker[WORLD][i] == number) {
				return i;		
			}
		}	

		/* en惉J[g[igQ? */
		select_region = Boxer(number).Area();
		for (i = 0; i < 16; i++) {
			if (ranker[select_region][i] == number){
				return i;		
			}	
		}	

		/* eg[igQ? */
		select_region = Boxer(number).Nation();
		for (i = 0; i < 16; i++) {
			if (ranker[select_region][i] == number) {
				return i;		
			}
		}
	} else {	
		/* Vlg[igQ or not ?*/
		select_region = Boxer(number).Nation();
		for (i = 0; i < 64; i++) {
			if (rookie[select_region][i] == number) {
				return i;		
			}
		}
	}
	return 0;	
}

int StoryHandler::MaxTournament(int region, int type) {
	int i;
	int max_boxer;
	
	max_boxer = 0;
	if (type == RANKER) {
		if (region >= JAPAN && region <= WORLD) {
			for (i = 0; i < 16; i++) {
				if (ranker[region][i] != -1){
					max_boxer = i;
				}
			}	
			return max_boxer;
		}	
	} else {
		if (region >= JAPAN && region <= SA_COUNTRIES) {
			for (i = 0; i < 64; i++) {
				if (rookie[region][i] != -1){
					max_boxer = i;
				}
			}	
			return max_boxer;
		}			
	}
	return 0;
}

int StoryHandler::TournamentBoxer(int region, int t_select, int number) {
	if (t_select == RANKER) {
		if (region >= JAPAN && region <= WORLD) {
			if (number >= 0 && number < 16) {
				return ranker[region][number];
			}
		}	
	} else {
		if (region >= JAPAN && region <= SA_COUNTRIES) {
			if (number >= 0 && number < 64) {
				return rookie[region][number];
			}
		}			
	}
	return NO_BOXER;
}

int StoryHandler::RegionMatch(int region) {
	int i, number1, number2;
	select_region = region;
	max_match = 0;
	for (i = 0; i < MAX_MATCH; i++) {
		if (match[i].match_type != NO_MATCH) {
			number1 = match[i].high_ranked;
			number2 = match[i].low_ranked;
			if (region == WORLD ||
				Boxer(number1).Area() == region ||					
				Boxer(number2).Area() == region ||					
				Boxer(number1).Nation() == region ||					
				Boxer(number2).Nation() == region) {
				region_match[max_match++] = i;				
			}				
		}	
	}
	for (i = 0; i < 16; i++) {
		if (max_match + i < MAX_MATCH) { 
			region_match[max_match+i] = OUT_OF_REGION;				
  		} else {
			break;		
		}
    }
	return max_match;
}

int StoryHandler::RegionResult(int region) {
	int i, number1, number2;
	select_region = region;
	max_match = 0;
	for (i = 0; i < MAX_MATCH; i++) {
		if (result[i].match_type != NO_MATCH) {
			number1 = result[i].high_ranked;
			number2 = result[i].low_ranked;
			if (region == WORLD ||
				Boxer(number1).Area() == region ||					
				Boxer(number2).Area() == region ||					
				Boxer(number1).Nation() == region ||					
				Boxer(number2).Nation() == region) {
				region_match[max_match++] = i;				
			}				
		}	
	}
	for (i = 0; i < 16; i++) {
		if (max_match + i < MAX_MATCH) { 
			region_match[max_match+i] = OUT_OF_REGION;				
  		} else {
			break;		
		}
    }
	return max_match;
}

int StoryHandler::WhichMatch(int number) {
	int i, number1, number2;
	for (i = 0; MatchExists(i) && i < MAX_MATCH; i++) {
		if (region_match[i] != OUT_OF_REGION) {
			number1 = match[region_match[i]].high_ranked;
			number2 = match[region_match[i]].low_ranked;
			if (number1 == number || number2 == number) {
				return i;			
			}			
		}	
	}
	return 0;
}

int StoryHandler::WhichResult(int number) {
	int i, number1, number2;
	for (i = 0; MatchExists(i) && i < MAX_MATCH; i++) {
		if (region_match[i] != OUT_OF_REGION) {
			number1 = result[region_match[i]].high_ranked;
			number2 = result[region_match[i]].low_ranked;
			if (number1 == number || number2 == number) {
				return i;			
			}			
		}	
	}
	return 0;
}
			 
/*
void StoryHandler::MakeTestData() {
	int i, j;
	for (j = 0; j < 10; j++) {
		for (i = 0; i < 4096; i++) {
			SimpleMatch(4095-i, i);				
		}
		SortData(SORT_TOTAL);
		InitRank();
		QuickSort();		
	}   
} */

int StoryHandler::AddMatch(StoryMatch new_match) {
	int i, j;
	for (i = 0; i < MAX_MATCH; i++) {
		if (match[i].match_type == NO_MATCH) {
			match[i] = new_match;
			break;		
		} else if (Boxer(match[i].high_ranked).RegionRank() > Boxer(new_match.high_ranked).RegionRank()) {
			for (j = MAX_MATCH - 1; j > i; j--) {
				match[j] = match[j-1];
			}
			match[i] = new_match;		
			break;
		}
	}
	return i;	
}

/*
void StoryHandler::DelMatch(int number) {
	int i;
	for (i = number; i < MAX_MATCH - 1; i++) {
		match[i] = match[i+1];
	}
	match[MAX_MATCH - 1].match_type = NO_MATCH;		
	for (i = 0; i < MAX_MATCH; i++) {
		if (region_match[i] > number) {
			region_match[i]--;		
		}
	}
} 
*/

void StoryHandler::AddRecord(int number, MatchHandler &match, BoxerHandler &handler1, BoxerHandler &handler2) {
	record[record_number].match = *SelectMatch(number);
	if (SelectMatch(number)->high_ranked == MY_BOXER) {
		record[record_number].my_corner = 1;
		record[record_number].boxer = Boxer(SelectMatch(number)->low_ranked);	
	} else {
		record[record_number].my_corner = 0;
		record[record_number].boxer = Boxer(SelectMatch(number)->high_ranked);		
	}
	record[record_number].year = (Uint8) Boxer(MY_BOXER).CareerYear();
	record[record_number].month = (Uint8) Month();		
	record[record_number].mark = (Uint8) Boxer(MY_BOXER).GetRecent(0);		
	
	match.SetKOTime(&record[record_number].round,
					&record[record_number].minute,
					&record[record_number].second,					
					&record[record_number].msecond);

	if (RecordCorner(record_number)) {
		record[record_number].my_judge1 = handler2.TotalJudge(0);
		record[record_number].my_judge2 = handler2.TotalJudge(1);
		record[record_number].my_judge3 = handler2.TotalJudge(2);
		record[record_number].judge1 = handler1.TotalJudge(0);
		record[record_number].judge2 = handler1.TotalJudge(1);
		record[record_number].judge3 = handler1.TotalJudge(2);
	} else {
		record[record_number].my_judge1 = handler1.TotalJudge(0);
		record[record_number].my_judge2 = handler1.TotalJudge(1);
		record[record_number].my_judge3 = handler1.TotalJudge(2);
		record[record_number].judge1 = handler2.TotalJudge(0);
		record[record_number].judge2 = handler2.TotalJudge(1);
		record[record_number].judge3 = handler2.TotalJudge(2);
	}

	record_number++;
}

void StoryHandler::AddScore() {
	int point;
	point = Boxer(MY_BOXER).Point() / 100;
	switch (Boxer(MY_BOXER).GetRecent(0)) {
		case 1:
		case 7:	
			score += point * point * point / 5;
		break;	
		case 2:	
			score += point * point * point / 10;
		break;	
		case 3:	
		case 4:	
			score += point * point * point / 20;
		break;
	}
//	fprintf(stderr, "Score = %d\n", score);
}

void StoryHandler::SetHighScores(int new_entry, int new_score) {
	int i;
	for (i = 9; i > new_entry; i--) {
		high_scores[i] = high_scores[i-1];
	}
	high_scores[new_entry] = new_score;
}


bool StoryHandler::Save() {
	SDL_RWops *rwops;
	rwops = SDL_RWFromFile("story.sav", "wb");
	if (rwops == NULL) {
		return false;	
	}	
 	SDL_RWwrite(rwops, &mode, sizeof(int), 1);
 	SDL_RWwrite(rwops, &month, sizeof(int), 1);
	SDL_RWwrite(rwops, sb, sizeof(StoryBoxer), MAX_BOXER);
	SDL_RWwrite(rwops, match, sizeof(StoryMatch), MAX_MATCH);
	SDL_RWwrite(rwops, result, sizeof(StoryMatch), MAX_MATCH);
	SDL_RWwrite(rwops, title, sizeof(StoryTitle), WORLD+1);
	SDL_RWwrite(rwops, ranker, sizeof(int), (WORLD+1)*16);
	SDL_RWwrite(rwops, rookie, sizeof(int), (SA_COUNTRIES+1)*64);
	SDL_RWwrite(rwops, &record_number, sizeof(int), 1);
	SDL_RWwrite(rwops, record, sizeof(StoryRecord), MAX_RECORD);
	SDL_RWwrite(rwops, &score, sizeof(int), 1);
	SDL_RWwrite(rwops, high_scores, sizeof(int), 10);
	SDL_RWclose(rwops);
	return true;
}

bool StoryHandler::Load() {
	SDL_RWops *rwops;
	rwops = SDL_RWFromFile("story.sav", "rb");
	if (rwops == NULL) {
		return false;	
	}	

 	SDL_RWread(rwops, &mode, sizeof(int), 1);
 	SDL_RWread(rwops, &month, sizeof(int), 1);
	SDL_RWread(rwops, sb, sizeof(StoryBoxer), MAX_BOXER);
	SDL_RWread(rwops, match, sizeof(StoryMatch), MAX_MATCH);
	SDL_RWread(rwops, result, sizeof(StoryMatch), MAX_MATCH);
	SDL_RWread(rwops, title, sizeof(StoryTitle), WORLD+1);
	SDL_RWread(rwops, ranker, sizeof(int), (WORLD+1)*16);
	SDL_RWread(rwops, rookie, sizeof(int), (SA_COUNTRIES+1)*64);
	SDL_RWread(rwops, &record_number, sizeof(int), 1);
	SDL_RWread(rwops, record, sizeof(StoryRecord), MAX_RECORD);
	SDL_RWread(rwops, &score, sizeof(int), 1);
	SDL_RWread(rwops, high_scores, sizeof(int), 10);
	SDL_RWclose(rwops);
	SortRank();
	return true;
}

