#ifndef STORYHANDLER_H
#define STORYHANDLER_H

#include "csdl.h"
#include "storyboxer.h"
#include "matchhandler.h"

#define MY_BOXER 0
#define NO_BOXER -1
#define MAX_BOXER 4096
#define MAX_MATCH 2050
#define MAX_RECORD 200

enum {
	CUSTOM_MODE,
	RANKING_MODE,
	ORIGINAL_MODE
};

enum {
	WORLD_TITLE_MATCH = 1,
	AREA_TITLE_MATCH,
	NATION_TITLE_MATCH,
	WORLD_CHAMPIONSHIP,
	AREA_CHAMPIONSHIP,
	NATION_CHAMPIONSHIP,
	WORLD_RANKER_FINAL,
	WORLD_RANKER_SEMIFINAL,
	WORLD_RANKER_QUARTERFINAL,
	AREA_RANKER_FINAL,
	AREA_RANKER_SEMIFINAL,
	AREA_RANKER_QUARTERFINAL,
	NATION_RANKER_FINAL,
	NATION_RANKER_SEMIFINAL,
	NATION_RANKER_QUARTERFINAL,
	NATION_RANKER_1ST,
	ROOKIE_KING_FINAL,
	ROOKIE_KING_SEMIFINAL,
	ROOKIE_KING_QUARTERFINAL,
	ROOKIE_KING_3RD,	
	ROOKIE_KING_2ND,
	ROOKIE_KING_1ST,
	ROUND10_MATCH,	
	ROUND8_MATCH,	
	ROUND6_MATCH,
	ROUND4_MATCH,
	DEBUT_MATCH,
	NO_MATCH
};

typedef struct {
	int high_ranked;
	int low_ranked;
	int match_type;
	int result;
	int replay;
} StoryMatch;

typedef struct {
	int defending;
	int holder;
	int ex_holder;	
	int challengers[5];
} StoryTitle;

typedef struct {
	StoryMatch match;
	StoryBoxer boxer;
	Uint8 my_corner;
	Uint8 year;
	Uint8 month;
	Uint8 mark;
	Uint8 round;
	Uint8 minute;
	Uint8 second;
	Uint8 msecond;

	Uint8 my_judge1;
	Uint8 my_judge2;
	Uint8 my_judge3;

	Uint8 judge1;
	Uint8 judge2;
	Uint8 judge3;

	Uint8 dummy1;
	Uint8 dummy2;	
} StoryRecord;

class Game;

class StoryHandler {
public:
	StoryHandler();
	~StoryHandler();
		
	void Init();
	int Mode() const {return mode;}
	int Month() const {return month;}
	void SetMode(int i) {mode = i;}
	StoryBoxer & Boxer(int i);
	StoryBoxer & SortBoxer(int i);
	int Sort(int i) const;
	int Order(int number);
	void SortRank() {SortData();}
	void SetData(int data_type = SORT_RANK);
	void SortData(int data_type = SORT_RANK) {
		SetData(data_type);
		QuickSort();
	}	
	void InitRank();
	void InitPoint();
	int SetRegion(int region = WORLD);
	void SortRegion(int region = WORLD) {
		SetRegion(region);
		SetData(SORT_REGIONRANK);
		QuickSort();		
	}

	void InsSort();
	void QuickSort();
	void BubbleSort();

	void InitTitle();
	int TitleDefending(int region) const {return title[region].defending;}
	void TitleDefendingP(int region) {title[region].defending++;}
	void ResetTitleDefending(int region) {title[region].defending = 0;}
	int TitleHolder(int region) const {return title[region].holder;}
	void SetTitleHolder(int region, int number);
	void TitleDefended(int region);
	void SetRigionMatch();

//	void FirstMatch();
//	void NextMatch();
//	void SetMatch(int h, int l, int t, int r);
	
	int MaxColumn(int number);
	int RankingOrder(int number);
	int SelectRegion() const {return select_region;}
	int TournamentBlock(int number);
	int MaxTournament(int region, int type);
	int TournamentBoxer(int region, int t_select, int number);
	int Pairing(int number, int max_number);
	bool IsInTournament(int region, int t_select, int number) {
		return (TournamentBoxer(region, t_select, number)) != NO_BOXER;
	}
	void LeaveTournament(int number);
	bool IsInRegion(int number) {return (Boxer(number).IsInRegion());}

	int RegionMatch(int region);
	int RegionResult(int region);
	StoryMatch* SelectMatch(int number) {return &match[region_match[number]];}
	StoryMatch* SelectResult(int number) {return &result[region_match[number]];}
	int WhichMatch(int number);
	int WhichResult(int number);
	int MaxMatch() const {return max_match;}
	bool MatchExists(int number) const {return (region_match[number] != OUT_OF_REGION);}

	void NextMonth(Game &game, int player = 1);
	void SetEvents(Game &menu, int player = 1); 
	void MatchRecord(StoryMatch *match);
	void SetNewRanking() {
		SortData(SORT_POINT);
		InitRank();	
	}
	void QuitBoxer(int number);
 	void SwapBoxer(int number);
	void AddRecord(int number, MatchHandler &match, BoxerHandler &handler1, BoxerHandler &handler2);

	StoryBoxer & RecordBoxer(int number) {return record[number].boxer;}
	StoryMatch & RecordMatch(int number) {return record[number].match;}
	int RecordCorner(int number) {return (int)record[number].my_corner;}
	int RecordNumber() const {return record_number;}
	int RecordYear(int number) const {return (int)record[number].year;}
	int RecordMonth(int number) const {return (int)record[number].month;}
	int RecordMark(int number) const {return (int)record[number].mark;}
	int RecordRound(int number) const {return (int)record[number].round;}
	int RecordMinute(int number) const {return (int)record[number].minute;}
	int RecordSecond(int number) const {return (int)record[number].second;}
	int RecordMSecond(int number) const {return (int)record[number].msecond;}
	int RecordMyJudge1(int number) const {return (int)record[number].my_judge1;}
	int RecordMyJudge2(int number) const {return (int)record[number].my_judge2;}
	int RecordMyJudge3(int number) const {return (int)record[number].my_judge3;}
	int RecordJudge1(int number) const {return (int)record[number].judge1;}
	int RecordJudge2(int number) const {return (int)record[number].judge2;}
	int RecordJudge3(int number) const {return (int)record[number].judge3;}

	void ClearRecord() {
		record_number = 0;
		score = 0;	
	}
	void AddScore();
	int Score() const {return score;}
	int HighScores(int i) const {return high_scores[i];}
	void SetHighScores(int new_entry, int new_score);
	
	bool Save();
	bool Load();

private:
	Random random;
	int mode;
	int month;
	int *sort;
	StoryBoxer *sb;

	int current_match;
	int max_match;
	int select_region;
	int region_match[MAX_MATCH];	
	StoryMatch match[MAX_MATCH];
	StoryMatch result[MAX_MATCH];	

	StoryTitle title[WORLD+1];
	int ranker[WORLD+1][16];
	int rookie[SA_COUNTRIES+1][64];

	int record_number;
	StoryRecord record[MAX_RECORD];
	
	int score;
	int high_scores[10];
	
	void WillOffer(int *offer, Game &menu);
	void WillAccept(int *accept, int number, Game &menu);
	int ChoiceLowRanked(int *accept, int start, Game &menu);
	void SetMatch(StoryMatch *set_match, int high, int low, int type, int state);
	void MatchMaking(Game &menu, int player = 1);
	void RetireBoxers(Game &menu, int player = 1);
	void AgeBoxers(Game &menu, int player = 1);
	void WillDefend(int *offer, int region, Game &menu);
	void WillChallenge(int *accept, int number, int month, Game &menu);
	int SelectCandidates(int *candidates);
	void DeleteCandidate(int *candidates, int delete_challenger);
	void SelectChallengers(Game &menu, int player = 1);
	void SelectDesignatedChallenger(int region, Game &menu, int player = 1);
	void SetChallengers();
	void SetNextSchedule(int region, int number);
	void WinnerToChallenger(int region, int challenger);
	void SetRankerTourament(int max_match, int max_number, int match_type, int regin);
	void SetRookieTourament(int max_match, int max_number, int match_type);
	int AddMatch(StoryMatch new_match);
  	void DelMatch(int number);

	int SimpleMatch(int number1, int number2);
	void TitleMatch(int region, StoryMatch *match);
	void Championship(int region, StoryMatch *match);
	void RankerFinal(int region, StoryMatch *match);
	void RookieFinal(StoryMatch *match);
	void WhichLeave(StoryMatch *match);
	void SetRest(StoryMatch *match);
	void SetRestRound8(StoryMatch *match);
	void SetRestRound4(StoryMatch *match);
	void SetRankerWinner(int region, int winner);
//	void MakeTestData();

	void WorldRankerQuarterFinal();
	void WorldRankerSemiFinal();
	void WorldRankerFinal();

	void AreaRankerQuarterFinal();
	void AreaRankerSemiFinal();
	void AreaRankerFinal();

	void NationRanker1stQualifing();
	void NationRankerQuarterFinal();
	void NationRankerSemiFinal();
	void NationRankerFinal();

	void RookieKing1stQualifing();
	void RookieKing2ndQualifing();
	void RookieKingShrink();
	void RookieKing3rdQualifing();
	void RookieKingQuarterFinal();
	void RookieKingSemiFinal();
	void RookieKingFinal();
	void ExRookieKing();
};

#endif // STORYHANDLER_H

