#include "sttf.h"

TTF::TTF() {
	ttf = NULL;
}

TTF::~TTF() {
	Close();
}

int TTF::Open(const char *fontname, int fontsize) {
	if (WasOpen()) {
		Close();	
	}
	
	FILE* fp = fopen(fontname, "r" );
	if(fp == NULL) {
		emes ("TTF::Open -> There's no such file.");
		efile(fontname);	
		return SDL_FAILURE;
	}
	fclose( fp );
	ttf = TTF_OpenFont(fontname, fontsize);	
    if (!WasOpen()) {
		emes ("TTF::Open -> Can't open font file.");
		efile(fontname);	
		return SDL_FAILURE;
	}
	smes ("TTF::Open");
	sfile(fontname);
	SetStyle(TTF_STYLE_NORMAL);
	return SDL_SUCCESS;
}

void TTF::SetStyle(int style) {
	if (WasOpen()) {
	 	TTF_SetFontStyle(ttf, style);
	}
}

void TTF::Close() {
	if (WasOpen()) {
		TTF_CloseFont(ttf);
		ttf = NULL;
		smes ("TTF::Close");
	} else {
		wmes ("TTF::Close");	
	}
}

TTF_Font* TTF::Font() {
	if (WasOpen()) {
		return ttf;
	}
	return NULL;	
}

bool TTF::WasOpen() {
	if (ttf == NULL) {
		return false;
	}
	return true;
} 

TTFs::TTFs() {
	ttfs = NULL;
}

TTFs::~TTFs() {
	Free();
}

int TTFs::Init(int number) {
	if(WasInit()) {
		Free();
	}
	ttfs = new TTF[number];
	cnumber = 0;
	mnumber = number;
	if(!WasInit()) {
		emes("TTFs::Init");		
		return SDL_FAILURE;	
	}
	smes("TTFs::Init");		
	return SDL_SUCCESS;
}

void TTFs::Free() {
	if (WasInit()) {
		delete [] ttfs;
		ttfs = NULL;
		smes("TTFs::Free");	
	} else {
		wmes("TTFs::Free");
	}
}

void TTFs::Choice(int number) {
	if (number >= 0 && number < mnumber) {
		cnumber = number;
	}
}

TTF_Font* TTFs::Font() {
	return ttfs[cnumber].Font();
}

SDL_Surface* TTFs::RenderSolid(const char* text) {
	static SDL_Color white = {0xFF, 0xFF, 0xFF, 0xFF};
//	return TTF_RenderUTF8_Solid(Font(), babel::sjis_to_utf8(text).c_str(), white);
#ifdef USE_BABEL
	return TTF_RenderUNICODE_Solid(Font(), (Uint16*)babel::sjis_to_unicode(text).data(), white);
#else
	sjis2uni(text, buf);
	return TTF_RenderUNICODE_Solid(Font(), buf, white);
#endif
}

SDL_Surface* TTFs::RenderShaded(const char* text) {
	static SDL_Color white = {0xFF, 0xFF, 0xFF, 0xFF};
	static SDL_Color black = {0x00, 0x00, 0x00, 0x00};
//	return TTF_RenderUTF8_Shaded(Font(), babel::sjis_to_utf8(text).c_str(), white, black);
#ifdef USE_BABEL
	return TTF_RenderUNICODE_Shaded(Font(), (Uint16*)babel::sjis_to_unicode(text).data(), white, black);
#else
	sjis2uni(text, buf);
	return TTF_RenderUNICODE_Shaded(Font(), buf, white, black);
#endif
}

SDL_Surface* TTFs::RenderBlended(const char* text) {
	static SDL_Color white = {0xFF, 0xFF, 0xFF, 0xFF};
//	return TTF_RenderUTF8_Blended(Font(), babel::sjis_to_utf8(text).c_str(), white);
#ifdef USE_BABEL
	return TTF_RenderUNICODE_Blended(Font(), (Uint16*)babel::sjis_to_unicode(text).data(), white);
#else
	sjis2uni(text, buf);
	return TTF_RenderUNICODE_Blended(Font(), buf, white);
#endif
}

int TTFs::Open(const char *fontname, int fontsize) {
	if (WasInit()) {
		return ttfs[cnumber].Open(fontname, fontsize);
	}
	return SDL_FAILURE;
}

void TTFs::SetStyle(int style) {
	if (WasInit()) {
		ttfs[cnumber].SetStyle(style);
	}
}

bool TTFs::WasInit() {
	if (ttfs == NULL) {
		return false;	
	}
	return true;
}


STTF::STTF() {
	wasinit = false;
	rendertype = RENDER_SHADED;
	Init();
}

STTF::~STTF() {
	Quit();
}

int STTF::Init() {
	if (WasInit()) {
		Quit();
	}
	if (TTF_Init() != SDL_SUCCESS) {
		emes("STTF::Init");
		rmes();	
		return SDL_FAILURE;
	}
	smes("STTF::Init");	
	wasinit = true;
	return SDL_SUCCESS;
}

int STTF::Init(int number) {
	return ttfs.Init(number);
}

void STTF::Quit() {
	if (WasInit()) {
		TTF_Quit();
		smes("STTF::Quit");	
		wasinit = false;
	} else {
		wmes("STTF::Quit");	
	}	
}

void STTF::Choice(int number) {
	ttfs.Choice(number);
}

int STTF::Open(const char* file, int fontsize) {
	return ttfs.Open(file, fontsize);
}

int STTF::Open(int number,  const char* file, int fontsize) {
	Choice(number);
	return Open(file, fontsize);
}

void STTF::SetStyle(int style) {
	ttfs.SetStyle(style);
}

void STTF::SetRender(RenderType setrendertype) {
	rendertype = setrendertype;
}

SDL_Surface* STTF::Render(const char* text) {
	switch (rendertype) {
		case RENDER_SOLID:
			return ttfs.RenderSolid(text);	
		break;
		case RENDER_BLENDED:
			return ttfs.RenderBlended(text);			
		break;
		default:
			return ttfs.RenderShaded(text);			
		break;
	}
}

SDL_Surface* STTF::Render(int number, const char* text) {
	Choice(number);
	switch (rendertype) {
		case RENDER_SOLID:
			return ttfs.RenderSolid(text);	
		break;
		case RENDER_BLENDED:
			return ttfs.RenderBlended(text);			
		break;
		default:
			return ttfs.RenderShaded(text);			
		break;
	}
}

void STTF::operator[](int number) {
	Choice(number);
}

bool STTF::WasInit() {
	return wasinit;
}

