#include "think.h"
#include "boxer.h"
#include "boxerhandler.h"
#include "matchhandler.h"

int Think::Consider(MatchHandler &match, BoxerHandler& m_handler, Boxer& m_boxer, BoxerHandler& o_handler){
	int percent;
	
	if (con_restround_less && (restround_less <= (match.MaxRound() - match.Round()))){
		return TR_THROUGH;
	}
	if (con_resttime_less && (resttime_less <= (match.RoundTime() - match.TimeCount()))){
		return TR_THROUGH;
	}

	if (con_mrestcount_less && (mrestcount_less <= (match.KOCount() - m_handler.Count()))){
		return TR_THROUGH;
	}
	if (con_mrestdown_less && (mrestdown_less <= (match.TKODown() - m_handler.Down()))){
		return TR_THROUGH;
	}

	if (con_distance_greater && (distance_greater >= m_handler.Distance())){
		return TR_THROUGH;
	}
	if (con_distance_less && (distance_less <= m_handler.Distance())){
		return TR_THROUGH;
	}
	if (con_mposition && (mposition != m_boxer.WhichPosition())){
		return TR_THROUGH;
	}
	if (con_msituation && (msituation - 1 > match.Situation(m_handler, o_handler) ||
						   msituation + 1 < match.Situation(m_handler, o_handler))){
		return TR_THROUGH;
	}

#if CHANGE_SPECIFICATIONS
	if (con_mswing_greater && (mswing_greater >= m_handler.CyanosisRate())){
		return TR_THROUGH;
	}
	if (con_mswing_less && (mswing_less <= m_handler.CyanosisRate())){
		return TR_THROUGH;
	}
	if (con_mspeed_greater && (mspeed_greater>= m_handler.DamageRate())){
		return TR_THROUGH;
	}
	if (con_mspeed_less && (mspeed_less <= m_handler.DamageRate())){
		return TR_THROUGH;
	}
	if (con_mstate  && (mstate != m_handler.State())){
		return TR_THROUGH;
	}

	if (con_oswing_greater && (oswing_greater >= o_handler.CyanosisRate())){
		return TR_THROUGH;
	}
	if (con_oswing_less && (oswing_less <= o_handler.CyanosisRate())){
		return TR_THROUGH;
	}
	if (con_ospeed_greater && (ospeed_greater >= o_handler.DamageRate())){
		return TR_THROUGH;
	}
	if (con_ospeed_less && (ospeed_less <= o_handler.DamageRate())){
		return TR_THROUGH;
	}
	if (con_ostate && (ostate != o_handler.State())){
		return TR_THROUGH;
	}
#else
	if (con_mswing_greater && (mswing_greater >= m_handler.SwingSpeed())){
		return TR_THROUGH;
	}
	if (con_mswing_less && (mswing_less <= m_handler.SwingSpeed())){
		return TR_THROUGH;
	}
	if (con_mspeed_greater && (mspeed_greater >= m_handler.MoveSpeed())){
		return TR_THROUGH;
	}
	if (con_mspeed_less && (mspeed_less <= m_handler.MoveSpeed())){
		return TR_THROUGH;
	}
	if (con_mstate && (mstate != m_handler.State())){
		return TR_THROUGH;
	}

	if (con_oswing_greater && (oswing_greater >= o_handler.SwingSpeed())){
		return TR_THROUGH;
	}
	if (con_oswing_less && (oswing_less <= o_handler.SwingSpeed())){
		return TR_THROUGH;
	}
	if (con_ospeed_greater && (ospeed_greater >= o_handler.MoveSpeed())){
		return TR_THROUGH;
	}
	if (con_ospeed_less && (ospeed_less <= o_handler.MoveSpeed())){
		return TR_THROUGH;
	}
	if (con_ostate && (ostate != o_handler.State())){
		return TR_THROUGH;
	}
#endif

	percent = match.Percent();	

	if (percent >= 0 && percent < action_rate[0]) {
		return Select(action[0]);	
	} else if (percent >= action_rate[0] && percent < action_rate[1]){
		return Select(action[1]);		
	} else if (percent >= action_rate[1] && percent < action_rate[2]){
		return Select(action[2]);
	} else if (percent >= action_rate[2] && percent < action_rate[3]){
		return Select(action[3]);
	} else {
		return Select(action[4]);	
	}
}

int Think::Select(int s_action) {
	static Random random;
	if (s_action == TR_MOVE_RANDOM) {
		s_action = TR_FORWARD + (random.Rand() % 4);
	} else if (s_action == TR_R_OFFENSE_RANDOM) {
		s_action = TR_RIGHT_N + (random.Rand() % 9);
	} else if (s_action == TR_L_OFFENSE_RANDOM) {
		s_action = TR_LEFT_N + (random.Rand() % 9);
	} else if (s_action == TR_DEFENSE_RANDOM) {
		s_action = TR_DEFENSE_N + (random.Rand() % 9);	
	}
	return s_action;
}

void Think::Reverse() {
	int i;
	for (i = 0; i < 5; i++) {
		if (action[i] == TR_RIGHT_N) {
			action[i] =	TR_LEFT_N;
		} else if(action[i] == TR_LEFT_N) {
			action[i] =	TR_RIGHT_N;		
		} else if (action[i] == TR_RIGHT_F) {
			action[i] =	TR_LEFT_F;
		} else if(action[i] == TR_LEFT_F) {
			action[i] =	TR_RIGHT_F;		
		} else if (action[i] == TR_RIGHT_B) {
			action[i] =	TR_LEFT_B;
		} else if(action[i] == TR_LEFT_B) {
			action[i] =	TR_RIGHT_B;		
		} else if (action[i] == TR_RIGHT_R) {
			action[i] =	TR_LEFT_R;
		} else if(action[i] == TR_LEFT_R) {
			action[i] =	TR_RIGHT_R;		
		} else if (action[i] == TR_RIGHT_L) {
			action[i] =	TR_LEFT_L;
		} else if(action[i] == TR_LEFT_L) {
			action[i] =	TR_RIGHT_L;		
		} else if (action[i] == TR_RIGHT_RF) {
			action[i] =	TR_LEFT_RF;
		} else if(action[i] == TR_LEFT_RF) {
			action[i] =	TR_RIGHT_RF;		
		} else if (action[i] == TR_RIGHT_LF) {
			action[i] =	TR_LEFT_LF;
		} else if(action[i] == TR_LEFT_LF) {
			action[i] =	TR_RIGHT_LF;		
		} else if (action[i] == TR_RIGHT_RB) {
			action[i] =	TR_LEFT_RB;
		} else if(action[i] == TR_LEFT_RB) {
			action[i] =	TR_RIGHT_RB;		
		} else if (action[i] == TR_RIGHT_LB) {
			action[i] =	TR_LEFT_LB;
		} else if(action[i] == TR_LEFT_LB) {
			action[i] =	TR_RIGHT_LB;		
		} else if (action[i] == TR_R_OFFENSE_RANDOM) {
			action[i] =	TR_L_OFFENSE_RANDOM;
		} else if(action[i] == TR_L_OFFENSE_RANDOM) {
			action[i] =	TR_R_OFFENSE_RANDOM;
		}
	}		
}

void Think::Print(MatchHandler &match, BoxerHandler& m_handler, Boxer& m_boxer, BoxerHandler& o_handler){
	if (con_restround_less) {
		fprintf(stderr, "cEh%2d < %2d is ", match.MaxRound() - match.Round(), restround_less);
		if (restround_less <= (match.MaxRound() - match.Round())) {
			fprintf(stderr, "FALSE\n");		
		} else {
			fprintf(stderr, "TRUE\n");			
		}
	}
    if (con_restround_less && (restround_less <= (match.MaxRound() - match.Round()))){
		return;
	}  
 
	if (con_resttime_less) {
		fprintf(stderr, "c^C%3d < %3d is ", (match.RoundTime() - match.TimeCount())/50, resttime_less/50);
		if (resttime_less <= (match.RoundTime() - match.TimeCount())) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}
	if (con_resttime_less && (resttime_less <= (match.RoundTime() - match.TimeCount()))){
		return;
	} 
 
	if (con_mrestcount_less) {
		fprintf(stderr, "cJEg%2d < %2d is ", match.KOCount() - m_handler.Count(), mrestcount_less);
		if (mrestcount_less <= match.KOCount() - m_handler.Count()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mrestcount_less && (mrestcount_less <= (match.KOCount() - m_handler.Count()))){
		return;
	}   

	if (con_mrestdown_less) {
		fprintf(stderr, "c_E%2d > %2d is ", match.TKODown() - m_handler.Down(), mrestdown_less);
		if (mrestdown_less <= match.TKODown() - m_handler.Down()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mrestdown_less && (mrestdown_less <= (match.TKODown() - m_handler.Down()))){
		return;
	}   

	if (con_distance_greater) {
		fprintf(stderr, "%4.3f > %4.3f is ", m_handler.Distance(), distance_greater);
		if (distance_greater >= m_handler.Distance()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_distance_greater && (distance_greater >= m_handler.Distance())){
		return;
	}

	if (con_distance_less) {
		fprintf(stderr, "%4.3f < %4.3f is ", m_handler.Distance(), distance_less);
		if (distance_less <= m_handler.Distance()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_distance_less && (distance_less <= m_handler.Distance())){
		return;
	}

	if (con_mposition) {
		fprintf(stderr, "ʒu%2d = %2d is ", m_boxer.WhichPosition(), mposition);
		if (mposition != m_boxer.WhichPosition()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mposition && (mposition != m_boxer.WhichPosition())){
		return;
	}

	if (con_msituation) {
		fprintf(stderr, "%2d <= %2d <= %2d is ", msituation-1, match.Situation(m_handler, o_handler), msituation+1);
		if (msituation - 1 < match.Situation(m_handler, o_handler) ||
			msituation + 1 > match.Situation(m_handler, o_handler)) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_msituation && (msituation != match.Situation(m_handler, o_handler))){
		return;
	}

	if (con_mswing_greater) {
		fprintf(stderr, "_%3.0f > %3.0f is ", (m_handler.SwingSpeed()-1.0)*100.0, (mswing_greater-1.0)*100.0);
		if (mswing_greater >= m_handler.SwingSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mswing_greater && (mswing_greater >= m_handler.SwingSpeed())){
		return;
	}
	
	if (con_mswing_less) {
		fprintf(stderr, "_%3.0f < %3.0f is ", (m_handler.SwingSpeed()-1.0)*100.0, (mswing_less-1.0)*100.0);
		if (mswing_less <= m_handler.SwingSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mswing_less && (mswing_less <= m_handler.SwingSpeed())){
		return;
	}

	if (con_mspeed_greater) {
		fprintf(stderr, "_[W%3.0f > %3.0f is ", (m_handler.MoveSpeed()-1.0)*100.0, (mspeed_greater-1.0)*100.0);
		if (mspeed_greater >= m_handler.MoveSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mspeed_greater && (mspeed_greater >= m_handler.MoveSpeed())){
		return;
	}

	if (con_mspeed_less) {
		fprintf(stderr, "_[W%3.0f < %3.0f is ", (m_handler.MoveSpeed()-1.0)*100.0, (mspeed_less-1.0)*100.0);
		if (mspeed_less <= m_handler.MoveSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mspeed_less && (mspeed_less <= m_handler.MoveSpeed())){
		return;
	}

	if (con_mstate) {
		fprintf(stderr, "%2d = %2d is ",m_handler.State(), mstate);
		if (mstate != m_handler.State()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_mstate && (mstate != m_handler.State())){
		return;
	}

	if (con_oswing_greater) {
		fprintf(stderr, "_%3.0f < %3.0f is ", (o_handler.SwingSpeed()-1.0)*100.0, (oswing_greater-1.0)*100.0);
		if (oswing_greater >= o_handler.SwingSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_oswing_greater && (oswing_greater >= o_handler.SwingSpeed())){
		return;
	}
	
	if (con_oswing_less) {
		fprintf(stderr, "_%3.0f < %3.0f is ", (o_handler.SwingSpeed()-1.0)*100.0, (oswing_less-1.0)*100.0);
		if (oswing_less <= o_handler.SwingSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_oswing_less && (oswing_less <= o_handler.SwingSpeed())){
		return;
	}

	if (con_ospeed_greater) {
		fprintf(stderr, "_[W%3.0f > %3.0f is ", (o_handler.MoveSpeed()-1.0)*100.0, (ospeed_greater-1.0)*100.0);
		if (ospeed_greater >= o_handler.MoveSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_ospeed_greater && (ospeed_greater >= o_handler.MoveSpeed())){
		return;
	}

	if (con_ospeed_less) {
		fprintf(stderr, "_[W%3.0f > %3.0f is ", (o_handler.MoveSpeed()-1.0)*100.0, (ospeed_less-1.0)*100.0);
		if (ospeed_less <= o_handler.MoveSpeed()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_ospeed_less && (ospeed_less <= o_handler.MoveSpeed())){
		return;
	}

	if (con_ostate) {
		fprintf(stderr, "%2d = %2d is ",o_handler.State(), ostate);
		if (ostate != o_handler.State()) {
			fprintf(stderr, "FALSE\n");			
		} else {
			fprintf(stderr, "TRUE\n");		
		}
	}  
	if (con_ostate && (ostate != o_handler.State())){
		return;
	}
}
